// ****************************************************************************
//
// Copyright (C) 2013 DIATEAM. All rights reserved.
//
// This file is part of diamesh.
//
// Hynesim is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License version 2 
// as published by the Free Software Foundation.
//
// Hynesim is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// ****************************************************************************

#ifndef ACCESSRIGHTS_ICE
#define ACCESSRIGHTS_ICE

#include <QtIce/qtice.ice>
#include <diamesh/types.ice>
#include <diamesh/hsetmanager.ice>

module Diamesh
{

class AccessRight;
class User;

["cpp:type:std::list<Diamesh::AccessRightPtr>"]
sequence<AccessRight> AccessRightList;

class AccessRight
{
    long id;
    string name;
    AccessRight parent;
    AccessRightList children;
};

["cpp:comparable"]
struct AccessControlList
{
    string target;
    string subject;
    LongList rights;
    long hSetId;
};
["cpp:type:std::list<Diamesh::AccessControlList>"]
sequence<AccessControlList> ACLList;

class ACLTarget
{
    long id = 0;
    long hSetId = -1;
};

["signal:void aclAdded(QString, QString)",
"signal:void aclChanged(QString, QString)",
"signal:void aclRemoved(QString, QString)"]
interface ACLManager extends QtIce::SignalEmitter
{
    AccessRightList getRegisteredRights();
    StringStringMap getTargetTypes();
    AccessControlList getACL(string target, string subject);
    StringList getSubjectsForTarget(ACLTarget target);
    ACLList getACLsForTarget(ACLTarget target);
    bool setACL(string target, string subject, LongList rightsIds, long hSetId);
    bool removeACL(string target, string subject);
    void targetRemoved(string target);
    void subjectRemoved(string target);
    bool testRightForObject(ACLTarget target, User subject, long rightId);
    bool testRight(string target, User subject, long rightId);
};

};

#endif // ACCESSRIGHTS_ICE
