/******************************************************************************
**
** Copyright (C) 2011-2012 DIATEAM. All rights reserved.
**
** This file is part of diamesh.
**
** Hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** Hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef ACCESSRIGHTSHELPER_H
#define ACCESSRIGHTSHELPER_H

#include <QtCore/QObject>
#include <QtCore/QHash>
#include <QtCore/QStringList>

#include <diamesh/accessrights.h>

namespace Diamesh
{


class AccessRightsHelper : public QObject
{
    Q_OBJECT

public:
    enum SubjectType {
        USER_SUBJECT,
        GROUP_SUBJECT,
        HSET_SUBJECT,
        OTHER_SUBJECT,
        INVALID_SUBJECT
    };

    explicit AccessRightsHelper(const AccessRightList &rightList, const Diamesh::StringStringMap &targetTypeMap, QObject *parent = 0);
    virtual ~AccessRightsHelper();

    bool hasRight(const QString &rightName) const;
    AccessRightList getRightList() const; // todo check if returning hash would not be better
    AccessRightPtr getRight(const QString &rightName) const;

    LongList addAccessRights(const LongList &rightsIds, LongList additionnalRightsIds) const;
    LongList addAccessRight(const LongList &rightsIds, long additionnalRightId) const;
    LongList removeAccessRights(const LongList &rightsIds, LongList toRemoveIds) const;
    LongList removeAccessRight(const LongList &rightsIds, long toRemoveId) const;
    Ice::Long rightFromName(const QString &name) const;
    LongList rightsFromNames(const QString &names) const;
    LongList rightsFromNames(const QStringList &names) const;
    QString nameFromRight(Ice::Long rightId) const;
    QStringList nameListFromRights(const LongList &rightsIds) const;
    QString namesFromRights(const LongList &rightsIds) const;
    LongList rightsIdsFromAccessRightList(const AccessRightList &rights) const;
    LongList validRightsIdsFromRightId(Ice::Long rightId) const;
    QPair<AccessRightsHelper::SubjectType, qlonglong> subjectDetails(const QString &subject);
    QPair<QString, qlonglong> targetDetails(const QString &target);
    QPair<QString, QString> targetDetailsFromObject(const ACLTargetPtr &target);
    QString targetFromObject(const ACLTargetPtr &target);

    bool rightsMatchesRightsNames(const LongList &rightsIds, const QStringList &rightsNames) const;

protected:
    explicit AccessRightsHelper(QObject *parent = 0);
    AccessRightPtr _getRight(Ice::Long rightId) const;
    virtual void _initSubjectTypes();
    Ice::Long _checkRight(const QString &rightName, Ice::Long parentId);
    void _addChild(Ice::Long rightId, const AccessRightPtr &child);
    void _registerRootAccessRight();
    void _registerAccessRight(const QString &rightName);
    void _registerTargetType(const QString &targetIceId, const QString &targetType);

private:
    Ice::Long _rootId;
    Ice::Long _nextId;
    QHash<Ice::Long, AccessRightPtr> _rightsHash;
    QHash<QString, Ice::Long> _nameToIdHash;
    QHash<QString, AccessRightsHelper::SubjectType> _subjectTypeHash;
    QHash<QString, AccessRightsHelper::SubjectType> _composedSubjectTypeHash;
    QHash<QString, QString> _targetTypeHash;

    friend class ACLManagerI;
};

} // namespace Diamesh

#endif // ACCESSRIGHTSHELPER_H
