/******************************************************************************
**
** Copyright (C) 2011-2012 DIATEAM. All rights reserved.
**
** This file is part of diamesh.
**
** Hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** Hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef AUTH_MANAGER_I_H
#define AUTH_MANAGER_I_H

#include <QtCore/QHash>
#include <QtCore/QReadWriteLock>

#include <Ice/Ice.h>

#include <diamesh/authmanager.h>

class QSettings;

namespace Diamesh
{

class PermissionsStorage;
class SqlDatabase;
class UserManagerI;

class AuthManagerI : public QObject, public Diamesh::AuthManager
{
    Q_OBJECT

public:
    AuthManagerI(const QString &configFileName, UserManagerI *userManager);
    virtual ~AuthManagerI();

    bool checkPermissions(const std::string &userName, const std::string &password, std::string &reason, const Ice::Current &) const;
    bool setPassword(const std::string &userName, const std::string &password, const Ice::Current &);

signals:
    void userPasswordSet(Ice::Long);

protected:
    PermissionsStorage *_storage;
    UserManagerI *_userManager;
};

class PermissionsStorage : public QObject
{
    Q_OBJECT

public:
    PermissionsStorage() {}
    virtual ~PermissionsStorage() {}

    virtual bool checkPermissions(const QString &userName, const QString &password) = 0;
    virtual bool setPassword(const QString &userName, const QString &password, const QString &oldName = QString()) = 0;

protected slots:
    virtual void _userRenamed(QString oldName, QString newName) = 0;
    virtual void _userRemoved(QString name) = 0;
};

class QSettingsPermissionsStorage : public PermissionsStorage
{
    Q_OBJECT

public:
    QSettingsPermissionsStorage(const QString &configFileName);

    bool checkPermissions(const QString &userName, const QString &password);
    bool setPassword(const QString &userName, const QString &password, const QString &oldName);

protected slots:
    void _userRenamed(QString oldName, QString newName);
    void _userRemoved(QString name);

private:
    QSettings *_settings;
};

class SqlPermissionsStorage : public PermissionsStorage
{
    Q_OBJECT

    struct Password
    {
        Password() : hash(QString()), salt(QString()) {}
        Password(const QString &hash_, const QString &salt_) : hash(hash_), salt(salt_) {}

        QString hash;
        QString salt;
    };

public:
    SqlPermissionsStorage(const QString &dbType, const QString &dbName, const QString &tableName);

    bool checkPermissions(const QString &userName, const QString &password);
    bool setPassword(const QString &userName, const QString &password, const QString &oldName);

protected slots:
    void _userRenamed(QString oldName, QString newName);
    void _userRemoved(QString name);

private:
    QString _hashPassword(const QString &password, const QString &salt);

    QReadWriteLock _lock;
    QString _tableName;
    QHash<QString, Password> _userPasswordHash;
    SqlDatabase *_db;
};

}

#endif // AUTH_MANAGER_I_H
