/******************************************************************************
**
** Copyright (C) 2011-2012 DIATEAM. All rights reserved.
**
** This file is part of diamesh.
**
** Hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** Hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef DIAMESH_CORE_APPLICATION_H
#define DIAMESH_CORE_APPLICATION_H

#include <Ice/CommunicatorF.h>
#include <Ice/PropertiesF.h>
#include <QtCore/QCoreApplication>
#include <QtCore/QHash>
#include <signal.h>

class QFile;
class QSocketNotifier;
struct option;

namespace Diamesh {

class Instance;
class Logger;

class CoreApplication : public QCoreApplication
{
    Q_OBJECT

public:
    enum ExitStatus
    {
        SUCCESSFUL = 0,
        FAILED = 1,
        STOPPED = 127
    };

    typedef void (*SignalHandler)(int);

protected:
    explicit CoreApplication(int &argc, char **argv, Instance *instance);

public:
    virtual ~CoreApplication();

    static CoreApplication *create(int &argc, char **argv, const option *opts, const QHash<QString, QString> &appConfigHash = QHash<QString, QString>());
    static CoreApplication *instance();

    void logToFile(const QString &logFile = "");

    bool notify(QObject *receiver, QEvent *event);
    void restoreSigChldHandler();

protected slots:
    void _handleSighup();
    void _handleTermSignals();
    void _handleFatalSignals();

protected:
    void _log(QtMsgType type, const QString &msg) const;

private:
    bool _setPidFile(const QString &path);
    void _execCommand(const QString &cmd);
    void _init();
    void _daemonize();
    void _cleanup();
    void _displayMessage(const QString &message);
    void _displayStatus(int pid = 0);
    void _displayAbortCmd();

    static void _hupSignalHandler(int sig);
    static void _termSignalHandler(int sig);
    static void _fatalSignalHandler(int sig);
    static void _qtMessageHandler(QtMsgType type, const char *msg);

signals:
    void reload(); // SIGHUP on Unix
    void shutdown();

private:
    static int _sigHupFd[2];
    static int _sigTermFd[2];
    static int _sigFatalFd[2];
    static volatile sig_atomic_t _sigTermHandlerOnce;
    static volatile sig_atomic_t _sigFatalHandlerOnce;
    QSocketNotifier *_sigHupSocketNotifier;
    QSocketNotifier *_sigTermSocketNotifier;
    QSocketNotifier *_sigFatalSocketNotifier;

    QString _pidFilePath;
    QFile *_pidFile;
    bool _foreground;
    QString _command;
    Logger *_logger;

    // hack to workaround a bug in VBoxXPCOM.so loaded by libvirt
    // http://vbox.innotek.de/pipermail/vbox-dev/2009-September/001869.html
    SignalHandler _qt_handler;
};

} // namespace Diamesh

#endif // DIAMESH_CORE_APPLICATION_H
