/******************************************************************************
**
** Copyright (C) 2011-2012 DIATEAM. All rights reserved.
**
** This file is part of diamesh.
**
** Hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** Hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef HNSLOGGER_H
#define HNSLOGGER_H

#include <QtCore/QObject>

#include <QtCore/QFile>
#include <QtCore/QString>

#include <diamesh/constants.h>

#define HNS_LOG(CAT,MSG) do {Logger::instance()->log(CAT,MSG);} while(0);

namespace Diamesh {

class Logger : public QObject
{
    Q_OBJECT

public:
    enum LoggingMethod {
        DEFAULT,
        SYSTEM,
        FILE
    };

    virtual ~Logger();

    static Logger *create(const QString &logMethod = Constants::SETTINGS_LOG_DEFAULT,
                          const QString &logFileName = "",
                          int logLevel = QtDebugMsg);

    void logToFile(const QString &logFileName);
    void log(QtMsgType type, const QString &msg);

protected:
    void _initMethodDefault();
    void _initMethodSystem();
    void _initMethodFile();
    QByteArray _formatMessage(QtMsgType type, const QString &msg) const;
    void _openLogFileHandler();
    void _deleteLogFileHandler();
    Logger(LoggingMethod logMethod, QString logFile, int logLevel);

protected slots:
    void _writeDataToFile(QByteArray data);

protected:
    QFile *_logFile;
    LoggingMethod _logMethod;
    QString _logFileName;
    int _logLevel;
    QByteArray _syslogIdent;
};

} // namespace Diamesh

#endif // HNSLOGGER_H
