/******************************************************************************
**
** Copyright (C) 2011-2012 DIATEAM. All rights reserved.
**
** This file is part of diamesh.
**
** Hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** Hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef MASTER_H
#define MASTER_H

#include <diamesh/master.h>
#include <diamesh/node.h>
#include <QtCore/QList>
#include <QtCore/QObject>
#include <QtCore/QHash>
#include <QtCore/QSharedPointer>

#include <Glacier2/Glacier2.h>
#include <diamesh/authmanager.h>
#include <diamesh/sessionManager.h>
#include <diamesh/usermanager.h>
#include <diamesh/qiceobj_master.h>
#include <diamesh/accessrights.h>

class QSettings;

namespace Diamesh
{

class AuthManagerI;
class NodeInfoI;
class SessionManagerI;
class UserManagerI;
class SqlDatabase;
class SqlUserStorage;
class NodePoller;
class ServiceDetails;
class ACLManagerI;
class HSetManagerI;

class MasterI : public _Diamesh_Master
{
    Q_OBJECT

public:
    MasterI();
    ~MasterI();

    bool addNode(const NodePrx &, const std::string &key, const ::Ice::Current & = Ice::Current());
    void removeNode(const NodePrx &, const std::string &key, const ::Ice::Current & = Ice::Current());
    AuthManagerPrx getAuthManager(const ::Ice::Current & = Ice::Current());
    SessionManagerPrx getSessionManager(const ::Ice::Current & = Ice::Current());
    UserManagerPrx getUserManager(const ::Ice::Current & = Ice::Current());
    NodeInfoMap getNodeInfoMap(const ::Ice::Current & = Ice::Current());
    VersionInfo getDiameshVersionInfo(const ::Ice::Current & = Ice::Current());
    SystemInfo getSystemInfo(const ::Ice::Current & = Ice::Current());
    ServiceInfo getServiceInfo(const ::Ice::Current & = Ice::Current());
    std::string getName(const ::Ice::Current & = Ice::Current());
    ACLManagerPrx getACLManager(const ::Ice::Current & = Ice::Current());
    HSetManagerPrx getHSetManager(const ::Ice::Current & = Ice::Current());
    Ice::Long getCapabilitiesFlags(const ::Ice::Current & = Ice::Current());

    static MasterI *instance();

    MasterPrx proxy();

    bool start();

    Ice::ObjectAdapterPtr adapter();
    Ice::CommunicatorPtr communicator();
    AuthManagerI *authManager();
    SessionManagerI* sessionManager();
    ACLManagerI* aclManager();
    HSetManagerI* hSetManager();

    void setUserManager(UserManagerI*);
    void setUserManager(SqlUserStorage* sqlUserStorage);
    void setBaseUserManager();
    UserManagerI* userManager();

    SqlDatabase* db();

    QList<QSharedPointer<NodeInfoI> > nodeInfoList();
    QSharedPointer<NodeInfoI> nodeInfo(const QString &key);
    virtual void setDefaultRightsForNewUser(const UserPtr &user, const GroupPtr &group, const UserPtr &currentUser);
    virtual void setDefaultRightsForNewGroup(const GroupPtr &group, const UserPtr &currentUser);


public slots:
    void receivedDisconnectSlot(const QString & key);

private slots:
    void _nodeInfoAddedSlot(const QString &nodeKey);

signals:
    void nodeInfoAdded(QString nodeKey);

protected:
    virtual bool _init();
    QString _dbName();
    bool _loadAuthorizedNodeList();
    virtual bool _addNode(const NodePrx &node, const QString &key);
    bool _addNodeInfo(NodeInfoI *nodeInfo);
    void _removeNode(const QString &key);
    virtual void _initHSetManager();
    virtual void _registerACLTargetTypes();
    virtual void _initACLManager();
    virtual void _registerRights();

    Ice::CommunicatorPtr _communicator;
    Ice::ObjectAdapterPtr _adapter;

    Ice::Identity _id;
    QString _name;
    QHash<QString, QSharedPointer<NodeInfoI> > _nodeInfoHash;

    QList<QString> _authorizedNodes;

    AuthManagerI *_authManager;
    SessionManagerI *_sessionManager;
    UserManagerI *_userManager;
    ACLManagerI *_aclManager;
    HSetManagerI *_hSetManager;

    SqlDatabase *_db;

    static MasterI *_instance;
    QSettings *_settings;

    QMap<QString, NodePoller * > _mapPoller;

    ServiceDetails * _serviceDetails;

    qlonglong _flags;

};

}

#endif // MASTER_H
