/******************************************************************************
**
** Copyright (C) 2011-2012 DIATEAM. All rights reserved.
**
** This file is part of diamesh.
**
** Hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** Hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef DIAMESH_SERVICEDETAILS_H
#define DIAMESH_SERVICEDETAILS_H

#include <QtCore/QString>
#include <QtCore/QDateTime>
#include <QtCore/QReadWriteLock>
#include <diamesh/info.h>

class QSettings;

namespace Diamesh {

class ServiceDetails
{
public:
    explicit ServiceDetails(const QString &serviceName, Ice::ObjectAdapterPtr adapter);
    ~ServiceDetails();

    void setName(const QString &serviceName);
    void setShortName(const QString &shortName);
    void setDescription(const QString &description);

    std::string getName();
    VersionInfo getDiameshVersionInfo();
    SystemInfo getSystemInfo();
    ServiceInfo getServiceInfo();

protected:
    QByteArray _readFile(const QString &filename);
    QString _readCommandOutput(const QString &command);

    void _initInformation();
    void _initKernelInfo();
    void _initProcessorsInfo();
    void _initHostInfo();
    void _initIceMaxThreadInfo();
    void _initSystemUptimeInfo();
    void _initMemoryInfo();
    void _initCpuUsageInfo();
    void _initDiskInfo();

    void _refreshDynamicSystemInformation();
    void _refreshDynamicServiceInformation();

    void _parseSystemUptimeInfo();
    void _parseMemoryInfo();
    void _parseCpuUsageInfo();
    void _computeServiceUptimeInfo();
    void _parseDiskInfo();

private:
    QString _shortNameFromSettings();
    QString _descriptionFromSettings();

    Ice::ObjectAdapterPtr _adapter;
    QSettings *_settings;
    QDateTime _serviceUptime;
    VersionInfo _versionInformation;
    SystemInfo _systemInformation;
    ServiceInfo _serviceInformation;
    QReadWriteLock _systemInformationLock;
    QReadWriteLock _serviceInformationLock;
};

} // namespace Diamesh

#endif // DIAMESH_SERVICEDETAILS_H
