/******************************************************************************
**
** Copyright (C) 2011-2012 DIATEAM. All rights reserved.
**
** This file is part of diamesh.
**
** Hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** Hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef SESSION_MANAGER_H
#define SESSION_MANAGER_H

#include <Ice/Ice.h>
#include <Glacier2/Glacier2.h>
#include <QtCore/QHash>

#include <diamesh/master.h>
#include <diamesh/sessionManager.h>
#include <diamesh/qiceobj_sessionManager.h>

namespace Diamesh
{

class AuthManagerI;
class UserManagerI;

class SessionI : public Session
{
    friend class SessionManagerI;

public:
    SessionI(Ice::Long userId, const std::string &ipAddress, const Glacier2::SessionControlPrx & control);
    ~SessionI();

    MasterPrx getMaster(const::Ice::Current &);
    void destroy(const::Ice::Current &);

    SessionInfo getInfo(const::Ice::Current &);

    Ice::Long userId();
    const Glacier2::SessionControlPrx &control();

private:
    SessionInfo _info;
    Glacier2::SessionControlPrx _control;
    SessionPrx _prx;
};

class SessionManagerI : public _Diamesh_SessionManager
{
    Q_OBJECT

public:
    SessionManagerI(const QString &config, AuthManagerI *permissionsVerifier,
                    UserManagerI *userManager);
    ~SessionManagerI();

    Glacier2::SessionPrx create(const std::string &userName,
                                const Glacier2::SessionControlPrx & control,
                                const Ice::Current &current);
    Ice::Long userFromSessionId(const QString &sessionId);
    UserPtr currentUser(const Ice::Current &current);
    bool isClientCall(const Ice::Current &current);

    SessionList getSessions(const::Ice::Current & = Ice::Current());
    SessionPrx getSession(const std::string &sessionId, const::Ice::Current & = Ice::Current());
    SessionI* session(const QString &sessionId);

    void removeSession(Ice::Long userId, const QString &sessionId);

public slots:
    void disconnectUser(Ice::Long, const::Ice::Current & = Ice::Current());

protected:
    SessionI* _session(const QString &sessionId, const UserPtr &user);
    QReadWriteLock *_lock;
    QHash<QString, SessionI *> _sessionHash;
    QMultiHash<Ice::Long, QString> _userIdSessionsHash;
    AuthManagerI *_authManager;
    UserManagerI *_userManager;
};

}

#endif // SESSION_MANAGER_H
