/******************************************************************************
**
** Copyright (C) 2011-2012 DIATEAM. All rights reserved.
**
** This file is part of diamesh.
**
** Hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** Hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef CLIENT_H
#define CLIENT_H

#include <diamesh/master.h>
#include <QtCore/QString>
#include <QtCore/QObject>
#include <diamesh/sessionManager.h>
#include <Glacier2/Router.h>
#include <QtCore/QSharedPointer>
#include <QtCore/QStringList>
#include <Ice/Initialize.h>
#include <diamesh/proxies_usermanager.h>
#include <QtCore/QtConcurrentRun>

class QSettings;
class QThread;

namespace Diamesh
{

class ConnectException : public QtConcurrent::Exception
{
public:
    ConnectException(const QString &message);
    ~ConnectException() throw() {}
    QString message() const;
    void raise() const { throw *this; }
    Exception *clone() const { return new ConnectException (*this); }
private:
    QString _message;
};

class SessionPoller : public QObject
{
    Q_OBJECT

    friend class SessionHelper;

    SessionPoller(Glacier2::RouterPrx &router);
    void timerEvent(QTimerEvent *);

signals:
    void disconnected();

private slots:
    void _start();

private:
    int _timerId;
    const Glacier2::RouterPrx &_router;
};

class SessionHelper : public QObject
{
    Q_OBJECT

    friend class SessionHelperFactory;

private:
    SessionHelper(const Ice::InitializationData &initData);
    void _connect(const QString &userName, const QString &password);
    SessionPrx _createSession(Glacier2::RouterPrx router, const QString &userName, const QString &password);

public:
    ~SessionHelper();

    bool isConnected();
    const QString &userName();
    const UserPtr &user();

    const Ice::ObjectAdapterPtr &adapter();
    const Ice::CommunicatorPtr &communicator();
    const MasterPrx &master();
    const SessionPrx &session();

public slots:
    void destroy();

private slots:
    void _userChangedSlot(Ice::Long userId);
    void _updateUserSlot(::Diamesh::UserPtr user);

signals:
    void disconnected();

protected:
    bool _isConnected;
    Ice::InitializationData _initData;
    QThread *_pollerThread;
    SessionPoller *_poller;
    UserManagerQPrx _userManager;

    Ice::ObjectAdapterPtr _adapter;
    Ice::CommunicatorPtr _communicator;
    MasterPrx _master;
    Glacier2::RouterPrx _router;
    SessionPrx _session;
    QString _userName;
    Ice::Long _userId;
    UserPtr _user;
    bool _alreadyDestroyed;
};

class PasswordStorage
{
    friend class SessionHelperFactory;

private:
    PasswordStorage();

public:
    virtual void savePassword(const QString &userName, const QString &password);
    virtual void clearPassword(const QString &userName);
    virtual QString getPassword(const QString &userName);
};

class SessionHelperFactory : public QObject
{
    Q_OBJECT

public:
    SessionHelperFactory();
    ~SessionHelperFactory();

    bool updateConnectionHistory() const;
    bool storePassword() const;

    void clearUserNameHistory(const QString &userName);

    void setRouterHost(const QString &host);
    void setRouterPort(int port);

    const QStringList &recentServers() const;
    const QStringList &recentUsernames() const;
    QString getPassword(const QString &userName) const;

    QSharedPointer<SessionHelper> connect();
    QSharedPointer<SessionHelper> connect(QString &error);

    QSharedPointer<SessionHelper> connect(const QString &userName, const QString &password);
    QSharedPointer<SessionHelper> connect(const QString &userName, const QString &password, QString &error);

public slots:
    void setUpdateConnectionHistory(bool value);
    void setStorePassword(bool value);

private:
    QString _routerHost;
    int _port;

    QStringList _recentServers;
    QStringList _recentUsernames;

    bool _updateConnectionHistory;
    bool _storePassword;
};

}

#endif // CLIENT_H
