/******************************************************************************
**
** Copyright (C) 2011-2012 DIATEAM. All rights reserved.
**
** This file is part of diamesh.
**
** Hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** Hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef SQLUSERSTORAGE_H
#define SQLUSERSTORAGE_H

#include <diamesh/usermanagerI.h>
#include <QtCore/QHash>

namespace Diamesh
{

class SqlDatabase;

class SqlUserStorage : public UserStorage
{
public:
    virtual ~SqlUserStorage();

    static SqlUserStorage* create(SqlDatabase *db);

    void init();

    virtual Ice::Long addUser(const ::Diamesh::UserPtr &user, const QHash<QString, QVariant> &additionalFields=QHash<QString, QVariant>());
    bool deleteUser(::Ice::Long userId);
    virtual Diamesh::UserPtr getUser(::Ice::Long userId);
    Ice::Long getUserIdFromName(const QString &userName);
    virtual bool updateUser(const ::Diamesh::UserPtr &user, const QHash<QString, QVariant> &additionalFields=QHash<QString, QVariant>());

    Ice::Long addGroup(const ::Diamesh::GroupPtr &group);
    bool deleteGroup(::Ice::Long groupId);
    Diamesh::GroupPtr getGroup(::Ice::Long groupId);
    bool updateGroup(const ::Diamesh::GroupPtr &group);

    bool addUserToGroup(::Ice::Long userId, ::Ice::Long groupId);
    bool removeUserFromGroup(::Ice::Long userId, ::Ice::Long groupId);

    QList<Ice::Long> getUsers();
    QList<Ice::Long> getGroups();

protected:
    SqlUserStorage(SqlDatabase *db);
    virtual void _loadFromDb();

    SqlDatabase *_db;

    QHash<Ice::Long, Diamesh::UserPtr> _usersHash;
    QHash<QString, Ice::Long> _usersNamesIdsHash;
    QHash<Ice::Long, Diamesh::GroupPtr> _groupsHash;

    QReadWriteLock _usersLock;
    QReadWriteLock _groupsLock;
};

} // namespace Diamesh

#endif // SQLUSERSTORAGE_H
