// ****************************************************************************
//
// Copyright (C) 2011-2012 DIATEAM. All rights reserved.
//
// This file is part of diamesh.
//
// Hynesim is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License version 2 
// as published by the Free Software Foundation.
//
// Hynesim is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// ****************************************************************************

#ifndef USERMANAGER_ICE
#define USERMANAGER_ICE

#include <QtIce/qtice.ice>
#include <diamesh/types.ice>
#include <diamesh/accessrights.ice>

module Diamesh
{

class User extends ACLTarget
{
    string name;
    bool isEnabled;
    long createdOn;
    long modifiedOn;
    LongList groupList;
};

class Group extends ACLTarget
{
    string name;
    long createdOn;
    long modifiedOn;
    LongList userList;
};

["signal:void userChanged(Ice::Long)",
"signal:void userRemoved(Ice::Long)",
"signal:void userAdded(Ice::Long)",
"signal:void groupChanged(Ice::Long)",
"signal:void groupRemoved(Ice::Long)",
"signal:void groupAdded(Ice::Long)"]
interface UserManager extends QtIce::SignalEmitter
{
    long addUser(User u);
    LongList cloneUser(User u, int clonesNumber);
    bool deleteUser(long userId);
    User getUser(long userId);
    bool updateUser(User u);

    long addGroup(Group g);
    bool deleteGroup(long groupId);
    Group getGroup(long groupId);
    bool updateGroup(Group g);

    bool addUserToGroup(long userId, long groupId);
    bool removeUserFromGroup(long userId, long groupId);

    LongList getUsers();
    LongList getGroups();

    long getUserIdFromName(string userName);
};

};

#endif // USERMANAGER_ICE
