/******************************************************************************
**
** Copyright (C) 2011-2012 DIATEAM. All rights reserved.
**
** This file is part of diamesh.
**
** Hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** Hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef USERMANAGERI_H
#define USERMANAGERI_H

#include <diamesh/qiceobj_usermanager.h>
#include <QtCore/QList>
#include <Glacier2/Glacier2.h>

namespace Diamesh
{


class UserStorage;

class UserManagerI : public _Diamesh_UserManager
{
    Q_OBJECT
public:

    explicit UserManagerI(UserStorage *storage, Ice::ObjectAdapterPtr adapter, QObject *parent = 0);
    virtual ~UserManagerI();

    Ice::Long addUser(const UserPtr &user, const ::Ice::Current & = Ice::Current());
    LongList cloneUser(const UserPtr &user, int clonesNumber, const ::Ice::Current & = Ice::Current());
    bool deleteUser(::Ice::Long userId, const ::Ice::Current & = Ice::Current());
    UserPtr getUser(::Ice::Long userId, const ::Ice::Current & = Ice::Current());
    bool updateUser(const UserPtr &user, const ::Ice::Current & = Ice::Current());

    Ice::Long addGroup(const GroupPtr &group, const ::Ice::Current & = Ice::Current());
    bool deleteGroup(::Ice::Long groupId, const ::Ice::Current & = Ice::Current());
    GroupPtr getGroup(::Ice::Long groupId, const ::Ice::Current & = Ice::Current());
    bool updateGroup(const GroupPtr &group, const ::Ice::Current & = Ice::Current());

    bool addUserToGroup(::Ice::Long userId, ::Ice::Long groupId, const::Ice::Current & = Ice::Current());
    bool removeUserFromGroup(::Ice::Long userId, ::Ice::Long groupId, const::Ice::Current & = Ice::Current());

    LongList getUsers(const ::Ice::Current & = Ice::Current());
    LongList getGroups(const ::Ice::Current & = Ice::Current());

    Ice::Long getUserIdFromName(const std::string &userName, const::Ice::Current & = Ice::Current());

    UserManagerPrx proxy();

signals:
    void userDisabled(Ice::Long);
    void userAdminRightsRemoved(Ice::Long);
    void userRenamed(QString oldName, QString newName);
    void userRemoved(QString name);

protected:
    bool _userExists(const QString &userName);
    Ice::Long _addGroup(const GroupPtr &group, bool setDefaultSettings, const UserPtr &currentUser);

private:
    UserStorage *_storage;
    Ice::Identity _id;
    Ice::ObjectAdapterPtr _adapter;
};

class UserStorage
{
public:
    UserStorage() {}
    virtual ~UserStorage() {}

    virtual void init() = 0;

    virtual Ice::Long addUser(const UserPtr &user, const QHash<QString, QVariant> &additionalFields=QHash<QString, QVariant>()) = 0;
    virtual bool deleteUser(::Ice::Long userId) = 0;
    virtual UserPtr getUser(::Ice::Long userId) = 0;
    virtual Ice::Long getUserIdFromName(const QString &userName) = 0;
    virtual bool updateUser(const UserPtr &user, const QHash<QString, QVariant> &additionalFields=QHash<QString, QVariant>()) = 0;

    virtual Ice::Long addGroup(const GroupPtr &group) = 0;
    virtual bool deleteGroup(::Ice::Long groupId) = 0;
    virtual GroupPtr getGroup(::Ice::Long groupId) = 0;
    virtual bool updateGroup(const GroupPtr &group) = 0;

    virtual bool addUserToGroup(::Ice::Long userId, ::Ice::Long groupId) = 0;
    virtual bool removeUserFromGroup(::Ice::Long userId, ::Ice::Long groupId) = 0;

    virtual QList<Ice::Long> getUsers() = 0;
    virtual QList<Ice::Long> getGroups() = 0;
};

} // namespace Diamesh

#endif // USERMANAGERI_H
