set(ICE_MODULES
  Freeze
  Glacier2
  IceBox
  IceDB
  IceGridFreezeDB
  IceGrid
  IceGridSqlDB
  IcePatch2
  Ice
  IceSSL
  IceStormFreezeDB
  IceStormService
  IceStorm
  IceStormSqlDB
  IceUtil
  IceXML
  Slice
)

if(Ice_FIND_COMPONENTS)
  foreach(component ${Ice_FIND_COMPONENTS})
    string(TOUPPER ${component} _COMPONENT)
    set(ICE_USE_${_COMPONENT} 1)
  endforeach()
endif()

set(ICE_POSSIBLE_INCLUDE_PATHS
	$ENV{ICE_HOME}/include
)

find_path(ICE_INCLUDE_DIRS IceUtil/Config.h
	${ICE_POSSIBLE_INCLUDE_PATHS}
)

if(ICE_INCLUDE_DIRS)
	file(READ "${ICE_INCLUDE_DIRS}/IceUtil/Config.h" _ice_config_h_contents)
	string(REGEX REPLACE ".*#define ICE_STRING_VERSION \"([0-9.]+)\".*" "\\1" ICE_VERSION "${_ice_config_h_contents}")
endif()

set(ICE_POSSIBLE_SLICES_PATHS
	$ENV{ICE_HOME}/slice
	$ENV{ICE_HOME}/../slice
	/usr/share/Ice-${ICE_VERSION}/slice
)

find_path(ICE_SLICE_DIRS Ice/Identity.ice
	${ICE_POSSIBLE_SLICES_PATHS}
)

find_program(ICE_SLICE2CPP_EXE slice2cpp
	$ENV{ICE_HOME}/bin
)

set(ICE_POSSIBLE_LIB_DIRS
	$ENV{ICE_HOME}/lib
	$ENV{ICE_HOME}/lib64
)

foreach(module ${ICE_MODULES})
  string(TOUPPER ${module} _MODULE)
  find_library(ICE_${_MODULE}_LIBRARY ${module} ${ICE_POSSIBLE_LIB_DIRS})
  
  if(ICE_USE_${_MODULE})
    if(ICE_${_MODULE}_LIBRARY)
      set(ICE_LIBRARIES ${ICE_${_MODULE}_LIBRARY} ${ICE_LIBRARIES})
    else()
      message("module ${module} not found")
    endif()
  endif()
endforeach()

include(FindPackageHandleStandardArgs)
find_package_handle_standard_args(Ice DEFAULT_MSG ICE_INCLUDE_DIRS ICE_SLICE_DIRS ICE_SLICE2CPP_EXE ICE_ICE_LIBRARY)

set(ICE_GENERATED_OUTPUT_DIR "")
set(ICE_INCLUDE_DIR_PREFIX "")
set(ICE_SLICE_INCLUDE_DIRS "")

macro(ICE_WRAP_SLICE out_srcs out_hdrs target)
  set(slice_options)
  foreach(_inc_dir ${ICE_SLICE_INCLUDE_DIRS})
    set(slice_options "-I${_inc_dir}" ${slice_options})
  endforeach()
  set(slice_options "--underscore" ${slice_options})
  if(ICE_INCLUDE_DIR_PREFIX)
    set(slice_options --include-dir ${ICE_INCLUDE_DIR_PREFIX} ${slice_options})
  endif()

  if(ICE_GENERATED_OUTPUT_DIR)
    set(_output_dir "${ICE_GENERATED_OUTPUT_DIR}")
  else()
    set(_output_dir "${CMAKE_CURRENT_BINARY_DIR}")
  endif()
  
  foreach(it ${ARGN})
    get_filename_component(basename ${it} NAME_WE)
    set(out_src ${_output_dir}/${basename}.cpp)
    set(out_hdr ${_output_dir}/${basename}.h)
    get_filename_component(it ${it} ABSOLUTE)

    add_custom_command(OUTPUT ${out_src} ${out_hdr}
                       COMMAND ${ICE_SLICE2CPP_EXE} 
                       ARGS ${it} --output-dir ${_output_dir} -I${ICE_SLICE_DIRS} -I${CMAKE_SOURCE_DIR} ${slice_options}
                       DEPENDS ${it}
                       IMPLICIT_DEPENDS CXX ${it}
    )

    set(${out_srcs} ${${out_srcs}} ${out_src})
    set(${out_hdrs} ${${out_hdrs}} ${out_hdr})
    set(_target_deps ${_target_deps} ${out_src} ${out_hdr})
  endforeach()

  add_custom_target(${target} DEPENDS ${_target_deps})
endmacro()
