/******************************************************************************
**
** Copyright (C) 2011-2012 DIATEAM. All rights reserved.
**
** This file is part of diamesh.
**
** Hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** Hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#include "diamesh/filetransferI.h"
#include <IceUtil/IceUtil.h>
#include <QtCore/QDebug>
#include <QtCore/QFileInfo>

namespace Diamesh{

FileTransferI::FileTransferI(const QString filename, Ice::ObjectAdapterPtr adapter) :
    _Diamesh_FileTransfer(),_filePath(filename),_adapter(adapter)
{
    _id.name = IceUtil::generateUUID();
    _adapter->add(this, _id);
}
FileTransferI::~FileTransferI()
{
}

FileTransferPrx FileTransferI::proxy()
{
    return FileTransferPrx::uncheckedCast(_adapter->createProxy(_id));
}

Diamesh::ByteArray FileTransferI::readFile(Ice::Long offset, Ice::Long num, const Ice::Current &)
{
    QByteArray arrayFile;
    QFile file(_filePath);
    if (!file.exists()) {
        qCritical () << tr("The file %1 does not exist").arg(_filePath);
        return Diamesh::ByteArray();
    }
    if (file.size() == 0){
        qCritical()<<tr("The file %1 is totally empty").arg(_filePath);
        return Diamesh::ByteArray();
    }
    if (file.isOpen()){
        file.close();
    }
    if (!file.open(QIODevice::ReadOnly)) {
        qCritical () << tr("The file %1 could not be opened").arg(_filePath);
        return Diamesh::ByteArray();
    }
    if (!file.seek(offset)) {
        qCritical()<<tr("The Offset is not good");
        return Diamesh::ByteArray();
    }

    arrayFile = file.read(num);
    if (arrayFile.isEmpty()) {
        qDebug()<<tr("No data was read from file %1").arg(_filePath);
        return Diamesh::ByteArray();
    }

    file.close() ;
    const char * data = arrayFile.constData();
    return Diamesh::ByteArray(data, data+arrayFile.size());
}

Ice::Long FileTransferI::getSize(const Ice::Current &)
{
    QFile file (_filePath);
    if (!file.exists()) {
            qCritical () << tr("The file %1 does not exist").arg(_filePath);
            return 0;
        }
    return file.size();
}

std::string FileTransferI::getFileName (const Ice::Current &)
{
    QFileInfo fileInfo(_filePath);
    if (!fileInfo.exists()) {
            qCritical () << tr("The file %1 does not exist").arg(_filePath);
            return "";
        }
    return fileInfo.fileName().toStdString();
}

void FileTransferI::setFilePath(const std::string &filepath, const Ice::Current &)
{
    _filePath = QString::fromStdString(filepath);
}

}//Diamesh
