/******************************************************************************
**
** Copyright (C) 2011-2012 DIATEAM. All rights reserved.
**
** This file is part of diamesh.
**
** Hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** Hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#include <diamesh/nodeinfoI.h>
#include <diamesh/masterI.h>

namespace Diamesh {

NodeInfoI::NodeInfoI(const QString &nodeKey, const NodePrx &node)
    : _nodeKey(nodeKey)
    , _node(node)
    , _hasNodeBeenRemoved(0)
{
    _identity.name = QString("nodeInfo_%1").arg(nodeKey).toStdString();
}

std::string NodeInfoI::getName(const ::Ice::Current &)
{
    return _nodeKey.toStdString();
}

Diamesh::VersionInfo NodeInfoI::getDiameshVersionInfo(const ::Ice::Current &)
{
    return _node->getDiameshVersionInfo();
}

Diamesh::SystemInfo NodeInfoI::getSystemInfo(const ::Ice::Current &)
{
    return _node->getSystemInfo();
}

Diamesh::ServiceInfo NodeInfoI::getServiceInfo(const ::Ice::Current &)
{
    return _node->getServiceInfo();
}

const NodePrx &NodeInfoI::node()
{
    return _node;
}

Ice::Identity NodeInfoI::identity()
{
    return _identity;
}

void NodeInfoI::emitNodeRemoved()
{
    _hasNodeBeenRemoved = 1;
    emit nodeRemoved();
}

void NodeInfoI::destroy()
{
     emit requestDeletion(Diamesh::MasterI::instance()->adapter(), _identity);
}

bool NodeInfoI::hasNodeBeenRemoved()
{
    return _hasNodeBeenRemoved;
}

} // namespace Diamesh
