/******************************************************************************
**
** Copyright (C) 2011-2012 DIATEAM. All rights reserved.
**
** This file is part of diamesh.
**
** Hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** Hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#include "diamesh/nodepoller.h"
#include <Ice/LocalException.h>
#include <QDebug>

namespace Diamesh
{



NodePoller::NodePoller(const NodePrx &node, const QString &key):
  _key(key)
  , _mapNodePrx(node->ice_timeout(10000))
  , _executionThread(new QThread())
  , _stop(false)
{
}

NodePoller::~NodePoller()
{
    _executionThread->quit();
}



void NodePoller::execute()
{
    QObject::connect(this, SIGNAL(killTimerSignal()), this, SLOT(_killTimerSlot()));
    QMetaObject::invokeMethod(this, "_moveToExecutionThread");
    QObject::connect(_executionThread, SIGNAL(started()), this, SLOT(_start()));
    _executionThread->start();
}


void NodePoller::_moveToExecutionThread()
{
    this->moveToThread(_executionThread);
}


void NodePoller::_start()
{
    _stop = false;
    _timerId = startTimer(1000);
}


void NodePoller::timerEvent(QTimerEvent *)
{
    _mutexStop.lock();
    if(!_stop) {
        QString error("");
        try {
            _mapNodePrx->ice_ping();
        }
        catch(const Ice::ConnectionLostException & e ){
            error =  QString(e.what());

        } catch(const Ice::ConnectionRefusedException & e){
            error =  QString(e.what());
        } catch(...) {
            error = tr("connection time out");
        }
        if(!error.isEmpty()){
            qWarning() << tr("NodePoller has caught an exception on node %1: %2").arg(_key).arg(error);
            _stop = true;
            emit nodeDisconnected(_key);
        }
    }
    _mutexStop.unlock();
}

void NodePoller::stop(){
    emit killTimerSignal();
    _mutexStop.lock();
    _stop = true;
    _mutexStop.unlock();
}

void NodePoller::_killTimerSlot()
{
    killTimer(_timerId);
}


}
