/******************************************************************************
**
** Copyright (C) 2011-2012 DIATEAM. All rights reserved.
**
** This file is part of diamesh.
**
** Hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** Hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#include <diamesh/application.h>
#include <diamesh/logger.h>
#include <diamesh/diamesh.h>

#include <getopt.h>
#include <unistd.h>
#include <errno.h>
#include <pwd.h>
#include <signal.h>
#include <sys/socket.h>
#include <stdlib.h>

#include <Ice/Properties.h>
#include <Ice/Ice.h>
#include <QtCore/QDateTime>
#include <QtCore/QSettings>
#include <QtCore/QStringList>
#include <QtCore/QTextCodec>
#include <QtCore/QFile>
#include <QtCore/QSocketNotifier>
#include <QtCore/QVector>
#include <QtCore/QLibraryInfo>
#include <QtCore/QLocale>
#include <QtCore/QTranslator>
#include <QtCore/QDebug>

struct option app_options[] = {
    { "help", no_argument, NULL, 'h' },
    { NULL, 0, NULL, 0 }
};

namespace Diamesh {

Application::Application(int &argc, char **argv, Instance *diameshInstance) :
    QApplication(argc, argv)
{
    QHash<QString, QString> appConfigHash = diameshInstance->appConfigHash();
    setApplicationName(appConfigHash.value(Diamesh::Constants::APP_CONFIG_NAME));
    setOrganizationName(appConfigHash.value(Diamesh::Constants::APP_CONFIG_ORGANIZATION));
    QTextCodec::setCodecForCStrings(QTextCodec::codecForName("UTF-8"));
    diameshInstance->loadTranslations();
    _diameshGuiTranslator = new QTranslator();
    QString language = diameshInstance->applicationLanguage();
    QString translationName = QString("diameshGui_%1").arg(language);
    if (_diameshGuiTranslator->load(translationName, QString(":/i18n")))
        installTranslator(_diameshGuiTranslator);
    else
        qCritical() << tr("Unable to load the translation file %s"), qPrintable(translationName);
}

Application::~Application()
{
    delete _diameshGuiTranslator;
}

Application *Application::create(int &argc, char **argv, const option *opts, const QHash<QString, QString> &appConfigHash)
{
    QVector<option> options;
    for (const option *o = app_options ; o->name ; ++o)
        options.append(*o);
    if (opts) {
        for (const option *o = opts ; o->name ; ++o)
            options.append(*o);
    }

    Instance *diameshInstance = Diamesh::initialize(argc, argv, options, appConfigHash);

    return new Application(argc, argv, diameshInstance);
}

Application *Application::instance()
{
    return qobject_cast<Application *>(QCoreApplication::instance());
}

bool Application::notify(QObject *receiver, QEvent *event)
{
    bool ret = false;
    try {
        ret = QApplication::notify(receiver, event);
    } catch(...) {
        Diamesh::instance()->handleException();
    }

    return ret;
}

}
