/******************************************************************************
**
** Copyright (C) 2011-2012 DIATEAM. All rights reserved.
**
** This file is part of diamesh.
**
** Hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** Hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#include <diamesh/coreapplication.h>
#include <diamesh/authmanager.h>
#include <diamesh/sessionhelper.h>
#include <diamesh/sessionManager.h>
#include <diamesh/usermanager.h>

using namespace Diamesh;

int main(int argc, char **argv)
{
    QSharedPointer<CoreApplication> app(CoreApplication::create(argc, argv, NULL));
    Q_UNUSED(app);

    SessionHelperFactory sessionFactory;
    QSharedPointer<SessionHelper> sessionHelper = sessionFactory.connect("admin", "admin");
    if (!sessionHelper)
        return 1;

    MasterPrx master = sessionHelper->master();
    UserManagerPrx userManager = master->getUserManager();
    AuthManagerPrx permissionsVerifier = master->getAuthManager();

    UserPtr fooUser = new User();
    fooUser->name = "foo";
    fooUser->isEnabled = true;
    fooUser->hSetId = 1;
    fooUser->id = userManager->addUser(fooUser);
    permissionsVerifier->setPassword("foo", "bar");

    Ice::Long adminId = sessionHelper->session()->getInfo().userId;
    UserPtr adminUser = userManager->getUser(adminId);
    if (userManager->deleteUser(adminId)) {
        qWarning("We shouldn't be able to delete our own account");
        return 1;
    }

    adminUser->isEnabled = false;
    if (userManager->updateUser(adminUser)) {
        qWarning("We shouldn't be able to disable our own account");
        return 1;
    }

    QSharedPointer<SessionHelper> fooSessionHelper = sessionFactory.connect("foo", "bar");
    UserManagerPrx fooUserManager = fooSessionHelper->master()->getUserManager();

    UserPtr dummyUser = new User();
    dummyUser->name = "dummy";
    dummyUser->hSetId = 1;
    if (fooUserManager->addUser(dummyUser)) {
        qWarning("We shouldn't be able to add a user");
        return 1;
    }

    if (fooUserManager->deleteUser(adminId)) {
        qWarning("We shouldn't be able to delete a user");
        return 1;
    }

    if (fooUserManager->updateUser(adminUser)) {
        qWarning("We shouldn't be able to update an other user");
        return 1;
    }

    fooUser->isEnabled = false;
    if (fooUserManager->updateUser(fooUser)) {
        qWarning("We shouldn't be able to disable our own account");
        return 1;
    }

    GroupPtr group = new Group();
    group->name = "dummy";
    if (fooUserManager->addGroup(group)) {
        qWarning("We shouldn't be able to add a group");
        return 1;
    }

    group = fooUserManager->getGroup(1);
    if (fooUserManager->deleteGroup(group->id)) {
        qWarning("We shouldn't be able to delete a group");
        return 1;
    }

    if (fooUserManager->updateGroup(group)) {
        qWarning("We shouldn't be able to update a group");
        return 1;
    }

    return 0;
}
