/******************************************************************************
**
** Copyright (C) 2011-2012 DIATEAM. All rights reserved.
**
** This file is part of diamesh.
**
** Hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** Hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#include <diamesh/coreapplication.h>
#include <diamesh/authmanager.h>
#include <diamesh/sessionhelper.h>
#include <diamesh/sessionManager.h>
#include <diamesh/usermanager.h>
#include <diamesh/filetransfer.h>
#include <diamesh/filetransferI.h>
#include "diamesh/proxies_filetransfer.h"
#include "diamesh/filetransferclient.h"
#include <testmaster.h>

#include <QtCore/QFile>
#include <QtCore/QDebug>
#include <QtCore/QString>
using namespace Diamesh;

int main(int argc, char **argv)
{
    QSharedPointer<CoreApplication> app(CoreApplication::create(argc, argv, NULL));
    Q_UNUSED(app);

    SessionHelperFactory sessionFactory;
    QSharedPointer<SessionHelper> sessionHelper = sessionFactory.connect("admin", "admin");
    if (!sessionHelper)
        return 1;

    Diamesh::TestMasterPrx master = Diamesh::TestMasterPrx::uncheckedCast(sessionHelper->session()->getMaster());
    FileTransferPrx file = master->createFileTransfer(""); //You can also put the filepath in the constructor
    file->setFilePath("./Makefile");

    FileTransferClient getFileObject(file,sessionHelper->adapter());
    getFileObject.start(QString("./MakefileByteLess"),12,1048546); /* 1048546 is the maximum size allowed by Ice for transferring files
                                                                      The file "MAkefileByteLess will have 12 bytes less, due to the offset != 0 */
    QEventLoop l;
    QObject::connect(&getFileObject, SIGNAL(transferIsFinished()), &l, SLOT(quit()));
    l.exec();

    QFileInfo makefileByteLess(QString("./MakefileByteLess"));
    QFileInfo makefileOriginal(QString("./Makefile"));
    if (makefileByteLess.size() == makefileOriginal.size()){
        qCritical()<< "The two files should not have identical size";
    }

    FileTransferPrx file2= master->createFileTransfer("");
    file2->setFilePath("./007client");
    FileTransferClient getFileObject2(file2,sessionHelper->adapter());
    getFileObject2.start(QString("./007clientCopy"));

    QObject::connect(&getFileObject2, SIGNAL(transferIsFinished()), &l, SLOT(quit()));
    l.exec();

    QFileInfo clientCopy(QString("./007clientCopy"));
    QFileInfo clientOriginal(QString("./007client"));
    if (clientCopy.size() != clientOriginal.size()){
        qCritical()<<"Files 007client and 007clientCopy should have the same size";
    }

    return 0;
}
