/******************************************************************************
**
** Copyright (C) 2011-2012 DIATEAM. All rights reserved.
**
** This file is part of diamesh.
**
** Hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** Hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#include <diamesh/coreapplication.h>
#include <diamesh/diamesh.h>
#include <diamesh/aclmanagerI.h>
#include <diamesh/accessrightshelper.h>

#include "testmaster.h"
#include <QtCore/QStringList>
#include <QtCore/QDebug>

int main(int argc, char **argv)
{
    Diamesh::CoreApplication * app = Diamesh::CoreApplication::create(argc, argv, NULL);
    Diamesh::TestMaster master;
    master.setBaseUserManager();
    master.start();
    Diamesh::ACLManagerI *manager = master.aclManager();
    if (!manager)
        qFatal("Could not access the ACL Manager");

    Diamesh::AccessRightList list = manager->getRegisteredRights();
    QList<Diamesh::AccessRightPtr> rightList = QList<Diamesh::AccessRightPtr>::fromStdList(list);



    qDebug() << QString("ACL Manager initiliazed with %1 rules").arg(QString::number(rightList.count()));
    foreach(const Diamesh::AccessRightPtr &right, rightList) {
        qDebug() << QString("Right '%1' (Id:%2) set with parent %3 and %4 children.")
                    .arg(QString::fromStdString(right->name))
                    .arg(QString::number(right->id))
                    .arg(QString::number(right->parent))
                    .arg(QString::number(right->children.size()));
    }

    Diamesh::AccessRightsHelper *localHelper = new Diamesh::AccessRightsHelper(rightList.toStdList(), manager->getTargetTypes());

    Diamesh::LongList aclRightList;
    bool result;

    aclRightList.push_back(2);
    result = manager->setACL("obj.1", "u.1", aclRightList, 1);
    if (!result)
        qWarning() << "Failed to set first ACL";
    else {
        Diamesh::AccessControlList acl = manager->getACL("obj.1", "u.1");
        qDebug() << QString("First ACL set for '%1/%2'")
                    .arg(QString::fromStdString(acl.target))
                    .arg(QString::fromStdString(acl.subject));
        QStringList rightNames = localHelper->nameListFromRights(acl.rights);
        foreach(const QString &rightName, rightNames)
            qDebug() << "with right: " << rightName;
    }

    aclRightList.clear();
    aclRightList.push_back(3);
    aclRightList.push_back(4);
    manager->setACL("obj.1", "u.2", aclRightList, 1);
    if (!result)
        qWarning() << "Failed to set second ACL";
    else {
        Diamesh::AccessControlList acl = manager->getACL("obj.1", "u.2");
        qDebug() << QString("Second ACL set for '%1/%2'")
                    .arg(QString::fromStdString(acl.target))
                    .arg(QString::fromStdString(acl.subject));
        QStringList rightNames = localHelper->nameListFromRights(acl.rights);
        foreach(const QString &rightName, rightNames)
            qDebug() << "with right: " << rightName;
    }

    aclRightList.clear();
    aclRightList.push_back(3);
    aclRightList.push_back(4);
    manager->setACL("obj.1", "u.1", aclRightList, 1);
    if (!result)
        qWarning() << "Failed to update first ACL";
    else {
        Diamesh::AccessControlList acl = manager->getACL("obj.1", "u.1");
        qDebug() << QString("First ACL updated for '%1/%2'")
                    .arg(QString::fromStdString(acl.target))
                    .arg(QString::fromStdString(acl.subject));
        QStringList rightNames = localHelper->nameListFromRights(acl.rights);
        foreach(const QString &rightName, rightNames)
            qDebug() << "with right: " << rightName;
    }

    aclRightList.clear();
    manager->setACL("obj.1", "u.2", aclRightList, 1);
    if (!result)
        qWarning() << "Failed to update second ACL with empty list";
    else {
        Diamesh::AccessControlList acl = manager->getACL("obj.1", "u.2");
        qDebug() << QString("Second ACL updated for '%1/%2'")
                    .arg(QString::fromStdString(acl.target))
                    .arg(QString::fromStdString(acl.subject));
        QStringList rightNames = localHelper->nameListFromRights(acl.rights);
        foreach(const QString &rightName, rightNames)
            qDebug() << "with right: " << rightName;
    }

    result = manager->removeACL("obj.1", "u.1");
    if (!result)
        qWarning() << "Failed to delete first ACL";
    else {
        Diamesh::AccessControlList acl = manager->getACL("obj.1", "u.1");
        qDebug() << QString("First ACL updated for '%1/%2'")
                    .arg(QString::fromStdString(acl.target))
                    .arg(QString::fromStdString(acl.subject));
        QStringList rightNames = localHelper->nameListFromRights(acl.rights);
        foreach(const QString &rightName, rightNames)
            qDebug() << "with right: " << rightName;
    }

    return app->exec();
}
