import sys, os, re, sqlite3
from os.path import dirname
import expect;

commonPath = dirname(__file__)
basePath = dirname(dirname(commonPath))
prefix = ""

def getDefaultArgs():
    args = ["--Ice.Warn.Connections=1"]
    args.append("--Ice.Default.Host=127.0.0.1")
    return args

def getTestExe(exe):
    return "./" + prefix + exe

def spawn(exe, args=[], startReader=True):
    cmd = exe + " " + " ".join(args)
    print "spawn:", cmd
    return expect.Expect(cmd, startReader=startReader)

def spawnServer(exe, args=[], count=1, adapter=None):
    server = spawn(exe, args)
    server.trace([re.compile("[^\n]+ ready\n")])
    if adapter:
        server.expect("%s ready\n" % adapter)
    else:
        while count > 0:
            server.expect("[^\n]+ ready\n", 2)
            count -= 1
    return server

def startServer(exe, args=[], adapter=None, count=1):
    args.append("--Ice.PrintAdapterReady=1")
    args.append("--Ice.ThreadPool.Server.Size=1")
    args.append("--Ice.ThreadPool.Server.SizeMax=3")
    args.append("--Ice.ThreadPool.Server.SizeWarn=0")
    args.extend(getDefaultArgs())
    return spawnServer(exe, args, count, adapter)

def startRouter():
    args = ['--Glacier2.Client.Endpoints="tcp -p 4063"']
    args.append('--Glacier2.Server.Endpoints="tcp"')
    args.append('--Glacier2.PermissionsVerifier=PermissionsVerifier:"tcp -p 5656"')
    args.append('--Glacier2.SessionManager=SessionManager:"tcp -p 5656"')
    args.append('--Glacier2.Client.ForwardContext=1')
    args.append('--Glacier2.Server.ForwardContext=1')
    args.append('--Glacier2.AddConnectionContext=2')
    exe = "glacier2router"
    if "ICE_HOME" in os.environ:
        exe = os.environ["ICE_HOME"] + "/bin/" + exe
    return startServer(exe, args)

def startMaster():
    args = ["--config=" + sys.path[0] + "/master.ini"]
    return startServer(getTestExe("master"), args)

def startNode():
    args = ["--config=" + sys.path[0] + "/node.ini"]
    return startServer(getTestExe("node"), args)

def startClient():
    args = ["--config=" + sys.path[0] + "/client.ini"]
    args.append('--Ice.Default.Router="Glacier2/router:tcp -p 4063"')
    args.append("--Ice.ACM.Client=0")
    args.extend(getDefaultArgs())
    client = spawn(getTestExe("client"), args)
    client.trace()
    return client

def createDB(sqlFile, dbFile='/tmp/diamesh.db'):
    if os.path.exists(dbFile):
        os.remove(dbFile)
    loadSQLFile(sqlFile, dbFile)

def createMasterDB(dbFile='/tmp/diamesh.db'):
    createDB(os.path.join(basePath, 'data/users.sql'), dbFile)
    loadSQLFile(os.path.join(basePath, 'data/auth.sql'), dbFile)
    loadSQLFile(os.path.join(basePath, 'data/accessrights.sql'), dbFile)
    loadSQLFile(os.path.join(commonPath, 'fillDb.sql'), dbFile)

def loadSQLFile(sqlFile, dbFile='/tmp/diamesh.db'):
    con = sqlite3.connect(dbFile)
    cur = con.cursor()
    cur.executescript(open(sqlFile).read())
    con.commit()
    cur.close()
    con.close()
