macro(hynesim_get_absolute_file_names outputList)
  foreach(_file ${ARGN})
    get_filename_component(_dummy ${_file} ABSOLUTE)
    set(${outputList} ${${outputList}} ${_dummy})
  endforeach()
endmacro(hynesim_get_absolute_file_names)

macro(hynesim_generate_doc)
  get_filename_component(HYNESIM_DOC_SRC ${HYNESIM_DOC} ABSOLUTE)
  if(NOT EXISTS ${HYNESIM_DOC_SRC})
    message(WARNING "Documentation directory ${HYNESIM_DOC_SRC} doesn't exist")
  endif()
  set(HYNESIM_DOC_DST ${CMAKE_BINARY_DIR}/doc)
  file(MAKE_DIRECTORY ${HYNESIM_DOC_DST})
  file(GLOB HYNESIM_DOC_ITEMS ${HYNESIM_DOC_SRC}/*.qhp)
  add_custom_target(doc_collection)
  foreach(_doc_item_path ${HYNESIM_DOC_ITEMS})
    get_filename_component(_doc_item ${_doc_item_path} NAME_WE)
    add_custom_target(doc_item_${_doc_item}
      COMMAND qhelpgenerator ${_doc_item_path} -o ${_doc_item}.qch
      COMMENT "Running qhelpgenerator on ${_doc_item_path}"
      WORKING_DIRECTORY ${HYNESIM_DOC_DST}
    )
    add_dependencies(doc_collection doc_item_${_doc_item})
  endforeach()
  install(CODE "execute_process(COMMAND ${CMAKE_BUILD_TOOL} doc_collection WORKING_DIRECTORY \"${CMAKE_CURRENT_BINARY_DIR}\")")
  install(DIRECTORY ${CMAKE_BINARY_DIR}/doc DESTINATION share/hynesim FILES_MATCHING PATTERN "*.qch")
endmacro(hynesim_generate_doc)
