// ****************************************************************************
//
// Copyright (C) 2003-2012 DIATEAM. All rights reserved.
//
// This file is part of Hynesim.
//
// Hynesim is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License version 2 
// as published by the Free Software Foundation.
//
// Hynesim is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// ****************************************************************************

#ifndef CATALOG_ICE
#define CATALOG_ICE

#include <QtIce/qtice.ice>
#include <common/entry.ice>
#include <common/types.ice>
#include <diamesh/types.ice>
#include <diamesh/accessrights.ice>

[["cpp:include:list"]]

module Hns
{

enum LockType {
    LOCK_TYPE_NOLOCK,
    LOCK_TYPE_NORMAL,
    LOCK_TYPE_CLONE,
    LOCK_TYPE_EXPORT
};

dictionary<long, LockType> LockTypeLongMap;

["signal:void entryChanged(Ice::Long)",
"signal:void entryRemoved(Ice::Long)",
"signal:void entryAdded(Ice::Long)",
"signal:void entryLocked(Ice::Long, Ice::Int)",
"signal:void entryUnlocked(Ice::Long)",
"signal:void entryACLsChanged(Ice::Long, Diamesh::ACLList)"]
interface Catalog extends QtIce::SignalEmitter
{
    Diamesh::LongList getEntries();
    Entry getEntry(long entryId);
    long addEntry(Entry e);
    bool deleteEntry(Entry e);
    bool updateEntry(Entry e);

    bool entryIsLocked(long entryId);
    LockTypeLongMap getLockedEntries();

    Diamesh::StringStringMap getFileStorages();

    Diamesh::ACLList getACLsForCatalogItem(long catalogItemId);
    bool setACLsForCatalogItem(long catalogItemId, Diamesh::ACLList aclList) throws RightError;

    string getUnusedUUID();
};

};

#endif // CATALOG_ICE
