// ****************************************************************************
//
// Copyright (C) 2003-2012 DIATEAM. All rights reserved.
//
// This file is part of Hynesim.
//
// Hynesim is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License version 2 
// as published by the Free Software Foundation.
//
// Hynesim is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// ****************************************************************************

#ifndef DOMAIN_ENTRY_ICE
#define DOMAIN_ENTRY_ICE

#include <common/entityentry.ice>
#include <common/snapshot.ice>

#include <diamesh/types.ice>

module Hns
{

struct Netcard
{
    long id;
    string type;
    string macAddress;
    string source;
    string target;
    string model;
    string description;
};

struct Disk 
{
    string type;
    string device;
    string source;
    string target;
    string targetBus;
    string driverName;
    string driverType;
};

struct OperatingSystem
{
    string type;
    string arch;
    string machine;
    string kernel;
    string initrd;
    string cmdline;
};

struct Graphics
{
    string keymap;
};

struct Video
{
    string videoModelType;
};

struct Input
{
    string type;
    string bus;
};

dictionary<long, Netcard> NetcardMap;

["cpp:type:std::list<Hns::Disk>"]
sequence<Disk> DiskList;

["cpp:type:std::list<Hns::Input>"]
sequence<Input> InputList;

class DomainEntry extends EntityEntry
{
    string type;
    OperatingSystem os;
    int memory;
    int vcpu;
    bool pae;
    bool acpi;
    bool apic;
    bool immutable = false;
    long aminterface;
    NetcardMap netcards;
    long netcardAutoId = 0;
    DiskList disks;
    bool sharedFolder = false;
    Graphics graphic;
    InputList inputs;
    Video vid;
    SnapshotList snapshots;
    string currentSnapshotName;
    string nodeKey;
    string baseUuid = "";
    Diamesh::StringList diffUuids;
    int usbRedirectionCount = 0;
    string usbControllerType;
};

};

#endif // DOMAIN_ENTRY_ICE

