// ****************************************************************************
//
// Copyright (C) 2003-2012 DIATEAM. All rights reserved.
//
// This file is part of Hynesim.
//
// Hynesim is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License version 2 
// as published by the Free Software Foundation.
//
// Hynesim is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// ****************************************************************************

#ifndef ENTITY_ICE
#define ENTITY_ICE

#include <QtIce/qtice.ice>
#include <diamesh/types.ice>
#include <diamesh/filetransfer.ice>
#include <common/capturejoboptions.ice>
#include <common/types.ice>
#include <common/snapshot.ice>
module Hns
{

class EntityEntry;
interface Tracer;
interface CaptureJob;

enum Status { UNDEFINED, STOPPED, STARTED, BUSY, PAUSED, SAVED, UNIMPLEMENTED };


enum RemoteDisplayType
{
    NoViewer,
    VncViewer,
    RdpViewer,
    SpiceViewer,
    TelnetViewer
};

struct RemoteDisplayInfo
{
    string host = "";
    int port = 0;
    RemoteDisplayType type = NoViewer;
    string password = "";
    bool tls = false;
    string ca = "";
    string certSubject ="";
};

/*Entities flags for the "capabilities" they have*/
const int NET_CAPTURE = 0x1;
const int REMOTE_DISPLAY = 0x2;
const int SNAPSHOT_FLAG = 0x4;
const int MEMORY_DUMP_FLAG = 0x8;

["cpp:type:std::list<Hns::Status>"]
sequence<Status> StatusList;

struct Plug
{
    long id = 0;
    string name;
    bool isPlugged = false;
};

dictionary<long, Plug> PlugMap;
dictionary<long, string> mapLongString;


["signal:void statusChanged(Ice::Int, Ice::Long)",
"signal:void implementFromModelChanged(bool)",
"signal:void plugListChanged(::Hns::PlugMap)",
"signal:void plugUsed(Ice::Long)",
"signal:void plugReleased(Ice::Long)",
"signal:void entryChanged()",
"signal:void interfaceUsed(Ice::Int, Ice::Int)",
"signal:void interfaceReleased(Ice::Int, Ice::Int)",
"signal:void captureJobFileDelete(QString)",
"signal:void peerLeft(QString)",
"signal:void peerJoined(QString)",
"signal:void capturableInterfacesListChanged(::Diamesh::IntList)",
"signal:void currentSnapshotChanged(::Hns::Snapshot)",
"signal:void allowedStatusListChanged()"]
interface Entity extends QtIce::SignalEmitter
{
    EntityEntry getEntry();
    Tracer *getTracer();
    bool updateEntry(EntityEntry e);
    ["amd"] bool define(string nodeId);
    bool start();
    bool stop();
    bool save();
    bool restore();
    bool suspend();
    bool resume();
    bool isDefinable();
    bool isDefined();
    bool undefine();
    bool destroy();
    long getId();
    PlugMap getPlugs();
    PlugMap getAvailablePlugs();
    string getPlugName(long plugId);
    idempotent Status getStatus();
    StatusList getAllowedStatuses();
    PcapInfoList getPcapInfoList();
    void setPcapInfoList(PcapInfoList listPcapInfo);
    void eraseCaptureFile(string file);
    int createCaptureJob(CaptureJobOptions c);
    bool cancelCaptureJob(int jobId);
    Diamesh::StringList getCaptureFiles();
    Diamesh::FileTransfer* downloadCaptureFile(string file);
    string getNodeName();
    ::Diamesh::IntList getAvailablePlugsForCapture();
    int getFlags();
    mapLongString getCaptureJobIdPath();
    PcapInfo getPcapInfoByJobId(int jobId);
    idempotent RemoteDisplayInfo getRemoteDisplayInfo();
    bool isImplementFromModel();
    void checkCaptureFile();
    bool createSnapshot(Hns::Snapshot snapshot);
    bool revertSnapshot(string snapshotName);
    bool deleteSnapshot(string snapshotName);
    Hns::Snapshot getSnapshot(string snapshotName);
    string getCurrentSnapshotName();
    IntIntMap getRunningCaptureJobs();
};

};

#endif // ENTITY_ICE
