/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// hynesim common headers
#include "common/fixutils.h"
#include "common/initializer.h"

// Generated hynesim headers
#include "domainentry.h"
#include "switchentry.h"
#include "dynamipsrouterentry.h"

// Qt headers
#include <QString>
#include <QMap>
#include <QHash>

namespace Hns {

// define here the methods which check for each type of entity whether an entry
bool _canPlugDomainEntry(const::Hns::EntityEntryPtr &entityEntry, int numPlug);
bool _canPlugSwitchEntry(const::Hns::EntityEntryPtr &entityEntry, int numPlug);
bool _canPlugDynamipsEntry(const::Hns::EntityEntryPtr &entityEntry, int numPlug);

typedef bool (*CanPlug)(const::Hns::EntityEntryPtr &entityEntry, int numPlug);

HNS_STATIC_HASH_INITIALIZER(_canPlugHash, CanPlugHash, QString, CanPlug)
CanPlugHash::CanPlugHash()
{
    insert(QString::fromStdString(DomainEntry::ice_staticId()), &_canPlugDomainEntry);
    insert(QString::fromStdString(SwitchEntry::ice_staticId()), &_canPlugSwitchEntry);
    insert(QString::fromStdString(DynamipsRouterEntry::ice_staticId()), &_canPlugDynamipsEntry);
}

bool canPlugEntry(const::Hns::EntityEntryPtr &entityEntry, int numPlug)
{
    QString className = QString::fromStdString(entityEntry->ice_id());
    if(_canPlugHash()->contains(className)) {
        return _canPlugHash()->value(className)(entityEntry, numPlug);
    }
    return (numPlug == 0);
}

/*******************************************************************************
* Custom method for DomainEntry
*/
bool _canPlugDomainEntry(const::Hns::EntityEntryPtr &entityEntry, int numPlug)
{
    DomainEntryPtr domainEntry = DomainEntryPtr::dynamicCast(entityEntry);
    QMap< ::Ice::Long, ::Hns::Netcard> mapCard(domainEntry->netcards);
    return (mapCard.keys().contains(numPlug) && domainEntry->aminterface != numPlug);
}

/*******************************************************************************
* Custom method for SwitchEntry
*/
bool _canPlugSwitchEntry(const::Hns::EntityEntryPtr &entityEntry, int numPlug)
{
    SwitchEntryPtr switchEntry = SwitchEntryPtr::dynamicCast(entityEntry);
    return (switchEntry->ports > numPlug);

}

/*******************************************************************************
* Custom method for DynamipsRouterEntry
*/
bool _canPlugDynamipsEntry(const::Hns::EntityEntryPtr &entityEntry, int numPlug)
{
    DynamipsRouterEntryPtr dynamipsEntry = DynamipsRouterEntryPtr::dynamicCast(entityEntry);
    int idxPlug = 0;
    ModulePositionMap::iterator itorPositionMap= dynamipsEntry->modulesMap.begin();
    while (itorPositionMap != dynamipsEntry->modulesMap.end()) {
            for (int i = 0 ; i < itorPositionMap->second.interfaces ; i++) {
                if (numPlug <= idxPlug++ ) {
                    return true;
                }
            }
        itorPositionMap++;
    }
    return false;
}

} // namespace Hns
