/******************************************************************************
**
** Copyright (C) 2013 DIATEAM. All rights reserved.
**
** This file is part of Hynesim.
**
** Hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** Hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef INITIALIZER_H
#define INITIALIZER_H

#define HNS_PARENT_NAME(a) a##_parent //Generate a string a_parent
/*
 *Declare a class TYPENAME, which derives from PARENTTYPE and its
 *constructor.
 *You have to implement the constructor
 *It also creates a variable VARNAME from type TYPENAME with Q_GLOBAL_STATIC
 *
 *Other macros are helper for specific types (QHash, QSet, ...)
*/
#define HNS_STATIC_GENERIC_INITIALIZER(VARNAME, TYPENAME, PARENTTYPE)           \
    class TYPENAME: public PARENTTYPE                                           \
    {                                                                           \
        public:                                                                 \
            TYPENAME();                                                         \
    };                                                                          \
    Q_GLOBAL_STATIC(TYPENAME, VARNAME)

/*The typedef if a workaround because preprocessor interprets ","*/
#define HNS_STATIC_HASH_INITIALIZER(VARNAME, TYPENAME, KEYTYPE, VALUETYPE)      \
    typedef QHash<KEYTYPE, VALUETYPE> HNS_PARENT_NAME(TYPENAME);                \
    HNS_STATIC_GENERIC_INITIALIZER(VARNAME, TYPENAME, HNS_PARENT_NAME(TYPENAME))

# define HNS_STATIC_SET_INITIALIZER(VARNAME, TYPENAME, TYPECONTAINED)           \
    HNS_STATIC_GENERIC_INITIALIZER(VARNAME, TYPENAME, QSet<TYPECONTAINED>)

#define HNS_STATIC_HASH(VARNAME, KEYTYPE, VALUETYPE)                            \
    typedef QHash<KEYTYPE, VALUETYPE> _hns_##VARNAME##_type ;                   \
    Q_GLOBAL_STATIC(_hns_##VARNAME##_type, VARNAME)

/*
 *Uncomment the line below if you want to see the generated code by using g++ -E
 *on this file
 **/
//HNS_STATIC_HASH_INITIALIZER(_varName, MyTypeName, keytype, valuetype)
#endif // INITIALIZER_H
