/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// hynesim common headers
#include "common/pluginmanager.h"

// Qt headers
#include <QDebug>
#include <QDir>
#include <QPluginLoader>
#include <QApplication>

namespace Hns {

/*******************************************************************************
* HnvPlugin
*/
Plugin::Plugin(QObject *parent)
    : QObject(parent)
    , _isEnabled(true)
{
}

bool Plugin::isEnabled() const
{
    return _isEnabled;
}

void Plugin::setEnabled(bool enabled)
{
    _isEnabled = enabled;
}


/*******************************************************************************
* PluginManager
*/
PluginManager::PluginManager(const QString &pluginPath, QObject *parent)
    : QObject(parent)
    , _pluginPath(pluginPath)
{
}

void PluginManager::buildPluginList()
{
    QString pluginPath = qApp->applicationDirPath() + "/plugins";
    if (!QFile::exists(pluginPath))
        pluginPath = _pluginPath;
    QDir pluginDir(pluginPath);
    foreach(QString fileName, pluginDir.entryList(QDir::Files)) {

        if (QFileInfo(fileName).suffix() != "so")
            continue;

        QPluginLoader loader(pluginDir.absoluteFilePath(fileName));

        QObject * plugin = loader.instance();
        if (plugin == 0) {
            qWarning() << tr("Plugin loading failed: %1").arg(loader.errorString());
            continue;
        }
        Plugin * hnsPlugin = qobject_cast<Plugin * >(plugin);
        if (hnsPlugin == 0)
            continue;

        if (!hnsPlugin->isEnabled())
            continue;

        qDebug() << tr("Plugin loaded: %1").arg(hnsPlugin->name());
        _pluginsList.append(hnsPlugin);
    }
}

} // namespace Hns
