/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef PLUGINMANAGER_H
#define PLUGINMANAGER_H

// Qt headers
#include <QObject>
#include <QList>

namespace Hns {

class PluginManager;

/*******************************************************************************
* HnvPlugin
*/
class Plugin : public QObject
{
    Q_OBJECT
public:
    Plugin(QObject *parent = 0);
    virtual QString name() const = 0;
    virtual QString description() const = 0;
    virtual bool isEnabled() const;
    virtual void setEnabled(bool enabled);

protected:
    bool _isEnabled;
};

/*******************************************************************************
* PluginManager
*/
class PluginManager : public QObject
{
    Q_OBJECT
public:
    explicit PluginManager(const QString &pluginPath, QObject *parent = 0);

    void buildPluginList();

    template <typename T> QList<T *> pluginListByType()
    {
        QList<T *> results;
        foreach (Plugin *plugin, _pluginsList) {
            T *result = qobject_cast<T *>(plugin);
            if (result)
                results.append(result);
        }
        return results;
    }

private:
    QList<Plugin *> _pluginsList;
    QString _pluginPath;
};

} // namespace Hns
Q_DECLARE_INTERFACE(Hns::Plugin, "org.hynesim.Plugin")

#endif // PLUGINMANAGER_H
