/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// hynesim common headers
#include "common/resourceutils.h"
#include "common/initializer.h"

// Generated hynesim headers
#include "common/domainentry.h"
#include "common/dynamipsrouterentry.h"

// Qt headers
#include <QHash>

namespace Hns {

// define here the methods which return the resources used by each type of entity
Resource _getResourceFromDomainEntry(const::Hns::EntityEntryPtr &entityEntry);
Resource _getResourceFromDynamipsRouterEntry(const::Hns::EntityEntryPtr &entityEntry);

typedef Resource (*GetResource)(const::Hns::EntityEntryPtr &);

HNS_STATIC_HASH_INITIALIZER(_getResourceHash, GetResourceHash, QString, GetResource)
GetResourceHash::GetResourceHash()
{
    insert(QString::fromStdString(DomainEntry::ice_staticId()), &_getResourceFromDomainEntry);
    insert(QString::fromStdString(DynamipsRouterEntry::ice_staticId()), &_getResourceFromDynamipsRouterEntry);
}

Resource getResourceFromEntry(const ::Hns::EntityEntryPtr &entityEntry)
{
    Resource resource;
    if (!entityEntry)
        return resource;

    QString className = QString::fromStdString(entityEntry->ice_id());
    if (_getResourceHash()->contains(className)) {
        GetResource getResource = _getResourceHash()->value(className);
        return getResource(entityEntry);
    } else {
        return resource;
    }
}

Resource _getResourceFromDomainEntry(const ::Hns::EntityEntryPtr &entityEntry)
{
    Hns::DomainEntryPtr domainEntry = Hns::DomainEntryPtr::dynamicCast(entityEntry);

    Resource resource;
    resource.cpu = domainEntry->vcpu;
    resource.memory = domainEntry->memory / 1024;
    return resource;
}


Resource _getResourceFromDynamipsRouterEntry(const ::Hns::EntityEntryPtr &entityEntry)
{
    Hns::DynamipsRouterEntryPtr dynamipsEntry = Hns::DynamipsRouterEntryPtr::dynamicCast(entityEntry);
    Resource resource;
    resource.cpu = 1;
    resource.memory = dynamipsEntry->ram;
    return resource;
}

} // namespace Hns
