// ****************************************************************************
//
// Copyright (C) 2003-2012 DIATEAM. All rights reserved.
//
// This file is part of Hynesim.
//
// Hynesim is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License version 2
// as published by the Free Software Foundation.
//
// Hynesim is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// ****************************************************************************

#ifndef TOPOLOGY_ICE
#define TOPOLOGY_ICE

#include <QtIce/qtice.ice>
#include <common/entity.ice>
#include <common/topologyentry.ice>
#include <common/layout.ice>
#include <diamesh/types.ice>
#include <diamesh/accessrights.ice>
#include <common/topologylink.ice>

[["cpp:include:list"]]
module Hns
{

class EntityEntry;
class WireEntry;
interface Entity;
interface Wire;
interface Tracer;

enum BalancingBehavior { BEHAVIOR_RANDOM, BEHAVIOR_FIRST, BEHAVIOR_AUTO };

struct DefineInfo {
    long entityId;
    bool isDefined = false;
    string nodeId = "";
    string errorMessage = "";
};

struct BalancingBehaviorInfo {
	string name;
	BalancingBehavior behavior;
};

dictionary<long, Entity*> LongEntityMap;
dictionary<long, TopologyLinkList> LongLinkListMap;
dictionary<long, DefineInfo> LongDefineInfoMap;
dictionary<int, BalancingBehaviorInfo> BalancingBehaviorMap;
sequence<long> IdImportedList;

["signal:void modified(QString, bool)",
"signal:void entryModified(QString)",
"signal:void entityAdded(Ice::Long,Ice::Long)",
"signal:void entryUsed(Ice::Long)",
"signal:void wireAdded(Ice::Long, Ice::Long)",
"signal:void entityRemoved(Ice::Long)",
"signal:void wireRemoved(Ice::Long)",
"signal:void layoutChanged()",
"signal:void layoutOptionsChanged()",
"signal:void entityLayoutChanged(Hns::EntityLayoutPtr)",
"signal:void wireLayoutChanged(Hns::WireLayoutPtr)",
"signal:void entityUnImplemented(Ice::Long)",
"signal:void entityImplemented(Ice::Long,Ice::Long)",
"signal:void entityPlugged(QString, QString)",
"signal:void entityUnplugged(QString, QString)",
"signal:void aclsChanged(Diamesh::ACLList)"]

interface Topology extends QtIce::SignalEmitter
{
    EntityTopologyLinkStruct createEntityFromEntry(EntityEntry entry, Layout l, TopologyLinkList links);
    EntityTopologyLinkStruct createEntityFromEntryId(long entryId, Layout l, TopologyLinkList links);
    Entity *implementModelEntity(long entityId, long entryId);
    bool unImplementModelEntity(long entityId);
    int createEntityFromClonedEntryId(long entryId, EntityLayout layout, long entryModelId);
    bool removeEntity(long entityId);
    bool plugEntity(long entityId, int entityPortId, long wireId, int wirePortId);
    bool unplugEntity(long entityId, int entityPortId, long wireId, int wirePortId);
    LongEntityMap getEntities();
    Entity *getEntity(long id);
    LongLinkListMap getTopologyLinks();
    TopologyLinkList getTopologyLink(long wireId);
    long getId();
    TopologyEntry getEntry();
    bool updateEntry(TopologyEntry entry);
    bool isModified();
    Tracer *getTracer();
    TopologyLayout getLayout();
    void setLayout(TopologyLayout layout);
    TopologyLayoutOptions getLayoutOptions();
    void setLayoutOptions(TopologyLayoutOptions options);
    EntityLayout getEntityLayout(long entityId);
    WireLayout getWireLayout(long wireId);
    void updateEntityLayout(EntityLayout layout);
    void updateWireLayout(WireLayout layout);
    Diamesh::LongList canBuildWithNode(string nodeName);
    LongDefineInfoMap computeDefineInfos(LongDefineInfoMap defineInfoMap, BalancingBehavior behavior);
    BalancingBehaviorMap getBehaviorMap();
    IdImportedList importTopology(long topologyEntryId, EntityLayout layout);
    Diamesh::ACLList getACLs();
    bool setACLs(Diamesh::ACLList aclList) throws RightError;
};

};

#endif // TOPOLOGY_ICE
