// ****************************************************************************
//
// Copyright (C) 2003-2012 DIATEAM. All rights reserved.
//
// This file is part of Hynesim.
//
// Hynesim is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License version 2 
// as published by the Free Software Foundation.
//
// Hynesim is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// ****************************************************************************

#ifndef TOPOLOGY_MANAGER_ICE
#define TOPOLOGY_MANAGER_ICE

#include <QtIce/qtice.ice>
#include <diamesh/types.ice>
#include <common/topologyentry.ice>
#include <common/entityinfo.ice>
#include <common/types.ice>
#include <common/layout.ice>

module Hns
{

interface Topology;

dictionary<long, string> LongStringMap;
dictionary<long, EntityInfo*> EntityInfoMap;

exception SaveError {
    string message = "generic save error";
};

["signal:void topologyLoaded(QString, Ice::Long)",
"signal:void topologyUnloaded(QString, Ice::Long, bool)",
"signal:void topologyModified(QString, bool)",
"signal:void topologySavedAs(QString, QString)",
"signal:void userOpenedTopology(Ice::Long, Ice::Long)",
"signal:void userClosedTopology(Ice::Long, Ice::Long)",
"signal:void requestEntityInfo(Ice::Long)",
"signal:void addEntityInfo(Ice::Long, QString)",
"signal:void removeEntityInfo(Ice::Long)"]
interface TopologyManager extends QtIce::SignalEmitter
{
    Topology *createTopology(bool isModel, TopologyLayoutOptions layoutOptions);
    Diamesh::LongList getTopologies();
    Topology *getTopologyById(long id);
    Topology *getTopologyByUuid(string uuid);
    Topology *loadTopology(long topologyEntryId) throws HnsError;
    bool saveTopology(long topologyId) throws SaveError;
    bool saveNewTopology(long topologyId, TopologyEntry entry) throws SaveError;
    bool saveTopologyAs(long topologyId, TopologyEntry entry) throws SaveError;
    long switchTopologyType(long topologyId, string newBackgroundColor) throws SaveError;
    bool closeTopology(long topologyId);
    bool unloadTopology(long topologyId);
    bool deleteTopology(long entryId);
    Diamesh::LongList getTopologyUsers(long topologyId);
    EntityInfo* getEntityInfo(long entryId);
    void requestGetEntityInfo(long entryId);
    EntityInfoMap getListEntityInfo();
};

};

#endif // TOPOLOGY_MANAGER_ICE
