#! /usr/bin/env python

import os
from ConfigParser import SafeConfigParser, NoSectionError, NoOptionError, ParsingError

class FakeSectionHeader(object):
    
    ini_filename        = ''
    fake_section        = None
    fp                  = None

    def __init__(self, ini_filename, fake_section):
        self.fp = open(ini_filename, 'r')
        self.fake_section = fake_section + os.linesep

    def readline(self):
        if self.fake_section:
            try: return self.fake_section
            finally: self.fake_section = None
        else:
            return self.fp.readline()

class HnsSettingsError(Exception):
    def __init__(self, value):
        self.value = value

    def __str__(self):
        return repr(self.value)

class HnsSettings(object):

    ini_filename        = ''
    ini_parser          = None
    ini_settings        = {}
    ini_default_section = '[General]'

    def __init__(self, ini_filename, ini_default_section = None):
        if not ini_filename:
            raise HnsSettingsError('ERROR: You must specify a file.')
        elif not os.access(ini_filename, os.F_OK) or not os.access(ini_filename, os.R_OK):
            raise HnsSettingsError('ERROR: Specified file does not exists or is unreadable.')
        else:
            if ini_default_section:
                self.ini_default_section = ini_default_section
            self.ini_filename = ini_filename
            self.ini_parser = SafeConfigParser()
            self.ini_parser.optionxform = str
            try:
                self.ini_parser.readfp(FakeSectionHeader(self.ini_filename, self.ini_default_section))
            except ParsingError as e:
                raise HnsSettingsError('ERROR: Specified file could not be parsed.' + os.linesep + e.message)
            except Exception as inst:
                print type(inst)
                print inst.args
                print inst.message
            section_list = self.ini_parser.sections()
            for section in section_list:
                item_list = self.ini_parser.items(section)
                self.ini_settings[section] = self.__parse_item_list(item_list)

    def __parse_item_list(self, item_list):
        section_dict = {}
        subsections = {}
        for key, value in item_list:
            if '\\' in key:
                real_key, sub_key = key.split('\\', 1)
                if not subsections.has_key(real_key):
                    subsections[real_key] = []
                subsections[real_key].append(tuple([sub_key, value]))
            elif '/' in key:
                real_key, sub_key = key.split('/', 1)
                if not subsections.has_key(real_key):
                    subsections[real_key] = []
                subsections[real_key].append(tuple([sub_key, value]))
            else:
                section_dict[key] = value
                
        for sub_key, sub_item_list in subsections.items():
            section_dict[sub_key] = self.__parse_item_list(sub_item_list)
        return section_dict
    
    def has_key(self, full_key):
        result = True
        key_list = full_key.split('/')
        settings = self.ini_settings
        for key in key_list:
            if settings.has_key(key):
                settings = settings[key]
            else:
                result = False
                break
        return result
    
    def get(self, full_key):
        result = True
        key_list = full_key.split('/')
        settings = self.ini_settings
        for key in key_list:
            if settings.has_key(key):
                settings = settings[key]
            else:
                result = False
                break
        if result:
            return settings
        else:
            return None
