#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 diateam. All rights reserved.
#
# This file is part of hynesim.
#
# Hynesim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Hynesim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# usage: changeVBoxDisksUUIDs.py /path/to/hnsEntryConfig.xml

from lxml import etree
import argparse
import os.path
from binascii import a2b_hex
from uuid import uuid4

def write_new_uuid(vdi_path):

    try:
        vdi_file = open(vdi_path, 'r+b')
    except:
        print('FATAL : Error opening file %s' % vdi_path)
        return

    uuid = uuid4()

    vdi_file.seek(0x188)
    uuid_hex = uuid.hex
    vdi_file.write(a2b_hex(uuid_hex[6:8]))
    vdi_file.write(a2b_hex(uuid_hex[4:6]))
    vdi_file.write(a2b_hex(uuid_hex[2:4]))
    vdi_file.write(a2b_hex(uuid_hex[0:2]))

    vdi_file.write(a2b_hex(uuid_hex[10:12]))
    vdi_file.write(a2b_hex(uuid_hex[8:10]))

    vdi_file.write(a2b_hex(uuid_hex[14:16]))
    vdi_file.write(a2b_hex(uuid_hex[12:14]))

    vdi_file.write(a2b_hex(uuid_hex[16:32]))

    print('INFO: %s wrote in the file %s' % (str(uuid), str(vdi_path)))

    vdi_file.close()


# TODO: handle domains with snapshots

parser = argparse.ArgumentParser()
parser.add_argument("domain_path", help="Absolute path of the domain")
args = parser.parse_args()

if not os.path.isdir(args.domain_path):
    print "You must specify a folder instead of '%s'" %(args.domain_path)
    exit(1)

domain_path = os.path.abspath(args.domain_path)

xml_path = os.path.join(args.domain_path, "hnsEntryConfig.xml")
if not os.path.isfile(xml_path):
    print "Could not find hnsEntryConfig.xml in the '%s' folder" %()
    exit(1)

xml_doc = etree.parse(xml_path)
xml_doc_root = xml_doc.getroot()
entity_node = xml_doc_root.find("Entity")
domain_node = entity_node.find("domain")

if domain_node.get("type") != "vbox":
    print "Domain must be of type vbox instead of %s" %(domain_node.get("type"))
    exit(1)

for devices in domain_node.findall("devices"):
    for child in devices:
        if (child.tag == "disk" and child.get("device") == "disk" and child.get("type") == "file"):
            disk_path = child.find("source").get("file")
            if os.path.isabs(disk_path):
                disk_path = os.path.normpath(disk_path)
            else:
                disk_path = os.path.normpath(os.path.join(domain_path, disk_path))

            write_new_uuid(disk_path)
