#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2013 diateam. All rights reserved.
#
# This file is part of hynesim.
#
# Hynesim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Hynesim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# usage: ./cloneVbox.py tmpEntryXmlFileInDestFolder sourceFolder
# testPath: tmpEntryXmlFileInDestFolder: /home/manuel/Devel/hynesim-test/catalog/entities/import/f429cb58-eeef-11e2-928b-14feb5ef3059/tmp.xml
#           sourceFolder:                /home/manuel/Devel/hynesim-test/catalog/entities/import/836e2480-2e38-42c9-aa8c-9cd965215777

#~ sudo ./cloneVbox.py /home/manuel/Devel/hynesim-test/catalog/entities/import/b4cce804-f2ce-11e2-8003-14feb5ef3059/tmp.xml /home/manuel/Devel/hynesim-test/catalog/entities/import/836e2480-2e38-42c9-aa8c-9cd965215777

from vboxapi import VirtualBoxManager
from vboxapi import VirtualBox_constants
import argparse
import os.path
from lxml import etree
import uuid
import sys
from time import sleep

#Init method
#def init():
#    mgr = VirtualBoxManager(None, None)
#    vb = mgr.vbox
#    vb_types = VirtualBox_constants.VirtualBoxReflectionInfo(True)
#    manager = mgr
#    mgr.deinit()
#    del mgr
#    return manager, vb, vb_types

#manager, vbox, vbox_types = init()
manager = VirtualBoxManager(None, None)
ctx={'mgr':manager,
     'vb':manager.vbox,
     'types':VirtualBox_constants.VirtualBoxReflectionInfo(True)}


def findIMediumByUuid(uuid, medium = None):
    listDisks = []
    ret = None
    if medium == None:
        listDisks = ctx['mgr'].getArray(ctx['vb'], "hardDisks")
    else:
        listDisks = ctx['mgr'].getArray(medium, "children")
    for disk in listDisks:
        if disk.id == str(uuid):
            ret = disk

    if not ret:
        for disk in listDisks:
            ret = findIMediumByUuid(uuid, disk)
            if ret:
                break

    return ret

def closeDisk(disk):
    if not disk:
        print "Need a disk to close"
        return -1
    for child in ctx['mgr'].getArray(disk, "children"):
        closeDisk(child)

    disk.close()
    del disk
    return 0

# update_progress() : Displays or updates a console progress bar
## Accepts a float between 0 and 1. Any int will be converted to a float.
## A value under 0 represents a 'halt'.
## A value at 1 or bigger represents 100%
def update_progress(progress, message = "Progress"):
    if not args.quiet:
        barLength = 20 # Modify this to change the length of the progress bar
        status = ""
        if isinstance(progress, int):
            progress = float(progress)
        if not isinstance(progress, float):
            progress = 0
            status = "error: progress var must be float\r\n"
        if progress < 0:
            progress = 0
            status = "Halt...\r\n"
        if progress >= 1:
            progress = 1
            status = "Done...\r\n"
        block = int(round(barLength*progress))
        text = "\r{0}: [{1}] {2}% {3}".format(message, "="*block + " "*(barLength-block), progress*100, status)
        sys.stdout.write(text)
        sys.stdout.flush()

def progressPercent(progress):
    if not args.quiet:
        if isinstance(progress, int):
            progress = float(progress)
        if not isinstance(progress, float):
            progress = 0
            status = "error: progress var must be float\r\n"
        if progress < 0:
            progress = 0
            status = "Halt...\r\n"
        if progress >= 1:
            progress = 1
            status = "Done...\r\n"
        block = int(progress*100)
        text = "{0}...".format(block)
        sys.stdout.write(text)
        sys.stdout.flush()

#############
#~ Arg parsing
#############
parser = argparse.ArgumentParser()
parser.add_argument("tmpEntryXmlFileInDestFolder", help="Absolute file path of a temporary xml entry located in the destination directory")
parser.add_argument("sourceFolder", help="Absolute folder path of the source entry")
parser.add_argument("--quiet", help="turn off the progressbar", action="store_true")
args = parser.parse_args()
#args are stored in args.tmpEntryXmlFileInDestFolder, args.sourceFolder

#############
#~ Folder check
#############
if not os.path.isfile(args.tmpEntryXmlFileInDestFolder):
    print "You must specify a file path instead of '%s'" %(args.tmpEntryXmlFileInDestFolder)
    exit(1)

if not os.path.isdir(args.sourceFolder):
    print "You must specify a folder instead of '%s'" %(args.sourceFolder)
    exit(1)

baseDestDir = os.path.dirname(args.tmpEntryXmlFileInDestFolder)

#############
#~ Read xml and extract absolute path of all disks
#############
xmlDoc = etree.parse(args.tmpEntryXmlFileInDestFolder)
xmlDocRoot = xmlDoc.getroot()
entityNode = xmlDocRoot.find("Entity")
domainNode = entityNode.find("domain")
domainSnapshotsNode = entityNode.find("domainsnapshots")
listDisksToClone = []

if domainNode.get("type") != "vbox":
    print "Entry must be a vbox instead of %s" %(domainNode.get("type"))
    exit(1)

#~ We iterate over the disks in the domain node
for disk in domainNode.findall("devices"):
    for child in disk:
        if (child.tag == "disk" and child.get("device") == "disk" and child.get("type") == "file"):
            absPathDisk = ""
            #~ The path of the disk need to be either absolute or juste the fileName (no ../.../.....//././../ stuff)
            if os.path.isabs(child.find("source").get("file")):
                absPathDisk = os.path.normpath(child.find("source").get("file"))
            else:
                absPathDisk = os.path.normpath(args.sourceFolder + "/" + child.find("source").get("file"))
            #~ We insert the disk only once, because as it is an absolute path (and it is also not possible to have the same vdi attached more than once)
            #~ there is no reason to have duplicates
            if not absPathDisk in listDisksToClone:
                ctx['vb'].openMedium(absPathDisk, ctx['types']._Values['DeviceType']['HardDisk'], ctx['types']._Values['AccessMode']['ReadWrite'], False)
                #~ We open the disk
                #~ because according to https://www.virtualbox.org/sdkref/interface_i_virtual_box.html#ae67fffa98f9a73b7f6b4365978658d9e
                #~ If we want to open a differencing media after, we need to open all its parents
                listDisksToClone.append(absPathDisk)


#~ Now we iterate over the disks in domainsnapshots
if domainSnapshotsNode != None:
    for domainSnapshotNode in domainSnapshotsNode:
        for disk in domainSnapshotNode.find("disks").findall("disk"):
            absPathDisk = ""
            #~ The path of the disk need to be either absolute or juste the fileName (no ../.../.....//././../ stuff)
            if os.path.isabs(disk.find("source").get("file")):
                absPathDisk = os.path.normpath(disk.find("source").get("file"))
            else:
                 absPathDisk = os.path.normpath(args.sourceFolder + "/" + disk.find("source").get("file"))
            #~ We insert the disk only once, because as it is an absolute path (and it is also not possible to have the same vdi attached more than once)
            #~ there is no reason to have duplicates
            if not absPathDisk in listDisksToClone:
                ctx['vb'].openMedium(absPathDisk, ctx['types']._Values['DeviceType']['HardDisk'], ctx['types']._Values['AccessMode']['ReadWrite'], False)
                #~ We open the disk
                #~ because according to https://www.virtualbox.org/sdkref/interface_i_virtual_box.html#ae67fffa98f9a73b7f6b4365978658d9e
                #~ If we want to open a differencing media after, we need to open all its parents
                listDisksToClone.append(absPathDisk)

#############
#~ We need to order the disk list because we have to clone parent disks first
#############
#~ We run through the list unordered
#~ Then, if a disk has no parent, we put it at the beginning of the orderedTupleList
#~ Then, for each disk, we put it in the ordered list only if its parent already is
orderedListDisk = [] #~ Each [(disk1Uuid, pathToDisk1), (disk2Uuid, pathToDisk2)]
while len(listDisksToClone) > 0:
    for disk in listDisksToClone:
        diskMedium = ctx['vb'].openMedium(disk, ctx['types']._Values['DeviceType']['HardDisk'], ctx['types']._Values['AccessMode']['ReadWrite'], False)
        diskUuid = diskMedium.id
        diskSize = diskMedium.size
        if not diskMedium.parent:
            orderedListDisk.append((diskUuid, disk, diskSize))
            listDisksToClone.remove(disk)
        else:
            parentUuid = diskMedium.parent.id
            for a in orderedListDisk:
                if parentUuid == a[0]:
                    orderedListDisk.append((diskUuid, disk, diskSize))
                    listDisksToClone.remove(disk)

#############
#~ Clone VBOX Disks
#############
#Calculate VM size
totalSize = 0
for disk in orderedListDisk:
    totalSize += disk[2]

totalProgress = 0.0
counterDisk = 0
mapSourceClonedDisks = [] #~ [(sourceUuid1, clonedUuid1),(sourceUuid2, clonedUuid2)]
for disk in orderedListDisk:
    counterDisk += 1
    newUuid = uuid.uuid4();
    #~ Creation of the disk name
    #~ If the source disk contains an uuid in the fileName, we will create a file with an uuid in the name
    #~ Else, we keep the same name (e.g. base.vdi, disk${i}.vdi,...)
    generateUuidFilename = False
    sourceDiskFilename = os.path.basename(disk[1])
    if disk[0] in sourceDiskFilename:
        generateUuidFilename = True
    fileToCreate = ""
    if generateUuidFilename:
        fileToCreate = "{%s}.vdi" %newUuid
    else:
        fileToCreate = os.path.basename(disk[1])
    if os.path.dirname(disk[1]).endswith("/Snapshots"):
        fileToCreate = "/Snapshots/" + fileToCreate
    fullPathToCreate = os.path.normpath(baseDestDir + "/" + fileToCreate)
    createdMedium = ctx['vb'].createHardDisk("vdi", fullPathToCreate)

    clonedParentUuid = None
    medium = findIMediumByUuid(disk[0])
    if medium.parent:
        for i in mapSourceClonedDisks:
            if i[0] == str(medium.parent.id):
                clonedParentUuid = i[1]

    sourceMedium = ctx['vb'].openMedium(disk[1], ctx['types']._Values['DeviceType']['HardDisk'], ctx['types']._Values['AccessMode']['ReadWrite'], False)
    #~ print "Cloning %s to %s" %(disk[1], fullPathToCreate)
    if (clonedParentUuid):
        progress = sourceMedium.cloneTo(createdMedium, sourceMedium.variant, findIMediumByUuid(clonedParentUuid))
    else:
        progress = sourceMedium.cloneToBase(createdMedium, sourceMedium.variant)
    if progress:
        while not progress.completed:
            message = "Cloning %s" %(sourceMedium.id)
            #~ update_progress(progress.percent/100.0, message)
            currentPercent = float(disk[2]) / float(totalSize)
            progressPercent(totalProgress + progress.percent/100.0 * currentPercent)
            sleep(0.5)
        #~ print("\n")

    createdMedium.setIds(True, str(newUuid), False, "")
    mapSourceClonedDisks.append((str(disk[0]), str(newUuid)))
    totalProgress += float(disk[2]) / float(totalSize)

#~ Now we iterate over the disks in domainsnapshots
#~ in order to replace the disk names in snapshots node
if domainSnapshotsNode != None:
    for domainSnapshotNode in domainSnapshotsNode:
        for disk in domainSnapshotNode.find("disks").findall("disk"):
            for i in mapSourceClonedDisks:
                if os.path.basename(findIMediumByUuid(i[0]).location) in disk.find("source").get("file"): #~ If there is the name of the disk file inside the xml
                    #~ We check if the recorded value is absolute or not, and we set the
                    #~ new value the same way
                    recordedValue = disk.find("source").get("file")
                    prefixRecordedValue = os.path.split(recordedValue)[0]
                    disk.find("source").set("file", os.path.join(prefixRecordedValue,os.path.basename(findIMediumByUuid(i[1]).location)))


xmlDoc.write(args.tmpEntryXmlFileInDestFolder)


#~ Close all virtualbox disks we have worked with
listBaseDisksToClose = []
for disk in ctx['mgr'].getArray(ctx['vb'], "hardDisks"):
    diskid = disk.id
    for i in mapSourceClonedDisks:
        if i[0] == diskid:
            #~ We call closeDisk with only base disk
            #~ If there is sourceDisk in vbox disks
            #~ cloned disk should also been closed
            listBaseDisksToClose.append(disk)
            listBaseDisksToClose.append(findIMediumByUuid(i[1]))

for diskToClose in listBaseDisksToClose:
    #~ print "Going to close '%s' at '%s'" %(diskToClose.id, diskToClose.location)
    closeDisk(diskToClose)

manager.deinit()
del manager


