#!/bin/bash
#
# Copyright (C) 2014 diateam. All rights reserved.
#
# This file is part of hynesim.
#
# Hynesim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Hynesim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# usage examples:
# ./data/scripts/hnsSpiceTerm.sh start 100 5900
# ./data/scripts/hnsSpiceTerm.sh stop 101 5901

PATH="/usr/bin:/usr/sbin:/bin:/sbin"

source $(dirname $0)/scriptLib.sh

if [[ $# < 2 ]]
then
    echo "Usage: $0 [vmname] [port]"
    exit 1
fi

vmname=$1
port=$2
keymap=$3

virsh -c lxc:/// list | grep $vmname
if [ $? -eq 0 ]
then
    echo "domain $vmname found and running"
else
    echo "domain $vmname is not found or not running"
    exit 1
fi

session=hns$vmname
while true
do
    screen -q -ls $session
    if [[ $? -eq 9 ]]
    then
        screenrc=$(mktemp)
        cat > $screenrc << EOF
        unbindall
EOF

        pts=$(virsh -c lxc:/// ttyconsole $vmname)
        screen -c $screenrc -dmS $session $pts
        rm $screenrc
    fi

    cmd="spiceterm --noauth --port $port --timeout 0"
    if [[ $keymap != "" ]]
    then
        cmd="${cmd} --keymap $keymap"
    fi

    cmd="${cmd} -- screen -r $session"
    $cmd

    res=$?
    if [[ $res != 0 ]] # process killed
    then
        echo "spiceterm killed, not continuing"
        exit 0
    fi
    res=$(virsh -c lxc:/// list |grep $vmname)
    if [[ $res == "" ]]
    then
        echo "$vmname has stopped"
        exit 0
    fi
done
