#!/bin/bash
#
# Copyright (C) 2011 diateam. All rights reserved.
#
# This file is part of hynesim.
#
# Hynesim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Hynesim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
# usage examples:
# ./data/scripts/hnsVncTerm.sh start 100 5900
# ./data/scripts/hnsVncTerm.sh stop 101 5901

PATH="/usr/bin:/usr/sbin:/bin:/sbin"

source $(dirname $0)/scriptLib.sh

veIndexFile="/tmp/veIndex"
host="localhost"

function addVmIndex()
{
	echo "addVmIndex"
	echo "$type $host $port $vmid" >> $veIndexFile
}

function delVmIndex()
{
	echo "delVmIndex"
	line="$type $host $port $vmid"
	sed -ie "/$line/d" $veIndexFile
}

log "$1 $2"

if [[ $# < 4 ]]
then
	echo "Usage: $0 start|stop [vmid] [port] qemu|ovz"
	exit 1
fi

action=$1
vmid=$2
port=$3
type=$4

if [[ $type == "ovz" ]]
then
	checkvmid=$(vzlist|grep running| cut -d' ' -f8|grep $vmid)
	if [[ $vmid != $checkvmid ]]
	then
		echo "vmid $vmid is not found or not running :"
		vzlist
		exit 1
	else
		echo "vmid $vmid found and running"
	fi

elif [[ $type == "lxc" ]]
then
	virsh -c lxc:/// list | grep $vmid
	if [ $? -eq 0 ]
	then
		echo "vmid $vmid found and running"
	else
		echo "vmid $vmid is not found or not running :"
		exit 1
	fi
fi
	
if [[ $action == "start" ]]
then
	addVmIndex
	while true
	do
		cmd=""
		if [[ $type == "ovz" ]]
		then
			cmd="vncterm -rfbport $port -c ssh -t root@localhost vzctl enter $vmid"
		elif [[ $type == "lxc" ]]
		then
			cmd="vncterm -rfbport $port -c ssh -t root@localhost virsh -c lxc:/// console $vmid"
		fi
		#echo $cmd
		$cmd
		res=$?
		if [[ $res != 0 ]] # process killed
		then
			echo "vncterm killed, not continuing"
			delVmIndex
			exit 0
		fi
		if [[ $type == "ovz" ]]
		then
			res=$(vzlist 2>/dev/null |grep $vmid)
		elif [[ $type == "lxc" ]]
		then
			res=$(virsh -c lxc:/// list |grep $vmid)
		fi
		
		if [[ $res == "" ]]
		then
		  echo "$vmid is not listed any more"
		  exit 0
		 fi
	done

elif [[ $action == "stop" ]]
then
	pid=""
	index=0
	# the pid might not start the line, there may be some spaces before.
	while [[ $pid == "" && $index -lt 10 ]]
	do
	index=$(($index+1))
	pid=$(ps -eo "%p %a"|grep vncterm|grep $vmid|cut -d' ' -f$index)
	done
	if [[ $pid == "" ]]
	then
		log "stop : pid not found for vncterm $vmid"
		exit 1
	else
		kill $pid
		delVmIndex
	fi
else
	log "action must be start or stop, instead of $action"
fi
