#!/bin/bash
#
# Copyright (C) 2011 diateam. All rights reserved.
#
# This file is part of hynesim.
#
# Hynesim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Hynesim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
##################################################################################################################################################
################################################################### PARAMETERS ###################################################################
##################################################################################################################################################

# Parameters count
if [ $# -lt 4 ]
then
    echo "Missing parameters : addSsid.sh <device> <mac> <interface> <ssid> <hidden> <auth> [<key>]"
    exit 1
fi

# Extraction
device="$1"
mac="$2"
iface="$3"
ssid="$4"
hidden="$5"
auth="$6"
key="$7"
channel="$8"
##################################################################################################################################################
################################################################## ENVIRONMENT ###################################################################
##################################################################################################################################################

# Directory for the configuration files of Hostapd
mkdir -p /tmp/hnsWifiAPConf

##################################################################################################################################################
############################################################### INTERFACE CREATION ###############################################################
##################################################################################################################################################
# Recuperation of the device with the interface given
phy="phy"`cat /sys/class/net/$device/phy80211/index` ||
{
    echo "No device available for $device"
    exit 1
}
# Creation of the interface
iw phy $phy interface add $iface type managed ||
{
    echo "Failed to create '$iface' interface"
    exit 1
}

# Assignation of the mac address
ip link set $iface addr $mac ||
{
    echo "Failed to assignate mac address '$mac' to '$iface' interface"
    sh ./hnsWifiApRemoveIface.sh $iface
    exit 1
}

##################################################################################################################################################
#################################################################### HOSTAPD #####################################################################
##################################################################################################################################################

# Configuration filename
ifaceConf="/tmp/hnsWifiAPConf/"$iface".conf"

# Text referring to encryption to add in the configuration file
encryption=""
mode=""
wpa1=""
wpa2=""
password=""
lengthPassphrase=${#key}
# No encryption
if [ "$auth" = "NONE" ]
then
    dummy=""

# Encryption mode : Wep
else if [ "$auth" = "WEP" ]
then
    password="wep_key0=$key"
    
# Encryption mode : WPA 1
else if [ "$auth" = "WPA1" ]
then
    encryption="wpa=1"
    wpa1="wpa_pairwise=CCMP TKIP"
    if [ $lengthPassphrase = 64 ]; then
        password="wpa_psk=$key"
    else
        password="wpa_passphrase=$key"
    fi

# Encryption mode : WPA 2
else if [ "$auth" = "WPA2" ]
then
    encryption="wpa=2"
    mode="wpa_key_mgmt=WPA-PSK"
    wpa2="rsn_pairwise=CCMP"
    if [ $lengthPassphrase = 64 ]; then
        password="wpa_psk=$key"
    else
        password="wpa_passphrase=$key"
    fi
    
# Encryption mode : WPA 1 and 2
else if [ "$auth" = "WPA12" ]
then
    encryption="wpa=3"
    mode="wpa_key_mgmt=WPA-PSK"
    wpa1="wpa_pairwise=CCMP TKIP"
    wpa2="rsn_pairwise=CCMP"
    if [ $lengthPassphrase = 64 ]; then
        password="wpa_psk=$key"
    else
        password="wpa_passphrase=$key"
    fi
# Bad encryption mode
else
    echo "Encryption mode '$auth' not managed"
    exit 1
fi fi fi fi fi

# We hide the network if told to
ignoreBroadcast=0
if [ "$hidden" != 0 ]
then
    ignoreBroadcast=1
fi

# Content of the configuration file for Hostapd
cat > "$ifaceConf" << EOF
interface=$iface
driver=nl80211
bridge=${iface}b
ssid=$ssid
ignore_broadcast_ssid=$ignoreBroadcast
channel=$channel
hw_mode=g
auth_algs=1
$encryption
$mode
$wpa1
$wpa2
$password
EOF


# Confirmation message
echo "Interface '$iface' successfully created"
