#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2012 diateam. All rights reserved.
#
# This file is part of hynesim.
#
# Hynesim is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Hynesim is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


import cmd
from optparse import OptionParser
import io
import os
import subprocess
import re
import shutil
import time
import libvirt
from HnsSettings import HnsSettings, HnsSettingsError
from HnsLogger import HnsLogger

class Util():
    path_config = None
    path_script = None
    config_reader = None

    @staticmethod
    def initConfig(path):
        Util.config_reader = HnsSettings(path)
        #init files
        if(path[0] == "/"):
            Util.path_config = os.path.dirname(path)
        else:
            joinPath = os.path.join(os.getcwd(), path)
            Util.path_config = os.path.dirname(joinPath)
        Util.path_script = os.getcwd()

    @staticmethod
    def getValueParser(domain, key, method = "str"):
        if method == "bool":
            if not Util.config_reader.has_key(domain + '/' + key):
                return False
            value = Util.config_reader.get(domain + '/' + key)
            if value == None or not value.isdigit():
                return False
            return bool(value)
        elif method == "str":
            if not Util.config_reader.has_key(domain + '/' + key):
                return ""
            value = Util.config_reader.get(domain + '/' + key)
            if value == None:
                return ""
            return value
        elif method == "int":
            if not Util.config_reader.has_key(domain + '/' + key):
                return -1
            value = Util.config_reader.get(domain + '/' + key)
            if value == None or value.isdigit():
                return int(value)
        return -1

    @staticmethod
    def hasConfigKey(domain, key):
        return Util.config_reader.has_key(domain + '/' + key)

    @staticmethod
    def checkBinary(program):
        message = "Binary not found: " + program
        result = False
        def is_executable(fpath):
            return os.path.exists(fpath) and os.access(fpath, os.X_OK)
        fpath, fname = os.path.split(program)
        if fpath:
            if is_executable(program):
                message = "Binary found: " + program
                result = True
        else:
            for path in os.environ["PATH"].split(os.pathsep):
                exe_file = os.path.join(path, program)
                if is_executable(exe_file):
                    message = "Binary found: " + exe_file
                    result = True
        return (message, result)

    @staticmethod
    def exectBinary(program, verbose = True, value = False):
        try:
            if value:
                idx = 0
                programList = list()
                programList.append(list())
                for arg in program:
                    if arg=="|":
                        idx +=1
                        programList.append(list())
                    else:
                        programList[idx].append(arg)

                programProcess = list()
                idx = 0
                for listArg in programList:
                    if idx == 0:
                        programProcess.append(subprocess.Popen(listArg, stdout=subprocess.PIPE))
                    else:
                        programProcess.append(subprocess.Popen(listArg, stdout=subprocess.PIPE, stdin = programProcess[idx-1].stdout))
                    idx += 1

                return programProcess[idx-1].communicate()[0]
            if verbose:
                return subprocess.call(program , shell = True)
            else:
                FNULL = open("/dev/null", "w")
                return subprocess.check_call(program, stdout = FNULL, stderr=FNULL )
        except:
            return -1

    @staticmethod
    def isNameAnUUID(name):
        if re.search('[a-fA-F0-9]{8}-[a-fA-F0-9]{4}-[a-fA-F0-9]{4}-[a-fA-F0-9]{4}-[a-fA-F0-9]{12}', name):
            return True;
        return False;

class BinaryTools():
    def __init__(self):
        self.listExec = list()

    def addBinary(self,binary):
        self.listExec.append(binary)

    def isRunning(self):
        listRunning = list()
        listNotRunning = list()
        for binary in self.listExec:
            result = Util.exectBinary(["pgrep", binary], value = True)
            if result !="":
                listRunning.append(binary)
            else :
                listNotRunning.append(binary)

        return [listRunning, listNotRunning]

    def killAll(self, signal = 15):
        for binary in self.listExec:
            Util.exectBinary(["pkill", "-{0}".format(signal), "-u", "0", binary], verbose = False)


class ServiceTools():
    def __init__(self, service):
        self.service = service

    def isExist(self):
        result = Util.checkBinary("/etc/init.d/" + self.service)
        if not result[1]:
            return False
        return True

    def getStatus(self):
        result =  Util.exectBinary(["service", self.service, "status"], value = True)
        if result != -1:
            results = result.split("\n")
            if len(results) > 1 and results[1] != "":
                return results[1]
            elif  len(results) > 0:
                return results[0]
        return str()

    def stop(self):
        Util.exectBinary(["service " + self.service + " stop"])

    def start(self, restart=False):
        if restart:
            Util.exectBinary(["service " + self.service + " stop > /dev/null"])
        Util.exectBinary(["service " + self.service + " start"])


class InterfaceTools():
    interfaces = None
    def __init__(self):
        if InterfaceTools.interfaces is None:
            InterfaceTools.__initInterfaces__()
        self.regExpTap = list()
        self.regExpBridge = list()
        self.regExpIW = list()
        self.regExpIfaceDev = list()
        self.regExpIfaceWifiDev = list()
        self.regExpVeth = list()
        self.regExpVxlan = list()

    @staticmethod
    def __initInterfaces__():
        listInterfaces = Util.exectBinary(["ip", "link", "|", "egrep", "^[0-9]+: ", "|", "cut", "-d", " ", "-f2", "|", "sed",  "s/://"], value = True)
        if listInterfaces != -1:
            InterfaceTools.interfaces = listInterfaces.split()
        else:
            InterfaceTools.interfaces = list()

    @staticmethod
    def __downiface__(iface):
        Util.exectBinary(["ip", "link", "set", "dev", iface, "down"], verbose = False)

    def __listInterfaceExist__(self, regExp):
        listIface = []
        listIfaceNotFound = []
        for reg in regExp:
            foundIface = False
            for iface in InterfaceTools.interfaces:
                if re.match(reg, iface):
                    listIface.append(iface)
                    foundIface = True
            if not foundIface:
                listIfaceNotFound.append(reg)
        return [listIface, listIfaceNotFound]

    ##tunctl
    def addRegExpTap(self, tuncl):
        self.regExpTap.append(tuncl)

    def tapListExist(self):
        return self.__listInterfaceExist__(self.regExpTap)

    def clearTap(self):
        deleteIface = False
        for iface in InterfaceTools.interfaces:
            for reg in self.regExpTap:
                if re.match(reg, iface):
                    Util.exectBinary(["ip", "link", "del", "dev", iface], verbose = False)
                    deleteIface = True
        if deleteIface:
            InterfaceTools.__initInterfaces__()

    ##brctl
    def addRegExpBridge(self,brtl):
        self.regExpBridge.append(brtl)

    def bridgeListExist(self):
        return self.__listInterfaceExist__(self.regExpBridge)

    def clearBridge(self):
        deleteIface = False
        for iface in InterfaceTools.interfaces:
            for reg in self.regExpBridge:
                if re.match(reg, iface):
                    Util.exectBinary(["ip", "link", "del", "dev", iface], verbose = False)
                    deleteIface = True
        if deleteIface:
            InterfaceTools.__initInterfaces__()

    ##iw
    def addRegExpIW(self,wi):
        self.regExpIW.append(wi)

    def iwListExist(self):
        return self.__listInterfaceExist__(self.regExpIW)

    def clearIW(self):
        deleteIface = False
        for iface in InterfaceTools.interfaces:
            for reg in self.regExpIW:
                if re.match(reg, iface):
                    Util.exectBinary(["iw", "dev", iface, "del"], verbose = False)
                    deleteIface = True
        if deleteIface:
            InterfaceTools.__initInterfaces__()

    ##ifaceDev
    def addRegExpIfaceDev(self,dev):
        self.regExpIfaceDev.append(dev)

    def ifaceDevListExist(self):
        return self.__listInterfaceExist__(self.regExpIfaceDev)

    def clearIfaceDev(self):
        deleteIface = False
        pathConf = "/dev"
        for myFile in os.listdir(pathConf):
            for reg in self.regExpIfaceDev:
                if re.match(reg, myFile):
                     os.remove(os.path.join(pathConf, myFile))
                     deleteIface = True
        if deleteIface:
            InterfaceTools.__initInterfaces__()

    ##ifaceDevWifi
    def addRegExpIfaceWifiDev(self,dev):
        self.regExpIfaceWifiDev.append(dev)

    def ifaceWifiCheckSystem(self):
        listResult = []
        listResultWarning = []
        commands = ["iw", "hostapd"]
        for binary in commands:
            result = Util.checkBinary(binary)
            if not result[1]:
                listResultWarning.append(result[0])
            else:
                listResult.append(result[0])
        return [listResult, listResultWarning]

    def ifaceWifiDevListExist(self):
        listIface = []
        listIfaceNotFound = []
        for iface in self.regExpIfaceWifiDev:
            if  Util.exectBinary(["iw", iface, "info" ] , verbose = False) !=0 or \
             Util.exectBinary(["ls", "/sys/class/net/" + iface + "/phy80211/index"], verbose=False) != 0:
                listIfaceNotFound.append(iface)
            else:
                physicalDevice = Util.exectBinary(["cat", "/sys/class/net/" + iface + "/phy80211/index"], value = True)
                iface_pair = (iface, physicalDevice)
                listIface.append(iface_pair)
        return [listIface, listIfaceNotFound]

    ##veth
    def addRegExpVeth(self, dev):
        self.regExpVeth.append(dev)

    def ifaceVethExist(self):
        return self.__listInterfaceExist__(self.regExpVeth)

    def clearVeth(self):
        deleteIface = False
        for iface in InterfaceTools.interfaces:
            for reg in self.regExpVeth:
                match = re.match(reg, iface)
                if match is not None:
                    interface = match.group(0)
                    Util.exectBinary(["ip", "link", "del", "dev", interface], verbose = False)
                    deleteIface = True
        if deleteIface:
            InterfaceTools.__initInterfaces__()

    #vxlan
    def addRegExpVxlan(self, dev):
        self.regExpVxlan.append(dev)

    def ifaceVxlanExist(self):
        return self.__listInterfaceExist__(self.regExpVxlan)

    def clearVxlan(self):
        deleteIface = False
        for iface in InterfaceTools.interfaces:
            for reg in self.regExpVxlan:
                if re.match(reg, iface):
                    Util.exectBinary(["ip", "link", "del", "dev", iface], verbose = False)
                    deleteIface = True
        if deleteIface:
            InterfaceTools.__initInterfaces__()

class ModuleTools():
    def __init__(self):
        self.loadModules = list()
        self.unloadModules = list()

    def addUnloadModule(self,module):
        self.unloadModules.append(module)

    def addLoadModule(self,module):
        self.loadModules.append(module)

    def unload(self):
        moduleOk = []
        moduleNok = []
        for module in self.unloadModules:
            if Util.exectBinary(["lsmod | grep " + module + ">/dev/null"]) == 0:
                if Util.exectBinary(["rmmod " + module]) != 0:
                    moduleNok.append(module)
                else:
                    moduleOk.append(module)
        return [moduleOk, moduleNok]

    def load(self):
        moduleOk = []
        moduleNok = []
        for module in self.loadModules:
            if Util.exectBinary(["modprobe " + module]) != 0:
                moduleNok.append(module)
            else:
                moduleOk.append(module)
        return [moduleOk, moduleNok]


class VirshTools():
    def __init__(self, sessionName):
        self.conn = None
        self.sessionName = sessionName
        self.regExpVersion = list()
        self.service = ServiceTools("libvirtd")
        self.has_snapshot = False

    def __connect_libvirt__(self):
        try:
            self.conn = libvirt.open(self.sessionName)
        except:
            self.conn = None
            return [False, "virsh: Can't connect to " + self.sessionName]
        return [True, ""]

    def __displayVm__(self,vm):
        status = ""
        if vm.state(0)[0] == libvirt.VIR_DOMAIN_BLOCKED:
            status = "is blocked on resource"
        elif vm.state(0)[0] == libvirt.VIR_DOMAIN_CRASHED:
            status = "is crashed"
        elif vm.state(0)[0] == libvirt.VIR_DOMAIN_NOSTATE:
            status = "has no state"
        elif vm.state(0)[0] == libvirt.VIR_DOMAIN_PAUSED:
            status = "is paused"
        elif vm.state(0)[0] == libvirt.VIR_DOMAIN_RUNNING:
            status = "is running"
        elif vm.state(0)[0] == libvirt.VIR_DOMAIN_SHUTDOWN:
            status = "is being shut down"
        elif vm.state(0)[0] == libvirt.VIR_DOMAIN_SHUTOFF:
            status = "is shut off"
        return vm.name() + ": " + status

    def addVersion(self, regExpVersion):
        self.regExpVersion.append(regExpVersion)

    def checkVersion(self):
        if self.conn is None:
            result = self.__connect_libvirt__()
            if not result[0]:
                return result
        version = str(self.conn.getVersion())
        #1000000 * major + 1000 * minor + micro
        versionMaj = self.conn.getVersion() / 1000000
        tmp = self.conn.getVersion() % 1000000
        versionInt = tmp / 1000
        versionMin = tmp % 1000
        versionPrint = str(versionMaj) + "." + str(versionInt) + "." + str(versionMin)
        versionFound = (len(self.regExpVersion) == 0)
        for expression in self.regExpVersion:
            if re.match(expression, version):
                versionFound = True
                break
        if not versionFound:
            return [False, "the version " + versionPrint + " is unsupported"]
        return [True, "Checked (version: " + versionPrint + ")"]

    def checkVM(self):
        if self.conn is None:
            result = self.__connect_libvirt__()
            if not result[0]:
                return result
        ##running machine
        error = []
        vms = []
        for idDomain in self.conn.listDomainsID():
            vms.append(self.conn.lookupByID(idDomain))
        for idDomain in self.conn.listDefinedDomains():
            vms.append(self.conn.lookupByName(idDomain))
        for vm in vms:
            if Util.isNameAnUUID(vm.name()):
                error.append(self.__displayVm__(vm))

        if len(error) == 0:
            return [True, ""]
        else:
            return [False, error]

    def stopVM(self):
        log = []
        if self.conn is None:
            result = self.__connect_libvirt__()
            if not result[0]:
                return result
        for idDomain in self.conn.listDomainsID():
            vm = self.conn.lookupByID(idDomain)
            if Util.isNameAnUUID(vm.name()):
                log.append("Stop " + vm.name())
                vm.destroy()
        return [True, log]

    def undefineVM(self):
        log = []
        if self.conn is None:
            result = self.__connect_libvirt__()
            if not result[0]:
                return result
        time.sleep(1) #because virsh has some troubles with destroy/undefine quickly Virtualbox machines
        for idDomain in self.conn.listDefinedDomains():
            vm = self.conn.lookupByName(idDomain)
            if Util.isNameAnUUID(vm.name()):
                log.append("Undefine " + vm.name())
                if self.has_snapshot:
                    try:
                        for snapshotName in vm.snapshotListNames(0):
                            snapshot = vm.snapshotLookupByName(snapshotName, 0)
                            snapshot.delete(libvirt.VIR_DOMAIN_SNAPSHOT_DELETE_METADATA_ONLY)
                    except libvirt.libvirtError:
                        HnsLogger.log_error("The driver " + self.conn.getURI().split(":")[0] +" does not support snapshot")
                vm.undefine()
        return [True, log]

    def restartService(self):
        if self.conn is not None:
            self.conn.close()
            self.conn = None
        self.service.start(True)


class AbstractElement() :
    def __init__(self):
        self.enable = False
        self.error = False
        self.title = str()
        self.blacklist = set()
        self.domainList = set()
        self.domainList.add("VirtualBox") #all domain
        self.domainList.add("OpenVZ")
        self.domainList.add("KvmQemu")
        self.domainList.add("VMware")

    def check(self) :
        return True

    def init(self) :
        return True

    def clean(self) :
        return True

    def isEnable(self) :
        return self.enable

    def doCheck(self):
        if self.enable:
            HnsLogger.log_msg_stderr("Checking " + self.title + "...")
            if self.error:
                return False
            return self.check()
        HnsLogger.log_msg_stderr(self.title + " is not enabled. Pass...")
        return True

    def doInit(self):
        if self.enable:
            HnsLogger.log_msg_stderr("Initializing " + self.title)
            if self.error:
                return False
            return self.init()
        HnsLogger.log_msg_stderr(self.title + " is not enabled. Pass...")
        return True

    def doClean(self):
        if self.enable:
            HnsLogger.log_msg_stderr("Cleaning up " + self.title)
            if self.error:
                return False
            return self.clean()
        HnsLogger.log_msg_stderr(self.title + " is not enabled. Pass...")
        return True

    def cleanBlackList(self):
        return "";


class FileElement(AbstractElement):
    def __init__(self):
        AbstractElement.__init__(self)
        self.title = "Folder"
        self.enable = True
        self.folderDict = dict()
        self.folderDict["scriptsDirectory"] = Util.getValueParser("Node", "scriptsDirectory")
        self.folderDict["sharedFolder"] = Util.getValueParser("Node", "sharedFolder")
        fileStorageSize = Util.getValueParser("Hynesim", "FileStorages/size", "int")
        if fileStorageSize <= 1:
            HnsLogger.log_error("Settings Key 'FileStorages/size' is missing in the configuration file.")
            self.error = True
        idx = 1
        while idx <= fileStorageSize:
            self.folderDict["FileStorage " + str(idx)] = Util.getValueParser("Hynesim", "FileStorages/" + str(idx) + "/path")
            idx += 1

    def check(self):
        if  self.folderDict["scriptsDirectory"] == "":
            HnsLogger.log_error("Settings Key 'scriptsDirectory' is missing in the configuration file.")
            self.error = True
            self.blacklist.add("All")
        os.chdir(Util.path_config)
        for (key, path_value) in self.folderDict.items():
            if path_value == "":
                continue
            elif os.path.isdir(path_value):
                HnsLogger.log_msg_stderr("[" + key + "] :" + os.path.abspath(path_value))
            else:
                self.error = True
                HnsLogger.log_error("Settings Value '" + key + "': "+ path_value + " is not a valid path.")
                if key == "scriptsDirectory":
                    self.blacklist.clear()
                    self.blacklist.add("All")
                elif key == "sharedFolder":
                    self.blacklist.update(self.domainList)

        os.chdir(Util.path_script)
        if self.error:
            return False
        return True


class NativeSwitchElement(AbstractElement):
    def __init__(self):
        AbstractElement.__init__(self)
        self.title = "NativeSwitch"
        self.enable = True
        self.binaryTools = BinaryTools()
        self.interfaceTools = InterfaceTools()
        self.interfaceTools.addRegExpBridge("^hns(\d+)s")

    def check(self):
        HnsLogger.log_msg_stderr("Checking for interfaces UP...")
        listIface = self.interfaceTools.bridgeListExist()
        if len(listIface[0]) > 0:
            self.error = True
            for iface in listIface[0]:
                HnsLogger.log_error("Network interface '" + iface + "' is still UP.")
        listIface = self.interfaceTools.tapListExist()
        if len(listIface[0]) > 0:
            self.error = True
            for iface in listIface[0]:
                HnsLogger.log_error("Network interface '" + iface + "' is still UP.")
        if self.error:
            self.blacklist.add("All")
            return False
        return True

    def clean(self):
        self.binaryTools.killAll()
        self.interfaceTools.clearBridge()
        return True

class TopologyGateElement(AbstractElement):
    def __init__(self):
        AbstractElement.__init__(self)
        self.title = "Topology Gate"
        self.enable = True
        self.interfaceTools = InterfaceTools()
        self.interfaceTools.addRegExpBridge("^hnstgb\d+")
        self.interfaceTools.addRegExpBridge("^hnstg\d+")
        self.interfaceTools.addRegExpVeth("(^hnstg\d+[ab])")

    def check(self):
        listIface = self.interfaceTools.bridgeListExist()
        if len(listIface[0]) > 0:
            self.error = True
            for iface in listIface[0]:
                HnsLogger.log_error("Network interface '" + iface + "' is still UP.")
        listIface = self.interfaceTools.ifaceVethExist()
        if len(listIface[0]) > 0:
            self.error = True
            for iface in listIface[0]:
                HnsLogger.log_error("Network interface '" + iface + "' is still UP.")
        if self.error:
            self.blacklist.add("TopologyGate")
            return False
        return True

    def clean(self):
        self.interfaceTools.clearBridge()
        self.interfaceTools.clearVeth()

class HostapdElement(AbstractElement):
    def __init__(self):
        AbstractElement.__init__(self)
        self.title = "Hostapd"
        self.enable = True
        self.binaryTools = BinaryTools()
        self.binaryTools.addBinary("hostapd")

    def check(self):
        result  = self.binaryTools.isRunning()
        for binaryRunning in result[0]:
            self.error = True
            HnsLogger.log_error(binaryRunning + " is running.")
        for binaryRunning in result[1]:
            HnsLogger.log_msg_stderr(binaryRunning + " is stopped.")
        if self.error:
            self.blacklist.add("WifiAP")
            return False
        return True

    def clean(self):
        self.binaryTools.killAll()
        return True


class EntityElement(AbstractElement):
    def __init__(self):
        AbstractElement.__init__(self)

    def parseEnable(self , key):
        return Util.hasConfigKey("Platforms", key)


class WireElement(EntityElement):
    def __init__(self):
        EntityElement.__init__(self)
        self.enable = True
        self.title = "Wire"
        self.interfaceTools = InterfaceTools()
        self.interfaceTools.addRegExpBridge("^hns\d+$")
        self.interfaceTools.addRegExpTap("^hns\d+(t|v)$")
        self.interfaceTools.addRegExpVeth("(^hnsw\d+[ab])")
        self.interfaceTools.addRegExpVxlan("^hnsvx\d+$")

    def check(self):
        HnsLogger.log_msg_stderr("Checking for interfaces UP...")
        listIface = self.interfaceTools.bridgeListExist()
        if len(listIface[0]) > 0:
            self.error = True
            for iface in listIface[0]:
                HnsLogger.log_error("Network interface '" + iface + "' is still UP.")
        listIface = self.interfaceTools.tapListExist()
        if len(listIface[0]) > 0:
            self.error = True
            for iface in listIface[0]:
                HnsLogger.log_error("Network interface '" + iface + "' is still UP.")
        listIface = self.interfaceTools.ifaceVethExist()
        if len(listIface[0]) > 0:
            self.error = True
            for iface in listIface[0]:
                HnsLogger.log_error("Network interface '" + iface + "' is still UP.")
        listIface = self.interfaceTools.ifaceVxlanExist()
        if len(listIface[0]) > 0:
            self.error = True
            for iface in listIface[0]:
                HnsLogger.log_error("Network interface '" + iface + "' is still UP.")
        if self.error:
            self.blacklist.add("All")
            return False
        return True

    def clean(self):
        self.interfaceTools.clearTap()
        self.interfaceTools.clearBridge()
        self.interfaceTools.clearVeth()
        self.interfaceTools.clearVxlan()
        return True


class VBoxElement(EntityElement):
    def __init__(self):
        EntityElement.__init__(self)
        self.title = "VirtualBox"
        self.enable = self.parseEnable("VirtualBox")
        self.virshTools = VirshTools("vbox:///session")
        self.virshTools.addVersion("^400[0-3]\d{3,3}")
        self.moduleTools = ModuleTools()
        self.moduleTools.addUnloadModule("kvm_intel")
        self.moduleTools.addUnloadModule("kvm")
        self.binaryTools = BinaryTools()
        self.binaryTools.addBinary("VBoxHeadless")
        self.serviceVbox = ServiceTools("vboxdrv")
        self.binaryTools.addBinary("VirtualBox")
        self.virshTools.has_snapshot = True

    def check(self):
        result = self.virshTools.checkVersion()
        if not result[0]:
            self.error = True
            HnsLogger.log_error(result[1])
        else:
            if result[1] != "":
                HnsLogger.log_msg_stderr(result[1])
            result = self.virshTools.checkVM()
            if result[0]:
                if len(result[1]) > 0:
                    for msg in result[1]:
                        HnsLogger.log_msg_stderr(msg)
            else:
                self.error = True
                if len(result[1]) > 0:
                    for error in result[1]:
                        HnsLogger.log_error(error)
        if self.error:
            self.blacklist.add("VirtualBox")
            return False
        return True

    def init(self):
        result = self.moduleTools.unload()
        if len(result[0]) > 0:
            for module in result[0]:
                HnsLogger.log_msg_stderr("Module succesfully unloaded: " + module)
        if len(result[1]) > 0:
            self.error = True
            for module in result[1]:
                HnsLogger.log_error("Module unloading failed: " + module)
        self.serviceVbox.start(True)
        self.virshTools.restartService()
        if self.error:
            return False
        return True

    def clean(self):
        result = self.virshTools.stopVM()
        #~ if not result[0]:
            #~ self.error = "\t"+result[1]
            #~ return False
        if len(result[1]) > 0:
            for msg in result[1]:
                HnsLogger.log_msg_stderr(msg)
        result = self.virshTools.undefineVM()
        #~ if not result[0]:
            #~ self.error = "\t"+result[1]
            #~ return False
        if len(result[1]) > 0:
            for msg in result[1]:
                HnsLogger.log_msg_stderr(msg)
        if os.path.isdir("/root/.VirtualBox"):
            shutil.rmtree("/root/.VirtualBox")
        if os.path.isdir("/root/VirtualBox VMs"):
            shutil.rmtree("/root/VirtualBox VMs")
        self.binaryTools.killAll(signal = 9)
        self.virshTools.restartService()
        return True

    def cleanBlackList(self):
        if self.enable:
            return "";
        else:
            return "VirtualBox"


class KVM_QemuElement(EntityElement):
    def __init__(self):
        EntityElement.__init__(self)
        self.title = "qemu/KVM"
        self.enable = self.parseEnable("KvmQemu")
        self.virshTools = VirshTools("qemu:///system")
        self.virshTools.addVersion("14001") #version 0.14.1
        self.virshTools.addVersion("1001002") #version 1.1.2
        self.virshTools.addVersion("2001002") #version 2.1.2
        self.virshTools.addVersion("2005000") #version 2.5.0
        self.moduleTools = ModuleTools()
        self.moduleTools.addLoadModule("kvm_intel")
        self.moduleTools.addLoadModule("kvm")
        self.serviceVbox = ServiceTools("vboxdrv")
        self.virshTools.has_snapshot = True

    def init(self):
        self.serviceVbox.stop()
        result = self.moduleTools.load()
        if len(result[0]) > 0:
            for module in result[0]:
                HnsLogger.log_msg_stderr("Module succesfully unloaded: " + module)
        if len(result[1]) > 0:
            self.error = True
            for module in result[1]:
                HnsLogger.log_error("Module unloading failed: " + module)
        self.virshTools.restartService()
        if self.error:
            return False
        return True

    def check(self):
        result = self.virshTools.checkVersion()
        if not result[0]:
            self.error = True
            HnsLogger.log_error(result[1])
        else:
            if result[1] != "":
                HnsLogger.log_msg_stderr(result[1])
            result = self.virshTools.checkVM()

            if result[0]:
                if len(result[1]) > 0:
                    for msg in result[1]:
                        HnsLogger.log_msg_stderr(msg)
            else:
                self.error = True
                if len(result[1]) > 0:
                    for error in result[1]:
                        HnsLogger.log_error(error)
        if self.error:
            self.blacklist.add("qemu/KVM")
            return False
        return True

    def clean(self):
        result = self.virshTools.stopVM()
        #~ if not result[0]:
            #~ self.error = "\t"+result[1]
            #~ return False
        if len(result[1]) > 0:
            for msg in result[1]:
                HnsLogger.log_msg_stderr(msg)
        result = self.virshTools.undefineVM()
        #~ if not result[0]:
            #~ self.error = "\t"+result[1]
            #~ return False
        if len(result[1]) > 0:
            for msg in result[1]:
                HnsLogger.log_msg_stderr(msg)
        return True

    def cleanBlackList(self):
        if self.enable:
            return "";
        else:
            return "KvmQemu"


class OpenVZElement(EntityElement):
    def __init__(self):
        EntityElement.__init__(self)
        self.enable = self.parseEnable("OpenVZ")
        self.title = "OpenVZ"
        self.service = ServiceTools("vz")
        self.virshTools = VirshTools("openvz:///system")
        self.virshTools.addVersion("^300002[4-9]") #version >= 3.0.24
        self.virshTools.addVersion("^30000[3-9]\d")
        self.virshTools.addVersion("^3000[1-9]\d{2,2}")
        self.virshTools.addVersion("^300[1-9]\d{3,3}")
        self.virshTools.addVersion("^30[1-9]\d{4,4}")
        self.virshTools.addVersion("^3[1-9]\d{5,5}")
        self.virshTools.addVersion("^[4-9]\d{6,6}")

    def check(self):
        if Util.exectBinary(["uname -r | egrep \"(ovz|openvz)\" > /dev/null"] ) != 0:
            self.error = True
            HnsLogger.log_error("The current kernel does not seem to support OpenVZ")
        else:
            exprConf = "^\d{3,3}.conf$"
            pathConf = "/etc/vz/conf"
            for conf_file in os.listdir(pathConf):
                if re.match(exprConf, conf_file):
                    self.error = True
                    HnsLogger.log_error("File found: " + os.path.join(pathConf, conf_file))
            if not self.service.isExist():
                self.error = True
                HnsLogger.log_error("Could not find service: " + self.service.service)
            else:
                results = self.service.getStatus()
                if results == "":
                    HnsLogger.log_error("Service is not responding to status request: " + self.service.service)
                    self.error = True
                else:
                    HnsLogger.log_msg_stderr(results)
            result = self.virshTools.checkVersion()
            if not result[0]:
                self.error = True
                HnsLogger.log_error(result[1])
            else:
                if result[1]!= "":
                    HnsLogger.log_msg_stderr(result[1])
                result = self.virshTools.checkVM()
                if result[0]:
                    if len(result[1]) > 0:
                        for msg in result[1]:
                            HnsLogger.log_msg_stderr(msg)
                else:
                    self.error = True
                    if len(result[1]) > 0:
                        for error in result[1]:
                            HnsLogger.log_error(error)
        if self.error:
            self.blacklist.add("OpenVZ")
            return False
        return True

    def init(self):
        self.service.start()
        self.virshTools.restartService()
        return True

    def clean(self):
        result = self.virshTools.stopVM()
        #~ if not result[0]:
            #~ self.error = "\t"+result[1]
            #~ return False
        if len(result[1]) > 0:
            for msg in result[1]:
                HnsLogger.log_msg_stderr(msg)

        exprConf = "^\d{3,3}.conf$"
        exprMount = "^\d{3,3}.mount$"
        pathConf = "/etc/vz/conf"
        for conf_file in os.listdir(pathConf):
            if re.match(exprConf, conf_file) or re.match(exprMount, conf_file):
                 os.remove(os.path.join(pathConf, conf_file))
        self.service.stop()
        return True

    def cleanBlackList(self):
        if self.enable:
            return "";
        else:
            return "OpenVZ"


class VmWareElement(EntityElement):
    def __init__(self):
        EntityElement.__init__(self)
        self.title = "VmWare"
        self.enable = self.parseEnable("VMware")
        self.virshTools = VirshTools("vmwarews:///session")
        self.virshTools.addVersion("^8000002") #version 8.0.2
        self.virshTools.addVersion("^10000001") #version 10.0.1
        self.virshTools.addVersion("^11001000") #version 11.1.0
        self.interfaceTools = InterfaceTools()
        self.interfaceTools.addRegExpTap("^vmnet\d+bt$")
        self.interfaceTools.addRegExpIfaceDev("^vmnet\d+$")
        self.interfaceTools.addRegExpBridge("^vmnet\d+b$")
        self.regPid = "vmnet\d+.pid"
        self.service = ServiceTools("vmware")
        self.binaryTools = BinaryTools()
        self.binaryTools.addBinary("vmnet-netifup")

    def clean(self):
        result = self.virshTools.stopVM()
        #~ if not result[0]:
            #~ self.error = "\t"+result[1]
            #~ return False
        if len(result[1]) > 0:
            for msg in result[1]:
                HnsLogger.log_msg_stderr(msg)
        result = self.virshTools.undefineVM()
        if result[0]:
            if len(result[1]) > 0:
                for msg in result[1]:
                    HnsLogger.log_msg_stderr(msg)
        else:
            self.error = True
            if len(result[1]) > 0:
                for error in result[1]:
                    HnsLogger.log_error(error)
            return False

        self.binaryTools.killAll()
        self.service.stop()
        self.interfaceTools.clearBridge()
        self.interfaceTools.clearTap()
        self.interfaceTools.clearIfaceDev()
        pathConf = "/var/run/hynesim"
        if os.path.isdir(pathConf):
            for conf_file in os.listdir(pathConf):
                if re.match(self.regPid, conf_file):
                    os.remove(os.path.join(pathConf ,conf_file))
        return True

    def init(self):
        self.service.start()
        self.virshTools.restartService()
        return True

    def check(self):
        result = self.virshTools.checkVersion()
        if not result[0]:
            self.error = True
            HnsLogger.log_error(result[1])
        else:
            if result[1]!= "":
                HnsLogger.log_msg_stderr(result[1])
            result = self.virshTools.checkVM()
            if result[0]:
                if len(result[1]) > 0:
                    for msg in result[1]:
                        HnsLogger.log_msg_stderr(msg)
            else:
                self.error = True
                if len(result[1]) > 0:
                    for error in result[1]:
                        HnsLogger.log_error(error)
        result = self.binaryTools.isRunning()
        if len(result[0]) > 0:
            self.error = True
            for binaryRunning in result[0]:
                HnsLogger.log_error(binaryRunning + " is running.")
        for binary in result[1]:
            HnsLogger.log_msg_stderr(binary + " is stopped.")

        listIface = self.interfaceTools.bridgeListExist()
        if len(listIface[0]) > 0:
            self.error = True
            for iface in listIface[0]:
                HnsLogger.log_error("Network interface '" + iface + "' is still UP.")
        listIface = self.interfaceTools.tapListExist()
        if len(listIface[0]) > 0:
            self.error = True
            for iface in listIface[0]:
                HnsLogger.log_error("Network interface '" + iface + "' is still UP.")
        listIface = self.interfaceTools.ifaceDevListExist()
        if len(listIface[0]) > 0:
            self.error = True
            for iface in listIface[0]:
                HnsLogger.log_error("Network interface '" + iface + "' is still UP.")

        if not self.service.isExist():
            HnsLogger.log_error("Could not find service: " + self.service.service)
            self.error = True
        else:
            results = self.service.getStatus()
            if results == "":
                HnsLogger.log_error("Service is not responding to status request: " + self.service.service)
                self.error = True
            else:
                HnsLogger.log_msg_stderr(results)
        #pathConf = "/dev"
        #for myFile in os.listdir(pathConf):
        #    if re.match(self.regExpIfaceDev, myFile):
        #         self.error += "\n\tdev found: /dev/" + myFile

        pathConf = "/var/run/hynesim"
        if os.path.isdir(pathConf):
            for conf_file in os.listdir(pathConf):
                if re.match(self.regPid, conf_file):
                    self.error = True
                    HnsLogger.log_error("File found: /var/run/hynesim/" + conf_file)
        if self.error:
            self.blacklist.add("VmWare")
            return False
        return True

    def cleanBlackList(self):
        if self.enable:
            return "";
        else:
            return "VMware"

class LXCElement(EntityElement):
    def __init__(self):
        EntityElement.__init__(self)
        self.title = "LXC"
        self.enable = self.parseEnable("LXC")
        self.virshTools = VirshTools("lxc:///")

    def find_cgroups(self):
        #~ Return True if mounted cgroups are matching required cgroups, false otherwise
        CGROUP_CPU="cpu"
        CGROUP_CPUSET="cpuset"
        CGROUP_CPUACCT="cpuacct"
        CGROUP_MEMORY="memory"
        CGROUP_DEVICES="devices"
        CGROUP_FREEZER="freezer"
        CGROUP_NET_CLS="net_cls"
        CGROUP_BLKIO="blkio"
        CGROUP_PERF_EVENT="perf_event"
        required_cgroup_mount_options=[CGROUP_CPU, CGROUP_CPUSET, CGROUP_CPUACCT, CGROUP_MEMORY, CGROUP_DEVICES, CGROUP_FREEZER, CGROUP_NET_CLS, CGROUP_BLKIO, CGROUP_PERF_EVENT]
        total_mounted_cgroups_options=[]
        missing_cgroup_opts =[]
        for l in open("/proc/mounts", "r"):
            if l.split(" ")[2] == "cgroup": #All the filesystems with type cgroup
                for opt in l.split(" ")[3].split(","): #Get mount options by filesystem
                    if not opt in total_mounted_cgroups_options:
                        total_mounted_cgroups_options.append(opt)

        for required_opt in required_cgroup_mount_options:
            if not required_opt in total_mounted_cgroups_options:
                     missing_cgroup_opts.append(required_opt)

        if len(missing_cgroup_opts) > 0:
            HnsLogger.log_error("One or more cgroups options are required: " + ", ".join(missing_cgroup_opts))
            return False
        else:
            HnsLogger.log_msg_stderr("Cgroup mounts are OK")
            return True

    def init(self):
        return True

    def checkSystem(self):
        listResult = []
        listResultWarning = []
        commands = ["screen", "spiceterm"]
        for binary in commands:
            result = Util.checkBinary(binary)
            if not result[1]:
                listResultWarning.append(result[0])
            else:
                listResult.append(result[0])
        return [listResult, listResultWarning]

    def check(self):
        result =  self.checkSystem()
        if len(result[0]) > 0:
            for msg in result[0]:
                HnsLogger.log_msg_stderr(msg)
        if len(result[1]) > 0:
            self.error = True
            for msg in result[1]:
                HnsLogger.log_error(msg)

        result = self.virshTools.checkVM()
        if result[0]:
            if len(result[1]) > 0:
                for msg in result[1]:
                    HnsLogger.log_msg_stderr(msg)
        else:
            self.error = True
            if len(result[1]) > 0:
                for error in result[1]:
                    HnsLogger.log_error(error)

        self.error = self.error or not self.find_cgroups() #~ find_cgroups return True if cgroups are ok, false otherwise
        if self.error:
            self.blacklist.add("LXC")
            return False
        return True

    def clean(self):
        result = self.virshTools.stopVM()
        if len(result[1]) > 0:
            for msg in result[1]:
                HnsLogger.log_msg_stderr(msg)
        result = self.virshTools.undefineVM()
        if len(result[1]) > 0:
            for msg in result[1]:
                HnsLogger.log_msg_stderr(msg)
        return True

    def cleanBlackList(self):
        if self.enable:
            return "";
        else:
            return "LXC"


class DynamipsElement(EntityElement):
    def __init__(self):
        EntityElement.__init__(self)
        self.enable = self.parseEnable("DynamipsRouter")
        self.title = "Dynamips (for cisco router)"
        self.port = Util.getValueParser("Platforms","DynamipsRouter/port","int")
        self.service = ServiceTools("dynamips-utils")
        self.binaryTools = BinaryTools()
        self.binaryTools.addBinary("dynamips")

    def check(self):
        if self.port == -1:
            self.error = "\tCan't find port\n"
        results = self.service.getStatus()
        if results == "":
            HnsLogger.log_error("Service is not responding to status request: " + self.service.service)
            self.error = True
        else:
            HnsLogger.log_msg_stderr(results)
        result = self.binaryTools.isRunning()


        if len(result[1]) > 0:
            self.error = True
            for binary in result[1]:
                HnsLogger.log_error(binary + " is not running.")
        if self.error:
            self.blacklist.add("DynamipsRouter")
            return False
        return True

    def clean(self):
        self.service.stop()
        return True

    def init(self):
        self.service.start(True)
        return True


class WifiAPElement(EntityElement):
    def __init__(self):
        EntityElement.__init__(self)
        self.enable = self.parseEnable("WifiAP")
        self.title = "Wifi AP"
        self.interfaceTools = InterfaceTools()
        self.interfaceTools.addRegExpTap("^hnsAp\dbt$")
        self.interfaceTools.addRegExpBridge("^hnsAp\db$")
        self.interfaceTools.addRegExpIW("^hnsAp\d$")
        self.devices = list()
        idx = 1
        while True:
            result = Util.getValueParser("Platforms", "WifiAP/wifiDevice/"+str(idx)+"/device")
            if result =="":
                break
            else:
                self.interfaceTools.addRegExpIfaceWifiDev(result)
            idx += 1

    def check(self):
        result =  self.interfaceTools.ifaceWifiCheckSystem()
        if len(result[0]) > 0:
            for msg in result[0]:
                HnsLogger.log_msg_stderr(msg)
        if len(result[1]) > 0:
            self.error = True
            for msg in result[1]:
                HnsLogger.log_error(msg)
        HnsLogger.log_msg_stderr("Checking devices...")
        result = self.interfaceTools.ifaceWifiDevListExist()
        if len(result[0]) > 0:
            for msg in result[0][0]:
                HnsLogger.log_msg_stderr(msg)
        if len(result[1]) > 0:
            self.error = True
            for msg in result[1]:
                HnsLogger.log_error(msg)
        HnsLogger.log_msg_stderr("Checking for interfaces UP...")
        listIface = self.interfaceTools.bridgeListExist()
        if len(listIface[0]) > 0:
            self.error = True
            for iface in listIface[0]:
                HnsLogger.log_error("Network interface '" + iface + "' is still UP.")
        listIface = self.interfaceTools.tapListExist()
        if len(listIface[0]) > 0:
            self.error = True
            for iface in listIface[0]:
                HnsLogger.log_error("Network interface '" + iface + "' is still UP.")
        listIface = self.interfaceTools.ifaceDevListExist()
        if len(listIface[0]) > 0:
            self.error = True
            for iface in listIface[0]:
                HnsLogger.log_error("Network interface '" + iface + "' is still UP.")
        if self.error:
            self.blacklist.add("WifiAP")
            return False
        return True

    def clean(self):
        self.interfaceTools.clearIW()
        self.interfaceTools.clearBridge()
        self.interfaceTools.clearTap()
        return True


class HybridNetCardElement(EntityElement):
    def __init__(self):
        EntityElement.__init__(self)
        self.enable = self.parseEnable("HybridNetcard")
        self.title = "HybridNetCard"
        self.interfaceTools = InterfaceTools()
        idx = 1
        while True:
            result = Util.getValueParser("Platforms", "HybridNetcard/cards/" + str(idx) + "/device")
            if result == "":
                break
            else:
                self.interfaceTools.addRegExpIfaceDev(result)
                self.interfaceTools.addRegExpBridge("^"+result+"b$")
                self.interfaceTools.addRegExpTap("^"+result+"(v|\w*t)$")
            idx += 1

    def check(self):
        HnsLogger.log_msg_stderr("Checking for network interfaces...")
        listIface = self.interfaceTools.ifaceDevListExist()
        if len(listIface[0]) == 0 and len(listIface[1]) == 0:
            self.error = True
            HnsLogger.log_error("No network interface specified in configuration.")
        else:
            if len(listIface[0]) > 0:
                for iface in listIface[0]:
                    HnsLogger.log_msg_stderr("Network interface '" + iface + "' found")
            if len(listIface[1]) > 0:
                self.error = True
                for iface in listIface[1]:
                    HnsLogger.log_error("Network interface '" + iface + "' not found, please check your configuration.")
        HnsLogger.log_msg_stderr("Checking for interfaces UP...")
        listIface = self.interfaceTools.bridgeListExist()
        if len(listIface[0]) > 0:
            self.error = True
            for iface in listIface[0]:
                HnsLogger.log_error("Network interface '" + iface + "' is still UP.")
        listIface = self.interfaceTools.tapListExist()
        if len(listIface[0]) > 0:
            self.error = True
            for iface in listIface[0]:
                HnsLogger.log_error("Network interface '" + iface + "' is still UP.")
        if self.error:
            self.blacklist.add("HybridNetcard")
            return False
        return True

    def clean(self):
        self.interfaceTools.clearTap()
        self.interfaceTools.clearBridge()
        return True

class OpenVSwitchElement(EntityElement):
    def __init__(self):
        EntityElement.__init__(self)
        self.title = "OpenVSwitch"
        self.enable = True
        self.interfaceTools = InterfaceTools()
        self.service = ServiceTools("openvswitch-switch")
        self.interfaceTools.addRegExpTap("^hnsOVS[0-9]+p[0-9]+$")
        self.binaryTools = BinaryTools()
        self.binaryTools.addBinary("ovs-vswitchd")

    def getOVSwitch(self):
        #Gettings Switches with ovs-vsctl show
        #Parsing results with egrep because results are like Bridge "hnsOVS1"
        #With the first sed, removing the pattern 'Bridge "'
        #With the 2nd sed, removing the pattern '"' after hnsOVS1
        #Results are now hnsOVS1
        switchs = Util.exectBinary(["ovs-vsctl", "show", "|", "egrep", "Bridge\ \"hnsOVS[0-9]+", "|", "sed", "-r", "s/[\ ]+Bridge\ \"//g", "|", "sed", "-r", "s/\"//g"], value = True)
        if (switchs != -1 ):
            return switchs.split()
        else:
            return list()

    def check(self):
        self.error = False

        if not self.service.isExist():
            self.error = True
            HnsLogger.log_error("Could not find service: " + self.service.service)
        else:
            results = self.service.getStatus()
            if results == "":
                HnsLogger.log_error("Service is not responding to status request: " + self.service.service)
                self.error = True
            else:
                HnsLogger.log_msg_stderr(results)

        result = self.binaryTools.isRunning()
        if len(result[1]) > 0:
            self.error = True
            for binary in result[1]:
                HnsLogger.log_error(binary + " is not running.")

        listIface = self.interfaceTools.tapListExist()
        if len(listIface[0]) > 0:
            self.error = True
            for iface in listIface[0]:
                HnsLogger.log_error("Network interface '" + iface + "' is still UP.")
        listSwitch = self.getOVSwitch()
        if len(listSwitch) > 0:
            self.error = True
            for switch in listSwitch:
                HnsLogger.log_error("OpenVSwitch '" + switch + "' is still UP.")

        if self.error:
            self.blacklist.add("OpenVSwitch")
            return False
        return True

    def clean(self):
        result = self.binaryTools.isRunning()
        if len(result[0]) > 0:
            self.interfaceTools.clearTap()
            listSwitch = self.getOVSwitch()
            for switch in listSwitch:
                HnsLogger.log_msg_stderr("Deleting '" + switch + "'")
                Util.exectBinary(["ovs-vsctl", "del-br", switch], value = True)

class IptablesElement(AbstractElement):
    def __init__(self):
        AbstractElement.__init__(self)
        self.title = "Iptables"
        self.enable = True
        self.appDict = dict()
        filterDict = dict()
        filterDict["filter"] = set([ "INPUT", "FORWARD", "OUTPUT" ])
        filterDict["mangle"] = set([ "PREROUTING", "POSTROUTING" ])
        self.appDict["iptables"] = dict(filterDict)
        self.appDict["ip6tables"] = dict(filterDict)
        del filterDict["mangle"]
        self.appDict["ebtables"] = dict(filterDict)

    def clean(self):
        for (app, filterDict) in self.appDict.items():
            for (filter, chains) in filterDict.items():
                for chain in chains:
                    #Flush Rules
                    Util.exectBinary([app, "-t", filter, "-F", "HYNESIM_" + chain], verbose = False)
                    #Remove Link
                    Util.exectBinary([app, "-t", filter, "-D", chain, "-j", "HYNESIM_" + chain], verbose = False)
                    #Remove hynesim chain
                    Util.exectBinary([app, "-t", filter, "-X", "HYNESIM_" + chain], verbose = False)
        return True


class NftablesElement(AbstractElement):
    def __init__(self):
        AbstractElement.__init__(self)
        self.title = "Nftables"
        self.enable = True

        rules = ["nft add table inet hynesim_filter",
                 "nft add table inet hynesim_mangle",
                 "nft add table bridge hynesim_filter",
                 "nft add chain inet hynesim_filter input { type filter hook input priority 0 ;}",
                 "nft add chain inet hynesim_filter output { type filter hook output priority 0 ;}",
                 "nft add chain inet hynesim_mangle prerouting { type filter hook prerouting priority 0; }",
                 "nft add chain inet hynesim_mangle postrouting { type filter hook postrouting priority 0; }",
                 "nft add chain bridge hynesim_filter input { type filter hook input priority 0 ;}",
                 "nft add chain bridge hynesim_filter output { type filter hook output priority 0 ;}",
                 "nft add chain bridge hynesim_filter forward { type filter hook forward priority 0 ;}"]
        for rule in rules:
            Util.exectBinary([elem for elem in rule.replace(
                "\n", "").split(" ")], verbose=False)

    def clean(self):
        rules = ['nft flush table bridge hynesim_filter',
                 'nft flush table inet hynesim_mangle',
                 'nft flush table inet hynesim_filter'
                 'nft delete chain bridge hynesim_filter forward { type filter hook forward priority 0 ;}',
                 'nft delete chain bridge hynesim_filter output { type filter hook output priority 0 ;}',
                 'nft delete chain bridge hynesim_filter input { type filter hook input priority 0 ;}',
                 'nft delete chain inet hynesim_mangle postrouting { type filter hook postrouting priority 0; }',
                 'nft delete chain inet hynesim_mangle prerouting { type filter hook prerouting priority 0; }',
                 'nft delete chain inet hynesim_filter output { type filter hook output priority 0 ;}',
                 'nft delete chain inet hynesim_filter input { type filter hook input priority 0 ;}',
                 'nft delete table bridge hynesim_filter',
                 'nft delete table inet hynesim_mangle',
                 'nft delete table inet hynesim_filter']

        for rule in rules:
            Util.exectBinary([elem for elem in rule.replace(
                "\n", "").split(" ")], verbose=False)

        return True


class Config() :
    def __init__(self):
        self.configpath = "/etc/hynesim/hynesim-node.ini"
        self.elementList = list()
        self.blacklist = set()
        self.build_blacklist = False
        self.use_syslog = False
        self.result = 0

    def launch(self) :
        parser = OptionParser()
        parser.add_option("", "--check", dest="check", action="store_true" , default = False,
                  help="Default action. Check Hynesim node configuration and capabilities.")
        parser.add_option("", "--init", dest="init", action="store_true" , default = False,
                  help="Initialize Hynesim node")
        parser.add_option("", "--cleanup", dest="cleanup", action="store_true" , default = False,
                  help="Clean up any ressources conflicting whith Hynesim node")
        parser.add_option("-c", "--config", dest="configpath", metavar="FILE",
                  help="Configuration file to use. Default to " + self.configpath + " when unspecified")
        parser.add_option("", "--output-node-args", dest="build_blacklist", action="store_true" , default = False,
                  help="Advanced user only. Used for degraded mode")
        parser.add_option("", "--use-syslog", dest="use_syslog", action="store_true", default = False,
                  help="Redirect ouput to syslog")
        try :
            (options, args) =  parser.parse_args()
        except :
            self.result = 1
            self.end()

        if options.build_blacklist is True:
            self.build_blacklist = True

        if options.use_syslog is True:
            self.use_syslog = True

        HnsLogger.loadLogger("hynesim-node", self.use_syslog)

        if os.geteuid() != 0:
            HnsLogger.log_error("This script requires root privileges.")
            self.result = 1
            self.end()

        if options.configpath is not None:
            self.configpath = options.configpath
        if not os.path.isfile(self.configpath):
            HnsLogger.log_error("Configuration file '" + self.configpath + "' does not exists or is unreadable.")
            self.result = 1
            self.end()
        try:
            Util.initConfig(self.configpath)
        except HnsSettingsError as e:
            HnsLogger.log_error(e.value)
            self.result = 1
            self.end()
        except Exception as e:
            HnsLogger.log_error("Unexpected error, ", sys.exc_info()[0])
            self.result = 1
            self.end()

        vBoxElement = VBoxElement()
        kvmElement = KVM_QemuElement()
        vmwareElement = VmWareElement()

        if kvmElement.isEnable():
            canAddKvm = True

            if vBoxElement.isEnable():
                HnsLogger.log_warning("Capability conflict, KVMQemu and VirtualBox are exclusive.")

                canAddKvm = False
                #directly add to blacklist vBox and kvm
                self.blacklist.add("KvmQemu")
                self.blacklist.add("VirtualBox")

            if vmwareElement.isEnable() :
                HnsLogger.log_warning("Capability conflict, KVMQemu and VmWare are exclusive.")
                canAddKvm = False
                #directly add to blacklist vmware and kvm
                self.blacklist.add("KvmQemu")
                self.blacklist.add("VMware")

        self.elementList.append(FileElement())
        self.elementList.append(LXCElement())
        self.elementList.append(kvmElement)
        self.elementList.append(vBoxElement)
        self.elementList.append(vmwareElement)
        self.elementList.append(OpenVZElement())
        self.elementList.append(WireElement())
        self.elementList.append(DynamipsElement())
        self.elementList.append(HostapdElement())
        self.elementList.append(WifiAPElement())
        self.elementList.append(HybridNetCardElement())
        self.elementList.append(IptablesElement())
        self.elementList.append(NftablesElement())
        self.elementList.append(OpenVSwitchElement())
        self.elementList.append(NativeSwitchElement())
        self.elementList.append(TopologyGateElement())

        self.enableBlacklist = options.build_blacklist

        doAction = False
        if options.check is True :
            self.check()
            doAction = True
        if options.cleanup is True :
            self.cleanup()
            doAction = True
        if options.init is True :
            self.init()
            doAction = True

        #default behavior
        if not doAction:
            self.check()

        self.end()

    def check(self) :
        hasError = False
        for element in self.elementList:
            if element.doCheck() is False :
                hasError = True
                self.blacklist.update(element.blacklist)
        if hasError:
            if self.enableBlacklist:
                if "All" in self.blacklist:
                    print "all"
                else:
                    list = str()
                    for element in self.elementList:
                        discard = element.cleanBlackList()
                        if discard != "":
                            self.blacklist.discard(discard)
                    for badElement in self.blacklist:
                        list += badElement+","
                    if list != "":
                        print list.rstrip(',')

    def cleanup(self) :
        for element in self.elementList:
            if element.doClean() is False :
                self.result = 1

    def init(self) :
        for element in self.elementList:
            if element.doInit() is False :
                self.result = 1

    def end(self):
        HnsLogger.unloadLogger()
        exit(self.result)


if __name__ == '__main__':
    config = Config()
    result = config.launch()
    exit(result)
