import hashlib
import sqlite3
import uuid

from __main__ import SqlMigrationError
from __main__ import Migration

USERS_TABLE = "users"
USERS_ID_COLUMN = "id"
USERS_NAME_COLUMN = "name"
USERS_ISADMIN_COLUMN = "isAdmin"
USERS_ISENABLED_COLUMN = "isEnabled"
USERS_CREATION_GROUP_COLUMN = "creationGroupId"
USERS_CREATION_PERMISSIONS_COLUMN = "creationPerms"
USERS_CREATED_ON_COLUMN = "createdOn"
USERS_MODIFIED_ON_COLUMN = "modifiedOn"
USERS_FIRSTNAME_COLUMN = "firstname"
USERS_LASTNAME_COLUMN = "lastname"
USERS_ISADVANCED_COLUMN = "isAdvanced"
USERS_HSETID_COLUMN = "hsetid"

USERS_TEMP_TABLE = "users_temp"

GROUPS_TABLE = "groups"
GROUPS_ID_COLUMN = "id"
GROUPS_NAME_COLUMN = "name"
GROUPS_CREATED_ON_COLUMN = "createdOn"
GROUPS_MODIFIED_ON_COLUMN = "modifiedOn"
GROUPS_HSETID_COLUMN = "hsetid"

ACLS_TABLE = "acls"
ACLS_TARGET_COLUMN = "target"
ACLS_SUBJECT_COLUMN = "subject"
ACLS_RIGHTS_COLUMN = "rights"
ACLS_HSETID_COLUMN = "hsetid"

HSETS_TABLE = "hsets"
HSETS_ID_COLUMN = "id"
HSETS_RANK_COLUMN = "rank"
HSETS_NAME_COLUMN = "name"
HSETS_DESCRIPTION_COLUMN = "description"


class ChangingUsersRight(Migration):

    def migrate_impl(self):
        db_path = self.db_name_path_dict['Diamesh/defaultDb']

        conn = sqlite3.connect(db_path)
        conn.row_factory = sqlite3.Row
        ########################################USERS_TABLE
        conn.execute("ALTER TABLE %s ADD %s integer" % (USERS_TABLE, USERS_HSETID_COLUMN))
        
        for row in conn.execute("SELECT * FROM %s" % USERS_TABLE):
            userName = row[USERS_NAME_COLUMN]
            isAdmin = row[USERS_ISADMIN_COLUMN]
            hsetId = 2 #We initialize the hsetId to users hsetId
            if (isAdmin == 1 or isAdmin == True or isAdmin == "true"):
                hsetId = 1
            
            query = "UPDATE %s SET %s = ? WHERE %s = ?" %(USERS_TABLE, USERS_HSETID_COLUMN, USERS_NAME_COLUMN)
            queryValues = (hsetId, userName)
            conn.execute(query, queryValues)

        ########################################USERS_TABLE

        ########################################GROUPS_TABLE
        conn.execute("ALTER TABLE %s ADD %s integer" % (GROUPS_TABLE, GROUPS_HSETID_COLUMN))
        
        for row in conn.execute("SELECT * FROM %s" % GROUPS_TABLE):
                ###Set admin hsetId by default for everyone
            hsetId = 1
            query = "UPDATE %s SET %s = ? " %(GROUPS_TABLE, GROUPS_HSETID_COLUMN)
            queryValues = (hsetId,)
            conn.execute(query, queryValues)

                ###If there is a user with the same name as the group, we change the hsetId to the user one
            groupName = row[GROUPS_NAME_COLUMN]
            queryValues = (groupName, )
            query = "SELECT * from %s WHERE %s = ?" % (USERS_TABLE, USERS_NAME_COLUMN)
            result = conn.execute(query, queryValues).fetchone() ###See http://docs.python.org/2/library/sqlite3.html#cursor-objects for the use of fetchone()
            if result is not None:
                hsetId = result[USERS_HSETID_COLUMN]
                query = "UPDATE %s SET %s = ? WHERE %s = ?" %(GROUPS_TABLE, GROUPS_HSETID_COLUMN, GROUPS_NAME_COLUMN)
                queryValues = (hsetId, userName)
                conn.execute(query, queryValues)
        ########################################GROUPS_TABLE

        ########################################HSETS_TABLE
        conn.execute("CREATE TABLE %s (%s integer primary key autoincrement, %s integer, %s text, %s text)" % (HSETS_TABLE, HSETS_ID_COLUMN, HSETS_RANK_COLUMN, HSETS_NAME_COLUMN, HSETS_DESCRIPTION_COLUMN))

        conn.execute("INSERT INTO %s VALUES(1, 0, 'Administrators', 'Administrators rank')" % HSETS_TABLE)
        conn.execute("INSERT INTO %s VALUES(2, 1, 'Users', 'Hynesim users')" % HSETS_TABLE)
        ########################################HSETS_TABLE

        ########################################ACLS_TABLE
        conn.execute("CREATE TABLE %s (%s text, %s text, %s text, %s integer, primary key(%s, %s))" % (ACLS_TABLE, ACLS_TARGET_COLUMN, ACLS_SUBJECT_COLUMN, ACLS_RIGHTS_COLUMN, ACLS_HSETID_COLUMN, ACLS_TARGET_COLUMN, ACLS_SUBJECT_COLUMN))

        conn.execute("INSERT INTO %s VALUES('*','u.1','*',1)" % ACLS_TABLE)
        conn.execute("INSERT INTO %s VALUES('u.2','u.2','diamesh.usermanager.user.edit diamesh.user',1)" % ACLS_TABLE)
        conn.execute("INSERT INTO %s VALUES('g.2','u.2','diamesh.usermanager.group.edit diamesh.usermanager.group.users.add diamesh.usermanager.group.users.remove',1)" % ACLS_TABLE)
        ########################################ACLS_TABLE

        
        ###Workaround because SQLite does not support DROP COLUMN, see https://www.sqlite.org/faq.html#q11
        conn.execute("CREATE TEMPORARY TABLE %s (%s integer primary key autoincrement, %s text unique not null, %s integer, %s integer, %s integer, %s integer, %s text, %s text, %s integer)" % (USERS_TEMP_TABLE, USERS_ID_COLUMN, USERS_NAME_COLUMN, USERS_ISENABLED_COLUMN, USERS_CREATED_ON_COLUMN, USERS_MODIFIED_ON_COLUMN, USERS_HSETID_COLUMN, USERS_FIRSTNAME_COLUMN, USERS_LASTNAME_COLUMN, USERS_ISADVANCED_COLUMN))
        conn.execute("INSERT INTO %s SELECT %s, %s, %s, %s, %s, %s, %s, %s, %s FROM %s" % (USERS_TEMP_TABLE, USERS_ID_COLUMN, USERS_NAME_COLUMN, USERS_ISENABLED_COLUMN, USERS_CREATED_ON_COLUMN, USERS_MODIFIED_ON_COLUMN, USERS_HSETID_COLUMN, USERS_FIRSTNAME_COLUMN, USERS_LASTNAME_COLUMN, USERS_ISADVANCED_COLUMN, USERS_TABLE))
        conn.execute("DROP TABLE %s" % USERS_TABLE)

        conn.execute("CREATE TABLE %s (%s integer primary key autoincrement, %s text unique not null, %s integer, %s integer, %s integer, %s integer, %s text, %s text, %s integer)" % (USERS_TABLE, USERS_ID_COLUMN, USERS_NAME_COLUMN, USERS_ISENABLED_COLUMN, USERS_CREATED_ON_COLUMN, USERS_MODIFIED_ON_COLUMN, USERS_HSETID_COLUMN, USERS_FIRSTNAME_COLUMN, USERS_LASTNAME_COLUMN, USERS_ISADVANCED_COLUMN))
        conn.execute("INSERT INTO %s SELECT %s, %s, %s, %s, %s, %s, %s, %s, %s FROM %s" % (USERS_TABLE, USERS_ID_COLUMN, USERS_NAME_COLUMN, USERS_ISENABLED_COLUMN, USERS_CREATED_ON_COLUMN, USERS_MODIFIED_ON_COLUMN, USERS_HSETID_COLUMN, USERS_FIRSTNAME_COLUMN, USERS_LASTNAME_COLUMN, USERS_ISADVANCED_COLUMN, USERS_TEMP_TABLE))
        conn.execute("DROP TABLE %s" % USERS_TEMP_TABLE)

        conn.commit()
        conn.close()
