/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef HNVMAINWINDOW_H
#define HNVMAINWINDOW_H

// Project headers
#include "logmessagewidget.h"

// diamesh headers
#include <diamesh/sessionhelper.h>

// Qt headers
#include <QDateTime>
#include <QMainWindow>
#include <QSharedPointer>

class QMessageBox;

namespace HnsClient {
class EntityEntryCatalog;
class TopologyEntryCatalog;
class TopologyManager;
} // namespace HnsClient

namespace Ui {
class HnvMainWindow;
} //namespace Ui

namespace Hns {
class PluginManager;
} // namespace Hns

namespace RemoteAccess {
class RemoteManager;
} // namespace RemoteAccess

namespace Hnv {
class ModeWidget;
class HyneviewSession;
class UsersModel;
class GroupsModel;
class SessionsModel;
class TopologyCatalogModel;
class LogMessageModel;
class TopologyWidget;
class SimulationModeWidget;
class EntityCatalogModel;
class PlatformMonitorModel;
class EntityInfoModel;
class ApplicationSettingsDialog;

/*******************************************************************************
* HnvMainWindow
*******************************************************************************/
class HnvMainWindow : public QMainWindow
{
    Q_OBJECT
public:
    static HnvMainWindow *create(QWidget *parent = 0);
    static HnvMainWindow *instance();
    static void messageHandler(QtMsgType type, const char *msg);

    ~HnvMainWindow();

    void registerModeWidget(ModeWidget *widget, bool isBottomWidget = false);
    void addFileMenuAction(QList<QAction *> actionList);
    HyneviewSession *session();
    QString generatePassword();
    QString aboutMessage(bool displayMasterInfo = false);

    HnsClient::EntityEntryCatalog *entityCatalog();
    HnsClient::TopologyEntryCatalog *topologyCatalog();
    HnsClient::TopologyManager *topologyManager();

    EntityCatalogModel *entityCatalogModel();
    EntityInfoModel *entityInfoModel();
    LogMessageModel *logMessageModel();
    TopologyCatalogModel *topologyCatalogModel();
    PlatformMonitorModel *platformMonitorModel();
    UsersModel *usersModel();
    GroupsModel *groupsModel();
    SessionsModel *sessionsModel();

    ApplicationSettingsDialog *applicationSettingsDialog();
    Hns::PluginManager *pluginManager();

public slots:
    void addLogMessage(LogMessageItem::MessageType type, const QString & msg, QDateTime dateTime = QDateTime::currentDateTime());
    void connectTriggered();
    void quickConnectTriggered();
    void remoteProcessStarted(QString title, QString message);
    void remoteProcessEnded();
    void zoomValueChangedSlot(int value);
    void cursorPositionChangedSlot(int x, int y, bool inScene);
    void clearLabelCoord();
    void loadingStarted(QString message);
    void loadingEnded(QString message);
    void entityCatalogLoadingSlot(int transactionCount, bool initialLoading);

signals:
    void sessionConnected(HyneviewSession *session);
    void hnsClientSessionConnected(); //TODO: should be rename as sessionConnected() in the end with no args
    void sessionDisconnected();
    void zoomValueChanged(int value);
    void enableQuickConnect(bool storePassword);
    void refreshRemoteDisplay();
    void newLogMessage(LogMessageItem::MessageType, const QString &msg, QDateTime dateTime);

protected:
    static QtMsgHandler _oldMsgHandler;
    void _initWindow();
    void closeEvent(QCloseEvent *event);

private slots:
    void _toggleSideBar();
    void _toggleRightSideBar();
    void _closeLogMessageWidget();
    void _toggleLogMessageWidget(bool);
    void _connectStartedSlot();
    void _connectEndedSlot();
    void _disconnectTriggered();
    void _sessionDisconnectedSlot();
    void on_action_About_triggered();
    void on_action_Options_triggered();
    void _newTraceSlot(QString message, QDateTime messageDate);
    void on_actionPopupStyle_triggered();
    void _updateACLFailedSlot(QString errorMessage);
    void on_action_Import_triggered();
    void _closeTopologiesFailedSlot();
    void _modeChanged(QWidget *w);

private:
    explicit HnvMainWindow(QWidget *parent = 0);
    bool _canTerminateCurrentSession();
    bool _closeTopologies();
    void _clickModeButton(int buttonIndex);
    void _connectionEstablished(QSharedPointer<Diamesh::SessionHelper> sessionHelper);
    void _disconnectSession(bool userAction);
    TopologyWidget *_topologyWidget();
    void _populateMenu(QMenu *menu, QList<QAction *> actionList);
    void _toggleLocalActions();

    Ui::HnvMainWindow *ui;
    HyneviewSession *_session;
    bool _connectActionRequested;
    static HnvMainWindow *_instance;
    TopologyCatalogModel *_topologyCatalogModel;
    LogMessageModel *_logMessageModel;
    UsersModel *_usersModel;
    GroupsModel *_groupsModel;
    SessionsModel *_sessionsModel;
    int _runningRemoteProcess;
    SimulationModeWidget *_simulationWidget;
    bool _logMessageDisplayed;
    int _loadingCount;
    QMessageBox *_waitingBox;
    bool _dataLoading;
    EntityCatalogModel *_entityCatalogModel;
    PlatformMonitorModel *_platforMonitorModel;
    EntityInfoModel *_entityInfoModel;
    Hns::PluginManager *_pluginManager;
    ApplicationSettingsDialog *_applicationSettings;
    bool _topologiesClosed;
};

} // namespace Hnv

#endif // HNVMAINWINDOW_H
