/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "hnvmainwindow.h"
#include "hnvstyle.h"

// hynesim common headers
#include "common/hns.h"

// Generated hynesim headers
#include "common/version.h"

// diamesh headers
#include <diamesh/application.h>
#include <diamesh/constants.h>
#include <diamesh/info.h>

// Qt headers
#include <QScopedPointer>
#include <QSharedPointer>
#include <QPixmapCache>

int main(int argc, char *argv[])
{
//    QApplication::setGraphicsSystem("raster");
    QHash<QString, QString> appConfigHash;
    appConfigHash.insert(Diamesh::Constants::APP_CONFIG_NAME, "hyneview");
    appConfigHash.insert(Diamesh::Constants::APP_CONFIG_ORGANIZATION, "diateam");
    QSharedPointer<Diamesh::Application> a(Diamesh::Application::create(argc, argv, 0, appConfigHash));
    Diamesh::VersionInfo info = Hns::versionInfo();
    QString version = QString::fromStdString(info.version);
    QString versionDescription = QString::fromStdString(info.versionDescription);
    QString revision = QString::fromStdString(info.revision);
    Hns::loadTranslation(QString("hyneview_"));
    Hns::loadTranslation(QString("hnscommon_"), false);
    QString appVersion = version;
    if (!versionDescription.isEmpty())
        appVersion.append(QString(" %1").arg(versionDescription));
    if (!revision.isEmpty())
        appVersion.append(QString(" (rev %1)").arg(revision));

    a->setApplicationVersion(appVersion);
    Hnv::HnvStyle *style = new Hnv::HnvStyle();
    a->setStyle(style);
    a->setAttribute(Qt::AA_DontShowIconsInMenus, false);
    a->setPalette(style->standardPalette());

    // Increase default cache size
    QPixmapCache::setCacheLimit(102400);

    QScopedPointer<Hnv::HnvMainWindow> window(Hnv::HnvMainWindow::create());
    window.data()->show();
    return a->exec();
}
