/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef HELPBROWSER_H
#define HELPBROWSER_H

// Qt headers
#include <QNetworkReply>
#include <QNetworkAccessManager>
#include <QNetworkRequest>
#include <QWebView>

class QHelpEngine;

namespace Hnv {

class HelpBrowserWidget;

/*******************************************************************************
* HelpNetworkReply
*/
class HelpNetworkReply : public QNetworkReply
{
public:
    HelpNetworkReply(const QNetworkRequest &request, const QByteArray &fileData, const QString &mimeType);
    virtual void abort();
    virtual qint64 bytesAvailable() const;

protected:
    virtual qint64 readData(char *data, qint64 maxlen);

private:
    QByteArray _data;
    qint64 _origLen;
};

/*******************************************************************************
* HelpNetworkAccessManager
*/
class HelpNetworkAccessManager : public QNetworkAccessManager
{
public:
    HelpNetworkAccessManager(QHelpEngine * engine, QObject *parent);

protected:
    virtual QNetworkReply *createRequest(Operation op, const QNetworkRequest &request, QIODevice *outgoingData = 0);
    QHelpEngine * _engine;
};

/*******************************************************************************
* HelpPage
*/
class HelpPage : public QWebPage
{
    Q_OBJECT
public:
    HelpPage(QObject *parent);

protected:
    virtual bool acceptNavigationRequest(QWebFrame *frame, const QNetworkRequest &request, NavigationType type);

signals:
    void titleChanged(QString title);

};

/*******************************************************************************
* HelpBrowser
*/
class HelpView : public QWebView
{
    Q_OBJECT
public:
    explicit HelpView(HelpBrowserWidget * browser, QHelpEngine * engine, QWidget *parent = 0);
    qlonglong viewID() const;

    bool isForwardAvailable() const;
    bool isBackwardAvailable() const;

public slots:
    void home();
    void nextPage();
    void previousPage();

signals:
    void viewChanged(qlonglong viewID, QString title);

protected:
    QWebView * createWindow(QWebPage::WebWindowType type);

protected slots:
    void _titleChangedSlot(QString title);

private:
    static qlonglong _nextViewID;
    qlonglong _viewID;
    HelpBrowserWidget * _browser;
    QHelpEngine * _engine;
};

} // namespace Hnv
#endif // HELPBROWSER_H
