/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef HNV_HELPBROWSERWIDGET_H
#define HNV_HELPBROWSERWIDGET_H

// Qt headers
#include <QWidget>
#include <QAbstractItemModel>

class QHelpEngine;

namespace Ui {
class HelpBrowserWidget;
} // namespace Ui

namespace Hnv {

class HelpView;
class LoadedHelpViewModel;

/*******************************************************************************
* HelpBrowserWidget
*/
class HelpBrowserWidget : public QWidget
{
    Q_OBJECT
public:
    explicit HelpBrowserWidget(QHelpEngine * helpEngine, QWidget *parent = 0);
    ~HelpBrowserWidget();
    void load(QUrl link);
    void activateView(qlonglong viewID);
    void closeView(qlonglong viewID);
    bool isForwardAvailable() const;
    bool isBackwardAvailable() const;
    HelpView * currentHelpView() const;
    qlonglong currentViewID() const;
    HelpView * viewByID(qlonglong viewID) const;
    QList<qlonglong> viewIDList() const;
    LoadedHelpViewModel * loadedViewModel() const;

    HelpView * createEmptyTab();

public slots:
    void home();
    void nextPage();
    void previousPage();

signals:
    void helpViewCreated(qlonglong viewID);
    void helpViewChanged(qlonglong viewID);
    void helpViewClosed(qlonglong viewID);

protected:
    void _setTitle(QString title = QString());
    void _updateNavigationActions(bool homeEnabled);

protected slots:
    void _viewChangedSlot(qlonglong viewID, QString title);
    void _closeViewSlot(qlonglong viewID = 0);
    void _currentViewChanged(int index);

private:
    Ui::HelpBrowserWidget *ui;
    QHelpEngine * _helpEngine;
    LoadedHelpViewModel * _loadedModel;
    QHash<qlonglong, HelpView *> _viewHash;
};

/*******************************************************************************
* LoadedHelpViewModel
*/
class LoadedHelpViewModel : public QAbstractItemModel
{
    Q_OBJECT
public:
    explicit LoadedHelpViewModel(HelpBrowserWidget * browser, QObject * parent = 0);
    virtual QModelIndex index(int row, int column, const QModelIndex &parent = QModelIndex()) const;
    virtual QModelIndex parent(const QModelIndex &child) const;
    int rowCount(const QModelIndex &parent) const;
    int columnCount(const QModelIndex &parent) const;
    QVariant data(const QModelIndex &index, int role) const;
    void clear();

protected slots:
    void _helpViewCreatedSlot(qlonglong browserID);
    void _helpViewChangedSlot(qlonglong browserID);
    void _helpViewClosedSlot(qlonglong browserID);

protected:
    HelpBrowserWidget * _browser;
    QList<qlonglong> _viewList;
};

} // namespace Hnv
#endif // HNV_HELPBROWSERWIDGET_H
