/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "helpcontentwidget.h"
#include "helpmodewidget.h"

// Qt headers
#include <QHelpEngine>
#include <QHelpContentModel>

namespace Hnv {

/*******************************************************************************
* HelpContentWidget
*******************************************************************************/
HelpContentWidget::HelpContentWidget(QWidget *parent)
    : SideBarTreeWidget(parent)
    , _helpModeWidget(0)
    , _model(0)
{
    QWidget * widget = sideBarParent();
    if (widget)
        _helpModeWidget = qobject_cast<HelpModeWidget *>(widget);
    if (_helpModeWidget)
        _model = _helpModeWidget->helpEngine()->contentModel();
    setViewModel(_model);
    connect(_treeView, SIGNAL(activated(QModelIndex)), this, SLOT(_viewActivatedSlot(QModelIndex)));
}

HelpContentWidget::~HelpContentWidget()
{
}

QString HelpContentWidget::title()
{
    return tr("Content");
}

QString HelpContentWidget::sideBarId()
{
    return "Content";
}

void HelpContentWidget::_indexClicked(const QModelIndex &index)
{
    _viewActivatedSlot(index);
}

void HelpContentWidget::_indexDoubleClicked(const QModelIndex &index)
{
    Q_UNUSED(index)
}

void HelpContentWidget::_contextMenuRequested(const QModelIndex &index, const QPoint &position)
{
    Q_UNUSED(index)
    Q_UNUSED(position)
}

void HelpContentWidget::_viewActivatedSlot(QModelIndex index)
{
    if (_model) {
        QHelpContentItem * item = _model->contentItemAt(index);
        if (item && _helpModeWidget)
            _helpModeWidget->activateLink(item->url());
    }
}

} // namespace Hnv
