/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "helpmodewidget.h"
#include "helpbrowserwidget.h"
#include "ressources.h"
#include "sidebar.h"
#include "helppagelistwidget.h"
#include "helpcontentwidget.h"
#include "helpsearchwidget.h"

// Qt headers
#include <QAction>
#include <QFile>
#include <QDir>
#include <QSettings>
#include <QApplication>
#include <QHelpEngine>
#include <QHelpSearchEngine>
#include <QtPlugin>

namespace Hnv {

/*******************************************************************************
 * HelpModePlugin
 ******************************************************************************/
HelpModePlugin::HelpModePlugin(QObject *parent)
    : ModePlugin(parent)
{
    setDisabledByDefault("HelpMode");
}

QString HelpModePlugin::name() const
{
    return tr("Help mode");
}

QString HelpModePlugin::description() const
{
    return tr("Help mode plugin");
}

void HelpModePlugin::_initModeWidget(QWidget *parent)
{
    if (!_modeWidgetSingleton)
        _modeWidgetSingleton = new HelpModeWidget(parent);
}

/*******************************************************************************
* HelpModeWidget
*/
HelpModeWidget::HelpModeWidget(QWidget *parent)
    : ModeWidget(parent)
{

    QString docPath = qApp->applicationDirPath() + "/doc"; // TODO : Need to use a constant
    if (!QFile::exists(docPath))
        docPath = "/usr/share/hynesim/doc"; // TODO : Need to use a configured prefix

    QString configDir = QFileInfo(QSettings(QSettings::UserScope, qApp->organizationName(), qApp->applicationName()).fileName()).path();
    _helpEngine = new QHelpEngine(configDir + "/hyneview.qhc", this);
    _browser = new HelpBrowserWidget(_helpEngine, this);
    _initDisplayArea(_browser);

    _helpEngine->setupData();

    QDir docDir(docPath);
    docDir.setFilter(QDir::Files | QDir::Hidden | QDir::NoSymLinks);
    QStringList filters;
    filters << "*.qch";
    docDir.setNameFilters(filters);
    QFileInfoList dirList = docDir.entryInfoList();
    for (int i = 0; i < dirList.size(); ++i) {
        QFileInfo fileInfo = dirList.at(i);
        _helpEngine->registerDocumentation(fileInfo.absoluteFilePath());
    }

    QList<SideBarItem *> itemList;
    itemList.append(new SideBarItem(HelpContentWidget::sideBarId(), HelpContentWidget::title(), HelpContentWidget::staticMetaObject, 1));
    itemList.append(new SideBarItem(HelpPageListWidget::sideBarId(), HelpPageListWidget::title(), HelpPageListWidget::staticMetaObject));
    itemList.append(new SideBarItem(HelpSearchWidget::sideBarId(), HelpSearchWidget::title(), HelpSearchWidget::staticMetaObject));
    _initSideBar(new SideBar(staticMetaObject.className(), HelpContentWidget::sideBarId(), itemList, this));
    _helpEngine->searchEngine()->reindexDocumentation();
}

HelpModeWidget::~HelpModeWidget()
{
}

QHelpEngine * HelpModeWidget::helpEngine()
{
    return _helpEngine;
}

bool HelpModeWidget::hasSideBar()
{
    return true;
}

bool HelpModeWidget::hasRightSideBar()
{
    return false;
}

/*******************************************************************************
* HelpModeWidget - Public slots
*/
void HelpModeWidget::activateLink(QUrl link)
{
    _browser->load(link);
}

/*******************************************************************************
* HelpModeWidget - Protected
*/
void HelpModeWidget::_initModeAction()
{
    _modeAction = new QAction(QIcon(Ressources::themedPath(Constants::ICON_HELP)), tr("Help"), this);
    _modeAction->setToolTip(tr("Switch to help"));
}

} // namespace Hnv

Q_EXPORT_PLUGIN2(helpModePlugin, Hnv::HelpModePlugin)
