/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "helppagelistwidget.h"
#include "helpmodewidget.h"
#include "helpbrowserwidget.h"

namespace Hnv {

/*******************************************************************************
* HelpPageListWidget
*******************************************************************************/
HelpPageListWidget::HelpPageListWidget(QWidget *parent)
    : SideBarTreeWidget(parent)
    , _helpWidget(0)
    , _loadedModel(0)
{
    HelpModeWidget * modeWidget = qobject_cast<HelpModeWidget*>(sideBarParent());
    if (modeWidget)
        _helpWidget = qobject_cast<HelpBrowserWidget*>(modeWidget->displayWidget());
    if (_helpWidget)
        _loadedModel = _helpWidget->loadedViewModel();
    setViewModel(_loadedModel);
    _initCloseDelegate(1);
}

QString HelpPageListWidget::title()
{
    return tr("Opened pages");
}

QString HelpPageListWidget::sideBarId()
{
    return "Opened pages";
}

void HelpPageListWidget::_indexClicked(const QModelIndex &index)
{
    if (index.isValid() && index.column() == 1)
        _closeIndex(index);
    else
        _displayIndex(index);
}

void HelpPageListWidget::_indexDoubleClicked(const QModelIndex &index)
{
    _displayIndex(index);
}

void HelpPageListWidget::_contextMenuRequested(const QModelIndex &index, const QPoint &position)
{
    Q_UNUSED(index)
    Q_UNUSED(position)
}

void HelpPageListWidget::_displayIndex(const QModelIndex &index)
{
    if (index.isValid() && _helpWidget)
        _helpWidget->activateView(index.data(Qt::UserRole).toLongLong());
}

void HelpPageListWidget::_closeIndex(const QModelIndex &index)
{
    if (index.isValid() && _helpWidget)
        _helpWidget->closeView(index.data(Qt::UserRole).toLongLong());
}

} // namespace Hnv
