/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef HNV_HELPSEARCHWIDGET_H
#define HNV_HELPSEARCHWIDGET_H

// Project headers
#include "sidebar.h"

// Qt headers
#include <QListWidget>

class QHelpSearchQueryWidget;
class QHelpSearchEngine;
class QHelpSearchQuery;

namespace Hnv {

class HelpModeWidget;

/*******************************************************************************
* HelpSearchResultWidget
*******************************************************************************/
class HelpSearchResultWidget : public QListWidget
{
    Q_OBJECT
public:
    explicit HelpSearchResultWidget(QWidget * parent = 0);
    ~HelpSearchResultWidget();
    void setEngine(QHelpSearchEngine * engine);
    void setResults(int hits);

signals:
    void requestShowLink(const QUrl &url);
private slots:
    void _itemActivatedSlot(QListWidgetItem * item);
private:
    QHelpSearchEngine * _searchEngine;
};

/*******************************************************************************
* HelpSearchWidget
*******************************************************************************/
class HelpSearchWidget : public SideBarSimpleWidget
{
    Q_OBJECT
public:
    Q_INVOKABLE explicit HelpSearchWidget(QWidget *parent = 0);
    ~HelpSearchWidget();

    static QString title();
    static QString sideBarId();

protected:
    void _initSearchEngine(QHelpSearchEngine * engine);

protected slots:
    void _searchSlot();
    void _searchingFinishedSlot(int hits);

private:
    QHelpSearchQueryWidget * _queryWidget;
    HelpSearchResultWidget * _resultWidget;
    QHelpSearchEngine * _searchEngine;
    QList<QHelpSearchQuery> _currentQueryList;
    bool _isSearching;
};

} // namespace Hnv

#endif // HNV_HELPSEARCHWIDGET_H
