/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "entityinfolistwidget.h"
#include "ressources.h"
#include "hnvmainwindow.h"
#include "platformmonitormodel.h"

// Qt headers
#include <QMenu>
#include <QAction>
#include <QTreeView>

namespace Hnv {

/*******************************************************************************
* SessionListWidget
*******************************************************************************/
EntityInfoListWidget::EntityInfoListWidget(QWidget *parent)
    : SideBarTreeWidget(parent)
    , _entityInfoListModel(NULL)
{
    _setIconSize(QSize(16, 16));
    HnvMainWindow * window = HnvMainWindow::instance();
    if (window)
        _entityInfoListModel = window->entityInfoModel();
    setViewModel(_entityInfoListModel);
    _treeView->setRootIsDecorated(true);
    _treeView->setAlternatingRowColors(true);
    _actionCollapseAll = new QAction(this);
    _actionCollapseAll->setText(tr("Collapse All"));
    _actionCollapseAll->setIcon(QIcon(Ressources::themedPath(Constants::ICON_COLLAPSE)));
    connect(_actionCollapseAll, SIGNAL(triggered()), this, SLOT(_action_CollapseAllSlot()));
    _actionExpandAll = new QAction(this);
    _actionExpandAll->setText(tr("Expand All"));
    _actionExpandAll->setIcon(QIcon(Ressources::themedPath(Constants::ICON_EXPAND)));
    connect(_actionExpandAll, SIGNAL(triggered()), this, SLOT(_action_ExpandAllSlot()));

    _contextMenu = new QMenu();
    _contextMenu->addAction(_actionCollapseAll);
    _contextMenu->addAction(_actionExpandAll);
}

EntityInfoListWidget::~EntityInfoListWidget()
{
    if (_contextMenu)
        delete _contextMenu;
}

QString EntityInfoListWidget::title()
{
    return tr("List Entities");
}

QString EntityInfoListWidget::sideBarId()
{
    return "List Entities";
}

void EntityInfoListWidget::_indexClicked(const QModelIndex &index)
{
    Q_UNUSED(index)
}

void EntityInfoListWidget::_indexDoubleClicked(const QModelIndex &index)
{
    Q_UNUSED(index)
}

void EntityInfoListWidget::_contextMenuRequested(const QModelIndex &index, const QPoint &position)
{
    if (_entityInfoListModel != NULL && _entityInfoListModel->rowCount(index.parent()) > 0)
        _contextMenu->exec(_globalPosition(position));
}

void EntityInfoListWidget::_action_ExpandAllSlot()
{
    _treeView->expandAll();
}

void EntityInfoListWidget::_action_CollapseAllSlot()
{
    _treeView->collapseAll();
}

} // namespace Hnv
