/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "meshlistwidget.h"
#include "platformmonitormodel.h"
#include "hnvmainwindow.h"
#include "platformmonitormodewidget.h"
#include "platformmonitorwidget.h"

namespace Hnv {

/*******************************************************************************
* MeshListWidget
*******************************************************************************/
MeshListWidget::MeshListWidget(QWidget *parent)
    : SideBarListWidget(parent)
    , _monitorWidget(0)
    , _monitorModel(0)
{
    _setIconSize(QSize(64, 64));
    HnvMainWindow * window = HnvMainWindow::instance();
    if (window)
        _monitorModel = window->platformMonitorModel();
    setViewModel(_monitorModel);
    PlatformMonitorModeWidget * modeWidget = qobject_cast<PlatformMonitorModeWidget*>(sideBarParent());
    if (modeWidget)
        _monitorWidget = qobject_cast<PlatformMonitorWidget*>(modeWidget->displayWidget());
    if (_monitorModel)
        connect(_monitorModel, SIGNAL(dataLoaded()), this, SLOT(_dataLoadedSlot()));
    _dataLoadedSlot();
}

MeshListWidget::~MeshListWidget()
{

}

QString MeshListWidget::title()
{
    return tr("Mesh");
}

QString MeshListWidget::sideBarId()
{
    return "Mesh";
}

void MeshListWidget::_indexClicked(const QModelIndex &index)
{
    _displayIndex(index);
}

void MeshListWidget::_indexDoubleClicked(const QModelIndex &index)
{
    _displayIndex(index);
}

void MeshListWidget::_contextMenuRequested(const QModelIndex &index, const QPoint &position)
{
    Q_UNUSED(index)
    Q_UNUSED(position)
}

void MeshListWidget::_displayIndex(const QModelIndex &index)
{
    if (index.isValid() && _monitorWidget) {
        MonitorItem * item = _monitorModel->monitorItemFromIndex(index);
        if (item) {
            _monitorWidget->setMonitorItem(item);
            item->refresh();
        }
    }
}

void MeshListWidget::_dataLoadedSlot()
{
    if (_monitorModel)
        _activateViewIndexSlot(_monitorModel->indexFromNode(_monitorModel->meshMonitorItem()));
}

} // namespace Hnv
