/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "platformmonitormodewidget.h"
#include "ressources.h"
#include "sidebar.h"
#include "meshlistwidget.h"
#include "sessionlistwidget.h"
#include "entityinfolistwidget.h"
#include "platformmonitorwidget.h"

// Qt headers
#include <QtPlugin>
#include <QAction>
#include <QIcon>

namespace Hnv {

/*******************************************************************************
* PlatformMonitorModeWidget
*/
PlatformMonitorModeWidget::PlatformMonitorModeWidget(QWidget *parent)
    : ModeWidget(parent)
{
    PlatformMonitorWidget * display = new PlatformMonitorWidget(this);
    _initDisplayArea(display);

    QList<SideBarItem *> itemList;
    itemList.append(new SideBarItem(MeshListWidget::sideBarId(), MeshListWidget::title(), MeshListWidget::staticMetaObject, 1));
    itemList.append(new SideBarItem(SessionListWidget::sideBarId(), SessionListWidget::title(), SessionListWidget::staticMetaObject, 0));
    itemList.append(new SideBarItem(EntityInfoListWidget::sideBarId(), EntityInfoListWidget::title(), EntityInfoListWidget::staticMetaObject, 0));
    _initSideBar(new SideBar(staticMetaObject.className(), MeshListWidget::sideBarId(), itemList, this));
}

bool PlatformMonitorModeWidget::hasSideBar()
{
    return true;
}

bool PlatformMonitorModeWidget::hasRightSideBar()
{
    return false;
}

/*******************************************************************************
* PlatformMonitorModeWidget - Protected
*/
void PlatformMonitorModeWidget::_initModeAction()
{
    _modeAction = new QAction(QIcon(Ressources::themedPath(Constants::ICON_PLATFORM_MONITOR)), tr("Platform monitor"), this);
}

/*******************************************************************************
* PlatformMonitorModePlugin
*/
PlatformMonitorModePlugin::PlatformMonitorModePlugin(QObject *parent)
    : ModePlugin(parent)
{
}

QString PlatformMonitorModePlugin::name() const
{
    return tr("Platform monitor");
}

QString PlatformMonitorModePlugin::description() const
{
    return tr("Platform monitor plugin");
}

void PlatformMonitorModePlugin::_initModeWidget(QWidget * parent)
{
    if (_modeWidgetSingleton)
        return;
    _modeWidgetSingleton = new PlatformMonitorModeWidget(parent);
}

} // namespace Hnv

Q_EXPORT_PLUGIN2(platformMonitorPlugin, Hnv::PlatformMonitorModePlugin)
