/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "platformmonitorwidget.h"
#include "ui_platformmonitorwidget.h"
#include "platformmonitormodel.h"
#include "generalinformationwidget.h"
#include "hynesimsettingswidget.h"
#include "systeminformationwidget.h"
#include "hardwaremonitoringwidget.h"
#include "hynesimmonitoringwidget.h"
#include "hnvwidgets.h"

// Qt headers
#include <QLabel>

namespace Hnv {

/*******************************************************************************
* MonitorWidget
*******************************************************************************/
MonitorWidget::MonitorWidget(QWidget *parent)
    : QFrame(parent)
{
    setStyleSheet("background-color:white; color:black;");
}

MonitorWidget::~MonitorWidget()
{
}

void MonitorWidget::_setLabelText(QLabel *label, QString labelText)
{
    if (labelText.isNull())
        label->setText(tr("N/A"));
    else
        label->setText(labelText);
}

void MonitorWidget::_clearLabel(QLabel *label)
{
    _setLabelText(label, QString());
}

/*******************************************************************************
* PlatformMonitorWidget
*******************************************************************************/
PlatformMonitorWidget::PlatformMonitorWidget(QWidget *parent)
    : QWidget(parent)
    , ui(new Ui::PlatformMonitorWidget)
    , _currentItem(0)
    , _maxColumns(2)
    , _currentRow(0)
    , _currentColumn(0)
    , _generalInfoWidget(0)
    , _hynesimSettingsWidget(0)
    , _systemInformationWidget(0)
    , _hardwareMonitorWidget(0)
    , _hynesimMonitorWidget(0)

{
    ui->setupUi(this);
    ui->toolBarWidget->setCloseable(false);
    setStyleSheet("Hnv--PlatformMonitorWidget QScrollArea QWidget #scrollAreaWidgetContents { background-image: url(':/themes/hynesim/icons/welcome-background.png'); background-repeat: repeat-xy; background-attachment: fixed;} RoundedFrame { background-color: white; } Hnv--RoundedFrame > QWidget { padding:8px; border:2px solid black;} Hnv--RoundedFrame #contentWidget { border-bottom-right-radius:10px; border-bottom-left-radius:10px;} Hnv--RoundedFrame #titleWidget { border-top-right-radius:10px; border-top-left-radius:10px; background-color: qlineargradient(spread:pad, x1:0, y1:0, x2:0, y2:1, stop:0 rgba(120, 120, 120, 255), stop:0.49 rgba(120, 120, 120,255), stop:0.5 rgba(90, 90, 90,255), stop:1 rgba(90, 90, 90, 255));}");

    _generalInfoWidget = new GeneralInformationWidget(this);
    _addFrame(tr("General"), _generalInfoWidget);

    _nextCell();

    _systemInformationWidget = new SystemInformationWidget(this);
    _addFrame(tr("System information"), _systemInformationWidget);

    _hynesimSettingsWidget = new HynesimSettingsWidget(this);
    _addFrame(tr("hynesim settings"), _hynesimSettingsWidget);

    _hardwareMonitorWidget = new HardwareMonitoringWidget(this);
    _addFrame(tr("Hardware monitoring"), _hardwareMonitorWidget);

    _hynesimMonitorWidget = new HynesimMonitoringWidget(this);
    _addFrame(tr("hynesim monitoring"), _hynesimMonitorWidget);

    QSpacerItem *spacer = new QSpacerItem(2, 4, QSizePolicy::Minimum, QSizePolicy::Expanding);

    if (_currentColumn != 0) {
        _currentRow++;
        _currentColumn = 0;
    }

    ui->informationLayout->addItem(spacer, _currentRow, _currentColumn);
    setMonitorItem(0);
}

PlatformMonitorWidget::~PlatformMonitorWidget()
{
    delete ui;
}

void PlatformMonitorWidget::setMonitorItem(MonitorItem *item)
{
    if (_currentItem)
        disconnect(_currentItem);

    _currentItem = item;
    if (_currentItem) {
        connect(_currentItem, SIGNAL(destroyed()), this, SLOT(_currentItemDestroyedSlot()));
        connect(_currentItem, SIGNAL(dataChanged()), this, SLOT(_currentItemDataChangedSlot()));
    }
    _displayCurrentItem();
}

void PlatformMonitorWidget::_displayCurrentItem()
{
    if (_currentItem)
        _setTitle(_currentItem->getName());
    else
        _setTitle(QString());
    _generalInfoWidget->setMonitorItem(_currentItem);
    _systemInformationWidget->setMonitorItem(_currentItem);
    _hynesimSettingsWidget->setMonitorItem(_currentItem);
    _hardwareMonitorWidget->setMonitorItem(_currentItem);
    _hynesimMonitorWidget->setMonitorItem(_currentItem);
}

void PlatformMonitorWidget::_setTitle(QString titleDetails)
{
    QString title = tr("Platform monitoring");
    if (!titleDetails.isNull())
        title += ": " + titleDetails;
    ui->toolBarWidget->setTitle(title);
}

void PlatformMonitorWidget::_addFrame(QString title, QWidget *widget)
{
    TitleToolBar * titleBar = new TitleToolBar(this, false);
    titleBar->setTitle(title);
    titleBar->setFixedHeight(36);
    ui->informationLayout->addWidget(new RoundedFrame(titleBar, widget, this), _currentRow, _currentColumn);
    _nextCell();
}

void PlatformMonitorWidget::_nextCell()
{
    _currentColumn++;
    if (_currentColumn == _maxColumns) {
        _currentRow++;
        _currentColumn = 0;
    }
}

void PlatformMonitorWidget::_currentItemDestroyedSlot()
{
    _currentItem = 0;
    _displayCurrentItem();
}

void PlatformMonitorWidget::_currentItemDataChangedSlot()
{
    _displayCurrentItem();
}

} // namespace Hnv
