/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef PLATFORMMONITORWIDGET_H
#define PLATFORMMONITORWIDGET_H

// Qt headers
#include <QWidget>
#include <QFrame>

namespace Ui {
class PlatformMonitorWidget;
} // namespace Ui

class QLabel;

namespace Diamesh {
struct VersionInfo;
} // namespace Diamesh

namespace Hnv {

class MonitorItem;
class GeneralInformationWidget;
class HynesimSettingsWidget;
class SystemInformationWidget;
class HardwareMonitoringWidget;
class HynesimMonitoringWidget;

/*******************************************************************************
* MonitorWidget
*******************************************************************************/
class MonitorWidget : public QFrame
{
    Q_OBJECT
public:
    explicit MonitorWidget(QWidget *parent = 0);
    ~MonitorWidget();

    virtual void setMonitorItem(MonitorItem *item) = 0;
protected:
    void _setLabelText(QLabel *label, QString labelText);
    void _clearLabel(QLabel *label);
};

/*******************************************************************************
* PlatformMonitorWidget
*******************************************************************************/
class PlatformMonitorWidget : public QWidget
{
    Q_OBJECT

public:
    explicit PlatformMonitorWidget(QWidget *parent = 0);
    ~PlatformMonitorWidget();

    void setMonitorItem(MonitorItem *item);

protected:
    void _displayCurrentItem();
    void _setTitle(QString titleDetails = QString());
    void _addFrame(QString title, QWidget *widget);
    void _nextCell();

protected slots:
    void _currentItemDestroyedSlot();
    void _currentItemDataChangedSlot();

private:
    Ui::PlatformMonitorWidget *ui;
    MonitorItem * _currentItem;
    int _maxColumns;
    int _currentRow;
    int _currentColumn;
    GeneralInformationWidget * _generalInfoWidget;
    HynesimSettingsWidget * _hynesimSettingsWidget;
    SystemInformationWidget * _systemInformationWidget;
    HardwareMonitoringWidget * _hardwareMonitorWidget;
    HynesimMonitoringWidget * _hynesimMonitorWidget;
};

} // namespace Hnv

#endif // PLATFORMMONITORWIDGET_H
