#ifndef RDPWORKER_H
#define RDPWORKER_H

// Project headers
#include "worker.h"
#include "remotemanager.h"

// Qt headers
#include <QImage>

// FreeRDP headers
#include <freerdp/freerdp.h>

class QThread;

namespace RemoteAccess {

class RdpWorker : public Worker
{
    Q_OBJECT
public:
    RdpWorker(QObject *parent = 0);
    ~RdpWorker();
    virtual void openConnection();
    virtual void closeConnection();
    virtual Display* createDisplay();
    void go();
signals:
    void imageChanged(QImage image, int x, int y, int width, int height);
    void cursorChanged(QImage image, int x, int y);
    void clearImaged();
    void sizeChanged(QSize size);
public slots:
    void mouseEventReceive(RemoteAccess::MouseInfo info);
    void keyboardEventReceive(RemoteAccess::KeyInfo info);
    void run();
private:
    void _setResolution(const QString &resolution, unsigned int &width, unsigned int &height);

    static int contextNew(freerdp *instance, rdpContext *context);
    static void contextFree(freerdp *instance, rdpContext *context);

    static int preConnect(rdp_freerdp *instance);
    static int postConnect(rdp_freerdp *instance);

    static void beginPaint(rdpContext *context);
    static void endPaint(rdpContext *context);

    static void pointerSet(rdpContext *context, rdpPointer *pointer);
    static void pointerNew(rdpContext *context, rdpPointer *pointer);
    static void pointerFree(rdpContext *context, rdpPointer *pointer);

    bool _close;
    rdp_freerdp *_instance;

    QThread *_thread;
};

struct _rdpHnsContext
{
    rdpContext rdpCtx;
    RdpWorker *worker;
};
typedef struct _rdpHnsContext RdpHnsContext;

} //namespace RemoteAccess
#endif // RDPWORKER_H
