/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of Hynesim.
**
** Hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** Hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef SPICEWORKER_H
#define SPICEWORKER_H

// Spice headers
#include <spice-client-glib-2.0/spice-session.h> //include before Qt because signals collides with GLIB symbols
#include <spice-client-glib-2.0/channel-main.h>

// Project headers
#include "worker.h"
#include "remotemanager.h"

// Qt headers
#include <QImage>

namespace RemoteAccess {

class UsbManager;
class SpiceUsbManager;

struct KeyHash : public QHash<quint32, quint16>
{
    KeyHash();
};

class SpiceWorker : public Worker
{
    Q_OBJECT

public:
    SpiceWorker(QObject *parent = 0);
    ~SpiceWorker();
    virtual void openConnection();
    virtual void closeConnection();
    virtual Display *createDisplay();
    virtual UsbManager *usbManager();
    virtual bool canRedirectUsb();

    int getKeyboardLockState();

signals:
    void imageChanged(QImage image, int x, int y, int w, int h);
    void sizeChanged(QSize size);
    void cursorChanged(QImage, int, int);
    void clipboardDataEvent(QString text);
    void clipboardSelectionEvent(QString text);
    void clipboardDataCleared();
    void clipboardSelectionCleared();

public slots:
    void keyboardEventReceive(RemoteAccess::KeyInfo keyInfo);
    void mouseEventReceive(RemoteAccess::MouseInfo mouseInfo);
    void clipboardDataEventReceive(QString text);
    void clipboardSelectionEventReceive(QString text);
    void releaseAllKeysSlot();

private:
    /*callbacks*/
    static void hns_channel_new(SpiceSession *session, SpiceChannel *channel, void *data);
    static void hns_channel_destroy(SpiceSession *session, SpiceChannel *channel, void *data);

    static void hns_main_channel_event(SpiceChannel *channel, SpiceChannelEvent event, void *data);
    static void hns_main_agent_update(SpiceChannel *channel, void *data);
    static void hns_main_mouse_update(SpiceChannel *channel, void *data);

    static void hns_display_primary_create(SpiceDisplayChannel *display, int format, int width, int height, int stride, int shmid, void *imgdata, void *data);
    static void hns_display_primary_destroy(SpiceDisplayChannel *display, void *data);
    static void hns_display_invalidate(SpiceDisplayChannel *display, int x, int y, int w, int h, void *data);

    static void hns_cursor_set(SpiceCursorChannel *channel, int width, int height, int hotX, int hotY, void *rgba, void *data);
    static void hns_cursor_reset(SpiceCursorChannel *channel, void *data);

    static bool hns_main_clipboard_selection_grab(SpiceMainChannel *mainChannel, uint selection, uint *types, uint ntypes, void *data);
    static void hns_main_clipboard_selection(SpiceMainChannel *mainChannel, uint selection, uint type, uchar *data, uint size, void *user_data);
    static void hns_main_clipboard_selection_release(SpiceMainChannel *mainChannel, uint selection, void *data);
    static bool hns_main_clipboard_selection_request(SpiceMainChannel *mainChannel, uint selection, uint types, void *data);

    static void hns_inputs_mofifiers(SpiceInputsChannel *channel, void *data);

    void disconnect_spice();
    void _handleClipboard(QString text, uint selectionType);

    /*util functions*/
    void setImageData(void *imgData);
    void *imageData();
    void setResolution(QSize size);
    QSize resolution();
    void setInputChannel(SpiceInputsChannel *inputChannel);
    void setMainChannel(SpiceMainChannel *mainChannel);
    void setAgentConnected(bool connected);
    bool agentConnected();
    QString clipboardDataString();
    QString clipboardSelectionString();
    void handle_mouseMove(int xPos, int yPos);
    void handle_mouseWheel(bool wheelUp);

    SpiceSession *_session;
    SpiceInputsChannel *_inputChannel;
    SpiceMainChannel *_mainChannel;
    QList<SpiceChannel *> _openedChannels;
    SpiceUsbManager *_spiceUsbManager;
    QSize _resolutionSize;
    uchar *_imgdata;
    bool _agentConnected;
    QString _clipboardDataString;
    QString _clipboardSelectionString;
    int _buttonMask;
    int _mouseMode;
    int _mouseLastX;
    int _mouseLastY;
    bool _canRedirectUsb;

    unsigned long _connectorHandler;

    QList<quint32> _pressedKeys;
};

} // namespace RemoteAccess

#endif //SPICEWORKER_H
