/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "telnetworker.h"
#include "display.h"

// Qt headers
#include <QX11EmbedContainer>
#include <QProcess>

namespace RemoteAccess {

TelnetWorker::TelnetWorker(QObject *parent)
    : Worker(parent)
{
    _widget = new QWidget(/*this*/);
    _container = new QX11EmbedContainer(_widget);
    connect(_container, SIGNAL(clientClosed()), this, SIGNAL(connectionEnded()));
}

TelnetWorker::~TelnetWorker()
{
}

void TelnetWorker::openConnection()
{
    _process = new QProcess(_container);
    _process->setProcessChannelMode(QProcess::ForwardedChannels);
    QStringList arguments;

    arguments << "-into" << QString::number(_container->winId());
    arguments << "-e" << "telnet";
    arguments << QString::fromStdString(_info.host) << QString::number(_info.port);

    _process->start("xterm", arguments);
    _container->resize(QSize(800, 600));
    _widget->resize(800, 600);
}

void TelnetWorker::closeConnection()
{
    _process->terminate();
    _process->waitForFinished(1000);
    _container->discardClient();
}

Display* TelnetWorker::createDisplay()
{
    Display *display = new WidgetDisplay(_widget);
    return display;
}

} // namespace RemoteAccess
