/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef VNCWORKER_H
#define VNCWORKER_H

// Project headers
#include "worker.h"
#include "remotemanager.h"

// Qt headers
#include <QImage>
#include <QMutex>
#include <QMap>

extern "C"
{
struct _GValueArray;
typedef struct _GValueArray GValueArray;

struct _VncConnection;
typedef struct _VncConnection VncConnection;

struct _VncCursor;
typedef struct _VncCursor VncCursor;

struct _VncBaseFramebuffer;
typedef struct _VncBaseFramebuffer VncBaseFramebuffer;
}

class QThread;

namespace RemoteAccess {

class VncWorker : public Worker
{
    Q_OBJECT
public:
    VncWorker(QObject *parent = 0);
    ~VncWorker();
    virtual void openConnection();
    virtual void closeConnection();
    virtual Display* createDisplay();

signals:
    void imageChanged(QImage image, int x, int y, int width, int height);
    void sizeChanged(QSize size);
    void cursorChanged(QImage image, int x, int y);
    void connectionEnded();
    void connectionError(QString);
    void clearImaged();

public slots:
    void mouseEventReceive(RemoteAccess::MouseInfo info);
    void keyboardEventReceive(RemoteAccess::KeyInfo info);

private:
    /*callbacks*/
    static void hns_cursor_changed(VncConnection *connection, VncCursor *cursor, void *data);
    static void hns_framebuffer_update(VncConnection *connection, int x, int y, int w, int h, void *data);
    static void hns_desktop_resize(VncConnection *connection, int width, int height, void *data);
    static void hns_auth_choose_type(VncConnection *connection, GValueArray *types, void *data);
    static void hns_initialized(VncConnection *connection, void *data);
    static void hns_disconnected(VncConnection *connection, void *data);

    void _initFrameBuffer();

    VncConnection *_connection;
    VncBaseFramebuffer *_baseFrameBuffer;

    QImage _image;
    int _width;
    int _height;
    int _buttonMask;
};

} // namespace RemoteAccess
#endif // VNCWORKER_H
