/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef CAPTUREJOBMODEL_H
#define CAPTUREJOBMODEL_H

// Generated hynesim headers
#include "common/types.h"
#include "common/entity.h"

// Qt headers
#include <QAbstractTableModel>

namespace HnsClient {
class GenericEntity;
} // namespace HnsClient

namespace Hnv {

/*******************************************************************************
* PcapInfoItem
*******************************************************************************/
class PcapInfoItem : public QObject
{
public:
    enum CaptureJobRole {
        StatusRole = Qt::UserRole + 10,
        JobIDRole,
        FilePathRole
    };

    enum Columns {
        COLUMN_INTERFACE,
        COLUMN_DATEBEGIN,
        COLUMN_DATENDED,
        COLUMN_TIMELIMIT,
        COLUMN_PACKETLIMIT,
        COLUMN_FILTERS,
        COLUMN_PACKETCOUNT,
        COLUMN_SIZE
    };

    explicit PcapInfoItem(HnsClient::GenericEntity * entity, Hns::PcapInfo pcapInfo, int jobID = -1, QObject *parent = 0);
    ~PcapInfoItem();
    void setPcapInfo(Hns::PcapInfo pcapInfo);
    Hns::PcapInfo getPcapInfo() const;
    bool isRunning() const;
    void setJobID(int jobID);
    int getJobID() const;
    QString getPath() const;
    QString getInterfaceName() const;
    QString getCreationDateTime() const;
    QString getEndDateTime() const;
    int getTimeLimit() const;
    int getPacketLimit() const;
    QString getFilters() const;
    int getPacketCount() const;
    QString getReadableSize() const;
    virtual QVariant data(int column = 0, int role = Qt::DisplayRole) const;
private:
    HnsClient::GenericEntity * _entity;
    Hns::PcapInfo _pcapInfo;
    int _jobID;
};

/*******************************************************************************
* CaptureJobModelLoader
*******************************************************************************/
class CaptureJobModelLoader : public QObject
{
    Q_OBJECT
public:
    explicit CaptureJobModelLoader(HnsClient::GenericEntity *entity, QObject *parent = 0);
    void start(int jobID);

protected slots:
    void _getPcapInfoByJobIDSlot(::Hns::PcapInfo pcapInfo);
signals:
    void ended(int jobID, ::Hns::PcapInfo pcapInfo);
private:
    HnsClient::GenericEntity *_entity;
    int _jobID;
};

/*******************************************************************************
* CaptureJobModel
*******************************************************************************/

class CaptureJobModel : public QAbstractTableModel
{
    Q_OBJECT
public:

    CaptureJobModel(HnsClient::GenericEntity *entity, QObject *parent = 0);
    ~CaptureJobModel();

    int rowCount(const QModelIndex &parent= QModelIndex()) const;
    int columnCount(const QModelIndex &parent = QModelIndex()) const;
    QVariant data(const QModelIndex &index, int role) const;
    QVariant headerData(int section, Qt::Orientation orientation, int role) const;
    QModelIndex index(int row, int column, const QModelIndex &parent = QModelIndex()) const;

protected:
    qlonglong _jobIDByPath(QString path);
    void _createPcapInfoItem(::Hns::PcapInfo pcapInfo);
    void _deletePcapInfoItem(PcapInfoItem * item);
    void _updatePcapInfoItem(int jobID);

protected slots:
    void _captureJobStarted(Ice::Int interfaceID, Ice::Int jobID);
    void _captureJobEnded(Ice::Int interfaceID, Ice::Int jobID);
    void _removeCaptureFile(QString fileName);
    void _getCaptureJobMapSlot(::Hns::mapLongString captureJobMap);
    void _getPcapInfoListSlot(::Hns::PcapInfoList pcapInfoList);
    void _captureJobLoaderEnded(int jobID, ::Hns::PcapInfo pcapInfo);
    void _captureJobLoaderUpdateEnded(int jobID, ::Hns::PcapInfo pcapInfo);

private :
    HnsClient::GenericEntity *_entity;
    QHash<qlonglong, QString> _runningJobsIDPath;
    QList<PcapInfoItem *> _pcapInfoItemsList;
    QHash<QString, PcapInfoItem * > _pathToItemHash;
    QHash<int, PcapInfoItem *> _jobsToItemHash;
    QList<int> _unknowJobEnded;

};
#endif // CAPTUREJOBMODEL_H

} // namespace Hnv
