/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef HNV_HNV_CATALOGITEM_H
#define HNV_HNV_CATALOGITEM_H

// hynesim hnsclient headers
#include <libhnsclient/common.h>

// Qt headers
#include <QObject>

class QPixmap;
class QVariant;

namespace HnsClient {
class CatalogItem;
} // namespace HnsClient

namespace Hnv {

/*******************************************************************************
* EntryItem
*******************************************************************************/
class EntryItem : public QObject
{
    Q_OBJECT
public:
    enum EntryItemRole {
        IDRole = Qt::UserRole + 10,
        IsReadableRole,
        IsWriteableRole,
        IsExecutableRole,
        IsRWXRole,
        IsLockedRole,
        IsSavedRole,
        IsValidRole,
        SyncingStatusRole,
        IsSyncingRole,
        IsModelRole,
        HasLinkedClonesRole,
        IsLinkedCloneRole
    };

    // TODO: SIZE_COLUMN should be specific to each entry type
    enum listColumn{
        COLUMN_SHORTNAME,
        COLUMN_LONGNAME,
        COLUMN_DESCRIPTION,
        COLUMN_TAGLIST,
        COLUMN_MODEL,
        SIZE_COLUMN
    };

    explicit EntryItem(HnsClient::CatalogItem *entry, QObject *parent = 0);
    ~EntryItem();

    virtual int columnCount() const;
    virtual Qt::ItemFlags flags(int column) const;
    virtual QVariant data(int column = 0, int role = Qt::DisplayRole) const;
    virtual bool setData(int column, const QVariant &value, int role);

    virtual HnsClient::CatalogItem *catalogItem() const;
    virtual qlonglong getID() const;
    virtual QString getShortName() const;
    virtual QString getLongName() const;
    virtual QString getDescription() const;
    virtual QString getTagList() const;
    virtual bool isReadable() const;
    virtual bool isWriteable() const;
    virtual bool isExecutable() const;
    virtual bool isRWX() const;
    virtual bool isLocked() const;
    virtual bool isSaved() const;
    virtual bool isValid() const;
    virtual bool isModel() const;
    virtual bool hasLinkedClones() const;
    virtual bool isLinkedClone() const;
    virtual HnsClient::SynchronizationStatus getSyncStatus() const;
    virtual bool isSyncing() const;
    virtual QString getToolTip() const;
    virtual QPixmap getPixmap() const;

protected:
    HnsClient::CatalogItem *_entry;

};

} // namespace Hnv

#endif // HNV_HNV_CATALOGITEM_H

