/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef HYNEVIEWSESSION_H
#define HYNEVIEWSESSION_H

// Generated hynesim headers
#include "common/usermanager.h"
#include "common/dynamipsrouterdb.h"

// diamesh headers
#include <diamesh/sessionhelper.h>

// Qt headers
#include <QObject>
#include <QSharedPointer>

class QStringList;

namespace Diamesh {
struct VersionInfo;
} // namespace Diamesh

namespace HnsClient {
class CatalogImporter;
class EntityEntryCatalog;
class EntityInfoList;
class JobManager;
class MacAddressManager;
class PlatformMonitor;
class Session;
class TopologyEntryCatalog;
class TopologyManager;
class UserManager;
class ACLManager;
class ResourceManager;
} // namespace HnsClient

namespace Hnv {

class TopologyAdministrator;

/*******************************************************************************
* HyneviewSessionParameters
*******************************************************************************/
struct HyneviewSessionParameters {
    QSharedPointer<Diamesh::SessionHelper> sessionHelper;
    Diamesh::SessionInfo sessionInformation;
    HnsClient::Session *sessionLibClient;
    TopologyAdministrator *topologyAdministrator;
};

/*******************************************************************************
* HyneviewSessionLoader
*******************************************************************************/
class HyneviewSessionLoader : public QObject
{
    Q_OBJECT
public:
    explicit HyneviewSessionLoader(QObject * parent = 0);
    ~HyneviewSessionLoader();
    void load(QSharedPointer<Diamesh::SessionHelper> sessionHelper, bool deleteOnCompleted = true);

    bool getLoadingResult();
    bool isCompleted();
    HyneviewSessionParameters getHyneviewSessionParameters();

signals:
    void completed(bool result, HyneviewSessionParameters parameters);

private:
    void _terminate();
    bool _result;
    bool _completed;
    bool _deleteOnCompleted;
    HyneviewSessionParameters _parameters;
};

/*******************************************************************************
* HyneviewSession
*******************************************************************************/
class HyneviewSession : public QObject
{
    Q_OBJECT
public:
    static HyneviewSession *instance();
    static HyneviewSession *create(QSharedPointer<Diamesh::SessionHelper> sessionHelper, QObject *parent = 0);

    bool isConnected();
    void disconnectSession();
    QString userName();
    Hns::UserPtr user();
    Diamesh::SessionHelper *sessionHelper();
    Diamesh::SessionInfo sessionInformation();
    QString masterName();
    Diamesh::VersionInfo masterVersionInfo();
    bool isAdvancedMode();

    TopologyAdministrator *topologyAdministrator();

    HnsClient::CatalogImporter *catalogImporter();
    HnsClient::EntityEntryCatalog *entityEntryCatalog();
    HnsClient::EntityInfoList *entityInfoList();
    HnsClient::JobManager *jobManager();
    HnsClient::MacAddressManager *macAddressManager();
    HnsClient::PlatformMonitor *platformMonitor();
    HnsClient::TopologyEntryCatalog *topologyEntryCatalog();
    HnsClient::TopologyManager *topologyManager();
    HnsClient::UserManager *userManager();
    HnsClient::ACLManager *aclManager();
    HnsClient::ResourceManager *resourceManager();

    QMap<QString, QString> entityCatalogStorageMap();
    QMap<QString, QString> topologyCatalogStorageMap();
    QStringList *getNetworkCardsVendor();
    void getDynamipsRouterData(QMap<QString, Hns::DataDynamipsRouter > & dataDynamipsMap, QMap<QString, Hns::DynamipsRouterModule > & dynamipsRouterModuleMap);

signals:
    void disconnected();

protected:
    HyneviewSessionParameters _parameters;

protected slots:
    void disconnectedSlot();
private:
    explicit HyneviewSession(HyneviewSessionParameters parameters, QObject * parent = 0);
    ~HyneviewSession();
    static HyneviewSession *_singleton;
};

} // namespace Hnv
#endif // HYNEVIEWSESSION_H
