/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef PLATFORMMONITORMODEL_H
#define PLATFORMMONITORMODEL_H

// Project headers
#include "treemodel.h"
#include "treemodelitem.h"

// Generated hynesim headers
#include "common/proxies_entityinfo.h"
#include "common/wireentry.h"

// diamesh headers
#include <diamesh/info.h>

// Qt headers
#include <QObject>

namespace HnsClient {
class PlatformMonitor;
class MasterInformation;
class NodeInformation;
class EntityInfo;
class EntityInfoList;
class ServiceDetails;
typedef QString (ServiceDetails::*ServiceQuery)();
typedef QString (NodeInformation::*NodeInformationQuery)();
} // namespace HnsClient

namespace Hnv {

/*******************************************************************************
* MonitorItem
*******************************************************************************/
class MonitorItem : public TreeModelItem
{
    Q_OBJECT
public:
    explicit MonitorItem(QObject * parent = 0);
    ~MonitorItem();

    virtual void refresh() = 0;

    virtual QVariant data(int column = 0, int role = Qt::DisplayRole) const;
    virtual QString getName() const = 0;
    virtual QString getVersion() = 0;
    virtual Diamesh::VersionInfo getVersionInfo() = 0;
    virtual Diamesh::SystemInfo getSystemInfo() = 0;
    virtual Diamesh::ServiceInfo getServiceInfo() = 0;

    virtual QString getServiceCapabilities() = 0;
    virtual QString getServiceHost() = 0;

    virtual QString getSystemKernel() = 0;
    virtual QString getSystemProcessor() = 0;
    virtual QString getSystemMemory() = 0;
    virtual QString getSystemSwap() = 0;

    virtual QString getHnsMaxThread() = 0;
    virtual QString getHnsMaxProcessors() = 0;
    virtual QString getHnsMaxMemory() = 0;

    virtual QString getSystemUptime() = 0;
    virtual QString getSystemCpuLoad() = 0;
    virtual QString getSystemMemoryLoad() = 0;
    virtual QString getSystemSwapLoad() = 0;
    virtual QString getDisksStatus() = 0;

    virtual QString getServiceUptime() = 0;
    virtual QString getHnsCpuUsage() = 0;
    virtual QString getHnsMemoryUsage() = 0;
    virtual QString getHnsDefinedEntities() = 0;
    virtual QString getHnsNetcards() = 0;

signals:
    void dataChanged();

protected:
    virtual QPixmap _getDecorationRole() const = 0;
    virtual QString _getTooltipRole() const = 0;

protected slots:
    void _infoChangedSlot();
};

/*******************************************************************************
* MeshMonitorItem
*******************************************************************************/
class MeshMonitorItem : public MonitorItem
{
    Q_OBJECT
public:
    explicit MeshMonitorItem(HnsClient::PlatformMonitor * monitor, QObject * parent = 0);
    ~MeshMonitorItem();

    virtual void refresh();

    virtual QString getName() const;
    virtual QString getVersion();
    virtual Diamesh::VersionInfo getVersionInfo();
    virtual Diamesh::SystemInfo getSystemInfo();
    virtual Diamesh::ServiceInfo getServiceInfo();

    virtual QString getServiceCapabilities();
    virtual QString getServiceHost();

    virtual QString getSystemKernel();
    virtual QString getSystemProcessor();
    virtual QString getSystemMemory();
    virtual QString getSystemSwap();

    virtual QString getHnsMaxThread();
    virtual QString getHnsMaxProcessors();
    virtual QString getHnsMaxMemory();

    virtual QString getSystemUptime();
    virtual QString getSystemCpuLoad();
    virtual QString getSystemMemoryLoad();
    virtual QString getSystemSwapLoad();
    virtual QString getDisksStatus();

    virtual QString getServiceUptime();
    virtual QString getHnsCpuUsage();
    virtual QString getHnsMemoryUsage();
    virtual QString getHnsDefinedEntities();
    virtual QString getHnsNetcards();

protected:
    QString _checkLineValue(QString host, QString name, QString value, QHash<QString, QString> &redudancyHash);
    QHash<QString, QString> _buildDataHash(HnsClient::ServiceQuery query, bool ignoreRedudancy = false, bool skipMaster = false);
    QHash<QString, QString> _buildDataHash(HnsClient::NodeInformationQuery query, bool ignoreRedudancy = false);

    QString _buildInfoTableLine(QString label, QString value, QString lineClass = "");
    QString _buildInfoTable(QHash<QString, QString> data, bool skipMaster = false);
    QString _buildInfoTable(HnsClient::ServiceQuery query, bool ignoreRedudancy = false, bool skipMaster = false);
    QString _buildInfoTable(HnsClient::NodeInformationQuery query, bool ignoreRedudancy = false, bool skipMaster = false);

    virtual QPixmap _getDecorationRole() const;
    virtual QString _getTooltipRole() const;
    HnsClient::PlatformMonitor * _monitor;
};

/*******************************************************************************
* MasterMonitorItem
*******************************************************************************/
class MasterMonitorItem : public MonitorItem
{
    Q_OBJECT
public:
    explicit MasterMonitorItem(HnsClient::MasterInformation * masterInformation, QObject * parent = 0);
    ~MasterMonitorItem();

    virtual void refresh();

    virtual QString getName() const;
    virtual QString getVersion();
    virtual Diamesh::VersionInfo getVersionInfo();
    virtual Diamesh::SystemInfo getSystemInfo();
    virtual Diamesh::ServiceInfo getServiceInfo();

    virtual QString getServiceCapabilities();
    virtual QString getServiceHost();

    virtual QString getSystemKernel();
    virtual QString getSystemProcessor();
    virtual QString getSystemMemory();
    virtual QString getSystemSwap();

    virtual QString getHnsMaxThread();
    virtual QString getHnsMaxProcessors();
    virtual QString getHnsMaxMemory();

    virtual QString getSystemUptime();
    virtual QString getSystemCpuLoad();
    virtual QString getSystemMemoryLoad();
    virtual QString getSystemSwapLoad();
    virtual QString getDisksStatus();

    virtual QString getServiceUptime();
    virtual QString getHnsCpuUsage();
    virtual QString getHnsMemoryUsage();
    virtual QString getHnsDefinedEntities();
    virtual QString getHnsNetcards();

protected:
    virtual QPixmap _getDecorationRole() const;
    virtual QString _getTooltipRole() const;

protected:
    HnsClient::MasterInformation * _masterInformation;
};

/*******************************************************************************
* NodeMonitorItem
*******************************************************************************/
class NodeMonitorItem : public MonitorItem
{
    Q_OBJECT
public:
    explicit NodeMonitorItem(HnsClient::NodeInformation * nodeInformation, QObject * parent = 0);
    ~NodeMonitorItem();

    virtual void refresh();

    virtual QString getName() const;
    virtual QString getVersion();
    virtual Diamesh::VersionInfo getVersionInfo();
    virtual Diamesh::SystemInfo getSystemInfo();
    virtual Diamesh::ServiceInfo getServiceInfo();

    virtual QString getServiceCapabilities();
    virtual QString getServiceHost();

    virtual QString getSystemKernel();
    virtual QString getSystemProcessor();
    virtual QString getSystemMemory();
    virtual QString getSystemSwap();

    virtual QString getHnsMaxThread();
    virtual QString getHnsMaxProcessors();
    virtual QString getHnsMaxMemory();

    virtual QString getSystemUptime();
    virtual QString getSystemCpuLoad();
    virtual QString getSystemMemoryLoad();
    virtual QString getSystemSwapLoad();
    virtual QString getDisksStatus();

    virtual QString getServiceUptime();
    virtual QString getHnsCpuUsage();
    virtual QString getHnsMemoryUsage();
    virtual QString getHnsDefinedEntities();
    virtual QString getHnsNetcards();

protected:
    virtual QPixmap _getDecorationRole() const;
    virtual QString _getTooltipRole() const;

protected:
    HnsClient::NodeInformation * _nodeInformation;
};

/*******************************************************************************
* PlatformMonitorModel
*******************************************************************************/
class PlatformMonitorModel : public TreeModel
{
    Q_OBJECT
public:
    explicit PlatformMonitorModel(QObject * parent = 0);
    ~PlatformMonitorModel();

    void setMonitor(HnsClient::PlatformMonitor * monitor);
    MonitorItem * monitorItemFromIndex(QModelIndex index);

    MeshMonitorItem * meshMonitorItem();
    MasterMonitorItem * masterMonitorItem();
    NodeMonitorItem * nodeMonitorItem(QString nodeName);

signals:
    void dataLoaded();

protected slots:
    void _itemDataChanged();

    void _nodeAddedSlot(QString nodeName);
    void _nodeRemovedSlot(QString nodeName);

protected:
    void _resetModel();
    void _populateModel();
    void _addNodeMonitorItem(HnsClient::NodeInformation * nodeInformation);

    HnsClient::PlatformMonitor * _monitor;
    MeshMonitorItem * _meshItem;
    MasterMonitorItem * _masterItem;
    QHash<QString, NodeMonitorItem *> _nodeItemHash;

};

/*******************************************************************************
* EntityInfoUpdater
*******************************************************************************/
class EntityInfoUpdater : public QObject
{
    Q_OBJECT
public:
    explicit EntityInfoUpdater(qlonglong entityId, Hns::EntityInfoQPrx entityInfo, QObject *parent = 0);
    ~EntityInfoUpdater();
    const QString& getShortName() const;
    const QString& getLongName() const;
    const QString& getType() const;
    const QString& getSubType() const;
    Hns::Status getStatus() const;
    void init();

signals:
    void entityUpdate();
    void entityStateChange(int);
    void completed(qlonglong);

protected slots:
    void _statusChangedSlot(Ice::Int, Ice::Long);
    void _entryChangedSlot(::Hns::EntityInfoData);
    void _getStatusSlot(::Hns::Status);
    void _getDataSlot(::Hns::EntityInfoData);

protected:
    Hns::EntityInfoQPrx _entityInfo;
    qlonglong _entityId;
    QString _shortName;
    QString _longName;
    QString _type;
    QString _subType;
    Hns::Status _status;
    long _idxStatus;
};

class EntityInfoModel;
/*******************************************************************************
* EntityInfoItem
*******************************************************************************/
class EntityInfoItem : public TreeModelItem
{
    Q_OBJECT
public:
    explicit EntityInfoItem(EntityInfoModel * entityInfoModel, /*QWeakPointer<EntityInfoUpdater>*/HnsClient::EntityInfo*, long entityId ,QObject *parent = 0);
    ~EntityInfoItem();
    virtual QVariant data(int column, int role = Qt::DisplayRole) const;
signals:
    void infoUpdate();
private:
    HnsClient::EntityInfo *_entityInfo;
    long _entityId;
};


/*******************************************************************************
* TypeInfoItem
*******************************************************************************/
class TypeInfoItem : public TreeModelItem
{
    Q_OBJECT
public:
    explicit TypeInfoItem(const QString &nameType, EntityInfoModel *entityInfoModel, QObject *parent = 0);
    ~TypeInfoItem();
    virtual QVariant data(int column, int role = Qt::DisplayRole) const;
    void addEntityInfo(long entityId, HnsClient::EntityInfo *, bool subType);
    bool removeEntityInfo(long entityId);
    void cleanup();

protected:
    QString _nameType;
    QMap<long , EntityInfoItem *> _entityInfoItemMap;
    QMap<QString, TypeInfoItem *> _subTypeInfoItemMap;
    EntityInfoModel * _entityInfoModel;
    QHash<long,QString> _entityBySubType;
};

/*******************************************************************************
* NodeInfoModel
*******************************************************************************/
class NodeInfoItem : public TreeModelItem
{
    Q_OBJECT
public:
    explicit NodeInfoItem(const QString & name, HnsClient::EntityInfoList * entityInfoList, HnsClient::PlatformMonitor * monitor,
                          EntityInfoModel * entityInfoModel, QObject *parent = 0);
    ~NodeInfoItem();
    virtual QVariant data(int column, int role = Qt::DisplayRole) const;
    void cleanup();
    void populate();

signals:
    void addEntity(QString,long);
    void removeEntity(long);

public slots:
    void removeEntitySlot(long);

protected slots:
    void _addEntitySlot(long);

protected:
    QString _nameNode;
    QMap<QString , TypeInfoItem *> _typeInfoItemMap;
    HnsClient::EntityInfoList * _entityInfoList;
    HnsClient::PlatformMonitor * _monitor;
    EntityInfoModel * _entityInfoModel;
    QHash<long, QString> _entityByType;
};


/*******************************************************************************
* EntityInfoModel
*******************************************************************************/
class EntityInfoModel : public TreeModel
{
    Q_OBJECT
public:
    explicit EntityInfoModel(QObject * parent = 0);
    ~EntityInfoModel();

    void setMonitorEntityInfoList(HnsClient::PlatformMonitor *monitor, HnsClient::EntityInfoList *entityInfoList);

signals:
    void dataLoaded();

protected slots:
    void _nodeAddedSlot(QString);
    void _nodeRemovedSlot(QString);
    void _entityRemoveSlot(qlonglong);
    void _entityAddSlot(QString, long);
    void _entityRemoveByNodeSlot(long);

public slots:
    void itemDataChanged();

protected:
    void _resetModel();
    void _populateModel();

    HnsClient::PlatformMonitor * _monitor;
    HnsClient::EntityInfoList * _entityInfoList;
    QMap< QString, NodeInfoItem * > _nodeInfoItemMap;
    QHash<long, QString> _entityByNode;
};

} // namespace Hnv
#endif // PLATFORMMONITORMODEL_H
