/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "topologyadministrator.h"

// hynesim hnsclient headers
#include <libhnsclient/entry.h>
#include <libhnsclient/topology.h>
#include <libhnsclient/topologyentrycatalog.h>
#include <libhnsclient/topologymanager.h>

namespace Hnv {

/*******************************************************************************
* TopologyAdministrator
*******************************************************************************/
TopologyAdministrator::TopologyAdministrator(QObject *parent)
    : QObject(parent)
    , _catalog(0)
    , _manager(0)
{
}

TopologyAdministrator::~TopologyAdministrator()
{
}

HnsClient::TopologyManager *TopologyAdministrator::manager()
{
    return _manager;
}

HnsClient::TopologyEntryCatalog *TopologyAdministrator::catalog()
{
    return _catalog;
}

void TopologyAdministrator::init(HnsClient::TopologyEntryCatalog *catalog, HnsClient::TopologyManager *manager)
{
    _catalog = catalog;
    _manager = manager;
    connect(_catalog, SIGNAL(topologyAdded(HnsClient::TopologyEntry*)), _manager, SLOT(_topologyEntryAddedSlot(HnsClient::TopologyEntry*)));
    emit reseted();
}

void TopologyAdministrator::reset()
{
    _catalog = 0;
    _manager = 0;
    emit reseted();
}

QList<HnsClient::TopologyEntry *> TopologyAdministrator::getCatalogEntryList()
{
    QList<HnsClient::TopologyEntry *> result;
    if (_catalog)
        result = _catalog->getTopologyEntryHash().values();
    return result;
}

HnsClient::Topology *TopologyAdministrator::topologyByUUID(QString uuid)
{
    HnsClient::Topology *result = 0;
    if (_manager)
        result = _manager->topologyByUUID(uuid);
    return result;
}

HnsClient::Topology *TopologyAdministrator::topologyByEntryId(qlonglong entryId)
{
    HnsClient::Topology *result = 0;
    if (_manager)
        result = _manager->topologyByEntryId(entryId);
    return result;
}

HnsClient::TopologyEntry *TopologyAdministrator::duplicateEntryByUUID(QString uuid, QObject *parent)
{
    if (!_manager || !_catalog)
        return 0;
    HnsClient::TopologyEntry *sourceEntry = 0;
    HnsClient::Topology *topology = topologyByUUID(uuid);
    if (topology)
        sourceEntry = topology->topologyEntry();
    else
        sourceEntry = _catalog->topologyEntryByUUID(uuid);

    if (sourceEntry)
        return qobject_cast<HnsClient::TopologyEntry *>(sourceEntry->duplicateEntry(parent));
    return 0;
}

void TopologyAdministrator::updateEntry(HnsClient::TopologyEntry *updatedEntry)
{
    if (!_manager || !_catalog)
        return;
    HnsClient::Topology *topology = topologyByUUID(updatedEntry->getUUID());
    if (topology)
        topology->updateEntry(updatedEntry);
    else
        _catalog->updateTopologyEntry(updatedEntry);
}

void TopologyAdministrator::fixTopology(HnsClient::TopologyEntry *topologyEntry)
{
    if (!_catalog || !topologyEntry)
        return;
    _catalog->fixTopologyEntry(topologyEntry);
}

void TopologyAdministrator::unloadTopology(QString topologyUUID)
{
    if (!_manager)
        return;
    _manager->unloadTopologyByUUID(topologyUUID, 0, 0);
}

void TopologyAdministrator::deleteTopology(QString topologyUUID)
{
    if (!_manager)
        return;
    _manager->deleteTopologyByUUID(topologyUUID, 0, 0);
}

void TopologyAdministrator::loadTopology(qlonglong entryId, QObject *receiver, const char *slot)
{
    if (!_manager)
        return;
    _manager->loadTopology(entryId, receiver, slot);
}

void TopologyAdministrator::loadTopology(QString topologyUUID, QObject *receiver, const char *slot)
{
    if (!_manager)
        return;
    _manager->loadTopologyByUUID(topologyUUID, receiver, slot);
}

void TopologyAdministrator::createTopology(int width, int height, QString colorName, bool isModel, QObject *receiver, const char *slot)
{
    if (!_manager)
        return;
    _manager->createTopology(width, height, colorName, isModel, receiver, slot);
}

void TopologyAdministrator::closeTopology(QString topologyUUID)
{
    if (!_manager)
        return;
    _manager->closeTopologyByUUID(topologyUUID);
}

bool TopologyAdministrator::saveTopology(HnsClient::Topology *topology, HnsClient::TopologyEntry *updatedEntry, QObject *receiver, const char *slot, Hns::ByteArray screenshotData)
{
    if (!_manager)
        return false;
    _manager->saveTopology(topology, updatedEntry, receiver, slot, screenshotData);
    return true;

}

bool TopologyAdministrator::saveTopologyAs(HnsClient::Topology *topology, HnsClient::TopologyEntry *updatedEntry, QObject *receiver, const char *slot, Hns::ByteArray screenshotData)
{
    if (!_manager)
        return false;
    _manager->saveTopologyAs(topology, updatedEntry, receiver, slot, screenshotData);
    return true;
}

void TopologyAdministrator::setScreenshot(QString topologyUUID, Hns::ByteArray data)
{
    if (!_catalog)
        return;
    _catalog->setScreenshotByUUID(topologyUUID, data);
}

void TopologyAdministrator::switchTopologyType(QString topologyUUID, QString backgroundColorName, QObject *receiver, const char *slot)
{
    if (!_manager)
        return;
    _manager->switchTopologyType(topologyUUID, backgroundColorName, receiver, slot);
}

} // namespace Hnv
