/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef TOPOLOGYADMINISTRATOR_H
#define TOPOLOGYADMINISTRATOR_H

// Generated hynesim headers
#include "common/types.h"

// Qt headers
#include <QObject>

namespace HnsClient {
class TopologyEntry;
class Topology;
class TopologyEntryCatalog;
class TopologyManager;
} // namespace HnsClient

namespace Hnv {

/*******************************************************************************
* TopologyAdministrator
*******************************************************************************/
class TopologyAdministrator : public QObject
{
    Q_OBJECT
public:
    explicit TopologyAdministrator(QObject *parent = 0);
    ~TopologyAdministrator();

    HnsClient::TopologyManager *manager();
    HnsClient::TopologyEntryCatalog *catalog();

    void init(HnsClient::TopologyEntryCatalog *catalog, HnsClient::TopologyManager *manager);
    void reset();

    QList<HnsClient::TopologyEntry *> getCatalogEntryList();
    HnsClient::Topology *topologyByUUID(QString uuid);
    HnsClient::Topology *topologyByEntryId(qlonglong entryId);

    HnsClient::TopologyEntry *duplicateEntryByUUID(QString uuid, QObject *parent = 0);
    void updateEntry(HnsClient::TopologyEntry *updatedEntry);
    void fixTopology(HnsClient::TopologyEntry *topologyEntry);
    void unloadTopology(QString topologyUUID);
    void deleteTopology(QString topologyUUID);
    void loadTopology(qlonglong entryId, QObject *receiver, const char *slot);
    void loadTopology(QString topologyUUID, QObject *receiver, const char *slot);
    void createTopology(int width, int height, QString colorName, bool isModel, QObject *receiver, const char *slot);
    void closeTopology(QString topologyUUID);

    bool saveTopology(HnsClient::Topology *topology, HnsClient::TopologyEntry *updatedEntry, QObject *receiver, const char *slot, Hns::ByteArray screenshotData = Hns::ByteArray());
    bool saveTopologyAs(HnsClient::Topology *topology, HnsClient::TopologyEntry *updatedEntry, QObject *receiver, const char *slot, Hns::ByteArray screenshotData = Hns::ByteArray());

    void setScreenshot(QString topologyUUID, Hns::ByteArray data);

    void switchTopologyType(QString topologyUUID, QString backgroundColorName, QObject *receiver, const char *slot);

signals:
    void reseted();

private:
    HnsClient::TopologyEntryCatalog *_catalog;
    HnsClient::TopologyManager *_manager;
};

} // namespace Hnv
#endif // TOPOLOGYADMINISTRATOR_H
