/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "dynamicfiltersortmodel.h"

namespace Hnv {

DynamicFilterSortModel::DynamicFilterSortModel(QObject *parent)
    : QSortFilterProxyModel(parent)
{
}

void DynamicFilterSortModel::setFilter(int column, const QStringList &filters)
{
    if (column < columnCount()) {
        QStringList currentFilter = _filters.value(column);
        if (currentFilter != filters) {
            _filters[column] = filters;
            invalidateFilter();
        }
    }
}

bool DynamicFilterSortModel::filterAcceptsRow(int source_row, const QModelIndex &source_parent) const
{
    bool result = true;
    int colCount = sourceModel()->columnCount(source_parent);
    for (int i = 0; i < colCount; ++i) {
        QStringList filter = _filters.value(i, QStringList());
        if (!filter.isEmpty()) {
            QString sourceData = sourceModel()->index(source_row, i, source_parent).data(filterRole()).toString();
            foreach(const QString &f, filter) {
                if (!sourceData.contains(f, sortCaseSensitivity())) {
                    result = false;
                    break;
                }
            }
        }
    }
    return result;
}

} // namespace Hnv
