/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "display.h"
#include "ressources.h"

// Qt headers
#include <QSettings>
#include <QWidget>
#include <QPainter>

namespace RemoteAccess {

/*******************************************************************************
* Display
*******************************************************************************/
Display::Display(QObject *parent)
    : QObject(parent)
{
}

Display::~Display()
{

}

/*******************************************************************************
* ImageDisplay
*******************************************************************************/
ImageDisplay::ImageDisplay(QObject *parent)
    : Display(parent)
{
    _createImage();
    connect(this, SIGNAL(imageChanged(QImage,int,int,int,int)), SLOT(_imageChangedSlot(QImage,int,int,int,int)));
    connect(this, SIGNAL(clearImaged()), SLOT(_clearImagedSlot()));
}

ImageDisplay::~ImageDisplay()
{

}

QImage ImageDisplay::getScreenShot()
{
    return _screenshot;
}

void ImageDisplay::_imageChangedSlot(QImage image, int x, int y, int width, int height)
{
    QPainter painter(&_screenshot);
    painter.drawImage(QRect(x, y, width, height), image);
}

void ImageDisplay::_clearImagedSlot()
{
    _createImage();
}

void ImageDisplay::_createImage()
{
    QSettings settings;
    QString resolution = settings.value(Hnv::Constants::REMOTEDISPLAY_RESOLUTION_OPTION).toString();
    if (resolution.isEmpty())
        resolution = "640x480";
    QStringList resolutionList = resolution.split("x");
    int width = resolutionList.at(0).toInt();
    int height = resolutionList.at(1).toInt();
    _screenshot = QImage(QSize(width, height), QImage::Format_RGB32);
    _screenshot.fill(Qt::black);
}

/*******************************************************************************
* WidgetDisplay
*******************************************************************************/
WidgetDisplay::WidgetDisplay(QWidget *widget, QObject *parent)
    : Display(parent)
    , _widget(widget)
{
}

WidgetDisplay::~WidgetDisplay()
{

}

QWidget* WidgetDisplay::widget()
{
    return _widget;
}

QImage WidgetDisplay::getScreenShot()
{
    //TODO: use console image in qrc
    QImage image(200, 200, QImage::Format_ARGB32);
    image.fill(Qt::black);
    return image;
}

} // namespace RemoteAccess
