/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef DISPLAY_H
#define DISPLAY_H

// Project headers
#include "remotemanager.h"

// Qt headers
#include <QObject>
#include <QImage>

class QWidget;

namespace RemoteAccess {

/*******************************************************************************
* Display
*******************************************************************************/
class Display : public QObject
{
    Q_OBJECT
public:
    explicit Display(QObject *parent = 0);
    ~Display();
    virtual QImage getScreenShot() = 0;
};

/*******************************************************************************
* ImageDisplay
*******************************************************************************/
class ImageDisplay : public Display
{
    Q_OBJECT
public:
    explicit ImageDisplay(QObject *parent = 0);
    ~ImageDisplay();
    virtual QImage getScreenShot();
signals:
    void mouseEventReceived(RemoteAccess::MouseInfo info);
    void keyboardEventReceived(RemoteAccess::KeyInfo info);
    void imageChanged(QImage image, int x, int y, int width, int height);
    void clearImaged();
    void sizeChanged(QSize size);
    void cursorChanged(QImage, int x, int y);
    void clipboardDataHostToGuest(QString text);
    void clipboardDataGuestToHost(QString text);
    void clipboardSelectionHostToGuest(QString text);
    void clipboardSelectionGuestToHost(QString text);
    void clipboardDataClearedGuestToHost();
    void clipboardSelectionClearedGuestToHost();
    void focusOutEvent();

private slots:
    void _imageChangedSlot(QImage image, int x, int y, int width, int height);
    void _clearImagedSlot();
private:
    void _createImage();

    QImage _screenshot;
};

/*******************************************************************************
* WidgetDisplay
*******************************************************************************/
class WidgetDisplay : public Display
{
    Q_OBJECT
public:
    explicit WidgetDisplay(QWidget *widget, QObject *parent = 0);
    ~WidgetDisplay();
    QWidget *widget();
    virtual QImage getScreenShot();
private:
    QWidget *_widget;
};

} // namespace RemoteAccess
#endif // DISPLAY_H
