/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "remoteclient.h"
#include "display.h"
#include "worker.h"
#include "remotemanager.h"

// Qt headers
#include <QImage>

namespace RemoteAccess {

RemoteClient::RemoteClient(Hns::RemoteDisplayInfo info, QObject *parent)
    : QObject(parent)
    , _info(info)
    , _worker(0)
{
    _worker = RemoteManager::instance()->_createWorker(_info);
    connect(_worker, SIGNAL(connectionEnded()), this, SLOT(connectionEnd()));
    connect(_worker, SIGNAL(connectionError(QString)), this, SLOT(connectionErrorSlot(QString)));
    connect(_worker, SIGNAL(canRedirectUsbChanged()), this, SIGNAL(canRedirectUsbChanged()));
}

RemoteClient::~RemoteClient()
{
    QMetaObject::invokeMethod(_worker, "deleteLater", Qt::QueuedConnection);
    delete _display;
}

void RemoteClient::openConnection()
{
    _worker->setInfo(_info);
    _worker->openConnection();
    _display = _worker->createDisplay();
}

int RemoteClient::id()
{
    return _id;
}

void RemoteClient::setId(int id)
{
    _id = id;
}

void RemoteClient::closeConnection()
{
    _worker->closeConnection();
}

Display* RemoteClient::display()
{
    return _display;
}

Hns::RemoteDisplayInfo RemoteClient::info()
{
    return _info;
}

QImage RemoteClient::getScreenShot()
{
    return _display->getScreenShot();
}

UsbManager *RemoteClient::usbManager()
{
    return _worker->usbManager();
}

bool RemoteClient::canRedirectUsb()
{
    return _worker->canRedirectUsb();
}

void RemoteClient::connectionEnd()
{
    emit connectionEnded(_id);
}

void RemoteClient::connectionErrorSlot(QString message)
{
    emit connectionError(_id, message);
}

} // namespace RemoteAccess
