/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef REMOTEMANAGER_H
#define REMOTEMANAGER_H

#include "usbmanager.h"

// Project headers
#include "hnvplugins.h"

// Qt headers
#include <QObject>
#include <QHash>
#include <QImage>

class QTimer;
class QSignalMapper;

namespace HnsClient {
class GenericEntity;
} // namespace HnsClient

namespace RemoteAccess {
class RemoteViewFactory;
class RemoteView;
class RemoteClient;
class UsbManager;

class RemoteManager : public QObject
{
    Q_OBJECT
public:
    static void create(QObject *parent = 0);
    static RemoteManager * instance();
    void newConnection(HnsClient::GenericEntity *entity);
    bool openConnection(int id, Hns::RemoteDisplayInfo info);
    void closeConnection(int id);
    bool updateShortname(int id, const QString &shortname);
    RemoteView *createRemoteView(int id);
    QImage screenshot(int id);
    QString getShortNameById(int id);
    Hns::RemoteDisplayInfo remoteDisplayInfo(int id);
    bool canRedirectUsb(int id);
    UsbManager *usbManager(int id);
    int usbDeviceConnected(UsbDevice device, int entityId);
    bool hasUsbDeviceConnected(int id);
    ~RemoteManager();
public slots:
    void remoteViewClosedSlot(int id);
signals:
    void connectionOpened(int id);
    void connectionEnded(int id);
    void connectionError(int id, QString message);
    void screenshotChanged(int,QImage);
    void needToChangeIndex(int id);
    void shortnameChanged(int id, const QString &shortname);
    void usbDeviceReset(int id, bool deviceConnected);
    void canRedirectUsbChanged(int id);

private slots:
    void _getRemoteDisplayInfo(int, Hns::RemoteDisplayInfo info);
    void _timerOut();
    void _closeAllConnections();
    void _connectionEndedSlot(int id);
    void _usbDevicesResetSlot(int id);
    void _canRedirectUsbChangedSlot(int id);

private:
    explicit RemoteManager(QObject *parent = 0);
    bool _canHandle(Hns::RemoteDisplayType type);
    bool _createClient(int id, Hns::RemoteDisplayInfo info);
    Worker *_createWorker(Hns::RemoteDisplayInfo info);

    static RemoteManager* _instance;
    QHash<Hns::RemoteDisplayType, RemoteWorkerPlugin *> _pluginHash;
    RemoteViewFactory *_remoteViewFactory;

    //informations
    QHash<int, RemoteClient *> _clientHash; //by id
    QHash<int, QString> _nameHash; //by id
    QTimer *_timer;
    QSignalMapper *_usbDeviceSignalMapper;
    QSignalMapper *_canRedirectUsbSignalMapper;

    friend class RemoteClient;
};

enum MouseButton
{
    None,
    Left,
    Mid,
    Right
};

enum State
{
    Move,
    Click,
    DoubleClick,
    Release,
    WheelUp,
    WheelDown
};

struct KeyInfo
{
    quint32 nativeVirtualKey;
    quint32 nativeVirtualCode;
    bool press;
};

struct MouseInfo
{
    int x;
    int y;
    MouseButton button;
    State state;
};

} // namespace RemoteAccess
#endif // REMOTEMANAGER_H
