/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef REMOTEVIEW_H
#define REMOTEVIEW_H

// Project headers
#include "remotemanager.h"

// Qt headers
#include <QWidget>
#include <QImage>

class QMouseEvent;
class QKeyEvent;

namespace RemoteAccess {

class Display;

/*******************************************************************************
* RemoteView
*******************************************************************************/
class RemoteView : public QWidget
{
    Q_OBJECT
public:
    explicit RemoteView(QWidget *parent = 0);
    virtual void setDisplay(Display *display) = 0;
    virtual void invalidate();
};

/*******************************************************************************
* WidgetRemoteView
*******************************************************************************/
class WidgetRemoteView : public RemoteView
{
    Q_OBJECT
public:
    Q_INVOKABLE WidgetRemoteView(QWidget *parent = 0);
    void setDisplay(Display *display);
    virtual void invalidate();
private:
    QWidget *_widget;
};

/*******************************************************************************
* ImageRemoteView
*******************************************************************************/
class ImageRemoteView : public RemoteView
{
    Q_OBJECT
public:
    Q_INVOKABLE ImageRemoteView(QWidget *parent = 0);
    void setDisplay(Display *display);
signals:
    void mouseEvent(RemoteAccess::MouseInfo);
    void keyEvent(RemoteAccess::KeyInfo);
    void clipboardDataEvent(QString text);
    void clipboardSelectionEvent(QString text);
    void focusOutEvent();
public slots:
    void imageReceive(QImage image, int x, int y, int w, int h);
    void sizeChange(QSize size);
    void clearImage();
    void cursorReceive(QImage, int hotX, int hotY);
    void connectionEnd();
    void clipboardDataEventSlot(QString text);
    void clipboardSelectionEventSlot(QString text);
    void clipboardDataClearedSlot();
    void clipboardSelectionClearedSlot();
protected:
    void paintEvent(QPaintEvent *event);
    void mouseMoveEvent(QMouseEvent *event);
    void mousePressEvent(QMouseEvent *event);
    void mouseReleaseEvent(QMouseEvent *event);
    void mouseDoubleClickEvent(QMouseEvent *event);
    void keyPressEvent(QKeyEvent *event);
    void keyReleaseEvent(QKeyEvent *event);
    void wheelEvent(QWheelEvent *event);
    bool event(QEvent *event);
    bool eventFilter(QObject *object, QEvent *event);
    void focusInEvent(QFocusEvent *event);
    void focusOutEvent(QFocusEvent *event);
    void enterEvent(QEvent *event);
    void leaveEvent(QEvent *event);

private slots:
    void _clipboardDataChangedSlot();
    void _clipboardSelectionChangedSlot();

private:
    QPixmap _image;
};

} // namespace RemoteAccess
#endif // REMOTEVIEW_H
