/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "remoteviewfactory.h"
#include "display.h"
#include "remoteview.h"

// Qt headers
#include <QDebug>

namespace RemoteAccess {

RemoteViewFactory::RemoteViewFactory(QObject *parent)
    : QObject(parent)
{
    registerMetaObject<ImageDisplay, ImageRemoteView>();
    registerMetaObject<WidgetDisplay, WidgetRemoteView>();
}

RemoteView* RemoteViewFactory::createRemoteView(Display *display)
{
    QString className = display->metaObject()->className();
    if (!_factoryHash.contains(className)) {
        qDebug() << tr("the factory does not contains this type of display");
        return 0;
    }
    QMetaObject metaObject = _factoryHash.value(className);
    RemoteView *view(qobject_cast<RemoteView*>(metaObject.newInstance()));
    view->setDisplay(display);
    return view;
}

} // namespace RemoteAccess
