/******************************************************************************
**
** Copyright (C) 2003-2017 DIATEAM. All rights reserved.
**
** This file is part of Hynesim.
**
** Hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** Hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#include "usbmanager.h"

namespace RemoteAccess {

UsbManager::UsbManager(QObject *parent)
    : QObject(parent)
{
}

int UsbManager::deviceSize() const
{
    return _devices.size();
}

UsbDevice UsbManager::device(int index)
{
    return _devices.at(index);
}

bool UsbManager::deviceConnected(UsbDevice device)
{
    foreach (const UsbDevice &dev, _devices) {
        bool sameDevice = (device.manufacturer == dev.manufacturer);
        sameDevice &= (device.product == dev.product);
        sameDevice &= (device.descriptor == dev.descriptor);
        sameDevice &= (device.bus == dev.bus);
        sameDevice &= (device.address == dev.address);
        if (sameDevice && dev.connected)
            return true;
    }
    return false;
}

bool UsbManager::hasDeviceConnected()
{
    return false;
}

void UsbManager::connectUsbDevice(UsbDevice device)
{
    Q_UNUSED(device)
}

void UsbManager::disconnectUsbDevice(UsbDevice device)
{
    Q_UNUSED(device)
}

void UsbManager::disconnectAllUsbDevices()
{
}

} // namespace RemoteAccess
