/******************************************************************************
**
** Copyright (C) 2003-2017 DIATEAM. All rights reserved.
**
** This file is part of Hynesim.
**
** Hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** Hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef USBMANAGER_H
#define USBMANAGER_H

#include <QObject>

namespace RemoteAccess {

struct UsbDevice {
    QString manufacturer;
    QString product;
    QString descriptor;
    QString bus;
    QString address;
    bool connected;
};

inline bool operator ==(const UsbDevice &left, const UsbDevice &right) {
    if (left.manufacturer != right.manufacturer)
        return false;
    if (left.product != right.product)
        return false;
    if (left.descriptor != right.descriptor)
        return false;
    if (left.bus != right.bus)
        return false;
    if (left.address != right.address)
        return false;
    if (left.connected != right.connected)
        return false;
    return true;
}

class UsbManager : public QObject
{
    Q_OBJECT
public:
    explicit UsbManager(QObject *parent = nullptr);
    int deviceSize() const;
    UsbDevice device(int index);
    bool deviceConnected(UsbDevice device);
    virtual bool hasDeviceConnected();
    virtual void connectUsbDevice(UsbDevice device);
    virtual void disconnectUsbDevice(UsbDevice device);
    virtual void disconnectAllUsbDevices();

protected:
    QList<UsbDevice> _devices;

signals:
    void devicesReset();
};

} //namespace RemoteAccess

#endif // USBMANAGER_H
