/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef TOPOLOGYCATALOGMODEL_H
#define TOPOLOGYCATALOGMODEL_H

// Project headers
#include "hnv_catalogitem.h"

// Qt headers
#include <QAbstractTableModel>

namespace HnsClient {
class TopologyEntry;
class Topology;
} // namespace HnsClient

namespace Hnv {
class TopologyAdministrator;

/*******************************************************************************
* TopologyItem
*******************************************************************************/
class TopologyItem : public EntryItem
{
    Q_OBJECT
public:
    enum TopologyEntryRole {
        StatusRole = EntryItem::IDRole + 100,
        IsNewRole,
        UUIDRole,
        IsLoadedRole,
        IsModifiedRole,
        ConnectedUsersRole,
        IsModelRole,
        ScreenShotRole
    };

    explicit TopologyItem(HnsClient::TopologyEntry *entry, HnsClient::Topology *topology, QObject *parent = 0);
    ~TopologyItem();
    void setTopology(HnsClient::Topology *topology);
    virtual QVariant data(int column = 0, int role = Qt::DisplayRole) const;
    HnsClient::TopologyEntry *topologyEntry() const;
    HnsClient::Topology *topology();
    virtual Qt::ItemFlags flags(const QModelIndex index) const;

    bool isNew() const;
    bool isLoaded() const;
    bool isModified() const;
    QList<qlonglong>  connectedUsers() const;
    int connectedUsersCount() const;

public slots:
    void addConnectedUser(qlonglong id);
    void removeConnectedUser(qlonglong id);
    void topologyChangedSlot();

signals:
    void connectedUserAdded(qlonglong id);
    void connectedUserRemoved(qlonglong id);
    void updated(QString uuid);

protected:
    QPixmap _topologyPreview() const;
    void _drawPreviewBorder(QPixmap &preview) const;
    void _drawPreviewStatusIcon(QPixmap &preview, const QString &iconPath) const;


private:
    HnsClient::Topology *_topology;
};


/*******************************************************************************
* TopologyCatalogModel
*******************************************************************************/
class TopologyCatalogModel : public QAbstractTableModel
{
    Q_OBJECT
public:
    explicit TopologyCatalogModel(QObject *parent = 0);
    ~TopologyCatalogModel();

    void initModel(TopologyAdministrator *administrator);
    void resetModel();

    int rowCount(const QModelIndex &parent) const;
    int columnCount(const QModelIndex &parent) const;
    QVariant data(const QModelIndex &index, int role) const;
    virtual bool setData ( const QModelIndex & index, const QVariant & value, int role = Qt::EditRole );
    QMimeData *mimeData(const QModelIndexList &indexes) const;
    QVariant headerData(int section, Qt::Orientation orientation, int role) const;
    QModelIndex index(int row, int column, const QModelIndex &parent = QModelIndex()) const;
    virtual Qt::ItemFlags flags(const QModelIndex &index) const;
    QList<TopologyItem *> topologyList();
    TopologyItem * itemById(qlonglong entryId);

private slots:
    void _topologyEntryAddedSlot(HnsClient::TopologyEntry *entry);
    void _topologyEntryUpdatedSlot(HnsClient::TopologyEntry *entry);
    void _topologyEntryRemovedSlot(qlonglong topologyEntryID);
    void _topologyLoadedSlot(QString topologyUUID);
    void _topologyUnloadedSlot(QString topologyUUID);
    void _topologyItemUpdatedSlot(QString uuid);
    void _topologyAddedSlot(HnsClient::Topology *topology);
    void _topologySavedAsSlot(QString oldUuid, QString newUuid);

private:
    void _resetData();
    void _populateCatalog();
    void _createTopologyItem(HnsClient::TopologyEntry *entry);
    TopologyItem *_itemByUUID(QString uuid);
    void _itemUpdated(TopologyItem *item);

    TopologyAdministrator *_administrator;
    QList<TopologyItem *> _topologyList;
    QHash<qlonglong, TopologyItem *> _topologyHash;
};

} // namespace Hnv
#endif // TOPOLOGYCATALOGMODEL_H
