/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef MULTICOLUMNSTOFLATPROXYMODEL_H
#define MULTICOLUMNSTOFLATPROXYMODEL_H

// Qt headers
#include <QAbstractProxyModel>

namespace Hnv {

/*******************************************************************************
* MultiColumnsToFlatProxyModel
*******************************************************************************/
class MultiColumnsToFlatProxyModel : public QAbstractProxyModel
{
    Q_OBJECT
public:
    explicit MultiColumnsToFlatProxyModel(QObject *parent = 0);
    explicit MultiColumnsToFlatProxyModel(QAbstractItemModel *model, QHash<int, QString> columns, QObject *parent = 0);

    virtual int columnCount(const QModelIndex &parent) const;
    virtual int rowCount(const QModelIndex &parent) const;
    virtual QVariant data(const QModelIndex &index, int role = Qt::DisplayRole) const;

    virtual QModelIndex index(int row, int column, const QModelIndex &parent = QModelIndex()) const;
    virtual QModelIndex parent(const QModelIndex &child) const;

    virtual QModelIndex mapFromSource(const QModelIndex &sourceIndex) const;
    virtual QModelIndex mapToSource(const QModelIndex &proxyIndex) const;

    virtual void setSourceModel(QAbstractItemModel *sourceModel);

    virtual Qt::ItemFlags flags(const QModelIndex &index) const;

    void setRole(int role);

public slots:
    void sourceDataChanged(QModelIndex, QModelIndex);
    void sourceRowsInserted(const QModelIndex &parent, int start, int end);
    void sourceRowsRemoved(const QModelIndex &parent, int start, int end);
    void sourceReseted();

private:
    struct ProxyMapper
    {
        QModelIndex sourceIndex;
        int wordIndex;
    };
    int _modelRole;
    QHash<int, QString> _sourceColumns;
    void _buildModelMap();
    void _buildModelMapBranch(int &position, const QModelIndex& parent);
    QList<ProxyMapper> _sourceIndexList;
    QHash<QModelIndex, int> _sourceIndexHash;


};

} // namespace Hnv
#endif // MULTICOLUMNSTOFLATPROXYMODEL_H
