/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef TREEMODEL_H
#define TREEMODEL_H

// Qt headers
#include <QAbstractItemModel>

namespace Hnv {

class TreeModelItem;

/*******************************************************************************
* TreeModel
*******************************************************************************/
class TreeModel : public QAbstractItemModel
{
    Q_OBJECT
public:
    explicit TreeModel(QObject *parent = 0);
    ~TreeModel();

    QModelIndex index(int row, int column, const QModelIndex &parent = QModelIndex()) const;
    QModelIndex parent(const QModelIndex &child) const;
    QModelIndex indexFromNode(TreeModelItem *item, int column = 0) const;
    TreeModelItem *itemFromIndex(const QModelIndex &index = QModelIndex()) const;

    int rowCount(const QModelIndex &parent = QModelIndex()) const;
    int columnCount(const QModelIndex &parent = QModelIndex()) const;
    Qt::ItemFlags flags(const QModelIndex &index) const;
    QVariant data(const QModelIndex &index, int role) const;
    bool setData(const QModelIndex &index, const QVariant &value, int role);

    virtual void addItem(TreeModelItem *parent, TreeModelItem *child);
    virtual TreeModelItem *removeItem(TreeModelItem *item);

protected slots:
    void _itemChangedSlot(TreeModelItem *item, int column);

protected:
    bool _isIndexValid(const QModelIndex &index) const;
    void _initRootItem(int columns);

    TreeModelItem *_rootItem;
};

} // namespace Hnv
#endif // TREEMODEL_H
