/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "treemodelitem.h"

namespace Hnv {

/*******************************************************************************
* TreeModelItem
*******************************************************************************/
int TreeModelItem::_nextID = 0;

TreeModelItem::TreeModelItem(int columns, QObject *parent)
    : QObject(parent)
    , _parentItem(0)
    , _columns(columns)
{
    _internalID = _nextID;
    _nextID++;
}

TreeModelItem::~TreeModelItem()
{
    clear();
}

int TreeModelItem::internalID() const
{
    return _internalID;
}

int TreeModelItem::internalType() const
{
    return 0;
}

QVariant TreeModelItem::internalSortValue() const
{
    return QVariant();
}

TreeModelItem *TreeModelItem::parentItem() const
{
    return _parentItem;
}

void TreeModelItem::setParentItem(TreeModelItem *parentItem)
{
    if (parentItem != this) {
        disconnect(_parentItem);
        _parentItem = parentItem;
    }
}

TreeModelItem *TreeModelItem::child(int row)
{
    if (row < _children.count())
        return _children.value(row);
    return 0;
}

void TreeModelItem::appendChild(TreeModelItem *child)
{
    child->setParentItem(this);
    _children.append(child);
    connect(child, SIGNAL(itemChanged(TreeModelItem*,int)), this, SIGNAL(itemChanged(TreeModelItem*,int)));
}

TreeModelItem *TreeModelItem::removeChild(int row)
{
    TreeModelItem * child = _children.takeAt(row);
    child->setParentItem(0);
    return child;
}

void TreeModelItem::clear()
{
    qDeleteAll(_children);
    _children.clear();
}

int TreeModelItem::row() const
{
    if (_parentItem != 0)
        return _parentItem->_children.indexOf(const_cast<TreeModelItem *>(this));
    return 0;
}

int TreeModelItem::childCount() const
{
    return _children.count();
}

int TreeModelItem::columnCount() const
{
    return _columns;
}

Qt::ItemFlags TreeModelItem::flags(int) const
{
    return Qt::ItemIsEnabled | Qt::ItemIsSelectable;
}

QVariant TreeModelItem::data(int, int role) const {
    switch(role) {
    case TreeModelItem::InternalIDRole:
        return internalID();
        break;
    case TreeModelItem::InternalTypeRole:
        return internalType();
        break;
    case TreeModelItem::InternalSortRole:
        return internalSortValue();
        break;
    case Qt::DisplayRole:
    case Qt::EditRole:
        return QVariant();
    }
    return QVariant();
}

bool TreeModelItem::setData(int, const QVariant &, int)
{
    return false;
}

void TreeModelItem::columnChanged(int column)
{
    if (column < columnCount())
        emit itemChanged(this, column);
}

} // namespace Hnv
