/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef TREEMODELITEM_H
#define TREEMODELITEM_H

// Qt headers
#include <QList>
#include <QVariant>

namespace Hnv {

/*******************************************************************************
* TreeModelItem
*******************************************************************************/
class TreeModelItem : public QObject
{
    Q_OBJECT
public:
    enum TreeItemRole {
        InternalIDRole = Qt::UserRole +1,
        InternalTypeRole,
        InternalSortRole
    };

    TreeModelItem(int columns = 0, QObject * parent = 0);
    virtual ~TreeModelItem();

    int internalID() const;
    virtual int internalType() const;
    virtual QVariant internalSortValue() const;

    TreeModelItem *parentItem() const;
    void setParentItem(TreeModelItem *parentItem);

    TreeModelItem *child(int row);
    void appendChild(TreeModelItem *child);

    TreeModelItem *removeChild(int row);
    void clear();

    int row() const;
    int childCount() const;
    virtual int columnCount() const;

    virtual Qt::ItemFlags flags(int column) const;
    virtual QVariant data(int column = 0, int role = Qt::DisplayRole) const;
    virtual bool setData(int column, const QVariant &value, int role = Qt::EditRole);

    void columnChanged(int column);

signals:
    void itemChanged(TreeModelItem *item, int column);

protected:
    TreeModelItem * _parentItem;
    QList<TreeModelItem *> _children;
    static int _nextID;

private:
    int _columns;
    int _internalID;
};

} // namespace Hnv
#endif // TREEMODELITEM_H
