/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "treesortfilterproxymodel.h"

// Qt headers
#include <QFont>

namespace Hnv {

/*******************************************************************************
* TreeSortFilterProxyModel
*******************************************************************************/
TreeSortFilterProxyModel::TreeSortFilterProxyModel(QObject *parent)
    : QSortFilterProxyModel(parent)
{
}

void TreeSortFilterProxyModel::setFilterColumnsList(QList<int> columns)
{
    _filterColumns = columns;
}

QVariant TreeSortFilterProxyModel::data(const QModelIndex &index, int role) const
{
    QVariant result = QSortFilterProxyModel::data(index, role);

    if (!filterRegExp().isEmpty() && result.canConvert<QFont>() && role == Qt::FontRole) {
        QModelIndex sourceIndex = mapToSource(index);
        QModelIndex rowIndex = sourceIndex.sibling(sourceIndex.row(), 0);
        if (_isItemValid(rowIndex, false)) {
            QFont fontRole = result.value<QFont>();
            fontRole.setBold(true);
            result = QVariant(fontRole);
        }
    }
    return result;
}

/*******************************************************************************
* TreeSortFilterProxyModel - Protected
*/
bool TreeSortFilterProxyModel::filterAcceptsRow(int source_row, const QModelIndex &source_parent) const
{
    QModelIndex rowIndex = sourceModel()->index(source_row, 0, source_parent);
    return _isItemValid(rowIndex);
}

/*******************************************************************************
* TreeSortFilterProxyModel - Private
*/
bool TreeSortFilterProxyModel::_isItemValid(QModelIndex rowIndex, bool recursive) const
{
    bool result = false;
    if (_filterColumns.count() == 0) {
        QModelIndex index = rowIndex.sibling(rowIndex.row(), filterKeyColumn());
        if (index.isValid())
            result = index.data(filterRole()).toString().contains(filterRegExp());
    } else {
        foreach(int column, _filterColumns) {
            QModelIndex index = rowIndex.sibling(rowIndex.row(), column);
            if (index.isValid())
                result = index.data(filterRole()).toString().contains(filterRegExp());
            if (result)
                break;
        }
    }
    if (recursive && !result)
        result = _hasValidChild(rowIndex);
    return result;
}

bool TreeSortFilterProxyModel::_hasValidChild(QModelIndex parent) const
{
    int childCount = sourceModel()->rowCount(parent);
    for(int row = 0; row < childCount; row++) {
        if (_isItemValid(parent.child(row, 0)))
            return true;
    }
    return false;
}

} // namespace Hnv
