/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef GROUPSMODEL_H
#define GROUPSMODEL_H

// hynesim common headers
#include "common/usermanager.h"

// Qt headers
#include <QAbstractTableModel>
#include <QSortFilterProxyModel>

namespace HnsClient {
class UserManager;
} // namespace HnsClient

namespace Hnv {

/*******************************************************************************
* GroupsModel
*******************************************************************************/
class GroupsModel : public QAbstractTableModel
{
    Q_OBJECT
public:
    struct GroupItem {
        qlonglong id;
        QString name;
        qlonglong createdOn;
        qlonglong modifiedOn;
        QList<qlonglong> users;
    };

    enum listColumn {
        COLUMN_ID,
        COLUMN_NAME,
        COLUMN_CREATED,
        COLUMN_MODIFIED,
        COLUMN_GROUPS,
        COLUMN_SIZE //always last
    };

    explicit GroupsModel(QObject *parent = 0);
    void initModel(HnsClient::UserManager *manager);
    void resetModel();

    int rowCount(const QModelIndex &parent) const;
    int columnCount(const QModelIndex &parent) const;
    QVariant data(const QModelIndex &index, int role) const;
    QVariant headerData(int section, Qt::Orientation orientation, int role) const;
    int groupIdByIndex(int index);

signals:

private slots:
    void _buildData();
    void _groupAdded(qlonglong groupID);
    void _groupChanged(qlonglong groupID);
    void _groupDeleted(qlonglong deletedGroupID);

private:
    GroupItem _createItem(Diamesh::GroupPtr groupPtr);

    HnsClient::UserManager *_manager;
    QList<qlonglong> _positionList;
    QHash<qlonglong, GroupItem> _groupHash;
};

/*******************************************************************************
* GroupsProxyModel
*******************************************************************************/
class GroupsProxyModel : public QSortFilterProxyModel
{
    Q_OBJECT
public:
    GroupsProxyModel(QObject *parent = 0);    
    void setAllowedGroupList(QList<qlonglong> allowedGroupList);

protected:
    bool filterAcceptsRow(int source_row, const QModelIndex &source_parent) const;

private:
    QList<qlonglong> _allowedGroupList;
};

} // namespace Hnv
#endif // GROUPSMODEL_H
