/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef USERSMODEL_H
#define USERSMODEL_H

// Qt headers
#include <QAbstractTableModel>
#include <QSortFilterProxyModel>

// Generated hynesim headers
#include "common/usermanager.h"

namespace HnsClient {
class UserManager;
} // namespace HnsClient

namespace Hnv {

/*******************************************************************************
* UsersFilterProxyModel
*******************************************************************************/
class UsersFilterProxyModel : public QSortFilterProxyModel
{
public:
    UsersFilterProxyModel(QObject * parent = 0);
    void acceptEnabled(bool accept);
    void acceptDisabled(bool accept);
    void acceptOnline(bool accept);
    void acceptOffline(bool accept);

protected:
    bool filterAcceptsRow(int source_row, const QModelIndex &source_parent) const;

private:
    bool _acceptEnabled;
    bool _acceptDisabled;
    bool _acceptOnline;
    bool _acceptOffline;
};

/*******************************************************************************
* UsersModel
*******************************************************************************/
class UsersModel : public QAbstractTableModel
{
    Q_OBJECT
public:
    struct UserItem {
        qlonglong id;
        bool isEnabled;
        QString name;
        QString firstName;
        QString lastName;
        qlonglong createdOn;
        qlonglong modifiedOn;
        QList<qlonglong> groupList;
    };

    enum {
        USER_ID_ROLE = Qt::UserRole + 1,
        ENABLED_ROLE,
        CONNECTED_ROLE
    };

    enum listColumn {
        COLUMN_ID,
        COLUMN_USERNAME,
        COLUMN_FIRSTNAME,
        COLUMN_LASTNAME,
        COLUMN_CREATED,
        COLUMN_LASTMODIFIED,
        COLUMN_GROUPS,
        COLUMN_SIZE //always last
    };

    explicit UsersModel(QObject *parent = 0);
    void initModel(HnsClient::UserManager *manager);
    void resetModel();
    int rowCount(const QModelIndex &parent) const;
    int columnCount(const QModelIndex &parent) const;
    QVariant data(const QModelIndex &index, int role) const;
    QVariant headerData(int section, Qt::Orientation orientation, int role) const;

private slots:
    void _buildData();
    void _userAdded(qlonglong newUserID);
    void _userChanged(qlonglong userID);
    void _userDeleted(qlonglong deletedUserID);

private:
    UserItem _createItem(Hns::UserPtr user);

    HnsClient::UserManager *_manager;
    QList<qlonglong> _positionList;
    QHash<qlonglong, UserItem> _userHash;

};

} // namespace Hnv
#endif // USERSMODEL_H
