#ifndef ENTITYACTION_H
#define ENTITYACTION_H

// Qt headers
#include <QObject>
#include <QKeySequence>
#include <QIcon>

class QAction;

namespace Hnv {

/*******************************************************************************
* EntityActionHandler
*******************************************************************************/
class EntityActionHandler : public QObject
{
    Q_OBJECT
public:
    explicit EntityActionHandler(QString uniqueID, QIcon icon, QString text, QString tooltip, QKeySequence shortcut, QObject * parent = 0);
    ~EntityActionHandler();
    QString uniqueID();
    QAction *action();
    bool isEnableAtStart();
    void setEnableAtStart(bool);
    bool isAlwaysDisable();
    void setAlwaysDisable(bool disable);

signals:
    void actionTriggered();
    void removeFromList(EntityActionHandler *actionHandler);

private:
    QString _actionUniqueID;
    QIcon _actionIcon;
    QString _actionText;
    QString _actionToolTip;
    QKeySequence _actionShortCut;
    bool _enableAtStart;
    bool _alwaysDisable;
};

/*******************************************************************************
* EntityActionDispatcher
*******************************************************************************/
class EntityActionDispatcher : public QObject
{
    Q_OBJECT
public:
    static QList<EntityActionDispatcher *> parseHandlerList(QList<QList<EntityActionHandler *> > handlerList);

    explicit EntityActionDispatcher(QObject *parent = 0);
    ~EntityActionDispatcher();
    bool registerHandler(EntityActionHandler *handler);
    void enableHandler(QString handlerID);
    bool isHandlerDisable(QString handler);
    QAction *action(bool startIsActivate);

private slots:
    void _actionDestroyedSlot();
    void _removeHandlerListSlot(EntityActionHandler *actionHandler);

private:
    QString _actionUniqueID;
    QList<EntityActionHandler *> _handlerList;
};

} // namespace Hnv
#endif // ENTITYACTION_H
