/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "hnvstyle.h"
#include "modebrowserwidget.h"
#include "ressources.h"
#include "hyneviewtoolbar.h"
#include "topologywidget.h"
#include "topologycontainer.h"
#include "overlayexpandablewidget.h"
#include "statusbarlabel.h"
#include "loadbalancerdialog.h"
#include "welcomescreenwidget.h"
#include "sidebar.h"

// Qt headers
#include <QPainter>
#include <QPixmap>
#include <QStyleOption>
#include <QPushButton>
#include <QComboBox>
#include <QToolBar>
#include <QProgressBar>
#include <QScrollBar>
#include <QTextEdit>
#include <QSpinBox>
#include <QScrollArea>
#include <QEvent>

namespace Hnv {

// Pixmap for checked Radio button, from Qt source
static const char * const qt_cleanlooks_radiobutton_checked[] = {
    "13 13 20 1",
    " 	c None",
    ".	c #A8ABAE",
    "+	c #596066",
    "@	c #283138",
    "#	c #A9ACAF",
    "$	c #A6A9AB",
    "%	c #6B7378",
    "&	c #8C9296",
    "*	c #A2A6AA",
    "=	c #61696F",
    "-	c #596065",
    ";	c #93989C",
    ">	c #777E83",
    ",	c #60686E",
    "'	c #252D33",
    ")	c #535B62",
    "!	c #21292E",
    "~	c #242B31",
    "{	c #1F262B",
    "]	c #41484E",
    "             ",
    "             ",
    "             ",
    "    .+@+#    ",
    "   $%&*&=#   ",
    "   -&;>,'+   ",
    "   @*>,)!@   ",
    "   +&,)~{+   ",
    "   #='!{]#   ",
    "    #+@+#    ",
    "             ",
    "             ",
    "             "};



static const char * const qt_cleanlooks_arrow_down_xpm[] = {
    "11 7 2 1",
    " 	c None",
    "x	c #000000",
    "           ",
    "  x     x  ",
    " xxx   xxx ",
    "  xxxxxxx  ",
    "   xxxxx   ",
    "    xxx    ",
    "     x     "};

static const char * const qt_cleanlooks_arrow_up_xpm[] = {
    "11 7 2 1",
    " 	c None",
    "x	c #000000",
    "     x     ",
    "    xxx    ",
    "   xxxxx   ",
    "  xxxxxxx  ",
    " xxx   xxx ",
    "  x     x  ",
    "           "};

static const char * const qt_scrollbar_button_arrow_up[] = {
    "7 4 2 1",
    "   c None",
    "*  c #BFBFBF",
    "   *   ",
    "  ***  ",
    " ***** ",
    "*******"};

static const char * const qt_scrollbar_button_arrow_down[] = {
    "7 4 2 1",
    "   c None",
    "*  c #BFBFBF",
    "*******",
    " ***** ",
    "  ***  ",
    "   *   "};

static const char * const qt_scrollbar_button_arrow_left[] = {
    "4 7 2 1",
    "   c None",
    "*  c #BFBFBF",
    "   *",
    "  **",
    " ***",
    "****",
    " ***",
    "  **",
    "   *"};

static const char * const qt_scrollbar_button_arrow_right[] = {
    "4 7 2 1",
    "   c None",
    "*  c #BFBFBF",
    "*   ",
    "**  ",
    "*** ",
    "****",
    "*** ",
    "**  ",
    "*   "};

static const char * const qt_spinbox_button_arrow_down[] = {
    "7 4 2 1",
    "   c None",
    "*  c #BFBFBF",
    "*******",
    " ***** ",
    "  ***  ",
    "   *   "};

static const char * const qt_spinbox_button_arrow_up[] = {
    "7 4 2 1",
    "   c None",
    "*  c #BFBFBF",
    "   *   ",
    "  ***  ",
    " ***** ",
    "*******"};


static QColor mergedColors(const QColor &colorA, const QColor &colorB, int factor = 50)
{
    const int maxFactor = 100;
    QColor tmp = colorA;
    tmp.setRed((tmp.red() * factor) / maxFactor + (colorB.red() * (maxFactor - factor)) / maxFactor);
    tmp.setGreen((tmp.green() * factor) / maxFactor + (colorB.green() * (maxFactor - factor)) / maxFactor);
    tmp.setBlue((tmp.blue() * factor) / maxFactor + (colorB.blue() * (maxFactor - factor)) / maxFactor);
    return tmp;
}



HnvStyle::HnvStyle()
    : QCommonStyle()
    , _animateTimer(0)
    , _animationFps(25)
    , _animateStep(0)
{
    _startTime.start();
}

HnvStyle::~HnvStyle()
{
}

void HnvStyle::drawComplexControl(ComplexControl control, const QStyleOptionComplex *option, QPainter *painter, const QWidget *widget) const
{
    switch(control) {
    case CC_ComboBox: {
        if (const QStyleOptionComboBox *comboOption = qstyleoption_cast<const QStyleOptionComboBox *>(option)) {
            if ((comboOption->subControls & SC_ComboBoxFrame)) {
                if (comboOption->frame && !qobject_cast<QToolBar *>(widget->parentWidget()))
                    _drawRoundedPanel(painter, option->rect, comboOption->palette.base(), QPen(Qt::NoPen), 5);
            }
            if (comboOption->subControls & SC_ComboBoxArrow) {
                QColor arrowColor;
                if (qobject_cast<QToolBar *>(widget->parentWidget()))
                    arrowColor = comboOption->palette.buttonText().color();
                else
                    arrowColor = comboOption->palette.text().color();

                QRect arrowRect = proxy()->subControlRect(CC_ComboBox, comboOption, SC_ComboBoxArrow, widget);
                    _drawIndicator(painter, option, QImage(qt_cleanlooks_arrow_down_xpm), arrowColor, arrowRect);
            }
            if (comboOption->subControls & SC_ComboBoxEditField) {
                // Focus rect
                if (comboOption->state & State_HasFocus && !comboOption->editable) {
                    QStyleOptionFocusRect focus;
                    focus.QStyleOption::operator=(*comboOption);
                    focus.rect = subElementRect(SE_ComboBoxFocusRect, comboOption, widget);
                    focus.state |= State_FocusAtBorder;
                    proxy()->drawPrimitive(PE_FrameFocusRect, &focus, painter, widget);
                }
            }
        }
        break;
    } // CC_ComboBox
    case CC_ScrollBar: {
        QCommonStyle::drawComplexControl(control, option, painter, widget);
        break;
    } // CC_ScrollBar
    case CC_Slider: {
        if (const QStyleOptionSlider *sliderOption = qstyleoption_cast<const QStyleOptionSlider *>(option)) {
            QRect groove = proxy()->subControlRect(CC_Slider, option, SC_SliderGroove, widget);
            QRect handle = proxy()->subControlRect(CC_Slider, option, SC_SliderHandle, widget);
            bool horizontal = sliderOption->orientation == Qt::Horizontal;
            if (sliderOption->subControls & SC_SliderTickmarks) {
                bool ticksAbove = sliderOption->tickPosition & QSlider::TicksAbove;
                bool ticksBelow = sliderOption->tickPosition & QSlider::TicksBelow;
                int interval = sliderOption->tickInterval;
                int len = proxy()->pixelMetric(PM_SliderLength, sliderOption, widget);
                int thickness = proxy()->pixelMetric(PM_SliderControlThickness, sliderOption, widget);
                int available = proxy()->pixelMetric(PM_SliderSpaceAvailable, sliderOption, widget);
                int maxTickLength = (horizontal ? sliderOption->rect.height() : sliderOption->rect.width()) - thickness;
                if (sliderOption->tickPosition & QSlider::TicksBothSides)
                    maxTickLength /= 2;
                if (maxTickLength <= 0)
                    maxTickLength = 1;
                if (interval <= 0) {
                    interval = sliderOption->singleStep;
                    if (QStyle::sliderPositionFromValue(sliderOption->minimum, sliderOption->maximum, interval, available)
                            - QStyle::sliderPositionFromValue(sliderOption->minimum, sliderOption->maximum, 0, available) < 3)
                        interval = sliderOption->pageStep;
                }
                if (interval <= 0)
                    interval = 1;
                int value = sliderOption->minimum;
                while (value < sliderOption->maximum + interval) {
                    if (value > sliderOption->maximum)
                        break;
                    int tickLength = maxTickLength;
                    int padding = 0;
                    if (tickLength > thickness)
                        tickLength = thickness;
                    if ( tickLength > 1) {
                        if (value != sliderOption->minimum && value != sliderOption->maximum) {
                            tickLength--;
                            padding++;
                        }
                    }
                    int pos = sliderPositionFromValue(sliderOption->minimum, sliderOption->maximum, value, available, sliderOption->upsideDown) + (len / 2);
                    if (horizontal) {
                        if (ticksAbove) {
                            painter->drawLine(pos, sliderOption->rect.top() + padding,
                                              pos, sliderOption->rect.top() + padding + tickLength);
                        }
                        if (ticksBelow) {
                            painter->drawLine(pos, sliderOption->rect.bottom() - padding - tickLength,
                                              pos, sliderOption->rect.bottom() - padding);
                        }
                    } else {
                        if (ticksAbove) {
                            painter->drawLine(sliderOption->rect.left() + padding, pos,
                                              sliderOption->rect.left() + padding + tickLength, pos);
                        }
                        if (ticksBelow) {
                            painter->drawLine(sliderOption->rect.right() - padding - tickLength, pos,
                                              sliderOption->rect.right() - padding, pos);
                        }
                    }
                    value += interval;
                }
            }
            if ((sliderOption->subControls & SC_SliderGroove) && groove.isValid()) {
                painter->fillRect(groove, Qt::blue);
                painter->fillRect(groove, option->palette.base());
                QRect clipRect;
                QLinearGradient gradient;
                if (horizontal) {
                    gradient.setStart(groove.left(), groove.center().y());
                    gradient.setFinalStop(groove.right(), groove.center().y());
                    if (sliderOption->upsideDown)
                        clipRect = QRect(handle.right(), groove.top(), groove.right() - handle.right(), groove.height());
                    else
                        clipRect = QRect(groove.left(), groove.top(), handle.left(), groove.height());
                } else {
                    gradient.setStart(groove.center().x(), groove.bottom());
                    gradient.setFinalStop(groove.center().y(), groove.top());
                    if (sliderOption->upsideDown)
                        clipRect = QRect(groove.left(), handle.bottom(), groove.width(), groove.height() - handle.bottom());
                    else
                        clipRect = QRect(groove.left(), groove.top(), groove.width(), handle.top() - groove.top());
                }
                gradient.setColorAt(0, Qt::green);
                gradient.setColorAt(0.5, Qt::yellow);
                gradient.setColorAt(1, QColor(255, 165, 0));
                _drawPanel(painter, clipRect, gradient);
            }
            if ((sliderOption->subControls & SC_SliderHandle) && handle.isValid()) {
                _drawRoundedPanel(painter, handle, option->palette.light(), QPen(option->palette.midlight(), 1), 5);
            }
        }
        break;
    } // CC_Slider
    case CC_ToolButton: {
        QCommonStyle::drawComplexControl(control, option, painter, widget);
        break;
    } // CC_ToolButton
    case CC_TitleBar: {
        QCommonStyle::drawComplexControl(control, option, painter, widget);
        break;
    } // CC_TitleBar
    case CC_GroupBox: {
        if (const QStyleOptionGroupBox *groupBoxOption = qstyleoption_cast<const QStyleOptionGroupBox *>(option)) {
            QStyleOptionGroupBox opt = *groupBoxOption;
            opt.textColor = option->palette.windowText().color();
            QCommonStyle::drawComplexControl(control, &opt, painter, widget);
        } else {
            QCommonStyle::drawComplexControl(control, option, painter, widget);
        }
        break;
    } // CC_GroupBox
    case CC_Dial: {
        QCommonStyle::drawComplexControl(control, option, painter, widget);
        break;
    } // CC_Dial
    case CC_MdiControls: {
        QCommonStyle::drawComplexControl(control, option, painter, widget);
        break;
    } // CC_MdiControls
    case CC_SpinBox: {
        if (const QStyleOptionSpinBox *spinBoxOption = qstyleoption_cast<const QStyleOptionSpinBox *>(option)) {
            if (spinBoxOption->subControls & SC_SpinBoxFrame) {

                QRect rect = option->rect;

                _drawRoundedPanel(painter, rect, Qt::white, QPen(Qt::NoPen), 5, true);

                QPainterPath path;

                QRect spinBoxUpRect = proxy()->subControlRect(CC_SpinBox, spinBoxOption, SC_SpinBoxUp, widget);
                QRect imgRectUp = QImage(qt_spinbox_button_arrow_up).rect();
                imgRectUp.moveCenter(spinBoxUpRect.center());

                int middleHeight = option->rect.height() / 2;
                path.moveTo(imgRectUp.left(), middleHeight);
                path.lineTo(imgRectUp.right(), middleHeight);

                painter->save();
                QPen pen(QColor(Qt::black));
                if (!option->state.testFlag(State_Enabled))
                    pen.setColor(option->palette.midlight().color().darker(150));
                pen.setWidth(1);
                painter->setPen(pen);
                painter->setRenderHint(QPainter::Antialiasing, false);
                painter->drawPath(path);
                painter->restore();

            }
            if (spinBoxOption->subControls & SC_SpinBoxEditField) {
                QRect spinBoxEditRect = proxy()->subControlRect(CC_SpinBox, spinBoxOption, SC_SpinBoxEditField, widget);
                _drawPanel(painter, spinBoxEditRect, QBrush(Qt::transparent), QPen(Qt::NoPen));
            }
            if (spinBoxOption->subControls & SC_SpinBoxUp) {
                QRect spinBoxUpRect = proxy()->subControlRect(CC_SpinBox, spinBoxOption, SC_SpinBoxUp, widget);

                QColor colorIndicator = option->palette.shadow().color();
                if (spinBoxOption->stepEnabled & QAbstractSpinBox::StepUpEnabled && option->state.testFlag(State_Enabled)) {
                    if ((spinBoxOption->activeSubControls & SC_SpinBoxUp && spinBoxOption->state & State_MouseOver && !(spinBoxOption->state & State_Sunken))
                            || ((spinBoxOption->state & State_Sunken) && (spinBoxOption->activeSubControls & SC_SpinBoxUp)))
                        colorIndicator = option->palette.dark().color();
                }
                else {
                    colorIndicator = option->palette.midlight().color().darker(150);
                    if (spinBoxOption->state & State_Sunken) {
                        colorIndicator = colorIndicator.darker(150); //cancel _drawIndicator modification
                    }
                }

                QColor imgColor = colorIndicator;
                QImage img(qt_spinbox_button_arrow_up);
                if (option->state.testFlag(State_Sunken))
                    imgColor = imgColor.lighter();
                img.setColor(1, imgColor.rgba());
                QRect imgRect = img.rect();
                imgRect.moveCenter(spinBoxUpRect.center());
                imgRect.moveBottom(option->rect.height() / 2 - imgRect.height());
                painter->drawImage(imgRect, img);
            }
            if (spinBoxOption->subControls & SC_SpinBoxDown) {
                QRect spinBoxDownRect = proxy()->subControlRect(CC_SpinBox, spinBoxOption, SC_SpinBoxDown, widget);

                QColor colorIndicator = option->palette.shadow().color();
                if (spinBoxOption->stepEnabled & QAbstractSpinBox::StepDownEnabled && option->state.testFlag(State_Enabled)) {
                    if ((spinBoxOption->activeSubControls & SC_SpinBoxDown && spinBoxOption->state & State_MouseOver && !(spinBoxOption->state & State_Sunken))
                            || ((spinBoxOption->state & State_Sunken) && (spinBoxOption->activeSubControls & SC_SpinBoxDown)))
                        colorIndicator = option->palette.dark().color();
                }
                else {
                    colorIndicator = option->palette.midlight().color().darker(150);
                    if (spinBoxOption->state & State_Sunken)
                        colorIndicator = colorIndicator.darker(150); //cancel _drawIndicator modification
                }

                QColor imgColor = colorIndicator;
                QImage img(qt_spinbox_button_arrow_down);
                if (option->state.testFlag(State_Sunken))
                    imgColor = imgColor.lighter();
                img.setColor(1, imgColor.rgba());
                QRect imgRect = img.rect();
                imgRect.moveCenter(spinBoxDownRect.center());
                imgRect.moveTop(option->rect.height() / 2 + imgRect.height());
                painter->drawImage(imgRect, img);
            }
        } // CC_SpinBox
        break;
    }
    default :
        QCommonStyle::drawComplexControl(control, option, painter, widget);
    } // switch(control)
}

void HnvStyle::drawControl(ControlElement controlElement, const QStyleOption *option, QPainter *painter, const QWidget *widget) const
{
    switch(controlElement) {
    case CE_PushButton: {
        QCommonStyle::drawControl(controlElement, option, painter, widget);
        break;
    } // CE_PushButton
    case CE_PushButtonBevel: {
        QCommonStyle::drawControl(controlElement, option, painter, widget);
        break;
    } // CE_PushButtonBevel
    case CE_PushButtonLabel: {
        QCommonStyle::drawControl(controlElement, option, painter, widget);
        break;
    } // CE_PushButtonLabel
    case CE_DockWidgetTitle: {
        QCommonStyle::drawControl(controlElement, option, painter, widget);
        break;
    } // CE_DockWidgetTitle
    case CE_Splitter: {
        _drawPanel(painter, option->rect, option->palette.shadow());
        break;
    } // CE_Splitter
    case CE_CheckBox: {
        QCommonStyle::drawControl(controlElement, option, painter, widget);
        break;
    } // CE_CheckBox
    case CE_CheckBoxLabel: {
        QCommonStyle::drawControl(controlElement, option, painter, widget);
        break;
    } // CE_CheckBoxLabel
    case CE_ComboBoxLabel: {
        if (const QStyleOptionComboBox *comboOption = qstyleoption_cast<const QStyleOptionComboBox *>(option)) {
            painter->save();
            if (qobject_cast<QToolBar *>(widget->parentWidget()))
                painter->setPen(option->palette.brightText().color());
            QCommonStyle::drawControl(controlElement, comboOption, painter, widget);
            painter->restore();
        }
        break;
    }
    case CE_ToolBar: {
        QLinearGradient gradient(0, 0, 0, option->rect.height());
        _setGrayGradient(gradient, option->palette);
        _drawPanel(painter, option->rect, gradient);
        if (const Hnv::ToolBar *toolbar =qobject_cast<const Hnv::ToolBar *>(widget)) {
            if (toolbar->borders() != ToolBar::None)
                _drawBorder(painter, option->rect, option->palette.shadow().color(), Borders(int(toolbar->borders())));
        }
        break;
    } // CE_ToolBar
    case CE_ToolButtonLabel: {
        if (const QStyleOptionToolButton *toolOption = qstyleoption_cast<const QStyleOptionToolButton *>(option)) {
            if (toolOption->features & QStyleOptionToolButton::Arrow) {
                if (widget && qobject_cast<QTabBar *>(widget->parent())) {
                    _drawPanel(painter, option->rect, option->palette.button(), Qt::NoPen);
                }
                    _drawArrow(toolOption, painter, widget);
            }
            else {
                QStyleOptionToolButton myOpt = *toolOption;
                if (myOpt.state.testFlag(State_On))
                    myOpt.palette.setBrush(QPalette::ButtonText, Qt::black);
                else
                    myOpt.palette.setBrush(QPalette::ButtonText, Qt::white);
                QCommonStyle::drawControl(controlElement, &myOpt, painter, widget);
            }
        }
        break;
    } // CE_ToolButtonLabel
    case CE_ScrollBarAddLine:
    case CE_ScrollBarSubLine: {
        _drawPanel(painter, option->rect, option->palette.dark());
        PrimitiveElement arrow;
        if (option->state & State_Horizontal) {
            if (controlElement == CE_ScrollBarAddLine)
                arrow = option->direction == Qt::LeftToRight ? PE_IndicatorArrowRight : PE_IndicatorArrowLeft;
            else
                arrow = option->direction == Qt::LeftToRight ? PE_IndicatorArrowLeft : PE_IndicatorArrowRight;
        } else {
            if (controlElement == CE_ScrollBarAddLine)
                arrow = PE_IndicatorArrowDown;
            else
                arrow = PE_IndicatorArrowUp;
        }
        QStyleOption arrowOpt = *option;
        arrowOpt.rect = option->rect/*.adjusted(4, 4, -4, -4)*/;
        proxy()->drawPrimitive(arrow, &arrowOpt, painter, widget);
        break;
    }
    case CE_ScrollBarAddPage:
    case CE_ScrollBarSubPage:
        _drawPanel(painter, option->rect, option->palette.dark());
        break;
    case CE_ScrollBarSlider: {
        QRect rect = option->rect;
        if (const QStyleOptionSlider *scrollBar = qstyleoption_cast<const QStyleOptionSlider *>(option)) {
            if (scrollBar->orientation == Qt::Vertical) {
                rect.setWidth(rect.width() / 2);
                rect.moveLeft(rect.width() / 2);
            }
            else {
                rect.setHeight(rect.height() / 2);
                rect.moveTop(rect.height() / 2);
            }
        }
        _drawPanel(painter, option->rect, option->palette.dark());
        _drawRoundedPanel(painter, rect, option->palette.midlight(), QPen(option->palette.button().color()), 10);
        break;
    }
    case CE_MenuBarEmptyArea: { // Menu bar background
        QLinearGradient gradient(0, 0, 0, option->rect.height());
        _setGrayGradient(gradient, option->palette);
        _drawPanel(painter, option->rect, QBrush(gradient));
        break;
    }
    case CE_MenuBarItem: {
        if (const QStyleOptionMenuItem *menuItemOption = qstyleoption_cast<const QStyleOptionMenuItem *>(option)) {
            QLinearGradient gradient(0, 0, 0, option->rect.height());
            _setGrayGradient(gradient, option->palette);
            QBrush bgBrush(gradient);
            _drawPanel(painter, menuItemOption->rect, bgBrush, Qt::NoPen);
            QStyleOptionMenuItem updatedOption = *menuItemOption;
            bool active = menuItemOption->state & State_Selected;
            bool hasFocus = menuItemOption->state & State_HasFocus;
            bool down = menuItemOption->state & State_Sunken;

            if (active && hasFocus) {
                QRect activeRect = menuItemOption->rect;
                Borders borders = None;
                if (down) {
                    activeRect.adjust(0, 0, -1, -1);
                    borders |= Bottom | Right;
                 } else {
                    activeRect.adjust(1, 1, 0, 0);
                    borders |= Top | Left;
                }
                _drawBorder(painter, activeRect, menuItemOption->palette.light().color(), borders);
            }
            QCommonStyle::drawControl(controlElement, &updatedOption, painter, widget);
        }
        break;
    }
    case CE_MenuItem: { // Actions du menu
        if (const QStyleOptionMenuItem *menuItemOption = qstyleoption_cast<const QStyleOptionMenuItem *>(option)) {
            painter->save();

            int x, y, w, h;
            menuItemOption->rect.getRect(&x, &y, &w, &h);
            int tab = menuItemOption->tabWidth;
            bool disabled = !(menuItemOption->state & State_Enabled);
            bool checked = menuItemOption->checkType != QStyleOptionMenuItem::NotCheckable ? menuItemOption->checked : false;
            bool hover = menuItemOption->state & State_Selected;

            QBrush fill = menuItemOption->palette.brush(hover && !disabled ? QPalette::Highlight : QPalette::Dark);
            painter->fillRect(menuItemOption->rect.adjusted(0, 0, -1, 0), fill);

            if (menuItemOption->menuItemType == QStyleOptionMenuItem::Separator){
                int yoff = y-1 + h / 2;
                painter->setPen(menuItemOption->palette.shadow().color());
                painter->drawLine(x + 2, yoff, x + w - 4, yoff);
                painter->setPen(menuItemOption->palette.light().color());
                painter->drawLine(x + 2, yoff + 1, x + w - 4, yoff + 1);
            } else {
                int xOffset = 2;
                if (menuItemOption->checkType != QStyleOptionMenuItem::NotCheckable) {
                    QStyleOptionButton menuItemCheckOption;
                    menuItemCheckOption.state = menuItemOption->checked ? State_On : State_None;
                    if (menuItemOption->state & State_Enabled) {
                        menuItemCheckOption.state |= State_Enabled;
                        if (menuItemOption->state & State_Sunken)
                            menuItemCheckOption.state |= State_Sunken;
                    }
                    menuItemCheckOption.rect = QRect(menuItemOption->rect.x() + xOffset, menuItemOption->rect.y(), 20, menuItemOption->rect.height());
                    menuItemCheckOption.rect = visualRect(option->direction, menuItemOption->rect, menuItemCheckOption.rect);

                    if (menuItemOption->checkType & QStyleOptionMenuItem::Exclusive) {
                        proxy()->drawPrimitive(PE_IndicatorRadioButton, &menuItemCheckOption, painter, widget);
                    }
                    else {
                        menuItemCheckOption.palette.setBrush(QPalette::Active, QPalette::Button, QBrush(Qt::transparent));
                        menuItemCheckOption.palette.setBrush(QPalette::Shadow, QBrush(Qt::transparent));
                        menuItemCheckOption.palette.setBrush(QPalette::Mid, QBrush(Qt::transparent));
                        proxy()->drawPrimitive(PE_IndicatorCheckBox, &menuItemCheckOption, painter, widget);
                    }
                }

                if (menuItemOption->menuHasCheckableItems)
                    xOffset += 20;

                if (!menuItemOption->icon.isNull()) {
                    QIcon::Mode mode = disabled ? QIcon::Disabled : QIcon::Normal;
                    if (hover && !disabled)
                        mode = QIcon::Active;
                    QPixmap pixmap;
                    if (checked)
                        pixmap = menuItemOption->icon.pixmap(proxy()->pixelMetric(PM_SmallIconSize, option, widget), mode, QIcon::On);
                    else
                        pixmap = menuItemOption->icon.pixmap(proxy()->pixelMetric(PM_SmallIconSize, option, widget), mode);
                    QRect pixmapTarget(menuItemOption->rect.x() + xOffset, menuItemOption->rect.y(), menuItemOption->maxIconWidth, menuItemOption->rect.height());
                    drawItemPixmap(painter, pixmapTarget, int(visualAlignment(option->direction, Qt::AlignCenter)), pixmap);
                }

                xOffset += menuItemOption->maxIconWidth + 2;

                QString s = menuItemOption->text;
                if (!s.isEmpty()) {
                    QRect textRect(menuItemOption->rect.x() + xOffset, menuItemOption->rect.y(),
                                   menuItemOption->rect.width() - xOffset - tab - 10, menuItemOption->rect.height());
                    QRect vTextRect = visualRect(menuItemOption->direction, menuItemOption->rect, textRect);
                    int t = s.indexOf(QLatin1Char('\t'));
                    int textFlags = Qt::AlignVCenter | Qt::TextShowMnemonic | Qt::TextDontClip | Qt::TextSingleLine;
                    if (!proxy()->styleHint(SH_UnderlineShortcut, menuItemOption, widget))
                        textFlags |= Qt::TextHideMnemonic;
                    textFlags |= Qt::AlignLeft;
                    QFont font = menuItemOption->font;
                    if (menuItemOption->menuItemType == QStyleOptionMenuItem::DefaultItem)
                        font.setBold(true);
                    if (t >= 0) {
                        QRect vShortcutRect = visualRect(menuItemOption->direction, menuItemOption->rect, QRect(textRect.topRight(), QPoint(menuItemOption->rect.right() - 1, textRect.bottom())));
                        painter->drawText(vShortcutRect, textFlags, s.mid(t + 1));
                    }
                    painter->setPen(option->palette.windowText().color());
                    painter->setFont(font);
                    painter->drawText(vTextRect, textFlags, s.left(t));
                }

                if (menuItemOption->menuItemType == QStyleOptionMenuItem::SubMenu) {
                    PrimitiveElement arrow;
                    arrow = menuItemOption->direction == Qt::LeftToRight ? PE_IndicatorArrowRight : PE_IndicatorArrowLeft;
                    QStyleOptionMenuItem arrowOption = *menuItemOption;
                    QRect arrowRect(QPoint(menuItemOption->rect.right() - 10, menuItemOption->rect.top()), QPoint(menuItemOption->rect.right(), menuItemOption->rect.bottom()));
                    arrowOption.rect = visualRect(menuItemOption->direction, menuItemOption->rect, arrowRect);
                    arrowOption.state = disabled ? State_None : State_Enabled;
                    proxy()->drawPrimitive(arrow, &arrowOption, painter, widget);
                }
            }
            painter->restore();
        }
        break;
    } // CE_MenuItem
    case CE_HeaderSection: {
        if (const QStyleOptionHeader *headerOption = qstyleoption_cast<const QStyleOptionHeader *>(option)) {
            QLinearGradient gradient(0, 0, 0, option->rect.height());
            _setGrayGradient(gradient, option->palette);
            QPen borderPen = Qt::NoPen;
            Borders border = None;

            if (headerOption->position != QStyleOptionHeader::OnlyOneSection && headerOption->position != QStyleOptionHeader::End) {
                if (headerOption->orientation == Qt::Horizontal) {
                    if (headerOption->direction == Qt::RightToLeft)
                        border = Left;
                    else
                        border = Right;
                } else {
                    border = Bottom;
                }
                QLinearGradient borderGradient(0, 0, 0, option->rect.height());
                _setSeparatorGradient(borderGradient, headerOption->palette);
                borderPen = QPen(borderGradient, 1);
            }
            _drawPanel(painter, option->rect, QBrush(gradient), Qt::NoPen);
            if (border != None)
                _drawBorder(painter, option->rect, borderPen, border);
        }
        break;
    } // CE_HeaderSection
    case CE_TabBarTab: {
        QCommonStyle::drawControl(controlElement, option, painter, widget);
        break;
    } // CE_TabBarTab
    case CE_TabBarTabShape: {
        if (const QStyleOptionTab *tabOption = qstyleoption_cast<const QStyleOptionTab *>(option)) {
            painter->save();
            QBrush bgBrush;
            if (qobject_cast<const TopologyContainerTabBar *>(widget)) {
                if (tabOption->state.testFlag(State_Selected))
                    bgBrush = option->palette.button();
                else
                    bgBrush = option->palette.mid();
                int padding = proxy()->pixelMetric(PM_DefaultFrameWidth) + 1;
                _drawRoundedPanel(painter, tabOption->rect.adjusted(padding, padding, -padding, -padding), bgBrush, QPen(option->palette.shadow().color()), 5);
            } else {
                QRect rect = option->rect;
                int x1 = rect.left();
                int x2 = rect.right();
                int y1 = rect.top();
                int y2 = rect.bottom();
                QTransform rotMatrix;
                bool flip = false;

                switch (tabOption->shape) {
                case QTabBar::RoundedNorth:
                    break;
                case QTabBar::RoundedSouth:
                    rotMatrix.rotate(180);
                    rotMatrix.translate(0, - rect.height() + 1);
                    rotMatrix.scale(-1, 1);
                    painter->setTransform(rotMatrix, true);
                    break;
                case QTabBar::RoundedWest:
                    rotMatrix.rotate(180 + 90);
                    rotMatrix.scale(-1, 1);
                    flip = true;
                    painter->setTransform(rotMatrix, true);
                    break;
                case QTabBar::RoundedEast:
                    rotMatrix.rotate(90);
                    rotMatrix.translate(0, - rect.width() + 1);
                    flip = true;
                    painter->setTransform(rotMatrix, true);
                    break;
                default: //triangular
                    QCommonStyle::drawControl(controlElement, option, painter, widget);
                    painter->restore();
                    return;
                }

                if (flip) {
                    rect = QRect(rect.y(), rect.x(), rect.height(), rect.width());
                    int temp = x1;
                    x1 = y1;
                    y1 = temp;
                    temp = x2;
                    x2 = y2;
                    y2 = temp;
                }
                QBrush bgBrush;
                if (tabOption->state.testFlag(State_Selected))
                    bgBrush = option->palette.button();
                else
                    bgBrush = option->palette.mid();
                painter->setBrush(bgBrush);
                int radius = 10;
                int diameter = radius * 2;
                QPainterPath tabPath;
                painter->setPen(Qt::NoPen);
                painter->setRenderHint(QPainter::Antialiasing, true);
                tabPath.moveTo(x1, y1 + radius);
                tabPath.lineTo(x1, y2);
                tabPath.lineTo(x2, y2);
                tabPath.lineTo(x2, y1 - radius);
                tabPath.arcTo(x2 - diameter, y1, diameter, diameter, 0, 90);
                tabPath.lineTo(x1 + radius, y1);
                tabPath.arcTo(x1, y1, diameter, diameter, 90, 90);
                tabPath.closeSubpath();
                painter->drawPath(tabPath);
            }
        }
        painter->restore();
        break;
    } // CE_TabBarTabShape
    case CE_TabBarTabLabel: {
        if (const QStyleOptionTab *tabOption = qstyleoption_cast<const QStyleOptionTab *>(option)) {
            QStyleOptionTabV3 updatedOption = *tabOption;
            if (qobject_cast<const QTabBar*>(widget) && tabOption->state.testFlag(State_Selected)) {
                const QTabBar *tabBar = qobject_cast<const QTabBar*>(widget);
                if (!tabBar->tabTextColor(tabBar->currentIndex()).isValid())
                    updatedOption.palette.setBrush(QPalette::WindowText, tabOption->palette.shadow());
            }
            QCommonStyle::drawControl(controlElement, &updatedOption, painter, widget);
        }
        break;
    } // CE_TabBarTabLabel
    case CE_ShapedFrame: {
        if (qobject_cast<OverlayExpandableWidget *>(widget->parentWidget())) {
            QLinearGradient gradient(0, 0, 0, widget->height());
            _setGrayGradient(gradient, option->palette);
            _drawPanel(painter, option->rect, gradient);
        } else if (qobject_cast<const StatusBarLabel*>(widget)) {
            _drawRoundedPanel(painter, option->rect, option->palette.base(), Qt::NoPen, 5);
        } else
            QCommonStyle::drawControl(controlElement, option, painter, widget);
        break;
    }
    case CE_ProgressBarGroove: {
        //from QCleanLookStyle
        painter->save();
        {
            QRect rect = option->rect;
            QColor dark = option->palette.dark().color();
            painter->fillRect(rect, option->palette.base());
            QColor borderColor = dark.lighter(110);
            painter->setPen(QPen(borderColor, 0));
            const QLine lines[4] = {
                QLine(QPoint(rect.left() + 1, rect.top()), QPoint(rect.right() - 1, rect.top())),
                QLine(QPoint(rect.left() + 1, rect.bottom()), QPoint(rect.right() - 1, rect.bottom())),
                QLine(QPoint(rect.left(), rect.top() + 1), QPoint(rect.left(), rect.bottom() - 1)),
                QLine(QPoint(rect.right(), rect.top() + 1), QPoint(rect.right(), rect.bottom() - 1))
            };
            painter->drawLines(lines, 4);
            QColor alphaCorner = mergedColors(borderColor, option->palette.background().color());
            QColor innerShadow = mergedColors(borderColor, option->palette.base().color());

            //corner smoothing
            painter->setPen(alphaCorner);
            const QPoint points[4] = {
                rect.topRight(),
                rect.topLeft(),
                rect.bottomRight(),
                rect.bottomLeft()
            };
            painter->drawPoints(points, 4);

            //inner shadow
            painter->setPen(innerShadow);
            painter->drawLine(QPoint(rect.left() + 1, rect.top() + 1),
                              QPoint(rect.right() - 1, rect.top() + 1));
            painter->drawLine(QPoint(rect.left() + 1, rect.top() + 1),
                              QPoint(rect.left() + 1, rect.bottom() + 1));

        }
        painter->restore();
        break;
    } // CE_ProgressBarGroove

    case CE_ProgressBarContents: {
        //from QCleanLookStyle
        painter->save();
        if (const QStyleOptionProgressBar *bar = qstyleoption_cast<const QStyleOptionProgressBar *>(option)) {
            QRect rect = bar->rect;
            bool vertical = false;
            bool inverted = false;
            bool indeterminate = (bar->minimum == 0 && bar->maximum == 0);

            // Get extra style options if version 2
            if (const QStyleOptionProgressBarV2 *bar2 = qstyleoption_cast<const QStyleOptionProgressBarV2 *>(option)) {
                vertical = (bar2->orientation == Qt::Vertical);
                inverted = bar2->invertedAppearance;
            }

            // If the orientation is vertical, we use a transform to rotate
            // the progress bar 90 degrees clockwise.  This way we can use the
            // same rendering code for both orientations.
            if (vertical) {
                rect = QRect(rect.left(), rect.top(), rect.height(), rect.width()); // flip width and height
                QTransform m = QTransform::fromTranslate(rect.height()-1, -1.0);
                m.rotate(90.0);
                painter->setTransform(m, true);
            }

            int maxWidth = rect.width() - 4;
            int minWidth = 4;
            qreal progress = qMax(bar->progress, bar->minimum); // workaround for bug in QProgressBar
            int progressBarWidth = (progress - bar->minimum) * qreal(maxWidth) / qMax(qreal(1.0), qreal(bar->maximum) - bar->minimum);
            int width = indeterminate ? maxWidth : qMax(minWidth, progressBarWidth);

            bool reverse = (!vertical && (bar->direction == Qt::RightToLeft)) || vertical;
            if (inverted)
                reverse = !reverse;

            QRect progressBar;
            if (!indeterminate) {
                if (!reverse) {
                    progressBar.setRect(rect.left() + 1, rect.top() + 1, width + 1, rect.height() - 3);
                } else {
                    progressBar.setRect(rect.right() - 1 - width, rect.top() + 1, width + 1, rect.height() - 3);
                }
            } else {
                int slideWidth = ((rect.width() - 4) * 2) / 3;
                int step = ((_animateStep * slideWidth) / _animationFps) % slideWidth;
                if ((((_animateStep * slideWidth) / _animationFps) % (2 * slideWidth)) >= slideWidth)
                    step = slideWidth - step;
                progressBar.setRect(rect.left() + 1 + step, rect.top() + 1,
                                    slideWidth / 2, rect.height() - 3);


            }
            QColor highlight = option->palette.color(QPalette::Normal, QPalette::Highlight);
            painter->setPen(QPen(highlight.darker(140), 0));

            QColor highlightedGradientStartColor = highlight.lighter(100);
            QColor highlightedGradientStopColor  = highlight.lighter(130);

            QLinearGradient gradient(rect.topLeft(), QPoint(rect.bottomLeft().x(),
                                                            rect.bottomLeft().y()*2));
            gradient.setColorAt(0, highlightedGradientStartColor);
            gradient.setColorAt(1, highlightedGradientStopColor);

            painter->setBrush(gradient);
            painter->drawRect(progressBar);

            painter->setPen(QPen(highlight.lighter(120), 0));
            painter->drawLine(QPoint(progressBar.left() + 1, progressBar.top() + 1),
                              QPoint(progressBar.right(), progressBar.top() + 1));
            painter->drawLine(QPoint(progressBar.left() + 1, progressBar.top() + 1),
                              QPoint(progressBar.left() + 1, progressBar.bottom() - 1));

            painter->setPen(QPen(highlightedGradientStartColor, 7.0));//QPen(option->palette.highlight(), 3));

            painter->save();
            painter->setClipRect(progressBar.adjusted(2, 2, -1, -1));
            for (int x = progressBar.left() - 32; x < rect.right() ; x+=18) {
                painter->drawLine(x, progressBar.bottom() + 1, x + 23, progressBar.top() - 2);
            }
            painter->restore();

        } // CE_ProgressBarContents
        painter->restore();
        break;
    }

    case CE_ProgressBarLabel: {
        //from QCommonStyle
        if (const QStyleOptionProgressBar *pb = qstyleoption_cast<const QStyleOptionProgressBar *>(option)) {
            bool vertical = false;
            if (const QStyleOptionProgressBarV2 *pb2 = qstyleoption_cast<const QStyleOptionProgressBarV2 *>(option)) {
                vertical = (pb2->orientation == Qt::Vertical);
            }
            if (!vertical) {
                QPalette::ColorRole textRole = QPalette::Shadow;
                proxy()->drawItemText(painter, pb->rect, Qt::AlignCenter | Qt::TextSingleLine, pb->palette,
                             pb->state & State_Enabled, pb->text, textRole);
            }
        }
        break;
    }

    default:
        QCommonStyle::drawControl(controlElement, option, painter, widget);
    }
}

void HnvStyle::drawPrimitive(PrimitiveElement element, const QStyleOption *option, QPainter *painter, const QWidget *widget) const
{
    switch (element) {
    case PE_PanelScrollAreaCorner: {
        // Corner beetween scrollbar in scroll area
        _drawPanel(painter, option->rect, option->palette.dark());
        break;
    } // PE_PanelScrollAreaCorner
    case PE_FrameStatusBarItem:
        // Override used to remove sunken style
        break;
    case PE_FrameFocusRect: {
        break;
    } // PE_FrameFocusRect
    case PE_PanelStatusBar: {
        // TODO : try to remove
        QLinearGradient gradient(0, 0, 0, option->rect.height());
        _setGrayGradient(gradient, option->palette);
        _drawPanel(painter, option->rect, gradient);
        _drawBorder(painter, option->rect, option->palette.shadow().color(), Top);
        break;
    } // PE_PanelStatusBar
    case PE_PanelButtonTool: {
        if (option->state.testFlag(State_Enabled) && (option->state.testFlag(State_MouseOver) || option->state.testFlag(State_On))) {
            QBrush bgBrush;
            QPen borderPen(Qt::NoPen);
            QRect buttonRect = option->rect.adjusted(1, 1, -1, -1);
            if (option->state.testFlag(State_On) && qobject_cast<Hnv::ButtonListWidget*>(widget->parentWidget())) {
                QLinearGradient gradient(0, 0, widget->width(), 0);
                gradient.setColorAt(0.0, option->palette.button().color());
                gradient.setColorAt(1.0, option->palette.light().color());
                bgBrush = QBrush(gradient);
                borderPen = QPen(Qt::black);
            } else if (option->state.testFlag(State_MouseOver)) {
                QLinearGradient gradient(0, 0, option->rect.width(), 0);
                if (option->state.testFlag(State_Sunken)) {
                    gradient.setColorAt(0, option->palette.button().color());
                    gradient.setColorAt(0.5, Qt::transparent);
                    gradient.setColorAt(1, option->palette.button().color());
                }
                else {
                    _setSeparatorGradient(gradient, option->palette);
                }
                bgBrush = QBrush(gradient);
                borderPen = QPen(bgBrush, 1.0);
            } else {
                qreal x = qreal((option->rect.right()) - option->rect.left()) / 2;
                qreal y = qreal(option->rect.bottom() - option->rect.top()) / 2;
                QPointF centerPoint(x, y);
                QRadialGradient gradient(centerPoint, qMax(qreal(buttonRect.width()), qreal(buttonRect.height())) / 2);
                gradient.setColorAt(0.0, option->palette.button().color());
                gradient.setColorAt(0.5, option->palette.button().color().lighter());
                gradient.setColorAt(1.0, Qt::transparent);
                bgBrush = QBrush(gradient);
            }
            _drawPanel(painter, buttonRect, bgBrush, Qt::NoPen, true);
            _drawBorder(painter, buttonRect, borderPen, ButtonHighlight);
        }
        if (widget && widget->parentWidget() && qobject_cast<Hnv::WelcomeScreenWidget *>(widget->parentWidget()->parentWidget()))
            _drawRoundedPanel(painter, option->rect, Qt::transparent, QPen(Qt::black), 10, true);
        break;
    } // PE_PanelButtonTool
    case PE_FrameDefaultButton: {
        _drawRoundedPanel(painter, option->rect, option->palette.highlight(), Qt::NoPen, 10);
        break;
    } // PE_FrameDefaultButton
    case PE_PanelButtonCommand: {
        if (qobject_cast<const QPushButton*>(widget)) {
            const QStyleOptionButton *optionButton = qstyleoption_cast<const QStyleOptionButton *>(option);
            QLinearGradient gradient(0, 0, 0, option->rect.height());
            int invertBorder = 0;
            QPen border(Qt::NoPen);
            if (option->state.testFlag(State_On)) {
                gradient.setColorAt(0.0, option->palette.dark().color().darker());
                gradient.setColorAt(0.1, option->palette.dark().color().darker());
                gradient.setColorAt(0.9, option->palette.dark().color());
                gradient.setColorAt(1.0, option->palette.dark().color().lighter(150));
                invertBorder = 2;
            } else {
                if (option->state.testFlag(State_Sunken)) {
                    gradient.setColorAt(0.0, option->palette.dark().color());
                    gradient.setColorAt(1.0, option->palette.mid().color());
                }
                else if (option->state.testFlag(State_MouseOver)) {
                    gradient.setColorAt(0.0, option->palette.mid().color().lighter());
                    gradient.setColorAt(0.2, option->palette.mid().color());
                    gradient.setColorAt(0.8, option->palette.dark().color());
                    gradient.setColorAt(1.0, option->palette.dark().color().lighter());
                    invertBorder = 1;
                }
                else {
                    invertBorder = 1;
                    gradient.setColorAt(0.0, option->palette.mid().color());
                    gradient.setColorAt(1.0, option->palette.dark().color());
                }
            }
            int radius = 10;

            QRect panelRect = option->rect.adjusted(1, 1, -1, -1);
            _drawRoundedPanel(painter, panelRect, gradient, border, radius);

            if (optionButton->features.testFlag(QStyleOptionButton::DefaultButton))
                return;
            QRect rect = option->rect;
            qreal diameter = radius * 2;
            QPainterPath borderPathNorth;
            QPainterPath borderPathSouth;

            borderPathSouth.moveTo(rect.left() + radius, rect.bottom());
            borderPathSouth.arcTo(rect.left(), rect.bottom() - diameter, diameter, diameter, 270, -45);
            borderPathSouth.moveTo(rect.left() + radius, rect.bottom());
            borderPathSouth.lineTo(rect.right() - radius, rect.bottom());
            borderPathSouth.arcTo(rect.right() - diameter, rect.bottom() - diameter, diameter, diameter, 270, 90);
            borderPathSouth.lineTo(rect.right(), rect.top() + radius);
            borderPathSouth.arcTo(rect.right() - diameter, rect.top(), diameter, diameter, 0, 45);

            borderPathNorth.arcMoveTo(rect.right() - diameter, rect.top(), diameter, diameter, 90);
            borderPathNorth.arcTo(rect.right() - diameter, rect.top(), diameter, diameter, 90, -45);
            borderPathNorth.moveTo(rect.right() - radius, rect.top());
            borderPathNorth.lineTo(rect.left() + radius, rect.top());
            borderPathNorth.arcTo(rect.left(), rect.top(), diameter, diameter, 90, 90);
            borderPathNorth.lineTo(rect.left(), rect.bottom() - radius);
            borderPathNorth.arcTo(rect.left(), rect.bottom() - diameter, diameter, diameter, 180, 45);

            painter->save();

            painter->setRenderHint(QPainter::Antialiasing, true);

            if (invertBorder == 0)
                painter->setPen(QPen(option->palette.dark().color().darker()));
            else
                painter->setPen(Qt::NoPen);
            painter->drawPath(borderPathNorth);

            if (invertBorder == 1)
                painter->setPen(QPen(option->palette.dark().color().darker()));
            else if (invertBorder == 2)
                painter->setPen(QPen(option->palette.dark().color().lighter()));
            else
                painter->setPen(Qt::NoPen);
            painter->drawPath(borderPathSouth);

            painter->restore();

        } else {
            // TODO : to remove
            QCommonStyle::drawPrimitive(element, option, painter, widget);
        }
        break;
    } // PE_PanelButtonCommand
    case PE_PanelLineEdit: {
        QRect lineEditRect = option->rect;
        if (qobject_cast<const SideBarContentWidget *>(widget->parentWidget())) {
            _drawPanel(painter, lineEditRect, option->palette.dark(), QPen(option->palette.dark().color()));
            lineEditRect.adjust(5, 0, -5, -5);
        }
        if (qobject_cast<const QSpinBox *>(widget->parentWidget())) {
            _drawRoundedPanel(painter, lineEditRect, QBrush(Qt::transparent), QPen(Qt::NoPen));
            break;
        }
        _drawRoundedPanel(painter, lineEditRect, option->palette.base(), QPen(Qt::NoPen), 5);
        break;
    } // PE_PanelLineEdit
    case PE_IndicatorViewItemCheck: {
        QStyleOptionButton button;
        button.QStyleOption::operator=(*option);
        button.state &= ~State_MouseOver;
        proxy()->drawPrimitive(PE_IndicatorCheckBox, &button, painter, widget);
        break;
    } // PE_IndicatorViewItemCheck
    case PE_IndicatorCheckBox: {
        QBrush bgBrush;
        QPen border(option->palette.shadow().color());
        if (option->state.testFlag(State_Enabled)) {
            bgBrush = option->palette.button();
            if (option->state.testFlag(State_MouseOver))
                border = option->palette.highlight().color();
        } else
            bgBrush = option->palette.mid();
        QRect rect = option->rect.adjusted(0, 0, -1, -1); // TODO voir si on peux corriger dans la taille par défault
        _drawPanel(painter, rect, bgBrush, border);
        if (option->state.testFlag(State_On)) {
            QPixmap img(Ressources::themedPath(Constants::ICON_CHECK_INDICATOR));
            if (img.width() > option->rect.width() ||img.height() > option->rect.height())
                img = img.scaled(option->rect.size());
            drawItemPixmap(painter, option->rect, int(visualAlignment(option->direction, Qt::AlignCenter)), img);
        }
        else if (option->state.testFlag(State_NoChange)) { //tri_state
            QPixmap pix(Ressources::themedPath(Constants::ICON_CHECK_INDICATOR));
            QImage img = pix.toImage();
            if (img.width() > option->rect.width() ||img.height() > option->rect.height())
                img = img.scaled(option->rect.size());
            QLinearGradient grad(0, 0, 0, img.height());
            grad.setColorAt(0, Qt::transparent);
            grad.setColorAt(1, QColor(0, 0, 0, 125));

            QPainter p(&img);
            p.fillRect(0, 0, img.width(), img.height(), grad);

            grad.setColorAt(0, QColor(0, 0, 0, 125));
            grad.setColorAt(1, Qt::transparent);
            p.fillRect(0, 0, img.width(), img.height(), grad);
            p.end();
            drawItemPixmap(painter, option->rect, int(visualAlignment(option->direction, Qt::AlignCenter)), QPixmap::fromImage(img));
        }
        break;
    } // PE_IndicatorCheckBox
    case PE_IndicatorRadioButton : {
        painter->save();
        QBrush bgBrush;
        QPen border(Qt::NoPen);
        if (option->state.testFlag(State_Enabled)) {
            bgBrush = option->palette.button();
            if (option->state.testFlag(State_MouseOver))
                border = option->palette.highlight().color();
        }
        else
            bgBrush = option->palette.mid();
        painter->setRenderHint(QPainter::Antialiasing, true);
        painter->setPen(border);
        painter->setBrush(bgBrush);

        QRect rect = option->rect;
        QPoint checkCenter = option->rect.center();
        rect.setSize(QSize(12, 12));
        rect.moveCenter(checkCenter);

        painter->drawEllipse(rect);
        if (option->state & (State_On )) {
            painter->setRenderHint(QPainter::SmoothPixmapTransform);
            painter->drawImage(rect, QImage(qt_cleanlooks_radiobutton_checked));
        } else if (option->state & State_Sunken) {
                painter->setBrush(option->palette.mid().color().darker(110));
                painter->setPen(Qt::NoPen);
                painter->drawEllipse(rect.adjusted(1, 1, -1, -1));
        }
        painter->restore();
        break;
    } // PE_IndicatorRadioButton
    case PE_IndicatorArrowLeft:
        _drawIndicator(painter, option, QImage(qt_scrollbar_button_arrow_left));
        break;
    case PE_IndicatorArrowRight:
        _drawIndicator(painter, option, QImage(qt_scrollbar_button_arrow_right));
        break;
    case PE_IndicatorArrowUp:
        _drawIndicator(painter, option, QImage(qt_scrollbar_button_arrow_up));
        break;
    case PE_IndicatorArrowDown:
        _drawIndicator(painter, option, QImage(qt_scrollbar_button_arrow_down));
        break;
    case PE_IndicatorHeaderArrow: {
        if (const QStyleOptionHeader *header = qstyleoption_cast<const QStyleOptionHeader *>(option)) {
            QImage arrow;
            if (header->sortIndicator & QStyleOptionHeader::SortUp)
                arrow = QImage(qt_cleanlooks_arrow_up_xpm);
            else if (header->sortIndicator & QStyleOptionHeader::SortDown)
                arrow = QImage(qt_cleanlooks_arrow_down_xpm);
            _drawIndicator(painter, option, arrow);
        }
        break;
    } // PE_IndicatorHeaderArrow
    case PE_Frame: {
        if (qobject_cast<const QTextEdit *>(widget))
            _drawRoundedPanel(painter, option->rect, option->palette.base(), QPen(Qt::NoPen), 5);
        else
            QCommonStyle::drawPrimitive(element, option, painter, widget);
        break;
    } // PE_Frame
    case PE_IndicatorBranch: {
        // This is _way_ too similar to the common style.
        static const int decoration_size = 9;
        int mid_h = option->rect.x() + option->rect.width() / 2;
        int mid_v = option->rect.y() + option->rect.height() / 2;
        int bef_h = mid_h;
        int bef_v = mid_v;
        int aft_h = mid_h;
        int aft_v = mid_v;
        if (option->state & State_Children) {
            int delta = decoration_size / 2;
            bef_h -= delta;
            bef_v -= delta;
            aft_h += delta;
            aft_v += delta;
            painter->drawLine(bef_h + 2, bef_v + 4, bef_h + 6, bef_v + 4);
            if (!(option->state & State_Open))
                painter->drawLine(bef_h + 4, bef_v + 2, bef_h + 4, bef_v + 6);
            QPen oldPen = painter->pen();
            painter->setPen(option->palette.dark().color());
            painter->drawRect(bef_h, bef_v, decoration_size - 1, decoration_size - 1);
            painter->setPen(oldPen);
        }
        QBrush brush(option->palette.dark().color(), Qt::Dense4Pattern);
        if (option->state & State_Item) {
            if (option->direction == Qt::RightToLeft)
                painter->fillRect(option->rect.left(), mid_v, bef_h - option->rect.left(), 1, brush);
            else
                painter->fillRect(aft_h, mid_v, option->rect.right() - aft_h + 1, 1, brush);
        }
        if (option->state & State_Sibling)
            painter->fillRect(mid_h, aft_v, 1, option->rect.bottom() - aft_v + 1, brush);
        if (option->state & (State_Open | State_Children | State_Item | State_Sibling))
            painter->fillRect(mid_h, option->rect.y(), 1, bef_v - option->rect.y(), brush);
        break;
    } // PE_IndicatorBranch
    case PE_FrameTabBarBase: {
        painter->fillRect(option->rect, proxy()->standardPalette().window());
        break;
    }
    case PE_FrameTabWidget: {
        _drawBorder(painter, option->rect, option->palette.light().color(), Left | Bottom | Right);
        QRect tabWidgetTabBarRect = proxy()->subElementRect(SE_TabWidgetTabBar, option, widget);
        if (tabWidgetTabBarRect.right() < option->rect.right()) {
            painter->save();
            painter->setBrush(option->palette.light());
            painter->drawLine(QPoint(tabWidgetTabBarRect.right(), option->rect.top()), QPoint(option->rect.right(), option->rect.top()));
            painter->restore();
        }
        break;
    }
    case PE_IndicatorTabClose: {
        QRect rect = option->rect.adjusted(5,0,0,0);
        if (option->state & State_Sunken) {
            QPixmap img(Ressources::themedPath(Constants::ICON_DARK_CLOSE_BUTTON));
            _drawCenteredImage(painter, rect, img.toImage());
        }
        else {
            QPixmap img(Ressources::themedPath(Constants::ICON_CLOSE_BUTTON));
            _drawCenteredImage(painter, rect, img.toImage());
        }
        break;
    }
    case PE_FrameGroupBox: {
        if (const QStyleOptionFrameV2 *frame = qstyleoption_cast<const QStyleOptionFrameV2*>(option)) {
            if (frame->features & QStyleOptionFrameV3::Flat) {
                QRect fr = frame->rect;
                QPoint p1(fr.x(), fr.y());
                QPoint p2(fr.x() + fr.width(), p1.y());
                qDrawShadeLine(painter, p1, p2, frame->palette, true, frame->lineWidth, frame->midLineWidth);
            }
            else {
                QRect fr = frame->rect;
                _drawBorder(painter, fr, option->palette.light().color());
            }
        }
        break;
    }
    case PE_FrameMenu: {
        _drawBorder(painter, option->rect, Qt::black);
        break;
    }
    case PE_IndicatorButtonDropDown: {
        if ((option->state & State_Sunken) | (option->state & State_MouseOver)) {
            QLinearGradient gradient(option->rect.left(), 0, option->rect.right(), 0);
            gradient.setColorAt(0.0, Qt::transparent);
            gradient.setColorAt(1.0, option->palette.button().color());
            QBrush bgBrush = QBrush(gradient);
            _drawPanel(painter, option->rect.adjusted(0, 1, -1, -1), bgBrush, Qt::NoPen);
            _drawBorder(painter, option->rect.adjusted(0, 1, -1, -1), option->palette.button().color(), Top | Left);
        }
        break;
    }
    default: {
        if (qobject_cast<const Hnv::ButtonListWidget*>(widget)) {
            QLinearGradient gradient(widget->width(), 0, 0, 0);
            _setGrayGradient(gradient, option->palette);
            _drawPanel(painter, option->rect, gradient);
            _drawBorder(painter, option->rect, option->palette.shadow().color(), Right);
        } else if (qobject_cast<const OverlayExpandableWidget *>(widget)) {
            _drawBorder(painter, option->rect, Qt::black);
        } else
            QCommonStyle::drawPrimitive(element, option, painter, widget);
        break;
    } // default
    } // switch (element)
}

int HnvStyle::pixelMetric(PixelMetric metric, const QStyleOption *option, const QWidget *widget) const
{
    int pm;
    switch (metric) {
    case PM_MenuBarPanelWidth:
        pm = 0;
        break;
    case PM_ButtonShiftHorizontal:
    case PM_ButtonShiftVertical:
        pm = 0;
        break;
    case PM_ToolBarItemSpacing:
        pm = 2;
        break;
    case PM_SliderLength:
        pm = 14;
        break;
    case PM_SliderControlThickness:
        pm = 8;
        break;
    case PM_ButtonIconSize:
        if (qobject_cast<const QPushButton*>(widget))
            pm = 16;
        else
            pm = 24;
        break;
    case PM_IndicatorWidth:
    case PM_IndicatorHeight:
        pm = 14;
        break;
    case PM_TabBarTabShiftVertical:
        pm = 0;
        break;
    default:
        pm = QCommonStyle::pixelMetric(metric, option, widget);
    } // switch (metric)
    return pm;
}

void HnvStyle::polish(QWidget *widget)
{
    if ((qobject_cast<Hnv::ModeBrowserWidget*>(widget)) || (qobject_cast<Hnv::ModeBrowserWidget*>(widget->parentWidget()))) {
        QFont widgetFont(widget->font());
        widgetFont.setBold(true);
        widgetFont.setPixelSize(10);
        widget->setFont(widgetFont);
    } else if (qobject_cast<Hnv::TopologyWidget*>(widget)) {
        QPalette widgetPalette(widget->palette());
        widgetPalette.setBrush(QPalette::Window, widgetPalette.midlight());
        widget->setPalette(widgetPalette);
    } else if (qobject_cast<QScrollArea *>(widget) && qobject_cast<LoadBalancerDialog*>(widget->parentWidget())) {
        QPalette widgetPalette(widget->palette());
        widgetPalette.setBrush(QPalette::Window, Qt::transparent);
        widget->setPalette(widgetPalette);
    } else if (qobject_cast<QScrollArea *>(widget)) {
        widget->setBackgroundRole(QPalette::Window);
        QPalette palette = widget->palette();
        palette.setColor(QPalette::Light, Qt::transparent);
        widget->setPalette(palette);
    } else if (qobject_cast<StatusBarLabel *>(widget)) {
        widget->setForegroundRole(QPalette::Text);
    } else if (qobject_cast<QComboBox*>(widget) && qobject_cast<QToolBar *>(widget->parentWidget())) {
        QPalette palette = widget->palette();
        QColor borderColor = Qt::gray;
        palette.setBrush(QPalette::Base, Qt::gray);
        palette.setBrush(QPalette::Window, borderColor);
        palette.setBrush(QPalette::Foreground, borderColor);
        widget->setPalette(palette);
    } else {
        QCommonStyle::polish(widget);
        if (qobject_cast<QAbstractButton*>(widget)
    #ifndef QT_NO_COMBOBOX
            || qobject_cast<QComboBox *>(widget)
    #endif
    #ifndef QT_NO_PROGRESSBAR
            || qobject_cast<QProgressBar *>(widget)
    #endif
    #ifndef QT_NO_SCROLLBAR
            || qobject_cast<QScrollBar *>(widget)
    #endif
    #ifndef QT_NO_SPLITTER
            || qobject_cast<QSplitterHandle *>(widget)
    #endif
            || qobject_cast<QAbstractSlider *>(widget)
    #ifndef QT_NO_SPINBOX
            || qobject_cast<QAbstractSpinBox *>(widget)
    #endif
            || (widget->inherits("QWorkspaceChild"))
            || (widget->inherits("QDockSeparator"))
            || (widget->inherits("QDockWidgetSeparator"))
            ) {
            widget->setAttribute(Qt::WA_Hover, true);
        }
    #ifndef QT_NO_PROGRESSBAR
        if (qobject_cast<QProgressBar *>(widget))
            widget->installEventFilter(this);
    #endif
    }
}

QSize HnvStyle::sizeFromContents(ContentsType type, const QStyleOption *option, const QSize &size, const QWidget *widget) const
{
    QSize newSize = QCommonStyle::sizeFromContents(type, option, size, widget);
    switch (type) {
    case CT_MenuItem: {
        if (const QStyleOptionMenuItem *menuItemOption = qstyleoption_cast<const QStyleOptionMenuItem *>(option)) {
            bool checkable = menuItemOption->menuHasCheckableItems;
            int maxIconWidth = menuItemOption->maxIconWidth;
            int w = size.width();
            int h = size.height();
            if (menuItemOption->menuItemType == QStyleOptionMenuItem::Separator) {
                w = 10;
                h = 9;
            } else {
                h =  menuItemOption->fontMetrics.height() + 10;
                if (!menuItemOption->icon.isNull()) {
                    int iconExtent = proxy()->pixelMetric(PM_SmallIconSize);
                    h = qMax(h, menuItemOption->icon.actualSize(QSize(iconExtent, iconExtent)).height() + 4);
                }
            }
            if (menuItemOption->text.contains(QLatin1Char('\t')))
                w += 12;
            if (maxIconWidth > 0)
                w += maxIconWidth + 6;
            if (checkable)
                w += 20;
            if (checkable || maxIconWidth > 0)
                w += 2;
            w += 22;
            newSize = QSize(w, h);
        }
        break;
    } // CT_MenuItem
    case CT_MenuBarItem: {
        if (!newSize.isEmpty())
            newSize += QSize(12, 4);
        break;
    } // CT_MenuBarItem
    case CT_LineEdit: {
        // Add margins around FilterLineEdit in EntityCatalogWidget
        if (qobject_cast<SideBarContentWidget *>(widget->parentWidget()))
            newSize += QSize(10, 5);
        break;
    } // CT_LineEdit
    case CT_PushButton: {
        if (newSize.height() < 32)
            newSize.setHeight(32);
        newSize += QSize(4, 0);
        break;
    } // CT_PushButton
    case CT_ToolButton: {
        if (widget && qobject_cast<QToolBar *>(widget->parentWidget()))
            newSize += QSize(1, 2);
        break;
    } // CT_ToolButton
    default:
        break;
    }
    return newSize;
}

QPalette HnvStyle::standardPalette() const
{
    QPalette palette = QCommonStyle::standardPalette();
    palette.setColor(QPalette::Active, QPalette::Window, QColor(90, 90, 90, 255));
    palette.setColor(QPalette::Inactive, QPalette::Window, QColor(90, 90, 90, 255));
    palette.setColor(QPalette::Disabled, QPalette::Window, QColor(90, 90, 90, 255));

    palette.setColor(QPalette::Active, QPalette::WindowText, Qt::white);
    palette.setColor(QPalette::Inactive, QPalette::WindowText, Qt::lightGray);
    palette.setColor(QPalette::Disabled, QPalette::WindowText, Qt::gray);

    palette.setColor(QPalette::Base, Qt::white);
    palette.setColor(QPalette::AlternateBase, QColor(250, 250, 255));

    palette.setColor(QPalette::Inactive, QPalette::ToolTipBase, QColor(50, 50, 50, 200));
    palette.setColor(QPalette::Inactive, QPalette::ToolTipText, Qt::white);

    palette.setColor(QPalette::Active, QPalette::Text, Qt::black);
    palette.setColor(QPalette::Inactive, QPalette::Text, Qt::black);
    palette.setColor(QPalette::Disabled, QPalette::Text, Qt::gray);

    palette.setColor(QPalette::Active, QPalette::ButtonText, Qt::white);
    palette.setColor(QPalette::Inactive, QPalette::ButtonText, Qt::lightGray);
    palette.setColor(QPalette::Disabled, QPalette::ButtonText, Qt::gray);

    palette.setColor(QPalette::Active, QPalette::BrightText, Qt::white);
    palette.setColor(QPalette::Inactive, QPalette::BrightText, Qt::lightGray);
    palette.setColor(QPalette::Disabled, QPalette::BrightText, Qt::gray);

    palette.setColor(QPalette::Active, QPalette::Light, QColor(253, 253, 253));
    palette.setColor(QPalette::Inactive, QPalette::Light, QColor(253, 253, 253));
    palette.setColor(QPalette::Disabled, QPalette::Light, QColor(253, 253, 253));

    palette.setColor(QPalette::Active, QPalette::Midlight, QColor(226, 223, 221));
    palette.setColor(QPalette::Inactive, QPalette::Midlight, QColor(226, 223, 221));
    palette.setColor(QPalette::Disabled, QPalette::Midlight, QColor(226, 223, 221));

    palette.setColor(QPalette::Active, QPalette::Button, QColor(179, 171, 167, 255));
    palette.setColor(QPalette::Inactive, QPalette::Button, QColor(179, 171, 167, 255));
    palette.setColor(QPalette::Disabled, QPalette::Button, QColor(179, 171, 167, 255));

    palette.setColor(QPalette::Active, QPalette::Mid, QColor(120, 120, 120, 255));
    palette.setColor(QPalette::Inactive, QPalette::Mid, QColor(120, 120, 120, 255));
    palette.setColor(QPalette::Disabled, QPalette::Mid, QColor(120, 120, 120, 255));

    palette.setColor(QPalette::Active, QPalette::Dark, QColor(90, 90, 90, 255));
    palette.setColor(QPalette::Inactive, QPalette::Dark, QColor(90, 90, 90, 255));
    palette.setColor(QPalette::Disabled, QPalette::Dark, QColor(90, 90, 90, 255));

    palette.setColor(QPalette::Active, QPalette::Shadow, Qt::black);
    palette.setColor(QPalette::Inactive, QPalette::Shadow, Qt::black);
    palette.setColor(QPalette::Disabled, QPalette::Shadow, Qt::black);

    palette.setColor(QPalette::Active, QPalette::Highlight, QColor(60, 160, 255, 255));
    palette.setColor(QPalette::Inactive, QPalette::Highlight, QColor(80, 190, 255, 255));
    palette.setColor(QPalette::Disabled, QPalette::Highlight, QColor(80, 190, 255, 255));

    palette.setColor(QPalette::Active, QPalette::HighlightedText, Qt::white);
    palette.setColor(QPalette::Inactive, QPalette::HighlightedText, Qt::white);
    palette.setColor(QPalette::Disabled, QPalette::HighlightedText, Qt::white);

    palette.setColor(QPalette::Active, QPalette::Link, Qt::blue);
    palette.setColor(QPalette::Inactive, QPalette::Link, Qt::blue);
    palette.setColor(QPalette::Disabled, QPalette::Link, Qt::gray);

    palette.setColor(QPalette::Active, QPalette::LinkVisited, Qt::magenta);
    palette.setColor(QPalette::Inactive, QPalette::LinkVisited, Qt::magenta);
    palette.setColor(QPalette::Disabled, QPalette::LinkVisited, Qt::gray);

    return palette;
}

int HnvStyle::styleHint(StyleHint hint, const QStyleOption *option, const QWidget *widget, QStyleHintReturn *returnData) const
{
    int result;
    switch (hint) {
    case SH_Menu_MouseTracking:
    case SH_MenuBar_MouseTracking:
        result = 1;
        break;
    case SH_DialogButtonBox_ButtonsHaveIcons:
        result = true;
        break;
    case SH_LineEdit_PasswordCharacter:
        result = 0x25CF;
        break;
    case SH_ComboBox_ListMouseTracking:
        result = true;
        break;
    default:
        result = QCommonStyle::styleHint(hint, option, widget, returnData);
        break;
    }
    return result;
}

QRect HnvStyle::subControlRect(ComplexControl complexControl, const QStyleOptionComplex *option, SubControl subControl, const QWidget *widget) const
{
    QRect scRect;
    switch (complexControl) {
    case CC_Slider: {
        scRect = QCommonStyle::subControlRect(complexControl, option, subControl, widget);
        if (const QStyleOptionSlider *sliderOption = qstyleoption_cast<const QStyleOptionSlider *>(option)) {
            switch (subControl) {
            case SC_SliderGroove: {
                QPoint grooveCenter = scRect.center();
                if (sliderOption->orientation == Qt::Horizontal) {
                    grooveCenter.ry() = sliderOption->rect.center().ry();
                    scRect.setHeight(2);
                } else {
                    scRect.setWidth(2);
                    grooveCenter.rx() = sliderOption->rect.center().rx();
                }
                scRect.moveCenter(grooveCenter);
                break;
            }
            case SC_SliderHandle: {
                QPoint handleCenter = scRect.center();
                if (sliderOption->orientation == Qt::Horizontal) {
                    handleCenter.ry() = sliderOption->rect.center().ry();
                } else {
                    handleCenter.rx() = sliderOption->rect.center().rx();
                }
                scRect.moveCenter(handleCenter);
                break;
            }
            default:
                break;
            }
        }
        break;
    }
    default:
        break;
    }
    if (scRect.isNull())
        scRect = QCommonStyle::subControlRect(complexControl, option, subControl, widget);
    return scRect;
}

QRect HnvStyle::subElementRect(SubElement element, const QStyleOption *option, const QWidget *widget) const
{
    QRect subRect;
    switch (element) {
        case SE_PushButtonFocusRect:
            subRect = QCommonStyle::subElementRect(element, option, widget).adjusted(0, 1, 0, -1);
        break;
    case SE_LineEditContents:
        subRect = QCommonStyle::subElementRect(element, option, widget);
        if (qobject_cast<SideBarContentWidget *>(widget->parentWidget()))
            subRect.adjust(5, 0, -5, -5);
        break;
    case SE_ShapedFrameContents:
        subRect = QCommonStyle::subElementRect(element, option, widget);
        if (qobject_cast<const StatusBarLabel *>(widget)) {
            subRect.adjust(2, 0, -2, 0);
        }
    default:
        break;
    }
    if (subRect.isNull())
        subRect = QCommonStyle::subElementRect(element, option, widget);
    return subRect;
}

void HnvStyle::unpolish(QWidget *widget)
{
    QCommonStyle::unpolish(widget);
    if (qobject_cast<QAbstractButton*>(widget)
#ifndef QT_NO_COMBOBOX
        || qobject_cast<QComboBox *>(widget)
#endif
#ifndef QT_NO_PROGRESSBAR
        || qobject_cast<QProgressBar *>(widget)
#endif
#ifndef QT_NO_SCROLLBAR
        || qobject_cast<QScrollBar *>(widget)
#endif
#ifndef QT_NO_SPLITTER
        || qobject_cast<QSplitterHandle *>(widget)
#endif
        || qobject_cast<QAbstractSlider *>(widget)
#ifndef QT_NO_SPINBOX
        || qobject_cast<QAbstractSpinBox *>(widget)
#endif
        || (widget->inherits("QWorkspaceChild"))
        || (widget->inherits("QDockSeparator"))
        || (widget->inherits("QDockWidgetSeparator"))
        ) {
        widget->setAttribute(Qt::WA_Hover, false);
    }
}

QIcon HnvStyle::standardIconImplementation(StandardPixmap standardIcon, const QStyleOption *option, const QWidget *widget) const
{
    QIcon icon;
    switch(standardIcon) {
    case SP_DialogOkButton :
    case SP_DialogYesButton : {
        icon = QIcon(Ressources::themedPath(Constants::ICON_CHECK));
        break;
    }
    case SP_DialogNoButton :
    case SP_DialogCancelButton :
    case SP_DialogCloseButton :{
        icon = QIcon(Ressources::themedPath(Constants::ICON_CANCEL));
        break;
    }
    case SP_DialogOpenButton : {
        icon = QIcon(Ressources::themedPath(Constants::ICON_OPEN));
        break;
    }
    case SP_DialogSaveButton : {
        icon = QIcon(Ressources::themedPath(Constants::ICON_SAVE));
        break;
    }
    case SP_MessageBoxQuestion : {
        icon = QIcon(Ressources::themedPath(Constants::ICON_HELP));
        break;
    }
    case SP_MessageBoxWarning : {
        icon = QIcon(Ressources::themedPath(Constants::ICON_WARNING));
        break;
    }
    default :
        icon = QCommonStyle::standardIconImplementation(standardIcon, option, widget);
        break;
    }
    return icon;
}

void HnvStyle::_drawIndicator(QPainter *painter, const QStyleOption *option, QImage image) const
{
    QColor imgColor(option->palette.buttonText().color());
    _drawIndicator(painter, option, image, imgColor, option->rect);
}

void HnvStyle::_drawIndicator(QPainter *painter, const QStyleOption *option, QImage image, QColor imgColor, QRect imgRect) const
{
    if (option->state.testFlag(State_Sunken))
        imgColor = imgColor.lighter();
    image.setColor(1, imgColor.rgba());
    _drawCenteredImage(painter, imgRect, image);
}

void HnvStyle::_drawPanel(QPainter *painter, const QRect &rect, const QBrush &brush, bool enableAntialiasing) const
{
    _drawPanel(painter, rect, brush, QPen(Qt::NoPen), enableAntialiasing);
}

void HnvStyle::_drawPanel(QPainter *painter, const QRect &rect, const QBrush &brush, const QPen &borderPen, bool enableAntialiasing) const
{
    painter->save();
    painter->setRenderHint(QPainter::Antialiasing, enableAntialiasing);
    painter->setPen(borderPen);
    painter->setBrush(brush);
    painter->drawRect(rect);
    painter->restore();
}

void HnvStyle::_drawRoundedPanel(QPainter *painter, const QRect &rect, const QBrush &brush, const QPen &borderPen, int radius, bool enableAntialiasing) const
{
    if (radius < 0)
        radius = qMin(rect.height(), rect.width()) / 4;
    painter->save();
    painter->setRenderHint(QPainter::Antialiasing, enableAntialiasing);
    painter->setBrush(brush);
    painter->setPen(borderPen);
    painter->drawRoundedRect(rect, radius, radius);
    painter->restore();
}

void HnvStyle::_drawBorder(QPainter *painter, const QRect &rect, const QPen &pen, Borders border) const
{
    painter->save();
    painter->setPen(pen);
    painter->setBrush(Qt::NoBrush);
    if (border.testFlag(Top))
        painter->drawLine(rect.topLeft(), rect.topRight());
    if (border.testFlag(Bottom))
        painter->drawLine(rect.bottomLeft(), rect.bottomRight());
    if (border.testFlag(Left))
        painter->drawLine(rect.topLeft(), rect.bottomLeft());
    if (border.testFlag(Right))
        painter->drawLine(rect.topRight(), rect.bottomRight());
    painter->restore();
}

void HnvStyle::_drawBorder(QPainter *painter, const QRect &rect, Qt::GlobalColor color, Borders border) const
{
    _drawBorder(painter, rect, QPen(color), border);
}

void HnvStyle::_drawCenteredImage(QPainter *painter, const QRect &rect, const QImage &image) const
{
    QRect targetRect = image.rect();
    targetRect.moveCenter(rect.center());
    painter->drawImage(targetRect, image);
}

void HnvStyle::_drawArrow(const QStyleOptionToolButton *toolbutton, QPainter *painter, const QWidget *widget) const
{
    QStyle::PrimitiveElement pe;
    switch (toolbutton->arrowType) {
    case Qt::LeftArrow:
        pe = QStyle::PE_IndicatorArrowLeft;
        break;
    case Qt::RightArrow:
        pe = QStyle::PE_IndicatorArrowRight;
        break;
    case Qt::UpArrow:
        pe = QStyle::PE_IndicatorArrowUp;
        break;
    case Qt::DownArrow:
        pe = QStyle::PE_IndicatorArrowDown;
        break;
    default:
        return;
    }
    proxy()->drawPrimitive(pe, toolbutton, painter, widget);
}

void HnvStyle::_setGrayGradient(QLinearGradient &gradient, QPalette palette) const
{
    gradient.setColorAt(0.0, palette.mid().color());
    gradient.setColorAt(0.2, palette.mid().color());
    gradient.setColorAt(0.8, palette.dark().color());
    gradient.setColorAt(1.0, palette.dark().color());
}

void HnvStyle::_setSeparatorGradient(QLinearGradient &gradient, QPalette palette) const
{
    gradient.setColorAt(0.0, Qt::transparent);
    gradient.setColorAt(0.5, palette.button().color());
    gradient.setColorAt(1.0, Qt::transparent);
}

bool HnvStyle::eventFilter(QObject *obj, QEvent *event)
{
    if (!obj->isWidgetType())
        return QObject::eventFilter(obj, event);

    switch(event->type()) {
#ifndef QT_NO_PROGRESSBAR
    case QEvent::StyleChange:
    case QEvent::Paint:
    case QEvent::Show:
        if (QProgressBar *bar = qobject_cast<QProgressBar *>(obj)) {
            // Animation by timer for progress bars that have their min and
            // max values the same
            if (bar->minimum() == bar->maximum())
                _startAnimation(this, bar);
            else
                _stopAnimation(this, bar);
        }
        break;
    case QEvent::Destroy:
    case QEvent::Hide:
        // Do static_cast because there is no type info when getting
        // the destroy event. We know that it is a QProgressBar, since
        // we only install a widget event filter for QScrollBars.
        _stopAnimation(this, static_cast<QProgressBar *>(obj));
        break;
#endif // QT_NO_PROGRESSBAR
    default:
        break;
    }
    return QCommonStyle::eventFilter(obj, event);
}


void HnvStyle::timerEvent(QTimerEvent *event)
{
#ifndef QT_NO_PROGRESSBAR
    if (event->timerId() == _animateTimer) {
        Q_ASSERT(_animationFps > 0);
        _animateStep = _startTime.elapsed() / (1000 / _animationFps);
        foreach (QProgressBar *bar, _animatedProgressBars)
            bar->update();
    }
#endif // QT_NO_PROGRESSBAR
    event->ignore();
}

void HnvStyle::_startAnimation(QObject *obj, QProgressBar *bar)
{
    if (!_animatedProgressBars.contains(bar)) {
        _animatedProgressBars << bar;
        if (!_animateTimer) {
            Q_ASSERT(_animationFps > 0);
            _animateTimer = obj->startTimer(1000 / _animationFps);
        }
    }
}

void HnvStyle::_stopAnimation(QObject *obj, QProgressBar *bar)
{
    _animatedProgressBars.removeAll(bar);
    if (_animatedProgressBars.isEmpty() && _animateTimer) {
        obj->killTimer(_animateTimer);
        _animateTimer = 0;
    }
}

} // namespace Hnv
