/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef HNVSTYLE_H
#define HNVSTYLE_H

// Qt headers
#include <QCommonStyle>
#include <QPalette>
#include <QElapsedTimer>

class QStyleOptionToolButton;
class QProgressBar;

namespace Hnv {

class HnvStyle : public QCommonStyle
{
    Q_OBJECT
public:

    enum Border {
        None = 0x0,
        Top = 0x1,
        Bottom = 0x2,
        Left = 0x4,
        Right = 0x8,
        All = Top | Bottom | Left | Right,
        ButtonHighlight = Top | Bottom
    };
    Q_DECLARE_FLAGS(Borders, Border)

    HnvStyle();
    ~HnvStyle();

    void drawComplexControl(ComplexControl control, const QStyleOptionComplex *option, QPainter *painter, const QWidget *widget) const;
    void drawControl(ControlElement controlElement, const QStyleOption *option, QPainter *painter, const QWidget *widget) const;
    void drawPrimitive(PrimitiveElement element, const QStyleOption *option, QPainter *painter, const QWidget *widget = 0) const;
    int pixelMetric(PixelMetric metric, const QStyleOption *option = 0, const QWidget *widget = 0) const;
    void polish(QWidget *widget);
    QSize sizeFromContents(ContentsType type, const QStyleOption *option, const QSize &size, const QWidget *widget) const;
    QPalette standardPalette() const;
    int styleHint(StyleHint hint, const QStyleOption *option, const QWidget *widget = 0, QStyleHintReturn *returnData = 0) const;
    QRect subControlRect(ComplexControl complexControl, const QStyleOptionComplex *option, SubControl subControl, const QWidget *widget) const;
    QRect subElementRect(SubElement element, const QStyleOption *option, const QWidget *widget = 0) const;
    void unpolish(QWidget *widget);

protected slots:
    QIcon standardIconImplementation(StandardPixmap standardIcon, const QStyleOption *option, const QWidget *widget) const;

protected:
    void _drawIndicator(QPainter *painter, const QStyleOption *option, QImage image) const;
    void _drawIndicator(QPainter *painter, const QStyleOption *option, QImage image, QColor imgColor, QRect imgRect) const;
    void _drawPanel(QPainter *painter, const QRect &rect, const QBrush &brush, bool enableAntialiasing = false) const;
    void _drawPanel(QPainter *painter, const QRect &rect, const QBrush &brush, const QPen &borderPen, bool enableAntialiasing = false) const;
    void _drawRoundedPanel(QPainter *painter, const QRect &rect, const QBrush &brush, const QPen &borderPen, int radius = -1, bool enableAntialiasing = true) const;
    void _drawBorder(QPainter *painter, const QRect &rect, const QPen &pen, Borders border = All) const;
    void _drawBorder(QPainter *painter, const QRect &rect, Qt::GlobalColor color, Borders border = All) const;
    void _drawCenteredImage(QPainter *painter, const QRect &rect, const QImage &image) const;
    void _drawArrow(const QStyleOptionToolButton *toolbutton, QPainter *painter, const QWidget *widget = 0) const;

    void _setGrayGradient(QLinearGradient &gradient, QPalette palette) const;
    void _setSeparatorGradient(QLinearGradient &gradient, QPalette palette) const;

    bool eventFilter(QObject *obj, QEvent *event);
    void timerEvent(QTimerEvent *event);

private:
    void _startAnimation(QObject *obj, QProgressBar *bar);
    void _stopAnimation(QObject *obj, QProgressBar *bar);


    int _animateTimer;
    int _animationFps;
    int _animateStep;
    QElapsedTimer _startTime;
    QList<QProgressBar *> _animatedProgressBars;
};

} // namespace Hnv

Q_DECLARE_OPERATORS_FOR_FLAGS(Hnv::HnvStyle::Borders)

#endif // HNVSTYLE_H
