/******************************************************************************
**
** Copyright (C) 2003-2015 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef ACCESSRIGHTWIDGET_H
#define ACCESSRIGHTWIDGET_H

// Generated hynesim headers
#include <common/catalogitem.h>
#include <common/entry.h>

// diamesh headers
#include <diamesh/accessrights.h>

// Qt headers
#include <QWidget>
#include <QAbstractItemModel>

namespace Ui {
class AccessRightWidget;
} // namespace Ui

namespace HnsClient {
class CatalogItem;
} //namespace HnsClient

namespace Hnv {
/*******************************************************************************
* AccessRightSubjectModel
*/
class AccessRightSubjectModel : public QAbstractListModel
{
    Q_OBJECT
public:
    explicit AccessRightSubjectModel(const Diamesh::ACLList &aclList, QObject *parent = 0);
    virtual int rowCount(const QModelIndex &parent) const;
    virtual QVariant data(const QModelIndex &index, int role) const;
    void removeSubject(const QString &subject);
    void addSubject(const QString &readable, const QString &data);

private:
    QList<QPair<QString, QString> > _subjectList;
};

/*******************************************************************************
* AccessRightModel
*/
class AccessRightModel : public QAbstractTableModel
{
    Q_OBJECT
public:
    enum {
        COLUMN_SUBJECT,
        COLUMN_PERMISSION,
        COLUMN_CLOSE,
        MODEL_COLUMN_COUNT  // Must be last of enum
    };

    explicit AccessRightModel(HnsClient::CatalogItem *catalogItem, QObject *parent = 0);
    virtual int rowCount(const QModelIndex &parent) const;
    virtual int columnCount(const QModelIndex &parent) const;
    virtual QVariant data(const QModelIndex &index, int role) const;
    virtual QVariant headerData(int section, Qt::Orientation orientation, int role) const;

    virtual void addShare(const Hns::CatalogItemPtr &catalogItem, const QString &subject) = 0;
    void removeShare(const Hns::CatalogItemPtr &catalogItem, const QString &subject);
    Diamesh::ACLList aclList();

protected:
    HnsClient::CatalogItem *_catalogItem;
    QList<Diamesh::AccessControlList> _aclList;
};

/*******************************************************************************
* AccessRightEntryModel
*/
class AccessRightEntryModel : public AccessRightModel
{
    Q_OBJECT
public:
    Q_INVOKABLE explicit AccessRightEntryModel(HnsClient::CatalogItem *catalogItem, QObject *parent = 0);
    void addShare(const Hns::CatalogItemPtr &catalogItem, const QString &subject);
};

/*******************************************************************************
* AccessRightFolderModel
*/
class AccessRightFolderModel : public AccessRightModel
{
    Q_OBJECT
public:
    Q_INVOKABLE explicit AccessRightFolderModel(HnsClient::CatalogItem *catalogItem, QObject *parent = 0);
    void addShare(const Hns::CatalogItemPtr &catalogItem, const QString &subject);
};

/*******************************************************************************
* AccessRightWidget
*/
class AccessRightWidget : public QWidget
{
    Q_OBJECT

public:
    explicit AccessRightWidget(QWidget *parent = 0);
    ~AccessRightWidget();
    void initialize(HnsClient::CatalogItem *catalogItem);
    Diamesh::ACLList aclList();

private slots:
    void on_subjectPushButton_clicked();
    void on_accessPoliciesTableView_clicked(const QModelIndex &index);

private:
    void _checkSubjectList();

    Ui::AccessRightWidget *ui;
    AccessRightModel *_rightModel;
    AccessRightSubjectModel *_subjectModel;
    bool _canModify;
    HnsClient::CatalogItem *_catalogItem;
};

} // namespace Hnv

#endif // ACCESSRIGHTWIDGET_H
