/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "accountdialog.h"
#include "ui_accountdialog.h"
#include "hyneviewsession.h"
#include "hnvmainwindow.h"
#include "ressources.h"

// hynesim hnsclient headers
#include <libhnsclient/usermanager.h>
#include <libhnsclient/session.h>

// Qt headers
#include <QLabel>

namespace Hnv {

/*******************************************************************************
* AccountDialog
*******************************************************************************/
AccountDialog::AccountDialog(QWidget *parent)
    : QDialog(parent)
    , ui(new Ui::AccountDialog)
    , _advancedButton(0)
{
    ui->setupUi(this);
    _user = Hns::UserPtr::dynamicCast(HyneviewSession::instance()->user()->ice_clone());

    ui->leFirstName->setText(QString::fromStdString(_user->firstname));
    ui->leLastName->setText(QString::fromStdString(_user->lastname));

    ui->toolButton->setDefaultAction(ui->action_Generate_password);
    ui->action_Generate_password->setIcon(QIcon(Ressources::themedPath(Constants::ICON_PSW_GENERATOR)));

    ui->lePassword->setPlaceholderText("********");
    ui->leConfirmPassword->setPlaceholderText("********");
}

AccountDialog::~AccountDialog()
{
    delete ui;
    _user = 0;
}

/*******************************************************************************
* AccountDialog - Private
*/
bool AccountDialog::isNotEmpty(QLabel *targetLabel, QString &text)
{
    bool isValid = false;
    text = text.trimmed();
    if (text == "") {
        isValid = false;
    } else {
        isValid = true;
    }
    setLabelValidationStyle(targetLabel, isValid);
    return isValid;
}

void AccountDialog::setLabelValidationStyle(QLabel *targetLabel, bool isValid)
{
    if (isValid)
        targetLabel->setStyleSheet("");
    else
        targetLabel->setStyleSheet("QLabel{color:red;}");
}

/*******************************************************************************
* AccountDialog - Private slots
*/
void AccountDialog::on_action_Generate_password_triggered()
{
    QString password = Hnv::HnvMainWindow::instance()->generatePassword();
    ui->lePassword->setText(password);
    ui->lePassword->setEchoMode(QLineEdit::Normal);
    ui->leConfirmPassword->setText(password);
    ui->leConfirmPassword->setEchoMode(QLineEdit::Normal);
}

void AccountDialog::on_lePassword_textEdited(const QString &arg1)
{
    Q_UNUSED(arg1);
    ui->lePassword->setEchoMode(QLineEdit::Password);
    ui->leConfirmPassword->setEchoMode(QLineEdit::Password);
}

void AccountDialog::on_leConfirmPassword_textEdited(const QString &arg1)
{
    Q_UNUSED(arg1);
    ui->lePassword->setEchoMode(QLineEdit::Password);
    ui->leConfirmPassword->setEchoMode(QLineEdit::Password);
}

void AccountDialog::on_buttonBox_accepted()
{
    QString lastName = ui->leLastName->text();
    QString firstName = ui->leFirstName->text();
    QString password = ui->lePassword->text();
    QString passwordConfirmation = ui->leConfirmPassword->text();

    bool isValid = isNotEmpty(ui->labelLastName, lastName);

    isValid &= isNotEmpty(ui->labelFirstName,firstName);

    if (password == "" && password==passwordConfirmation){
        isValid &= true;
    }
    else if (password !="" && password == passwordConfirmation){
        isValid &= true;
    }
     else{
        isValid &= false;
        setLabelValidationStyle(ui->labelPassword,false);
        setLabelValidationStyle(ui->labelConfirmPassword,false);
    }

    if (isValid){
        HnsClient::UserManager *userManager = HnsClient::Session::instance()->userManager();
        _user->firstname = firstName.toStdString();
        _user->lastname = lastName.toStdString();
        userManager->updateUser(_user, password, 0, 0);
        accept();
    }

}

} // namespace Hnv
