/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "applicationsettingsdialog.h"
#include "ui_applicationsettingsdialog.h"
#include "globalsettingswidget.h"
#include "topologyformatsettingswidget.h"
#include "viewerssettingspage.h"

// Qt headers
#include <QToolButton>

namespace Hnv {

ApplicationSettingsDialog::ApplicationSettingsDialog(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::ApplicationSettingsDialog)
{
    ui->setupUi(this);

    addPageToSettings(new GlobalSettingsWidget(this));
    addPageToSettings(new TopologyFormatSettingsWidget(this));
    addPageToSettings(new ViewersSettingsPage(this));

    connect(ui->listWidget, SIGNAL(currentButtonChanged(int)), ui->stackedWidget, SLOT(setCurrentIndex(int)));
}

ApplicationSettingsDialog::~ApplicationSettingsDialog()
{
    delete ui;
    QHash<QToolButton*, ApplicationSettingsInterfacePage *>::iterator i;
    for (i = _pageContainer.begin(); i != _pageContainer.end(); ++i)
        delete (*i);
}

void ApplicationSettingsDialog::addPageToSettings(ApplicationSettingsInterfacePage *page)
{
    _pageContainer.insert(page->button(), page);
    ui->stackedWidget->addWidget(page);
    ui->listWidget->addButton(page->button());
}

void ApplicationSettingsDialog::on_buttonBox_accepted()
{
    bool check = true;
    QHash<QToolButton*, ApplicationSettingsInterfacePage *>::iterator i;
    for (i = _pageContainer.begin(); i != _pageContainer.end() && check; ++i) {
        (*i)->validation(&check);
        if (check)
            (*i)->save();
    }
    if (check)
        accept();
    else
        ui->listWidget->setCurrentButton((*--i)->button());
}

void ApplicationSettingsDialog::on_buttonBox_rejected()
{
    QHash<QToolButton*, ApplicationSettingsInterfacePage *>::iterator i;
    for (i = _pageContainer.begin(); i != _pageContainer.end(); ++i)
        (*i)->reload();
}

} // namespace Hnv
