/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "globalsettingswidget.h"
#include "ui_globalsettingswidget.h"
#include "ressources.h"

// diamesh headers
#include <diamesh/constants.h>

// Qt headers
#include <QSettings>
#include <QDir>

namespace Hnv {

GlobalSettingsWidget::GlobalSettingsWidget(QWidget *parent) :
    ApplicationSettingsInterfacePage(parent),
    ui(new Ui::GlobalSettingsWidget)
{
    ui->setupUi(this);
    _setButton(_generateButton(tr("Main"), QIcon(Ressources::themedPath(Constants::ICON_PREFERENCE_MAIN))));

    ui->modeBrowserStateComboBox->insertItem(Constants::ModeBrowserFull, tr("Text and Icon"));
    ui->modeBrowserStateComboBox->insertItem(Constants::ModeBrowserIcon, tr("Icon only"));
    ui->modeBrowserStateComboBox->insertItem(Constants::ModeBrowserMini, tr("Mini icon"));

    ui->themeComboBox->addItems(QDir(":/themes").entryList(QDir::Dirs, QDir::NoSort));

    ui->languageComboBox->addItem("default");
    ui->languageComboBox->addItem("en");
    QString translationPath = qApp->applicationDirPath() + "/translations";
    if (!QFile::exists(translationPath))
        translationPath = "/usr/share/hynesim/translations";
    QDir dir(translationPath);
    dir.setNameFilters(QStringList() << "hyneview_*.qm");
    ui->languageComboBox->addItems(dir.entryList().replaceInStrings(".qm", "").replaceInStrings("hyneview_", ""));

    reload();
}

GlobalSettingsWidget::~GlobalSettingsWidget()
{
    delete ui;
}

void GlobalSettingsWidget::reload()
{
    QSettings settings;

    settings.beginGroup(Constants::MAIN_WINDOW_SETTINGS);
    ui->windowsSIzeCheckBox->setChecked(settings.value(Constants::MAIN_WINDOW_SIZE_IS_SAVED, Constants::MAIN_WINDOW_DEFAULT_STATE_SAVE).toBool());
    ui->modeBrowserStateComboBox->setCurrentIndex(settings.value(Constants::MODEBROWSERBAR_KEY, Constants::MODEBROWSERBAR_DEFAULT_STATE).toInt());
    settings.endGroup();

    settings.beginGroup(Constants::LOG_MESSAGE_SETTINGS);
    ui->messageBoxStateCheckBox->setChecked(settings.value(Constants::LOG_MESSAGE_STATE_IS_SAVED, Constants::LOG_MESSAGE_DEFAULT_STATE_SAVE).toBool());
    settings.endGroup();

    settings.beginGroup(Constants::SIDEBAR_SETTINGS);
    ui->sideBarStateCheckBox->setChecked(settings.value(Constants::SIDEBAR_STATE_IS_SAVED, Constants::SIDEBAR_DEFAULT_STATE_SAVE).toBool());
    settings.endGroup();

    settings.beginGroup(Constants::PASSWORD_SETTINGS_GROUP);
    _allowedChars = settings.value(Constants::PASSWORD_ALLOWED_CHARACTERS, Constants::PASSWORD_ALLOWED_CHARACTERS_LIST).toString();
    _passwordLength = settings.value(Constants::PASSWORD_LENGTH_KEY, Constants::PASSWORD_LENGTH).toInt();
    settings.endGroup();

    settings.beginGroup(Constants::SETTINGS_GROUP_GLOBAL);
    QString theme = settings.value(Constants::SETTINGS_KEY_THEME, Constants::SETTINGS_DEFAULT_THEME).toString();
    settings.endGroup();

    settings.beginGroup(Diamesh::Constants::SETTINGS_GROUP);
    QString language = settings.value(Diamesh::Constants::SETTINGS_TRANSLATION_LANGUAGE, Constants::SETTINGS_LANGUAGE_DEFAULT).toString();

    ui->pwdCharactersLineEdit->setText(_allowedChars);
    ui->pwdLengthSpinBox->setValue(_passwordLength);
    ui->themeComboBox->setCurrentIndex(ui->themeComboBox->findText(theme));
    ui->languageComboBox->setCurrentIndex(ui->languageComboBox->findText(language));

}

void GlobalSettingsWidget::validation(bool *)
{
}

void GlobalSettingsWidget::save()
{
    QSettings settings;

    settings.beginGroup(Constants::MAIN_WINDOW_SETTINGS);
    settings.setValue(Constants::MAIN_WINDOW_SIZE_IS_SAVED, ui->windowsSIzeCheckBox->isChecked());
    settings.setValue(Constants::MODEBROWSERBAR_KEY, ui->modeBrowserStateComboBox->currentIndex());
    settings.endGroup();

    settings.beginGroup(Constants::LOG_MESSAGE_SETTINGS);
    settings.setValue(Constants::LOG_MESSAGE_STATE_IS_SAVED, ui->messageBoxStateCheckBox->isChecked());
    settings.endGroup();

    settings.beginGroup(Constants::SIDEBAR_SETTINGS);
    settings.setValue(Constants::SIDEBAR_STATE_IS_SAVED, ui->sideBarStateCheckBox->isChecked());
    settings.endGroup();

    settings.beginGroup(Constants::PASSWORD_SETTINGS_GROUP);
    settings.setValue(Constants::PASSWORD_ALLOWED_CHARACTERS, ui->pwdCharactersLineEdit->text());
    settings.setValue(Constants::PASSWORD_LENGTH_KEY, ui->pwdLengthSpinBox->value());
    settings.endGroup();

    settings.beginGroup(Constants::SETTINGS_GROUP_GLOBAL);
    settings.setValue(Constants::SETTINGS_KEY_THEME, ui->themeComboBox->currentText());
    settings.endGroup();

    settings.beginGroup(Diamesh::Constants::SETTINGS_GROUP);
    QString language(ui->languageComboBox->currentText());
    if (language == "default")
        settings.remove(Diamesh::Constants::SETTINGS_TRANSLATION_LANGUAGE);
    else
        settings.setValue(Diamesh::Constants::SETTINGS_TRANSLATION_LANGUAGE, language);
}

} // namespace Hnv
