/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "topologyformatsettingswidget.h"
#include "ui_topologyformatsettingswidget.h"
#include "simulationmodewidget.h"
#include "ressources.h"
#include "topologywidget.h"

// Qt headers
#include <QColorDialog>
#include <QSettings>

namespace Hnv {

TopologyFormatSettingsWidget::TopologyFormatSettingsWidget(QWidget *parent) :
    ApplicationSettingsInterfacePage(parent),
    ui(new Ui::TopologyFormatSettingsWidget)
{
    ui->setupUi(this);
    ui->tbColor->setIcon(QIcon(Ressources::themedPath(Constants::ICON_COLORIZE)));
    ui->tbModelColor->setIcon(QIcon(Ressources::themedPath(Constants::ICON_COLORIZE)));
    ui->tbWireColor->setIcon(QIcon(Ressources::themedPath(Constants::ICON_COLORIZE)));

    _setButton(_generateButton(tr("Topology"), QIcon(Ressources::themedPath(Constants::ICON_PROPERTIES))));

    QSettings settings;

    settings.beginGroup(Constants::TOPOLOGY_SETTINGS);
    QStringList enabledFormats = settings.value(Constants::TOPOLOGY_ENABLED_FORMAT).toStringList();
    settings.endGroup();

    foreach (QString s, settings.childGroups())
        if (s.startsWith(QVariant(Constants::TOPOLOGY_SETTINGS).toString() +"Formats-")) {
            settings.beginGroup(s);
            QString idTopo = settings.value(Constants::TOPOLOGY_ID_KEY).toString();
            QString descTopo = settings.value(Constants::TOPOLOGY_DESCRIPTION).toString();
            if (enabledFormats.contains(idTopo))
                ui->defaultFormatComboBox->addItem(descTopo, idTopo);
            settings.endGroup();
        }

    ui->defaultFormatComboBox->addItem(QString(tr("Custom format")), QString());

    ui->defaultSizeUnitComboBox->addItem(tr("Millimeters"), "mm");
    ui->defaultSizeUnitComboBox->addItem(tr("Pixels"), "px");
    ui->defaultSizeUnitComboBox->addItem(tr("Inches"), "in");
    ui->elidedModeComboBox->addItem(tr("Left"),Qt::ElideLeft);
    ui->elidedModeComboBox->addItem(tr("Right"),Qt::ElideRight);
    ui->elidedModeComboBox->addItem(tr("Middle"),Qt::ElideMiddle);
    ui->elidedModeComboBox->addItem(tr("None"),Qt::ElideNone);

    ui->wireOriginModeComboBox->insertItems(0, QStringList() << tr("Center") << tr("Automated") << tr("Fixed"));

    reload();
}

TopologyFormatSettingsWidget::~TopologyFormatSettingsWidget()
{
    delete ui;
}

void TopologyFormatSettingsWidget::reload()
{
    QSettings settings;

    settings.beginGroup(Constants::TOPOLOGY_SETTINGS);
    ui->labelColor->setText("#"+ settings.value(Constants::TOPOLOGY_DEFAULT_BACKGROUND_COLOR_KEY, Constants::TOPOLOGY_DEFAULT_BACKGROUND_COLOR).toString());
    ui->labelModelColor->setText("#"+ settings.value(Constants::TOPOLOGY_DEFAULT_MODEL_BACKGROUND_COLOR_KEY, Constants::TOPOLOGY_DEFAULT_MODEL_BACKGROUND_COLOR).toString());
    ui->labelWireColor->setText("#"+ settings.value(Constants::WIRE_DEFAULT_COLOR, "000000").toString());
    ui->defaultFormatComboBox->setCurrentIndex(ui->defaultFormatComboBox->findData(settings.value(Constants::TOPOLOGY_DEFAULT_FORMAT, QString()).toString()));
    ui->defaultSizeUnitComboBox->setCurrentIndex(ui->defaultSizeUnitComboBox->findData(settings.value(Constants::TOPOLOGY_DEFAULT_SIZE_UNIT, "px").toString()));
    ui->angleDropingDoubleSpinBox->setValue(settings.value(Constants::ANGLE_FOR_DROPING_ENTITIES_KEY,QVariant(60.0)).toDouble());
    ui->radiusDropingSpinBox->setValue(settings.value(Constants::RADIUS_FOR_DROPING_ENTITIES_KEY,QVariant(256)).toInt());
    ui->elidedModeComboBox->setCurrentIndex(ui->elidedModeComboBox->findData(settings.value(Constants::ELIDE_MODE_KEY,0).toInt()));
    ui->displayGridCheckBox->setChecked(settings.value(Constants::TOPOLOGY_DEFAULT_DISPLAY_GRID).toBool());
    settings.endGroup();

    settings.beginGroup(Constants::WIRES_SETTINGS);
    int originMode = settings.value(Constants::WIRE_ORIGIN_MODE, 0).toInt();
    settings.endGroup();

    ui->labelColor->setStyleSheet("QLabel{background:"+ ui->labelColor->text() +";}");
    ui->labelModelColor->setStyleSheet("QLabel{background:"+ui->labelModelColor->text() +";}");
    ui->labelWireColor->setStyleSheet("QLabel{background:"+ ui->labelWireColor->text() +";}");
    ui->wireOriginModeComboBox->setCurrentIndex(originMode);
}

void TopologyFormatSettingsWidget::validation(bool *check)
{
    if (!QColor(ui->labelColor->text()).isValid() || !QColor(ui->labelModelColor->text()).isValid())
        (*check) = false;
}

void TopologyFormatSettingsWidget::save()
{
    QSettings settings;

    settings.beginGroup(Constants::WIRES_SETTINGS);
    settings.setValue(Constants::WIRE_ORIGIN_MODE, ui->wireOriginModeComboBox->currentIndex());
    settings.endGroup();

    settings.beginGroup(Constants::TOPOLOGY_SETTINGS);
    settings.setValue(Constants::TOPOLOGY_DEFAULT_BACKGROUND_COLOR_KEY, ui->labelColor->text().split("#")[1]);
    settings.setValue(Constants::TOPOLOGY_DEFAULT_MODEL_BACKGROUND_COLOR_KEY, ui->labelModelColor->text().split("#")[1]);
    settings.setValue(Constants::WIRE_DEFAULT_COLOR, ui->labelWireColor->text().split("#")[1]);
    settings.setValue(Constants::TOPOLOGY_DEFAULT_FORMAT, ui->defaultFormatComboBox->itemData(ui->defaultFormatComboBox->currentIndex()).toString());
    settings.setValue(Constants::TOPOLOGY_DEFAULT_SIZE_UNIT, ui->defaultSizeUnitComboBox->itemData(ui->defaultSizeUnitComboBox->currentIndex()).toString());
    settings.setValue(Constants::ANGLE_FOR_DROPING_ENTITIES_KEY, QVariant(ui->angleDropingDoubleSpinBox->value()));
    settings.setValue(Constants::RADIUS_FOR_DROPING_ENTITIES_KEY,QVariant(ui->radiusDropingSpinBox->value()));
    settings.setValue(Constants::ELIDE_MODE_KEY,ui->elidedModeComboBox->itemData(ui->elidedModeComboBox->currentIndex()));
    settings.setValue(Constants::TOPOLOGY_DEFAULT_DISPLAY_GRID, QVariant(ui->displayGridCheckBox->isChecked()));
    settings.endGroup();
    SimulationModeWidget::instance()->topologyWidget()->refreshAllEntities();
}

void TopologyFormatSettingsWidget::on_tbColor_clicked()
{
    QColor returnColor = QColorDialog::getColor(QColor(ui->labelColor->text()),this,tr("Color"));
    if (returnColor.isValid())
        ui->labelColor->setText(returnColor.name());
    ui->labelColor->setStyleSheet("QLabel{background:"+ ui->labelColor->text() +";}");
}

void TopologyFormatSettingsWidget::on_tbModelColor_clicked()
{
    QColor returnColor = QColorDialog::getColor(QColor(ui->labelModelColor->text()),this,tr("Color"));
    if (returnColor.isValid())
        ui->labelModelColor->setText(returnColor.name());
    ui->labelModelColor->setStyleSheet("QLabel{background:"+ ui->labelModelColor->text() +";}");
}

void TopologyFormatSettingsWidget::on_tbWireColor_clicked()
{
    QColor returnColor = QColorDialog::getColor(QColor(ui->labelWireColor->text()),this,tr("Color"));
    if (returnColor.isValid())
        ui->labelWireColor->setText(returnColor.name());
    ui->labelWireColor->setStyleSheet("QLabel{background:"+ ui->labelWireColor->text() +";}");
}

} // namespace Hnv
