/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "viewerssettingspage.h"
#include "ui_viewerssettingspage.h"
#include "ressources.h"

// Qt headers
#include <QSettings>

namespace Hnv {

ViewersSettingsPage::ViewersSettingsPage(QWidget *parent) :
    ApplicationSettingsInterfacePage(parent),
    ui(new Ui::ViewersSettingsPage)
{
    ui->setupUi(this);
    _setButton(_generateButton(tr("Viewers"), QIcon(Ressources::themedPath(Constants::ICON_PREFERENCE_VIEWER))));
    reload();
}

ViewersSettingsPage::~ViewersSettingsPage()
{
    delete ui;
}

void ViewersSettingsPage::reload()
{
    QSettings settings;
    ui->leResolution->setText(settings.value(Constants::REMOTEDISPLAY_RESOLUTION_OPTION, Constants::REMOTEDISPLAY_DEFAULT_RESOLUTION_OPTION).toString());
    ui->leResolution->setToolTip(Constants::REMOTEDISPLAY_DEFAULT_RESOLUTION_OPTION);
    ui->leResolution->setStyleSheet("");
    ui->cbDetachedTab->setCheckState((Qt::CheckState)(settings.value(Constants::DETACHED_TAB_OPTION, Qt::Unchecked)).toInt());
}

void ViewersSettingsPage::validation(bool *check)
{
    QString resolution = ui->leResolution->text();
    QRegExp regExp("^[0-9]+x[0-9]+$");
    if (resolution.isEmpty() || !regExp.exactMatch(resolution)) {
        ui->labelDefaultResolution->setStyleSheet("QLabel{color:red;}");
        (*check) = false;
    }
    else {
        ui->labelDefaultResolution->setStyleSheet("");
    }
}

void ViewersSettingsPage::save()
{
    QSettings settings;
    settings.setValue(Constants::REMOTEDISPLAY_RESOLUTION_OPTION, ui->leResolution->text());
    settings.setValue(Constants::DETACHED_TAB_OPTION, ui->cbDetachedTab->checkState());
}

} // namespace Hnv
