/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/
#ifndef HNV_DYNAMICFILTERWIDGET_H
#define HNV_DYNAMICFILTERWIDGET_H

// Qt headers
#include <QWidget>
#include <QMap>

class QSignalMapper;

namespace Ui {
class DynamicFilterWidget;
} // namespace Ui

namespace Hnv {

class DynamicFilterWidget : public QWidget
{
    Q_OBJECT
public:
    explicit DynamicFilterWidget(QWidget *parent = 0);
    ~DynamicFilterWidget();
    void addFilter(const QString &key, const QString &text);

signals:
    void filterTextChanged(const QString &key, const QStringList &filters);

protected:
    enum FilterColumns {
        State,
        Value,
        Remove = 4
    };
    struct FilterData {
        bool    enabled;
        QString key;
        QString label;
        QString value;
    };

    void _createFilterWidget(const QString &key, const QString &text);
    void _removeFilterWidget(int row);
    FilterData _dataFromRow(int row);
    QStringList _filtersFromKey(const QString &key);
    bool _isDataValid(const FilterData &data);

protected slots:
    void _filterTextChanged(int row);
    void _filterStateChanged(int row);
    void _filterRemoved(int row);

private slots:
    void on_addFilterPb_clicked();

private:
    Ui::DynamicFilterWidget *ui;
    QMap<QString, QList<int> > _filterRows;
    QSignalMapper *_filterMapper;
    QSignalMapper *_filterStateMapper;
    QSignalMapper *_removeFilterMapper;

    QString _propertyKey;
};

} // namespace Hnv

#endif // HNV_DYNAMICFILTERWIDGET_H
