/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef ENTITYCATALOGWIDGET_H
#define ENTITYCATALOGWIDGET_H

// Project headers
#include "sidebar.h"

// Qt headers
#include <QModelIndexList>
#include <QItemSelection>

class QCompleter;
class QMimeData;
class QModelIndex;
class QPersistentModelIndex;

namespace Ui {
class EntityCatalogWidget;
} // namespace Ui

namespace HnsClient {
class EntityEntryCatalog;
} // namespace HnsClient

namespace Hnv {
class HyneviewSession;
class EntityCatalogModel;
class TreeSortFilterProxyModel;
class ItemStates;
class LinkedClonesFilterProxyModel;

/*******************************************************************************
* EntityCatalogWidget
*/
class EntityCatalogWidget : public SideBarContentWidget
{
    Q_OBJECT

public:
    Q_INVOKABLE explicit EntityCatalogWidget(QWidget *parent = 0);
    ~EntityCatalogWidget();

    static QString title();
    static QString sideBarId();

    void editItem(QModelIndex index);

    QModelIndex selectedItemIndex();
    QModelIndexList selectedItemIndexes();
    int getItemInternalType(QModelIndex targetIndex);

    bool canHandleMimeData(const QMimeData *data, QString &datatype);
    bool canHandleMimeData(const QMimeData *data);

public slots:
    void catalogLoadedSlot();
    void connected(HyneviewSession *session);
    void beginDisconnect();
    void updateTreeViewCell(QPersistentModelIndex index);

signals:
    void beginTransaction(QPersistentModelIndex index);
//    void loadingTransaction(int count); // TODO : check if needed
    void serverDisconnect();

protected:
    void addFolder(QModelIndex parentIdx);
    void _toggleActions();
    void _parseCatalogItem(QModelIndex, ItemStates *item);
    bool _canAddCategory(QModelIndex index, ItemStates *item, bool isAdmin);
    bool _canRead(QModelIndex index, ItemStates *item);
    bool _canEdit(QModelIndex index, ItemStates *item, bool isAdmin);
    bool _canDelete(QModelIndex index, ItemStates *item, bool isAdmin);
    bool _canRemoveFromCategory(QModelIndex index, ItemStates *item, bool isAdmin);
    bool _canClone(QModelIndex index, ItemStates *item, bool isAdmin);
    bool _canBuildFromModel(QModelIndex index, ItemStates *item, bool isAdmin);
    bool _canDiscardSaveState(QModelIndex index, ItemStates *item, bool isAdmin);
    bool _canExport(QModelIndex index, ItemStates *item, bool isAdmin);
    int _parseSelectedItems(QList<qlonglong> &validFolderIDList, QHash<qlonglong, QList<qlonglong> > &validEntityHash, bool orphansAreValid = true);

private slots:
    void _beginTransactionSlot(QPersistentModelIndex sourceIndex);

    void _focusOnItemSlot(QModelIndex index);

    void _collapsedSlot(QModelIndex index);
    void _expandedSlot(QModelIndex index);
    void _catalogSelectionChanged(QItemSelection,QItemSelection);
    void _contextMenuRequested(QPoint pos);
    void _modelDataChangedSlot(QModelIndex topLeft, QModelIndex bottomRight);

    void on_catalogFilterEdit_textChanged(QString currentText);
    void on_action_Add_category_triggered();
    void on_action_Rename_triggered();
    void on_action_Collapse_All_triggered();
    void on_action_Expand_All_triggered();
    void on_action_Properties_triggered();
    void on_action_Clone_Entity_triggered();
    void on_action_Delete_triggered();
    void on_action_Remove_from_category_triggered();
    void on_action_Build_Model_From_triggered();
    void on_action_Discard_save_state_triggered();
    void on_action_Set_custom_graphic_triggered();
    void on_action_Export_triggered();
    void on_action_Show_linked_domains_triggered(bool checked);
    void _exportFinished(bool result);

private:
    Ui::EntityCatalogWidget *ui;
    HnsClient::EntityEntryCatalog *_catalog;
    EntityCatalogModel *_catalogModel;
    TreeSortFilterProxyModel * _proxyModel;
    LinkedClonesFilterProxyModel *_linkedClonesFilterProxyModel;
    HyneviewSession * _session;
    QCompleter * _myCompleter;
};

} // namespace Hnv
#endif // ENTITYCATALOGWIDGET_H
