// Project headers
#include "graphicselectordialog.h"
#include "ui_graphicselectordialog.h"
#include "ressources.h"
#include "hyneviewsession.h"
#include "libhnsclient/resourcemanager.h"

// Qt headers
#include <QPainter>
#include <QFileDialog>
#include <QSvgRenderer>
#include <QPushButton>

namespace Hnv {

GraphicSelectorDialog::GraphicSelectorDialog(QHash<QString, QByteArray> customGraphicHash, QWidget *parent)
    : QDialog(parent)
    , ui(new Ui::GraphicSelectorDialog)
    , _customGraphicHash(customGraphicHash)
    , _newFile(false)
{
    ui->setupUi(this);
    _browseButton = ui->buttonBox->addButton(tr("Browse"), QDialogButtonBox::ActionRole);
    _browseButton->setIcon(QIcon(Ressources::themedPath(Constants::ICON_OPEN)));
    _clearButton = ui->buttonBox->addButton(tr("Default icon"), QDialogButtonBox::ActionRole);
    _clearButton->setIcon(QIcon(Ressources::themedPath(Constants::ICON_CLEAR_MESSAGES)));
    _deleteButton = ui->buttonBox->addButton(tr("Delete"), QDialogButtonBox::ActionRole);
    _deleteButton->setIcon(QIcon(Ressources::themedPath(Constants::ICON_DELETE)));
    ui->buttonBox->setTabOrder(_browseButton, _clearButton);
    ui->buttonBox->setTabOrder(ui->buttonBox->button(QDialogButtonBox::Ok), ui->buttonBox->button(QDialogButtonBox::Close));
    connect(_browseButton, SIGNAL(clicked()), this, SLOT(_browseButtonClickedSlot()));
    connect(_clearButton, SIGNAL(clicked()), this, SLOT(_clearButtonClickedSlot()));
    connect(_deleteButton, SIGNAL(clicked()), this, SLOT(_deleteButtonClickedSlot()));
    QHashIterator<QString, QByteArray> iterator(_customGraphicHash);
    while(iterator.hasNext()) {
        iterator.next();
        QPixmap customGraphic(QSize(64,64));
        customGraphic.fill(Qt::transparent);
        QSvgRenderer renderer(iterator.value());
        QPainter svgpainter(&customGraphic);
        renderer.render(&svgpainter);
        new QListWidgetItem(QIcon(customGraphic), iterator.key(), ui->graphicList);
    }
    connect(ui->graphicList, SIGNAL(itemSelectionChanged()), SLOT(_itemSelectionChangedSlot()));
    _deleteButton->setDisabled(true);
}

GraphicSelectorDialog::~GraphicSelectorDialog()
{
    delete ui;
}

bool GraphicSelectorDialog::isFileNew() const
{
    return _newFile;
}

QString GraphicSelectorDialog::selectedFileName() const
{
    return _selectedFileName;
}

void GraphicSelectorDialog::on_buttonBox_accepted()
{
    QList<QListWidgetItem *> itemList = ui->graphicList->selectedItems();
    if (!itemList.isEmpty()) {
        _newFile = false;
        _selectedFileName = itemList.at(0)->text();
        accept();
    }
}

void GraphicSelectorDialog::_browseButtonClickedSlot()
{
    QString fileName = QFileDialog::getOpenFileName(this, tr("Open File"),
                                                    "",
                                                    "SVG (*.svg)");
    if (!fileName.isEmpty()) {
        _newFile = true;
        _selectedFileName = fileName;
        accept();
    }
}

void GraphicSelectorDialog::_clearButtonClickedSlot()
{
    _newFile = false;
    _selectedFileName = QString();
    accept();
}

void GraphicSelectorDialog::_deleteButtonClickedSlot()
{
    QList<QListWidgetItem *> itemList = ui->graphicList->selectedItems();
    if (!itemList.isEmpty()) {
        HyneviewSession::instance()->resourceManager()->deleteResource(itemList.at(0)->text());
        QListWidgetItem *item = itemList.at(0);
        ui->graphicList->removeItemWidget(item);
        delete item;
    }
}

void GraphicSelectorDialog::_itemSelectionChangedSlot()
{
    QList<QListWidgetItem *> itemList = ui->graphicList->selectedItems();
    _deleteButton->setDisabled(itemList.isEmpty() ? true : false);
}

} // namespace Hnv
