/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef HNV_ENTITYMONITORWIDGET_H
#define HNV_ENTITYMONITORWIDGET_H

// Project headers
#include "sidebar.h"
#include "treemodel.h"
#include "treemodelitem.h"

// Generated hynesim headers
#include "common/monitordata.h"

// hynesim hnsclient headers
#include <libhnsclient/common.h>

namespace HnsClient {
class GenericEntity;
class NodeMonitor;
} // namespace HnsClient

namespace Hnv {

/*******************************************************************************
* EntityMonitoringNode
*******************************************************************************/
class EntityMonitoringNode : public TreeModelItem
{
    Q_OBJECT
public:
    enum EntityMonitoringNodeColumn {
          COLUMN_KEY
        , COLUMN_VALUE
        , NODE_COLUMN_COUNT
    };

    explicit EntityMonitoringNode(QString key, QString unit = QString(), QObject *parent = 0);
    ~EntityMonitoringNode();
    virtual Qt::ItemFlags flags(int column) const;
    virtual QVariant data(int column = 0, int role = Qt::DisplayRole) const;
    void updateValue(QString value, QString unit = QString());

private:
    QString _key;
    QString _value;
    QString _unit;
};


/*******************************************************************************
* EntityMonitoringModel
*******************************************************************************/
class EntityMonitoringModel : public TreeModel
{
    Q_OBJECT
public:
    explicit EntityMonitoringModel(QWidget *parent = 0);
    ~EntityMonitoringModel();
    void setEntity(HnsClient::GenericEntity *entity);
    virtual QVariant headerData(int section, Qt::Orientation orientation, int role) const;

private slots:
    void _monitorDataUpdated();
    void _updateDomainData(Hns::DomainMonitoringDataPtr domainData);
    EntityMonitoringNode *_nodeByKey(QString key, QString unit);
    void _updateSizeNode(EntityMonitoringNode *node, qreal value, HnsClient::BytesUnitEnum unit, bool asRate = false);
    void _monitorDestroyedSlot();

private:
    qlonglong _entityId;
    HnsClient::NodeMonitor *_monitor;
    QHash<QString, EntityMonitoringNode *> _nodeHash;
};


/*******************************************************************************
* EntityMonitorWidget
*******************************************************************************/
class EntityMonitorWidget : public SideBarTreeWidget
{
    Q_OBJECT

public:
    Q_INVOKABLE explicit EntityMonitorWidget(QWidget *parent = 0);
    ~EntityMonitorWidget();
    static QString title();
    static QString sideBarId();

protected:
    virtual void _indexClicked(const QModelIndex &index);
    virtual void _indexDoubleClicked(const QModelIndex &index);
    virtual void _contextMenuRequested(const QModelIndex &index, const QPoint &position);
    void _setEntity(HnsClient::GenericEntity *entity);

private slots:
    void _selectionChangedSlot(QList<HnsClient::GenericEntity*> selectedEntities);
    void _topologyChangedSlot();
    void _currentEntityDestroyed();
    void _currentEntityStatusChanged();

private:
    EntityMonitoringModel *_monitoringModel;
    HnsClient::GenericEntity *_currentEntity;
};

} // namespace Hnv

#endif // HNV_ENTITYMONITORWIDGET_H
