/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "domainnetworkcardwidget.h"
#include "ui_domainnetworkcardwidget.h"

// hynesim common headers
#include "libhnsclient/common.h"

namespace Hnv {
/*******************************************************************************
* DomainNetworkCardWidget
*/
DomainNetworkCardWidget::DomainNetworkCardWidget(QWidget *parent, QString domainType)
    : QWidget(parent)
    , ui(new Ui::DomainNetworkCardWidget)
    , _domainType(domainType)
{
    ui->setupUi(this);
}

DomainNetworkCardWidget::~DomainNetworkCardWidget()
{
    delete ui;
}

void DomainNetworkCardWidget::displayErrors(int error)
{
    if (error & HnsClient::macAdressError) {
        ui->labelName->setStyleSheet("");
        ui->macAddressLabel->setStyleSheet("QLabel{color:red;}");
    } else if (error & HnsClient::macAdressIsUsed) {
        ui->labelName->setStyleSheet("");
        ui->macAddressLabel->setStyleSheet("QLabel{color:red;}");
    } else if (error & HnsClient::nameIsUsed) {
        ui->macAddressLabel->setStyleSheet("");
        ui->labelName->setStyleSheet("QLabel{color:red;}");
    } else if (error & HnsClient::nameIsInvalid) {
        ui->macAddressLabel->setStyleSheet("");
        ui->labelName->setStyleSheet("QLabel{color:red;}");
    }
}

void DomainNetworkCardWidget::clearErrors()
{
    ui->labelName->setStyleSheet("");
    ui->macAddressLabel->setStyleSheet("");
}

void DomainNetworkCardWidget::setCardInfo(Hns::Netcard card, Hns::DomainEntryPropertiesPtr domainProperties)
{
    _card = card;
    QString cardModelName = QString::fromStdString(_card.model);
    if (_domainType == "qemu") {
        ui->modelComboBox->setEditable(true);
        ui->modelComboBox->addItem(cardModelName, cardModelName);
    } else {
        foreach (std::string model, domainProperties->networkCardsModelsList) {
            ui->modelComboBox->addItem(QString::fromStdString(model),QVariant(QString::fromStdString(model)));
        }
        if (ui->modelComboBox->count()==0) {
            ui->modelComboBox->addItem(QString(tr("Default")),QVariant(QString()));
            ui->modelComboBox->setEnabled(false);
        }
        if (cardModelName == QString("")) {
            ui->modelComboBox->setCurrentIndex(0);
        } else {
            ui->modelComboBox->setCurrentIndex(ui->modelComboBox->findText(cardModelName));
        }
    }

    foreach (std::string type, domainProperties->networkCardsTypesList) {
        ui->typeComboBox->addItem(QString::fromStdString(type),QVariant(QString::fromStdString(type)));
    }
    if (ui->typeComboBox->count()==0) {
        ui->typeComboBox->addItem(QString(tr("Default")),QVariant(QString()));
        ui->typeComboBox->setEnabled(false);
    }
    if ((QString::fromStdString(_card.type)) == QString("")) {
        ui->typeComboBox->setCurrentIndex(0);
    } else {
        ui->typeComboBox->setCurrentIndex(ui->typeComboBox->findText(QString(_card.type.c_str())));
    }
    ui->macAddressGen->setMacAddress(QString::fromStdString(_card.macAddress));
    ui->descriptionTextEdit->setText(QString::fromStdString(_card.description));

    QString numberEth = QString::fromStdString(_card.source);

    QRegExp regExp("[0-9]");
    numberEth.remove(0,numberEth.indexOf(regExp));
    int numberPort = numberEth.toInt();
    ui->spinBox->setValue(numberPort );
}

Hns::Netcard DomainNetworkCardWidget::getCardInfo()
{
    _card.id = ui->spinBox->value();
    _card.source = QString("eth%1").arg(QString::number(_card.id)).toStdString();
    _card.type = ui->typeComboBox->itemData(ui->typeComboBox->currentIndex()).toString().toStdString();
    if (_domainType == "qemu")
        _card.model = ui->modelComboBox->currentText().toStdString();
    else
        _card.model = ui->modelComboBox->itemData(ui->modelComboBox->currentIndex()).toString().toStdString();
    _card.macAddress = ui->macAddressGen->macAddress().toStdString();
    _card.description = ui->descriptionTextEdit->toPlainText().toStdString();
    return _card;
}

void DomainNetworkCardWidget::setDisabled(bool disabled)
{
    ui->descriptionTextEdit->setDisabled(disabled);
    ui->macAddressGen->setDisabled(disabled);
    ui->spinBox->setDisabled(disabled);
    ui->modelComboBox->setDisabled(disabled);
    ui->typeComboBox->setDisabled(disabled);
}

void DomainNetworkCardWidget::initNewCard()
{
    ui->macAddressGen->generate();
}

} // namespace Hnv
