/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef DOMAINPAGE_H
#define DOMAINPAGE_H

// Project headers
#include "hnv_entryproperties.h"

// Generated hynesim headers
#include "common/domainentry.h"

// Qt headers
#include <QSpinBox>

namespace Ui {
class DomainPage;
} // namespace Ui

namespace Hnv {

class QMultipleOfSpinBox : public QSpinBox
{
    Q_OBJECT

public:
    QMultipleOfSpinBox(QWidget *parent = 0);
    ~QMultipleOfSpinBox();

    void setMultiple(int multiple);

    void fixup(QString &str) const;
    QValidator::State validate(QString &input, int &pos) const;

private:
    int _multiple;
};

/*******************************************************************************
* DomainPage
*/
class DomainPage : public EntryPropertiesPage
{
    Q_OBJECT
public:
    Q_INVOKABLE explicit DomainPage(HnsClient::CatalogItem *entry, Hns::EntryPropertiesPtr entryProperties, QWidget *parent = 0, bool readOnly = false, bool defined = false);
    ~DomainPage();
    virtual void initFocus();
    virtual QToolButton *getPageButton();

protected:
    void _disableWidgetsWhenNecessary(bool necessary);
    void _displayType(QString type);
    void _displayMemory(int memory);
    void _displayVCpu(int vcpu);
    void _displayPae(bool pae);
    void _displayAcpi(bool acpi);
    void _displayApic(bool apic);
    void _displayImmutable(bool immutable);
    void _displaySharedFolder(bool sharedFolder);
    void _displayActionManagerInterfaceList(qlonglong current, QMap<Ice::Long, Hns::Netcard> networkCardMap);
    void _displayUsbRedirectionCount(int usbRedirectionCount);
    void _displayUsbControllerType(QString usbControllerType);
    void _displayVideo(QString videoModelType);

    virtual void _validate(bool &isModified, bool &isInvalid);

private slots:
    void updateApicValue(int value);
    void on_memoryHorizontalSlider_actionTriggered(int action);
    void _getMemoryValue(int value);
    void _getVcpuValue(int value);
    void _setMemoryValue(int value);
    void _setVcpuValue(int value);
    void on_apicCheckBox_clicked(bool checked);

    void on_memoryHorizontalSlider_valueChanged(int value);

private:
    Ui::DomainPage *ui;
    int _memoryMultiple;
    bool _apicOldValue;
};

} // namespace Hnv
#endif // DOMAINPAGE_H
