/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "domainstoragepage.h"
#include "ui_domainstoragepage.h"
#include "ressources.h"
#include "domainstoragewidget.h"

// hynesim hnsclient headers
#include <libhnsclient/entry.h>

// Qt headers
#include <QToolButton>

namespace Hnv {

/*******************************************************************************
* DomainStoragePage
*/
DomainStoragePage::DomainStoragePage(HnsClient::CatalogItem *entry, Hns::EntryPropertiesPtr entryProperties, QWidget *parent, bool readOnly , bool defined)
    : EntryPropertiesPage(entry,entryProperties, parent, readOnly , defined)
    , ui(new Ui::DomainStoragePage)
{
    ui->setupUi(this);
    _properties = entryProperties;
    _domainProperties = Hns::DomainEntryPropertiesPtr::dynamicCast(_properties);
    HnsClient::DomainEntry * domain = qobject_cast<HnsClient::DomainEntry *>(_sourceEntry);
    if (domain) {
        foreach (Hns::Disk disk, domain->getDisks()) {
            _initStorageTab(disk);
        }
    }
}

DomainStoragePage::~DomainStoragePage()
{
    delete ui;
}

void DomainStoragePage::initFocus()
{

}

QToolButton *DomainStoragePage::getPageButton()
{
    return _generatePageButton(tr("Storage"), QIcon(Ressources::themedPath(Constants::ICON_DATABASE)), _domainProperties->hasStorageSettings);
}

void DomainStoragePage::_disableWidgetsWhenNecessary(bool necessary)
{
    Q_UNUSED(necessary)
}

void DomainStoragePage::_initStorageTab(Hns::Disk disk)
{
    DomainStorageWidget *storageWidget = _addStorageTab(QString::fromStdString(disk.source));
    storageWidget->setStorageInfos(disk, _domainProperties);
}

DomainStorageWidget *DomainStoragePage::_addStorageTab(QString label)
{
    DomainStorageWidget *storageWidget = new DomainStorageWidget(this);
    ui->tabWidget->addTab(storageWidget, label);
    return storageWidget;
    return 0;
}

void DomainStoragePage::_validate(bool &isModified, bool &isInvalid)
{
    HnsClient::ValidationResult result;
    QList<Hns::Disk> disksList;
    int i = 0;
    for (i = 0; i != ui->tabWidget->count(); i++) {
        DomainStorageWidget *page = getStorageWidget(i);
        Hns::Disk disk = page->getStorageInfos();
        disksList.append(disk);
    }
    HnsClient::DomainEntry * domain = qobject_cast<HnsClient::DomainEntry *>(_sourceEntry);
    if (domain) {
        result = domain->validateDisks(disksList, _errorDiskList);
        _computeValidationState(result, isModified, isInvalid);
        if (isInvalid) {
            for (i = 0; i != ui->tabWidget->count(); i++) {
                DomainStorageWidget *page = getStorageWidget(i);
                page->clearErrors();
                Hns::Disk disk = page->getStorageInfos();
                int error = _errorDiskList.value(QString::fromStdString(disk.source));
                if (error != 0) {
                    ui->tabWidget->setCurrentIndex(i);
                    page->displayErrors(error);
                }
            }
        }
    } else {
        return;
    }
}

DomainStorageWidget *DomainStoragePage::getStorageWidget(int index)
{
    DomainStorageWidget *result = 0;
    result = dynamic_cast<DomainStorageWidget*>(ui->tabWidget->widget(index));
    return result;
}

} // namespace Hnv
