/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "domainstoragewidget.h"
#include "ui_domainstoragewidget.h"

// hynesim hnsclient headers
#include <libhnsclient/common.h>

namespace Hnv {

/*******************************************************************************
* DomainStorageWidget
*/
DomainStorageWidget::DomainStorageWidget(QWidget *parent) :
    QWidget(parent),
    ui(new Ui::DomainStorageWidget)
{
    ui->setupUi(this);
    ui->sourceLineEdit->setButtonTooltip(tr("Do not change this value unless you know what you are doing."));
    ui->imageTypeLineEdit->setButtonTooltip(tr("Do not change this value unless you know what you are doing."));
}

DomainStorageWidget::~DomainStorageWidget()
{
    delete ui;
}

void DomainStorageWidget::setStorageInfos(Hns::Disk disk, Hns::DomainEntryPropertiesPtr domainProperties)
{
    _domainProperties = domainProperties;
    _disk = disk;
    ui->sourceLineEdit->setText(_disk.source.c_str());
    ui->imageTypeLineEdit->setText(_disk.driverType.c_str());

    foreach (std::string bus, _domainProperties->storageBusTypesList) {
        ui->busComboBox->addItem(QString::fromStdString(bus),QVariant(QString::fromStdString(bus)));
    }
    ui->busComboBox->setCurrentIndex(ui->busComboBox->findText(_disk.targetBus.c_str()));
    ui->deviceLineEdit->setText(_disk.target.c_str());

    ui->sourceLineEdit->setReadOnly(true);
    ui->busComboBox->setEnabled(false);
    ui->deviceLineEdit->setReadOnly(true);
    ui->imageTypeLineEdit->setReadOnly(true);
}

Hns::Disk DomainStorageWidget::getStorageInfos()
{
    _disk.source = ui->sourceLineEdit->text().toStdString();
    _disk.targetBus = ui->busComboBox->currentText().toStdString();
    _disk.target = ui->deviceLineEdit->text().toStdString();
    _disk.driverType = ui->imageTypeLineEdit->text().toStdString();

    return _disk;
}

void DomainStorageWidget::on_busComboBox_currentIndexChanged(const QString &arg1)
{
    QMap<std::string, std::string> copy(_domainProperties->storageTargetMap);
    QString device = copy.value(arg1.toStdString()).c_str();
    QChar c1 ('a');
    QChar c2 (c1.unicode() + _domainProperties->nbDisksMax-1);
    QRegExp rx(QString("%1[%2-%3]").arg(device).arg(c1).arg(c2));
    QRegExpValidator *validator = new QRegExpValidator(rx);
    ui->deviceLineEdit->setValidator(validator);
}

void DomainStorageWidget::displayErrors(int error)
{
    if (error & HnsClient::deviceIsInvalid)
        ui->deviceLabel->setStyleSheet("QLabel{color:red;}");
}

void DomainStorageWidget::clearErrors()
{
    ui->deviceLabel->setStyleSheet("");
}

} // namespace Hnv
