/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "dynamipsconfigurationpage.h"
#include "ui_dynamipsconfigurationpage.h"
#include "ressources.h"
#include "simulationmodewidget.h"
#include "topologywidget.h"

// hynesim hnsclient headers
#include <libhnsclient/entry.h>
#include <libhnsclient/entity.h>

// Qt headers
#include <QToolButton>
#include <QMessageBox>

namespace Hnv {
/*******************************************************************************
* DynamipsRouterPage
*/
DynamipsConfigurationPage::DynamipsConfigurationPage(HnsClient::CatalogItem *entry, Hns::EntryPropertiesPtr entryProperties, QWidget *parent, bool readOnly, bool defined)
    : EntryPropertiesPage(entry,entryProperties, parent, readOnly , defined)
    , ui(new Ui::DynamipsConfigurationPage),
      _dynamipsEntity(NULL)
{
    ui->setupUi(this);

    ui->pushButtonFileToRom->setText("\u2192");
    ui->pushButtonRomToFile->setText("\u2190");
    ui->reloadFile->setIcon(QIcon(Ressources::themedPath(Constants::ICON_REFRESH)));
    ui->reloadRom->setIcon(QIcon(Ressources::themedPath(Constants::ICON_REFRESH)));

    //TODO: we have to access to the entity by topologymanager
    HnsClient::DynamipsRouterEntry * dynamipsrouter = qobject_cast<HnsClient::DynamipsRouterEntry *>(_sourceEntry);
    if(dynamipsrouter == NULL) return;

    ui->applyOnBoot->setChecked(dynamipsrouter->getApplyOnBoot());
    ui->autoExtract->setChecked(dynamipsrouter->getAutoExtract());
    QString configuration64 = dynamipsrouter->getConfiguration();

    QString configuration = QByteArray::fromBase64(configuration64.toAscii()).data();
    ui->textEditFile->setPlainText(configuration);

    bool canLoadRom = false;
    if (_defined) {
        TopologyWidget * topologywidget = SimulationModeWidget::instance()->topologyWidget();
        QList<HnsClient::GenericEntity *> entitesSelected =  topologywidget->getCurrentEntitiesSelected();
        //at this time the properties dialogue is modal, change when properties's box change to no modal.
        if (entitesSelected.size() == 1) {
            _dynamipsEntity =  qobject_cast<HnsClient::DynamipsRouterEntity *>(entitesSelected.at(0));
            if (_dynamipsEntity) {
                ui->textEditRom->setPlainText(tr("Loading..."));
                //auto extract to see the real configuration
                connect(_dynamipsEntity, SIGNAL(pushConfigurationSignal()), this, SLOT(_pushConfigurationSlot()) );
                _dynamipsEntity->extractConfiguration(this, SLOT(_printConfigurationFrom64(::std::string)));
                canLoadRom = true;
            }
        }
    }
    if (!canLoadRom) {
        ui->reloadRom->setDisabled(true);
        ui->pushButtonFileToRom->setDisabled(true);
        ui->pushButtonRomToFile->setDisabled(true);
    }
    _disableWidgetsWhenNecessary(!isWriteable() || _readOnly);
}

DynamipsConfigurationPage::~DynamipsConfigurationPage()
{
    delete ui;
}

void DynamipsConfigurationPage::initFocus()
{
}

QToolButton *DynamipsConfigurationPage::getPageButton()
{
    return _generatePageButton(tr("Configuration"), QIcon(Ressources::themedPath(Constants::ICON_PLATFORM_MONITOR)));
}


/*******************************************************************************
* DynamipsRouterPage - Protected
*/
void DynamipsConfigurationPage::_disableWidgetsWhenNecessary(bool necessary)
{
    ui->reloadFile->setDisabled(necessary || !ui->reloadFile->isEnabled());
    ui->textEditFile->setDisabled(necessary || ui->textEditFile->isReadOnly());
    ui->pushButtonFileToRom->setDisabled(necessary || !ui->pushButtonFileToRom->isEnabled());
    ui->pushButtonRomToFile->setDisabled(necessary || !ui->pushButtonRomToFile->isEnabled());
    ui->reloadRom->setDisabled(necessary || !ui->reloadRom->isEnabled());
    ui->applyOnBoot->setDisabled(necessary || !ui->applyOnBoot->isEnabled());
    ui->autoExtract->setDisabled(necessary || !ui->autoExtract->isEnabled());
}


void DynamipsConfigurationPage::_validate(bool &isModified, bool &isInvalid)
{
    HnsClient::DynamipsRouterEntry * dynamipsrouter = qobject_cast<HnsClient::DynamipsRouterEntry *>(_sourceEntry);

    QByteArray data = QByteArray(ui->textEditFile->toPlainText().toAscii());
    QByteArray data64 = data.toBase64();
    _computeValidationState(dynamipsrouter->validateConfiguration(data64.data()),isModified, isInvalid);
    _computeValidationState(dynamipsrouter->validateApplyOnBot(ui->applyOnBoot->isChecked()), isModified, isInvalid);
    _computeValidationState(dynamipsrouter->validateAutoExtract(ui->autoExtract->isChecked()), isModified, isInvalid);
}



void DynamipsConfigurationPage::on_reloadRom_clicked()
{
    _pushConfigurationSlot();

}


void DynamipsConfigurationPage::on_reloadFile_clicked()
{
    HnsClient::DynamipsRouterEntry * dynamipsrouter = qobject_cast<HnsClient::DynamipsRouterEntry *>(_sourceEntry);
    QString configuration = QByteArray::fromBase64(dynamipsrouter->getConfiguration().toAscii()).data();
    ui->textEditFile->setPlainText(configuration);
}

void DynamipsConfigurationPage::on_pushButtonRomToFile_clicked()
{
    ui->textEditFile->setPlainText(ui->textEditRom->toPlainText());
}

void DynamipsConfigurationPage::on_pushButtonFileToRom_clicked()
{
    if (_dynamipsEntity) {
        QByteArray data = QByteArray(ui->textEditFile->toPlainText().toAscii());
        QByteArray data64 = data.toBase64();
        _dynamipsEntity->pushConfiguration(data64.data());
    }
}


void DynamipsConfigurationPage::_pushConfigurationSlot()
{
    if (_dynamipsEntity) {
        ui->textEditRom->setPlainText(tr("Loading..."));
        _dynamipsEntity->extractConfiguration(this, SLOT(_printConfigurationFrom64(::std::string)));
    }
}


void DynamipsConfigurationPage::_printConfigurationFrom64(::std::string string)
{
    QString configuration64 = QString::fromStdString(string);
    if (configuration64.isEmpty()) {
        QMessageBox::critical(this, tr("Can't find the configuration"), tr("Can't find the configuration"));
    }
    QString configuration = QByteArray::fromBase64(configuration64.toAscii()).data();
    ui->textEditRom->setPlainText(configuration);
}

} // namespace Hnv
