/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "dynamipsrouterpage.h"
#include "ui_dynamipsrouterpage.h"
#include "ressources.h"
#include "hyneviewsession.h"

// hynesim common headers
#include "common/hns.h"

// hynesim hnsclient headers
#include <libhnsclient/macaddressmanager.h>
#include <libhnsclient/entry.h>

// Qt headers
#include <QComboBox>
#include <QLabel>
#include <QToolButton>

namespace Hnv {

/*******************************************************************************
* DynamipsSlotWidget
*/
DynamipsSlotWidget::DynamipsSlotWidget(int numSlot,int sizeLabel , QWidget *parent)
    : QWidget(parent)
    , _numSlot(numSlot)
{
    QHBoxLayout *hLayout = new QHBoxLayout();
    hLayout->setContentsMargins(0, 0, 0, 0);

    hLayout->setSizeConstraint(QLayout::SetDefaultConstraint);
    _labelSlot = new QLabel(QString("Slot %1:").arg(_numSlot), this);
    _labelSlot->setMinimumSize(QSize(sizeLabel, 0));
    hLayout->addWidget(_labelSlot);


    _comboboxSlot = new QComboBox(this);
    _comboboxSlot->setMinimumSize(QSize(150,0));
    hLayout->addWidget(_comboboxSlot);

    _labelInterface  = new QLabel(this);
    QSizePolicy sizePolicyInterface(QSizePolicy::Preferred, QSizePolicy::Preferred);
    sizePolicyInterface.setHorizontalStretch(0);
    sizePolicyInterface.setVerticalStretch(0);
    sizePolicyInterface.setHeightForWidth(_labelInterface->sizePolicy().hasHeightForWidth());
    _labelInterface->setMinimumSize(QSize(0, 0));
    _labelInterface->setMaximumSize(QSize(110, 16777215));
    _labelInterface->setSizePolicy(sizePolicyInterface);
    hLayout->addWidget(_labelInterface);
    connect(_comboboxSlot, SIGNAL(currentIndexChanged(QString)), this, SLOT(_comboboxChanged(QString)));

    QSpacerItem * spacer = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);
    hLayout->addItem(spacer);

    setLayout(hLayout);
}

void DynamipsSlotWidget::fillCombobox(::Hns::ModuleMap listModule, const QString & defaultValue)
{
    _listModule = listModule;
    if (listModule.size() == 0){
        _comboboxSlot->addItem("");
        return;
    }
    ::Hns::ModuleMap::iterator itor(listModule.begin());
    bool isMb = false;
    int goodIndex = -1;
    int currentIndex = 0;
    while(itor != listModule.end()) {
        isMb |= itor->second.isMB;
        QString slot = QString::fromStdString(itor->second.name);
        _comboboxSlot->addItem(slot);
        if(defaultValue == slot)
            goodIndex = currentIndex;
        currentIndex++;
        itor++;
    }
    if(!isMb) {
        _comboboxSlot->insertItem(0,"");
        goodIndex++;
    }

    if (goodIndex == -1)
        _comboboxSlot->setCurrentIndex(0);
    else
        _comboboxSlot->setCurrentIndex(goodIndex);
}

bool DynamipsSlotWidget::getModuleInfo(int & position, QString & name)
{
    if(_comboboxSlot->currentText().isEmpty())
        return false;
    position = _numSlot;
    name = _comboboxSlot->currentText();
    return true;
}

void DynamipsSlotWidget::_comboboxChanged(QString newValue)
{
    _labelInterface->clear();
    ::std::string moduleName = newValue.toStdString();
    if(_listModule.find(moduleName) != _listModule.end()) {
        _labelInterface->setText(QString(""+tr("Interface(s): %1").arg(_listModule[moduleName].interfaces)));
    }
}

DynamipsSlotWidget::~DynamipsSlotWidget()
{
}

/*******************************************************************************
* DynamipsRouterPage
*/
DynamipsRouterPage::DynamipsRouterPage(HnsClient::CatalogItem *entry, Hns::EntryPropertiesPtr entryProperties, QWidget *parent, bool readOnly, bool defined)
    : EntryPropertiesPage(entry,entryProperties, parent, readOnly , defined)
    , _multiple(4)
    , ui(new Ui::DynamipsRouterPage)
    , _change(false)
{
    ui->setupUi(this);
    ui->frame->setHidden(true);
    _macAddressManager = HyneviewSession::instance()->macAddressManager();
    HnsClient::DynamipsRouterEntry * dynamipsrouter = qobject_cast<HnsClient::DynamipsRouterEntry *>(_sourceEntry);
    if(dynamipsrouter == NULL) return;

    HyneviewSession::instance()->getDynamipsRouterData(_dataDynamipsMap,_dynamipsRouterModuleMap);

    ui->tbGenerateRandom->setIcon(QIcon(Ressources::themedPath(Constants::ICON_GENERATE_MAC_ADRESS)));
    ui->lineEditIdle->setText(dynamipsrouter->getIdlePc());
    ui->lineEditIdle->setButtonTooltip(tr("Do not change this value unless you know what you are doing."));
    ui->lineEditIdle->setWarningOnFocus(true);
    ui->lineEditAddrMac->setText(dynamipsrouter->getMacAdress());

    _fillPlatform(dynamipsrouter->getRouter());
    connect(ui->comboBoxPlatform, SIGNAL(currentIndexChanged(QString)), this, SLOT(_fillChassis(QString)));

    ui->memorySpinBox->setMultiple(_multiple);
    ui->memorySpinBox->setSingleStep(_multiple);
    _displayMemory(dynamipsrouter->getMemory());
    connect(ui->memoryHorizontalSlider, SIGNAL(valueChanged(int)), this, SLOT(_getMemoryValue(int)));
    connect(ui->memorySpinBox, SIGNAL(valueChanged(int)), this, SLOT(_setMemoryValue(int)));
    ui->lineEdit->setText(dynamipsrouter->getImage());
    _change = true;
    _disableWidgetsWhenNecessary(!isWriteable() || _readOnly || (_defined && !_canModifiedWhenDefined));
}

DynamipsRouterPage::~DynamipsRouterPage()
{
    delete ui;
}

void DynamipsRouterPage::initFocus()
{
}

QToolButton *DynamipsRouterPage::getPageButton()
{
    return _generatePageButton(tr("System"), QIcon(Ressources::themedPath(Constants::ICON_SYSTEM_SETTINGS)));
}

/*******************************************************************************
* DynamipsRouterPage - Protected
*/
QString DynamipsRouterPage::_moduleName(Hns::DynamipsRouterModule module)
{
    return QString::fromStdString(module.name);
}

void DynamipsRouterPage::_fillPlatform(const QString & plateform)
{
    QMapIterator<QString, Hns::DataDynamipsRouter> itor(_dataDynamipsMap);
    while(itor.hasNext()) {
        itor.next();
        ui->comboBoxPlatform->addItem(itor.key());
    }
    ui->comboBoxPlatform->setCurrentIndex(ui->comboBoxPlatform->findText(plateform));
    _fillChassis(plateform);
}

void DynamipsRouterPage::_fillChassis(const QString & plateform)
{
    disconnect(ui->comboBoxChassis);
    ui->comboBoxChassis->clear();
    ::Hns::DataDynamipsRouter dataDynamipsRouter = _dataDynamipsMap.value(plateform);
    ::Hns::ChassisMap::iterator itor = dataDynamipsRouter.availableChassis.begin();
    while(itor != dataDynamipsRouter.availableChassis.end()) {
        ui->comboBoxChassis->addItem(QString::fromStdString(itor->second.name));
        itor++;
    }

    if(!_change) {
        HnsClient::DynamipsRouterEntry * dynamipsrouter = qobject_cast<HnsClient::DynamipsRouterEntry *>(_sourceEntry);
        ui->comboBoxChassis->setCurrentIndex(ui->comboBoxChassis->findText(dynamipsrouter->getChassis()));
    }
    _fillSlot(ui->comboBoxChassis->currentText());
    connect(ui->comboBoxChassis, SIGNAL(currentIndexChanged(QString)), this, SLOT(_fillSlot(QString)));
}

void DynamipsRouterPage::_fillSlot(const QString & chassis)
{
    Hns::ModulePositionMap modulemap;
    if(!_change) {
        HnsClient::DynamipsRouterEntry * dynamipsrouter = qobject_cast<HnsClient::DynamipsRouterEntry *>(_sourceEntry);
        modulemap = dynamipsrouter->getModulesMap();
        adjustSize();
    }
    _clearLayout(ui->scrollAreaWidgetContents->layout());

    _setDynamipsSlotWidget.clear();

    //list of slot
    ::Hns::DataDynamipsRouter dataDynamipsRouter = _dataDynamipsMap.value(ui->comboBoxPlatform->currentText());
    ::Hns::DynamipsRouterChassis dynamipsRouterChassis = dataDynamipsRouter.availableChassis[chassis.toStdString()];

    int nbSlot = dynamipsRouterChassis.numSlots;

    for (int i = 0 ; i < nbSlot; i++) {

        DynamipsSlotWidget * dynamipsSlotWidget = new DynamipsSlotWidget(i,
                                                                         ui->gridLayout->itemAtPosition(0,0)->widget()->size().width(),
                                                                         ui->scrollAreaWidgetContents);
        ::Hns::ModuleMap moduleMap;
        if(i == 0 && dynamipsRouterChassis.availableModules.size() != 0)
            moduleMap = dynamipsRouterChassis.availableModules;
        else
            moduleMap = dataDynamipsRouter.availableModules;

        QString valueSlot("");
        if(modulemap.find(i) != modulemap.end()) {
            valueSlot = QString::fromStdString(modulemap[i].name);
        }

        dynamipsSlotWidget->fillCombobox(moduleMap,valueSlot);
        _setDynamipsSlotWidget.insert(dynamipsSlotWidget);
        ui->scrollAreaWidgetContents->layout()->addWidget(dynamipsSlotWidget);
    }

    QSpacerItem * spacer = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);
    ui->scrollAreaWidgetContents->layout()->addItem(spacer);
}

void DynamipsRouterPage::_disableWidgetsWhenNecessary(bool necessary)
{
    ui->comboBoxPlatform->setDisabled(necessary);
    ui->comboBoxChassis->setDisabled(necessary);
    ui->lineEditAddrMac->setDisabled(necessary);
    ui->tbGenerateRandom->setDisabled(necessary);
    ui->memoryHorizontalSlider->setDisabled(necessary);
    ui->memorySpinBox->setDisabled(necessary);
    ui->lineEdit->setDisabled(necessary);
    foreach(DynamipsSlotWidget * dynawidget,_setDynamipsSlotWidget) {
        dynawidget->setDisabled(necessary);
    }
    ui->lineEditIdle->setDisabled(necessary);
}

void DynamipsRouterPage::_validate(bool &isModified, bool &isInvalid)
{
    HnsClient::DynamipsRouterEntry * dynamipsrouter = qobject_cast<HnsClient::DynamipsRouterEntry *>(_sourceEntry);

    _computeValidationState(dynamipsrouter->validateIdle(ui->lineEditIdle->text()),isModified, isInvalid);
    if (isInvalid == HnsClient::Invalid) {
        ui->lIdle->setStyleSheet("QLabel{color:red;}");
    } else {
        ui->lIdle->setStyleSheet("");
    }

    _computeValidationState(dynamipsrouter->validatePlatform(ui->comboBoxPlatform->currentText()),isModified, isInvalid);
//    from router, we must change nvram and needSetChassis
    _computeValidationState(dynamipsrouter->validateNvRam(_dataDynamipsMap.value(ui->comboBoxPlatform->currentText()).nvram),isModified, isInvalid);
    _computeValidationState(dynamipsrouter->validateNeedSetChassis(_dataDynamipsMap.value(ui->comboBoxPlatform->currentText()).needSetChassis),isModified, isInvalid);
    _computeValidationState(dynamipsrouter->validateChassis(ui->comboBoxChassis->currentText()),isModified, isInvalid);
    _computeValidationState(dynamipsrouter->validateMacAdress(ui->lineEditAddrMac->text()),isModified, isInvalid);

    QString nameModule;
    int position;
    ::Hns::ModulePositionMap positionMap;
    foreach(DynamipsSlotWidget * dynamipsWidget, _setDynamipsSlotWidget) {
        if(dynamipsWidget->getModuleInfo(position,nameModule)) {
            positionMap[position] = _dynamipsRouterModuleMap[nameModule];
        }
    }
    _computeValidationState(dynamipsrouter->validateModules(positionMap),isModified, isInvalid);
    _computeValidationState(dynamipsrouter->validateMemory(ui->memorySpinBox->value()),isModified, isInvalid);
    _computeValidationState(dynamipsrouter->validateImage(ui->lineEdit->text()),isModified, isInvalid);
}

void DynamipsRouterPage::on_tbGenerateRandom_clicked()
{
    _macAddressManager->generateRandomAddress("Cisco Systems, Inc.", this, SLOT(_generateRandomAddressSlot(QString)));
}

void DynamipsRouterPage::_generateRandomAddressSlot(QString address)
{
    ui->lineEditAddrMac->setText(address);
}

void DynamipsRouterPage::_displayMemory(int memory)
{
    ui->memoryHorizontalSlider->setValue(memory);
    ui->memorySpinBox->setValue(memory);
}

void DynamipsRouterPage::_setMemoryValue(int value)
{
    int min = ui->memoryHorizontalSlider->minimum();
    int max = ui->memoryHorizontalSlider->maximum();

    if (value>max)
        value = max;
    if (value < min)
        value = min;

    ui->memoryHorizontalSlider->setValue(value);
}

void DynamipsRouterPage::_getMemoryValue(int value)
{
    ui->memorySpinBox->setValue(value);;
}

void DynamipsRouterPage::_clearLayout(QLayout * layout)
{
    QLayoutItem *item;
    while(layout->count()>0) {
        item = layout->takeAt(0);
        if (item->layout()) {
            _clearLayout(item->layout());
            item->layout()->deleteLater();
        }
        if (item->widget()) {
            item->widget()->deleteLater();
        }
        delete item;

    }
}

void DynamipsRouterPage::on_memoryHorizontalSlider_actionTriggered(int action)
{
    if (action == QSlider::SliderMove)
        ui->memoryHorizontalSlider->setSliderPosition(Hns::getNearestMultipleOf(ui->memoryHorizontalSlider->sliderPosition(), _multiple));
}

} // namespace Hnv
