/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef DYNAMIPSROUTERPAGE_H
#define DYNAMIPSROUTERPAGE_H

// Project headers
#include "hnv_entryproperties.h"

// Generated hynesim headers
#include "common/dynamipsrouterentry.h"

// Qt headers
#include <QString>
#include <QMap>
#include <QSet>
#include <QWidget>

class QComboBox;
class QLabel;

namespace Ui {
class DynamipsRouterPage;
} // namespace Ui

namespace HnsClient {
class MacAddressManager;
} // namespace HnsClient

namespace Hnv {
class DynamipsSlotWidget : public QWidget
{
    Q_OBJECT

public:
    explicit DynamipsSlotWidget(int numSlot,int sizeLabel ,QWidget *parent=0);
    virtual ~DynamipsSlotWidget();
    void fillCombobox(::Hns::ModuleMap listModule, const QString & defaultValue);
    bool getModuleInfo(int &position, QString &name);

private slots:
    void _comboboxChanged(QString);

protected:
    int _numSlot;
    QLabel * _labelSlot;
    QComboBox * _comboboxSlot;
    QLabel * _labelInterface;
    ::Hns::ModuleMap _listModule;
};

/*******************************************************************************
* DynamipsRouterPage
*/
class DynamipsRouterPage : public EntryPropertiesPage
{
    Q_OBJECT

public:
    Q_INVOKABLE explicit DynamipsRouterPage(HnsClient::CatalogItem *entry, Hns::EntryPropertiesPtr entryProperties, QWidget *parent = 0, bool readOnly = false, bool defined = false);
    ~DynamipsRouterPage();
    virtual void initFocus();
    virtual QToolButton *getPageButton();

protected:
    void _disableWidgetsWhenNecessary(bool necessary);
    virtual void _validate(bool &isModified, bool &isInvalid);
    QString _moduleName(Hns::DynamipsRouterModule module);
    void _fillPlatform(const QString & plateform);
    void _displayMemory(int memory);

private slots:
    void _fillChassis(const QString & plateform);
    void _fillSlot(const QString & chassis);
    void on_tbGenerateRandom_clicked();
    void _generateRandomAddressSlot(QString address);
    void _clearLayout(QLayout * layout);
    void _setMemoryValue(int value);
    void _getMemoryValue(int value);
    void on_memoryHorizontalSlider_actionTriggered(int action);

private:
    int _multiple;
    HnsClient::MacAddressManager *_macAddressManager;
    Ui::DynamipsRouterPage *ui;
    QMap<QString, Hns::DataDynamipsRouter > _dataDynamipsMap;
    QMap<QString, Hns::DynamipsRouterModule > _dynamipsRouterModuleMap;
    QSet<DynamipsSlotWidget *> _setDynamipsSlotWidget;
    bool _change;
};

} // namespace Hnv
#endif // DYNAMIPSROUTERPAGE_H
