/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "extrapage.h"
#include "ui_extrapage.h"
#include "ressources.h"

// hynesim hnsclient headers
#include <libhnsclient/entry.h>

// Qt headers
#include <QToolButton>

namespace Hnv {

/*******************************************************************************
* Extrapage
*/
ExtraPage::ExtraPage(HnsClient::CatalogItem *entry, Hns::EntryPropertiesPtr entryProperties, QWidget *parent, bool readOnly, bool defined, bool editabletemplate)
    : EntryPropertiesPage(entry,entryProperties, parent, readOnly , defined, editabletemplate)
    , ui(new Ui::ExtraPage)
{
    ui->setupUi(this);
    _displayCreationDate(_entry()->getCreationDate());
    _displayModificationDate(_entry()->getModificationDate());
    _displayUUID(_entry()->getUUID());
    _displayPath(_entry()->getPath());
    HnsClient::DomainEntry * domain = qobject_cast<HnsClient::DomainEntry *>(_sourceEntry);
    HnsClient::SwitchEntry * switchEntry = qobject_cast<HnsClient::SwitchEntry*>(_sourceEntry);
    HnsClient::EntityEntry * entityEntry = qobject_cast<HnsClient::EntityEntry *>(_sourceEntry);
    HnsClient::DynamipsRouterEntry * dynamipsRouterEntry = qobject_cast<HnsClient::DynamipsRouterEntry *>(_sourceEntry);
    ui->nameLabel->setText(_sourceEntry->getShortName());
    _displayTemplate(entityEntry);
    ui->archTitleLabel->setVisible(false);
    ui->archLabel->setVisible(false);
    if(domain) {
        Hns::OperatingSystem opSystem = domain->getOperatingSystem();
        ui->typeHubLabel->setText(domain->getType());
        ui->typeHubTitle->setText(tr("Type:"));
        ui->iconLabel->setPixmap(Ressources::getTypeIcon(domain->getType()));
        ui->stpLabel->setVisible(false);
        ui->stpTitle->setVisible(false);
        QString arch = QString::fromStdString(opSystem.arch);
        if (!arch.isEmpty()) {
            ui->archTitleLabel->setVisible(true);
            ui->archLabel->setVisible(true);
            ui->archLabel->setText(arch);
        }
    } else if(switchEntry){
        ui->iconLabel->setHidden(true);
        ui->typeHubTitle->setText(tr("Use as hub:"));
        if(switchEntry->getIsHub())
            ui->typeHubLabel->setText(tr("Yes"));
        else
            ui->typeHubLabel->setText(tr("No"));
        ui->stpTitle->setText(tr("Spanning Tree Protocol:"));
        if(switchEntry->getStpEnabled())
            ui->stpLabel->setText(tr("Enabled"));
        else
            ui->stpLabel->setText(tr("Disabled"));
    } else if(dynamipsRouterEntry) {
        ui->stpLabel->setVisible(false);
        ui->stpTitle->setVisible(false);
        ui->typeHubLabel->setVisible(false);
        ui->typeHubTitle->setVisible(false);
        ui->iconLabel->setPixmap(QPixmap(Ressources::themedPath(Constants::GRAPHIC_DYNAMIPSROUTER)));
    }
    else {
        ui->stpLabel->setVisible(false);
        ui->stpTitle->setVisible(false);
        ui->typeHubLabel->setVisible(false);
        ui->typeHubTitle->setVisible(false);
        ui->iconLabel->setPixmap(QPixmap(Ressources::themedPath(Constants::ICON_TOPOLOGY)));
    }
}

ExtraPage::~ExtraPage()
{
    delete ui;
}

HnsClient::Entry * ExtraPage::_entry()
{
    return dynamic_cast<HnsClient::Entry*>(_sourceEntry);
}

void ExtraPage::initFocus()
{
    ui->creationDateLabel->setFocus();
}

QToolButton *ExtraPage::getPageButton()
{
    return _generatePageButton(tr("Details"), QIcon(Ressources::themedPath(Constants::ICON_DETAILS)));
}

/*******************************************************************************
* Extrapage - Protected
*/

void ExtraPage::_disableWidgetsWhenNecessary(bool necessary)
{
    Q_UNUSED(necessary)
}

void ExtraPage::_displayCreationDate(QString date)
{
    ui->creationDateLabel->setText(date);
}

void ExtraPage::_displayModificationDate(QString date)
{
    ui->lastModLabel->setText(date);
}

void ExtraPage::_displayUUID(QString uuid)
{
    ui->uuidLabel->setText(uuid);
}

void ExtraPage::_displayPath(QString path)
{
    ui->pathLabel->setText(path);
}

void ExtraPage::_validate(bool &isModified, bool &isInvalid)
{
    _computeValidationState(_validatePath(), isModified, isInvalid);
    HnsClient::EntityEntry * entityentry = qobject_cast<HnsClient::EntityEntry *>(_sourceEntry);
     if(entityentry){
         _computeValidationState(entityentry->validateTemplate(ui->checkBoxTemplate->isChecked()), isModified, isInvalid);
     }
}

int ExtraPage::_validatePath()
{
    return HnsClient::Unmodified;
}

void ExtraPage::_displayTemplate(HnsClient::EntityEntry * entityEntry){
    if(!entityEntry){
        ui->checkBoxTemplate->setVisible(false);
    }
    else {
        if (entityEntry->isModel()) {
            ui->checkBoxTemplate->setEnabled(false);
         } else {
            ui->checkBoxTemplate->setChecked(entityEntry->isTemplate());
            if(!_editabletemplate || !isWriteable())
                ui->checkBoxTemplate->setDisabled(true);
        }
    }
}

bool ExtraPage::_isTopologyEntry()
{
    QString className = QString::fromStdString(_entry()->entry()->ice_id());
    if (className == QString::fromStdString(Hns::TopologyEntry::ice_staticId()))
        return true;
    return false;
}

} // namespace Hnv
