/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "generalpage.h"
#include "ui_generalpage.h"
#include "ressources.h"

// hynesim hnsclient headers
#include <libhnsclient/entry.h>

// Qt headers
#include <QToolButton>

namespace Hnv {

/*******************************************************************************
* GeneralPage
*/
GeneralPage::GeneralPage(HnsClient::CatalogItem *entry, Hns::EntryPropertiesPtr entryProperties, QWidget *parent, bool readOnly, bool defined)
    : EntryPropertiesPage(entry, entryProperties,parent,readOnly, defined)
    , _ui(new Ui::GeneralPage)
{
    _ui->setupUi(this);
    _canModifiedWhenDefined = true;
    _displayShortName(_sourceEntry->getShortName());
    _displayLongName(_sourceEntry->getLongName());
    _displayDescription(_sourceEntry->getDescription());
    _displayTags(_sourceEntry->getTagList());
    _disableWidgetsWhenNecessary(!isWriteable() || _readOnly || (_defined && !_canModifiedWhenDefined));
}

GeneralPage::~GeneralPage()
{
    delete _ui;
}

void GeneralPage::initFocus()
{
    _ui->shortNameLabel->setFocus();
}

QToolButton *GeneralPage::getPageButton()
{
    return _generatePageButton(tr("General"), QIcon(Ressources::themedPath(Constants::ICON_PROPERTIES)));
}

/*******************************************************************************
* GeneralPage - Protected
*/
void GeneralPage::_disableWidgetsWhenNecessary(bool necessary)
{
    _ui->shortNameLabel->setReadOnly(necessary);
    _ui->descriptionTextEdit->setReadOnly(necessary);
    _ui->longNameLabel->setReadOnly(necessary);
    _ui->longNameLabel->setReadOnly(necessary);
    _ui->tagsLabel->setReadOnly(necessary);
}

void GeneralPage::_displayShortName(QString shortName)
{
    _ui->shortNameLabel->setText(shortName);
}

void GeneralPage::_displayLongName(QString longName)
{
    _ui->longNameLabel->setText(longName);
}

void GeneralPage::_displayDescription(QString description)
{
    _ui->descriptionTextEdit->setPlainText(description);
}

void GeneralPage::_displayTags(QString sourceTagList)
{
    _ui->tagsLabel->setText(sourceTagList);
}

HnsClient::ValidationResult GeneralPage::_validateShortName()
{
    HnsClient::ValidationResult result = _sourceEntry->validateShortName(_ui->shortNameLabel->text());
    if (result == HnsClient::Invalid)
        _ui->shortNameTitleLabel->setStyleSheet("QLabel{color:red;}");
    else
        _ui->shortNameTitleLabel->setStyleSheet("");
    return result;
}

HnsClient::ValidationResult GeneralPage::_validateLongName()
{
    return _sourceEntry->validateLongName(_ui->longNameLabel->text());
}

HnsClient::ValidationResult GeneralPage::_validateDescription()
{
    return _sourceEntry->validateDescription(_ui->descriptionTextEdit->toPlainText());
}

HnsClient::ValidationResult GeneralPage::_validateTags()
{
    return _sourceEntry->validateTagList(_ui->tagsLabel->text());
}

void GeneralPage::_validate(bool &isModified, bool &isInvalid)
{
    _computeValidationState(_validateShortName(), isModified, isInvalid);
    _computeValidationState(_validateLongName(), isModified, isInvalid);
    _computeValidationState(_validateDescription(), isModified, isInvalid);
    _computeValidationState(_validateTags(), isModified, isInvalid);
}

} // namespace Hnv
