/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef ENTRYPROPERTIES_H
#define ENTRYPROPERTIES_H

// Generated hynesim headers
#include "common/entryproperties.h"
#include "common/usermanager.h"

// hynesim hnsclient headers
#include <libhnsclient/common.h>

// Qt headers
#include <QWidget>
#include <QDialog>
#include <QMap>

class QToolButton;

namespace Ui {
class EntryProperties;
} // namespace Ui

namespace HnsClient {
class CatalogItem;
} // namespace HnsClient

namespace Hnv {

/*******************************************************************************
* EntryPropertiesPage
*/
class EntryPropertiesPage : public QWidget
{
    Q_OBJECT
public:
    explicit EntryPropertiesPage(HnsClient::CatalogItem *entry, Hns::EntryPropertiesPtr entryProperties, QWidget *parent = 0, bool readOnly = false, bool define = false, bool editabletemplate = false);
    virtual QString getItemName();
    virtual bool isNew();
    virtual bool isWriteable();
    virtual HnsClient::ValidationResult validate();
    virtual void initFocus() = 0;
    virtual void init();
    virtual QToolButton *getPageButton() = 0;

protected:
    QToolButton *_generatePageButton(const QString &title, const QIcon &icon, bool enabled = true);

    bool _isWriteable;
    bool _canModifiedWhenDefined;
    bool _readOnly;
    bool _defined;
    bool _editabletemplate;
    HnsClient::CatalogItem *_sourceEntry;
    Hns::UserPtr _currentUser;
    virtual void _validate(bool &isModified, bool &isInvalid) = 0;
    void _computeValidationState(int result, bool &isModified, bool &isInvalid);
    virtual void _disableWidgetsWhenNecessary(bool necessary) = 0;
    
signals:
    void validationRequested();
    void warningMessageChanged(QString);
};

/*******************************************************************************
* EntryProperties
*/
class EntryProperties : public QDialog
{
    Q_OBJECT
public:
    explicit EntryProperties(HnsClient::CatalogItem *entry,  QWidget *parent = 0, bool readOnly = false, bool defined = false, bool editabletemplate = false, bool displayAccessRight = true);
    ~EntryProperties();

    template <typename T> T *getEntryPropertiesPage()
    {
        foreach (EntryPropertiesPage *page, _propertyPages) {
            if (qobject_cast<T *>(page) != NULL) {
                T *valueToReturn = qobject_cast<T *>(page);
                return valueToReturn;
            }
        }
        return NULL;
    }

protected:
    void _initPages();
    void _initDialog();
    virtual void showEvent(QShowEvent * );

protected slots:
    void _adjustSizeToContent(int widgetIndex);
    void on_buttonBox_accepted();
    void _getPropertiesSlot(::Hns::EntryPropertiesPtr);
    void _setWarningMessage(QString message);

private:
    QMap<std::string, std::string> _storageMap;
    Ui::EntryProperties *_ui;
    HnsClient::CatalogItem *_sourceEntry;
    EntryPropertiesPage *_firstWidget;
    Hns::EntryPropertiesPtr _entryProperty;
    QList<EntryPropertiesPage *> _propertyPages;
    bool _readOnly;
    bool _defined;
    bool _editabletemplate;
    bool _displayAccessPage;
};

} // namespace Hnv
#endif // ENTRYPROPERTIES_H
