/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "hybridnetcardpage.h"
#include "ui_hybridnetcardpage.h"
#include "ressources.h"

// hynesim hnsclient headers
#include <libhnsclient/entry.h>

// Qt headers
#include <QToolButton>

namespace Hnv {
/*******************************************************************************
* HybridNetcardPage
*/
HybridNetcardPage::HybridNetcardPage(HnsClient::CatalogItem *entry, Hns::EntryPropertiesPtr entryProperties, QWidget *parent, bool readOnly, bool defined)
    : EntryPropertiesPage(entry,entryProperties, parent, readOnly , defined)
    , ui(new Ui::HybridNetcardPage)
{
    ui->setupUi(this);
    HnsClient::HybridNetcardEntry * hybridNetcardEntry = qobject_cast<HnsClient::HybridNetcardEntry*>(_sourceEntry);
    if (hybridNetcardEntry) {
        _displayNodeName(hybridNetcardEntry->getNodeName());
        _displayPlugIndex(hybridNetcardEntry->getPlugIndex());
    }
    _disableWidgetsWhenNecessary(!isWriteable() || _readOnly || (_defined && !_canModifiedWhenDefined));

}

HybridNetcardPage::~HybridNetcardPage()
{
    delete ui;
}

void HybridNetcardPage::initFocus()
{
}

QToolButton *HybridNetcardPage::getPageButton()
{
    return _generatePageButton(tr("System"), QIcon(Ressources::themedPath(Constants::ICON_SYSTEM_SETTINGS)));
}


/*******************************************************************************
* HybridNetcardPage - Protected
*/
void HybridNetcardPage::_disableWidgetsWhenNecessary(bool necessary)
{
    ui->leNodeName->setDisabled(necessary);
    ui->lePlugIndex->setDisabled(necessary);
}
void HybridNetcardPage::_displayNodeName(QString nodeName)
{
    ui->leNodeName->setText(nodeName);
}

void HybridNetcardPage::_displayPlugIndex(QString plugIndex)
{
    ui->lePlugIndex->setText(plugIndex);
}

void HybridNetcardPage::_validate(bool &isModified, bool &isInvalid)
{
    HnsClient::HybridNetcardEntry *entry = qobject_cast<HnsClient::HybridNetcardEntry *>(_sourceEntry);
    _computeValidationState(entry->validateNodeName(ui->leNodeName->text()), isModified, isInvalid);
    _computeValidationState(entry->validatePlugIndex(ui->lePlugIndex->text()), isModified, isInvalid);
}

} // namespace Hnv
