// Project headers
#include "propertypagefactory.h"
#include "hnv_entryproperties.h"
#include "generalpage.h"
#include "extrapage.h"
#include "domainpage.h"
#include "domainnetworkpage.h"
#include "domainstoragepage.h"
#include "switchpage.h"
#include "vlanpage.h"
#include "hybridnetcardpage.h"
#include "wifiappage.h"
#include "wirepage.h"
#include "dynamipsrouterpage.h"
#include "dynamipsconfigurationpage.h"
#include "accessrightpage.h"
#include "topologygatepage.h"
#include "topologypage.h"

// hynesim common headers
#include "common/initializer.h"

// Generated hynesim headers
#include "common/domainentry.h"
#include "common/wireentry.h"
#include "common/switchentry.h"
#include "common/hybridnetcardentry.h"
#include "common/wifiapentry.h"
#include "common/dynamipsrouterentry.h"
#include "common/topologygateentry.h"

// hynesim hnsclient headers
#include <libhnsclient/entry.h>

namespace Hnv {

HNS_STATIC_HASH_INITIALIZER(_pagesHash, PagesHash, QString, QList<QMetaObject>)
PagesHash::PagesHash()
{
    QList<QMetaObject> pageList;
    pageList.append(DomainPage::staticMetaObject);
    pageList.append(DomainStoragePage::staticMetaObject);
    pageList.append(DomainNetworkPage::staticMetaObject);
    insert(QString::fromStdString(Hns::DomainEntry::ice_staticId()), pageList);
    pageList.clear();
    pageList.append(WirePage::staticMetaObject);
    insert(QString::fromStdString(Hns::WireEntry::ice_staticId()), pageList);
    pageList.clear();
    pageList.append(SwitchPage::staticMetaObject);
    pageList.append(VlanPage::staticMetaObject);
    insert(QString::fromStdString(Hns::SwitchEntry::ice_staticId()), pageList);
    pageList.clear();
    pageList.append(HybridNetcardPage::staticMetaObject);
    insert(QString::fromStdString(Hns::HybridNetcardEntry::ice_staticId()), pageList);
    pageList.clear();
    pageList.append(WifiAPPage::staticMetaObject);
    insert(QString::fromStdString(Hns::WifiAPEntry::ice_staticId()), pageList);
    pageList.clear();
    pageList.append(DynamipsRouterPage::staticMetaObject);
    pageList.append(DynamipsConfigurationPage::staticMetaObject);
    insert(QString::fromStdString(Hns::DynamipsRouterEntry::ice_staticId()), pageList);
    pageList.clear();
    pageList.append(TopologyGatePage::staticMetaObject);
    insert(QString::fromStdString(Hns::TopologyGateEntry::ice_staticId()), pageList);
    pageList.clear();
    pageList.append(TopologyPage::staticMetaObject);
    insert(QString::fromStdString(Hns::TopologyEntry::ice_staticId()), pageList);
}

PropertyPageFactory::PropertyPageFactory(HnsClient::CatalogItem *entry, QObject *parent) :    
     QObject(parent)
   , _catalogItem(entry)
{
}

QList<EntryPropertiesPage *> PropertyPageFactory::getPropertiesPages(QWidget *parent, Hns::EntryPropertiesPtr entryProperties, bool readOnly, bool defined, bool editabletemplate, bool displayAccessPage)
{
    QList<EntryPropertiesPage *> pages;
    pages.append(new GeneralPage(_catalogItem, entryProperties, parent, readOnly));
    HnsClient::Entry *entry = qobject_cast<HnsClient::Entry *>(_catalogItem);
    QString entryClassName;
    if (entry)
        entryClassName = QString::fromStdString(entry->entry()->ice_id());
    QList<QMetaObject> extraPageList;
    if(_pagesHash()->contains(entryClassName)) {
        extraPageList = _pagesHash()->value(entryClassName);
    }
    else if (entry){
        QVector<std::string> ice_ids = QVector<std::string>::fromStdVector(entry->entry()->ice_ids());
        for(int i = ice_ids.size() - 1; i >= 0; i--) {
            QString parentClassName = QString::fromStdString(ice_ids.at(i));
            if (_pagesHash()->contains(parentClassName)) {
                extraPageList = _pagesHash()->value(parentClassName);
                _pagesHash()->insert(entryClassName, extraPageList);
            }
        }
    }
    foreach(QMetaObject metaObj, extraPageList) {
        QGenericArgument sourceItemArg = Q_ARG(HnsClient::CatalogItem *, _catalogItem);
        QGenericArgument parentArg = Q_ARG(QWidget *, parent);
        QGenericArgument ROArg = Q_ARG(bool , readOnly);
        QGenericArgument definedArg = Q_ARG(bool , defined);
        QGenericArgument entryPropertiesArg=Q_ARG(Hns::EntryPropertiesPtr , entryProperties);
        QObject * obj = metaObj.newInstance(sourceItemArg, entryPropertiesArg, parentArg, ROArg, definedArg);
        if (obj) {
            EntryPropertiesPage * pageObj = qobject_cast<EntryPropertiesPage *>(obj);
            if(pageObj){
                pages.append(pageObj);
            }
        }
    }

    if (displayAccessPage)
        pages.append(new AccessRightPage(_catalogItem, entryProperties, parent, readOnly, defined));
    if (entry)
        pages.append(new ExtraPage(_catalogItem, entryProperties, parent, readOnly, defined, editabletemplate));
    return pages;
}

} // namespace Hnv
