/******************************************************************************
**
** Copyright (C) 2003-2013 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "topologygatepage.h"
#include "ui_topologygatepage.h"
#include "ressources.h"

// hynesim hnsclient headers
#include <libhnsclient/entry.h>

namespace Hnv {

TopologyGatePage::TopologyGatePage(HnsClient::CatalogItem *entry, Hns::EntryPropertiesPtr entryProperties, QWidget *parent, bool readOnly, bool defined)
    : EntryPropertiesPage(entry, entryProperties, parent, readOnly , defined)
    , ui(new Ui::TopologyGatePage)
{
    ui->setupUi(this);
    HnsClient::TopologyGateEntry *topologyGateEntry = qobject_cast<HnsClient::TopologyGateEntry *>(_sourceEntry);
    if (topologyGateEntry) {
        _displayNetworkName(topologyGateEntry->getNetworkName());
    }
    _disableWidgetsWhenNecessary(!isWriteable() || _readOnly || (_defined && !_canModifiedWhenDefined));
}

TopologyGatePage::~TopologyGatePage()
{
    delete ui;
}

void TopologyGatePage::initFocus()
{
}

//QListWidgetItem *TopologyGatePage::getPager()
//{
//    return new QListWidgetItem(QIcon(Ressources::themedPath(Constants::ICON_SYSTEM_SETTINGS)), tr("System"));
//}

QToolButton *TopologyGatePage::getPageButton()
{
    return _generatePageButton(tr("System"), QIcon(Ressources::themedPath(Constants::ICON_SYSTEM_SETTINGS)));
}

/*******************************************************************************
* TopologyGatePage - Protected
*/
void TopologyGatePage::_displayNetworkName(QString name)
{
    ui->networkNameLineEdit->setText(name);
}

void TopologyGatePage::_validate(bool &isModified, bool &isInvalid)
{
    HnsClient::TopologyGateEntry *entry = qobject_cast<HnsClient::TopologyGateEntry *>(_sourceEntry);
    _computeValidationState(entry->validateNetworkName(ui->networkNameLineEdit->text()), isModified, isInvalid);
}

void TopologyGatePage::_disableWidgetsWhenNecessary(bool necessary)
{
    ui->networkNameLineEdit->setDisabled(necessary);
}

} // namespace Hnv
