/******************************************************************************
**
** Copyright (C) 2003-2015 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "topologypage.h"
#include "ui_topologypage.h"
#include "ressources.h"

// hynesim hnsclient headers
#include <libhnsclient/entry.h>

namespace Hnv {

TopologyPage::TopologyPage(HnsClient::CatalogItem *entry, Hns::EntryPropertiesPtr entryProperties, QWidget *parent, bool readOnly, bool defined)
    : EntryPropertiesPage(entry, entryProperties, parent, readOnly, defined)
    , ui(new Ui::TopologyPage)
{
    ui->setupUi(this);
    HnsClient::TopologyEntry *topologyEntry = qobject_cast<HnsClient::TopologyEntry *>(_sourceEntry);
    ui->persistentCheckBox->setChecked(topologyEntry->isPersistent());
    ui->persistentCheckBox->setDisabled(readOnly);
}

TopologyPage::~TopologyPage()
{
    delete ui;
}

void TopologyPage::initFocus()
{
}

QToolButton *TopologyPage::getPageButton()
{
    return _generatePageButton(tr("Topology"), QIcon(Ressources::themedPath(Constants::ICON_SYSTEM_SETTINGS)));
}

void TopologyPage::_disableWidgetsWhenNecessary(bool necessary)
{
    Q_UNUSED(necessary)
}

void TopologyPage::_validate(bool &isModified, bool &isInvalid)
{
    HnsClient::TopologyEntry *entry = qobject_cast<HnsClient::TopologyEntry *>(_sourceEntry);
    _computeValidationState(entry->validateIsPersistent(ui->persistentCheckBox->isChecked()), isModified, isInvalid);
}

} // namespace Hnv
