/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "userselectordialog.h"
#include "ui_userselectordialog.h"
#include "treesortfilterproxymodel.h"
#include "usersmodel.h"
#include "groupsmodel.h"

// Qt headers
#include <QModelIndex>

namespace Hnv {

UserSelectorDialog::UserSelectorDialog(UsersModel *usersModel, QWidget *parent)
    : QDialog(parent)
    , ui(new Ui::UserSelectorDialog)
    , _usersModel(usersModel)
    , _selectedItemID(0)
{
    ui->setupUi(this);
    UsersFilterProxyModel * filterModel = new UsersFilterProxyModel(this);
    filterModel->setSourceModel(_usersModel);
    filterModel->setDynamicSortFilter(true);
    _sortModel = new TreeSortFilterProxyModel(this);
    QList<int> filterColumns;
    filterColumns << 1 << 2 << 3;
    _sortModel->setFilterColumnsList(filterColumns);
    _sortModel->setDynamicSortFilter(true);
    _sortModel->setSortCaseSensitivity(Qt::CaseInsensitive);
    _sortModel->setFilterCaseSensitivity(Qt::CaseInsensitive);
    _sortModel->setSourceModel(filterModel);
    ui->tableView->setModel(_sortModel);
    ui->tableView->hideColumn(0);
    ui->tableView->hideColumn(4);
    ui->tableView->hideColumn(5);
    ui->tableView->hideColumn(6);
    ui->tableView->hideColumn(7);
    ui->tableView->resizeColumnsToContents();
}

UserSelectorDialog::UserSelectorDialog(GroupsModel *groupsModel, QList<qlonglong> userGroupList, QWidget *parent)
    : QDialog(parent)
    , ui(new Ui::UserSelectorDialog)
    , _groupsModel(groupsModel)
    , _selectedItemID(0)
{
    ui->setupUi(this);
    GroupsProxyModel * filterModel = new GroupsProxyModel(this);
    filterModel->setAllowedGroupList(userGroupList);
    filterModel->setDynamicSortFilter(true);
    filterModel->setSourceModel(_groupsModel);
    _sortModel = new TreeSortFilterProxyModel(this);
    QList<int> filterColumns;
    filterColumns << 1 << 4;
    _sortModel->setFilterColumnsList(filterColumns);
    _sortModel->setDynamicSortFilter(true);
    _sortModel->setSortCaseSensitivity(Qt::CaseInsensitive);
    _sortModel->setFilterCaseSensitivity(Qt::CaseInsensitive);
    _sortModel->setSourceModel(filterModel);
    ui->tableView->setModel(_sortModel);
    ui->tableView->hideColumn(0);
    ui->tableView->hideColumn(2);
    ui->tableView->hideColumn(3);
    ui->tableView->resizeColumnsToContents();
}

UserSelectorDialog::~UserSelectorDialog()
{
    delete ui;
}

void UserSelectorDialog::setCurrentItemID(qlonglong itemID)
{
    QAbstractItemModel * model = ui->tableView->model();
    QModelIndexList matchList = model->match(model->index(0,0), Qt::DisplayRole, QVariant(itemID));
    if (matchList.count())
        ui->tableView->selectionModel()->setCurrentIndex(matchList.first(), QItemSelectionModel::ClearAndSelect | QItemSelectionModel::Rows);
}

qlonglong UserSelectorDialog::getSelectedItemID()
{
    return _selectedItemID;
}

void UserSelectorDialog::on_lineEdit_textChanged(const QString &arg1)
{
    _sortModel->setFilterFixedString(arg1);
}

void UserSelectorDialog::on_tableView_doubleClicked(const QModelIndex &index)
{
    _acceptWithIndex(index);
}

void UserSelectorDialog::on_buttonBox_accepted()
{
    QModelIndex selected = ui->tableView->currentIndex();
    if (selected.isValid()) {
        _acceptWithIndex(selected);
    }
}

void UserSelectorDialog::_acceptWithIndex(QModelIndex index)
{
    _selectedItemID = index.data(Qt::UserRole).toLongLong();
    accept();
}

} // namespace Hnv
