/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/
#ifndef VLANMODEL_H
#define VLANMODEL_H

// Generated hynesim headers
#include "common/switchentry.h"

// Qt headers
#include <QAbstractItemModel>
#include <QStyledItemDelegate>

namespace Hnv {

struct PortLocalInformation
{
    int ID;
    bool tagged;
    bool operator ==(const PortLocalInformation &a) const
    {
        if (a.ID == this->ID && a.tagged == this->tagged)
            return true;
        return false;
    }
    bool operator <(const PortLocalInformation &a) const
    {
        if (this->ID < a.ID)
            return true;
        return false;
    }
};

struct PortItem
{
    int ID;
    int state;
};

struct VlanLocalInformation
{
    QString name;
    int ID;
    QString description;
    QList<PortLocalInformation> Ports;
};

/*****************************VLAN MODEL******************************/
class VlanModel : public QAbstractTableModel
{
    Q_OBJECT
public:
    enum {
        COLUMN_ID,
        COLUMN_PORTS,
        COLUMN_EDIT,
        COLUMN_DELETE,
        COLUMN_SIZE
    };
    enum {
        STATE_NONE,
        STATE_TAGGED,
        STATE_UNTAGGED
    };

    explicit VlanModel(Hns::VlanMap vlanList, QObject *parent =0);
    ~VlanModel();

    void clear();
    void addVlan(VlanLocalInformation vlan);
    void removeVlan(int ID);

    int rowCount(const QModelIndex &parent) const;
    int columnCount(const QModelIndex &parent) const;
    QVariant data(const QModelIndex &index, int role) const;
    QVariant headerData(int section, Qt::Orientation orientation, int role) const;
    bool setData(const QModelIndex &index, const QVariant &value, int role);
    QList<int > getIDs();
    QMap<Ice::Int, Hns::Vlan> getVlanMap();
    VlanLocalInformation getVlan(int id);
    QHash<int, VlanLocalInformation> getVlans();
    void updatePorts(QList<PortItem> oldList, QList<PortItem> newList, int ID);
    void portNumberChange(int n);
    void setPortsNb(int n);
    int portsNb();

private:
    QList<int> _vlanList;
    QHash <int, VlanLocalInformation> _vlanPositionHash;
    QString _parsePorts(QList<PortLocalInformation> ports) const;
    int _portsNb;

};

/**************************VLAN DELEGATE*********************/
class DelegateVlan : public QStyledItemDelegate
{
    Q_OBJECT
public:
    DelegateVlan(int deleteColumn, int labelColumn, int editColumn, QSize iconSize, QObject *parent = 0);
    ~DelegateVlan();
    void paint(QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index) const;
    QSize sizeHint ( const QStyleOptionViewItem & option, const QModelIndex & index ) const;

private:
    int _deleteColumn;
    int _labelColumn;
    int _editColumn;
    QSize _iconSize;

};

} // namespace Hnv
#endif // VLANMODEL_H
