/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "vlanpage.h"
#include "ui_vlanpage.h"
#include "ressources.h"
#include "newvlanform.h"
#include "switchpage.h"
#include "vlanmodel.h"
#include "vlanpopup.h"

// Generated hynesim headers
#include "common/switchentryproperties.h"

// hynesim hnsclient headers
#include <libhnsclient/entry.h>

// Qt headers
#include <QToolButton>
#include <QWidget>
#include <QModelIndex>
#include <QMessageBox>

namespace Hnv {

VlanPage::VlanPage(HnsClient::CatalogItem *entry, Hns::EntryPropertiesPtr entryProperties, QWidget *parent, bool readOnly, bool defined)
    : EntryPropertiesPage(entry,entryProperties, parent,readOnly, defined )
    , ui(new Ui::VlanPage)
{
    ui->setupUi(this);
    this->adjustSize();
    this->setMinimumSize(size());

    _parentPage = qobject_cast<EntryProperties*>(parentWidget());

    HnsClient::SwitchEntry * switchEntry = qobject_cast<HnsClient::SwitchEntry*>(_sourceEntry);
    if (switchEntry) {
        _model = new VlanModel(switchEntry->getVlan(), this);
        _model->setPortsNb(switchEntry->getPorts());
    }
    ui->tableViewVLan->setModel(_model);
    DelegateVlan *delegate = new DelegateVlan(VlanModel::COLUMN_DELETE, VlanModel::COLUMN_PORTS, VlanModel::COLUMN_EDIT, ui->tableViewVLan->iconSize(), this);
    ui->tableViewVLan->setItemDelegate(delegate);

    ui->tableViewVLan->horizontalHeader()->resizeSection(0,50); // resize the ID column

    ui->tableViewVLan->horizontalHeader()->setStretchLastSection(false);
    ui->tableViewVLan->horizontalHeader()->setResizeMode(VlanModel::COLUMN_PORTS, QHeaderView::Stretch);

    ui->tableViewVLan->horizontalHeader()->setResizeMode(VlanModel::COLUMN_EDIT, QHeaderView::Fixed);
    ui->tableViewVLan->horizontalHeader()->resizeSection(VlanModel::COLUMN_EDIT, 32);

    ui->tableViewVLan->horizontalHeader()->setResizeMode(VlanModel::COLUMN_DELETE, QHeaderView::Fixed);
    ui->tableViewVLan->horizontalHeader()->resizeSection(VlanModel::COLUMN_DELETE, 32);
    ui->tableViewVLan->viewport()->setAttribute(Qt::WA_Hover);
    
    Hns::SwitchEntryPropertiesPtr switchProperties = Hns::SwitchEntryPropertiesPtr::dynamicCast(entryProperties);
    if (switchProperties->vlanSupported == false) {
        ui->pushButtonAdd->setDisabled(true);
        ui->tableViewVLan->setDisabled(true);
    }
}

VlanPage::~VlanPage()
{
    delete _model;
    delete ui;
}

void VlanPage::initFocus()  {

}

QToolButton *VlanPage::getPageButton()
{
    return _generatePageButton(tr("VLAN"), QIcon(Ressources::themedPath(Constants::ICON_VLAN)));
}

void VlanPage::_disableWidgetsWhenNecessary(bool necessary)
{
    Q_UNUSED(necessary)
}

void VlanPage::_validate(bool &isModified, bool &isInvalid)
{
    HnsClient::SwitchEntry *entry = qobject_cast<HnsClient::SwitchEntry *>(_sourceEntry);
    _computeValidationState(entry->validateVlan(_model->getVlanMap()), isModified, isInvalid);
}

void VlanPage::on_tableViewVLan_pressed(const QModelIndex &index)
{
    if (index.column() == VlanModel::COLUMN_DELETE) {
        int id = _model->data(index, Qt::UserRole+4).toInt();
        if (id != 0) {
            int question = QMessageBox::warning(this, tr("Confirmation required"), tr("Are you sure to delete this VLAN ?"), QMessageBox::No | QMessageBox::Yes, QMessageBox::No);
            if (question == QMessageBox::Yes) {
                _model->removeVlan(id);
            }
        }
    }
    else if (index.column() == VlanModel::COLUMN_EDIT) {
        VlanPopup *popup = new VlanPopup(_model->getVlan(_model->data(index, Qt::UserRole+4).toInt())
                                         , _model->portsNb(), this);
        if (popup->exec() == QDialog::Accepted) {
            _model->setData(index, popup->getName() , Qt::UserRole+2);
            _model->setData(index, popup->getDescription(), Qt::UserRole+1);
            _model->updatePorts(popup->getOldPortList(), popup->getNewPortList(), _model->data(index, Qt::UserRole + 4).toInt());

        }
        delete popup;
    }
}


void VlanPage::on_pushButtonAdd_clicked()
{
    QList<int > ids = _model->getIDs();
    NewVlanForm *dialog = new NewVlanForm(ids, this);
    if (dialog->exec() == QDialog::Accepted) {
        VlanLocalInformation vlan;
        vlan.ID = dialog->getID();
        vlan.name = dialog->getName();
        vlan.description = dialog->getDescription();
        _model->addVlan(vlan);
    }
    delete dialog;
}

void VlanPage::on_tableViewVLan_doubleClicked(const QModelIndex &index)
{
    if (index.column() != VlanModel::COLUMN_DELETE) {
        on_tableViewVLan_pressed(_model->index(index.row(), VlanModel::COLUMN_EDIT));
    }
}

void VlanPage::portNumberChangedSlot(int numberPort)
{
    _model->portNumberChange(numberPort);
}

void VlanPage::init()
{
    SwitchPage *switchPage = _parentPage->getEntryPropertiesPage<SwitchPage>();
    if (!switchPage)
        return;
    connect(switchPage, SIGNAL(portNumberChanged(int)), this, SLOT(portNumberChangedSlot(int)));
}

} // namespace Hnv
