#ifndef VLANPOPUP_H
#define VLANPOPUP_H

// Project headers
#include "vlanmodel.h"

// Qt headers
#include <QDialog>

class QModelIndex;
class QComboBox;
class QSignalMapper;

namespace Ui {
class VlanPopup;
} // namespace Ui

namespace Hnv {

class PortModel;
class PortDelegate;

class VlanPopup : public QDialog
{
    Q_OBJECT
public:
    explicit VlanPopup(VlanLocalInformation vlan, int nbPorts, QWidget *parent = 0);
    ~VlanPopup();
    QString getName();
    QString getDescription();
    QList<PortItem> getOldPortList();
    QList<PortItem> getNewPortList();

    
private slots:
    void on_buttonBox_accepted();

private:
    Ui::VlanPopup *ui;
    int _id;
    QString _name;
    QString _description;
    QList<PortItem> _oldList;
    VlanLocalInformation _vlan;
    int _nbPorts;
    PortModel *_portModel;
    PortDelegate *_portDelegate;
};


/******************************PORT MODEL**************************/
class PortModel : public QAbstractTableModel
{
    Q_OBJECT
public:
    enum {
        COLUMN_PORT_ID,
        COLUMN_PORT_STATE,
        COLUMN_PORT_SIZE
    };

    explicit PortModel(int nbPorts, int idVlan, QObject *parent =0);
    ~PortModel();

    void init(VlanLocalInformation vlan);
    void clear();
    void addPort(PortItem port);
    void removePort(int ID);

    int rowCount(const QModelIndex &parent) const;
    int columnCount(const QModelIndex &parent) const;
    QVariant headerData(int section, Qt::Orientation orientation, int role) const;
    QVariant data(const QModelIndex &index, int role) const;
    bool setData(const QModelIndex &index, const QVariant &value, int role);
    QList<PortItem> getPortList();

public slots:
    void cbChanged(int port, QComboBox *cb);
private:
    QList<PortItem> _portList;
    QHash <int, int> _portPositionHash;
    int _idVlan;

};
/*****************************PORT DELEGATE***************************/
class PortDelegate : public QStyledItemDelegate
{
    Q_OBJECT
public:
    explicit PortDelegate(QList<PortItem> portList, int idVlan, QObject *parent = 0);
    ~PortDelegate();
    QWidget * createEditor(QWidget *parent, const QStyleOptionViewItem &option, const QModelIndex &index) const;
    virtual void setModelData(QWidget *editor, QAbstractItemModel *model, const QModelIndex &index) const;
    virtual void setEditorData(QWidget *editor, const QModelIndex &index) const;
    virtual void updateEditorGeometry(QWidget *editor, const QStyleOptionViewItem &option, const QModelIndex &index) const;
public slots:
    void comboBoxChanged(int idx);
signals:
    void cbChanged(int id, QComboBox *cb);
private:
    mutable QHash<int, QComboBox *> _statePorts;
    QSignalMapper * _mapper;
    QList<PortItem> _portList;
    int _idVlan;
};

} // namespace Hnv
#endif // VLANPOPUP_H
