/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "wifiappage.h"
#include "ui_wifiappage.h"
#include "ressources.h"
#include "hyneviewsession.h"

// hynesim hnsclient headers
#include <libhnsclient/macaddressmanager.h>
#include <libhnsclient/entry.h>

// Qt headers
#include <QToolButton>
#include <QWidget>

namespace Hnv {

WifiAPPage::WifiAPPage(HnsClient::CatalogItem *entry, Hns::EntryPropertiesPtr entryProperties, QWidget *parent, bool readOnly /*= false*/, bool defined /*= false*/)
    : EntryPropertiesPage(entry,entryProperties, parent, readOnly , defined),
    ui(new Ui::WifiAPPage)
{
    ui->setupUi(this);
    _macAddressManager = HyneviewSession::instance()->macAddressManager();
    _wifiApEntry = qobject_cast<HnsClient::WifiAPEntry*>(_sourceEntry);
    ui->tbGenerateRandom->setIcon(QIcon(Ressources::themedPath(Constants::ICON_GENERATE_MAC_ADRESS)));
    QString chan;
    if (_wifiApEntry) {
        ui->cbAuthType->addItem("NONE","NONE");
        ui->cbAuthType->addItem("WEP","WEP");
        ui->cbAuthType->addItem("WPA1","WPA1");
        ui->cbAuthType->addItem("WPA2","WPA2");
        ui->cbAuthType->addItem("WPA12","WPA12");
        for (int i = 1 ; i < 12 ; i++) {
            chan = QString::number(i) + " : 2." + QString::number(407 + i*5) + " GHz";
            ui->cbChannel->addItem(chan, chan);
        }
        _displayChannel(_wifiApEntry->getChannel());
        _displaySsid(_wifiApEntry->getSsid());
        _displayPassphrase(_wifiApEntry->getPassword());
        _activateAuthType(_wifiApEntry->getAuthentificationType());
        _displayMacAddress(_wifiApEntry->getMacAdress());
        _activateHiddenState(_wifiApEntry->getIsHidden());
    }
    _activateCheckBoxHexaWhenNeeded();
    _disableWidgetsWhenNecessary(!isWriteable() || _readOnly || (_defined && !_canModifiedWhenDefined));
}

WifiAPPage::~WifiAPPage()
{
    delete ui;
}

void WifiAPPage::initFocus()
{

}

QToolButton *WifiAPPage::getPageButton()
{
    return _generatePageButton(tr("Settings"), QIcon(Ressources::themedPath(Constants::ICON_SYSTEM_SETTINGS)));
}

/*************************************SLOTS*********************************/
void WifiAPPage::_disableWidgetsWhenNecessary(bool necessary)
{
    ui->leSsid->setDisabled(necessary);
    ui->lePassphrase->setDisabled(necessary);
    ui->leMacAdress->setDisabled(necessary);
    ui->cbAuthType->setDisabled(necessary);
    ui->checkBoxHidden->setDisabled(necessary);
    ui->cbHexa->setDisabled(necessary);
    ui->tbGenerateRandom->setDisabled(necessary);
    ui->cbChannel->setDisabled(necessary);
}

void WifiAPPage::on_cbHexa_stateChanged(int arg1)
{
    Q_UNUSED(arg1);
    on_cbAuthType_currentIndexChanged(ui->cbAuthType->itemData(ui->cbAuthType->currentIndex()).toString());
}

void WifiAPPage::on_cbAuthType_currentIndexChanged(const QString &arg1)
{
    on_lePassphrase_textChanged(ui->lePassphrase->text());
    ui->labelPassphraseInfo->setText("");
    if (arg1 == QString::fromStdString("NONE")){
        ui->lePassphrase->hide();
        ui->labelPassphrase->hide();
        ui->labelPassphraseInfo->hide();
        ui->cbHexa->hide();
    } else {
        ui->labelPassphrase->show();
        ui->labelPassphraseInfo->show();
        ui->lePassphrase->show();
        ui->cbHexa->show();
        ui->lePassphrase->setEnabled(isWriteable() && (!_readOnly) && (!_defined ||_canModifiedWhenDefined ));
    }
    if (arg1 == QString::fromStdString("WEP")){
        if (ui->cbHexa->isChecked()){
            ui->labelPassphraseInfo->setText(tr("%1, %2 or %3 hexadecimal digits").arg(Constants::WEP_SIZE_HEXA_40_BITS).arg(Constants::WEP_SIZE_HEXA_128_BITS).arg(Constants::WEP_SIZE_HEXA_152_BITS));
        } else {
            ui->labelPassphraseInfo->setText(tr("%1, %2 or %3 characters").arg(Constants::WEP_SIZE_DEC_40_BITS).arg(Constants::WEP_SIZE_DEC_128_BITS).arg(Constants::WEP_SIZE_DEC_152_BITS));
        }
    } else if (arg1 == QString::fromStdString("WPA1") || arg1 == QString::fromStdString("WPA2") || arg1 == QString::fromStdString("WPA12")){
        if (ui->cbHexa->isChecked()){
            ui->labelPassphraseInfo->setText(tr("%1 hexadecimal digits").arg(Constants::WPA_SIZE_HEXA));
        } else {
            ui->labelPassphraseInfo->setText(tr("between %1 and %2 characters").arg(Constants::WPA_SIZE_MIN).arg(Constants::WPA_SIZE_MAX));
        }
    }
}

void WifiAPPage::_displaySsid(QString ssid)
{
    ui->leSsid->setText(ssid);
}

void WifiAPPage::_displayPassphrase(QString passphrase)
{
    ui->lePassphrase->setText(passphrase);
}

void WifiAPPage::_displayMacAddress(QString macAdress)
{
    ui->leMacAdress->setText(macAdress);
}

void WifiAPPage::_displayChannel(int channel)
{
    ui->cbChannel->setCurrentIndex(channel - 1);
}

void WifiAPPage::_activateAuthType(QString authType)
{
    ui->cbAuthType->setCurrentIndex(ui->cbAuthType->findData(authType));
    on_cbAuthType_currentIndexChanged(authType);
}

void WifiAPPage::_activateHiddenState(bool hidden)
{
    ui->checkBoxHidden->setChecked(hidden);
}

void WifiAPPage::on_lePassphrase_textChanged(const QString &arg1)
{
    bool result = _wifiApEntry->checkPassphrase(arg1,ui->cbAuthType->itemData(ui->cbAuthType->currentIndex()).toString(),ui->cbHexa->isChecked());
    if (!result){
        ui->labelPassphrase->setStyleSheet("QLabel{color:red;}");
    } else {
        ui->labelPassphrase->setStyleSheet("");
    }
}

void WifiAPPage::_activateCheckBoxHexaWhenNeeded()
{
    QString passphrase = ui->lePassphrase->text();
    QString authTypeRequested = ui->cbAuthType->itemData(ui->cbAuthType->currentIndex()).toString();
    if (passphrase.size() == Constants::WEP_SIZE_HEXA_128_BITS || passphrase.size() == Constants::WEP_SIZE_HEXA_152_BITS || passphrase.size() == Constants::WEP_SIZE_HEXA_40_BITS) {
        if (authTypeRequested == "WEP"){
            if (_wifiApEntry->isHex(passphrase,passphrase.size())){
                ui->cbHexa->setChecked(true);
            } else {
                ui->cbHexa->setChecked(false);
            }
        }
    } else if (passphrase.size() == Constants::WPA_SIZE_HEXA){
        if (authTypeRequested.startsWith("WPA")){
            if (_wifiApEntry->isHex(passphrase,passphrase.size())){
                ui->cbHexa->setChecked(true);
            } else {
                ui->cbHexa->setChecked(false);
            }
        }
    }
}

void WifiAPPage::_validate(bool &isModified, bool &isInvalid)
{
    _computeValidationState(_validateSSID(),isModified,isInvalid);
    _computeValidationState(_validateMacAddress(),isModified,isInvalid);
    _computeValidationState(_validateAuthType(),isModified,isInvalid);
    _computeValidationState(_validateChannel(), isModified, isInvalid);
    _computeValidationState(_wifiApEntry->validateHidden(ui->checkBoxHidden->checkState()),isModified,isInvalid);
    if(!ui->lePassphrase->isHidden())
        _computeValidationState(_validatePassphrase(),isModified,isInvalid);
}

void WifiAPPage::on_tbGenerateRandom_clicked()
{
    _macAddressManager->generateRandomAddress("", this, SLOT(_generateRandomAddressSlot(QString)));
}

void WifiAPPage::_generateRandomAddressSlot(QString address)
{
    ui->leMacAdress->setText(address);
}

HnsClient::ValidationResult WifiAPPage::_validateSSID()
{
    HnsClient::ValidationResult result = _wifiApEntry->validateSsid(ui->leSsid->text());
    if (result == HnsClient::Invalid)
        ui->ssidLabel->setStyleSheet("QLabel{color:red;}");
    else
        ui->ssidLabel->setStyleSheet("");
    return result;
}

HnsClient::ValidationResult WifiAPPage::_validateMacAddress()
{
    HnsClient::ValidationResult result = _wifiApEntry->validateMacAddress(ui->leMacAdress->text());
    if (result == HnsClient::Invalid)
        ui->labelMAcAdress->setStyleSheet("QLabel{color:red;}");
    else
        ui->labelMAcAdress->setStyleSheet("");
    return result;
}

HnsClient::ValidationResult WifiAPPage::_validateAuthType()
{
    HnsClient::ValidationResult result = _wifiApEntry->validateAuthentificationType(ui->cbAuthType->itemData(ui->cbAuthType->currentIndex()).toString());
    if (result == HnsClient::Invalid)
        ui->labelAuthentificationType->setStyleSheet("QLabel{color:red;}");
    else
        ui->labelAuthentificationType->setStyleSheet("");
    return result;
}

HnsClient::ValidationResult WifiAPPage::_validatePassphrase()
{
    HnsClient::ValidationResult result = _wifiApEntry->validatePassword(ui->lePassphrase->text(),ui->cbHexa->isChecked());
    if (result == HnsClient::Invalid)
        ui->labelPassphrase->setStyleSheet("QLabel{color:red;}");
    else
        ui->labelPassphrase->setStyleSheet("");
    return result;
}

HnsClient::ValidationResult WifiAPPage::_validateChannel()
{
    HnsClient::ValidationResult result = _wifiApEntry->validateChannel(ui->cbChannel->currentIndex() + 1);
    if(result == HnsClient::Invalid)
        ui->labelChannel->setStyleSheet("QLabel{color:red;}");
    else
        ui->labelChannel->setStyleSheet("");
    return result;
}

} // namespace Hnv
