/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef WIFIAPPAGE_H
#define WIFIAPPAGE_H

// Project headers
#include "hnv_entryproperties.h"

class QWidget;

namespace Ui {
class WifiAPPage;
} // namespace Ui

namespace HnsClient {
class WifiAPEntry;
class MacAddressManager;
} // namespace HnsClient

namespace Hnv {

class WifiAPPage : public EntryPropertiesPage
{
    Q_OBJECT

public:
    Q_INVOKABLE explicit WifiAPPage(HnsClient::CatalogItem *entry, Hns::EntryPropertiesPtr entryProperties, QWidget *parent = 0, bool readOnly = false, bool defined = false) ;
    ~WifiAPPage();
    virtual void initFocus();
    virtual QToolButton *getPageButton();

protected:
    void _disableWidgetsWhenNecessary(bool necessary);
    virtual void _validate(bool &isModified, bool &isInvalid);
    void _displaySsid(QString ssid);
    void _displayPassphrase(QString passphrase);
    void _displayMacAddress(QString macAdress);
    void _displayChannel(int channel);
    void _activateAuthType(QString authType);
    void _activateHiddenState(bool hidden);
    void _activateCheckBoxHexaWhenNeeded();
    HnsClient::ValidationResult _validateSSID();
    HnsClient::ValidationResult _validateMacAddress();
    HnsClient::ValidationResult _validateAuthType();
    HnsClient::ValidationResult _validatePassphrase();
    HnsClient::ValidationResult _validateChannel();

private slots:
    void on_cbAuthType_currentIndexChanged(const QString &arg1);
    void on_lePassphrase_textChanged(const QString &arg1);
    void on_cbHexa_stateChanged(int arg1);
    void on_tbGenerateRandom_clicked();
    void _generateRandomAddressSlot(QString address);

private:
    Ui::WifiAPPage *ui;
    HnsClient::WifiAPEntry *_wifiApEntry;
    HnsClient::MacAddressManager *_macAddressManager;
};

} // namespace Hnv
#endif // WIFIAPPAGE_H
