/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "filterlineedit.h"
#include "ressources.h"

// Qt headers
#include <QPainter>

namespace Hnv {

/*******************************************************************************
* FilterLineEdit
*/
FilterLineEdit::FilterLineEdit(QWidget *parent)
    : ButtonLineEdit(Ressources::themedPath(Constants::ICON_CLEAR), 16 ,parent)
{
    connect(this, SIGNAL(buttonClicked()), this, SLOT(clear()));
    connect(this, SIGNAL(textChanged(const QString&)), this, SLOT(_updateClearButton(const QString&)));
    QMargins margins = textMargins();
    int frameWidth = style()->pixelMetric(QStyle::PM_DefaultFrameWidth);
    margins.setLeft(20 + frameWidth);
    setTextMargins(margins);
}

void FilterLineEdit::paintEvent(QPaintEvent *event)
{
    ButtonLineEdit::paintEvent(event);
    QPainter localPainter(this);
    QImage searchImg(Ressources::themedPath(Constants::ICON_SEARCH));
    QStyleOptionFrameV2 panel;
    initStyleOption(&panel);
    localPainter.setBackgroundMode(Qt::TransparentMode);
    QRect lineEditContentsRect = style()->subElementRect(QStyle::SE_LineEditContents, &panel, this);
    int y = lineEditContentsRect.y();
    if (searchImg.height() < lineEditContentsRect.height())
        y += (lineEditContentsRect.height() - searchImg.height()) / 2;
    else
        searchImg = searchImg.scaled(lineEditContentsRect.height(), lineEditContentsRect.height());
    localPainter.drawImage(lineEditContentsRect.x() + 2, y, searchImg);
}

/*******************************************************************************
* FilterLineEdit - Private slots
*/

void FilterLineEdit::_updateClearButton(const QString &text)
{
    setButtonVisible(!text.isEmpty());
}

} // namespace Hnv
