/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef GROUPSMANAGERWIDGET_H
#define GROUPSMANAGERWIDGET_H

// Qt headers
#include <QWidget>
#include <QItemSelection>

class QModelIndex;

namespace Ui {
class GroupsManagerWidget;
} // namespace Ui

namespace HnsClient {
class UserManager;
} // namespace HnsClient

namespace Hnv {

class GroupsModel;
class TreeSortFilterProxyModel;
class FilterLineEdit;

/*******************************************************************************
* GroupsManagerWidget
*/

class GroupsManagerWidget : public QWidget
{
    Q_OBJECT
public:
    explicit GroupsManagerWidget(QWidget *parent = 0);
    ~GroupsManagerWidget();

    void setAccessor(HnsClient::UserManager *accessor);
    void setGroupsModel(GroupsModel *groupsModel = 0);

protected slots:
    void _selectionChanged(QItemSelection selected, QItemSelection deselected);

private slots:
    void on_action_Delete_group_triggered();
    void on_action_Edit_group_triggered();
    void on_action_New_group_triggered();

    void _enableControls(bool enable);
    void _displayErrorMessage(QString message);
    void _rowInserted();
    void _groupFilter_textChanged(QString currentText);
    void on_groupsTableView_doubleClicked(QModelIndex index);
    void _customMenuRequested(QPoint pos);
    
    void _successDeleteGroup(qlonglong, QString, bool);
    void _successAddGroup(qlonglong, QString, bool);
    void _successUpdateGroup(qlonglong, QString, bool);

private:
    Ui::GroupsManagerWidget *ui;
    HnsClient::UserManager *_accessor;
    TreeSortFilterProxyModel * _groupsProxyModel;
    GroupsModel *_groupsModel;
    FilterLineEdit *_filterLineEdit;

    void _editGroup(int targetRow);
};

} // namespace Hnv
#endif // GROUPSMANAGERWIDGET_H
