/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "hyneviewstatusbar.h"
#include "statusbarlabel.h"
#include "ressources.h"
#include "zoomsliderwidget.h"

// Qt headers
#include <QAction>
#include <QToolButton>
#include <QSettings>

namespace Hnv {

/*******************************************************************************
* HyneviewStatusBar
*/
HyneviewStatusBar::HyneviewStatusBar(QWidget *parent)
    : QStatusBar(parent)
{
    setMaximumHeight(26);
    _sideBarAction = new QAction(this);
    _sideBarAction->setIcon(QIcon(Ressources::themedPath(Constants::ICON_HIDE_SIDEBAR)));
    _sideBarAction->setToolTip(tr("Show/hide side bar"));
    _sideBarAction->setShortcut(QKeySequence(tr("Alt+S")));
    _sideBarAction->setText(tr("Show left side bar"));
    _sideBarAction->setCheckable(true);
    QToolButton * sideBarButton = new QToolButton(this);
    sideBarButton->setIconSize(QSize(16, 16));
    sideBarButton->setAutoRaise(true);
    sideBarButton->setDefaultAction(_sideBarAction);
    addWidget(sideBarButton);
    connect(_sideBarAction, SIGNAL(triggered()), this, SIGNAL(toggleSideBar()));

    _messagesAction = new QAction(this);
    _messagesAction->setIcon(QIcon(Ressources::themedPath(Constants::ICON_MESSAGES)));
    _messagesAction->setToolTip(tr("Show/hide messages bar"));
    _messagesAction->setText(tr("Show message bar"));
    _messagesAction->setShortcut(QKeySequence(tr("Alt+D")));
    _messagesAction->setCheckable(true);
    QSettings settings;
    settings.beginGroup(Constants::LOG_MESSAGE_SETTINGS);
    _messagesAction->setChecked(settings.value(Constants::LOG_MESSAGE_IS_DISPLAY).toBool());
    settings.endGroup();

    QToolButton * messagesButton = new QToolButton(this);
    messagesButton->setIconSize(QSize(16, 16));
    messagesButton->setAutoRaise(true);
    messagesButton->setDefaultAction(_messagesAction);
    addWidget(messagesButton);
    connect(_messagesAction, SIGNAL(triggered(bool)), this, SIGNAL(toggleLogMessageWidget(bool)));

    _rightSideBarAction = new QAction(this);
    _rightSideBarAction->setIcon(QIcon(Ressources::themedPath(Constants::ICON_HIDE_SIDEBAR_RIGHT)));
    _rightSideBarAction->setToolTip(tr("Show/hide right side bar"));
    _rightSideBarAction->setText(tr("Show right side bar"));
    _rightSideBarAction->setCheckable(true);
    QToolButton * rightSideBarButton = new QToolButton(this);
    rightSideBarButton->setIconSize(QSize(16, 16));
    rightSideBarButton->setAutoRaise(true);
    rightSideBarButton->setDefaultAction(_rightSideBarAction);
    addWidget(rightSideBarButton);
    connect(_rightSideBarAction, SIGNAL(triggered()), this, SIGNAL(toggleRightSideBar()));

    _zoomSliderWidget = new ZoomSliderWidget(this);
    addPermanentWidget(_zoomSliderWidget);
    connect(_zoomSliderWidget, SIGNAL(valueChanged(int)), this, SIGNAL(zoomValueChanged(int)));
    _zoomSliderWidget->zoomChanged(-1);

    _cursorPosLabel = new StatusBarLabel(this);
    _cursorPosLabel->defineMaxSizeByPattern( "(10000,10000)" );
    _cursorPosLabel->setAlignment(Qt::AlignCenter);
    addPermanentWidget(_cursorPosLabel);

    _netStatusWidget = new ConnectionStatusWidget(this);
    addPermanentWidget(_netStatusWidget);

    _statusBarActionlist.append(_messagesAction);
    _statusBarActionlist.append(_sideBarAction);
    _statusBarActionlist.append(_rightSideBarAction);
}

/*******************************************************************************
* HyneviewStatusBar - Public slots
*/
void HyneviewStatusBar::displayCursorPosition(int x, int y, bool isContainer)
{
    _cursorPosLabel->setEnabled(isContainer);
    _cursorPosLabel->setText(QString("(%1, %2)").arg(QString::number(x)).arg(QString::number(y) ));
}

void HyneviewStatusBar::setNetworkStatus(ConnectionStatusWidget::ConnectionState state, QString message)
{
    _netStatusWidget->setStatus(state, message);
}

void HyneviewStatusBar::setZoomValue(int value)
{
    _zoomSliderWidget->zoomChanged(value);
}

void HyneviewStatusBar::clearLabelCoord()
{
    _cursorPosLabel->clear();
}

void HyneviewStatusBar::uncheckMessageAction()
{
    _messagesAction->setChecked(false);
}

void HyneviewStatusBar::enableSideBar(bool enable, bool isVisible)
{
    _sideBarAction->setEnabled(enable);
    if (enable)
        _sideBarAction->setChecked(isVisible);
}

void HyneviewStatusBar::enableRightSideBar(bool enable, bool isVisible)
{
    _rightSideBarAction->setEnabled(enable);
    if (enable)
        _rightSideBarAction->setChecked(isVisible);
}

QList<QAction *> HyneviewStatusBar::statusBarActions()
{
    return _statusBarActionlist;
}

void HyneviewStatusBar::enableTopologyActions(bool enable)
{
    _zoomSliderWidget->setEnabled(enable);
}
} // namespace Hnv
