/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef HYNEVIEWTOOLBAR_H
#define HYNEVIEWTOOLBAR_H

// Qt headers
#include <QVariant>
#include <QToolBar>

class QComboBox;
class QLabel;
class QToolButton;
class QAbstractItemModel;

namespace Hnv {

/*******************************************************************************
* ToolBar
*/
class ToolBar : public QToolBar
{
    Q_OBJECT
public:
    enum Borders {
        None = 0x0,
        Top = 0x1,
        Bottom = 0x2,
        Left = 0x4,
        Right = 0x8,
        All = Top | Bottom | Left | Right
    };
    Q_DECLARE_FLAGS(BordersFlags, Borders)

    explicit ToolBar(QWidget *parent, bool closeable = true, int leftMargin = 0);
    QAction * insertSpacer(QAction *before = 0, int size = -1);
    void setCloseable(bool closeable);
    QAction * closeAction();
    void setBorders(Borders borders);
    Borders borders() const;
    void setLeftMargin(int margin);
    int leftMargin() const;

signals:
    void closeTriggered();

protected:
    QAction *_closeAction;
    Borders _borders;
    int _leftMargin;
    QAction *_leftMarginAction;

};

/*******************************************************************************
* TitleToolBar
*/
class TitleToolBar : public ToolBar
{
    Q_OBJECT
public:
    explicit TitleToolBar(QWidget *parent, bool closeable = true);
    void setTitle(QString title);
    QString title();
    void addToolBarAction(QAction *action, bool afterTitle = true);

protected:
    QLabel * _titleLabel;
    QAction * _titleAction;
    QAction *_spacerAction;

};

/*******************************************************************************
* SelectorToolBar
*/
class SelectorToolBar : public ToolBar
{
    Q_OBJECT
public:
    explicit SelectorToolBar(const QStringList &itemList = QStringList(), QWidget *parent = 0, bool closeable = true);
    void setItemList(const QStringList &itemList);
    void addItem(const QString &itemText, QVariant itemData, bool setActive = false);
    void updateItemText(const QString &text, QVariant itemData);
    void removeItem(QVariant itemData);
    QAction * comboAction();

public slots:
    void setSelectedItem(QVariant itemData);
    void setSelectedItem(int index);

signals:
    void currentIndexChanged(QString itemTitle);
    void currentIndexChanged(QVariant itemData);

protected slots:
    void _currentIndexChanged(int index);

protected:
    QComboBox * _comboBox;
    QAction * _comboBoxAction;
};

/*******************************************************************************
* SideBarToolBar
*/
class SideBarToolBar : public SelectorToolBar
{
    Q_OBJECT
public:
    explicit SideBarToolBar(QList<QPair<QString,QString> > itemList, QWidget *parent = 0, bool closeable = true);

signals:
    void splitTriggered();

};

/*******************************************************************************
* TopologyToolBar
*/
class TopologyToolBar : public SelectorToolBar
{
    Q_OBJECT
public:
    explicit TopologyToolBar(QWidget *parent = 0, const QStringList &topologyList = QStringList(), bool closeable = true);
    void initModel(QAbstractItemModel *model);

    void addEntity(qlonglong entityId, const QString &shortName);
    void removeEntity(int entityId);
    void updateEntity(int entityId, const QString &shortName);
    void resetObjectList(const QHash<qlonglong, QString> &objectList);
    void setSelectedEntity(qlonglong entityID);

signals:
    void currentObjectIndexChanged(qlonglong);

private slots:
    void _currentObjectIndexChangedSlot(int index);
    void _topologyIndexChangedSlot(const QString &text);

private:
    QComboBox * _objectsComboBox;
    QAction * _actionSaveTopology;
    QAction * _actionSaveTopologyAs;
    QAction * _actionSetInsertMode;
    QAction * _actionSetMoveMode;

};

} // namespace Hnv
#endif // HYNEVIEWTOOLBAR_H
