#include "customrightdialog.h"
#include "ui_customrightdialog.h"

#include "common/topologyentry.h"

#include <QVBoxLayout>
#include <QCheckBox>

namespace Hnv {

CustomRightDialog::CustomRightDialog(Hns::ImportOptionsList importList, QWidget *parent)
    : QDialog(parent)
    , ui(new Ui::CustomRightDialog)
{
    ui->setupUi(this);
    _model = new QStringListModel(this);
    QStringList list;
    int i = 0;
    QList<Hns::ImportOptions> options = QList<Hns::ImportOptions>::fromStdList(importList);
    foreach (const Hns::ImportOptions &opt, options) {
        list << QString::fromStdString(opt.newName);
        AccessRightWidget *rightWidget = new AccessRightWidget(this);
        _widgetHash.insert(i, rightWidget);

        QCheckBox *checkBox = new QCheckBox(this);
        checkBox->setChecked(true);
        checkBox->setText(tr("Generate new mac addresses"));
        _checkboxHash.insert(i, checkBox);

        QWidget *contentWidget = new QWidget(this);
        QVBoxLayout *layout = new QVBoxLayout(contentWidget);
        layout->addWidget(checkBox);
        layout->addWidget(rightWidget);
        ui->stackedWidget->addWidget(contentWidget);

        Hns::TopologyEntryPtr topologyEntry = new Hns::TopologyEntry();
        HnsClient::TopologyEntry *topology = new HnsClient::TopologyEntry(topologyEntry, QPixmap(), this);
        topology->validateACLList(opt.acls);
        rightWidget->initialize(topology);
        ++i;
    }

    _model->setStringList(list);
    ui->tableView->setModel(_model);

    ui->tableView->selectRow(0);
    ui->stackedWidget->setCurrentIndex(0);

    connect(ui->tableView, SIGNAL(clicked(QModelIndex)), this, SLOT(onTableViewClickedSlot(const QModelIndex &)));
}

CustomRightDialog::~CustomRightDialog()
{
    qDeleteAll(_entryHash);
    delete _model;
    delete ui;
}

Hns::ImportOptionsList CustomRightDialog::options()
{
    Hns::ImportOptionsList ret;
    QStringList stringList = _model->stringList();
    int i = 0;
    foreach (const QString &str, stringList) {
        Hns::ImportOptions opt;
        opt.newName = str.toStdString();
        opt.acls = _widgetHash.value(i)->aclList();
        opt.flags = 0;
        opt.flags |= _checkboxHash.value(i)->isChecked() ? Hns::ImportOptionsFlags::GENERATE_MAC_ADDRESSES : 0;
        ret.push_back(opt);
        ++i;
    }

    return ret;
}

void CustomRightDialog::onTableViewClickedSlot(const QModelIndex &index)
{
    if (!index.isValid())
        return;
    int row = index.row();
    if (row < _widgetHash.size())
        ui->stackedWidget->setCurrentIndex(row);
}

} //Hnv
