// Project headers
#include "importentrymodel.h"

// hynesim hnsclient headers
#include <libhnsclient/entry.h>

namespace Hnv {

enum {
    ImportableKeyRole = Qt::UserRole + 1
};

/*******************************************************************************
 *****ImportEntryModel
 *******************************************************************************/
ImportEntryModel::ImportEntryModel(const QMap<QString, HnsClient::EntityEntry *> importableEntryMap, QObject *parent)
    : QAbstractTableModel(parent),
      _rowCount(0)
{
    _pathList= importableEntryMap.keys();
    foreach (const HnsClient::EntityEntry *entry, importableEntryMap.values()) {
        Hns::ImportableEntryPtr importableEntry = new Hns::ImportableEntry();
        importableEntry->entry = Hns::EntityEntryPtr::dynamicCast(entry->entityEntry()->ice_clone());
        _importableEntryList.append(importableEntry);
    }
    _rowCount= _importableEntryList.size();
}

ImportEntryModel::~ImportEntryModel()
{

}

int ImportEntryModel::rowCount(const QModelIndex &parent) const
{
    if (!parent.isValid())
        return _rowCount;
    return 0;
}

int ImportEntryModel::columnCount(const QModelIndex &parent) const
{
    if (!parent.isValid())
        return COLUMN_COUNT;
    return 0;
}

QVariant ImportEntryModel::headerData(int section, Qt::Orientation orientation, int role) const
{
    if (role == Qt::DisplayRole && orientation == Qt::Horizontal) {
        switch (section) {
        case COLUMN_NAME:
            return tr("Name");
        case COLUMN_UUID:
            return tr("Uuid");
        }
    }
    return QAbstractTableModel::headerData(section, orientation, role);
}

QVariant ImportEntryModel::data(const QModelIndex &index, int role) const
{
    if (!index.isValid() || index.row() >= _importableEntryList.count() || index.row() >= _pathList.count())
        return QVariant();
    Hns::ImportableEntryPtr importable = _importableEntryList.at(index.row());
    if (!importable)
        return QVariant();
    if (role == ImportableKeyRole)
        return QString::fromStdString(importable->path);
    else if (role == Qt::DisplayRole) {
        switch(index.column()) {
        case COLUMN_NAME:
            return QString::fromStdString(importable->entry->shortName);
        case COLUMN_UUID:
            return _pathList.at(index.row());
        }
    } else if (role == Qt::UserRole) {
        return _pathList.at(index.row());
    }
    return QVariant();
}

void ImportEntryModel::setEntryList(const QList<Hns::ImportableEntryPtr> list)
{
    beginResetModel();
    _importableEntryList = list;
    endResetModel();
}

void ImportEntryModel::setImportableMap(const QMap<QString, HnsClient::EntityEntry *> map)
{
    beginResetModel();
    _importableEntryList.clear();
    _pathList= map.keys();
    foreach (const HnsClient::EntityEntry *entry, map.values()) {
        Hns::ImportableEntryPtr importableEntry = new Hns::ImportableEntry();
        importableEntry->entry = Hns::EntityEntryPtr::dynamicCast(entry->entityEntry()->ice_clone());
        _importableEntryList.append(importableEntry);
    }
    _rowCount= _importableEntryList.size();
    endResetModel();
}

void ImportEntryModel::resetImportableMap()
{
    beginResetModel();
    _importableEntryList.clear();
    endResetModel();
}

} // namespace Hnv
