/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef CAPTUREJOBDIALOG_H
#define CAPTUREJOBDIALOG_H

//#include "entitycatalogmodel.h"
//#include "common/capturejoboptions.h"

// Qt headers
#include <QDialog>
#include <QModelIndex>
#include <QHash>

class QSortFilterProxyModel;
class QItemSelection;

namespace Ui {
class CaptureJobDialog;
} // namespace Ui

namespace Hns {
struct CaptureJobOptions;
} // namespace Hns

namespace HnsClient {
class GenericEntity;
class FileTransferClient;
} // namespace HnsClient

namespace Hnv {

class CaptureJobDialog : public QDialog
{
    Q_OBJECT

public:
    explicit CaptureJobDialog(HnsClient::GenericEntity *entity,QWidget *parent = 0);
    ~CaptureJobDialog();
    Hns::CaptureJobOptions getOptions();

protected:
    void _stopRunningCapture(QModelIndexList indexList);
    void _downloadFiles(QModelIndexList indexList);
    void _deleteJobs(QModelIndexList indexList);

private slots:
    void _updateActionsSlot();
    void _entityDestroyedSlot();
    void _entityStatusChangedSlot();
    void tableViewModelSelectionChanged(const QItemSelection &currentIndex, const QItemSelection &previouxIndex);

    void on_tableView_doubleClicked(const QModelIndex &index);
    void on_tableView_customContextMenuRequested(const QPoint &pos);
    void on_actionStop_triggered();
    void on_actionDownload_triggered();
    void on_actionDelete_triggered();

    void on_actionReuse_settings_triggered();

    void _entityDownloadFileSlot(HnsClient::FileTransferClient *);
    void _fileSizeSlot(QString, qlonglong);
    void _transferFinishedSlot(QString remoteFileName, QString path);

private:
    Ui::CaptureJobDialog *ui;
    bool _sourceValid;
    HnsClient::GenericEntity *_entity;
    QString _entityShortName;
    QSortFilterProxyModel *_proxyModel;
    QHash<QString, HnsClient::FileTransferClient*> _fileTransfertClientHash;
};

} // namespace Hnv
#endif // CAPTUREJOBDIALOG_H
