/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "optionseditionwidget.h"
#include "ui_optionseditionwidget.h"

// Generated hynesim headers
#include "common/capturejoboptions.h"

// hynesim hnsclient headers
#include <libhnsclient/entity.h>

namespace Hnv {

OptionsEditionWidget::OptionsEditionWidget(QWidget *parent)
    : QWidget(parent)
    , ui(new Ui::OptionsEditionWidget)
{
    ui->setupUi(this);
}

OptionsEditionWidget::~OptionsEditionWidget()
{
    delete ui;
}

Hns::CaptureJobOptions OptionsEditionWidget::getOptions()
{
    //TODO: avaible ports are not updated
    Hns::CaptureJobOptions captureJobOptions;
    captureJobOptions.filteringterms = ui->teFilter->toPlainText().toStdString();
    captureJobOptions.interfaceList.push_back(ui->cbInterface->itemData(ui->cbInterface->currentIndex()).toInt());
    captureJobOptions.numberpackets = ui->sbPackets->value();
    captureJobOptions.timer = ui->sbSeconds->value();
    return captureJobOptions;
}

void OptionsEditionWidget::setEntity(HnsClient::GenericEntity *entity)
{
    _entity = entity;
    connect(_entity, SIGNAL(availableInterfacesForCaptureChanged(QList<int>)), this, SLOT(_availableInterfacesForCaptureChanged(QList<int>)));
    setInterfaceList();
}

void OptionsEditionWidget::setCaptureSettings(Hns::PcapInfo sourceInfo)
{
    ui->teFilter->setText(QString::fromStdString(sourceInfo.filters));
    ui->sbPackets->setValue(sourceInfo.packetsLimit);
    ui->sbSeconds->setValue(sourceInfo.timeLimit);
}

void OptionsEditionWidget::setInterfaceList()
{
    /* Seeking for id and source of the netcard selected for capture */
    _availableInterfacesForCaptureChanged(_entity->getAvailableInterfacesForCapture());
}

void OptionsEditionWidget::enableInterface(bool state)
{
    ui->teFilter->setEnabled(state);
    ui->sbPackets->setEnabled(state);
    ui->sbSeconds->setEnabled(state);
    ui->pbLaunchCaptureJob->setEnabled(state);
}

void OptionsEditionWidget::_availableInterfacesForCaptureChanged(QList<int> listAvailableInterfaces)
{
    ui->cbInterface->clear();
    _mapInterfaces.clear();
    if (listAvailableInterfaces.contains(Hns::captureAll))
        _mapInterfaces.insert(Hns::captureAll, tr("All"));

    QMap<int, Hns::Plug> plugs = _entity->getPlugs();

    foreach (int i, listAvailableInterfaces) {
        if (plugs.contains(i))
            _mapInterfaces.insert(i, QString::fromStdString(plugs.value(i).name));
    }

    QMapIterator<int, QString> iterator(_mapInterfaces);
    while (iterator.hasNext()) {
        iterator.next();
        ui->cbInterface->addItem(iterator.value(), QVariant::fromValue(iterator.key()));
    }
    enableInterface(_mapInterfaces.count() > 0);
}

void OptionsEditionWidget::on_pbLaunchCaptureJob_clicked()
{
    _entity->createCaptureJob(getOptions());
}

void OptionsEditionWidget::on_pbResetCaptureJob_clicked()
{
    ui->cbInterface->setCurrentIndex(0);
    ui->teFilter->setText("");
    ui->sbSeconds->setValue(0);
    ui->sbPackets->setValue(0);
}

} // namespace Hnv
