/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef LOADBALANCERDIALOG_H
#define LOADBALANCERDIALOG_H

// Qt headers
#include <QDialog>
#include <QModelIndex>

// Generated hynesim headers
#include "common/topology.h"

namespace Ui {
class LoadBalancerDialog;
} // namespace Ui

class QLabel;
class QSpacerItem;

namespace HnsClient {
class GenericEntity;
class Topology;
class PlatformMonitor;
class NodeInformation;
} // namespace HnsClient

namespace Hnv {

class LoadBalancerModel;
class NodeResourceWidget;
class ListNodeDelegate;

struct EntityLocalInformation
{
    QString shortName;
    QString toolTip;
    qlonglong id;
    unsigned short cpu;
    unsigned short memory;
    bool isDefined;
    QString nodeName;
    QStringList listAvailableNodeName;
    int isSelected;
    QString warning;
    QPixmap pixmap;
};

enum OverLoadCharge {
    NO_CHARGE= 0x01,
    CPU_CHARGE = 0x02,
    MEMORY_CHARGE = 0x04,
    FULL_CHARGE = CPU_CHARGE | MEMORY_CHARGE
};

enum {
    CHECK,
    UNCHECK,
    NOTHING
};

class LoadBalancerDialog : public QDialog
{
    Q_OBJECT
public:
    explicit LoadBalancerDialog(const QList<HnsClient::GenericEntity *> &listEntitiesSelected , HnsClient::Topology *topology ,QWidget *parent = 0);
    ~LoadBalancerDialog();

public slots:
    void nodeChangeToComboboxSlot(qlonglong,const QString &);
    void nodeChangeToCheckSlot(qlonglong,int);

private slots:
    void _modelResetSlot();
    void _rowsInsertedSlot(QModelIndex parent, int start, int end);
    void _overloadedSlot(const QString & , int);
    void _computeDefineInfosSlot(::Hns::LongDefineInfoMap);
    void _nodeAddedSlot(QString nodeKey);
    void _nodeRemovedSlot(QString nodeKey);
    void _entityRefreshedSlot(HnsClient::GenericEntity *genericEntity);

    void on_buttonBox_accepted();
    void on_buttonCompute_clicked();
    void on_buttonReset_clicked();
    void on_actionSelectAll_triggered();
    void on_actionUnselect_All_triggered();
    void on_actionInverse_Selection_triggered();

protected:
    void _resetData();
    void _initData(const QList<HnsClient::GenericEntity *> & listEntitiesSelected = QList<HnsClient::GenericEntity *>());
    void _cleanAll();
    QHash<qlonglong, EntityLocalInformation> _getEntitiesFromTopology(bool completeInformation = true);
    void _initEntityList(const QList<HnsClient::GenericEntity *> & listEntitiesSelected);
    void _initNodeWidgets();
    void _noRessourceWidget();
    void _fillCombobox();
    QString _validateEntityData();
    QString _validateNodeData();
    void _displayError(QString error);
    bool _isDataValid();
    void _compute();
    void _enableInterface(bool enable = true, bool runningTransaction = false);
    void _defineEntity();
    void _addNodeWidget(QString nodeKey, HnsClient::NodeInformation* nodeInformation);
    void _removeNodeWidget(QString nodeKey);
    EntityLocalInformation _generateEntityLocalInformation(HnsClient::GenericEntity *genericEntity, bool completeInformation = true);

private:
    Ui::LoadBalancerDialog *_ui;
    HnsClient::Topology *_topology;
    LoadBalancerModel * _loadBalancerModel;
    QHash<qlonglong, EntityLocalInformation> _entityInformationHash;
    QHash<QString,NodeResourceWidget *> _nodeWidgetHash;
    QHash<QString, QString> _nodeWarningHash;
    bool _needToResizeLastColumn;
    HnsClient::PlatformMonitor * _platformMonitor;
    ListNodeDelegate * _delegate;
    bool _canCompute;
    QLabel *_noNodeLabel;
    QSpacerItem *_nodeVerticalSpacer;

};

} // namespace Hnv
#endif // CAPTUREJOBDIALOG_H
