/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef LOADBALANCERMODEL_H
#define LOADBALANCERMODEL_H

// Qt headers
#include <QAbstractItemModel>

namespace Hnv {

struct EntityLocalInformation;

class EntityEntry;

enum {
    COLUMN_CHECK,
    COLUMN_SHORT_NAME,
    COLUMN_COMBOBOX,
    COLUMN_CPU,
    COLUMN_MEMORY,
    COLUMN_WARNING,
    COLUMN_SIZE
};

class LoadBalancerModel : public QAbstractTableModel
{
    Q_OBJECT
public:
    explicit LoadBalancerModel(QObject *parent = 0);
    ~LoadBalancerModel();

    void clear();
    void setEntityList(QList<EntityLocalInformation> entityList);
    void addEntity(EntityLocalInformation entity);
    void removeEntity(qlonglong entityID);
    void changeEntity(qlonglong entityID, EntityLocalInformation localInfo);

    int rowCount(const QModelIndex &parent) const;
    int columnCount(const QModelIndex &parent) const;
    QVariant data(const QModelIndex &index, int role) const;
    QVariant headerData(int section, Qt::Orientation orientation, int role) const;
    bool setData(const QModelIndex &index, const QVariant &value, int role = Qt::EditRole);
    Qt::ItemFlags flags(const QModelIndex &index) const;
    int getRowFromID(qlonglong entityID);

signals:
    void comboBoxChange(qlonglong,const QString & newNodeName);

private:
    QList<EntityLocalInformation> _entityList;
    QHash <qlonglong, int> _entityPositionHash;
};

} // namespace Hnv
#endif //LOADBALANCERMODEL_H
