/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef LOGMESSAGEWIDGET_H
#define LOGMESSAGEWIDGET_H

// Qt headers
#include <QWidget>
#include <QAbstractListModel>
#include <QDateTime>

class QSortFilterProxyModel;

namespace Ui {
class LogMessageWidget;
} // namespace Ui

namespace Hnv {

/*******************************************************************************
* LogMessageItem
*/
class LogMessageItem {

public:
    enum MessageType {
        Debug = QtDebugMsg,
        Warning = QtWarningMsg,
        Critical = QtCriticalMsg,
        Fatal = QtFatalMsg,
        HnsTrace
    };

    enum {
        MessageTypeRole = Qt::UserRole,
        MessageDateRole,
        MessageSortRole
    };
    explicit LogMessageItem(LogMessageItem::MessageType type, QString msg, QDateTime dateTime);
    LogMessageItem::MessageType type();
    QString message();
    QDateTime date();

private:
    LogMessageItem::MessageType _type;
    QString _message;
    QDateTime _dateTime;
};

/*******************************************************************************
* LogMessageModel
*/
class LogMessageModel : public QAbstractListModel
{
    Q_OBJECT
public:
    explicit LogMessageModel(QObject *parent);
    int rowCount(const QModelIndex &parent) const;
    QVariant data(const QModelIndex &index, int role) const;
    void clear();

public slots:
    void addMessage(LogMessageItem::MessageType type, QString msg, QDateTime dateTime);

private:
    QHash<LogMessageItem::MessageType, QString> _messageTypeStringHash;
    QList<LogMessageItem> _messageList;
};

/*******************************************************************************
* LogMessageWidget
*/
class LogMessageWidget : public QWidget
{
    Q_OBJECT

public:
    explicit LogMessageWidget(QWidget *parent = 0);
    ~LogMessageWidget();

signals:
    void closeTriggered();

private slots:
    void _setFilter();
    void _triggerScrollToBottom();
    void _scrollToBottom();

    void on_action_Clear_triggered();

protected:
    void showEvent(QShowEvent *);

private:
    Ui::LogMessageWidget *ui;
    QSortFilterProxyModel * _proxyModel;
    bool _waitToScroll;
};

} // namespace Hnv
#endif // LOGMESSAGEWIDGET_H
