// Project headers
#include "macaddressmanagerwidget.h"
#include "ui_macaddressmanagerwidget.h"
#include "hyneviewsession.h"
#include "ressources.h"

// hynesim hnsclient headers
#include <libhnsclient/macaddressmanager.h>

// Qt headers
#include <QCompleter>

namespace Hnv {

MacAddressManagerWidget::MacAddressManagerWidget(QWidget *parent)
    : QWidget(parent)
    , ui(new Ui::MacAddressManagerWidget)
    , _currentMacVendor(new QPair<QString, QString>())
{
    ui->setupUi(this);
    _macAddressManager = HyneviewSession::instance()->macAddressManager();
    ui->randomButton->setIcon(QIcon(Ressources::themedPath(Constants::ICON_GENERATE_MAC_ADRESS)));
    QCompleter *completer = new QCompleter(*HyneviewSession::instance()->getNetworkCardsVendor(), this);
    completer->setCaseSensitivity(Qt::CaseInsensitive);
    ui->vendorLine->setCompleter(completer);
    ui->vendorLine->setPlaceholderText(tr("Enter vendor name"));
    connect(completer, SIGNAL(activated(QString)), this, SLOT(completerActivated(QString)));
}

MacAddressManagerWidget::~MacAddressManagerWidget()
{
    delete ui;
    if (_currentMacVendor)
        delete _currentMacVendor;
    _currentMacVendor = NULL;
}

void MacAddressManagerWidget::setReadOnly(bool checked)
{
    ui->macAddressLine->setDisabled(checked);
    ui->vendorLine->setDisabled(checked);
    ui->randomButton->setDisabled(checked);
}

void MacAddressManagerWidget::generate(QString vendor)
{
    _macAddressManager->generateRandomAddress(vendor, this, SLOT(_generateRandomAddressSlot(QString)));
}

// Getter/Setter
QString MacAddressManagerWidget::macAddress()
{
    return ui->macAddressLine->text();
}

void MacAddressManagerWidget::setMacAddress(QString address)
{
    ui->macAddressLine->setText(address);
}

// PRIVATE SLOTS
void MacAddressManagerWidget::on_macAddressLine_textChanged(const QString &macAddress)
{
    QString prefix = macAddress.left(8);
    if (prefix.contains(QRegExp("^([a-fA-F0-9]{2}:){2}[a-fA-F0-9]{2}")))
        if (prefix != _currentMacVendor->first)
            _macAddressManager->getVendorFromPrefix(prefix, this, SLOT(_getVendorFromPrefixSlot(QString)));
    _currentMacVendor->first = prefix;
}

void MacAddressManagerWidget::on_randomButton_clicked()
{
    generate(ui->vendorLine->text());
}

void MacAddressManagerWidget::_getVendorFromPrefixSlot(QString vendor)
{
    _currentMacVendor->second = vendor;
    ui->vendorLine->setText(_currentMacVendor->second);
}

void MacAddressManagerWidget::_generateRandomAddressSlot(QString address)
{
    ui->macAddressLine->setText(address);
}

void MacAddressManagerWidget::completerActivated(const QString &vendor)
{
    if (HyneviewSession::instance()->getNetworkCardsVendor()->contains(vendor))
        generate(vendor);
}

} // namespace Hnv
