#ifndef MEMORYDUMPDIALOG_H
#define MEMORYDUMPDIALOG_H

#include <QDialog>
#include <QAbstractItemModel>

#include "common/types.h"

namespace Ui {
class MemoryDumpDialog;
}

namespace HnsClient {
class GenericEntity;
class DomainEntity;
class Job;
class FileTransferClient;
}

namespace Hnv {

/*******************************************************************************
 * MemoryDumpModel
 *******************************************************************************/
class MemoryDumpModel : public QAbstractTableModel
{
    Q_OBJECT
public:
    enum {
        COLUMN_NAME,
        COLUMN_DATE,
        COLUMN_SIZE,
        COLUMN_COUNT
    };
    enum {
        FileNameRole = Qt::UserRole + 1,
        NameRole
    };
    explicit MemoryDumpModel(HnsClient::DomainEntity *domain, QObject *parent = 0);
    ~MemoryDumpModel();
    int rowCount(const QModelIndex &parent) const;
    int columnCount(const QModelIndex &parent) const;
    QVariant headerData(int section, Qt::Orientation orientation, int role) const;
    QVariant data(const QModelIndex &index, int role) const;

private slots:
    void _memoryDumpAddedSlot(::Hns::MemoryDumpInfo info);
    void _memoryDumpRemovedSlot(QString fileName);

private:
    HnsClient::DomainEntity *_domain;
    QList<Hns::MemoryDumpInfo> _dumps;
};

/*******************************************************************************
 * MemoryDumpDialog
 *******************************************************************************/
class MemoryDumpDialog : public QDialog
{
    Q_OBJECT

public:
    explicit MemoryDumpDialog(HnsClient::GenericEntity *genericEntity, QWidget *parent = 0);
    ~MemoryDumpDialog();

private slots:
    void _domainShortNameChangedSlot();
    void _domainStatusChangedSlot();
    void _selectionChangedSlot();

    void on_buttonBox_rejected();
    void on_downloadPushButton_clicked();
    void on_deletePushButton_clicked();
    void on_createPushButton_clicked();
    void on_memoryDumpNameLineEdit_textChanged(const QString &arg1);

    void _createMemoryDumpJobSlot(int jobId);
    void _deleteMemoryDumpSlot();
    void _getJobSlot(HnsClient::Job *job);
    void _getFileTransferClient(HnsClient::FileTransferClient *client);
    void _transferClientFinishedSlot(QString remoteFileName, QString localFileName);

    void _jobFinishedSlot();

private:
    Ui::MemoryDumpDialog *ui;
    HnsClient::DomainEntity *_domain;
    HnsClient::Job *_job;
    HnsClient::FileTransferClient *_client;
};

} // namespace Hnv

#endif // MEMORYDUMPDIALOG_H
