/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef MODEBROWSERWIDGET_H
#define MODEBROWSERWIDGET_H

// Project headers
#include "hnvwidgets.h"

// Qt headers
#include <QToolButton>
#include <QSplitter>

class QButtonGroup;
class QActionGroup;

namespace Ui {
class ModeBrowserWidget;
} // namespace Ui

namespace Hnv {
class HyneviewSession;
class LogMessageWidget;
class ModeWidget;
class SideBar;

/*******************************************************************************
* ModeButton
*******************************************************************************/
class ModeButton : public QToolButton
{
    Q_OBJECT
public:
    explicit ModeButton(ModeWidget *modeWidget);
    ModeWidget * modeWidget();
private:
    ModeWidget * _modeWidget;
};

/*******************************************************************************
* BrowserToolButton
*******************************************************************************/
class BrowserToolButton : public QToolButton
{
    Q_OBJECT
public:
    explicit BrowserToolButton(QWidget *parent = 0);
};

/*******************************************************************************
* ModeDisplayArea
*******************************************************************************/
class ModeDisplayArea : public QSplitter
{
    Q_OBJECT
public:
    explicit ModeDisplayArea(QWidget *displayWidget, QWidget *parent = 0);
    ~ModeDisplayArea();
    QWidget * displayWidget();
public slots:
    void toggleLogMessageWidget(bool);
signals:
    void logMessageWidgetClosed();
private:
    QWidget * _displayWidget;
    LogMessageWidget * _logMessageWidget;
};

/*******************************************************************************
* ModeWidget
*******************************************************************************/
class ModeWidget : public QSplitter
{
    Q_OBJECT
public:
    explicit ModeWidget(QWidget *parent = 0);
    ~ModeWidget();
    virtual void initWidget();
    ModeButton *getModeButton();
    void setModeEnabled(bool enabled);
    ModeDisplayArea *displayArea();
    QWidget * displayWidget();
    virtual void restoreDefaultSettings();
    virtual void saveSettings();
    virtual bool hasSideBar() = 0;
    virtual bool hasRightSideBar() = 0;
    bool isSideBarVisible();
    bool isRightSideBarVisible();
public slots:
    virtual void toggleSideBar();
    virtual void toggleRightSideBar();
    void toggleLogMessageWidget(bool checked);
signals:
    void hidden();
    void logMessageWidgetClosed();
    void sideBarHidden();
    void rightSideBarHidden();
protected:
    virtual void _initModeButton();
    virtual void _initModeAction() = 0;
    virtual void _initDisplayArea(QWidget *displayWidget);
    virtual void _initSideBar(SideBar * sideBar);
    virtual void _initRightSideBar(SideBar *sideBar);
    ModeButton * _modeButton;
    QAction * _modeAction;
    SideBar * _sideBar;
    SideBar * _rightSideBar;
private:
    ModeDisplayArea * _displayArea;
};

/*******************************************************************************
* ModeBrowserWidget
*******************************************************************************/
class ModeBrowserWidget : public ButtonListWidget
{
    Q_OBJECT
public:
    explicit ModeBrowserWidget(QWidget *parent = 0);
    ~ModeBrowserWidget();
    void registerModeWidget(ModeWidget *modeWidget, bool isBottomWidget = false);
    QList<QAction *> fileMenuActions();
    QList<QAction *> modeMenuActions();
    void switchToMode(int index);

    QButtonGroup * modeButtons();
    virtual void restoreDefaultSettings();
    virtual void saveSettings();
signals:
    void modeChanged(QWidget *mode);
    void connectTriggered();
    void quickConnectTriggered();
    void disconnectedTriggered();
    void sidebarEnabled(bool enabled, bool isVisible);
    void rightSidebarEnabled(bool enabled, bool isVisible);

public slots:
    void modeButtonTriggered();
    void setConnected(HyneviewSession *);
    void setDisconnected();
    void enableQuickConnect(bool enable);

private slots:
    void _modeHidden();

    void on_action_Account_triggered();
    void _sideBarHiddenSlot();
    void _rightSideBarHiddenSlot();

private:
    Ui::ModeBrowserWidget *ui;
    QButtonGroup * _mModeButtons;
    QActionGroup * _actionGroup;
    QList<ModeButton *> _mTopButtons;
    QList<ModeButton *> _mBottomButtons;
    QList<QAction *> _modeActionlist;
    bool _actionQuickConnectEnable;
};

} // namespace Hnv
#endif // MODEBROWSERWIDGET_H
