/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "modellistwidget.h"
#include "hnvmainwindow.h"
#include "topologycatalogmodel.h"
#include "simulationmodewidget.h"
#include "topologywidget.h"

// Qt headers
#include <QAction>
#include <QMenu>
#include <QListView>

namespace Hnv {

/*******************************************************************************
* ToolTipFilterProxyModel
*******************************************************************************/
ToolTipFilterProxyModel::ToolTipFilterProxyModel(QObject *parent)
    : QSortFilterProxyModel(parent)
{
}

QVariant ToolTipFilterProxyModel::data(const QModelIndex &index, int role) const
{
    if(role == Qt::ToolTipRole)
        return QSortFilterProxyModel::data(index, TopologyItem::ScreenShotRole);
    else
        return QSortFilterProxyModel::data(index, role);
}

Qt::ItemFlags ToolTipFilterProxyModel::flags(const QModelIndex &index) const
{
    if (index.isValid()) {
        QVariant variantEditable = QSortFilterProxyModel::data(index, TopologyItem::IsModifiedRole);
        if(variantEditable.isValid() && variantEditable.toBool())
            return Qt::ItemIsSelectable;

    }
    return QSortFilterProxyModel::flags(index);
}


/*******************************************************************************
* ModelListWidget
*******************************************************************************/
ModelListWidget::ModelListWidget(QWidget *parent)
    : SideBarListWidget(parent)
    , _topologyWidget(0)
    , _proxyModel(NULL)
{
    _view()->setEditTriggers(QAbstractItemView::NoEditTriggers);
    _view()->setDragEnabled(true);
    _view()->setDragDropMode(QAbstractItemView::DragOnly);
    _actionProperties = new QAction(this);
    _actionProperties->setText(tr("Properties"));
    _actionProperties->setToolTip(tr("Display model properties"));
    connect(_actionProperties, SIGNAL(triggered()), this, SLOT(_actionPropertiesTriggeredSlot()));

    QWidget * widget = sideBarParent();
    if (widget)
        _simulationModeWidget = dynamic_cast<SimulationModeWidget*>(widget);
    if (_simulationModeWidget) {
        _topologyWidget = _simulationModeWidget->topologyWidget();
        if (_topologyWidget) {
            _proxyModel = new ToolTipFilterProxyModel(this);
            _proxyModel->setSourceModel(HnvMainWindow::instance()->topologyCatalogModel());
            _proxyModel->setFilterRole(TopologyItem::IsModelRole);
            _proxyModel->setDynamicSortFilter(true);
            _proxyModel->setFilterRegExp(QRegExp("true"));
            _proxyModel->setFilterKeyColumn(0);
            _proxyModel->setFilterCaseSensitivity(Qt::CaseInsensitive);
        }
    }
    setViewModel(_proxyModel);
}

ModelListWidget::~ModelListWidget()
{
}

QString ModelListWidget::title()
{
    return tr("Model Topologies");
}

QString ModelListWidget::sideBarId()
{
    return "Model Topologies";
}

void ModelListWidget::_indexClicked(const QModelIndex &index)
{
    Q_UNUSED(index)
}

void ModelListWidget::_indexDoubleClicked(const QModelIndex &index)
{
    _displayTopologyProperties(index);
}

void ModelListWidget::_contextMenuRequested(const QModelIndex &index, const QPoint &position)
{
    if (index.isValid()) {
        QMenu contextMenu;
        contextMenu.addAction(_actionProperties);
        contextMenu.exec(_globalPosition(position));
    }
}

/*******************************************************************************
* ModelListWidget - Private slots
*/
void ModelListWidget::_actionPropertiesTriggeredSlot()
{
    _displayTopologyProperties(_currentIndex());
}

/*******************************************************************************
* ModelListWidget - Private
*/
void ModelListWidget::_displayTopologyProperties(QModelIndex index)
{
    TopologyItem * item = static_cast<TopologyItem *>(_proxyModel->mapToSource(index).internalPointer());
    if (item)
        _topologyWidget->displayTopologyProperties(item);
}

} // namespace Hnv
