/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef MODELLISTWIDGET_H
#define MODELLISTWIDGET_H

// Project headers
#include "sidebar.h"

// Qt headers
#include <QSortFilterProxyModel>

class QAction;

namespace Hnv {

class SimulationModeWidget;
class TopologyWidget;

/*******************************************************************************
* ToolTipFilterProxyModel
*******************************************************************************/
class ToolTipFilterProxyModel : public QSortFilterProxyModel
{
    Q_OBJECT
public:
    ToolTipFilterProxyModel(QObject * parent = 0);
    QVariant data(const QModelIndex &index, int role = Qt::DisplayRole) const;
    Qt::ItemFlags flags(const QModelIndex &index) const;
};


/*******************************************************************************
* ModelListWidget
*******************************************************************************/
class ModelListWidget : public SideBarListWidget
{
    Q_OBJECT
public:
    Q_INVOKABLE explicit ModelListWidget(QWidget *parent = 0);
    ~ModelListWidget();
    static QString title();
    static QString sideBarId();

protected:
    virtual void _indexClicked(const QModelIndex &index);
    virtual void _indexDoubleClicked(const QModelIndex &index);
    virtual void _contextMenuRequested(const QModelIndex &index, const QPoint &position);

private slots:
    void _actionPropertiesTriggeredSlot();

private:
    void _displayTopologyProperties(QModelIndex index);

    SimulationModeWidget * _simulationModeWidget;
    TopologyWidget * _topologyWidget;
    ToolTipFilterProxyModel * _proxyModel;
    QAction * _actionProperties;
};

} // namespace Hnv

#endif // MODELLISTWIDGET_H
