/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "noderesourcewidget.h"
#include "ui_noderesourcewidget.h"
#include "ressources.h"
#include "loadbalancerdialog.h"

// hynesim hnsclient headers
#include <libhnsclient/platformmonitor.h>
#include <libhnsclient/utility.h>

namespace Hnv {

NodeResourceWidget::NodeResourceWidget(HnsClient::NodeInformation *nodeInformation, QWidget *parent)
    : QWidget(parent)
    , ui(new Ui::NodeResourceWidget)
    , _nodeKey(nodeInformation->getName())
    , _nodeName(nodeInformation->getShortName())
    , _initialDefinedEntities(nodeInformation->getHnsDefinedEntities())
    , _previsionDefinedEntities(0)
    , _initialCpu(nodeInformation->getHnsCurrentCpu())
    , _totalCpu(nodeInformation->getHnsMaxCpu())
    , _previsionCpu(0)
    , _initialMemory(nodeInformation->getHnsCurrentResources().memory)
    , _totalMemory(nodeInformation->getHnsMaxResources().memory)
    , _previsionMemory(0)
    , _overloaded(NO_CHARGE)
{
    ui->setupUi(this);
    ui->nodeNameLabel->setText(QString("<u>%1</u>").arg(_nodeName));
    QString text = QObject::tr("Number of entities: cur = ");
    ui->numberEntitieslabel->setText(text + QString("<b>%1</b>").arg(QString::number(_initialDefinedEntities)));
    ui->warningCPULabel->setVisible(false);
    ui->warningMemoryLabel->setVisible(false);

    bool overflow = false;
    _drawProgressBar(_initialCpu, _totalCpu, ui->CPUcurrentProgressBar, overflow, false);
    _drawProgressBar(-1, _totalCpu, ui->CPUPrevisionProgressBar, overflow);
    if (overflow) {
        ui->CPUNameLabel->setStyleSheet("QLabel{color:red;}");
        _overloaded = CPU_CHARGE;
    }
    overflow = false;
    _drawProgressBar(_initialMemory, _totalMemory, ui->memoryCurrentProgressBar, overflow, false, true);
    _drawProgressBar(-1, _totalMemory, ui->memoryPrevisionProgressBar, overflow, true);
    if (overflow) {
        ui->MemoryNameLabel->setStyleSheet("QLabel{color:red;}");
        _overloaded |=  MEMORY_CHARGE;
    }
}

NodeResourceWidget::~NodeResourceWidget()
{
    delete ui;
}

QString NodeResourceWidget::nodeKey()
{
    return _nodeKey;
}

QString NodeResourceWidget::nodeName()
{
    return _nodeName;
}

qlonglong NodeResourceWidget::initialCpu()
{
    return _initialCpu;
}

qlonglong NodeResourceWidget::inititalMemory()
{
    return _initialMemory;
}

void NodeResourceWidget::_drawProgressBar(int value, int valueMax,QProgressBar *progressBar , bool &overflow ,bool prev, bool valueToDisplayIsMemory)
{
    progressBar->setRange(0, qMax(value,valueMax));
    progressBar->setValue(value);
    QString formatToDraw = "";
    if (!prev || (value != 0 )) {
        if (!valueToDisplayIsMemory)
            formatToDraw = QString("%1 / %2").arg(QString::number(value)).arg(QString::number(valueMax));
        else
            formatToDraw = QString("%1 / %2").arg(HnsClient::Utility::makeSizeReadable(value, HnsClient::MebiBytes)).arg(HnsClient::Utility::makeSizeReadable(valueMax, HnsClient::MebiBytes));
        if (valueMax != 0 && value > 0) {
            int percent = (double)value/(double)valueMax * 100;
            formatToDraw += " ("+ QString::number(percent) +"%)";
        }
    }
    progressBar->setFormat(formatToDraw);
    progressBar->setEnabled(true);
    if (value > valueMax) {
        progressBar->setStyleSheet("text-align: center; selection-background-color: red;");
        overflow = true;
    } else if (prev && value != -1 ) {
        progressBar->setStyleSheet("text-align: center; selection-background-color: #43ACE8;");
    } else if (!prev) {
        progressBar->setStyleSheet("text-align: center; selection-background-color: #94C2FF;");
    } else {
        //progressBar->setFormat(format);
        progressBar->setFormat("");
        progressBar->setEnabled(false);
        progressBar->setRange(0, 100);
        progressBar->setValue(100);
        progressBar->setStyleSheet("text-align: center;selection-background-color: gray;");
    }
}

void NodeResourceWidget::removePrevisionResource(int cpu, int memory)
{
    _previsionCpu -= cpu;
    _previsionMemory -= memory;
    _previsionDefinedEntities --;
    bool overflow = false;
    bool chargeChange = false;
    if(_previsionDefinedEntities == 0)
        _drawProgressBar(-1, _totalCpu, ui->CPUPrevisionProgressBar, overflow);
     else
        _drawProgressBar(_initialCpu + _previsionCpu, _totalCpu, ui->CPUPrevisionProgressBar, overflow);

    if (!overflow && (_overloaded & CPU_CHARGE)) {
        ui->CPUNameLabel->setStyleSheet("");
        _overloaded &= ~CPU_CHARGE;
        chargeChange = true;
        ui->warningCPULabel->setVisible(false);
    }
    overflow = false;
    if (_previsionDefinedEntities == 0)
        _drawProgressBar(-1, _totalMemory, ui->memoryPrevisionProgressBar, overflow,true, true);
    else
        _drawProgressBar(_initialMemory + _previsionMemory, _totalMemory, /*ui->progressBar_PrevisionMemory*/ui->memoryPrevisionProgressBar, overflow, true, true);
    if (!overflow && (_overloaded & MEMORY_CHARGE)) {
        ui->MemoryNameLabel->setStyleSheet("");
        _overloaded &= ~MEMORY_CHARGE;
        chargeChange = true;
        ui->warningMemoryLabel->setVisible(false);
    }
    QString text = QObject::tr("Number of entities: cur = ");
    if (_previsionDefinedEntities == 0) {
        ui->numberEntitieslabel->setText(text + QString("<b>%1</b>").arg(QString::number(_initialDefinedEntities)));
        ui->warningCPULabel->setVisible(false);
        ui->warningMemoryLabel->setVisible(false);
    } else {
        ui->numberEntitieslabel->setText(text + QString("<b>%1</b>").arg(QString::number(_initialDefinedEntities))
                                  + " / est = "
                                  +QString(" <b>%1 <I>(+%2)</I></b>")
                                  .arg(QString::number(_initialDefinedEntities + _previsionDefinedEntities))
                                  .arg(QString::number(_previsionDefinedEntities)));
    }
    if(chargeChange)
        emit overloaded(_nodeKey, _overloaded);
}

void NodeResourceWidget::addPrevisionResource(int cpu, int memory)
{
    _previsionCpu += cpu;
    _previsionMemory += memory;
    bool overflow = false;

    _drawProgressBar(_initialCpu + _previsionCpu, _totalCpu, ui->CPUPrevisionProgressBar, overflow);
    if (overflow && !(_overloaded & CPU_CHARGE)) {
        _overloaded |= CPU_CHARGE;
        ui->CPUNameLabel->setStyleSheet("QLabel{color:red;}");
        emit overloaded(_nodeKey, _overloaded);
        ui->warningCPULabel->setVisible(true);
    }
    overflow = false;
    _drawProgressBar(_initialMemory + _previsionMemory, _totalMemory, /*ui->progressBar_PrevisionMemory*/ui->memoryPrevisionProgressBar, overflow, true, true);
    if (overflow && !(_overloaded & MEMORY_CHARGE)) {
        _overloaded |= MEMORY_CHARGE;
        ui->MemoryNameLabel->setStyleSheet("QLabel{color:red;}");
        ui->warningMemoryLabel->setVisible(true);
        emit overloaded(_nodeKey, _overloaded);
    }
    _previsionDefinedEntities ++;

    QString text = QObject::tr("Number of entities: cur = ");
    ui->numberEntitieslabel->setText(text + QString("<b>%1</b>").arg(QString::number(_initialDefinedEntities))
                              + " / est = "
                              +QString(" <b>%1 <I>(+%2)</I></b>")
                              .arg(QString::number(_initialDefinedEntities + _previsionDefinedEntities))
                              .arg(QString::number(_previsionDefinedEntities)));
}

} // namespace Hnv
