/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "overlaywidget.h"

// Qt headers
#include <QSize>
#include <QEvent>
#include <QStyle>
#include <QAbstractScrollArea>

namespace Hnv {

/*******************************************************************************
* OverlayWidget
*******************************************************************************/
OverlayWidget::OverlayWidget(QWidget *parent)
    : StyledWidget(parent)
{
    _attachCorner = Qt::TopLeftCorner;
    parent->installEventFilter(this);
}

OverlayWidget::OverlayWidget(Qt::Corner corner, QWidget *parent)
    : StyledWidget(parent)
    , _attachCorner(corner)
{
    parent->installEventFilter(this);
}

void OverlayWidget::setAttachCorner(Qt::Corner corner)
{
    if (_attachCorner == corner)
        return;
    _attachCorner = corner;
    _attachToParent();
}

void OverlayWidget::setParent(QWidget *parent)
{
    StyledWidget::setParent(parent);
    _attachToParent();
}

/*******************************************************************************
* OverlayWidget - Protected
*/
void OverlayWidget::_attachToParent()
{
    QWidget *pWidget = parentWidget();
    if (pWidget != 0) {
        QSize widgetSize = size();
        int frameWidth = pWidget->style()->pixelMetric(QStyle::PM_DefaultFrameWidth);
        QMargins parentMargins = pWidget->contentsMargins();
        int lMargin = parentMargins.left();
//        int rMargin = parentMargins.right();
        int tMargin = parentMargins.top();
//        int bMargin = parentMargins.bottom();

        QRect parentRect = pWidget->contentsRect();
        QAbstractScrollArea * scrollWidget = qobject_cast<QAbstractScrollArea*>(pWidget);
        if (scrollWidget)
            parentRect = scrollWidget->viewport()->rect();

        int xPos = 0;
        int yPos = 0;
        int xRightPos = parentRect.right() - widgetSize.width() + lMargin + frameWidth;
        int yBottomPos = parentRect.height() - widgetSize.height() + tMargin + frameWidth;

        switch(_attachCorner) {
        case Qt::TopLeftCorner:
            break;
        case Qt::TopRightCorner:
            xPos = xRightPos;
            break;
        case Qt::BottomLeftCorner:
            yPos = yBottomPos;
            break;
        case Qt::BottomRightCorner:
            xPos = xRightPos;
            yPos = yBottomPos;
        }
        if (xPos != x() || yPos != y())
            move(xPos, yPos);
        setVisible(true);
    }
}

void OverlayWidget::paintEvent(QPaintEvent *event)
{
    _attachToParent();
    StyledWidget::paintEvent(event);
}

bool OverlayWidget::eventFilter(QObject *obj, QEvent *event)
{
    if(obj == parent() && event->type() == QEvent::Paint)
        _attachToParent();
    return StyledWidget::eventFilter(obj, event);
}

} // namespace Hnv
