/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "propertiestreewidget.h"
#include "simulationmodewidget.h"
#include "topologywidget.h"

// hynesim common headers
#include "common/initializer.h"

// Generated hynesim headers
#include "common/domainentry.h"
#include "common/switchentry.h"
#include "common/wireentry.h"
#include "common/hybridnetcardentry.h"
#include "common/topologygateentry.h"
#include "common/dynamipsrouterentry.h"

// hynesim hnsclient headers
#include <libhnsclient/common.h>
#include <libhnsclient/entity.h>
#include <libhnsclient/entry.h>

// Qt headers
#include <QTreeView>
#include <QMetaObject>
#include <QMetaProperty>
#include <QComboBox>
#include <QPainter>
#include <QCheckBox>
#include <QSpinBox>
#include <QApplication>

namespace Hnv {

HNS_STATIC_HASH_INITIALIZER(_sectionHash, SectionHash, QString, QList<EntityPropertiesSection::EntityPropertiesSectionDetails>)
SectionHash::SectionHash()
{
    QList<EntityPropertiesSection::EntityPropertiesSectionDetails> sectionDetailsList;
    sectionDetailsList.append(EntityPropertiesSection::EntityPropertiesSectionDetails(QObject::tr("Entity"), HnsClient::GenericEntity::staticMetaObject));
    sectionDetailsList.append(EntityPropertiesSection::EntityPropertiesSectionDetails(QObject::tr("Domain"), HnsClient::DomainEntity::staticMetaObject));
    insert(QString::fromStdString(Hns::DomainEntry::ice_staticId()), sectionDetailsList);
    sectionDetailsList.clear();

    sectionDetailsList.append(EntityPropertiesSection::EntityPropertiesSectionDetails(QObject::tr("Entity"), HnsClient::GenericEntity::staticMetaObject));
    sectionDetailsList.append(EntityPropertiesSection::EntityPropertiesSectionDetails(QObject::tr("Switch"), HnsClient::SwitchEntity::staticMetaObject));
    insert(QString::fromStdString(Hns::SwitchEntry::ice_staticId()), sectionDetailsList);
    sectionDetailsList.clear();

    sectionDetailsList.append(EntityPropertiesSection::EntityPropertiesSectionDetails(QObject::tr("Wire"), HnsClient::WireEntity::staticMetaObject));
    insert(QString::fromStdString(Hns::WireEntry::ice_staticId()), sectionDetailsList);
    sectionDetailsList.clear();

    sectionDetailsList.append(EntityPropertiesSection::EntityPropertiesSectionDetails(QObject::tr("Entity"), HnsClient::GenericEntity::staticMetaObject));
    insert(QString::fromStdString(Hns::HybridNetcardEntry::ice_staticId()), sectionDetailsList);
    sectionDetailsList.clear();

    sectionDetailsList.append(EntityPropertiesSection::EntityPropertiesSectionDetails(QObject::tr("Entity"), HnsClient::GenericEntity::staticMetaObject));
    insert(QString::fromStdString(Hns::TopologyGateEntry::ice_staticId()), sectionDetailsList);
    sectionDetailsList.clear();

    sectionDetailsList.append(EntityPropertiesSection::EntityPropertiesSectionDetails(QObject::tr("Entity"), HnsClient::GenericEntity::staticMetaObject));
    insert(QString::fromStdString(Hns::WifiAPEntry::ice_staticId()), sectionDetailsList);
    sectionDetailsList.clear();

    sectionDetailsList.append(EntityPropertiesSection::EntityPropertiesSectionDetails(QObject::tr("Entity"), HnsClient::GenericEntity::staticMetaObject));
    insert(QString::fromStdString(Hns::DynamipsRouterEntry::ice_staticId()), sectionDetailsList);
    sectionDetailsList.clear();
}

HNS_STATIC_HASH_INITIALIZER(_propertyNodeHash, PropertyNodeHash, QString, QMetaObject)
PropertyNodeHash::PropertyNodeHash()
{
    insert("wireType", WireTypeNode::staticMetaObject);
    insert("switchType", SwitchTypeNode::staticMetaObject);
}

HNS_STATIC_HASH_INITIALIZER(_propertyWorkerHash, PropertyWorkerHash, QString, QMetaObject)
PropertyWorkerHash::PropertyWorkerHash()
{
    insert("wireType", ComboBoxWorker::staticMetaObject);
    insert("switchType", ComboBoxWorker::staticMetaObject);
    insert("isHub", CheckBoxWorker::staticMetaObject);
}

/*******************************************************************************
* PropertyDelegateWorker
*******************************************************************************/
PropertyDelegateWorker::PropertyDelegateWorker(QObject *parent)
    : QObject(parent)
{
}

PropertyDelegateWorker::~PropertyDelegateWorker()
{
}

bool PropertyDelegateWorker::paint(HnsClient::GenericEntity *entity, QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index) const
{
    Q_UNUSED(entity)
    Q_UNUSED(painter)
    Q_UNUSED(option)
    Q_UNUSED(index)
    return false;
}

bool PropertyDelegateWorker::createEditor(HnsClient::GenericEntity *entity, QWidget *parent, const QStyleOptionViewItem &option, const QModelIndex &index, QWidget **resultWidget) const
{
    Q_UNUSED(entity)
    Q_UNUSED(parent)
    Q_UNUSED(option)
    Q_UNUSED(index)
    Q_UNUSED(resultWidget)
    return false;
}

bool PropertyDelegateWorker::setEditorData(HnsClient::GenericEntity *entity, QWidget *editor, const QModelIndex &index) const
{
    Q_UNUSED(entity)
    Q_UNUSED(editor)
    Q_UNUSED(index)
    return false;
}

bool PropertyDelegateWorker::setModelData(HnsClient::GenericEntity *entity, QWidget *editor, QAbstractItemModel *model, const QModelIndex &index) const
{
    Q_UNUSED(entity)
    Q_UNUSED(editor)
    Q_UNUSED(model)
    Q_UNUSED(index)
    return false;
}

bool PropertyDelegateWorker::updateEditorGeometry(HnsClient::GenericEntity *entity, QWidget *editor, const QStyleOptionViewItem &option, const QModelIndex &index) const
{
    Q_UNUSED(entity)
    Q_UNUSED(editor)
    Q_UNUSED(option)
    Q_UNUSED(index)
    return false;
}


/*******************************************************************************
* ComboBoxWorker
*******************************************************************************/
ComboBoxWorker::ComboBoxWorker(QObject *parent)
    : PropertyDelegateWorker(parent)
{
}

ComboBoxWorker::~ComboBoxWorker()
{
}

bool ComboBoxWorker::createEditor(HnsClient::GenericEntity *entity, QWidget *parent, const QStyleOptionViewItem &option, const QModelIndex &index, QWidget **resultWidget) const
{
    Q_UNUSED(entity)
    Q_UNUSED(option)
    bool result = false;
    QMap<QString, QVariant> comboBoxItemMap = index.data(PropertyNode::PROPERTY_EDITOR_DATA_ROLE).toMap();
    if (comboBoxItemMap.count() > 0) {
        QComboBox *cb = new QComboBox(parent);
        QMapIterator<QString, QVariant> iterator(comboBoxItemMap);
        while(iterator.hasNext()) {
            iterator.next();
            cb->addItem(iterator.key(), iterator.value());
        }
        *resultWidget = cb;
        result = true;
    }
    return result;
}

bool ComboBoxWorker::setEditorData(HnsClient::GenericEntity *entity, QWidget *editor, const QModelIndex &index) const
{
    Q_UNUSED(entity)
    if (QComboBox *cb = qobject_cast<QComboBox *>(editor)) {
        QString currentText = index.data(Qt::DisplayRole).toString();
        int cbIndex = cb->findText(currentText);
        if (cbIndex >= 0)
            cb->setCurrentIndex(cbIndex);
        return true;
    }
    return false;
}

bool ComboBoxWorker::setModelData(HnsClient::GenericEntity *entity, QWidget *editor, QAbstractItemModel *model, const QModelIndex &index) const
{
    Q_UNUSED(entity)
    if(QComboBox *cb = qobject_cast<QComboBox *>(editor)) {
        model->setData(index, cb->itemData(cb->currentIndex()));
        return true;
    }
    return false;
}


/*******************************************************************************
* CheckBoxWorker
*******************************************************************************/
CheckBoxWorker::CheckBoxWorker(QObject *parent)
    : PropertyDelegateWorker(parent)
{
}

CheckBoxWorker::~CheckBoxWorker()
{
}

bool CheckBoxWorker::paint(HnsClient::GenericEntity *entity, QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index) const
{
    Q_UNUSED(entity)
    Q_UNUSED(index)

    if (option.showDecorationSelected && option.state.testFlag(QStyle::State_Selected)) {
        painter->fillRect(option.rect, option.palette.highlight());
    }
    QVariant displayValue = index.data();
    if (displayValue.isValid() && displayValue.type() == QVariant::Bool) {
        QStyleOptionButton checkBoxStyleOption;
        checkBoxStyleOption.state |= QStyle::State_Raised;
        checkBoxStyleOption.state |= QStyle::State_Enabled;
        if (displayValue == true)
            checkBoxStyleOption.state |= QStyle::State_On;
        else
            checkBoxStyleOption.state |= QStyle::State_Off;
        QRect checkBoxRect = QApplication::style()->subElementRect(QStyle::SE_CheckBoxIndicator, &checkBoxStyleOption);
        checkBoxStyleOption.rect = QApplication::style()->alignedRect(Qt::LayoutDirectionAuto, Qt::AlignCenter, checkBoxRect.size(), option.rect);
        QApplication::style()->drawControl(QStyle::CE_CheckBox, &checkBoxStyleOption, painter);
    }
    return true;
}

bool CheckBoxWorker::createEditor(HnsClient::GenericEntity *entity, QWidget *parent, const QStyleOptionViewItem &option, const QModelIndex &index, QWidget **resultWidget) const
{
    Q_UNUSED(entity)
    Q_UNUSED(option)
    bool result = false;
    QVariant displayValue = index.data();
    if (displayValue.isValid() && displayValue.type() == QVariant::Bool) {
        QCheckBox *cb = new QCheckBox(parent);
        *resultWidget = cb;
        result = true;
    }
    return result;
}

bool CheckBoxWorker::setEditorData(HnsClient::GenericEntity *entity, QWidget *editor, const QModelIndex &index) const
{
    Q_UNUSED(entity)
    if (QCheckBox *cb = qobject_cast<QCheckBox *>(editor)) {
        cb->setChecked(index.data().toBool());
        return true;
    }
    return false;
}

bool CheckBoxWorker::setModelData(HnsClient::GenericEntity *entity, QWidget *editor, QAbstractItemModel *model, const QModelIndex &index) const
{
    Q_UNUSED(entity)
    if(QCheckBox *cb = qobject_cast<QCheckBox *>(editor)) {
        model->setData(index, cb->isChecked());
        return true;
    }
    return false;
}

bool CheckBoxWorker::updateEditorGeometry(HnsClient::GenericEntity *entity, QWidget *editor, const QStyleOptionViewItem &option, const QModelIndex &index) const
{
    Q_UNUSED(entity)
    Q_UNUSED(index)
    if(QCheckBox *cb = qobject_cast<QCheckBox *>(editor)) {
        QRect targetRect = QApplication::style()->subElementRect(QStyle::SE_CheckBoxIndicator, &option);
        targetRect = QApplication::style()->alignedRect(Qt::LayoutDirectionAuto, Qt::AlignCenter, targetRect.size(), option.rect);
        cb->move(targetRect.topLeft());
        return true;
    }
    return false;
}



/*******************************************************************************
* PropertyDelegateFactory
*******************************************************************************/
PropertyDelegateFactory::PropertyDelegateFactory(QObject *parent)
    : QObject(parent)
{
}

PropertyDelegateFactory::~PropertyDelegateFactory()
{
    qDeleteAll(_workerHash);
}

void PropertyDelegateFactory::setEntity(HnsClient::GenericEntity *entity)
{
    _entity = entity;
}

bool PropertyDelegateFactory::paint(QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index)
{
    bool result = false;
    PropertyDelegateWorker *worker = _propertyWorker(index);
    if (worker) {
        worker->metaObject()->invokeMethod(
                    worker
                    , "paint"
                    , Q_RETURN_ARG(bool, result)
                    , Q_ARG(HnsClient::GenericEntity *, _entity)
                    , Q_ARG(QPainter *, painter)
                    , Q_ARG(const QStyleOptionViewItem &, option)
                    , Q_ARG(const QModelIndex &, index));
    }
    return result;
}

bool PropertyDelegateFactory::createEditor(QWidget *parent, const QStyleOptionViewItem &option, const QModelIndex &index, QWidget **resultWidget)
{
    bool result = false;
    PropertyDelegateWorker *worker = _propertyWorker(index);
    if (worker) {
        worker->metaObject()->invokeMethod(
                    worker
                    , "createEditor"
                    , Q_RETURN_ARG(bool, result)
                    , Q_ARG(HnsClient::GenericEntity *, _entity)
                    , Q_ARG(QWidget *, parent)
                    , Q_ARG(const QStyleOptionViewItem &, option)
                    , Q_ARG(const QModelIndex &, index)
                    , Q_ARG(QWidget **, resultWidget));
    }
    return result;
}

bool PropertyDelegateFactory::setEditorData(QWidget *editor, const QModelIndex &index)
{
    bool result = false;
    PropertyDelegateWorker *worker = _propertyWorker(index);
    if (worker) {
        worker->metaObject()->invokeMethod(
                    worker
                    , "setEditorData"
                    , Q_RETURN_ARG(bool, result)
                    , Q_ARG(HnsClient::GenericEntity *, _entity)
                    , Q_ARG(QWidget *, editor)
                    , Q_ARG(const QModelIndex &, index));
    }
    return result;
}

bool PropertyDelegateFactory::setModelData(QWidget *editor, QAbstractItemModel *model, const QModelIndex &index)
{
    bool result = false;
    PropertyDelegateWorker *worker = _propertyWorker(index);
    if (worker) {
        worker->metaObject()->invokeMethod(
                    worker
                    , "setModelData"
                    , Q_RETURN_ARG(bool, result)
                    , Q_ARG(HnsClient::GenericEntity *, _entity)
                    , Q_ARG(QWidget *, editor)
                    , Q_ARG(QAbstractItemModel *, model)
                    , Q_ARG(const QModelIndex &, index));
    }
    return result;
}

bool PropertyDelegateFactory::updateEditorGeometry(QWidget *editor, const QStyleOptionViewItem &option, const QModelIndex &index)
{
    bool result = false;
    PropertyDelegateWorker *worker = _propertyWorker(index);
    if (worker) {
        worker->metaObject()->invokeMethod(
                    worker
                    , "updateEditorGeometry"
                    , Q_RETURN_ARG(bool, result)
                    , Q_ARG(HnsClient::GenericEntity *, _entity)
                    , Q_ARG(QWidget *, editor)
                    , Q_ARG(const QStyleOptionViewItem &, option)
                    , Q_ARG(const QModelIndex &, index));
    }
    return result;
}

PropertyDelegateWorker *PropertyDelegateFactory::_propertyWorker(const QModelIndex &index)
{
    PropertyDelegateWorker *worker = 0;
    QVariant propertyData = index.data(PropertyNode::PROPERTY_NAME_ROLE);
    if (propertyData.isValid())
        worker = _propertyWorker(propertyData.toString());
    return worker;
}

PropertyDelegateWorker *PropertyDelegateFactory::_propertyWorker(const QString &propertyName)
{
    PropertyDelegateWorker *result = 0;
    if (!propertyName.isEmpty() && _propertyWorkerHash()->contains(propertyName)) {
        if (!_workerHash.contains(propertyName)) {
            QMetaObject workerMetaObject = _propertyWorkerHash()->value(propertyName);
            QObject *newObject = workerMetaObject.newInstance(Q_ARG(QObject *, this));
            if (newObject)
                _workerHash.insert(propertyName, qobject_cast<PropertyDelegateWorker *>(newObject));
        }
        result = _workerHash.value(propertyName);
    }
    return result;
}


/*******************************************************************************
* PropertyDelegate
*******************************************************************************/
PropertyDelegate::PropertyDelegate(QObject *parent)
    : QStyledItemDelegate(parent)
    , _delegateFactory(new PropertyDelegateFactory(this))
{
}

PropertyDelegate::~PropertyDelegate()
{
    _delegateFactory->deleteLater();
}

void PropertyDelegate::setEntity(HnsClient::GenericEntity *entity)
{
    _delegateFactory->setEntity(entity);
}

void PropertyDelegate::paint(QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index) const
{
    Q_ASSERT(index.isValid());

    QStyleOptionViewItemV4 opt = option;
    initStyleOption(&opt, index);
    bool factoryResult = _delegateFactory->paint(painter, opt, index);
    if (!factoryResult)
        QStyledItemDelegate::paint(painter, option, index);
}

QWidget *PropertyDelegate::createEditor(QWidget *parent, const QStyleOptionViewItem &option, const QModelIndex &index) const
{
    if (!index.isValid())
        return 0;

    QWidget *result = 0;
    QStyleOptionViewItemV4 opt = option;
    initStyleOption(&opt, index);
    bool factoryResult = _delegateFactory->createEditor(parent, opt, index, &result);
    if (!factoryResult)
        result = QStyledItemDelegate::createEditor(parent, option, index);

    QMap<QString, QVariant> propertyInformation = index.data(PropertyNode::PROPERTY_INFORMATION_ROLE).toMap();
    if (propertyInformation.count() > 0) {
        if (QSpinBox *spinBox = qobject_cast<QSpinBox *>(result)) {
            QVariant minData = propertyInformation.value(HnsClient::Constants::PROPERTY_MINIMUM);
            QVariant maxData = propertyInformation.value(HnsClient::Constants::PROPERTY_MAXIMUM);
            if (minData.isValid())
                spinBox->setMinimum(minData.toInt());
            if (maxData.isValid())
                spinBox->setMaximum(maxData.toInt());
        }
    }

    return result;
}

void PropertyDelegate::setEditorData(QWidget *editor, const QModelIndex &index) const
{
    bool factoryResult = _delegateFactory->setEditorData(editor, index);
    if (!factoryResult)
        QStyledItemDelegate::setEditorData(editor, index);
}

void PropertyDelegate::setModelData(QWidget *editor, QAbstractItemModel *model, const QModelIndex &index) const
{
    bool factoryResult = _delegateFactory->setModelData(editor, model, index);
    if (!factoryResult)
        QStyledItemDelegate::setModelData(editor, model, index);
}

void PropertyDelegate::updateEditorGeometry(QWidget *editor, const QStyleOptionViewItem &option, const QModelIndex &index) const
{
    if (!editor)
        return;
    Q_ASSERT(index.isValid());

    bool factoryResult = _delegateFactory->updateEditorGeometry(editor, option, index);
    if (!factoryResult)
        QStyledItemDelegate::updateEditorGeometry(editor, option, index);
}


/*******************************************************************************
* PropertyNode
*******************************************************************************/
PropertyNode::PropertyNode(QObject *parent)
    : TreeModelItem(NODE_COLUMN_COUNT, parent)
    , _persistentEditorRequested(false)
{
}

PropertyNode::~PropertyNode()
{
}

Qt::ItemFlags PropertyNode::flags(int column) const
{
    Qt::ItemFlags result = TreeModelItem::flags(column);
    if (column == COLUMN_VALUE && isWriteable())
        result |= Qt::ItemIsEditable;
    return result;
}

QVariant PropertyNode::data(int column, int role) const
{
    switch (role) {
    case PROPERTY_INFORMATION_ROLE: {
        return propertyInformation();
        break;
    }
    case PROPERTY_EDITOR_DATA_ROLE: {
        return propertyEditorData();
        break;
    }
    case PROPERTY_NAME_ROLE: {
        return propertyName();
        break;
    }
    case KEY_ROLE: {
        return nodeKey();
        break;
    }
    case VALUE_ROLE: {
        return nodeValue();
        break;
    }
    case WRITEABLE_ROLE: {
        return isWriteable();
        break;
    }
    case Qt::DecorationRole: {
        if (column == COLUMN_KEY)
            return decoration();
        break;
    }
    case Qt::DisplayRole: {
        if (column == COLUMN_KEY)
            return nodeKey();
        if (column == COLUMN_VALUE)
            return nodeDecoratedValue();
        break;
    }
    case Qt::EditRole: {
        if (column == COLUMN_VALUE)
            return nodeValue();
        break;
    }
    case Qt::ForegroundRole: {
        QBrush foregroundBrush = TreeModelItem::data(column, role).value<QBrush>();
        if (!isWriteable()) {
            foregroundBrush = QApplication::palette().mid();
        }
        return foregroundBrush;
        break;
    }
    default:
        break;
    }
    return TreeModelItem::data(column, role);
}

bool PropertyNode::setData(int column, const QVariant &value, int role)
{
    bool result = false;
    if (column == COLUMN_VALUE && isWriteable() && role == Qt::EditRole)
        result = setValue(value);
    return result;
}

QVariant PropertyNode::nodeDecoratedValue() const
{
    return nodeValue();
}

QList<PropertyNode *> PropertyNode::_persistentEditorRequestList()
{
    QList<PropertyNode *> result;
    if (_persistentEditorRequested)
        result.append(this);
    for(int row = 0; row < childCount(); ++row) {
        PropertyNode *childNode = qobject_cast<PropertyNode *>(child(row));
        if (childNode)
            result.append(childNode->_persistentEditorRequestList());
    }
    return result;
}


/*******************************************************************************
* EntityPropertyNode
*******************************************************************************/
EntityPropertyNode::EntityPropertyNode(HnsClient::GenericEntity *entity, const QMetaProperty &property, QObject *parent)
    : PropertyNode(parent)
    , _entity(entity)
    , _propertyName(property.name())
    , _isReadOnly(!property.isWritable())
    , _key(HnsClient::GenericEntity::propertyDescription(_propertyName))
{
    if (property.hasNotifySignal()) {
        QString sig = QString("%1%2")
                .arg(QString::number(QSIGNAL_CODE))
                .arg(property.notifySignal().signature());
        connect(_entity, qPrintable(sig), this, SLOT(_dataChangedSlot()));
    }
}

EntityPropertyNode::EntityPropertyNode(HnsClient::GenericEntity *entity, const QString &key, QObject *parent)
    : PropertyNode(parent)
    , _entity(entity)
    , _propertyName(QString())
    , _isReadOnly(true)
    , _key(key)
{
}

EntityPropertyNode::~EntityPropertyNode()
{
}

bool EntityPropertyNode::isWriteable() const
{
    return (!_isReadOnly && !_entity->isDefined());
}

QString EntityPropertyNode::nodeKey() const
{
    return _key;
}

QVariant EntityPropertyNode::nodeValue() const
{
    QVariant value;
    if (!_propertyName.isEmpty()) {
        QVariant data = _entity->property(qPrintable(_propertyName));
        if (data.isValid())
            value = data;
        else
            value = tr("INVALID PROPERTY");
    }
    return value;
}

bool EntityPropertyNode::setValue(const QVariant &value)
{
    return _entity->setProperty(qPrintable(_propertyName), value);
}

QIcon EntityPropertyNode::decoration() const
{
    return QIcon();
}

QString EntityPropertyNode::propertyName() const
{
    return _propertyName;
}

QMap<QString, QVariant> EntityPropertyNode::propertyEditorData() const
{
    return QMap<QString, QVariant>();
}

QMap<QString, QVariant> Hnv::EntityPropertyNode::propertyInformation() const
{
    return _entity->propertyInformation(propertyName());
}

void EntityPropertyNode::_dataChangedSlot()
{
    columnChanged(COLUMN_VALUE);
}

HnsClient::GenericEntity *EntityPropertyNode::entity() const
{
    return _entity;
}


/*******************************************************************************
* EntityPropertyDecorativeNode
*******************************************************************************/
EntityPropertyDecorativeNode::EntityPropertyDecorativeNode(HnsClient::GenericEntity *entity, QString key, QObject *parent)
    : EntityPropertyNode(entity, key, parent)
{
}

EntityPropertyDecorativeNode::~EntityPropertyDecorativeNode()
{
}

QVariant EntityPropertyDecorativeNode::data(int column, int role) const
{
    QPalette appPalette = QApplication::palette();
    if (role == Qt::BackgroundRole)
        return appPalette.dark();
    else if (role == Qt::ForegroundRole)
        return appPalette.light();
    else if (role == Qt::FontRole) {
        QFont font = EntityPropertyNode::data(column, role).value<QFont>();
        font.setBold(true);
        return font;
    }
    return EntityPropertyNode::data(column, role);
}


/*******************************************************************************
* EntityPropertiesSection
*******************************************************************************/
EntityPropertiesSection::EntityPropertiesSectionDetails::EntityPropertiesSectionDetails(QString name, QMetaObject metaObject)
{
    sectionName = name;
    targetMetaObject = metaObject;
}

EntityPropertiesSection::EntityPropertiesSection(HnsClient::GenericEntity *entity, QString key, const QMetaObject &metaObject, QObject *parent)
    : EntityPropertyDecorativeNode(entity, key, parent)
{
    for(int i = metaObject.propertyOffset(); i < metaObject.propertyCount(); ++i) {
        QMetaProperty property = metaObject.property(i);
        QMetaObject nodeMetaObject = _propertyNodeHash()->value(QString::fromStdString(property.name()), EntityPropertyNode::staticMetaObject);
        QGenericArgument arg1 = Q_ARG(HnsClient::GenericEntity *, entity);
        QGenericArgument arg2 = Q_ARG(QMetaProperty, property);
        QGenericArgument arg3 = Q_ARG(QObject *, this);

        QObject *newObject = nodeMetaObject.newInstance(arg1, arg2, arg3);
        if (newObject) {
            EntityPropertyNode *node = qobject_cast<EntityPropertyNode *>(newObject);
            if (node)
                appendChild(node);
        }
    }
}

EntityPropertiesSection::~EntityPropertiesSection()
{
}


/*******************************************************************************
* WireTypeNode
*******************************************************************************/
WireTypeNode::WireTypeNode(HnsClient::GenericEntity *entity, const QMetaProperty &property, QObject *parent)
    : EntityPropertyNode(entity, property, parent)
{
}

WireTypeNode::~WireTypeNode()
{
}

QVariant WireTypeNode::nodeDecoratedValue() const
{
    QVariant result = nodeValue();
    HnsClient::WireEntity *wireEntity = qobject_cast<HnsClient::WireEntity *>(entity());
    if (wireEntity)
        result = QVariant(wireEntity->wireEntry()->supportedImplementation().value(nodeValue().toString()));
    return result;
}

QMap<QString, QVariant> WireTypeNode::propertyEditorData() const
{
    QMap<QString, QVariant> editorData;
    HnsClient::WireEntity *wireEntity = qobject_cast<HnsClient::WireEntity *>(entity());
    if (wireEntity) {
        QMapIterator<QString, QString> iterator(wireEntity->wireEntry()->supportedImplementation());
        while(iterator.hasNext()) {
            iterator.next();
            editorData.insert(iterator.value(), iterator.key());
        }
    }
    return editorData;
}


/*******************************************************************************
* SwitchTypeNode
*******************************************************************************/
SwitchTypeNode::SwitchTypeNode(HnsClient::GenericEntity *entity, const QMetaProperty &property, QObject *parent)
    : EntityPropertyNode(entity, property, parent)
{
}

SwitchTypeNode::~SwitchTypeNode()
{
}

QVariant SwitchTypeNode::nodeDecoratedValue() const
{
    QVariant result = nodeValue();
    HnsClient::SwitchEntity *switchEntity = qobject_cast<HnsClient::SwitchEntity *>(entity());
    if (switchEntity)
        result = QVariant(switchEntity->switchEntry()->supportedImplementation().value(nodeValue().toString()));
    return result;
}

QMap<QString, QVariant> SwitchTypeNode::propertyEditorData() const
{
    QMap<QString, QVariant> editorData;
    HnsClient::SwitchEntity *switchEntity = qobject_cast<HnsClient::SwitchEntity *>(entity());
    if (switchEntity) {
        QMapIterator<QString, QString> iterator(switchEntity->switchEntry()->supportedImplementation());
        while(iterator.hasNext()) {
            iterator.next();
            editorData.insert(iterator.value(), iterator.key());
        }
    }
    return editorData;
}


/*******************************************************************************
* PropertiesModel
*******************************************************************************/
PropertiesModel::PropertiesModel(QWidget *parent)
    : TreeModel(parent)
{
    _initRootItem(PropertyNode::NODE_COLUMN_COUNT);
}

PropertiesModel::~PropertiesModel()
{
}

void PropertiesModel::setEntity(HnsClient::GenericEntity *entity)
{
    beginResetModel();
    _persistentEditorRequestIndexes.clear();
    _rootItem->clear();

    if (entity) {
        QList<EntityPropertiesSection::EntityPropertiesSectionDetails> sectionList;
        QString iceId = entity->iceId();
        if(_sectionHash()->contains(iceId))
            sectionList = _sectionHash()->value(iceId);
        else {
            QVector<std::string> ice_ids = QVector<std::string>::fromStdVector(entity->getEntry()->entityEntry()->ice_ids());
            for(int i = ice_ids.size() - 1; i >= 0; i--) {
                QString parentClassName = QString::fromStdString(ice_ids.at(i));
                if (_sectionHash()->contains(parentClassName)) {
                    sectionList = _sectionHash()->value(parentClassName);
                    _sectionHash()->insert(iceId, sectionList);
                }
            }
        }
        foreach(EntityPropertiesSection::EntityPropertiesSectionDetails details, sectionList) {
            EntityPropertiesSection *newSection = new EntityPropertiesSection(entity, details.sectionName, details.targetMetaObject, this);
            _rootItem->appendChild(newSection);
            foreach(PropertyNode *node, newSection->_persistentEditorRequestList()) {
                QModelIndex idx = indexFromNode(node, PropertyNode::COLUMN_VALUE);
                if (idx.isValid())
                    _persistentEditorRequestIndexes.append(idx);
            }
        }
    }
    endResetModel();
}

QVariant PropertiesModel::headerData(int section, Qt::Orientation orientation, int role) const
{
    if (role == Qt::DisplayRole && orientation == Qt::Horizontal) {
        switch (section) {
        case PropertyNode::COLUMN_KEY:
            return tr("Property");
            break;
        case PropertyNode::COLUMN_VALUE:
            return tr("Value");
            break;
        default:
            break;
        }
    }
    return TreeModel::headerData(section, orientation, role);
}

QModelIndexList PropertiesModel::persistentEditorRequestList() const
{
    return _persistentEditorRequestIndexes;
}


/*******************************************************************************
* PropertiesTreeWidget
*******************************************************************************/
PropertiesTreeWidget::PropertiesTreeWidget(QWidget *parent)
    : SideBarTreeWidget(parent)
    , _propertiesModel(new PropertiesModel(this))
    , _currentEntity(0)
{
    _delegate = new PropertyDelegate(this);
    _treeView->setSelectionBehavior(QAbstractItemView::SelectRows);
    _treeView->header()->setVisible(true);
    _treeView->setItemDelegateForColumn(PropertyNode::COLUMN_VALUE, _delegate);


    _treeView->setSelectionMode(QAbstractItemView::ExtendedSelection);
    QWidget *widget = sideBarParent();
    if (widget) {
        SimulationModeWidget *simulationModeWidget = qobject_cast<SimulationModeWidget *>(widget);
        if (simulationModeWidget) {
            TopologyWidget *topologyWidget = simulationModeWidget->topologyWidget();
            if (topologyWidget) {
                setViewModel(_propertiesModel);
                connect(topologyWidget, SIGNAL(selectionChanged(QList<HnsClient::GenericEntity*>)), this, SLOT(_selectionChangedSlot(QList<HnsClient::GenericEntity*>)));
                connect(topologyWidget, SIGNAL(topologyChanged(HnsClient::Topology*)), this, SLOT(_topologyChangedSlot()));
                _selectionChangedSlot(topologyWidget->getCurrentEntitiesSelected(true));
            }
        }
    }
}

PropertiesTreeWidget::~PropertiesTreeWidget()
{
    _currentEntity = 0;
    clearModel();
}

QString PropertiesTreeWidget::title()
{
    return tr("Properties");
}

QString PropertiesTreeWidget::sideBarId()
{
    return "Properties";
}

void PropertiesTreeWidget::_indexClicked(const QModelIndex &index)
{
    if (index.isValid()) {
    }
}

void PropertiesTreeWidget::_indexDoubleClicked(const QModelIndex &index)
{
    if (index.isValid()) {
    }
}

void PropertiesTreeWidget::_contextMenuRequested(const QModelIndex &index, const QPoint &position)
{
    Q_UNUSED(position)
    if (index.isValid()) {
    }
}

void PropertiesTreeWidget::_setEntity(HnsClient::GenericEntity *entity)
{
    if (_currentEntity)
        disconnect(_currentEntity, 0, this, 0);
    _currentEntity = entity;
    if (_currentEntity)
        connect(_currentEntity, SIGNAL(destroyed()), this, SLOT(_currentEntityDestroyed()));

    _propertiesModel->setEntity(entity);
    foreach(const QModelIndex idx, _propertiesModel->persistentEditorRequestList())
        _treeView->openPersistentEditor(idx);
    _delegate->setEntity(entity);
}

void PropertiesTreeWidget::_selectionChangedSlot(QList<HnsClient::GenericEntity *> selectedEntities)
{
    if (selectedEntities.count() == 1)
        _setEntity(selectedEntities.first());
    else if (selectedEntities.count() == 0)
        _setEntity(0);
    _treeView->expandToDepth(1);
}

void PropertiesTreeWidget::_topologyChangedSlot()
{
    SimulationModeWidget *simulationModeWidget = SimulationModeWidget::instance();
    if (simulationModeWidget) {
        TopologyWidget *topologyWidget = simulationModeWidget->topologyWidget();
        if (topologyWidget) {
            _selectionChangedSlot(topologyWidget->getCurrentEntitiesSelected(true));
        }
    }
}

void PropertiesTreeWidget::_currentEntityDestroyed()
{
    _setEntity(0);
}

} // namespace Hnv
