/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef HNV_PROPERTIESTREEWIDGET_H
#define HNV_PROPERTIESTREEWIDGET_H

// Project headers
#include "sidebar.h"
#include "treemodel.h"
#include "treemodelitem.h"

// Qt headers
#include <QStyledItemDelegate>

namespace HnsClient {
class GenericEntity;
} // namespace HnsClient

namespace Hnv {

/*******************************************************************************
* PropertyDelegateWorker
*******************************************************************************/
class PropertyDelegateWorker : public QObject
{
    Q_OBJECT
public:
    explicit PropertyDelegateWorker(QObject *parent = 0);
    ~PropertyDelegateWorker();
    Q_INVOKABLE virtual bool paint(HnsClient::GenericEntity *entity, QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index) const;
    Q_INVOKABLE virtual bool createEditor(HnsClient::GenericEntity *entity, QWidget *parent, const QStyleOptionViewItem &option, const QModelIndex &index, QWidget **resultWidget) const;
    Q_INVOKABLE virtual bool setEditorData(HnsClient::GenericEntity *entity, QWidget *editor, const QModelIndex &index) const;
    Q_INVOKABLE virtual bool setModelData(HnsClient::GenericEntity *entity, QWidget *editor, QAbstractItemModel *model, const QModelIndex &index) const;
    Q_INVOKABLE virtual bool updateEditorGeometry(HnsClient::GenericEntity *entity, QWidget *editor, const QStyleOptionViewItem &option, const QModelIndex &index) const;
};

/*******************************************************************************
* ComboBoxWorker
*******************************************************************************/
class ComboBoxWorker : public PropertyDelegateWorker
{
    Q_OBJECT
public:
    Q_INVOKABLE explicit ComboBoxWorker(QObject *parent = 0);
    ~ComboBoxWorker();
    virtual bool createEditor(HnsClient::GenericEntity *entity, QWidget *parent, const QStyleOptionViewItem &option, const QModelIndex &index, QWidget **resultWidget) const;
    virtual bool setEditorData(HnsClient::GenericEntity *entity, QWidget *editor, const QModelIndex &index) const;
    virtual bool setModelData(HnsClient::GenericEntity *entity, QWidget *editor, QAbstractItemModel *model, const QModelIndex &index) const;
};


/*******************************************************************************
* CheckBoxWorker
*******************************************************************************/
class CheckBoxWorker : public PropertyDelegateWorker
{
    Q_OBJECT
public:
    Q_INVOKABLE explicit CheckBoxWorker(QObject *parent = 0);
    ~CheckBoxWorker();
    virtual bool paint(HnsClient::GenericEntity *entity, QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index) const;
    virtual bool createEditor(HnsClient::GenericEntity *entity, QWidget *parent, const QStyleOptionViewItem &option, const QModelIndex &index, QWidget **resultWidget) const;
    virtual bool setEditorData(HnsClient::GenericEntity *entity, QWidget *editor, const QModelIndex &index) const;
    virtual bool setModelData(HnsClient::GenericEntity *entity, QWidget *editor, QAbstractItemModel *model, const QModelIndex &index) const;
    virtual bool updateEditorGeometry(HnsClient::GenericEntity *entity, QWidget *editor, const QStyleOptionViewItem &option, const QModelIndex &index) const;
};


/*******************************************************************************
* PropertyDelegateFactory
*******************************************************************************/
class PropertyDelegateFactory : public QObject
{
    Q_OBJECT
public:
    explicit PropertyDelegateFactory(QObject *parent = 0);
    ~PropertyDelegateFactory();
    void setEntity(HnsClient::GenericEntity *entity);
    bool paint(QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index);
    bool createEditor(QWidget *parent, const QStyleOptionViewItem &option, const QModelIndex &index, QWidget **resultWidget);
    bool setEditorData(QWidget *editor, const QModelIndex &index);
    bool setModelData(QWidget *editor, QAbstractItemModel *model, const QModelIndex &index);
    bool updateEditorGeometry(QWidget *editor, const QStyleOptionViewItem &option, const QModelIndex &index);

private:
    PropertyDelegateWorker *_propertyWorker(const QModelIndex &index);
    PropertyDelegateWorker *_propertyWorker(const QString &propertyName);
    QHash<QString, PropertyDelegateWorker *> _workerHash;
    HnsClient::GenericEntity *_entity;
};


/*******************************************************************************
* PropertyDelegate
*******************************************************************************/
class PropertyDelegate : public QStyledItemDelegate
{
    Q_OBJECT
public:
    PropertyDelegate(QObject *parent = 0);
    ~PropertyDelegate();
    void setEntity(HnsClient::GenericEntity *entity);

protected:
    void paint(QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index) const;
    QWidget *createEditor(QWidget *parent, const QStyleOptionViewItem &option, const QModelIndex &index) const;
    void setEditorData(QWidget *editor, const QModelIndex &index) const;
    void setModelData(QWidget *editor, QAbstractItemModel *model, const QModelIndex &index) const;
    void updateEditorGeometry(QWidget *editor, const QStyleOptionViewItem &option, const QModelIndex &index) const;

private:
    PropertyDelegateFactory *_delegateFactory;
};


/*******************************************************************************
* PropertyNode
*******************************************************************************/
class PropertyNode : public TreeModelItem
{
    Q_OBJECT
public:
    enum PropertyNodeColumn {
          COLUMN_KEY
        , COLUMN_VALUE
        , NODE_COLUMN_COUNT
    };

    enum PropertyNodeRole {
          KEY_ROLE = TreeModelItem::InternalSortRole +1
        , VALUE_ROLE
        , WRITEABLE_ROLE
        , PROPERTY_NAME_ROLE
        , PROPERTY_EDITOR_DATA_ROLE
        , PROPERTY_INFORMATION_ROLE
    };

    explicit PropertyNode(QObject *parent);
    ~PropertyNode();
    virtual Qt::ItemFlags flags(int column) const;
    virtual QVariant data(int column = 0, int role = Qt::DisplayRole) const;
    virtual bool setData(int column, const QVariant &value, int role);

    virtual bool isWriteable() const = 0;
    virtual QString nodeKey() const = 0;
    virtual QVariant nodeValue() const = 0;
    virtual bool setValue(const QVariant &value) = 0;
    virtual QIcon decoration() const = 0;
    virtual QString propertyName() const = 0;
    virtual QMap<QString, QVariant> propertyEditorData() const = 0;
    virtual QMap<QString, QVariant> propertyInformation() const = 0;
    virtual QVariant nodeDecoratedValue() const;

protected:
    QList<PropertyNode *> _persistentEditorRequestList();
    bool _persistentEditorRequested;

    friend class PropertiesModel;
};


/*******************************************************************************
* EntityPropertyNode
*******************************************************************************/
class EntityPropertyNode : public PropertyNode
{
    Q_OBJECT
public:
    Q_INVOKABLE explicit EntityPropertyNode(HnsClient::GenericEntity *entity, const QMetaProperty &property, QObject *parent = 0);
    EntityPropertyNode(HnsClient::GenericEntity *entity, const QString &key, QObject *parent = 0);
    ~EntityPropertyNode();

    virtual bool isWriteable() const;
    virtual QString nodeKey() const;
    virtual QVariant nodeValue() const;
    virtual bool setValue(const QVariant &value);
    virtual QIcon decoration() const;
    virtual QString propertyName() const;
    virtual QMap<QString, QVariant> propertyEditorData() const;
    virtual QMap<QString, QVariant> propertyInformation() const;

signals:
    void dataChanged();

protected slots:
    void _dataChangedSlot();

protected:
    HnsClient::GenericEntity *entity() const;

private:
    HnsClient::GenericEntity *_entity;
    QString _propertyName;
    bool _isReadOnly;
    QString _key;
};


/*******************************************************************************
* EntityPropertyDecorativeNode
*******************************************************************************/
class EntityPropertyDecorativeNode : public EntityPropertyNode
{
    Q_OBJECT
public:
    explicit EntityPropertyDecorativeNode(HnsClient::GenericEntity *entity, QString key, QObject *parent = 0);
    ~EntityPropertyDecorativeNode();
    virtual QVariant data(int column = 0, int role = Qt::DisplayRole) const;
};


/*******************************************************************************
* EntityPropertiesSection
*******************************************************************************/
class EntityPropertiesSection : public EntityPropertyDecorativeNode
{
    Q_OBJECT
public:
    struct EntityPropertiesSectionDetails
    {
        EntityPropertiesSectionDetails(QString name, QMetaObject metaObject);

        QString sectionName;
        QMetaObject targetMetaObject;
    };

    explicit EntityPropertiesSection(HnsClient::GenericEntity *entity, QString key, const QMetaObject &metaObject, QObject *parent = 0);
    ~EntityPropertiesSection();
};


/*******************************************************************************
* WireTypeNode
*******************************************************************************/
class WireTypeNode : public EntityPropertyNode
{
    Q_OBJECT
public:
    Q_INVOKABLE explicit WireTypeNode(HnsClient::GenericEntity *entity, const QMetaProperty &property, QObject *parent = 0);
    ~WireTypeNode();
    virtual QVariant nodeDecoratedValue() const;
    virtual QMap<QString, QVariant> propertyEditorData() const;
};


/*******************************************************************************
* SwitchTypeNode
*******************************************************************************/
class SwitchTypeNode : public EntityPropertyNode
{
    Q_OBJECT
public:
    Q_INVOKABLE explicit SwitchTypeNode(HnsClient::GenericEntity *entity, const QMetaProperty &property, QObject *parent = 0);
    ~SwitchTypeNode();
    virtual QVariant nodeDecoratedValue() const;
    virtual QMap<QString, QVariant> propertyEditorData() const;
};


/*******************************************************************************
* PropertiesModel
*******************************************************************************/
class PropertiesModel : public TreeModel
{
    Q_OBJECT
public:
    explicit PropertiesModel(QWidget *parent = 0);
    ~PropertiesModel();
    void setEntity(HnsClient::GenericEntity *entity);
    virtual QVariant headerData(int section, Qt::Orientation orientation, int role) const;
    QModelIndexList persistentEditorRequestList() const;

private:
    QModelIndexList _persistentEditorRequestIndexes;
};


/*******************************************************************************
* PropertiesTreeWidget
*******************************************************************************/
class PropertiesTreeWidget : public SideBarTreeWidget
{
    Q_OBJECT

public:
    Q_INVOKABLE explicit PropertiesTreeWidget(QWidget *parent = 0);
    ~PropertiesTreeWidget();
    static QString title();
    static QString sideBarId();

protected:
    virtual void _indexClicked(const QModelIndex &index);
    virtual void _indexDoubleClicked(const QModelIndex &index);
    virtual void _contextMenuRequested(const QModelIndex &index, const QPoint &position);
    void _setEntity(HnsClient::GenericEntity *entity);

private slots:
    void _selectionChangedSlot(QList<HnsClient::GenericEntity*> selectedEntities);
    void _topologyChangedSlot();
    void _currentEntityDestroyed();

private:
    PropertiesModel *_propertiesModel;
    PropertyDelegate *_delegate;
    HnsClient::GenericEntity *_currentEntity;
};

} // namespace Hnv

#endif // HNV_PROPERTIESTREEWIDGET_H
