/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "simulationmodewidget.h"
#include "ressources.h"
#include "entitycatalogwidget.h"
#include "topologylistwidget.h"
#include "modellistwidget.h"
#include "propertiestreewidget.h"
#include "entitymonitorwidget.h"
#include "sidebar.h"
#include "topologywidget.h"
#include "zoompreview.h"
#include "topologycontentwidget.h"

// Qt headers
#include <QAction>

namespace Hnv {

/*******************************************************************************
* SimulationModeWidget
*******************************************************************************/
SimulationModeWidget * SimulationModeWidget::_instance = NULL;

//Static
SimulationModeWidget* SimulationModeWidget::create(QWidget *parent)
{
    if (!_instance)
        _instance = new SimulationModeWidget(parent);
    return _instance;
}

SimulationModeWidget* SimulationModeWidget::instance()
{
    return _instance;
}

SimulationModeWidget::SimulationModeWidget(QWidget *parent)
    : ModeWidget(parent)
{
    QString sideBarID = metaObject()->className();
    _topologyWidget = new TopologyWidget(this);
    _initDisplayArea(_topologyWidget);
    QList<SideBarItem *> itemList;
    itemList.append(new SideBarItem(EntityCatalogWidget::sideBarId(), EntityCatalogWidget::title(), EntityCatalogWidget::staticMetaObject, 1));
    itemList.append(new SideBarItem(TopologyListWidget::sideBarId(), TopologyListWidget::title(), TopologyListWidget::staticMetaObject));
    itemList.append(new SideBarItem(ModelListWidget::sideBarId(), ModelListWidget::title(), ModelListWidget::staticMetaObject));
    itemList.append(new SideBarItem(ZoomPreviewSideBarWidget::sideBarId(), ZoomPreviewSideBarWidget::title(), ZoomPreviewSideBarWidget::staticMetaObject));
    itemList.append(new SideBarItem(TopologyContentWidget::sideBarId(), TopologyContentWidget::title(), TopologyContentWidget::staticMetaObject));
    itemList.append(new SideBarItem(PropertiesTreeWidget::sideBarId(), PropertiesTreeWidget::title(), PropertiesTreeWidget::staticMetaObject));
    itemList.append(new SideBarItem(EntityMonitorWidget::sideBarId(), EntityMonitorWidget::title(), EntityMonitorWidget::staticMetaObject));
    _initSideBar(new SideBar(sideBarID, EntityCatalogWidget::sideBarId(), itemList, this));

    itemList.clear();
    sideBarID.append("_right");
    itemList.append(new SideBarItem(EntityCatalogWidget::sideBarId(), EntityCatalogWidget::title(), EntityCatalogWidget::staticMetaObject, 1));
    itemList.append(new SideBarItem(TopologyListWidget::sideBarId(), TopologyListWidget::title(), TopologyListWidget::staticMetaObject));
    itemList.append(new SideBarItem(ModelListWidget::sideBarId(), ModelListWidget::title(), ModelListWidget::staticMetaObject));
    itemList.append(new SideBarItem(ZoomPreviewSideBarWidget::sideBarId(), ZoomPreviewSideBarWidget::title(), ZoomPreviewSideBarWidget::staticMetaObject));
    itemList.append(new SideBarItem(TopologyContentWidget::sideBarId(), TopologyContentWidget::title(), TopologyContentWidget::staticMetaObject));
    itemList.append(new SideBarItem(PropertiesTreeWidget::sideBarId(), PropertiesTreeWidget::title(), PropertiesTreeWidget::staticMetaObject));
    itemList.append(new SideBarItem(EntityMonitorWidget::sideBarId(), EntityMonitorWidget::title(), EntityMonitorWidget::staticMetaObject));
    _initRightSideBar(new SideBar(sideBarID, TopologyContentWidget::sideBarId(), itemList, this));
}

SimulationModeWidget::~SimulationModeWidget()
{
    delete _topologyWidget;
}
/*******************************************************************************
* SimulationModeWidget - Protected
*/
void SimulationModeWidget::_initModeAction()
{
    _modeAction = new QAction(QIcon(Ressources::themedPath(Constants::ICON_SIMULATION)), tr("Simulation"), this);
}
/*******************************************************************************
* SimulationModeWidget - Private
*/
TopologyWidget * SimulationModeWidget::topologyWidget()
{
    return _topologyWidget;
}

bool SimulationModeWidget::hasSideBar()
{
    return true;
}

bool SimulationModeWidget::hasRightSideBar()
{
    return true;
}

} // namespace Hnv
