/******************************************************************************
**
** Copyright (C) 2003-2013 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "snapshotcreationdialog.h"
#include "ui_snapshotcreationdialog.h"

// hynesim hnsclient headers
#include <libhnsclient/entity.h>

// Qt headers
#include <QDateTime>
#include <QDebug>

namespace Hnv {

SnapshotCreationDialog::SnapshotCreationDialog(HnsClient::DomainEntity *entity, QWidget *parent, const char *slot) :
    QDialog(parent),
    ui(new Ui::SnapshotCreationDialog),
    _sourceEntity(entity),
    _returnSlot(slot)
{
    ui->setupUi(this);
    QString date = QDateTime::currentDateTime().toString("ddMMyyyy-hhmmss");
    ui->saveLineEdit->setText(date);
    ui->saveLineEdit->setButtonTooltip(tr("Snapshot name must be unique."));
}

SnapshotCreationDialog::~SnapshotCreationDialog()
{
    delete ui;
}

void SnapshotCreationDialog::on_buttonBox_accepted()
{
    if(ui->saveLineEdit->text().isEmpty())
        ui->saveLabel->setStyleSheet("color: red");
    else {
        foreach(Hns::Snapshot snap, _sourceEntity->snapshotList()) {
            if (QString::fromStdString(snap.name) == ui->saveLineEdit->text()) {
                ui->saveLineEdit->setButtonVisible(true);
                qCritical() << tr("Snapshot name must be unique.");
                return;
            }
        }
        Hns::Snapshot snapshot;
        snapshot.name = ui->saveLineEdit->text().toStdString();
        snapshot.description = ui->descTextEdit->toPlainText().toStdString();
        _sourceEntity->createSnapshot(snapshot, parent(), _returnSlot);
        accept();
        emit dialogRejected(false);
    }
}

void SnapshotCreationDialog::on_buttonBox_rejected()
{
    emit dialogRejected(true);
    reject();
}

} // namespace Hnv
