/******************************************************************************
**
** Copyright (C) 2003-2013 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef SNAPSHOTMANAGER_H
#define SNAPSHOTMANAGER_H

// hynesim hnsclient headers
#include <libhnsclient/entity.h>

// Qt headers
#include <QDialog>
#include <QModelIndex>

namespace Ui {
class SnapshotManager;
} // namespace Ui

namespace Hnv {

class SnapshotTreeModel;
class SnapshotTreeModelItem;

class SnapshotManager : public QDialog
{
    Q_OBJECT
    
public:
    explicit SnapshotManager(HnsClient::DomainEntity *entity, QWidget *parent = 0);
    ~SnapshotManager();

private slots:
    void on_actionCreate_snapshot_triggered();
    void on_actionRevert_to_snapshot_triggered();
    void on_actionDelete_snapshot_triggered();

    void _detailsIsVisible(bool visible);
    void _selectedSnapshotChangedSlot();
    void _currentSnapshotChangedSlot(::Hns::Snapshot snapshot);
    void _enableActions(bool enabled);
    void _resizeColumns();
    void _snapshotDeletedSlot(bool result, const QString &snapshotName);
    void _snapshotCreatedSlot(bool result, const QString &snapshotName);
    void _snapshotRevertedSlot(bool result, const QString &snapshotName);


private:
    void _populateModel();
    void _updateIndex(QModelIndex index);
    void _updateButtons(QModelIndex index);

    Ui::SnapshotManager *ui;
    SnapshotTreeModel *_model;
    HnsClient::DomainEntity *_entity;
    QHash<QString, SnapshotTreeModelItem *> _items;
};

} // namespace Hnv
#endif // SNAPSHOTMANAGER_H
