/******************************************************************************
**
** Copyright (C) 2003-2013 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "snapshottreemodel.h"
#include "snapshottreemodelitem.h"

namespace Hnv {

SnapshotTreeModel::SnapshotTreeModel(int columns, QObject *parent)
    : TreeModel(parent)
    , _currentSnapshot(NULL)
{
    _initRootItem(columns);
}

SnapshotTreeModel::~SnapshotTreeModel()
{
}

void SnapshotTreeModel::add(SnapshotTreeModelItem *parent, SnapshotTreeModelItem *child)
{
    if (parent)
        addItem(parent, child);
    else
        addItem(_rootItem, child);
}

QVariant SnapshotTreeModel::headerData(int section, Qt::Orientation orientation, int role) const
{
    if (role == Qt::DisplayRole && orientation == Qt::Horizontal) {
        switch (section) {
        case SnapshotTreeModelItem::COLUMN_NAME:
            return tr("Name");
        case SnapshotTreeModelItem::COLUMN_CURRENT_DATE:
            return tr("Creation Date");
        case SnapshotTreeModelItem::COLUMN_DESCRIPTION:
            return tr("Description");
        }
    }
    return TreeModel::headerData(section, orientation, role);
}

SnapshotTreeModelItem *SnapshotTreeModel::getCurrentSnapshot()
{
    return _currentSnapshot;
}

void SnapshotTreeModel::setCurrentSnapshot(SnapshotTreeModelItem *item)
{
    if (item)
        item->setCurrentSnapshot(true);
    if (_currentSnapshot)
        _currentSnapshot->setCurrentSnapshot(false);
    _currentSnapshot = item;
}

} // namespace Hnv
