/******************************************************************************
**
** Copyright (C) 2003-2013 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "snapshottreemodelitem.h"

// Qt headers
#include <QDateTime>
#include <QFont>

namespace Hnv {

SnapshotTreeModelItem::SnapshotTreeModelItem(Hns::Snapshot snapshot, bool isCurrentSnapshot, int columns, QObject *parent)
    : TreeModelItem(columns, parent)
    , _snapshot(snapshot)
    , _isCurrentSnapshot(isCurrentSnapshot)
{
}

SnapshotTreeModelItem::~SnapshotTreeModelItem()
{
}

QVariant SnapshotTreeModelItem::data(int column, int role) const
{
    if (role == Qt::DisplayRole) {
        switch(column) {
        case COLUMN_NAME:
            return QString::fromStdString(_snapshot.name);
        case COLUMN_CURRENT_DATE:
            return QDateTime::fromTime_t(QString::fromStdString(_snapshot.creationDateTime).toUInt()).toString("dd/MM/yyyy-hh:mm:ss");
        case COLUMN_DESCRIPTION:
            return QString::fromStdString(_snapshot.description);
        }
    } else if (role == Qt::FontRole && _isCurrentSnapshot) {
        QFont font;
        font.setBold(true);
        return QVariant(font);
    }
    return TreeModelItem::data(column, role);
}

Hns::Snapshot SnapshotTreeModelItem::getSnapshot()
{
    return _snapshot;
}

bool SnapshotTreeModelItem::isCurrentSnapshot()
{
    return _isCurrentSnapshot;
}

void SnapshotTreeModelItem::setCurrentSnapshot(bool isCurrentSnapshot)
{
    _isCurrentSnapshot = isCurrentSnapshot;
}

} // namespace Hnv
