/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "connectionstatuswidget.h"
#include "ui_connectionstatuswidget.h"
#include "ressources.h"

namespace Hnv {

/*******************************************************************************
* ConnectionStatusWidget
*******************************************************************************/
ConnectionStatusWidget::ConnectionStatusWidget(QWidget *parent)
    : QWidget(parent)
    , _ui(new Ui::ConnectionStatusWidget)
{
    _ui->setupUi(this);
    _onlinePixmap = QPixmap(Ressources::themedPath(Constants::ICON_STATUS_ONLINE));
    _offlinePixmap = QPixmap(Ressources::themedPath(Constants::ICON_STATUS_OFFLINE));
    _workingPixmap = QPixmap(Ressources::themedPath(Constants::ICON_STATUS_WORKING));
    _errorPixmap = QPixmap(Ressources::themedPath(Constants::ICON_STATUS_ERROR));
    _ui->messageLabel->defineMaxSizeByPattern(tr("An error occured"));
    setStatus(ConnectionStatusWidget::Offline);
    _ui->iconLabel->setPixmap(Ressources::themedPath(Constants::ICON_STATUS_OFFLINE));
}

ConnectionStatusWidget::~ConnectionStatusWidget()
{
    delete _ui;
}

void ConnectionStatusWidget::setStatus(ConnectionStatusWidget::ConnectionState state, QString message)
{
    QString displayMessage;
    switch(state) {
    case ConnectionStatusWidget::Online:
        _ui->iconLabel->setPixmap(_onlinePixmap);
        displayMessage = tr("Connected");
        break;
    case ConnectionStatusWidget::Offline:
        _ui->iconLabel->setPixmap(_offlinePixmap);
        displayMessage = tr("Disconnected");
        break;
    case ConnectionStatusWidget::Working:
        _ui->iconLabel->setPixmap(_workingPixmap);
        displayMessage = tr("Loading ...");
        break;
    case ConnectionStatusWidget::Error:
        _ui->iconLabel->setPixmap(_errorPixmap);
        displayMessage = tr("An error occured");
        break;
    default:
        // Should be never reached
        break;
    }
    if (message != "")
        displayMessage = message;
    _ui->messageLabel->setText(displayMessage);
}

} // namespace Hnv
