/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "zoomsliderwidget.h"
#include "ui_zoomsliderwidget.h"

// Qt headers
#include <QLineEdit>
#include <QRegExpValidator>

namespace Hnv {

/*******************************************************************************
* ZoomSliderWidget
*******************************************************************************/
ZoomSliderWidget::ZoomSliderWidget(QWidget *parent)
    : QWidget(parent)
    , ui(new Ui::ZoomSliderWidget)
    , _zoomMaxi(400)
    , _zoomMini(25)
{
    ui->setupUi(this);
    _initValue = 100;
    _factorValue = 2;
    ui->comboBox->setMinimumContentsLength(6);
    _listZooms.append(25);
    _listZooms.append(50);
    _listZooms.append(75);
    _listZooms.append(100);
    _listZooms.append(150);
    _listZooms.append(200);
    _listZooms.append(300);
    _listZooms.append(400);
    qSort(_listZooms.begin(),_listZooms.end());
    foreach (int i,_listZooms){
        ui->comboBox->addItem(_intToCbItem(i),QVariant(i));
    }

    _regExpCb.setPattern("([0-9]+)[\\s]?[%]?$");
    _validator = new QRegExpValidator(_regExpCb,this);
    ui->comboBox->setValidator(_validator);
    connect(ui->horizontalSlider, SIGNAL(valueChanged(int)), this,SLOT(_valueChanged(int)));
    connect(ui->comboBox->lineEdit(),SIGNAL(editingFinished()),this,SLOT(_editingFinished()));
    ui->horizontalSlider->setValue(_initValue);
    ui->comboBox->setCurrentIndex(ui->comboBox->findData(_initValue));
}

ZoomSliderWidget::~ZoomSliderWidget()
{
    delete _validator;
    delete ui;
}

/*******************************************************************************
* ZoomSliderWidget - Public slots
*/
void ZoomSliderWidget::zoomChanged(int value)
{
    bool enabled = false;
    if (value != -1) {
        enabled = true;
        ui->horizontalSlider->setValue(value);
    } else
        ui->horizontalSlider->setValue(100);
    ui->horizontalSlider->setEnabled(enabled);
    ui->comboBox->setEnabled(enabled);
}

/*******************************************************************************
* ZoomSliderWidget - Private slots
*/
void ZoomSliderWidget::_valueChanged(int value)
{
    ui->comboBox->lineEdit()->setText(_intToCbItem(value));
    emit valueChanged(value);
}

void ZoomSliderWidget::on_comboBox_currentIndexChanged(int index)
{
    emit valueChanged(ui->comboBox->itemData(index).toInt());
}

void ZoomSliderWidget::_editingFinished()
{
    _checkRange();
    int zoom = _getIntFromCb();

    if(zoom !=-1 && !_listZooms.contains(zoom)){
        if (ui->comboBox->findData(QVariant(zoom))==-1) {
            int index = 0;
            while(index < ui->comboBox->count()) {
                if (ui->comboBox->itemData(index).toInt() < zoom)
                    index++;
                else
                    break;
            }
            _listZooms.insert(index,zoom);
            ui->comboBox->insertItem(index,_intToCbItem(zoom),QVariant(zoom));
            ui->comboBox->setCurrentIndex(index);
        } else if (ui->comboBox->findData(QVariant(zoom))!=-1){
            ui->comboBox->setCurrentIndex(ui->comboBox->findData(QVariant(zoom)));
        }
    }

}
void ZoomSliderWidget::_checkRange()
{
    int zoom = _getIntFromCb();
    if (zoom!=-1){
        if (zoom<_zoomMini){
            ui->comboBox->lineEdit()->setText(_intToCbItem(_zoomMini));
        } else if (zoom > _zoomMaxi){
            ui->comboBox->lineEdit()->setText(_intToCbItem(_zoomMaxi));
        }
    }
}

int ZoomSliderWidget::_getIntFromCb()
{
    if(_regExpCb.exactMatch(ui->comboBox->lineEdit()->text())){
        return _regExpCb.cap(1).toInt();
    } else {
        return -1;
    }
}

QString ZoomSliderWidget::_intToCbItem(int i)
{
    return tr("%1 %").arg(QString::number(i));
}

} // namespace Hnv

