/******************************************************************************
**
** Copyright (C) 2011-2012 DIATEAM. All rights reserved.
**
** This file is part of HYNESIM.
**
** Hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** Hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef ENTITIESMODEL_H
#define ENTITIESMODEL_H

// Qt headers
#include <QAbstractItemModel>
#include <QStyledItemDelegate>

namespace HnsClient {

class TopologyEntry;

} // namespace HnsClient

namespace Hnv {

class TopologyAdministrator;
class TopologyCatalogModel;

//the structure is created instead of a QPair in case we want to add some information about the entity
struct EntitiesLocalInformation {
    QString name;
    QString uuid;
    bool state;
};

class EntitiesModel : public QAbstractTableModel
{
    Q_OBJECT
public:
    enum {
        COLUMN_NAME,
        COLUMN_CHECKBOX,
        COLUMN_SIZE
    };
    explicit EntitiesModel(TopologyAdministrator *administrator, QObject *parent = 0);
    ~EntitiesModel();
    int rowCount(const QModelIndex &parent = QModelIndex()) const;
    int columnCount(const QModelIndex &parent) const;
    QVariant headerData(int section, Qt::Orientation orientation, int role) const;
    QVariant data(const QModelIndex &index, int role) const;
    bool setData(const QModelIndex &index, const QVariant &value, int role);
    bool getStateByUuid(QString uuid);

public slots:
    void topologyChange(int id);

protected:
    void _populate(HnsClient::TopologyEntry *topologyEntry, bool update = false);
    void _refresh(HnsClient::TopologyEntry *topologyEntry);

private slots:
    void _topologyEntryUpdatedSlot(HnsClient::TopologyEntry *entry);

private:
    TopologyAdministrator *_administrator;
    QList<EntitiesLocalInformation> _valuesToShow;
    int _id;
};

/*******************************DELEGATE*******************************/

class CheckBoxDelegate : public QStyledItemDelegate
{
    Q_OBJECT
public:
    CheckBoxDelegate(QObject *parent = 0);
    ~CheckBoxDelegate();
    void paint(QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index) const;
    bool editorEvent(QEvent *event, QAbstractItemModel *model, const QStyleOptionViewItem &option, const QModelIndex &index);
};

} // namespace Hnv
#endif // ENTITIESMODEL_H
