/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "topologydetailswidget.h"
#include "ui_topologydetailswidget.h"
#include "ressources.h"

// hynesim hnsclient headers
#include <libhnsclient/usermanager.h>
#include <libhnsclient/session.h>
#include <libhnsclient/entry.h>
#include <libhnsclient/topology.h>

namespace Hnv {

/*******************************************************************************
* TopologyDetailsWidget
*******************************************************************************/
TopologyDetailsWidget::TopologyDetailsWidget(QWidget *parent)
    : QWidget(parent)
    , _ui(new Ui::TopologyDetailsWidget)
{
    _ui->setupUi(this);
//    _ui->uuidLabel->setText("oooooooo-oooo-oooo-oooo-oooooooooo");
//    _ui->uuidLabel->setMinimumSize(_ui->uuidLabel->sizeHint());
//    _ui->uuidLabel->setText("");
//    _ui->descriptionLabel->setFixedWidth(_ui->uuidLabel->width());

    _ui->lastModLabel->setText("00 MMM 0000 00:00:00");
    _ui->lastModLabel->setMinimumSize(_ui->lastModLabel->sizeHint());
    _ui->lastModLabel->setText("");
}

TopologyDetailsWidget::~TopologyDetailsWidget()
{
    delete _ui;
}

void TopologyDetailsWidget::setItem(HnsClient::TopologyEntry *topologyEntry, HnsClient::Topology *topology)
{
    HnsClient::UserManager *userManager = HnsClient::Session::instance()->userManager();
    QString shortName = topologyEntry->getShortName();
    QString longName = topologyEntry->getLongName();
    if (topology && topology->isModified())
        shortName.append(QString("*"));
    if (longName == "")
        longName = shortName;
    _ui->previewBox->setTitle(_ui->previewBox->fontMetrics().elidedText(tr("Preview: %1").arg(shortName), Qt::ElideRight, Constants::WIDTH_PREVIEW));
    _ui->nameLabel->setText(_ui->nameLabel->fontMetrics().elidedText(longName, Qt::ElideRight, Constants::WIDTH_PREVIEW));
    QString description = topologyEntry->getDescription();
    if (description == "")
        description = tr("<Unspecified>");
//    _ui->descriptionLabel->setText(description);
    _ui->creationDateLabel->setText(topologyEntry->getCreationDate());
    _ui->lastModLabel->setText(topologyEntry->getModificationDate());
//    _ui->uuidLabel->setText(sourceItem->getUUID());
    _ui->modelCheckBox->setChecked(topologyEntry->topologyEntry()->isModel);

    QPixmap preview = _getScreenshot(topologyEntry);
    _ui->textEdit->setWordWrapMode(QTextOption::WordWrap);
    if (preview.isNull())
        preview.load((Ressources::themedPath(Constants::DEFAULT_PREVIEW)),Constants::DEFAULT_PREVIEW);

    if(topologyEntry->topologyEntry()->isModel)
        preview = HnsClient::TopologyEntry::getScreenshotModel(preview);

    _ui->screenshot->setPixmap(preview);
    if (topology) {
        QStringList usersConnected = userManager->getUsernamesFromIdList(topology->connectedUsers());
        _ui->textEdit->setText(usersConnected.join(","));
    }
    else {
        _ui->textEdit->setText(tr("Topology isn't currently loaded"));
    }
}

void TopologyDetailsWidget::setDefaultsDetails()
{
    _ui->previewBox->setTitle(tr("Preview"));
    QPixmap preview;
    preview.load((Ressources::themedPath(Constants::DEFAULT_PREVIEW)),Constants::DEFAULT_PREVIEW);
    _ui->screenshot->setPixmap(preview);
    _ui->nameLabel->setText("");
    _ui->creationDateLabel->clear();
    _ui->lastModLabel->clear();
    _ui->textEdit->setText(tr("N/A"));
    _ui->modelCheckBox->setChecked(false);
}

QPixmap TopologyDetailsWidget::_getScreenshot(HnsClient::TopologyEntry *entry)
{
    return entry->getScreenshot();
}

} // namespace Hnv
