/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef TOPOLOGYDIALOG_H
#define TOPOLOGYDIALOG_H

// Qt headers
#include <QDialog>
#include <QListView>

class QSortFilterProxyModel;

namespace HnsClient {
class TopologyEntry;
} // namespace HnsClient

namespace Ui {
class TopologyDialog;
} // namespace Ui

namespace Hnv {
class TopologyExportWidget;
class TreeSortFilterProxyModel;
class TopologyCatalog;
class TopologyCatalogModel;
class TopologyItem;
class TopologyAdministrator;

/*******************************************************************************
* TopologyTableView
*******************************************************************************/
class TopologyListView : public QListView
{
    Q_OBJECT
public:
    explicit TopologyListView(QWidget *parent = 0);
    ~TopologyListView();

protected:
    virtual void mousePressEvent(QMouseEvent *event);
};

/*******************************************************************************
* TopologyDialog
*******************************************************************************/
class TopologyDialog : public QDialog
{
    Q_OBJECT
public:
    explicit TopologyDialog(TopologyAdministrator *administrator, TopologyCatalogModel *model, QWidget *parent = 0);
    ~TopologyDialog();
    QList<HnsClient::TopologyEntry *> selectedEntries();

private slots:
    void on_action_Delete_triggered();
    void on_action_Properties_triggered();
    void on_action_Unload_triggered();
    void on_action_Fix_triggered();
    void on_catalogFilterEdit_textChanged(QString currentText);
    void on_buttonBox_accepted();
    void on_listView_doubleClicked(QModelIndex index);

    void _listViewSelectionChangedSlot(const QItemSelection & selected, const QItemSelection & deselected);
    void _sourceDataChanged(QModelIndex topLeft, QModelIndex bottomRight);
    void on_comboBox_currentIndexChanged(const QString &arg1);
    void _updateACLFailedSlot(QString errorMessage);

    void on_actionSwitchExportDetails_triggered();

private:
    void _updateCurrentItem(TopologyItem * item = 0);
    void _updateCurrentItems(QList<TopologyItem *> items);
    TopologyItem * _getSelectedItem();
    QList<TopologyItem *> _getSelectedItems();
    TopologyItem * _getItemFromProxyIndex(QModelIndex proxyIndex);
    static TopologyItem * _getItemFromIndex(QModelIndex index);

    Ui::TopologyDialog *_ui;
    TopologyAdministrator *_administrator;
    TopologyCatalogModel *_sourceModel;
    TreeSortFilterProxyModel *_proxyModel;
    QSortFilterProxyModel *_proxyFilterModel;
    QList<HnsClient::TopologyEntry *> _selectedEntries;
    QList< QPair<QString, QString > > _filterModelList;
    TopologyExportWidget *_exportWidget;
};

} // namespace Hnv
#endif // TOPOLOGYDIALOG_H
