/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef HNV_TOPOLOGYLISTWIDGET_H
#define HNV_TOPOLOGYLISTWIDGET_H

// Project headers
#include "sidebar.h"

class QAction;

namespace Hnv {

class SimulationModeWidget;
class TopologyWidget;

/*******************************************************************************
* TopologyListWidget
*******************************************************************************/
class TopologyListWidget : public SideBarTreeWidget
{
    Q_OBJECT

public:
    Q_INVOKABLE explicit TopologyListWidget(QWidget *parent = 0);
    ~TopologyListWidget();
    static QString title();
    static QString sideBarId();

protected:
    virtual void _indexClicked(const QModelIndex &index);
    virtual void _indexDoubleClicked(const QModelIndex &index);
    virtual void _contextMenuRequested(const QModelIndex &index, const QPoint &position);

private slots:
    void _actionPropertiesTriggeredSlot();
    void _actionCloseTopologyTriggeredSlot();
    void _actionCloseAllTopologiesTriggeredSlot();
    void _actionCloseAllTopologiesExpectTriggeredSlot();
    void _topologyActivatedSlot(QVariant topologyUUID);

private:
    QString _topologyUUIDFromIndex(QModelIndex index);

    void _activateTopology(QString topologyUUID);
    void _activateTopology(QModelIndex index);

    void _displayTopologyProperties(QModelIndex index);
    void _displayTopologyProperties(QString topologyUUID);

    void _closeTopology(QModelIndex index);
    void _closeTopology(QString topologyUUID);

    void _closeAllTopologies(QString excludedTopologyUUID = QString());

    SimulationModeWidget *_simulationModeWidget;
    TopologyWidget *_topologyWidget;
    QAction *_actionProperties;
    QAction *_actionCloseTopology;
    QAction *_actionCloseAllTopologies;
    QAction *_actionCloseAllTopologiesExpect;
};

} // namespace Hnv

#endif // HNV_TOPOLOGYLISTWIDGET_H
