// ****************************************************************************
//
// Copyright (C) 2003-2014 DIATEAM. All rights reserved.
//
// This file is part of Hynesim.
//
// Hynesim is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License version 2
// as published by the Free Software Foundation.
//
// Hynesim is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// ****************************************************************************

// Project headers
#include "entitypluggingdialog.h"
#include "ui_entitypluggingdialog.h"
#include "entitypluggingwidget.h"

// hynesim hnsclient headers
#include <libhnsclient/entity.h>

// Qt headers
#include <QMessageBox>
#include <QScrollArea>

namespace Hnv {

EntityPluggingDialog::EntityPluggingDialog(HnsClient::Entity *entity, QMap<qlonglong, QString> colorMap, QPixmap screenshot, QWidget *parent)
    : QDialog(parent)
    , ui(new Ui::EntityPluggingDialog)
    , _entity(entity)
    , _trullyRejected(false)
{

    ui->setupUi(this);
    setWindowTitle(tr("Plug an entity"));

    QMap<int, Hns::Plug> portMap = entity->getAvailablePlugs();
    QMap<int, Hns::Plug>::const_iterator itor = portMap.begin();

    QMap<QString, QPair<QString, int> > sortMapPlugName; //modele ie eth0 -> eth000 eth10 -> eth010

    while(itor != portMap.end()){
        QString formatedPlugName = QString(itor.value().name.c_str());

        QRegExp regExp("([^0-9])([0-9])$");
        formatedPlugName.replace(regExp, "\\100\\2");

        regExp.setPattern("([^0-9])([0-9][0-9])$");
        formatedPlugName.replace(regExp, "\\10\\2");

        sortMapPlugName[formatedPlugName] = qMakePair(QString(itor.value().name.c_str()), (int)itor.key());
        itor++;
    }

    ui->screenshotLabel->setPixmap(screenshot);
    QMapIterator<qlonglong, QString> mapIterator(colorMap);
    QWidget *contentWidget = new QWidget();
    QVBoxLayout *layout = new QVBoxLayout();
    layout->setContentsMargins(0,0,0,0);
    while (mapIterator.hasNext()) {
        mapIterator.next();
        EntityPluggingWidget *pluggingWidget = new EntityPluggingWidget(sortMapPlugName, mapIterator.key(), mapIterator.value());
        layout->addWidget(pluggingWidget);
        _pluggingWidgetsHash.insert(mapIterator.key(), pluggingWidget);
    }
    contentWidget->setLayout(layout);
    QScrollArea *sc = new QScrollArea();
    sc->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Expanding);
    sc->setWidget(contentWidget);
    sc->setMinimumWidth(contentWidget->width() + 20);
    sc->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    ui->verticalLayout->insertWidget(ui->verticalLayout->count() - 1, sc);

    adjustSize();
    connect(ui->buttonBox, SIGNAL(accepted()), this, SLOT(_onAcceptedSlot()));
}

EntityPluggingDialog::~EntityPluggingDialog()
{
    delete ui;
}

QMap<qlonglong, Hns::Plug> EntityPluggingDialog::plugMap() const
{
    return _plugMap;
}

bool EntityPluggingDialog::isTrullyRejected()
{
    return _trullyRejected;
}

void EntityPluggingDialog::reject()
{
    QObject *senderObject = sender();
    if (senderObject && (senderObject == this || senderObject == ui->buttonBox))
        _trullyRejected = true;
    QDialog::reject();
}

void EntityPluggingDialog::_onAcceptedSlot()
{
    _trullyRejected = true;
    _plugMap.clear();
    QList<int> plugIdList;
    QHashIterator<qlonglong, EntityPluggingWidget *> hashIterator(_pluggingWidgetsHash);
    while (hashIterator.hasNext()) {
        hashIterator.next();
        EntityPluggingWidget *pluggingWidget = hashIterator.value();
        if (!pluggingWidget->isSelected())
            continue;
        qlonglong wireId = hashIterator.key();
        int plugId = pluggingWidget->selectedPlugId();
        if (plugIdList.contains(plugId)) {
            QMessageBox::warning(this, tr("Error"), tr("Something is wrong with your configuration, operation aborted"));
            reject();
            return;
        }
        else
            plugIdList.append(plugId);
        _plugMap.insert(wireId, _entity->getPlugs().value(plugId));
    }
    accept();
}

} // namespace Hnv
