// ****************************************************************************
//
// Copyright (C) 2003-2014 DIATEAM. All rights reserved.
//
// This file is part of Hynesim.
//
// Hynesim is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License version 2
// as published by the Free Software Foundation.
//
// Hynesim is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// ****************************************************************************

// Project headers
#include "entitypluggingwidget.h"
#include "ui_entitypluggingwidget.h"

// Qt headers
#include <QPainter>

namespace Hnv {

EntityPluggingWidget::EntityPluggingWidget(QMap<QString, QPair<QString, int> > map, qlonglong wireId, const QString &color, QWidget *parent)
    : QWidget(parent)
    , ui(new Ui::EntityPluggingWidget)
    , _wireId(wireId)
{
    ui->setupUi(this);
    QPixmap pixmap(20, 20); //todo: constants
    pixmap.fill(QColor(color));

    //2 px white border
    QPainter painter(&pixmap);
    painter.setBrush(Qt::white);
    painter.setPen(Qt::transparent);
    painter.drawRect(QRect(0,0,20,2));
    painter.drawRect(QRect(0,2,2,16));
    painter.drawRect(QRect(18,2,2,16));
    painter.drawRect(QRect(0,18,20,2));
    ui->colorLabel->setPixmap(pixmap);

    QMap<QString, QPair<QString, int> >::iterator itorSorted = map.begin();
    while(itorSorted != map.end()){
        ui->plugComboBox->addItem(itorSorted.value().first,itorSorted.value().second);
        itorSorted++;
    }
    ui->plugComboBox->setCurrentIndex(0);
}

EntityPluggingWidget::~EntityPluggingWidget()
{
    delete ui;
}

int EntityPluggingWidget::selectedPlugId() const
{
    return ui->plugComboBox->itemData(ui->plugComboBox->currentIndex()).toInt();
}

bool EntityPluggingWidget::isSelected() const
{
    return ui->checkBox->isChecked();
}

void EntityPluggingWidget::on_checkBox_clicked(bool checked)
{
    ui->wireLabel->setEnabled(checked);
    ui->plugComboBox->setEnabled(checked);
}

} // namespace Hnv
