/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef TOPOLOGYVIEW_H
#define TOPOLOGYVIEW_H

// Project headers
#include "zoomingview.h"

// Qt headers
#include <QColor>
#include <QMap>
#include <QPair>
#include <QPointF>

class QMimeData;

namespace Hnv {

class EntityGraphics;
class TopologyScene;

/*******************************************************************************
* TopologyView
*******************************************************************************/
class TopologyView : public ZoomingView
{
    Q_OBJECT
public:
    enum InsertMode {
        SelectMode,
        InsertWireMode,
        InsertWire1AngleMode,
        InsertWire2AnglesMode
    };

    explicit TopologyView(QWidget *parent = 0);
    explicit TopologyView(QGraphicsScene *scene, QWidget *parent = 0);

    void setSceneRect(qreal x, qreal y, qreal w, qreal h, bool centerView = false);

    static QStringList supportedMimeTypes();
    static bool canHandleMimeData(const QMimeData *data);
    static bool canHandleMimeData(const QMimeData *data, QString &datatype);

    TopologyScene *topologyScene();
    void emitRequestWireEntityItem(QPointF from, QPointF to);
    void matchZoomToItems();
    void matchZoomToRect(QRectF targetRect);

    InsertMode insertMode();
    void setInsertMode(InsertMode insertMode);
    void setIsWriteable(bool isWriteable);
    void setIsModel(bool isModel);
    void disableDroppableMode();
    bool isModel();
    QColor getWireColor();
    void setWireColor(QColor color);

signals:
    void requestEntityCreation(qlonglong entityID, QPoint position, EntityGraphics *overlappedEntity);
    void requestTopologyImport(qlonglong topologyId, QPoint position);
    void requestWireEntityItem(QPointF from, QPointF to);
    void contextMenuRequested(QPointF screnePos);
    void wirePluggingRequested(qlonglong entityId);

protected:
    virtual void dropEvent(QDropEvent *event);
    virtual void dragEnterEvent(QDragEnterEvent *event);
    virtual void dragMoveEvent(QDragMoveEvent *event);

    virtual void mousePressEvent(QMouseEvent* event);
    virtual void mouseMoveEvent(QMouseEvent* event);
    virtual void mouseReleaseEvent(QMouseEvent* event);
    virtual void wheelEvent(QWheelEvent *event);

    virtual void drawBackground(QPainter *painter, const QRectF &rect);

    virtual void _initView();
    QPoint _validateItemPosition(QPoint position, QRect itemRect);
    QHash<qlonglong, QPoint> _calculatePosition(QList<qlonglong> listItemsID, QPoint initialPosition);

    InsertMode _mode;
    bool _isWriteable;
    bool _isModel;
    QPair<qlonglong, EntityGraphics *> _currentEntityGraphics;
    QMap<qlonglong, QPair< EntityGraphics*, QPointF> > _itemSelected;
    QMap<qlonglong, QPair< EntityGraphics*, QPointF> > _selectedItemsWithCtrl;
    QPair<qlonglong, EntityGraphics*> _unselectedItemWithCtrl;
    QPointF _translationPosition;
    bool _mouseMoved;
    bool _mouseMovedWithCtrl;
    QColor _wireColor;

private slots:
    void _checkDeleted(qlonglong);
};

} // namespace Hnv
#endif // TOPOLOGYVIEW_H
