/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "wireplugselector.h"
#include "ui_wireplugselector.h"

// hynesim hnsclient headers
#include <libhnsclient/entity.h>

// Qt headers
#include <QRegExp>
#include <QPair>
#include <QDebug>

namespace Hnv {

/*******************************************************************************
* WirePlugSelector
*******************************************************************************/
WirePlugSelector::WirePlugSelector(HnsClient::Entity *from, HnsClient::Entity *to, int &fromPort, int &toPort, QWidget *parent)
    : QDialog(parent)
    , ui(new Ui::WirePlugSelector)
    , _fromEntity(from)
    , _toEntity(to)
    , _fromSelectedPort(fromPort)
    , _toSelectedPort(toPort)
{
    ui->setupUi(this);
    if (!from) {
        ui->fromComboBox->hide();
        ui->fromLabel->hide();
    } else {
        QMap<int, Hns::Plug> fromMap = from->getAvailablePlugs();
        if(!fromMap.size()) {
            qCritical() << tr("Source entity has no available plug. Aborting");
            ui->fromLabel->setDisabled(true);
        }
        QMap<int, Hns::Plug>::const_iterator itor = fromMap.begin();

        QMap<QString, QPair<QString, int> > sortMapPlugName; //modele ie eth0 -> eth000 eth10 -> eth010

        while(itor != fromMap.end()){
            QString formatedPlugName = QString(itor.value().name.c_str());

            QRegExp regExp("([^0-9])([0-9])$");
            formatedPlugName.replace(regExp, "\\100\\2");

            regExp.setPattern("([^0-9])([0-9][0-9])$");
            formatedPlugName.replace(regExp, "\\10\\2");

            sortMapPlugName[formatedPlugName] = qMakePair(QString(itor.value().name.c_str()), (int)itor.key());
            itor++;
        }

        QMap<QString, QPair<QString, int> >::iterator itorSorted = sortMapPlugName.begin();
        while(itorSorted != sortMapPlugName.end()){
            ui->fromComboBox->addItem(itorSorted.value().first,itorSorted.value().second);
            itorSorted++;
        }
        ui->fromComboBox->setCurrentIndex(ui->fromComboBox->findData(fromMap.begin().key()));
        ui->fromLabel->setText(from->getShortName());

    }
    if (!to) {
        ui->toComboBox->hide();
        ui->toLabel->hide();
    } else {
        QMap<int, Hns::Plug> toMap = to->getAvailablePlugs();
        if(!toMap.size()) {
            qCritical() << tr("Source entity has no available plug. Aborting");
            ui->fromLabel->setDisabled(true);
        }
        QMap<int, Hns::Plug>::const_iterator itor = toMap.begin();

        QMap<QString, QPair<QString, int> > sortMapPlugName; //modele ie eth0 -> eth000 eth10 -> eth010

        while(itor != toMap.end()){
            QString formatedPlugName = QString(itor.value().name.c_str());

            QRegExp regExp("([^0-9])([0-9])$");
            formatedPlugName.replace(regExp, "\\100\\2");

            regExp.setPattern("([^0-9])([0-9][0-9])$");
            formatedPlugName.replace(regExp, "\\10\\2");

            sortMapPlugName[formatedPlugName] = qMakePair(QString(itor.value().name.c_str()), (int)itor.key());
            itor++;
        }

        QMap<QString, QPair<QString, int> >::iterator itorSorted = sortMapPlugName.begin();
        while(itorSorted != sortMapPlugName.end()){
            ui->toComboBox->addItem(itorSorted.value().first,itorSorted.value().second);
            itorSorted++;
        }
        ui->toComboBox->setCurrentIndex(ui->toComboBox->findData(toMap.begin().key()));
        ui->toLabel->setText(to->getShortName());
    }

}

WirePlugSelector::~WirePlugSelector()
{
    delete ui;
}

/*******************************************************************************
* WirePlugSelector - Private slots
*/
void WirePlugSelector::on_buttonBox_accepted()
{
    _fromSelectedPort = ui->fromComboBox->itemData(ui->fromComboBox->currentIndex()).toInt();
    _toSelectedPort = ui->toComboBox->itemData(ui->toComboBox->currentIndex()).toInt();
}

void WirePlugSelector::on_buttonBox_rejected()
{
    reject();
}

} // namespace Hnv
