/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "wireplugtoentitydialog.h"
#include "ui_wireplugtoentitydialog.h"

// hynesim hnsclient headers
#include <libhnsclient/entity.h>

namespace Hnv {

WirePlugToEntityDialog::WirePlugToEntityDialog(HnsClient::GenericEntity *entity, QWidget *parent)
    : QDialog(parent)
    , ui(new Ui::WirePlugToEntityDialog)
    , _entity(entity)
    , _trullyRejected(false)
{
    ui->setupUi(this);
    connect(ui->buttonBox, SIGNAL(accepted()), this, SLOT(_onAcceptedSlot()));
    connect(ui->buttonBox, SIGNAL(rejected()), this, SLOT(_onRejectedSlot()));
    setWindowTitle(tr("Wire plugging"));

    ui->entityNameLabel->setText(entity->getShortName());

    QMap<int, Hns::Plug> portMap = entity->getAvailablePlugs();
    QMap<int, Hns::Plug>::const_iterator itor = portMap.begin();

    QMap<QString, QPair<QString, int> > sortMapPlugName; //modele ie eth0 -> eth000 eth10 -> eth010

    while(itor != portMap.end()){
        QString formatedPlugName = QString(itor.value().name.c_str());

        QRegExp regExp("([^0-9])([0-9])$");
        formatedPlugName.replace(regExp, "\\100\\2");

        regExp.setPattern("([^0-9])([0-9][0-9])$");
        formatedPlugName.replace(regExp, "\\10\\2");

        sortMapPlugName[formatedPlugName] = qMakePair(QString(itor.value().name.c_str()), (int)itor.key());
        itor++;
    }

    QMap<QString, QPair<QString, int> >::iterator itorSorted = sortMapPlugName.begin();
    while(itorSorted != sortMapPlugName.end()){
        ui->entityPlugComboBox->addItem(itorSorted.value().first,itorSorted.value().second);
        itorSorted++;
    }
}

WirePlugToEntityDialog::~WirePlugToEntityDialog()
{
    delete ui;
}

int WirePlugToEntityDialog::entityPort()
{
    return _entityPort;
}

bool WirePlugToEntityDialog::isTrullyRejected()
{
    return _trullyRejected;
}

void WirePlugToEntityDialog::reject()
{
    QObject *senderObject = sender();
    if (senderObject && (senderObject == this || senderObject == ui->buttonBox))
        _trullyRejected = true;
    QDialog::reject();
}

void WirePlugToEntityDialog::_onAcceptedSlot()
{
    _entityPort = ui->entityPlugComboBox->itemData(ui->entityPlugComboBox->currentIndex()).toInt();
    accept();
}

void WirePlugToEntityDialog::_onRejectedSlot()
{
    _trullyRejected = true;
    reject();
}

} // namespace Hnv
