#ifndef TOPOLOGYCONTAINER_H
#define TOPOLOGYCONTAINER_H

// Qt headers
#include <QHash>
#include <QWidget>
#include <QDialog>
#include <QTabBar>

class QMenu;
class QStackedWidget;
class QVBoxLayout;

namespace Ui {
class TopologyContainer;
} // namespace Ui

namespace RemoteAccess {
class TopologyScreenshotDeclarativeView;
class ScreenshotModel;
class RemoteView;
} // namespace RemoteAccess

namespace Hnv {
class TopologyViewArea;
class RemoteControlSeparateWindow;
class SeparateWindowTabBar;

/*******************************************************************************
* TopologyContainer
*******************************************************************************/
class TopologyContainer : public QWidget
{
    Q_OBJECT
    
public:
    explicit TopologyContainer(TopologyViewArea *topologyViewArea, QWidget *parent = 0);
    ~TopologyContainer();
    TopologyViewArea *topologyViewArea();
signals:
    void remoteViewClosed(int id);
    void remoteDisplayOpened();
protected:
    bool eventFilter(QObject *object, QEvent *event);
    void focusInEvent(QFocusEvent *event);
    void focusOutEvent(QFocusEvent *event);
private slots:
    void _connectionOpenedSlot(int id);
    void _connectionEndedSlot(int id);
    void _tabClosedRequestedSlot(int index);
    void _separateWindowQuit(int id);
    void _detachTab(int index);
    void _retachTab(int index);
    void _focusOnSeparateWindowRequestedSlot(int index);
    void _switchTabSlot(int id);
    void _newViewRequestedSlot(int index, int idSender, int idReceiver);
    void _detachTabWindowSlot(int id);
    void _retachTabWindowSlot(int id);
    void _currentIndexChangedSlot(int index);
    void _entityDestroyedSlot(qlonglong entityId);
    void _entityStatusUndefinedSlot(qlonglong entityId);
    void _errorMessageSlot(int id, QString message);
    void _shortnameChangedSlot(int id,const QString &shortname);
private:
    Ui::TopologyContainer *ui;
    TopologyViewArea *_topologyViewArea;
    RemoteAccess::TopologyScreenshotDeclarativeView *_screenshotView;
    RemoteAccess::ScreenshotModel *_screenshotModel;
    QHash<int, int> _idHash; //id, index
    QHash<int, bool> _isTabHash;
    QHash<int, RemoteControlSeparateWindow*> _separateWindowHash; //by id
    int _windowCount;
    QHash<int, RemoteControlSeparateWindow*> _windowHash;
    QHash<int, RemoteAccess::RemoteView *>_viewHash;
    QList<qlonglong> _temporaryViewsList;
};

/*******************************************************************************
* TopologyContainerTabBar
*******************************************************************************/
class TopologyContainerTabBar : public QTabBar
{
    Q_OBJECT
public:
    TopologyContainerTabBar(bool isMain = true, QWidget *parent = 0);
    ~TopologyContainerTabBar();
    void addView(int index, bool isTab);
    void removeView(int index);
    void setId(int id);
signals:
    void detachTabRequested(int index);
    void retachTabRequested(int index);
    void focusOnSeparatedWindowRequested(int id);
protected:
    void mousePressEvent(QMouseEvent *event);
    void mouseMoveEvent(QMouseEvent *event);
    void wheelEvent(QWheelEvent *event);
private slots:
    void _indexChangedSlot(int index);
private:
    int _selectedIndex(const QPoint &pos);

    QMenu *_menu;
    QAction *_actionDetachTab;
    QAction *_actionRetachTab;
    QHash<int, bool> _isTabHash; //by index
    int _indexSelected;
    QPoint _dragStartPosition;
    bool _isMain;
    int _id;
    int _wheelIndex;
    QColor _selectedColor;

private slots:
    void _actionDetachTabTriggeredSlot();
    void _actionRetachTabTriggeredSlot();
};

/*******************************************************************************
* RemoteControlSeparateWindow
*******************************************************************************/
class RemoteControlSeparateWindow : public QDialog
{
    Q_OBJECT
public:
    explicit RemoteControlSeparateWindow(int id, QWidget *widgetToDisplay, int idView, QWidget *parent = 0);
    ~RemoteControlSeparateWindow();
    QWidget *widget(int id);
    QHash<int, QWidget*> widgets();
    void backToTab();
    void addView(QWidget *widget, int id);
    int numberOfView();
    void closeView(int id);
    int idFromIndex(int index);
    void raiseById(int id);
    void updateShortname(int id, const QString &shortname);
signals:
    void quit(int id);
    void newViewRequested(int index, int idSender, int idReceiver);
    void detachTabRequested(int id);
    void retachTabRequested(int id);
protected:
    void dragEnterEvent(QDragEnterEvent *event);
    void dropEvent(QDropEvent *event);
private slots:
    void _dialogDestroy();
    void _tabCloseRequestedSlot(int index);
    void _detachTabRequestedSlot(int index);
    void _retachTabRequestedSlot(int index);
private:
    QVBoxLayout *_layout;
    QStackedWidget *_stackedWidget;
    int _id;
    QHash<int, QWidget*> _viewHash; //by id
    QHash<int, int> _idHash; //id, index
    TopologyContainerTabBar *_tabBar;
};

} // namespace Hnv

#endif // TOPOLOGYCONTAINER_H
