/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "topologysavealldialog.h"
#include "ui_topologysavealldialog.h"
#include "topologyviewarea.h"

// hynesim hnsclient headers
#include <libhnsclient/entry.h>

// Qt headers
#include <QListWidgetItem>

namespace Hnv {

TopologySaveAllDialog::TopologySaveAllDialog(QHash<QString, TopologyViewArea *> hashArea, QWidget *parent)
    : QDialog(parent)
    , ui(new Ui::TopologySaveAllDialog)
    , _hashArea(hashArea)
{
    ui->setupUi(this);
    setWindowTitle(tr("Save Topology"));
    ui->warningLabel->setWordWrap(true);
    ui->warningLabel->setText(tr("One or more topologies are modified, select the ones you want to save"));
    foreach (TopologyViewArea *area,_hashArea) {
        QListWidgetItem *item = new QListWidgetItem(area->topologyEntry()->getShortName(), ui->lwModifiedTopo);
        item->setData(Qt::UserRole,area->topologyEntry()->getUUID());
        Qt::ItemFlags flags = item->flags();
        if (!area->topologyEntry()->isWriteable()) {
            item->setFlags(flags & (!Qt::ItemIsEnabled) & (!Qt::ItemIsUserCheckable));
            item->setText(item->text() + tr(" (ReadOnly)"));
            item->setCheckState(Qt::Unchecked);
        } else {
            item->setCheckState(Qt::Checked);
            item->setFlags(flags | Qt::ItemIsUserCheckable | Qt::ItemIsEnabled);
        }
        ui->lwModifiedTopo->addItem(item);
    }
}

TopologySaveAllDialog::~TopologySaveAllDialog()
{
    delete ui;
}

QList<QString> TopologySaveAllDialog::getTopologyUUIDToSave()
{
    return _topologyUUIDToSave;
}

void TopologySaveAllDialog::on_buttonBox_accepted()
{
    int i = 0;
    while (i<ui->lwModifiedTopo->count()){
        if (ui->lwModifiedTopo->item(i)->checkState()){
            _topologyUUIDToSave.append(ui->lwModifiedTopo->item(i)->data(Qt::UserRole).toString());
        }
        i++;
    }
    accept();
}

void TopologySaveAllDialog::on_pbSelectAll_clicked()
{
    int i = 0;
    while (i<ui->lwModifiedTopo->count()){
        if (ui->lwModifiedTopo->item(i)->flags() && Qt::ItemIsEnabled){
            ui->lwModifiedTopo->item(i)->setCheckState(Qt::Checked);
        }
        i++;
    }
}

void TopologySaveAllDialog::on_pbDeselectAll_clicked()
{
    int i = 0;
    while (i<ui->lwModifiedTopo->count()){
        ui->lwModifiedTopo->item(i)->setCheckState(Qt::Unchecked);
        i++;
    }
}

} // namespace Hnv
