/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "topologysavedialog.h"
#include "ui_topologysavedialog.h"
#include "hyneviewsession.h"

// hynesim hnsclient headers
#include <libhnsclient/entry.h>

//#include "common/proxies_topologycatalog.h"

namespace Hnv {

/*******************************************************************************
* TopologySaveDialog
*******************************************************************************/
TopologySaveDialog::TopologySaveDialog(HnsClient::TopologyEntry *topologyEntry, QWidget *parent)
    : QDialog(parent)
    , ui(new Ui::TopologySaveDialog)
    , _topologyEntry(topologyEntry)
{
    ui->setupUi(this);
    ui->saveAsLineEdit->setText(_topologyEntry->getShortName());
    _displayPath(_topologyEntry->getPath());
}

TopologySaveDialog::~TopologySaveDialog()
{
    delete ui;
}
/*******************************************************************************
* TopologySaveDialog - Protected
*/
void TopologySaveDialog::_displayPath(QString path)
{
    HyneviewSession * session = HyneviewSession::instance();
    if (session) {
        ui->pathComboBox->clear();
        ui->pathComboBox->show();
        QMapIterator<QString, QString> iterator(session->topologyCatalogStorageMap());
        int index = 0;
        while (iterator.hasNext()) {
            iterator.next();
            QString storageName = iterator.value();
            ui->pathComboBox->addItem(storageName, QVariant(storageName + ":/"));
            if (iterator.key() == path) {
                ui->pathComboBox->setCurrentIndex(index);
                ui->pathComboBox->setDisabled(true);
            }
            index++;
        }
        if(ui->pathComboBox->count() == 1)
            ui->pathComboBox->setDisabled(true);
    }
}

HnsClient::ValidationResult TopologySaveDialog::_validateShortName()
{
    HnsClient::ValidationResult result = _topologyEntry->validateShortName(ui->saveAsLineEdit->text());
    if (result == HnsClient::Invalid)
        ui->saveAsLabel->setStyleSheet("QLabel{color:red;}");
    else
        ui->saveAsLabel->setStyleSheet("");
    return result;
}

HnsClient::ValidationResult TopologySaveDialog::_validatePath()
{
    QString path = ui->pathComboBox->itemData(ui->pathComboBox->currentIndex()).toString();
    return _topologyEntry->validatePath(path);
}

void TopologySaveDialog::_computeValidationState(HnsClient::ValidationResult result, bool &isModified, bool &isInvalid)
{
    if (result != HnsClient::Unmodified) {
        isModified = true;
        if (result == HnsClient::Invalid)
            isInvalid = true;
    }
}
/*******************************************************************************
* TopologySaveDialog - Private slots
*/
void TopologySaveDialog::on_buttonBox_accepted()
{
    bool isModified = false;
    bool isInvalid = false;
    _computeValidationState(_validateShortName(), isModified, isInvalid);
    _computeValidationState(_validatePath(), isModified, isInvalid);
    if (!isInvalid)
        accept();
}

} // namespace Hnv
