#ifndef TOPOLOGYSCREENSHOTDECLARATIVEVIEW_H
#define TOPOLOGYSCREENSHOTDECLARATIVEVIEW_H

// Generated hynesim headers
#include "common/entity.h"

// Qt headers
#include <QAbstractTableModel>
#include <QHash>
#include <QImage>
#include <QDeclarativeView>
#include <QDeclarativeImageProvider>

namespace RemoteAccess {

enum {
    COLUMN_NAME,
    COLUMN_TYPE,
    COLUMN_ADDR,
    COLUMN_PORT,
    COLUMN_IMAGE,
    COLUMN_SIZE
};

/*******************************************************************************
* ScreenshotModelProvider
*******************************************************************************/
class ScreenshotModelProvider : public QDeclarativeImageProvider
{
public:
    ScreenshotModelProvider();
    ~ScreenshotModelProvider();
    QImage requestImage(const QString &id, QSize *size, const QSize &requestedSize);
    void setImage(QImage img);
private:
    QImage _img;
};

/*******************************************************************************
* ScreenshotModel
*******************************************************************************/
class ScreenshotModel : public QAbstractTableModel
{
    Q_OBJECT
public:
    ScreenshotModel(QString topologyUuid, QObject *parent = 0);
    ~ScreenshotModel();

    void clear();
    void addView(int id, Hns::RemoteDisplayInfo info);

    int rowCount(const QModelIndex &parent) const;
    int columnCount(const QModelIndex &parent) const;
    QVariant data(const QModelIndex &index, int role) const;
    bool setData(const QModelIndex &index, const QVariant &value, int role);
    ScreenshotModelProvider *screenshotModelProvider();

public slots:
    void screenshotChange(int id, QImage image);
    void removeView(int id);

private:
    QList<int> _idList; //list of id
    QHash<int, Hns::RemoteDisplayInfo> _dataHash; //id, info
    QHash<int, QImage> _screenshotHash; //id, screenshot
    ScreenshotModelProvider *_provider;
    QString _topologyUuid;
};

/*******************************************************************************
* MainAnchor
*******************************************************************************/
class MainAnchor : public QObject //TODO: rename this class
{
    Q_OBJECT
    Q_PROPERTY(QColor color READ color WRITE setColor NOTIFY colorChanged)
    Q_PROPERTY(int width READ width WRITE setWidth NOTIFY widthChanged)
    Q_PROPERTY(int height READ height WRITE setHeight NOTIFY heightChanged)

public:
    MainAnchor(QObject *parent = 0);
    QColor color();
    void setColor(QColor color);
    int width();
    void setWidth(int width);
    int height();
    void setHeight(int height);

signals:
    void colorChanged();
    void widthChanged();
    void heightChanged();

private:
    QColor _color;
    int _width;
    int _height;
};

/*******************************************************************************
* TopologyScreenshotDeclarativeView
*******************************************************************************/
class TopologyScreenshotDeclarativeView : public QDeclarativeView
{
    Q_OBJECT
public:
    explicit TopologyScreenshotDeclarativeView(ScreenshotModel * model, QWidget *parent = 0);
signals:
    void screenshotClicked(int id);

protected:
    void resizeEvent(QResizeEvent *event);
private:
    MainAnchor *_anchor;
    ScreenshotModel *_model;
};

} // namespace RemoteAccess
#endif // TOPOLOGYSCREENSHOTDECLARATIVEVIEW_H
