/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef TOPOLOGYSIZEDIALOG_H
#define TOPOLOGYSIZEDIALOG_H

// Project headers
#include "ressources.h"

// Qt headers
#include <QDialog>
#include <QHash>

namespace Ui {
class TopologySizeDialog;
} // namespace Ui

namespace Hnv {

/*******************************************************************************
* TopologySizeDialog
*******************************************************************************/
class TopologySizeDialog : public QDialog
{
    Q_OBJECT
public:
    struct formatTemplate {
        QString id;
        QString description;
        int     width;  //Pixels
        int     height; //Pixels
    };
    explicit TopologySizeDialog(bool isNew, QWidget *parent = 0);
    ~TopologySizeDialog();
    int getWidth();
    int getHeight();
    bool isModel();
    void setWidth(int value);
    void setHeight(int value);
    void setSize(int w,int h);
    QColor getColor();
    void setColor(QColor value);
    void setMinimalSize(int minW, int minH);
    void setIsModel(bool isModel);

private slots:
    void on_cbFormat_currentIndexChanged(int index);
    void on_cbUnit_currentIndexChanged(int index);
    void on_tbLandscape_clicked(bool checked);
    void on_tbPortrait_clicked(bool checked);
    void on_sbWidth_editingFinished();
    void on_sbHeight_editingFinished();
    void on_sbWidth_valueChanged(double arg1);
    void on_sbHeight_valueChanged(double arg1);
    void on_tbColor_clicked();
    void on_typeTopology_toggled(bool checked);
    void on_buttonBox_accepted();

private:
    double _pixelToInch(int ,double resolution = Constants::DEFAULT_IMPRESSION_RESOLUTION);
    double _pixelToMillimeter(int ,double resolution = Constants::DEFAULT_IMPRESSION_RESOLUTION);
    double _millimeterToInch(double mmValue);
    int _millimeterToPixel(double ,double resolution = Constants::DEFAULT_IMPRESSION_RESOLUTION);
    double _inchToMillimeter(double );
    int _inchToPixel(double ,double resolution = Constants::DEFAULT_IMPRESSION_RESOLUTION);
    void _update();
    void _checkButtons();
    QString _getUnit();
    int _pixelValue(double value,QString unit);
    void _swapSbValue();
    void _updateLabelValue(double value,QString unit,QString type);
    double _unitValue(int value,QString unit);
    void _isExistingFormat();
    void _handleButtons();
    void _changeDecimals();
    void _setColorPreview(const QString& name);

    Ui::TopologySizeDialog *ui;
    int _miniPixels;
    int _maxiPixels;
    int _defaultSizePixels;
    QString _defaultSizeUnit;
    QColor _defaultColor;
    int _width;   //Taille en Pixels
    int _height;  //Taille en Pixels
    QColor _topologyColor;
    QColor _modelColor;
    QString _iniFormat;
    QString _iniSizeUnit;
    QHash<QString,formatTemplate> _hashModels;
    QHash<QString,QString> _hashUnits;
    int _mininalWidth;
    int _minimalHeight;
};

} // namespace Hnv
#endif // TOPOLOGYSIZEDIALOG_H
