/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef TOPOLOGYWIDGET_H
#define TOPOLOGYWIDGET_H

// Generated hynesim headers
#include "common/layout.h"

// Qt headers
#include <QAbstractItemModel>
#include <QWidget>

class QToolButton;
class QSignalMapper;
class QMenu;

namespace Ui {
class TopologyWidget;
} // namespace Ui

namespace HnsClient {
class GenericEntity;
class Entity;
class TopologyEntry;
class Topology;
class TopologyManager;
class TopologyEntryCatalog;
} // namespace HnsClient

namespace Hnv {
class HyneviewSession;
class EntityGraphics;
class EntityToolBar;
class TopologyViewArea;
class TopologyContentModel;
class TopologyToolBar;
class LoadedTopologyModel;
class GraphicsItem;
class WireGraphics;
class TopologyView;
class TopologyItem;
class TopologyContainer;

/*******************************************************************************
* LoadedTopologyModel
*******************************************************************************/
class LoadedTopologyModel : public QAbstractItemModel
{
    Q_OBJECT
public:
    explicit LoadedTopologyModel(QObject * parent = 0);
    virtual QModelIndex index(int row, int column, const QModelIndex &parent = QModelIndex()) const;
    virtual QModelIndex parent(const QModelIndex &child) const;
    int rowCount(const QModelIndex &parent) const;
    int columnCount(const QModelIndex &parent) const;
    QVariant data(const QModelIndex &index, int role = Qt::DisplayRole) const;
    void clear();

public slots:
    void topologyOpenedSlot(HnsClient::Topology *topology);
    void topologyChangedSlot(HnsClient::Topology *topology);
    void topologyClosedSlot(QString topologyUUID);

private:
    QList<HnsClient::Topology *> _topologyList;
};


/*******************************************************************************
* TopologyWidget
*******************************************************************************/
class TopologyWidget : public QWidget
{
    Q_OBJECT
public:
    explicit TopologyWidget(QWidget *parent = 0);
    ~TopologyWidget();
    LoadedTopologyModel *loadedTopologyModel();
    TopologyContentModel *topologyContentModel();

    void setCurrentTopology(QString topologyUUID);                              // Used by TopologyListWidget
    void displayTopologyProperties(QString topologyUUID = QString());           // Used by TopologyListWidget
    void displayTopologyProperties(TopologyItem *item);                         // Used by ModelListWidget

    void createTopology();
    void displayTopologyManager();

    TopologyView *getCurrentView();
    QAction *getActionNew();
    QAction *getActionOpen();

    void setEntitySelection(QList<qlonglong> entityIDList);
    void centerOnEntity(qlonglong entityID);
    void deleteItemsByIDs(QList<qlonglong> itemList);
    QList<HnsClient::GenericEntity *> getCurrentEntitiesSelected(bool includeWires = false);
    void refreshAllEntities();
    void displayEntityPropertiesById(qlonglong entityId);

public slots:
    void setDetailOverlayCorner(Qt::Corner corner); // TODO: unused ?
    void hideDetailOverlay(bool checked); // TODO : unused ?
    bool closeTopologySlot(QString topologyUUID = QString(), bool forceClose = false);               // Used by TopologyListWidget
    bool closeAllTopologiesSlot(QString excludedTopologyUUID = QString());      // Used by TopologyListWidget

signals:
    void cursorPositionChanged(int x, int y, bool inScene);
    void zoomValueChanged(int value);
    void selectionChanged(QList<HnsClient::GenericEntity *> selectedEntities);
    void topologyOpened(HnsClient::Topology *topology);
    void topologyChanged(HnsClient::Topology *topology);
    void topologyActivated(QVariant topologyUUID);
    void topologyClosed(QString topologyUUID);
    void clearLabelCoord();
    void topologySaveFailed();

private slots:
    void on_action_Create_topology_triggered();
    void on_action_Open_topology_triggered();
    void on_action_Close_topology_triggered();
    void on_action_Close_all_topologies_triggered();
    void on_action_Save_topology_triggered();
    void on_action_Save_topology_as_triggered();
    void on_action_Topology_properties_triggered();
    void on_action_Display_grid_toggled(bool checked);
    void on_action_Delete_selected_items_triggered();
    void on_action_Properties_triggered();
    void on_action_Pause_entity_triggered();
    void on_action_Save_entity_triggered();
    void on_action_Start_entity_triggered();
    void on_action_Stop_entity_triggered();
    void on_action_Undefine_entity_triggered();
    void on_action_Remote_display_triggered();
    void on_action_Usb_redirection_triggered();
    void on_action_Resize_Scene_triggered();
    void on_action_Print_Topology_triggered();
    void on_action_Print_Selection_triggered();
    void on_action_Print_View_triggered();
    void on_action_Snapshot_Topology_triggered();
    void on_action_Snapshot_View_triggered();
    void on_action_Snapshot_Selection_triggered();
    void on_action_Toggle_Entity_Label_triggered(bool checked);
    void on_action_Display_Capture_Job_triggered();
    void on_action_Select_all_triggered();
    void on_action_Load_Balancer_triggered();
    void on_action_Unimplement_entity_triggered();
    void on_action_Build_Topology_From_triggered();
    void on_action_Build_Model_From_triggered();
    void on_action_Force_to_Default_triggered();
    void on_action_Force_to_Hide_triggered();
    void on_action_Force_to_Show_triggered();
    void on_action_Custom_Graphic_triggered();

    void _contextMenuRequestedSlot(QPointF screenPos);
    void _insertModeSelectedSlot(QAction *sourceAction);
    void _currentTopologyChangedSlot(int index);
    void _setCurrentTopologySlot(QVariant uuidData);
    void _setSelectedEntitiesSlot(qlonglong entityId);
    void _zoomValueChangedSlot(int value);

    void _connectedSlot(HyneviewSession *session);
    void _disconnectedSlot();
    void _topologyAreaLoadedSlot(qlonglong topologyID);
    void _topologyUnloadedSlot(QString uuid, bool isDeleted);
    void _entityCreatedSlot(HnsClient::GenericEntity *entity);
    void _entityChangedSlot(HnsClient::GenericEntity *entity);
    void _entityRemovedSlot(qlonglong entityID);
    void _loadTopologySlot(HnsClient::Topology *topology);
    void _loadTopologySwitchedSlot(HnsClient::Topology *topology);
    void _createTopologySlot(HnsClient::Topology *topology);

    void _topologySavedSlot(QString topologyUUID, bool result, QString exceptionMessage);
    void _topologySavedAndClosedSlot(QString topologyUUID, bool result, QString exceptionMessage);
    void _topologySavedAsSlot(QString topologyUUID, QString newUUID, bool result, QString exceptionMessage);

    void _switchTopologyTypeSlot(qlonglong entryId);
    void _drawScreenShotSlot(QString uuid);

    void _topologySavedAsByTopologyManagerSlot(QString, QString);
    void _entitySelectionChangedSlot();
    void _defineMappingTriggered(const QString& nodeName);
    void _defineBehaviorMappingTriggered(int idx);

    void on_action_Create_snapshot_triggered();
    void on_actionSnapshot_manager_triggered();

    void on_action_Current_wire_color_triggered();
    void on_action_Change_wire_color_triggered();
    void on_action_Toggle_Status_Icons_toggled(bool checked);

    void on_action_display_memory_dump_triggered();

    void _canRedirectUsbChangedSlot(int id);

private:
    void _handleTopology(HnsClient::Topology *topology, QString errorTitle, QString errorMessage);
    void _displayTopologyProperties(TopologyViewArea *targetArea);
    void _displayTopologyProperties(HnsClient::TopologyEntry *entry, QObject *source);
    bool _closeTopology(TopologyViewArea *targetArea, bool forceToClose = false);
    void _destroyAllTopologies();
    void _destroyTopology(QString topologyUUID, bool requestClose = false);
    void _destroyTopology(TopologyViewArea *targetArea, bool requestClose = false);
    void _saveTopology(TopologyViewArea *targetArea);
    bool _saveAndCloseTopology(TopologyViewArea *targetArea);
    bool _saveTopologyAs(TopologyViewArea *targetArea);
    bool _saveTopology(HnsClient::Topology *topology, QObject *receiver, const char *slot);
    void _switchTopologyType(HnsClient::Topology *topology = 0);
    HnsClient::TopologyEntry *_displaySaveDialog(HnsClient::Topology *topology);
    void _setScreenshot(QString uuid);
    void _handleSaveResult(QString uuid, bool result, QString exceptionMessage = QString());
    void _activateSimulationMode();
    void _toggleGrid(bool toggle);
    TopologyViewArea *_getArea(QString topologyUUID = QString());
    void _initViewArea(HnsClient::Topology *topology);
    void _refreshViewArea();
    bool _populateDefineMenu();
    void _deleteDefineSubMenu();
    void _disconnectDefineAction();

    Ui::TopologyWidget *_ui;
    QHash<QString, TopologyViewArea*> _areaHash;
    QHash<TopologyViewArea *, TopologyContainer *> _containerHash;
    LoadedTopologyModel *_loadedModel;
    bool _gridEnabled;
    QToolButton *_defineButton;
    QToolButton *_forceDisplayLabel;
    QToolButton *_printButton;
    QToolButton *_snapshotButton;
    QToolButton *_drawWireButton;
    QSignalMapper *_defineMapper;
    QSignalMapper *_defineBehaviorMapper;
    QList<QAction *> _hideWhenModelList;
    QSignalMapper *_drawScreenshotMapper;
    QSignalMapper *_revertMapper;
    QSignalMapper *_deleteMapper;
    QColor _wireColor;
    QHash<Hns::LabelPolicy, QAction *> _labelPolicyActionsHash;

//private slots:
//    void _refreshOverlayWidgetSlot();

};

} // namespace Hnv
#endif // TOPOLOGYWIDGET_H
