/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "usermanagertabs.h"
#include "ui_usermanagertabs.h"
#include "hnvmainwindow.h"
#include "ressources.h"

// hynesim hnsclient headers
#include <libhnsclient/usermanager.h>
#include <libhnsclient/session.h>

namespace Hnv {

/*******************************************************************************
* UserManagerTabs
*******************************************************************************/
UserManagerTabs::UserManagerTabs(QWidget *parent)
    : QWidget(parent)
    , ui(new Ui::UserManagerTabs)
{
    ui->setupUi(this);
    ui->toolBar->setCloseable(false);
    ui->toolBar->setToolButtonStyle(Qt::ToolButtonTextBesideIcon);
    ui->toolBar->addAction(ui->action_Users);
    ui->toolBar->addAction(ui->action_Groups);
    QActionGroup * group = new QActionGroup(this);
    group->addAction(ui->action_Users);
    ui->action_Users->setIcon(QIcon(Ressources::themedPath(Constants::ICON_USER)));
    group->addAction(ui->action_Groups);
    ui->action_Groups->setIcon(QIcon(Ressources::themedPath(Constants::ICON_GROUP)));
    connect(HnvMainWindow::instance(), SIGNAL(sessionConnected(HyneviewSession*)), this, SLOT(_setAccessor()));
    connect(HnvMainWindow::instance(), SIGNAL(sessionDisconnected()), this, SLOT(_sessionDisconnectedSlot()));
    ui->usersPage->setUsersModel(HnvMainWindow::instance()->usersModel());
    ui->groupsPage->setGroupsModel(HnvMainWindow::instance()->groupsModel());
}

void UserManagerTabs::_setAccessor()
{
    HnsClient::UserManager * userManager = HnsClient::Session::instance()->userManager();    
    ui->usersPage->setAccessor(userManager);
    ui->groupsPage->setAccessor(userManager);
}

void UserManagerTabs::_sessionDisconnectedSlot()
{
    ui->usersPage->setAccessor(0);
    ui->groupsPage->setAccessor(0);
}

UserManagerTabs::~UserManagerTabs()
{
    delete ui;
}
/*******************************************************************************
* UserManagerTabs - Private slots
*/
void UserManagerTabs::on_action_Users_triggered()
{
    ui->stackedWidget->setCurrentWidget(ui->usersPage);
}

void UserManagerTabs::on_action_Groups_triggered()
{
    ui->stackedWidget->setCurrentWidget(ui->groupsPage);
}

} // namespace Hnv
