/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "userclonedialog.h"
#include "ui_userclonedialog.h"
#include "hnvmainwindow.h"
#include "ressources.h"

namespace Hnv {

/*******************************************************************************
* UserCloneDialog
*/

UserCloneDialog::UserCloneDialog(QString userName, int &cloneCount, QString &password, QWidget *parent)
    : QDialog(parent)
    , ui(new Ui::UserCloneDialog)
    , _cloneCount(cloneCount)
    , _password(password)
{
    ui->setupUi(this);
    ui->labelUsernameInfo->setText(userName);
    ui->toolButton->setIcon(QIcon(Ressources::themedPath(Constants::ICON_PSW_GENERATOR)));

    QString windowTitle = tr("Clone %1").arg(userName);
    setWindowTitle(windowTitle);
}

UserCloneDialog::~UserCloneDialog()
{
    delete ui;
}

/*******************************************************************************
* UserCloneDialog - Private
*/


bool UserCloneDialog::isNotEmpty(QLabel *targetLabel, QString &text)
{
    bool isValid = false;
    text = text.trimmed();
    if (text == "") {
        isValid = false;
    } else {
        isValid = true;
    }
    setLabelValidationStyle(targetLabel, isValid);
    return isValid;
}

void UserCloneDialog::setLabelValidationStyle(QLabel *targetLabel, bool isValid)
{
    if (isValid)
        targetLabel->setStyleSheet("");
    else
        targetLabel->setStyleSheet("QLabel{color:red;}");
}

/*******************************************************************************
* UserCloneDialog - Private slots
*/

void UserCloneDialog::on_buttonBox_accepted()
{
    _cloneCount = ui->sbCloneCount->value();
    QString password = ui->lePassword->text();
    QString passwordConfirmation = ui->leConfirmPassword->text();

    bool isValid = isNotEmpty(ui->labelPassword, password);
    isValid &= isNotEmpty(ui->labelConfirmPassword, passwordConfirmation);

    if (password != passwordConfirmation) {
        setLabelValidationStyle(ui->labelPassword, false);
        setLabelValidationStyle(ui->labelConfirmPassword, false);
        isValid = false;
    }
    if (isValid) {
        _password = password;
        accept();
    }
}

void UserCloneDialog::on_toolButton_clicked()
{
    QString password = Hnv::HnvMainWindow::instance()->generatePassword();
    ui->lePassword->setText(password);
    ui->lePassword->setEchoMode(QLineEdit::Normal);
    ui->leConfirmPassword->setText(password);
    ui->leConfirmPassword->setEchoMode(QLineEdit::Normal);
}

void UserCloneDialog::on_lePassword_textEdited(QString )
{
    ui->lePassword->setEchoMode(QLineEdit::Password);
    ui->leConfirmPassword->setEchoMode(QLineEdit::Password);
}

void UserCloneDialog::on_leConfirmPassword_textEdited(QString )
{
    ui->lePassword->setEchoMode(QLineEdit::Password);
    ui->leConfirmPassword->setEchoMode(QLineEdit::Password);
}

} // namespace Hnv
