/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "usereditdialog.h"
#include "ui_usereditdialog.h"
#include "hyneviewsession.h"
#include "hnvmainwindow.h"
#include "ressources.h"

// Qt headers
#include <QModelIndex>
#include <QDateTime>

namespace Hnv {

/*******************************************************************************
* UserEditDialog
*******************************************************************************/
UserEditDialog::UserEditDialog(Hns::UserPtr &user, QString &password, QHash<qlonglong, QString> groupNameHash, QWidget *parent)
    : QDialog(parent)
    , _userData(user)
    , _password(password)
    , _groupNameHash(groupNameHash)
    , ui(new Ui::UserEditDialog)
{
    ui->setupUi(this);
    ui->toolButton->setDefaultAction(ui->action_Generate_password);
    ui->action_Generate_password->setIcon(QIcon(Ressources::themedPath(Constants::ICON_PSW_GENERATOR)));
    Hns::UserPtr userAdmin = Hns::UserPtr::dynamicCast(HyneviewSession::instance()->user());

    QString userName = QString::fromStdString(_userData->name);
    ui->leUsername->setText(userName);
    ui->leFirstName->setText(QString::fromStdString(_userData->firstname));
    ui->leLastName->setText(QString::fromStdString(_userData->lastname));

    ui->cbEnabled->setChecked(_userData->isEnabled);
    ui->cbIsAdvanced->setChecked(_userData->isAdvanced);

    _originalLogin = QString::fromStdString(_userData->name);

    if (_userData->createdOn == 0) {
        userName = tr("New user");
    } else {
        ui->labelCreationInfo->setText(QDateTime::fromMSecsSinceEpoch(_userData->createdOn).toString(Qt::SystemLocaleShortDate));
        ui->labelLastModificationInfo->setText(QDateTime::fromMSecsSinceEpoch(_userData->modifiedOn).toString(Qt::SystemLocaleShortDate));
        ui->lePassword->setPlaceholderText("********");
        ui->leConfirmPassword->setPlaceholderText("********");
    }
    QString windowTitle = tr("%1 properties").arg(userName);

    QList<Ice::Long> groupIDList = QList<Ice::Long>::fromStdList(_userData->groupList);

    foreach(Ice::Long key, _groupNameHash.keys()) {
        if (groupIDList.contains(key)) {
            QListWidgetItem *item = new QListWidgetItem(_groupNameHash.value(key),ui->listGroups);
            item->setData(Qt::UserRole, qlonglong(key));
        } else {
            QListWidgetItem *item = new QListWidgetItem(_groupNameHash.value(key),ui->listAvailableGroups);
            item->setData(Qt::UserRole, qlonglong(key));
        }

    }
    if (userAdmin->isAdvanced==false){
        ui->frameGroup->hide();
    }
    resize(minimumSizeHint());
    setWindowTitle(windowTitle);
}

UserEditDialog::~UserEditDialog()
{
    delete ui;
}
/*******************************************************************************
* UserEditDialog - Private
*/
bool UserEditDialog::isNotEmpty(QLabel *targetLabel, QString &text)
{
    bool isValid = false;
    text = text.trimmed();
    if (text == "") {
        isValid = false;
    } else {
        isValid = true;
    }
    setLabelValidationStyle(targetLabel, isValid);
    return isValid;
}

void UserEditDialog::setLabelValidationStyle(QLabel *targetLabel, bool isValid)
{
    if (isValid)
        targetLabel->setStyleSheet("");
    else
        targetLabel->setStyleSheet("QLabel{color:red;}");
}
/*******************************************************************************
* UserEditDialog - Private slots
*/
void UserEditDialog::on_removeAll_clicked()
{
    // TODO: a controler
    int itemCount = ui->listGroups->count();
    for(int i=0; i < itemCount; i++) {
        QListWidgetItem *item = ui->listGroups->takeItem(0);
        ui->listAvailableGroups->addItem(item);
    }
}

void UserEditDialog::on_removeSelected_clicked()
{
    // TODO: a controler
    QList<QListWidgetItem *> selectedItems = ui->listGroups->selectedItems();
    foreach(QListWidgetItem *item, selectedItems) {
        ui->listGroups->takeItem( ui->listGroups->row(item));
        ui->listAvailableGroups->addItem(item);
    }
}

void UserEditDialog::on_allAll_clicked()
{
    // TODO: a controler
    int itemCount = ui->listAvailableGroups->count();
    for(int i=0; i < itemCount; i++) {
        QListWidgetItem *item = ui->listAvailableGroups->takeItem(0);
        ui->listGroups->addItem(item);
    }
}

void UserEditDialog::on_addSelected_clicked()
{
    // TODO: a controler
    QList<QListWidgetItem *> selectedItems = ui->listAvailableGroups->selectedItems();
    foreach(QListWidgetItem *item, selectedItems) {
        ui->listAvailableGroups->takeItem( ui->listAvailableGroups->row(item));
        ui->listGroups->addItem(item);
    }
}

void UserEditDialog::on_buttonBox_accepted()
{
    QString userName = ui->leUsername->text();
    QString password = ui->lePassword->text();
    QString passwordConfirmation = ui->leConfirmPassword->text();
    QString firstName = ui->leFirstName->text();
    QString lastName = ui->leLastName->text();

    bool isValid = isNotEmpty(ui->labelUsername, userName);
    if (userName.contains(" ") && _originalLogin != userName) {
        isValid = false;
        setLabelValidationStyle(ui->labelUsername, false);
    }
    isValid &= isNotEmpty(ui->labelFirstName, firstName);
    isValid &= isNotEmpty(ui->labelLastName, lastName);

    if (_userData->createdOn == 0 || password != "") {
        isValid &= isNotEmpty(ui->labelPassword, password);
        isValid &= isNotEmpty(ui->labelConfirmPassword, passwordConfirmation);
    }

    if (password != passwordConfirmation) {
        setLabelValidationStyle(ui->labelPassword, false);
        setLabelValidationStyle(ui->labelConfirmPassword, false);
        isValid = false;
    }

    if (isValid) {
        _userData->name = userName.toStdString();
        _userData->firstname = firstName.toStdString();
        _userData->lastname = lastName.toStdString();
        _userData->isEnabled = (ui->cbEnabled->checkState() == Qt::Unchecked ? false : true);
        _userData->isAdvanced = (ui->cbIsAdvanced->checkState() == Qt::Unchecked ? false : true);
        Diamesh::LongList groupList;
        int itemCount = ui->listGroups->count();
        for(int i=0; i < itemCount; i++) {
            QListWidgetItem *item = ui->listGroups->item(i);
            groupList.push_back(item->data(Qt::UserRole).toLongLong());
        }
        _userData->groupList = groupList;
        _password = password;
        accept();
    }
}

void UserEditDialog::on_action_Generate_password_triggered()
{
    QString password = Hnv::HnvMainWindow::instance()->generatePassword();
    ui->lePassword->setText(password);
    ui->lePassword->setEchoMode(QLineEdit::Normal);
    ui->leConfirmPassword->setText(password);
    ui->leConfirmPassword->setEchoMode(QLineEdit::Normal);
}

void UserEditDialog::on_lePassword_textEdited(QString )
{
    ui->lePassword->setEchoMode(QLineEdit::Password);
    ui->leConfirmPassword->setEchoMode(QLineEdit::Password);
}

void UserEditDialog::on_leConfirmPassword_textEdited(QString )
{
    ui->lePassword->setEchoMode(QLineEdit::Password);
    ui->leConfirmPassword->setEchoMode(QLineEdit::Password);
}

void UserEditDialog::on_listAvailableGroups_doubleClicked(const QModelIndex &index)
{
    QListWidgetItem *item = ui->listAvailableGroups->takeItem(index.row());
    ui->listGroups->addItem(item);
}

void UserEditDialog::on_listGroups_doubleClicked(const QModelIndex &index)
{
    QListWidgetItem *item = ui->listGroups->takeItem(index.row());
    ui->listAvailableGroups->addItem(item);
}

void UserEditDialog::on_leUsername_textEdited(const QString &text) {
    QString lowerText = text.toLower();
    ui->leUsername->setText(lowerText);
}

} // namespace Hnv
