/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef USERSMANAGERWIDGET_H
#define USERSMANAGERWIDGET_H

// Qt headers
#include <QWidget>
#include <QItemSelection>

class QSortFilterProxyModel;
class QModelIndex;
class QToolButton;

namespace Ui {
class UsersManagerWidget;
} // namespace Ui

namespace HnsClient {
class UserManager;
} // namespace HnsClient

namespace Hnv {
class UsersModel;
class UsersFilterProxyModel;
class TreeSortFilterProxyModel;
class FilterLineEdit;

/*******************************************************************************
* UsersManagerWidget
*******************************************************************************/

class UsersManagerWidget : public QWidget
{
    Q_OBJECT

public:
    explicit UsersManagerWidget(QWidget *parent = 0);
    ~UsersManagerWidget();
    void setAccessor(HnsClient::UserManager *accessor);
    void setUsersModel(UsersModel *userModel = 0);

protected slots:
    void selectionChanged(QItemSelection selected, QItemSelection deselected);
    void _contextMenuRequested(QPoint pos);

private slots:
    void on_action_Offline_toggled(bool checked);
    void on_action_Disabled_toggled(bool checked);
    void on_action_Enabled_toggled(bool checked);
    void on_action_Online_toggled(bool checked);
    void on_action_All_triggered();
    void on_action_Delete_user_triggered();
    void on_action_Edit_user_triggered();
    void on_action_Clone_user_triggered();
    void on_action_New_user_triggered();
    void on_action_Disconnect_user_triggered();

    void _enableControls(bool enable);
    void _displayErrorMessage(QString message);
    void _rowInserted();
    void _filterLineEdit_textChanged(QString currentText);
    void on_usersTableView_doubleClicked(QModelIndex index);
    
    void _successDeleteUser(qlonglong, QString, bool);
    void _successAddUser(qlonglong, QString, bool);
    void _successCloneUser(qlonglong, QString, bool);
    void _successUpdateUser(qlonglong, QString, bool);

private:
    void _editUser(QModelIndex triggerIndex);

    Ui::UsersManagerWidget *ui;
    HnsClient::UserManager *_accessor;
    TreeSortFilterProxyModel * _usersProxyModel;
    UsersFilterProxyModel * _usersFilterProxyModel;
    UsersModel *_usersModel;
    QToolButton *_filterButton;
    FilterLineEdit *_filterLineEdit;
};

} // namespace Hnv
#endif // USERSMANAGERWIDGET_H
