/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "welcomemodewidget.h"
#include "welcomescreenwidget.h"
#include "ressources.h"

// Qt headers
#include <QAction>

namespace Hnv {

WelcomeModeWidget::WelcomeModeWidget(QWidget *parent)
    : ModeWidget(parent)
{
    setStyleSheet("Hnv--WelcomeModeWidget { background-image: url(':/themes/hynesim/icons/welcome-background.png'); background-repeat: repeat-xy; background-attachment: fixed;}");
    _screen = new WelcomeScreenWidget(this);
    _initDisplayArea(_screen);
    connect(_screen, SIGNAL(connectTriggered()), this, SIGNAL (connectTriggered()));
    connect(_screen, SIGNAL(quickConnectTriggered()), this, SIGNAL(quickConnectTriggered()));
    connect(_screen, SIGNAL(newSessionTriggered()), this, SIGNAL(newSessionTriggered()));
    connect(_screen, SIGNAL(openSessionTriggered()), this, SIGNAL(openSessionTriggered()));
}

WelcomeModeWidget::~WelcomeModeWidget()
{
}

bool WelcomeModeWidget::hasSideBar()
{
    return false;
}

bool WelcomeModeWidget::hasRightSideBar()
{
    return false;
}

void WelcomeModeWidget::_initModeAction()
{
    _modeAction = new QAction(QIcon(Ressources::themedPath(Constants::ICON_WELCOME)), tr("Welcome"), this);
}

void WelcomeModeWidget::connectedSlot(HyneviewSession *session)
{
    Q_UNUSED(session);
    _screen->setConnected(true);
}

void WelcomeModeWidget::disconnectedSlot()
{
    _screen->setConnected(false);
}


void WelcomeModeWidget::enableQuickConnect(bool enable){
    _screen->enableQuickConnect(enable);
}

} // namespace Hnv
