/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// Project headers
#include "welcomescreenwidget.h"
#include "ui_welcomescreenwidget.h"
#include "ressources.h"
#include "hnvmainwindow.h"

// Qt headers
#include <QToolButton>

namespace Hnv {

WelcomeScreenWidget::WelcomeScreenWidget(QWidget *parent)
    : QWidget(parent)
    , _ui(new Ui::WelcomeScreenWidget)
    , _enableQuickConnect(false)
{
    _ui->setupUi(this);
    _ui->frame->setStyleSheet("Hnv--WelcomeScreenWidget > QFrame{background: lightgray}");
    _ui->aboutLabel->setStyleSheet("background: white; color: black");
    _ui->openSessionButton->setIcon(QIcon(Ressources::themedPath(Constants::ICON_OPEN)));
    _ui->newSessionButton->setIcon(QIcon(Ressources::themedPath(Constants::ICON_NEW)));
    _ui->hynesimlogo->setPixmap(Ressources::themedPath(Constants::ICON_LOGO));
    _ui->toolBar->setCloseable(0);
    _ui->connectButton->setIcon(QIcon(Ressources::themedPath(Constants::ICON_CONNECT)));
    _ui->quickConnectButton->setIcon(QIcon(Ressources::themedPath(Constants::ICON_QUICK_CONNECT)));
    connect(_ui->connectButton, SIGNAL(clicked()), SIGNAL(connectTriggered()));
    connect(_ui->quickConnectButton, SIGNAL(clicked()), SIGNAL(quickConnectTriggered()));
    connect(_ui->newSessionButton, SIGNAL(clicked()), this, SIGNAL(newSessionTriggered()));
    connect(_ui->openSessionButton, SIGNAL(clicked()), this, SIGNAL(openSessionTriggered()));

    QToolButton *aboutButton = new QToolButton(this);
    aboutButton->setText(tr("About..."));
    aboutButton->setAutoExclusive(true);
    aboutButton->setCheckable(true);
    aboutButton->setChecked(true);
    aboutButton->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Preferred);
    _ui->toolBar->addWidget(aboutButton);
    _ui->toolBar->setBorders(ToolBar::None);
    connect(aboutButton, SIGNAL(clicked()), this, SLOT(_aboutSlot()));

//    QToolButton *simulationButton = new QToolButton(this);
//    simulationButton->setText(tr("Simulation"));
//    simulationButton->setAutoExclusive(true);
//    simulationButton->setCheckable(true);
//    simulationButton->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Preferred);
//    _ui->toolBar->addWidget(simulationButton);
//    connect(simulationButton, SIGNAL(clicked()), this, SLOT(_simulationSlot()));

    _ui->aboutLabel->setText(HnvMainWindow::instance()->aboutMessage());
}

WelcomeScreenWidget::~WelcomeScreenWidget()
{
    delete _ui;
}

void WelcomeScreenWidget::setConnected(bool connected)
{
    _ui->newSessionButton->setEnabled(connected);
    _ui->openSessionButton->setEnabled(connected);
    _ui->connectButton->setEnabled(!connected);
    if(!connected && !_enableQuickConnect)
      _ui->quickConnectButton->setEnabled(false);
    else
      _ui->quickConnectButton->setEnabled(!connected);
}

void WelcomeScreenWidget::_aboutSlot()
{
    _ui->stackedWidget->setCurrentWidget(_ui->aboutPage);
}

void WelcomeScreenWidget::_simulationSlot()
{
    _ui->stackedWidget->setCurrentWidget(_ui->simulationPage);
}

void WelcomeScreenWidget::enableQuickConnect(bool enable){
    _ui->quickConnectButton->setEnabled(enable);
    _enableQuickConnect = enable;
}

} // namespace Hnv
