/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef ZOOMINGVIEW_H
#define ZOOMINGVIEW_H

// Qt headers
#include <QGraphicsView>

namespace Hnv {

/*******************************************************************************
* ZoomingView
*******************************************************************************/
class ZoomingView : public QGraphicsView
{
    Q_OBJECT
public:
    explicit ZoomingView(QWidget * parent = 0);
    explicit ZoomingView(QGraphicsScene * scene, QWidget * parent = 0);
    void toggleGrid(bool toggle);
    void setCenter(const QPointF& poCenterPoint);
    void moveCenter(const QPointF &delta);
    void setScene(QGraphicsScene *newScene);
    void showOutline(bool show = true);
    void setZoomOutLimit(int limit);
    void setZoomInLimit(int limit);
    double currentZoomFactor();
    int currentZoomValue();
    //void toggleOpenGL( const bool pbEnabled );

public slots:
    void zoomOut(QPointF centerPos = QPointF());
    void zoomIn(QPointF centerPos = QPointF());
    void setZoomValue(double zoomValue);

signals:
    void cursorPositionChanged(int x, int y, bool inScene);
    void zoomValueChanged(int value);
    void viewPainted();

protected:
    QPoint _panningOrigin;
    QPointF _currentCenterPoint;
    QGraphicsItem * _targetItem;

    void _initView();
    void _zoom(double zoomFactor, QPointF centerPos = QPointF());
    virtual void wheelEvent(QWheelEvent *event);
    virtual void mousePressEvent(QMouseEvent* event);
    virtual void mouseReleaseEvent(QMouseEvent* event);
    virtual void mouseMoveEvent(QMouseEvent* event);
    virtual void drawBackground(QPainter *painter, const QRectF &rect);
    virtual void paintEvent(QPaintEvent *event);
    void _updateCoordonnate(QMouseEvent* event);

private:
    bool _showOutline;
    double _zoomFactor;
    int _zoomOutLimit;
    int _zoomInLimit;
    bool _displayGrid;
};

} // namespace Hnv
#endif // ZOOMINGVIEW_H
