/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef ZOOMPREVIEW_H
#define ZOOMPREVIEW_H

// Project headers
#include "sidebar.h"

// Qt headers
#include <QVariant>
#include <QGraphicsView>

class QWidget;

namespace Ui {
class ZoomPreview;
} // namespace Ui

namespace Hnv {
class SimulationModeWidget;
class TopologyWidget;
class ZoomingView;

/*******************************************************************************
* ZoomPreviewView
*******************************************************************************/
class ZoomPreviewView : public QGraphicsView
{
    Q_OBJECT
public:
    explicit ZoomPreviewView(QWidget *parent = 0);
    void setLinkedView(ZoomingView *linkedView);
    void setAreaSize(int value, bool expandToMax = true);

protected slots:
    void _sceneRectChangedSlot(QRectF scRect);
    void _linkedViewRefreshedSlot();

protected:
    void drawForeground(QPainter *painter, const QRectF &rect);
    virtual void _outlineLinkedArea();
    virtual void _updateSize(QRectF sceneRect);
    virtual void mousePressEvent(QMouseEvent *event);
    virtual void mouseMoveEvent(QMouseEvent *event);

    ZoomingView *_linkedView;
    QRectF _linkedRect;
    int _areaSize;
    bool _expandToMax;
};

/*******************************************************************************
* ZoomPreviewSideBarWidget
*******************************************************************************/
class ZoomPreviewSideBarWidget : public SideBarSimpleWidget
{
    Q_OBJECT
public:
    Q_INVOKABLE explicit ZoomPreviewSideBarWidget(QWidget *parent = 0);
    ~ZoomPreviewSideBarWidget();
    static QString title();
    static QString sideBarId();
    void setLinkedView(ZoomingView *linkedView);

protected:
    virtual void resizeEvent(QResizeEvent *event);

private slots:
    void _topologyActivatedSlot(QVariant topologyUUID);
private:
    SimulationModeWidget *_simulationModeWidget;
    TopologyWidget *_topologyWidget;
    ZoomPreviewView *_previewWidget;
};

} // namespace Hnv
#endif // ZOOMPREVIEW_H
