/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef HNSCLIENT_ACLMANAGER_H
#define HNSCLIENT_ACLMANAGER_H

// Generated hynesim headers
#include "common/catalogitem.h"
#include "common/usermanager.h"

// diamesh headers
#include <diamesh/proxies_accessrights.h>
#include <diamesh/hsetmanager.h>
#include <diamesh/accessrightshelper.h>

// Qt headers
#include <QObject>
#include <QString>

namespace HnsClient {

class ACLManagerPrivate;

/*******************************************************************************
* ACLManager
*******************************************************************************/
class ACLManager : public QObject
{
    Q_OBJECT
    Q_DECLARE_PRIVATE(ACLManager)
public:
    ~ACLManager();

    Diamesh::AccessRightsHelper *helper() const;
    Diamesh::AccessRightsHelper::SubjectType getSubjectType(const QString &subject) const;
    QString readableSubject(const QString &subject) const;
    QString readableRights(const Hns::CatalogItemPtr &catalogItem, const Diamesh::LongList &rights) const;

    bool canBuildAclForCatalogItem(const Hns::CatalogItemPtr &target, const Hns::UserPtr &user);

    Diamesh::AccessControlList buildACLForCatalogItem(const Hns::CatalogItemPtr &target, const QString &subject, const QStringList &rights) const;
    Diamesh::AccessControlList buildACL(const Diamesh::ACLTargetPtr &target, const QString &subject, const QStringList &rights) const;

    Hns::UserPtr userFromSubject(const QString &subject) const;
    Diamesh::GroupPtr groupFromSubject(const QString &subject) const;
    Diamesh::HierarchicalSet hSetFromSubject(const QString &subject) const;

    void getACLsForTarget(const Diamesh::ACLTargetPtr &target, QObject *receiver, const char *slot);

signals:
    void loaded();

protected:
    explicit ACLManager(QObject *parent = 0);
    void _load(Diamesh::ACLManagerQPrx qprx);
    void _callEnded();
    void _checkLoading();
    void _init();
    void _loadAccessRightsHelper();
    QString _prefixFromCatalogItem(const Hns::CatalogItemPtr &target) const;

protected slots:
    void _loadGetRegisteredRightsSlot(::Diamesh::AccessRightList rightList);
    void _loadGetTargetTypesSlot(::Diamesh::StringStringMap typeMap);
    void _getACLsForTargetSlot(::Diamesh::ACLList aclList, QObject *obj);

    void _aclAddedSlot(QString target, QString subject);
    void _aclChangedSlot(QString target, QString subject);
    void _aclRemovedSlot(QString target, QString subject);

private:
    friend class Session;

    Hns::UserPtr _userFromSubject(qlonglong userId) const;
    Diamesh::GroupPtr _groupFromSubject(qlonglong groupId) const;
    Diamesh::HierarchicalSet _hSetFromSubject(qlonglong hSetId) const;

    ACLManagerPrivate * const d_ptr;
};

} // namespace HnsClient

#endif //HNSCLIENT_ACLMANAGER_H
