/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// hynesim hnsclient headers
#include "libhnsclient/catalog.h"
#include "libhnsclient/private/catalog_p.h"

namespace HnsClient {

/*******************************************************************************
* CatalogPrivate
*******************************************************************************/
CatalogPrivate::CatalogPrivate()
    : QObject()
{
    remoteCallCount = 0;
}

CatalogPrivate::~CatalogPrivate()
{
}


/*******************************************************************************
* GetACLsForEntryCookie
*******************************************************************************/
GetACLsForEntryCookie::GetACLsForEntryCookie(qlonglong entryId)
    : QObject()
    , _entryId(entryId)
{
}

GetACLsForEntryCookie::~GetACLsForEntryCookie()
{
}

qlonglong GetACLsForEntryCookie::getEntryId() const
{
    return _entryId;
}


/*******************************************************************************
* Catalog
*******************************************************************************/
Catalog::Catalog(CatalogPrivate *d, QObject *parent)
    : QObject(parent)
    , d_ptr(d)
{
}

Catalog::~Catalog()
{
    delete d_ptr;
}

bool Catalog::isEntryLocked(qlonglong entryId) const
{
    Q_D(const Catalog);
    return (d->lockedEntries.contains(entryId));
}

Hns::LockType Catalog::lockType(qlonglong entryId) const
{
    Q_D(const Catalog);
    if (!isEntryLocked(entryId))
        return Hns::LOCK_TYPE_NOLOCK;
    return d->lockedEntries.value(entryId);
}

/*******************************************************************************
* Catalog - Protected slots
*/
void Catalog::_entryLockedSlot(Ice::Long entryID, Ice::Int lockType)
{
    Q_D(Catalog);
    if (!d->lockedEntries.contains(entryID))
        d->lockedEntries.insert(entryID, ::Hns::LockType(lockType));
    _updateEntryLockedStatus(entryID, true);
}

void Catalog::_entryUnlockedSlot(Ice::Long entryID)
{
    Q_D(Catalog);
    if (d->lockedEntries.contains(entryID))
        d->lockedEntries.remove(entryID);
    _updateEntryLockedStatus(entryID, false);
}

void Catalog::_getLockedEntriesSlot(Hns::LockTypeLongMap lockedEntries)
{
    Q_D(Catalog);
    for (Hns::LockTypeLongMap::const_iterator it = lockedEntries.cbegin() ; it != lockedEntries.cend() ; ++it)
        d->lockedEntries.insert(it->first, it->second);
    _catalogProxy()->begin_getEntries(this, SLOT(_getEntriesSlot(::Diamesh::LongList)));
}

void Catalog::_getEntriesSlot(::Diamesh::LongList entryList)
{
    Q_D(Catalog);
    d->remoteCallCount--;
    _parseEntryList(entryList);
}
/*******************************************************************************
* Catalog - Protected
*/
void Catalog::_loadEntries()
{
    Q_D(Catalog);
    d->remoteCallCount++;
    _catalogProxy()->begin_getLockedEntries(this, SLOT(_getLockedEntriesSlot(::Hns::LockTypeLongMap)));
}

} // namespace HnsClient
