/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef HNSCLIENT_CATALOG_H
#define HNSCLIENT_CATALOG_H

// Generated hynesim headers
#include "common/proxies_catalog.h"

// Qt headers
#include <QObject>

class QPixmap;
class QString;
class QVariant;

namespace HnsClient {
class CatalogPrivate;

/*******************************************************************************
* Catalog
*******************************************************************************/
class Catalog : public QObject
{
    Q_OBJECT
    Q_DECLARE_PRIVATE(Catalog)
public:
    explicit Catalog(CatalogPrivate *d, QObject *parent = 0);
    ~Catalog();

    bool isEntryLocked(qlonglong entryId) const;
    ::Hns::LockType lockType(qlonglong entryId) const;

signals:
    void updateACLFailed(QString errorMessage);


protected slots:
    virtual void _entryChangedSlot(Ice::Long entryID) = 0;
    virtual void _entryRemovedSlot(Ice::Long entryID) = 0;
    virtual void _entryAddedSlot(Ice::Long entryID) = 0;
    virtual void _entryLockedSlot(Ice::Long entryID, Ice::Int lockType);
    virtual void _entryUnlockedSlot(Ice::Long entryID);

    virtual void _getLockedEntriesSlot(::Hns::LockTypeLongMap lockedEntries);
    virtual void _getEntriesSlot(::Diamesh::LongList entryList);

protected:
    virtual Hns::CatalogQPrx _catalogProxy() const = 0;
    virtual void _loadEntries();
    virtual void _parseEntryList(Diamesh::LongList entryList) = 0;
    virtual void _updateEntryLockedStatus(Ice::Long entryID, bool isLocked) = 0;

    CatalogPrivate * const d_ptr;
};

} // namespace HnsClient

#endif //HNSCLIENT_CATALOG_H
