/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef HNSCLIENT_CATALOGIMPORTER_H
#define HNSCLIENT_CATALOGIMPORTER_H

// hynesim libclient headers
#include "libhnsclient/entry.h"

// Generated hynesim headers
#include "common/proxies_catalogimporter.h"
#include "common/archivedescriptor.h"

// Qt headers
#include <QObject>

namespace HnsClient {

class CatalogImporterPrivate;

/*******************************************************************************
* CatalogImporter
*******************************************************************************/
class CatalogImporter : public QObject {
    Q_OBJECT
    Q_DECLARE_PRIVATE(CatalogImporter)
public:
    ~CatalogImporter();
    // Methods based on remote calls
    void importArchive(const QString &tStorage, const QString &eStorage, const Hns::ImportableArchivePtr &archive, const Hns::ImportOptionsList &optionsList, QObject *receiver, const char *slot);
    void exportTopology(Hns::ArchiveDescriptorPtr archiveDescriptor, bool isArchive, QObject *receiver, const char *slot);
    void getArchiveList(QObject *receiver, const char *slot);
    void exportEntityEntry(HnsClient::EntityEntry *entry, QObject *receiver, const char *slot);
    void importEntityEntry(QString entityEntryPath, QString entityStorage, QObject *receiver, const char *slot);
    void getImportableEntityEntry(QObject *receiver, const char *slot);
signals:
    void loaded();
    void progressUpdated(int value);

protected:
    explicit CatalogImporter(QObject *parent = 0);
    void _load(Hns::CatalogImporterQPrx catalogImporterQPrx);
    Hns::EntityEntryDescriptorPtr _createEntityEntryDescriptor(HnsClient::EntityEntry *entry);

    CatalogImporterPrivate * const d_ptr;

protected slots:
    void _importArchiveSlot(bool success, QObject *obj);
    void _exportTopologySlot(bool result, QObject *obj);
    void _getArchiveListSlot(::Hns::ImportableArchiveList list, QObject *obj);
    void _showEntityEntryImportSlot(::Hns::EntityEntryToImportMap importMap, QObject *obj);
    void _importFolderSlot(bool success, QObject *obj);
    void _exportEntityEntrySlot(bool result, QObject *obj);

private:
    friend class Session;
};

} // namespace HnsClient
#endif // HNSCLIENT_CATALOGIMPORTER_H
