/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef HNSCLIENT_COMMON_H
#define HNSCLIENT_COMMON_H

// Qt headers
#include <QObject>
#include <QString>
#include <QHash>
#include <QMetaObject>

namespace HnsClient {

typedef QHash<QString, QMetaObject> StringMetaObjectHash;

enum ValidationResult {
    Valid,
    Invalid,
    Unmodified
};

enum SynchronizationStatus {
    Synced,
    Syncing,
    Error
};

enum BytesUnitEnum {
    Bytes,
    KibiBytes,
    MebiBytes,
    GibiBytes,
    TebiBytes
};

enum networkCardError{ macAdressError = 0x1,
    macAdressIsUsed = 0x10,
    nameIsUsed = 0x100,
    nameIsInvalid = 0x1000
};

enum storageDeviceError {
    deviceIsInvalid = 0x1
};

namespace Constants {
const int WPA_SIZE_MIN = 8;
const int WPA_SIZE_MAX = 63;
const int WPA_SIZE_HEXA = 64;
const int WEP_SIZE_HEXA_40_BITS = 10;
const int WEP_SIZE_HEXA_128_BITS = 26;
const int WEP_SIZE_HEXA_152_BITS = 32;
const int WEP_SIZE_DEC_40_BITS = 5;
const int WEP_SIZE_DEC_128_BITS = 13;
const int WEP_SIZE_DEC_152_BITS = 16;

const char * const FORMAT_PREVIEW = "png";

const char * const TRANSLATION_NAME = "libhnsclient_";

const char * const RDP_VIEWER_COMMAND_KEY = "rdpViewer";
const char * const RDP_VIEWER_DEFAULT_COMMAND = "rdesktop -T %TITLE% -k %KEYLAY%";
const char * const VNC_VIEWER_COMMAND_KEY = "vncViewer";
const char * const VNC_VIEWER_DEFAULT_COMMAND = "vncviewer -title %TITLE%";
const char * const VIEWER_KEYBOARD_LAYOUT = "viewerKeyboardLayout";
const char * const VIEWER_DEFAULT_KEYBOARD_LAYOUT = "fr";
const char * const TERMINAL_VIEWER = "xterm";
const char * const TERMINAL_VIEWER_PATH = "/usr/bin/xterm";
const char * const TELNET = "telnet";
const char * const TELNET_PATH = "/usr/bin/telnet";

const char * const PROPERTY_MINIMUM = "minimum";
const char * const PROPERTY_MAXIMUM = "maximum";

}// namespace Constants

} // namespace HnsClient

#endif // HNSCLIENT_COMMON_H
