/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// hynesim hnsclient headers
#include "libhnsclient/dynamipsrouterdb.h"
#include "libhnsclient/private/dynamipsrouterdb_p.h"

namespace HnsClient {

/*******************************************************************************
* DynamipsRouterDBPrivate
*******************************************************************************/
DynamipsRouterDBPrivate::DynamipsRouterDBPrivate()
{
    dynamipsRouterDBQPrx = 0;
    remoteCallCount = 0;
}

DynamipsRouterDBPrivate::~DynamipsRouterDBPrivate()
{
    dynamipsRouterDBQPrx = 0;
    dynamipsRouterMap.clear();
    dynamipsModuleMap.clear();
}


/*******************************************************************************
* DynamipsRouterDB
*******************************************************************************/
DynamipsRouterDB::DynamipsRouterDB(QObject *parent)
    : QObject(parent)
    , d_ptr(new DynamipsRouterDBPrivate)
{
}

DynamipsRouterDB::~DynamipsRouterDB()
{
    delete d_ptr;
}

/*! \fn QMap<QString, Hns::DataDynamipsRouter > getDynamipsRouterMap() const
 *\brief Local method. Return a QMap which describes supported dynamips platform
 */
QMap<QString, Hns::DataDynamipsRouter > DynamipsRouterDB::getDynamipsRouterMap() const
{
    Q_D(const DynamipsRouterDB);
    return d->dynamipsRouterMap;
}

/*! \fn QMap<QString, Hns::DynamipsRouterModule > getDynamipsModuleMap() const
 *\brief Local method. Return a QMap which describes supported dynamips module
 */
QMap<QString, Hns::DynamipsRouterModule> DynamipsRouterDB::getDynamipsModuleMap() const
{
    Q_D(const DynamipsRouterDB);
    return d->dynamipsModuleMap;
}

void DynamipsRouterDB::_load(Hns::DynamipsRouterDBQPrx qprx)
{
    Q_D(DynamipsRouterDB);
    d->dynamipsRouterDBQPrx = qprx;
    d->remoteCallCount = 2;
    d->dynamipsRouterDBQPrx->begin_getDataDynamipsMap(this, SLOT(_loadDynamipsRouterMapSlot(::Hns::DataDynamipsRouterMap)));
    d->dynamipsRouterDBQPrx->begin_getDynamipsRouterModuleMap(this, SLOT(_loadDynamipsModuleMapSlot(::Hns::DataDynamipsRouterModuleMap)));
}

void DynamipsRouterDB::_callEnded()
{
    Q_D(DynamipsRouterDB);
    d->remoteCallCount--;
    _checkLoading();
}

void DynamipsRouterDB::_checkLoading()
{
    Q_D(DynamipsRouterDB);
    if (d->remoteCallCount == 0)
        _init();
}

void DynamipsRouterDB::_init()
{
    emit loaded();
}

void DynamipsRouterDB::_loadDynamipsRouterMapSlot(::Hns::DataDynamipsRouterMap routerMap)
{
    Q_D(DynamipsRouterDB);
    QMap<std::string, Hns::DataDynamipsRouter> routerQMap(routerMap);
    QMapIterator<std::string, Hns::DataDynamipsRouter> iterator(routerQMap);
    while(iterator.hasNext()) {
        iterator.next();
        d->dynamipsRouterMap.insert(QString::fromStdString(iterator.key()), iterator.value());
    }
    _callEnded();
}

void DynamipsRouterDB::_loadDynamipsModuleMapSlot(::Hns::DataDynamipsRouterModuleMap moduleMap)
{
    Q_D(DynamipsRouterDB);
    QMap<std::string, Hns::DynamipsRouterModule> moduleQMap(moduleMap);
    QMapIterator<std::string, Hns::DynamipsRouterModule> iterator(moduleQMap);
    while(iterator.hasNext()) {
        iterator.next();
        d->dynamipsModuleMap.insert(QString::fromStdString(iterator.key()), iterator.value());
    }
    _callEnded();
}

} // namespace HnsClient
