/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// hynesim hnsclient headers
#include "libhnsclient/entity.h"
#include "libhnsclient/private/entity_p.h"
#include "libhnsclient/session.h"
#include "libhnsclient/private/privatesessionaccessor.h"
#include "libhnsclient/topology.h"
#include "libhnsclient/entry.h"
#include "libhnsclient/tracermanager.h"
#include "libhnsclient/filetransferclient.h"
#include "libhnsclient/common.h"

// hynesim common headers
#include "common/initializer.h"

// Generated hynesim headers
#include "common/proxies_dynamipsrouter.h"
#include "common/proxies_domain.h"

// diamesh headers
#include <diamesh/filetransferclient.h>

// Qt headers
#include <QMetaProperty>

namespace HnsClient {

HNS_STATIC_HASH_INITIALIZER(_propertiesDescriptionHash, PropertiesDescriptionHash, QString, QString)
PropertiesDescriptionHash::PropertiesDescriptionHash()
{
    insert("shortName", QObject::tr("Short name"));
    insert("longName", QObject::tr("Long name"));
    insert("description", QObject::tr("Description"));
    insert("tags", QObject::tr("Tags"));
    insert("domainType", QObject::tr("Type"));
    insert("memory", QObject::tr("Memory"));
    insert("vCpu", QObject::tr("Number of CPU"));
    insert("portCount", QObject::tr("Number of ports"));
    insert("isHub", QObject::tr("Use as Hub"));
    insert("stpEnabled", QObject::tr("STP enabled"));
    insert("switchType", QObject::tr("Type"));
    insert("wireType", QObject::tr("Type"));
    insert("visualLayout", QObject::tr("Angles"));
    insert("lossLR", QObject::tr("Loss L/R"));
    insert("delayLR", QObject::tr("Delay L/R"));
    insert("dupLR", QObject::tr("Duplicate L/R"));
    insert("bandwidthLR", QObject::tr("Bandwidth L/R"));
    insert("mtuLR", QObject::tr("MTU L/R"));
    insert("lossRL", QObject::tr("Loss R/L"));
    insert("delayRL", QObject::tr("Delay R/L"));
    insert("dupRL", QObject::tr("Duplicate R/L"));
    insert("bandwidthRL", QObject::tr("Bandwidth R/L"));
    insert("mtuRL", QObject::tr("MTU R/L"));
}

/*******************************************************************************
* DownloadCaptureFileCookie
*******************************************************************************/
DownloadCaptureFileCookie::DownloadCaptureFileCookie(QString remoteFileName, QObject *receiver, const char *slot)
    : QObject()
    , _remoteFileName(remoteFileName)
{
    connect(this, SIGNAL(completed(HnsClient::FileTransferClient*)), receiver, slot);
}

DownloadCaptureFileCookie::~DownloadCaptureFileCookie()
{
}

void DownloadCaptureFileCookie::emitCompleted(FileTransferClient *client)
{
    emit completed(client);
    deleteLater();
}

QString DownloadCaptureFileCookie::remoteFileName()
{
    return _remoteFileName;
}

/*******************************************************************************
* RemoteDisplayInfoCookie
*******************************************************************************/
RemoteDisplayInfoCookie::RemoteDisplayInfoCookie(int id, QObject *receiver, const char *slot)
    : QObject()
    , _id(id)
{
    connect(this, SIGNAL(completed(int,Hns::RemoteDisplayInfo)), receiver, slot);
}

RemoteDisplayInfoCookie::~RemoteDisplayInfoCookie()
{
}

void RemoteDisplayInfoCookie::emitCompleted(Hns::RemoteDisplayInfo info)
{
    emit completed(_id, info);
    deleteLater();
}

/*******************************************************************************
* SnapshotCookie
*******************************************************************************/
SnapshotCookie::SnapshotCookie(const QString &snapshotName, QObject *receiver, const char *slot)
    : QObject()
    , _snapshotName(snapshotName)
{
    connect(this, SIGNAL(completed(bool,QString)), receiver, slot);
}

SnapshotCookie::~SnapshotCookie()
{
}

void SnapshotCookie::emitCompleted(bool result)
{
    emit completed(result, _snapshotName);
    deleteLater();
}

/*******************************************************************************
* MemoryDumpCookie
*******************************************************************************/
MemoryDumpCookie::MemoryDumpCookie(QObject *receiver, const char *slot)
    : QObject()
{
    connect(this, SIGNAL(completed(int)), receiver, slot);
}

MemoryDumpCookie::~MemoryDumpCookie()
{
}

void MemoryDumpCookie::emitCompleted(int jobId)
{
    emit completed(jobId);
    deleteLater();
}

/*******************************************************************************
* DownloadMemoryDumpCookie
*******************************************************************************/
DownloadMemoryDumpCookie::DownloadMemoryDumpCookie(QObject *receiver, const char *slot)
    : QObject()
{
    connect(this, SIGNAL(completed(HnsClient::FileTransferClient*)), receiver, slot);
}

DownloadMemoryDumpCookie::~DownloadMemoryDumpCookie()
{
}

void DownloadMemoryDumpCookie::emitCompleted(FileTransferClient *client)
{
    emit completed(client);
    deleteLater();
}

QString DownloadMemoryDumpCookie::remoteFileName() const
{
    return _remoteFileName;
}

void DownloadMemoryDumpCookie::setRemoteFileName(const QString &remoteFileName)
{
    _remoteFileName = remoteFileName;
}

QString DownloadMemoryDumpCookie::localFileName() const
{
    return _localFileName;
}

void DownloadMemoryDumpCookie::setLocalFileName(const QString &localFileName)
{
    _localFileName = localFileName;
}

/*******************************************************************************
* GenericEntityPrivate
*******************************************************************************/
GenericEntityPrivate::GenericEntityPrivate()
{
    entityId = -1;
    entityQtProxy = 0;
    tracerQtProxy = 0;
    entry = 0;
    localEntry = 0;
    status = Hns::UNDEFINED;
    cptStatus = -1;
    remoteCallCount = 0;
    topology = 0;
    nodeName.clear();
}

GenericEntityPrivate::~GenericEntityPrivate()
{
    entityQtProxy = 0;
    tracerQtProxy = 0;
    entry->deleteLater();
    entry = 0;
    if (localEntry) {
        localEntry->deleteLater();
        localEntry = 0;
    }
    topology = 0;
}


/*******************************************************************************
* WireEntityPrivate
*******************************************************************************/
WireEntityPrivate::WireEntityPrivate()
    : GenericEntityPrivate()
{
    toEntity = 0;
    fromEntity = 0;
}

WireEntityPrivate::~WireEntityPrivate()
{
    toEntity = 0;
    fromEntity = 0;
}


/*******************************************************************************
* EntityPrivate
*******************************************************************************/
EntityPrivate::EntityPrivate()
    : GenericEntityPrivate()
{
}

EntityPrivate::~EntityPrivate()
{
}


/*******************************************************************************
* DomainEntityPrivate
*******************************************************************************/
DomainEntityPrivate::DomainEntityPrivate()
    : EntityPrivate()
{
    domainQtProxy = 0;
}

DomainEntityPrivate::~DomainEntityPrivate()
{
    domainQtProxy = 0;
}


/*******************************************************************************
* SwitchEntityPrivate
*******************************************************************************/
SwitchEntityPrivate::SwitchEntityPrivate()
    : EntityPrivate()
{
}

SwitchEntityPrivate::~SwitchEntityPrivate()
{
}


/*******************************************************************************
* HybridNetcardEntityPrivate
*******************************************************************************/
HybridNetcardEntityPrivate::HybridNetcardEntityPrivate()
    : EntityPrivate()
{
    hybridQtProxy = 0;
}

HybridNetcardEntityPrivate::~HybridNetcardEntityPrivate()
{
    hybridQtProxy = 0;
}


/*******************************************************************************
* WifiAPEntityPrivate
*******************************************************************************/
WifiAPEntityPrivate::WifiAPEntityPrivate()
    : EntityPrivate()
{
    wifiQtProxy = 0;
}

WifiAPEntityPrivate::~WifiAPEntityPrivate()
{
    wifiQtProxy = 0;
}


/*******************************************************************************
* DynamipsRouterEntityPrivate
*******************************************************************************/
DynamipsRouterEntityPrivate::DynamipsRouterEntityPrivate()
    : EntityPrivate()
{
}

DynamipsRouterEntityPrivate::~DynamipsRouterEntityPrivate()
{
}

/*******************************************************************************
* TopologyGateEntityPrivate
*******************************************************************************/
TopologyGateEntityPrivate::TopologyGateEntityPrivate()
    : EntityPrivate()
{
    topologyGateQtProxy = 0;
}

TopologyGateEntityPrivate::~TopologyGateEntityPrivate()
{
    topologyGateQtProxy = 0;
}


/*******************************************************************************
* GenericEntity
*******************************************************************************/
GenericEntity::GenericEntity(GenericEntityPrivate *d, QObject *parent)
    : QObject(parent)
    , d_ptr(d)
{
    d->remoteDisplayInfoLoaded = false;
}

GenericEntity::~GenericEntity()
{
    unregisterTracer();
    delete d_ptr;
}

QString GenericEntity::propertyDescription(const QString &propertyName)
{
    return _propertiesDescriptionHash()->value(propertyName, QString());
}

QMap<QString, QVariant> HnsClient::GenericEntity::propertyInformation(const QString &propertyName) const
{
    Q_D(const GenericEntity);
    QMap<QString, QVariant> information;
    if (d->propertyInformationMap.contains(propertyName))
        information = d->propertyInformationMap.value(propertyName).toMap();
    return information;
}

/*! \fn getId() const
 *\brief Local method. Returns the entity identifier
 */
qlonglong GenericEntity::getId() const
{
    Q_D(const GenericEntity);
    return d->entityId;
}

/*! \fn *getEntry() const
 *\brief Local method. Returns the internal entry as a HnsClient::EntityEntry object
 */
EntityEntry *GenericEntity::getEntry() const
{
    Q_D(const GenericEntity);
    return d->entry;
}

/*! \fn getHnsEntry() const
 *\brief Local method. Returns the internal entry as a Hns::EntityEntryPtr object
 */
Hns::EntityEntryPtr GenericEntity::getHnsEntry() const
{
    Q_D(const GenericEntity);
    return d->entry->entityEntry();
}

/*! \fn iceId() const
 *\brief Local method. Returns the ICE identifier of the internal entry
 */
QString GenericEntity::iceId() const
{
    Q_D(const GenericEntity);
    return QString::fromStdString(d->entry->entityEntry()->ice_id());
}

/*! \fn getEntryId() const
 *\brief Local method. Returns the entity entry identifier
 */
qlonglong GenericEntity::getEntryId() const
{
    Q_D(const GenericEntity);
    return d->entry->getID();
}

/*! \fn getShortName() const
 *\brief Local method. Returns the entity short name
 */
QString GenericEntity::getShortName() const
{
    return _currentEntry()->getShortName();
}

/*! \fn longName() const
 *\brief Local method. Returns the entity long name
 */
QString GenericEntity::longName() const
{
    return _currentEntry()->getLongName();
}

/*! \fn description() const
 *\brief Local method. Returns the entity description
 */
QString GenericEntity::description() const
{
    return _currentEntry()->getDescription();
}

/*! \fn tags() const
 *\brief Local method. Returns the entity tags
 */
QString GenericEntity::tags() const
{
    return _currentEntry()->getTagList();
}

/*! \fn getAllowedStatus() const
 *\brief Local method. Returns the allowed status for the entity
 */
QList<Hns::Status> GenericEntity::getAllowedStatus() const
{
    Q_D(const GenericEntity);
    return d->allowedStatus;
}

/*! \fn getStatus() const
 *\brief Local method. Returns the current status of the entity
 */
Hns::Status GenericEntity::getStatus() const
{
    Q_D(const GenericEntity);
    return d->status;
}

/*! \fn isDefined() const
 *\brief Local method. Returns true if the entity is currently defined
 */
bool GenericEntity::isDefined() const
{
    return (getStatus() != Hns::UNDEFINED);
}
/*! \fn isStarted() const
 *\brief TODO
 */
bool GenericEntity::isStarted() const
{
    return (getStatus() == Hns::STARTED);
}

/*! \fn isModel() const
 *\brief Local method. Returns true if the entity is a model entity
 */
bool GenericEntity::isModel() const
{
    Q_D(const GenericEntity);
    return d->entry->isModel();
}

/*! \fn getNodeList() const
 *\brief Local method. Returns the list of node which can define the entity
 */
QStringList GenericEntity::getNodeList() const
{
    Q_D(const GenericEntity);
    return d->nodeList;
}

/*! \fn getNodeName() const
 *\brief Local method. Returns an empty string if the entity is not defined, otherwise the name of the node used by the entity
 */
QString GenericEntity::getNodeName() const
{
    Q_D(const GenericEntity);
    return d->nodeName;
}

/*! \fn getCaptureJobHash() const
 *\brief Local method.
 */
QHash<int, int> GenericEntity::getCaptureJobHash() const
{
    Q_D(const GenericEntity);
    return d->captureJobHash;
}

/*! \fn getPlugs() const
 *\brief Local method. Returns a QMap of Hns::Plug with plug identifier as key
 */
QMap<int, Hns::Plug> GenericEntity::getPlugs() const
{
    Q_D(const GenericEntity);
    return d->plugMap;
}

/*! \fn getAvailablePlugs()
 *\brief Local method. Returns a QMap of unused Hns::Plug with plug identifier as key
 */
QMap<int, Hns::Plug> GenericEntity::getAvailablePlugs() const
{
    Q_D(const GenericEntity);
    QMap<int, Hns::Plug> result;
    foreach(const Hns::Plug &plug, d->plugMap)
        if (!plug.isPlugged)
            result.insert(plug.id, plug);
    return result;
}

/*! \fn getAvailableInterfacesForCapture() const
 *\brief Local method. Returns a list of plug identifier which
 */
QList<int> GenericEntity::getAvailableInterfacesForCapture() const
{
    Q_D(const GenericEntity);
    return d->availableInterfacesForCapture;
}

/*! \fn getFlags() const
 *\brief Local method. Returns the entity capabilities flags
 */
int GenericEntity::getFlags() const
{
    Q_D(const GenericEntity);
    return d->capabilitiesFlags;
}

/*! \fn isImplementedFromModel()
 *\brief TODO
 */
bool GenericEntity::isImplementedFromModel() const
{
    Q_D(const GenericEntity);
    return d->isImplementedFromModel;
}

/*! \fn registerTracer()
 *\brief TODO
 */
void GenericEntity::registerTracer()
{
    Q_D(GenericEntity);
    PrivateSessionAccessor::instance()->session()->tracerManager()->registerEntityTracer(this, d->tracerQtProxy);
}

/*! \fn unregisterTracer()
 *\brief TODO
 */
void GenericEntity::unregisterTracer()
{
    Q_D(GenericEntity);
    PrivateSessionAccessor::instance()->session()->tracerManager()->unregisterEntityTracer(this, d->tracerQtProxy);
}

/*! \fn userCanRead()
 *\brief Local method. Returns true if current user can read the entry which describe the entity
 */
bool GenericEntity::userCanRead() const
{
    Q_D(const GenericEntity);
    return d->entry->isReadable();
}

/*! \fn userCanWrite()
 *\brief Local method. Returns true if current user can modify the entry which describe the entity
 */
bool GenericEntity::userCanWrite() const
{
    Q_D(const GenericEntity);
    return d->entry->isWriteable();
}

/*! \fn userCanExec()
 *\brief Local method. Returns true if current user can use the entry which describe the entity
 */
bool GenericEntity::userCanExec() const
{
    Q_D(const GenericEntity);
    return d->entry->isExecutable();
}

/*! \fn addNode(QString nodeName)
 *\brief TODO
 *\param nodeName
 */
void GenericEntity::addNode(QString nodeName)
{
    Q_D(GenericEntity);
    if (!d->nodeList.contains(nodeName)) {
        d->nodeList.append(nodeName);
        refresh();
    }
}

/*! \fn removeNode(QString nodeName)
 *\brief TODO
 *\param nodeName
 */
void GenericEntity::removeNode(QString nodeName)
{
    Q_D(GenericEntity);
    if (d->nodeList.contains(nodeName)) {
        d->nodeList.removeAll(nodeName);
        refresh();
    }
}

/*! \fn define(QString nodeName)
 *\brief Remote method. Define the entity on the specified node
 *\param nodeName Node identifier to be used to define the entity
 */
void GenericEntity::define(QString nodeName)
{
    Q_D(GenericEntity);
    if (d->status == Hns::UNDEFINED && d->allowedStatus.contains(Hns::STOPPED)) {
        if (d->nodeList.contains(nodeName))
            d->entityQtProxy->begin_define(nodeName.toStdString(), this, SLOT(_defineSlot(bool)));
        else
            qDebug() << tr("Aborting 'define' operation. Entity %1 (%2) is not supported by node %3").arg(getShortName()).arg(QString::number(getId())).arg(nodeName);
    } else
        qDebug() << tr("Aborting 'define' operation. Entity %1 (%2) is already defined.").arg(getShortName()).arg(QString::number(getId()));
}

/*! \fn start()
 *\brief Remote method. Start the entity
 */
void GenericEntity::start()
{
    Q_D(GenericEntity);
    if (d->status == Hns::STOPPED && d->allowedStatus.contains(Hns::STARTED))
        d->entityQtProxy->begin_start(this, SLOT(_startSlot(bool)));
    else if (d->status == Hns::PAUSED && d->allowedStatus.contains(Hns::STARTED))
        d->entityQtProxy->begin_resume(this, SLOT(_resumeSlot(bool)));
    else if (d->status == Hns::SAVED && d->allowedStatus.contains(Hns::STARTED))
        d->entityQtProxy->begin_restore(this, SLOT(_restoreSlot(bool)));
    else
        qDebug() << tr("Aborting 'start' operation. Current entity %1 (%2) state forbid this operation.").arg(getShortName()).arg(QString::number(getId()));
}

/*! \fn suspend()
 *\brief Remote method. Suspend the entity
 */
void GenericEntity::suspend()
{
    Q_D(GenericEntity);
    if (d->status == Hns::STARTED && d->allowedStatus.contains(Hns::PAUSED))
        d->entityQtProxy->begin_suspend(this, SLOT(_suspendSlot(bool)));
    else
        qDebug() << tr("Aborting 'suspend' operation. Entity %1 (%2) is not currently started").arg(getShortName()).arg(QString::number(getId()));
}

/*! \fn stop()
 *\brief Remote method. Stop the entity
 */
void GenericEntity::stop()
{
    Q_D(GenericEntity);
    if (d->status == Hns::STARTED && d->allowedStatus.contains(Hns::STOPPED))
        d->entityQtProxy->begin_stop(this, SLOT(_stopSlot(bool)));
    else
        qDebug() << tr("Aborting 'stop' operation. Entity %1 (%2) is not currently started").arg(getShortName()).arg(QString::number(getId()));
}

/*! \fn save()
 *\brief Remote method. Save to disk the entity
 */
void GenericEntity::save()
{
    Q_D(GenericEntity);
    if (d->status == Hns::STARTED && d->allowedStatus.contains(Hns::SAVED))
        d->entityQtProxy->begin_save(this, SLOT(_saveSlot(bool)));
    else
        qDebug() << tr("Aborting 'save' operation. Entity %1 (%2) is not currently started").arg(getShortName()).arg(QString::number(getId()));
}

/*! \fn undefine()
 *\brief Remote method. Undefine the entity
 */
void GenericEntity::undefine()
{
    Q_D(GenericEntity);
    if (d->status != Hns::UNDEFINED)
        d->entityQtProxy->begin_undefine(this, SLOT(_undefineSlot(bool)));
    else
        qDebug() << tr("Aborting 'undefine' operation. Entity %1 (%2) is not currently defined.").arg(getShortName()).arg(QString::number(getId()));
}

/*! \fn destroy()
 *\brief Remote method. Delete the entity
 */
void GenericEntity::destroy()
{
    Q_D(GenericEntity);
    d->topology->removeEntity(d->entityId);
}

/*! \fn unImplement()
 *\brief Remote method. Un-implement the entity
 */
void GenericEntity::unImplement()
{
    Q_D(GenericEntity);
    d->topology->unImplementEntity(d->entityId);
}

/*! \fn createCaptureJob(Hns::CaptureJobOptions options)
 *\brief Remote method. Create a new capture job with specified options
 *\param options A set of capture job options used to define the new capture job
 */
void GenericEntity::createCaptureJob(Hns::CaptureJobOptions)
{
    qDebug()<<tr("Not supported yet.");
}

/*! \fn cancelCaptureJob(Ice::Int jobId)
 *\brief Remote method. Cancel running capture job by Id
 *\param jobId Job identifier to be canceled
 */
void GenericEntity::cancelCaptureJob(Ice::Int)
{
    qDebug()<<tr("Not supported yet.");
}

/*! \fn updateEntry(EntityEntry *updatedEntry)
 *\brief TODO
 *\param updateEntry
 */
void GenericEntity::updateEntry(EntityEntry *updatedEntry)
{
    Q_D(GenericEntity);
    d->entityQtProxy->begin_updateEntry(updatedEntry->entityEntry());
}

void GenericEntity::setShortName(const QString &shortName)
{
    if (shortName != getShortName()) {
        HnsClient::EntityEntry *updatedEntry = _localEntry();
        if (updatedEntry && updatedEntry->setShortName(shortName))
            updateEntry(updatedEntry);
    }
}

void GenericEntity::setLongName(const QString &value)
{
    if (value != longName()) {
        HnsClient::EntityEntry *updatedEntry = _localEntry();
        if (updatedEntry && updatedEntry->setLongName(value))
            updateEntry(updatedEntry);
    }
}

void GenericEntity::setDescription(const QString &value)
{
    if (value != description()) {
        HnsClient::EntityEntry *updatedEntry = _localEntry();
        if (updatedEntry && updatedEntry->setDescription(value))
            updateEntry(updatedEntry);
    }
}

void GenericEntity::setTags(const QString &value)
{
    if (value != tags()) {
        HnsClient::EntityEntry *updatedEntry = _localEntry();
        if (updatedEntry && updatedEntry->setTagList(value))
            updateEntry(updatedEntry);
    }
}

/*! \fn getPcapInfoByJobId(int jobID, QObject *receiver, const char * slot)
 *\brief TODO
 *\param jobID
 *\param receiver object connected to internal signal
 *\param slot TODO
 */
void GenericEntity::getPcapInfoByJobId(int jobID, QObject *receiver, const char *slot)
{
    Q_D(GenericEntity);
    d->entityQtProxy->begin_getPcapInfoByJobId(jobID, receiver, slot);
}

/*! \fn getCaptureJobIdPath(QObject *receiver, const char * slot);
 *\brief TODO
 *\param receiver object connected to internal signal
 *\param slot TODO
 */
void GenericEntity::getCaptureJobIdPath(QObject *receiver, const char *slot)
{
    Q_D(GenericEntity);
    d->entityQtProxy->begin_getCaptureJobIdPath(receiver, slot);
}

/*! \fn getPcapInfoList(QObject *receiver, const char * slot)
 *\brief TODO
 *\param receiver object connected to internal signal
 *\param slot TODO
 */
void GenericEntity::getPcapInfoList(QObject *receiver, const char *slot)
{
    Q_D(GenericEntity);
    d->entityQtProxy->begin_getPcapInfoList(receiver, slot);
}

/*! \fn downloadFile(QStringList sourceList, QObject *receiver, const char *slot)
 *\brief Create a cookie object with file name of each element of sourceList, receiver and slot parameters and give it to begin_downloadCaptureFile.
 *\param sourceList list of file name
 *\param receiver object connected to internal signal
 *\param slot Return slot connected to signal "completed(FileTransferClient *client)"
 */
void GenericEntity::downloadCaptureFile(QStringList sourceList, QObject *receiver, const char *slot)
{
    Q_D(GenericEntity);
    foreach (const QString file, sourceList) {
        DownloadCaptureFileCookie *cookie = new DownloadCaptureFileCookie(file, receiver, slot);
        d->entityQtProxy->begin_downloadCaptureFile(file.toStdString(), this, SLOT(_downloadCaptureFileSlot(::Diamesh::FileTransferPrx,QObject*)), 0, 0, cookie);
    }
}

/*! \fn eraseFile(QStringList sourceList)
 *\brief TODO
 *\param sourceList list of file name
 */
void GenericEntity::eraseFile(QStringList sourceList)
{
    Q_D(GenericEntity);
    foreach (const QString &file, sourceList)
        d->entityQtProxy->begin_eraseCaptureFile(file.toStdString());
}

/*! \fn refreshNodeList()
 *\brief TODO
 */
void GenericEntity::refreshNodeList()
{
    Q_D(GenericEntity);
    PrivateSessionAccessor::instance()->session()->getEntryNodeInfoMap(d->entry->entityEntry(), this, SLOT(_getEntryNodeInfoMap(::Hns::NodeInfoMap)));
}

/*! \fn refresh()
 *\brief TODO
 */
void GenericEntity::refresh()
{
    emit refreshed(this);
}

/*! \fn getRemoteDisplayInfo(QObject *receiver, const char * slot)
 *\brief RemoteMethod ask remote display Information
 *\param receiver object connected to internal signal
 *\param slot Return slot connected to signal must be type : void slot(int id, Hns::RemoteDisplayInfo info)
 */
void GenericEntity::getRemoteDisplayInfo(QObject *receiver, const char *slot)
{
    Q_D(GenericEntity);
    RemoteDisplayInfoCookie *cookie = new RemoteDisplayInfoCookie(d->entityId, receiver, slot);
    if (!d->remoteDisplayInfoLoaded)
        d->entityQtProxy->begin_getRemoteDisplayInfo(this, SLOT(_getRemoteDisplayInfoSlot(::Hns::RemoteDisplayInfo,QObject*)),0, 0, cookie);
    else {
        cookie->emitCompleted(d->remoteDisplayInfo);
    }
}

void GenericEntity::_loadIdSlot(Ice::Long entityId)
{
    Q_D(GenericEntity);
    d->entityId = entityId;
    _callEnded();
}

void GenericEntity::_loadEntrySlot(Hns::EntityEntryPtr entry)
{
    _loadEntryData(entry);
}

void GenericEntity::_loadStatusSlot(Hns::Status currentStatus)
{
    if (_handleStatusData(currentStatus, this, SLOT(_loadNodeNameSlot(::std::string)))) {
        _callEnded();
    }
}

void GenericEntity::_loadAllowedStatusesSlot(Hns::StatusList allowedStatus)
{
    Q_D(GenericEntity);
    d->allowedStatus = QList<Hns::Status>::fromStdList(allowedStatus);
    _callEnded();
}

void GenericEntity::_loadPlugsSlot(Hns::PlugMap plugMap)
{
    _handlePlugsData(plugMap);
    _callEnded();
}

void GenericEntity::_loadAvailablePlugsForCaptureSlot(Diamesh::IntList interfaceList)
{
    _handleAvailablePlugsForCaptureData(interfaceList);
    _callEnded();
}

void GenericEntity::_loadFlagsSlot(Ice::Int flags)
{
    Q_D(GenericEntity);
    d->capabilitiesFlags = flags;
    _callEnded();
}

void GenericEntity::_loadEntryNodeInfoMapSlot(::Hns::NodeInfoMap nodeMap)
{
    _handleEntryNodeInfoMapData(nodeMap);
    _callEnded();
}

void GenericEntity::_loadNodeNameSlot(std::string nodeName)
{
    _handleNodeNameData(nodeName);
    _callEnded();
}

void GenericEntity::_loadCaptureJobHashSlot(Hns::IntIntMap map)
{
    Q_D(GenericEntity);
    std::map<Ice::Int, Ice::Int>::iterator it = map.begin();
    while (it != map.end()) {
        d->captureJobHash.insert(it->first, it->second);
        ++it;
    }
    _callEnded();
}

void GenericEntity::_loadIsImplementFromModelSlot(bool isImplement)
{
    _handleIsImplementFromModelData(isImplement);
    _callEnded();
}

void GenericEntity::_loadTracerSlot(Hns::TracerPrx tracerProxy)
{
    Q_D(GenericEntity);
    Hns::TracerQPrx qprx = Hns::TracerQPrx(tracerProxy, PrivateSessionAccessor::instance()->adapter());
    d->tracerQtProxy = qprx;
    registerTracer();
    _callEnded();
}

void GenericEntity::_defineSlot(bool result)
{
    if (!result)
        qCritical() << tr("Unable to define entity %1").arg(getShortName());
}

void GenericEntity::_undefineSlot(bool result)
{
    if (!result)
        qCritical() << tr("Unable to undefine entity %1").arg(getShortName());
}

void GenericEntity::_startSlot(bool result)
{
    if (!result)
        qCritical() << tr("Unable to start entity %1").arg(getShortName());
}

void GenericEntity::_resumeSlot(bool result)
{
    if (!result)
        qCritical() << tr("Unable to resume entity %1").arg(getShortName());
}

void GenericEntity::_restoreSlot(bool result)
{
    if (!result)
        qCritical() << tr("Unable to restore entity %1").arg(getShortName());
}

void GenericEntity::_suspendSlot(bool result)
{
    if (!result)
        qCritical() << tr("Unable to suspend entity %1").arg(getShortName());
}

void GenericEntity::_stopSlot(bool result)
{
    if (!result)
        qCritical() << tr("Unable to stop entity %1").arg(getShortName());
}

void GenericEntity::_saveSlot(bool result)
{
    if (!result)
        qCritical() << tr("Unable to save entity %1").arg(getShortName());
}

void GenericEntity::_removeSlot(bool result)
{
    if (!result)
        qCritical() << tr("Unable to remove entity %1").arg(getShortName());
}

void GenericEntity::_unImplementSlot(bool result)
{
    if (!result)
        qCritical() << tr("Unable to unimplement entity %1").arg(getShortName());
}

void GenericEntity::_downloadCaptureFileSlot(Diamesh::FileTransferPrx transferProxy, QObject *obj)
{
    DownloadCaptureFileCookie *cookie = qobject_cast<DownloadCaptureFileCookie *>(obj);
    if (cookie && transferProxy) {
        FileTransferClient *client = new FileTransferClient(new Diamesh::FileTransferClient(transferProxy, PrivateSessionAccessor::instance()->adapter()), cookie->remoteFileName(), this);
        cookie->emitCompleted(client);
    }
}

void GenericEntity::_getNodeNameSlot(std::string nodeName)
{
    _handleNodeNameData(nodeName);
    emit statusChanged(this);
    refresh();
}

void GenericEntity::_getEntrySlot(Hns::EntityEntryPtr entry)
{
    Q_D(GenericEntity);
    _resetLocalEntry();

    QHash<int, QVariant> propertyHash;
    int propertyCount = metaObject()->propertyCount();
    for (int index = 0 ; index < propertyCount ; index++) {
        propertyHash.insert(index, metaObject()->property(index).read(this));
    }

    d->entry->setSourceCatalogItem(entry);
    d->entry->refreshProperties();
    PrivateSessionAccessor::instance()->session()->getEntryNodeInfoMap(d->entry->entityEntry(), this, SLOT(_getEntryNodeInfoMap(::Hns::NodeInfoMap)));
    emit entryChanged(this);

    QHashIterator<int, QVariant> iterator(propertyHash);
    while (iterator.hasNext()) {
        iterator.next();
        int index = iterator.key();
        QVariant property = iterator.value();
        if (property != metaObject()->property(index).read(this)) {
            metaObject()->property(index).notifySignal().invoke(this);
        }
    }
}

void GenericEntity::_getEntryNodeInfoMap(Hns::NodeInfoMap nodeMap)
{
    _handleEntryNodeInfoMapData(nodeMap);
    refresh();
}

void GenericEntity::_getRemoteDisplayInfoSlot(Hns::RemoteDisplayInfo remoteDisplayInfo, QObject *obj)
{
    Q_D(GenericEntity);
    RemoteDisplayInfoCookie *cookie = qobject_cast<RemoteDisplayInfoCookie*>(obj);
    d->remoteDisplayInfo = remoteDisplayInfo;
    d->remoteDisplayInfoLoaded = true;
    if (cookie)
        cookie->emitCompleted(remoteDisplayInfo);
}

void GenericEntity::_capturableInterfacesListChangedSlot(Diamesh::IntList interfaceList)
{
    _handleAvailablePlugsForCaptureData(interfaceList);
}

void GenericEntity::_captureJobFileDeleteSlot(QString fileName)
{
    emit removeCaptureFiles(fileName);
}

void GenericEntity::_entryChangedSlot()
{
    Q_D(GenericEntity);
    d->entityQtProxy->begin_getEntry(this, SLOT(_getEntrySlot(::Hns::EntityEntryPtr)));
}

void GenericEntity::_implementFromModelChangedSlot(bool isImplemented)
{
    _handleIsImplementFromModelData(isImplemented);
}

void GenericEntity::_interfaceUsedSlot(Ice::Int interfaceId, Ice::Int jobId)
{
    Q_D(GenericEntity);
    d->captureJobHash.insert(interfaceId, jobId);
    qDebug()<<tr("Entity \"%1\" capture job id \"%2\" using interface \"%3\".").arg(getShortName()).arg(jobId).arg(interfaceId);
    emit captureJobUseInterface(interfaceId, jobId);
}

void GenericEntity::_interfaceReleasedSlot(Ice::Int interfaceId, Ice::Int jobId)
{
    Q_D(GenericEntity);
    d->captureJobHash.remove(interfaceId);
    qDebug()<<tr("Entity \"%1\" capture job id \"%2\" released interface \"%3\".").arg(getShortName()).arg(jobId).arg(interfaceId);
    emit captureJobReleasedInterface(interfaceId, jobId);
}

void GenericEntity::_plugListChangedSlot(Hns::PlugMap plugMap)
{
    _handlePlugsData(plugMap);
    refresh();
}

void GenericEntity::_plugReleasedSlot(Ice::Long plugId)
{
    _setPlugUsed(plugId, false);
    refresh();
}

void GenericEntity::_plugUsedSlot(Ice::Long plugId)
{
    _setPlugUsed(plugId, true);
    refresh();
}

void GenericEntity::_statusChangedSlot(Ice::Int status, Ice::Long index)
{
    Q_D(GenericEntity);
    if (index > d->cptStatus) {
        d->cptStatus = index;
        _handleStatusData(Hns::Status(status), this, SLOT(_getNodeNameSlot(::std::string)));
    }
}

void GenericEntity::_allowedStatusChangedSlot()
{
    Q_D(GenericEntity);
    d->entityQtProxy->begin_getAllowedStatuses(this, SLOT(_loadAllowedStatusesSlot(::Hns::StatusList)));
}

void GenericEntity::_createCaptureJobSlot(Ice::Int jobId)
{
    if (jobId == -1)
        qCritical() << tr("Entity \"%1\" could not create capture job.").arg(getShortName());
    else
        qDebug() << tr("Entity \"%1\": capture job created with id: \"%2\".").arg(getShortName()).arg(jobId);
}

void GenericEntity::_cancelCaptureJobSlot(bool cancelSuccess)
{
    if (!cancelSuccess)
        qCritical() << tr("Entity \"%1\" could not cancel capture job.").arg(getShortName());
    else
        qDebug() << tr("Entity \"%1\" successfuly cancelled capture job.").arg(getShortName());
}

void GenericEntity::_load(Hns::EntityQPrx entityQtProxy, Topology *topology)
{
    Q_D(GenericEntity);
    d->entityQtProxy = entityQtProxy;
    d->topology = topology;

    d->remoteCallCount = 10;
    d->entityQtProxy->begin_getId(this, SLOT(_loadIdSlot(::Ice::Long)));
    d->entityQtProxy->begin_getEntry(this, SLOT(_loadEntrySlot(::Hns::EntityEntryPtr)));
    d->entityQtProxy->begin_getStatus(this, SLOT(_loadStatusSlot(::Hns::Status)));
    d->entityQtProxy->begin_getAllowedStatuses(this, SLOT(_loadAllowedStatusesSlot(::Hns::StatusList)));
    d->entityQtProxy->begin_getPlugs(this, SLOT(_loadPlugsSlot(::Hns::PlugMap)));
    d->entityQtProxy->begin_getAvailablePlugsForCapture(this, SLOT(_loadAvailablePlugsForCaptureSlot(::Diamesh::IntList)));
    d->entityQtProxy->begin_getFlags(this, SLOT(_loadFlagsSlot(::Ice::Int)));
    d->entityQtProxy->begin_isImplementFromModel(this, SLOT(_loadIsImplementFromModelSlot(bool)));
    d->entityQtProxy->begin_getTracer(this, SLOT(_loadTracerSlot(::Hns::TracerPrx)));
    d->entityQtProxy->begin_getRunningCaptureJobs(this, SLOT(_loadCaptureJobHashSlot(::Hns::IntIntMap)));
}

void GenericEntity::_callEnded()
{
    Q_D(GenericEntity);
    d->remoteCallCount--;
    _checkLoading();
}

void GenericEntity::_checkLoading()
{
    Q_D(GenericEntity);
    if (d->remoteCallCount == 0)
        _init();
}

void GenericEntity::_init()
{
    Q_D(GenericEntity);
    connect(d->entityQtProxy.data(), SIGNAL(capturableInterfacesListChanged(::Diamesh::IntList)),this,SLOT(_capturableInterfacesListChangedSlot(::Diamesh::IntList)));
    connect(d->entityQtProxy.data(), SIGNAL(captureJobFileDelete(QString)), this, SLOT(_captureJobFileDeleteSlot(QString)));
    connect(d->entityQtProxy.data(), SIGNAL(entryChanged()), this, SLOT(_entryChangedSlot()));
    connect(d->entityQtProxy.data(), SIGNAL(implementFromModelChanged(bool)), this, SLOT(_implementFromModelChangedSlot(bool)));
    connect(d->entityQtProxy.data(), SIGNAL(interfaceUsed(Ice::Int, Ice::Int)), this, SLOT(_interfaceUsedSlot(Ice::Int,Ice::Int)));
    connect(d->entityQtProxy.data(), SIGNAL(interfaceReleased(Ice::Int, Ice::Int)), this, SLOT(_interfaceReleasedSlot(Ice::Int,Ice::Int)));
    connect(d->entityQtProxy.data(), SIGNAL(plugListChanged(::Hns::PlugMap)), this, SLOT(_plugListChangedSlot(::Hns::PlugMap)));
    connect(d->entityQtProxy.data(), SIGNAL(plugReleased(Ice::Long)), this, SLOT(_plugReleasedSlot(Ice::Long)));
    connect(d->entityQtProxy.data(), SIGNAL(plugUsed(Ice::Long)), this, SLOT(_plugUsedSlot(Ice::Long)));
    connect(d->entityQtProxy.data(), SIGNAL(statusChanged(Ice::Int, Ice::Long)), this, SLOT(_statusChangedSlot(Ice::Int, Ice::Long)));
    connect(d->entityQtProxy.data(), SIGNAL(currentSnapshotChanged(::Hns::Snapshot)), this, SIGNAL(currentSnapshotChanged(::Hns::Snapshot)));
    connect(d->entityQtProxy.data(), SIGNAL(allowedStatusListChanged()), this, SLOT(_allowedStatusChangedSlot()));
    _fillPropertyInformation();
    emit loaded();
}

void GenericEntity::_loadEntryData(Hns::EntityEntryPtr entry)
{
    Q_D(GenericEntity);
    d->entry = EntryFactory::createEntityEntry(entry, this);
    PrivateSessionAccessor::instance()->session()->getEntryNodeInfoMap(d->entry->entityEntry(), this, SLOT(_loadEntryNodeInfoMapSlot(::Hns::NodeInfoMap)));
}

bool GenericEntity::_handleStatusData(Hns::Status status, QObject *receiver, const char *slot)
{
    bool result = true;
    Q_D(GenericEntity);
    if (d->status != status) {
        d->status = status;
        if (d->status == Hns::UNDEFINED || d->status == Hns::UNIMPLEMENTED) {
            d->nodeName.clear();
            d->remoteDisplayInfoLoaded = false;
        } else if (d->nodeName.isEmpty()) {
            result = false;
            d->entityQtProxy->begin_getNodeName(receiver, slot);
        }
    }
    emit statusChanged(this);
    return result;
}

void GenericEntity::_handleAvailablePlugsForCaptureData(Diamesh::IntList interfaceList)
{
    Q_D(GenericEntity);
    d->availableInterfacesForCapture = QList<int>::fromStdList(interfaceList);
    emit availableInterfacesForCaptureChanged(d->availableInterfacesForCapture);
}

void GenericEntity::_handleNodeNameData(std::string nodeName)
{
    Q_D(GenericEntity);
    d->nodeName = QString::fromStdString(nodeName);
}

void GenericEntity::_handleEntryNodeInfoMapData(Hns::NodeInfoMap nodeMap)
{
    Q_D(GenericEntity);
    d->nodeList.clear();
    QMap<std::string, Hns::NodeInfoPrx> nodeQMap(nodeMap);
    foreach(std::string const &nodeName, nodeQMap.keys())
        d->nodeList.append(QString::fromStdString(nodeName));
}

void GenericEntity::_handleIsImplementFromModelData(bool isImplement)
{
    Q_D(GenericEntity);
    d->isImplementedFromModel = isImplement;
}

void GenericEntity::_setPlugUsed(int plugId, bool used)
{
    Q_D(GenericEntity);
    if (d->plugMap.contains(plugId))
        d->plugMap[plugId].isPlugged = used;
}

void GenericEntity::_handlePlugsData(Hns::PlugMap plugMap)
{
    Q_D(GenericEntity);
    QMap<Ice::Long, Hns::Plug> plugQMap(plugMap);
    QMapIterator<Ice::Long, Hns::Plug> iterator(plugQMap);
    d->plugMap.clear();
    while(iterator.hasNext()) {
        iterator.next();
        d->plugMap.insert((int)iterator.key(), iterator.value());
    }
}

EntityEntry *GenericEntity::_currentEntry() const
{
    Q_D(const GenericEntity);
    if (d->localEntry)
        return d->localEntry;
    return d->entry;
}

EntityEntry *GenericEntity::_localEntry()
{
    Q_D(GenericEntity);
    if (!d->localEntry)
        d->localEntry = qobject_cast<HnsClient::EntityEntry *>(getEntry()->duplicateEntry(this));
    return d->localEntry;
}

void GenericEntity::_resetLocalEntry()
{
    Q_D(GenericEntity);
    if (d->localEntry)
        delete d->localEntry;
    d->localEntry = 0;
}

void HnsClient::GenericEntity::_fillPropertyInformation()
{
    Q_D(GenericEntity);
    d->propertyInformationMap.clear();
}


/*******************************************************************************
* WireEntity
*******************************************************************************/
WireEntity::WireEntity(Topology *parent)
    : GenericEntity(new WireEntityPrivate, parent)
{
}

WireEntity::~WireEntity()
{
}

/*! \fn destroy()
 *\brief TODO
 *\param
 */
void WireEntity::destroy()
{
    Q_D(WireEntity);
    d->topology->removeWire(d->entityId);
}

/*! \fn setLayout(Hns::WireLayoutPtr layout)
 *\brief TODO
 *\param layout
 */
void WireEntity::setLayout(Hns::WireLayoutPtr layout)
{
    Q_D(WireEntity);
    if (layout != d->layout && layout->id == getId()) {
        d->layout = layout;
        emit layoutChanged();
    }
}

/*! \fn getLayout()
 *\brief TODO
 */
Hns::WireLayoutPtr WireEntity::getLayout() const
{
    Q_D(const WireEntity);
    return d->layout;
}

/*! \fn fromEntityId() const
 *\brief TODO
 */
qlonglong WireEntity::fromEntityId() const
{
    Q_D(const WireEntity);
    return d->fromId;
}

/*! \fn toEntityId() const
 *\brief TODO
 */
qlonglong WireEntity::toEntityId() const
{
    Q_D(const WireEntity);
    return d->toId;
}

/*! \fn fromEntityPlugId() const
 *\brief TODO
 */
int WireEntity::fromEntityPlugId() const
{
    Q_D(const WireEntity);
    return d->fromPlug;
}

/*! \fn toEntityPlugId() const
 *\brief TODO
 */
int WireEntity::toEntityPlugId() const
{
    Q_D(const WireEntity);
    return d->toPlug;
}

/*! \fn *fromEntity()
 *\brief TODO
 */
GenericEntity *WireEntity::fromEntity() const
{
    Q_D(const WireEntity);
    return d->fromEntity;
}

/*! \fn *toEntity()
 *\brief TODO
 */
GenericEntity *WireEntity::toEntity() const
{
    Q_D(const WireEntity);
    return d->toEntity;
}

QString WireEntity::wireType() const
{
    return _currentWireEntry()->getType();
}

WireEntity::VisualLayout WireEntity::visualLayout() const
{
    return VisualLayout(getLayout()->numberAngles);
}

WireEntry *WireEntity::wireEntry() const
{
    return qobject_cast<HnsClient::WireEntry *>(getEntry());
}

void WireEntity::setWireType(const QString &value)
{
    if (value != wireType()) {
        HnsClient::WireEntry *updatedEntry = _localWireEntry();
        if (updatedEntry && updatedEntry->setType(value))
            updateEntry(updatedEntry);
    }
}

int WireEntity::lossLR() const
{
    return _currentWireEntry()->getLossLR();
}

int WireEntity::delayLR() const
{
    return _currentWireEntry()->getDelayLR();
}

int WireEntity::dupLR() const
{
    return _currentWireEntry()->getDupLR();
}

int WireEntity::bandwidthLR() const
{
    return _currentWireEntry()->getBandwidthLR();
}

int WireEntity::mtuLR() const
{
    return _currentWireEntry()->getMtuLR();
}

int WireEntity::lossRL() const
{
    return _currentWireEntry()->getLossRL();
}

int WireEntity::delayRL() const
{
    return _currentWireEntry()->getDelayRL();
}

int WireEntity::dupRL() const
{
    return _currentWireEntry()->getDupRL();
}

int WireEntity::bandwidthRL() const
{
    return _currentWireEntry()->getBandwidthRL();
}

int WireEntity::mtuRL() const
{
    return _currentWireEntry()->getMtuRL();
}

void WireEntity::setLossLR(int value)
{
    if (value != lossLR()) {
        HnsClient::WireEntry *updatedEntry = _localWireEntry();
        if (updatedEntry && updatedEntry->setLossLR(value))
            updateEntry(updatedEntry);
    }
}

void WireEntity::setDelayLR(int value)
{
    if (value != delayLR()) {
        HnsClient::WireEntry *updatedEntry = _localWireEntry();
        if (updatedEntry && updatedEntry->setDelayLR(value))
            updateEntry(updatedEntry);
    }
}

void WireEntity::setDupLR(int value)
{
    if (value != dupLR()) {
        HnsClient::WireEntry *updatedEntry = _localWireEntry();
        if (updatedEntry && updatedEntry->setDupLR(value))
            updateEntry(updatedEntry);
    }
}

void WireEntity::setBandwidthLR(int value)
{
    if (value != bandwidthLR()) {
        HnsClient::WireEntry *updatedEntry = _localWireEntry();
        if (updatedEntry && updatedEntry->setBandwidthLR(value))
            updateEntry(updatedEntry);
    }
}

void WireEntity::setMtuLR(int value)
{
    if (value != mtuLR()) {
        HnsClient::WireEntry *updatedEntry = _localWireEntry();
        if (updatedEntry && updatedEntry->setMtuLR(value))
            updateEntry(updatedEntry);
    }
}

void WireEntity::setLossRL(int value)
{
    if (value != lossRL()) {
        HnsClient::WireEntry *updatedEntry = _localWireEntry();
        if (updatedEntry && updatedEntry->setLossRL(value))
            updateEntry(updatedEntry);
    }
}

void WireEntity::setDelayRL(int value)
{
    if (value != delayRL()) {
        HnsClient::WireEntry *updatedEntry = _localWireEntry();
        if (updatedEntry && updatedEntry->setDelayRL(value))
            updateEntry(updatedEntry);
    }
}

void WireEntity::setDupRL(int value)
{
    if (value != dupRL()) {
        HnsClient::WireEntry *updatedEntry = _localWireEntry();
        if (updatedEntry && updatedEntry->setDupRL(value))
            updateEntry(updatedEntry);
    }
}

void WireEntity::setBandwidthRL(int value)
{
    if (value != bandwidthRL()) {
        HnsClient::WireEntry *updatedEntry = _localWireEntry();
        if (updatedEntry && updatedEntry->setBandwidthRL(value))
            updateEntry(updatedEntry);
    }
}

void WireEntity::setMtuRL(int value)
{
    if (value != mtuRL()) {
        HnsClient::WireEntry *updatedEntry = _localWireEntry();
        if (updatedEntry && updatedEntry->setMtuRL(value))
            updateEntry(updatedEntry);
    }
}

bool WireEntity::_loadWire(Hns::WireQPrx wireQtProxy, QString from, QString to, Topology *topology)
{
    Q_D(WireEntity);
    _splitWireEndInfo(from, d->fromId, d->fromPlug);
    _splitWireEndInfo(to, d->toId, d->toPlug);
    d->entityQtProxy = wireQtProxy;
    return _tryLoading(topology);
}

bool WireEntity::_loadWire(Hns::WireQPrx wireQtProxy, Topology *topology)
{
    Q_D(WireEntity);
    d->entityQtProxy = wireQtProxy;
    return _tryLoading(topology);
}

bool WireEntity::_tryLoading(Topology *topology)
{
    Q_D(WireEntity);
    bool result = false;
    _load(d->entityQtProxy, topology);
    result = true;
    return result;
}

void WireEntity::_splitWireEndInfo(QString endInfo, qlonglong &endId, int &endPort)
{
    QStringList infoSplit = endInfo.split(":");
    if (infoSplit.count() == 2) {
        endId = infoSplit.at(0).toLongLong();
        endPort = infoSplit.at(1).toInt();
    }
}

void WireEntity::_init()
{
    Q_D(WireEntity);
    WireEntry *wireEntry = qobject_cast<WireEntry*>(d->entry);
    if (wireEntry) {
        QList<Hns::TopologyLink> links = d->topology->getLinksByWireId(getId());
        foreach (const Hns::TopologyLink link, links) {
            QString str;
            int wirePort = QString::fromStdString(link.wireSide).split(':').last().toInt();
            if (!QString::fromStdString(link.entitySide).isEmpty()) {
                qlonglong entityId = QString::fromStdString(link.entitySide).split(':').first().toLongLong();
                int entityPortId = QString::fromStdString(link.entitySide).split(':').last().toInt();
                str = d->topology->getEntityById(entityId)->getShortName() + ":" + QString::number(entityPortId);
            }
            if (wirePort == 0)
                wireEntry->setFromEntity(str);
            else
                wireEntry->setToEntity(str);
        }
    }
    GenericEntity::_init();
}

void WireEntity::_toEntityEntryChangedSlot(GenericEntity *)
{
    Q_D(WireEntity);
    WireEntry *wireEntry = qobject_cast<WireEntry*>(d->entry);
    if (wireEntry) {
        wireEntry->setToEntity(toEntity()->getShortName() + " (" + QString::fromStdString(toEntity()->getPlugs()[toEntityPlugId()].name) + ")");
    }
}

void WireEntity::_fromEntityEntryChangedSlot(GenericEntity *)
{
    Q_D(WireEntity);
    WireEntry *wireEntry = qobject_cast<WireEntry*>(d->entry);
    if (wireEntry) {
        wireEntry->setFromEntity(fromEntity()->getShortName() + " (" + QString::fromStdString(fromEntity()->getPlugs()[fromEntityPlugId()].name) + ")");
    }
}

void WireEntity::_loadEntryData(Hns::EntityEntryPtr entry)
{
    Q_D(WireEntity);
    GenericEntity::_loadEntryData(entry);
    HnsClient::WireEntry *wireEntry = qobject_cast<HnsClient::WireEntry*>(d->entry);
    if (wireEntry && toEntity() && fromEntity()) {
        wireEntry->setToEntity(toEntity()->getShortName() + " (" + QString::fromStdString(toEntity()->getPlugs()[toEntityPlugId()].name) + ")");
        wireEntry->setFromEntity(fromEntity()->getShortName() + " (" + QString::fromStdString(fromEntity()->getPlugs()[fromEntityPlugId()].name) + ")");
    }
}

WireEntry *WireEntity::_currentWireEntry() const
{
    return qobject_cast<HnsClient::WireEntry *>(_currentEntry());
}

WireEntry *WireEntity::_localWireEntry()
{
    return qobject_cast<HnsClient::WireEntry *>(_localEntry());
}

void WireEntity::_fillPropertyInformation()
{
    Q_D(WireEntity);
    GenericEntity::_fillPropertyInformation();
    QMap<QString, QVariant> informationMap;

    //delay
    informationMap.insert(HnsClient::Constants::PROPERTY_MINIMUM, 0); //no max
    d->propertyInformationMap.insert("delayLR", QVariant(informationMap));
    d->propertyInformationMap.insert("delayRL", QVariant(informationMap));
    informationMap.clear();

    //loss
    informationMap.insert(HnsClient::Constants::PROPERTY_MINIMUM, 0);
    informationMap.insert(HnsClient::Constants::PROPERTY_MAXIMUM, 100);
    d->propertyInformationMap.insert("lossLR", QVariant(informationMap));
    d->propertyInformationMap.insert("lossRL", QVariant(informationMap));
    informationMap.clear();

    //dup
    informationMap.insert(HnsClient::Constants::PROPERTY_MINIMUM, 0);
    informationMap.insert(HnsClient::Constants::PROPERTY_MAXIMUM, 99);
    d->propertyInformationMap.insert("dupLR", QVariant(informationMap));
    d->propertyInformationMap.insert("dupRL", QVariant(informationMap));
    informationMap.clear();

    //bandwidth
    informationMap.insert(HnsClient::Constants::PROPERTY_MINIMUM, 0); //no max
    d->propertyInformationMap.insert("bandwidthLR", QVariant(informationMap));
    d->propertyInformationMap.insert("bandwidthRL", QVariant(informationMap));
    informationMap.clear();

    //MTU
    informationMap.insert(HnsClient::Constants::PROPERTY_MINIMUM, 0);
    informationMap.insert(HnsClient::Constants::PROPERTY_MAXIMUM, 1500);
    d->propertyInformationMap.insert("mtuLR", QVariant(informationMap));
    d->propertyInformationMap.insert("mtuRL", QVariant(informationMap));
}


/*******************************************************************************
* Entity
*******************************************************************************/
Entity::Entity(EntityPrivate *d, QObject *parent)
    : GenericEntity(d, parent)
{
}

Entity::~Entity()
{
}

void Entity::_setLayout(Hns::EntityLayoutPtr layout)
{
    Q_D(Entity);
    if (layout != d->layout && layout->id == getId()) {
        d->layout = layout;
        emit layoutChanged();
    }
}

/*! \fn getLayout()
 *\brief TODO
 */
Hns::EntityLayoutPtr Entity::getLayout() const
{
    Q_D(const Entity);
    return d->layout;
}


/*******************************************************************************
* DomainEntity
*******************************************************************************/
DomainEntity::DomainEntity(QObject *parent)
    : Entity(new DomainEntityPrivate, parent)
{
}

DomainEntity::~DomainEntity()
{
}

/*! \fn createCaptureJob(Hns::CaptureJobOptions options)
 *\brief TODO
 *\param options
 */
void DomainEntity::createCaptureJob(Hns::CaptureJobOptions options)
{
    Q_D(DomainEntity);
    d->entityQtProxy->begin_createCaptureJob(options, this, SLOT(_createCaptureJobSlot(::Ice::Int)));
}

/*! \fn cancelCaptureJob(Ice::Int jobId)
 *\brief TODO
 *\param jobId
 */
void DomainEntity::cancelCaptureJob(Ice::Int jobId)
{
    Q_D(DomainEntity);
    if (d->captureJobHash.values().contains(jobId))
        d->entityQtProxy->begin_cancelCaptureJob(jobId, this, SLOT(_cancelCaptureJobSlot(bool)));
    else
        qWarning() << tr("Entity \"%1\" does not have a capture job with id: \"%2\".").arg(getShortName()).arg(jobId);
}

void DomainEntity::createSnapshot(Hns::Snapshot snapshot, QObject *receiver, const char *slot)
{
    Q_D(DomainEntity);
    SnapshotCookie *cookie = new SnapshotCookie(QString::fromStdString(snapshot.name), receiver, slot);
    d->entityQtProxy->begin_createSnapshot(snapshot, this, SLOT(_createSnapshotSlot(bool, QObject *)), 0, 0, cookie);
}

void DomainEntity::revertSnapshot(QString snapshotName, QObject *receiver, const char *slot)
{
    Q_D(DomainEntity);
    SnapshotCookie *cookie = new SnapshotCookie(snapshotName, receiver, slot);
    d->entityQtProxy->begin_revertSnapshot(snapshotName.toStdString(), this, SLOT(_revertSnapshotSlot(bool, QObject *)), 0, 0, cookie);
}

void DomainEntity::deleteSnapshot(const QString &snapshotName, QObject *receiver, const char *slot)
{
    Q_D(DomainEntity);
    SnapshotCookie *cookie = new SnapshotCookie(snapshotName, receiver, slot);
    d->entityQtProxy->begin_deleteSnapshot(snapshotName.toStdString(), this, SLOT(_deleteSnapshotSlot(bool,QObject*)), 0, 0, cookie);
}

QString DomainEntity::currentSnapshotName()
{
    Q_D(DomainEntity);
    HnsClient::DomainEntry *domEntry = qobject_cast<HnsClient::DomainEntry *>(d->entry);
    return domEntry->getCurrentSnapshotName();
}

QStringList DomainEntity::snapshotNameList()
{
    Q_D(DomainEntity);
    QStringList snapshotList;
    HnsClient::DomainEntry *domainEntry = qobject_cast<HnsClient::DomainEntry *>(d->entry);
    if (!domainEntry)
        return snapshotList;
    foreach (Hns::Snapshot snapshot, QList<Hns::Snapshot>::fromStdList(domainEntry->getSnapshotList())) {
        snapshotList.append(QString::fromStdString(snapshot.name));
    }
    return snapshotList;
}

QList<Hns::Snapshot> DomainEntity::snapshotList()
{
    Q_D(DomainEntity);
    HnsClient::DomainEntry *domainEntry = qobject_cast<HnsClient::DomainEntry *>(d->entry);
    if (domainEntry)
        return QList<Hns::Snapshot>::fromStdList(domainEntry->getSnapshotList());
    return QList<Hns::Snapshot>();
}

void DomainEntity::snapshotbyName(QString name, QObject *receiver, const char *slot)
{
    Q_D(DomainEntity);
    d->entityQtProxy->begin_getSnapshot(name.toStdString(), receiver, slot);
}

int DomainEntity::usbRedirectionCount()
{
    return _currentDomainEntry()->getUsbRedirectionCount();
}

QList<Hns::MemoryDumpInfo> DomainEntity::memoryDumpInfos()
{
    Q_D(DomainEntity);
    return d->memoryDumpInfoList;
}

void DomainEntity::createMemoryDumpJob(const QString &name, QObject *receiver, const char *slot)
{
    Q_D(DomainEntity);
    MemoryDumpCookie *cookie = new MemoryDumpCookie(receiver, slot);
    d->domainQtProxy->begin_createMemoryDumpJob(name.toStdString(), this, SLOT(_createMemoryDumpJobSlot(::Ice::Int,QObject*)), 0, 0, cookie);
}

void DomainEntity::deleteMemoryDump(QString fileName, QObject *receiver, const char *slot)
{
    Q_D(DomainEntity);
    MemoryDumpCookie *cookie = new MemoryDumpCookie(receiver, slot);
    d->domainQtProxy->begin_deleteMemoryDump(fileName.toStdString(), this, SLOT(_deleteMemoryDumpSlot(QObject*)), 0, 0, cookie);
}

void DomainEntity::downloadMemoryDump(const QString &remoteFileName, const QString &localFilename, QObject *receiver, const char *slot)
{
    Q_D(DomainEntity);
    DownloadMemoryDumpCookie *cookie = new DownloadMemoryDumpCookie(receiver, slot);
    cookie->setRemoteFileName(remoteFileName);
    cookie->setLocalFileName(localFilename);
    d->domainQtProxy->begin_downloadMemoryDump(remoteFileName.toStdString(), this, SLOT(_downloadMemoryDumpSlot(::Diamesh::FileTransferPrx,QObject*)), 0, 0, cookie);
}

QString DomainEntity::domainType() const
{
    return _currentDomainEntry()->getType();
}

int DomainEntity::memory() const
{
    return _currentDomainEntry()->getMemory() / 1024;
}

int DomainEntity::vCpu() const
{
    return _currentDomainEntry()->getVCpu();
}

void DomainEntity::_createSnapshotSlot(bool result, QObject *obj)
{
    SnapshotCookie *cookie = qobject_cast<SnapshotCookie *>(obj);
    if (!cookie) {
        return;
    }
    cookie->emitCompleted(result);
}

void DomainEntity::_revertSnapshotSlot(bool result, QObject *obj)
{
    SnapshotCookie *cookie = qobject_cast<SnapshotCookie *>(obj);
    if (!cookie) {
        return;
    }
    cookie->emitCompleted(result);
}

void DomainEntity::_deleteSnapshotSlot(bool result, QObject *obj)
{
    SnapshotCookie *cookie = qobject_cast<SnapshotCookie *>(obj);
    if (!cookie) {
        return;
    }
    cookie->emitCompleted(result);
}

void DomainEntity::_createMemoryDumpJobSlot(Ice::Int jobId, QObject *obj)
{
    MemoryDumpCookie *cookie = qobject_cast<MemoryDumpCookie *>(obj);
    if (!cookie)
        return;
    cookie->emitCompleted(jobId);
}

void DomainEntity::_deleteMemoryDumpSlot(QObject *obj)
{
    MemoryDumpCookie *cookie = qobject_cast<MemoryDumpCookie *>(obj);
    if (!cookie)
        return;
    cookie->emitCompleted(0);
}

void DomainEntity::_getMemoryDumpInfoListSlot(Hns::MemoryDumpInfoList list)
{
    Q_D(DomainEntity);
    d->memoryDumpInfoList = QList<Hns::MemoryDumpInfo>::fromStdList(list);
    _callEnded();
}

void DomainEntity::_memoryDumpAddedSlot(Hns::MemoryDumpInfo info)
{
    Q_D(DomainEntity);
    d->memoryDumpInfoList.append(info);
    emit memoryDumpAdded(info);
}

void DomainEntity::_memoryDumpRemovedSlot(QString fileName)
{
    Q_D(DomainEntity);
    foreach (const Hns::MemoryDumpInfo &info, d->memoryDumpInfoList) {
        if (info.fileName == fileName.toStdString())
            d->memoryDumpInfoList.removeAll(info);
    }
    emit memoryDumpDeleted(fileName);
}

void DomainEntity::_downloadMemoryDumpSlot(Diamesh::FileTransferPrx fileTransferPrx, QObject *obj)
{
    DownloadMemoryDumpCookie *cookie = qobject_cast<DownloadMemoryDumpCookie *>(obj);
    if (!cookie || !fileTransferPrx)
        return;
    FileTransferClient *client = new FileTransferClient(new Diamesh::FileTransferClient(fileTransferPrx, PrivateSessionAccessor::instance()->adapter()), cookie->remoteFileName(), this);
    client->setLocalAbsoluteFileName(cookie->localFileName());
    cookie->emitCompleted(client);
}

void DomainEntity::_load(Hns::EntityQPrx entityQtProxy, Topology *topology)
{
    Q_D(DomainEntity);
    Entity::_load(entityQtProxy, topology);
    d->domainQtProxy = Hns::DomainQPrx::uncheckedCast(d->entityQtProxy);
    if (d->domainQtProxy) {
        ++d->remoteCallCount;
        d->domainQtProxy->begin_getMemoryDumpInfoList(this, SLOT(_getMemoryDumpInfoListSlot(::Hns::MemoryDumpInfoList)));
    }
}

void DomainEntity::_init()
{
    Q_D(DomainEntity);
    if (d->domainQtProxy) {
        connect(d->domainQtProxy.data(), SIGNAL(memoryDumpAdded(::Hns::MemoryDumpInfo)), this, SLOT(_memoryDumpAddedSlot(::Hns::MemoryDumpInfo)));
        connect(d->domainQtProxy.data(), SIGNAL(memoryDumpDeleted(QString)), this, SLOT(_memoryDumpRemovedSlot(QString)));
    }
    Entity::_init();
}

DomainEntry *DomainEntity::_currentDomainEntry() const
{
    return qobject_cast<HnsClient::DomainEntry *>(_currentEntry());
}

/*******************************************************************************
* SwitchEntity
*******************************************************************************/
SwitchEntity::SwitchEntity(QObject *parent)
    : Entity(new SwitchEntityPrivate, parent)
{
}

SwitchEntity::~SwitchEntity()
{
}

/*! \fn createCaptureJob(Hns::CaptureJobOptions options)
 *\brief TODO
 *\param
 */
void SwitchEntity::createCaptureJob(Hns::CaptureJobOptions options)
{
    Q_D(SwitchEntity);
    d->entityQtProxy->begin_createCaptureJob(options, this, SLOT(_createCaptureJobSlot(::Ice::Int)));
}

/*! \fn cancelCaptureJob(Ice::Int jobId)
 *\brief TODO
 *\param jobId
 */
void SwitchEntity::cancelCaptureJob(Ice::Int jobId)
{
    Q_D(SwitchEntity);
    if (d->captureJobHash.values().contains(jobId))
        d->entityQtProxy->begin_cancelCaptureJob(jobId, this, SLOT(_cancelCaptureJobSlot(bool)));
    else
        qWarning() << tr("Entity \"%1\" does not have a capture job with id: \"%2\".").arg(getShortName()).arg(jobId);
}

QString SwitchEntity::switchType() const
{
    return _currentSwitchEntry()->getType();
}

int SwitchEntity::portCount() const
{
    return _currentSwitchEntry()->getPorts();
}

bool SwitchEntity::isHub() const
{
    return _currentSwitchEntry()->getIsHub();
}

bool SwitchEntity::stpEnabled() const
{
    return _currentSwitchEntry()->getStpEnabled();
}

int SwitchEntity::portMirroring() const
{
    return _currentSwitchEntry()->getPortMirroring();
}

void SwitchEntity::setSwitchType(const QString &value)
{
    if (value != switchType()) {
        HnsClient::SwitchEntry *updatedEntry = _localSwitchEntry();
        if (updatedEntry && updatedEntry->setSwitchType(value))
            updateEntry(updatedEntry);
    }
}

void SwitchEntity::setPortCount(int value)
{
    if (value != portCount()) {
        HnsClient::SwitchEntry *updatedEntry = _localSwitchEntry();
        if (updatedEntry && updatedEntry->setPortCount(value))
            updateEntry(updatedEntry);
    }
}

void SwitchEntity::setIsHub(bool value)
{
    if (value != isHub()) {
        HnsClient::SwitchEntry *updatedEntry = _localSwitchEntry();
        if (updatedEntry && updatedEntry->setIsHub(value))
            updateEntry(updatedEntry);
    }
}

void SwitchEntity::setStpEnabled(bool value)
{
    if (value != stpEnabled()) {
        HnsClient::SwitchEntry *updatedEntry = _localSwitchEntry();
        if (updatedEntry && updatedEntry->setStpEnabled(value))
            updateEntry(updatedEntry);
    }
}

void SwitchEntity::setPortMirroring(int value)
{
    if (value != portMirroring()) {
        HnsClient::SwitchEntry *updatedEntry = _localSwitchEntry();
        if (updatedEntry && updatedEntry->setPortMirroring(value))
            updateEntry(updatedEntry);
    }
}

SwitchEntry *SwitchEntity::switchEntry() const
{
    return qobject_cast<HnsClient::SwitchEntry *>(getEntry());
}

SwitchEntry *SwitchEntity::_currentSwitchEntry() const
{
    return qobject_cast<HnsClient::SwitchEntry *>(_currentEntry());
}

SwitchEntry *SwitchEntity::_localSwitchEntry()
{
    return qobject_cast<HnsClient::SwitchEntry *>(_localEntry());
}

void HnsClient::SwitchEntity::_fillPropertyInformation()
{
    Q_D(SwitchEntity);
    Entity::_fillPropertyInformation();
    QMap<QString, QVariant> informationMap;
    informationMap.insert(HnsClient::Constants::PROPERTY_MINIMUM, 4);
    informationMap.insert(HnsClient::Constants::PROPERTY_MAXIMUM, 64);
    d->propertyInformationMap.insert("portCount", QVariant(informationMap));
}


/*******************************************************************************
* HybridNetcardEntity
*******************************************************************************/
HybridNetcardEntity::HybridNetcardEntity(QObject *parent)
    : Entity(new HybridNetcardEntityPrivate, parent)
{
}

HybridNetcardEntity::~HybridNetcardEntity()
{
}

/*! \fn createCaptureJob(Hns::CaptureJobOptions options)
 *\brief TODO
 *\param options
 */
void HybridNetcardEntity::createCaptureJob(Hns::CaptureJobOptions options)
{
    Q_D(HybridNetcardEntity);
    d->entityQtProxy->begin_createCaptureJob(options, this, SLOT(_createCaptureJobSlot(::Ice::Int)));
}

/*! \fn cancelCaptureJob(Ice::Int jobId)
 *\brief TODO
 *\param jobId
 */
void HybridNetcardEntity::cancelCaptureJob(Ice::Int jobId)
{
    Q_D(HybridNetcardEntity);
    if (d->captureJobHash.values().contains(jobId))
        d->entityQtProxy->begin_cancelCaptureJob(jobId, this, SLOT(_cancelCaptureJobSlot(bool)));
    else
        qWarning() << tr("Entity \"%1\" does not have a capture job with id: \"%2\".").arg(getShortName()).arg(jobId);
}

QMap<QString, bool> HybridNetcardEntity::getMacAddresses() const
{
    Q_D(const HybridNetcardEntity);
    return d->macAddresses;
}

void HybridNetcardEntity::_loadGetMacAddressesSlot(Hns::MacAddressMap addressesMap)
{
    Q_D(HybridNetcardEntity);
    QMap<std::string, bool> macAddressesMap(addressesMap);
    d->macAddresses.clear();
    QMapIterator<std::string, bool> iterator(macAddressesMap);
    while(iterator.hasNext()) {
        iterator.next();
        d->macAddresses.insert(QString::fromStdString(iterator.key()), iterator.value());
    }
    _callEnded();
}

void HybridNetcardEntity::_newMacAddrSlot(QString macAddress)
{
    Q_D(HybridNetcardEntity);
    if (!d->macAddresses.contains(macAddress)) {
        d->macAddresses.insert(macAddress, true);
        emit macAddressAdded(macAddress);
    }
}

void HybridNetcardEntity::_macAddrGoneSlot(QString macAddress)
{
    Q_D(HybridNetcardEntity);
    if (d->macAddresses.contains(macAddress)) {
        d->macAddresses.remove(macAddress);
        emit macAddressRemoved(macAddress);
    }
}

void HybridNetcardEntity::_macBecameActiveSlot(QString macAddress)
{
    Q_D(HybridNetcardEntity);
    if (!d->macAddresses.contains(macAddress))
        d->macAddresses.insert(macAddress, true);
    else
        d->macAddresses[macAddress] = true;
    emit macAddressChanged(macAddress, true);
}

void HybridNetcardEntity::_macBecameInactiveSlot(QString macAddress)
{
    Q_D(HybridNetcardEntity);
    if (!d->macAddresses.contains(macAddress))
        d->macAddresses.insert(macAddress, false);
    else
        d->macAddresses[macAddress] = false;
    emit macAddressChanged(macAddress, false);
}

void HybridNetcardEntity::_load(Hns::EntityQPrx entityQtProxy, Topology *topology)
{
    Q_D(HybridNetcardEntity);
    Entity::_load(entityQtProxy, topology);
    d->hybridQtProxy = Hns::HybridNetcardQPrx::uncheckedCast(d->entityQtProxy);
    if (d->hybridQtProxy) {
        d->remoteCallCount++;
        d->hybridQtProxy->begin_getMacAddresses(this, SLOT(_loadGetMacAddressesSlot(::Hns::MacAddressMap)));
    }
}

void HybridNetcardEntity::_init()
{
    Q_D(HybridNetcardEntity);
    if (d->hybridQtProxy) {
        connect(d->hybridQtProxy.data(), SIGNAL(newMacAddr(QString)), this, SLOT(_newMacAddrSlot(QString)));
        connect(d->hybridQtProxy.data(), SIGNAL(macAddrGone(QString)), this, SLOT(_macAddrGoneSlot(QString)));
        connect(d->hybridQtProxy.data(), SIGNAL(macBecameActive(QString)), this, SLOT(_macBecameActiveSlot(QString)));
        connect(d->hybridQtProxy.data(), SIGNAL(macBecameInactive(QString)), this, SLOT(_macBecameInactiveSlot(QString)));
    }
    Entity::_init();
}

bool HybridNetcardEntity::_handleStatusData(Hns::Status status, QObject *receiver, const char *slot)
{
    Q_D(HybridNetcardEntity);
    if (d->status != status) {
        if (status == Hns::UNDEFINED || status == Hns::UNIMPLEMENTED || status == Hns::STOPPED) {
            foreach(const QString &macAddress, d->macAddresses.keys())
                emit macAddressRemoved(macAddress);
            d->macAddresses.clear();
        }
    }
    return Entity::_handleStatusData(status, receiver, slot);
}


/*******************************************************************************
* WifiAPEntity
*******************************************************************************/
WifiAPEntity::WifiAPEntity(QObject *parent)
    : Entity(new WifiAPEntityPrivate, parent)
{
}

WifiAPEntity::~WifiAPEntity()
{
}

/*! \fn createCaptureJob(Hns::CaptureJobOptions options)
 *\brief TODO
 *\param options
 */
void WifiAPEntity::createCaptureJob(Hns::CaptureJobOptions options)
{
    Q_D(WifiAPEntity);
    d->entityQtProxy->begin_createCaptureJob(options, this, SLOT(_createCaptureJobSlot(::Ice::Int)));
}

/*! \fn cancelCaptureJob(Ice::Int jobId);
 *\brief TODO
 *\param jobId
 */
void WifiAPEntity::cancelCaptureJob(Ice::Int jobId)
{
    Q_D(WifiAPEntity);
    if (d->captureJobHash.values().contains(jobId))
        d->entityQtProxy->begin_cancelCaptureJob(jobId, this, SLOT(_cancelCaptureJobSlot(bool)));
    else
        qWarning() << tr("Entity \"%1\" does not have a capture job with id: \"%2\".").arg(getShortName()).arg(jobId);
}

/*! \fn connectedPeers() const
 *\brief TODO
 *\param
 */
QStringList WifiAPEntity::connectedPeers() const
{
    Q_D(const WifiAPEntity);
    return d->connectedPeers;
}

/*! \fn getSSID() const
 *\brief TODO
 *\param
 */
QString WifiAPEntity::getSSID() const
{
    Q_D(const WifiAPEntity);
    HnsClient::WifiAPEntry *wifiEntry = qobject_cast<HnsClient::WifiAPEntry *>(d->entry);
    if (!wifiEntry)
        return QString();
    return wifiEntry->getSsid();
}

void WifiAPEntity::_loadConnectedPeersSlot(Diamesh::StringList connectedPeerList)
{
    Q_D(WifiAPEntity);
    d->connectedPeers.clear();
    foreach (std::string peer, connectedPeerList)
        d->connectedPeers.append(QString::fromStdString(peer));
    _callEnded();
}

void WifiAPEntity::_peerJoinedSlot(QString peer)
{
    Q_D(WifiAPEntity);
    if (!d->connectedPeers.contains(peer)) {
        d->connectedPeers.append(peer);
        emit peerJoined(peer);
    }
}

void WifiAPEntity::_peerLeftSlot(QString peer)
{
    Q_D(WifiAPEntity);
    if (d->connectedPeers.contains(peer)) {
        d->connectedPeers.removeAll(peer);
        emit peerLeft(peer);
    }
}

void WifiAPEntity::_load(Hns::EntityQPrx entityQtProxy, Topology *topology)
{
    Q_D(WifiAPEntity);
    Entity::_load(entityQtProxy, topology);
    d->wifiQtProxy = Hns::WifiAPQPrx::uncheckedCast(d->entityQtProxy);
    if (d->wifiQtProxy) {
        d->remoteCallCount++;
        d->wifiQtProxy->begin_connectedPeers(this, SLOT(_loadConnectedPeersSlot(::Diamesh::StringList)));
    }
}

void WifiAPEntity::_init()
{
    Q_D(WifiAPEntity);
    if (d->wifiQtProxy) {
        connect(d->wifiQtProxy.data(), SIGNAL(peerJoined(QString)), this, SLOT(_peerJoinedSlot(QString)));
        connect(d->wifiQtProxy.data(), SIGNAL(peerLeft(QString)), this, SLOT(_peerLeftSlot(QString)));
    }
    Entity::_init();
}

bool WifiAPEntity::_handleStatusData(Hns::Status status, QObject *receiver, const char *slot)
{
    Q_D(WifiAPEntity);
    if (d->status != status) {
        if (status == Hns::UNDEFINED || status == Hns::UNIMPLEMENTED || status == Hns::STOPPED) {
            foreach(const QString &peer, d->connectedPeers)
                emit peerLeft(peer);
            d->connectedPeers.clear();
        }
    }
    return Entity::_handleStatusData(status, receiver, slot);
}


/*******************************************************************************
* DynamipsRouterEntity
*******************************************************************************/
DynamipsRouterEntity::DynamipsRouterEntity(QObject *parent)
    : Entity(new DynamipsRouterEntityPrivate, parent)
{
}

DynamipsRouterEntity::~DynamipsRouterEntity()
{
}

/*! \fn createCaptureJob(Hns::CaptureJobOptions options);
 *\brief TODO
 *\param options
 */
void DynamipsRouterEntity::createCaptureJob(Hns::CaptureJobOptions options)
{
    Q_D(DynamipsRouterEntity);
    d->entityQtProxy->begin_createCaptureJob(options, this, SLOT(_createCaptureJobSlot(::Ice::Int)));
}

/*! \fn cancelCaptureJob(Ice::Int jobId)
 *\brief TODO
 *\param jobId
 */
void DynamipsRouterEntity::cancelCaptureJob(Ice::Int jobId)
{
    Q_D(DynamipsRouterEntity);
    if (d->captureJobHash.values().contains(jobId))
        d->entityQtProxy->begin_cancelCaptureJob(jobId, this, SLOT(_cancelCaptureJobSlot(bool)));
    else
        qWarning() << tr("Entity \"%1\" does not have a capture job with id: \"%2\".").arg(getShortName()).arg(jobId);
}

/*! \fn fixIdle(QObject *receiver, const char *slot)
 *\brief TODO
 *\param receiver
 *\param slot
 */
void DynamipsRouterEntity::fixIdle(QObject *receiver, const char *slot)
{
    Q_D(DynamipsRouterEntity);
    Hns::DynamipsRouterQPrx qprx = Hns::DynamipsRouterQPrx::uncheckedCast(d->entityQtProxy);
    qprx->begin_fixIdle(receiver, slot);
}

/*! \fn extractConfiguration(QObject *object, const char *slot)
 *\brief TODO
 *\param object
 *\param slot
 */
void DynamipsRouterEntity::extractConfiguration(QObject *object, const char *slot)
{
    Q_D(DynamipsRouterEntity);
    Hns::DynamipsRouterQPrx qprx = Hns::DynamipsRouterQPrx::uncheckedCast(d->entityQtProxy);
    qprx->begin_extractConfiguration(object, slot);
}

/*! \fn pushConfiguration(const QString &configuration)
 *\brief TODO
 *\param configuration
 */
void DynamipsRouterEntity::pushConfiguration(const QString &configuration)
{
    Q_D(DynamipsRouterEntity);
    Hns::DynamipsRouterQPrx qprx = Hns::DynamipsRouterQPrx::uncheckedCast(d->entityQtProxy);
    qprx->begin_pushConfiguration(configuration.toStdString(), this, SIGNAL(pushConfigurationSignal()));
}

/*******************************************************************************
* TopologyGateEntity
*******************************************************************************/
TopologyGateEntity::TopologyGateEntity(QObject *parent)
    : Entity(new TopologyGateEntityPrivate, parent)
{
}

TopologyGateEntity::~TopologyGateEntity()
{
}

/*! \fn createCaptureJob(Hns::CaptureJobOptions options);
 *\brief TODO
 *\param options
 */
void TopologyGateEntity::createCaptureJob(Hns::CaptureJobOptions options)
{
    Q_D(TopologyGateEntity);
    d->entityQtProxy->begin_createCaptureJob(options, this, SLOT(_createCaptureJobSlot(::Ice::Int)));
}

/*! \fn cancelCaptureJob(Ice::Int jobId)
 *\brief TODO
 *\param jobId
 */
void TopologyGateEntity::cancelCaptureJob(Ice::Int jobId)
{
    Q_D(TopologyGateEntity);
    if (d->captureJobHash.values().contains(jobId))
        d->entityQtProxy->begin_cancelCaptureJob(jobId, this, SLOT(_cancelCaptureJobSlot(bool)));
    else
        qWarning() << tr("Entity \"%1\" does not have a capture job with id: \"%2\".").arg(getShortName()).arg(jobId);
}

} // namespace HnsClient
