/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// hynesim hnsclient headers
#include "libhnsclient/entityentrycatalog.h"
#include "libhnsclient/private/entityentrycatalog_p.h"
#include "libhnsclient/entry.h"
#include "libhnsclient/private/privatesessionaccessor.h"
#include "libhnsclient/session.h"
#include "libhnsclient/jobmanager.h"
#include "resourcemanager.h"

namespace HnsClient {

/*******************************************************************************
* GetEntityEntryCookie
*******************************************************************************/
GetEntityEntryCookie::GetEntityEntryCookie(qlonglong entryId)
    : QObject()
    , _entryId(entryId)
{
}

GetEntityEntryCookie::~GetEntityEntryCookie()
{
}

qlonglong GetEntityEntryCookie::entryId() const
{
    return _entryId;
}

/*******************************************************************************
* AddFolderCookie
*******************************************************************************/
AddFolderCookie::AddFolderCookie(QString folderName)
    : QObject()
    , _folderName(folderName)
{
}

AddFolderCookie::~AddFolderCookie()
{
}

QString AddFolderCookie::getFolderName()
{
    return _folderName;
}


/*******************************************************************************
* UpdateFolderCookie
*******************************************************************************/
UpdateFolderCookie::UpdateFolderCookie(Hns::FolderPtr folder)
    : QObject()
    , _folder(folder)
{
}

UpdateFolderCookie::~UpdateFolderCookie()
{
}

Hns::FolderPtr UpdateFolderCookie::folder()
{
    return _folder;
}

/*******************************************************************************
* DeleteFolderCookie
*******************************************************************************/
DeleteFolderCookie::DeleteFolderCookie(QString folderName, qlonglong Id)
    : QObject()
    , _folderName(folderName)
    , _Id(Id)
{
}

DeleteFolderCookie::~DeleteFolderCookie()
{
}

QString DeleteFolderCookie::getFolderName()
{
    return _folderName;
}

qlonglong DeleteFolderCookie::getId()
{
    return _Id;
}


/*******************************************************************************
* UpdateEntityCookie
*******************************************************************************/
UpdateEntryCookie::UpdateEntryCookie(Hns::EntityEntryPtr entry)
    : QObject()
    , _entry(entry)
{
}

UpdateEntryCookie::~UpdateEntryCookie()
{
}

Hns::EntityEntryPtr UpdateEntryCookie::entityEntry() const
{
    return _entry;
}

QString UpdateEntryCookie::getEntryName() const
{
    return QString::fromStdString(_entry->shortName);
}

qlonglong UpdateEntryCookie::getEntryId() const
{
    return _entry->id;
}


/*******************************************************************************
* DeleteEntityCookie
*******************************************************************************/
DeleteEntryCookie::DeleteEntryCookie(QString entryName, qlonglong entryId)
    : QObject()
    , _entryName(entryName)
    , _entryId(entryId)
{
}

DeleteEntryCookie::~DeleteEntryCookie()
{
}

QString DeleteEntryCookie::getEntryName() const
{
    return _entryName;
}

qlonglong DeleteEntryCookie::getEntryId() const
{
    return _entryId;
}


/*******************************************************************************
* CloneEntityEntryCookie
*******************************************************************************/
CloneEntryCookie::CloneEntryCookie(EntityEntry *sourceEntry)
    : QObject()
    , _entryName(sourceEntry->getShortName())
    , _entryId(sourceEntry->getID())
{
}

CloneEntryCookie::~CloneEntryCookie()
{
}

QString CloneEntryCookie::getEntryName()
{
    return _entryName;
}

qlonglong CloneEntryCookie::getEntryId()
{
    return _entryId;
}


/*******************************************************************************
* BuildModelFromEntityEntryCookie
*******************************************************************************/
BuildModelFromEntityEntryCookie::BuildModelFromEntityEntryCookie(EntityEntry *sourceEntry)
    : QObject()
    , _entryName(sourceEntry->getShortName())
    , _entryId(sourceEntry->getID())
{
}

BuildModelFromEntityEntryCookie::~BuildModelFromEntityEntryCookie()
{
}

QString BuildModelFromEntityEntryCookie::getEntryName()
{
    return _entryName;
}

Ice::Long BuildModelFromEntityEntryCookie::getEntryID()
{
    return _entryId;
}


/*******************************************************************************
* DiscardSaveStateCookie
*******************************************************************************/
DiscardSaveStateCookie::DiscardSaveStateCookie(EntityEntry *sourceEntry)
    : QObject()
    , _entryName(sourceEntry->getShortName())
    , _entryID(sourceEntry->getID())
{
}

DiscardSaveStateCookie::~DiscardSaveStateCookie()
{
}

Ice::Long DiscardSaveStateCookie::getEntryID()
{
    return _entryID;
}

QString DiscardSaveStateCookie::getEntryName()
{
    return _entryName;
}


/*******************************************************************************
* GetEntityEntryUpdateCookie
*******************************************************************************/
GetEntityEntryUpdateCookie::GetEntityEntryUpdateCookie(qlonglong entryId)
    : QObject()
    , _entryId(entryId)
{
}

GetEntityEntryUpdateCookie::~GetEntityEntryUpdateCookie()
{
}

qlonglong GetEntityEntryUpdateCookie::getEntryId()
{
    return _entryId;
}


/*******************************************************************************
* GetEntityEntryUpdateCookie
*******************************************************************************/
GetFolderUpdateCookie::GetFolderUpdateCookie(qlonglong entryId)
    : QObject()
    , _entryId(entryId)
{
}

GetFolderUpdateCookie::~GetFolderUpdateCookie()
{
}

qlonglong GetFolderUpdateCookie::getFolderId()
{
    return _entryId;
}

/*******************************************************************************
* GetUnusedUUIDCookie
*******************************************************************************/
GetUnusedUUIDCookie::GetUnusedUUIDCookie(qlonglong entityId, QObject *receiver, const char *slot)
    : QObject()
    , _entityId(entityId)
{
    connect(this, SIGNAL(completed(QString,qlonglong)), receiver, slot);
}

GetUnusedUUIDCookie::~GetUnusedUUIDCookie()
{
}

void GetUnusedUUIDCookie::emitCompleted(QString uuid)
{
    emit completed(uuid, _entityId);
    deleteLater();
}

/*******************************************************************************
* EntityEntryCatalogPrivate
*******************************************************************************/
EntityEntryCatalogPrivate::EntityEntryCatalogPrivate()
    : CatalogPrivate()
{
    catalogQtProxy = 0;
    topologyManagerQtProxy = 0;
}

EntityEntryCatalogPrivate::~EntityEntryCatalogPrivate()
{
    qDeleteAll(entityHash);
    entityHash.clear();
    qDeleteAll(folderHash);
    folderHash.clear();
    catalogQtProxy = 0;
    topologyManagerQtProxy = 0;
}


/*******************************************************************************
* EntityCatalog
*******************************************************************************/
EntityEntryCatalog::EntityEntryCatalog(QObject *parent)
    : Catalog(new EntityEntryCatalogPrivate, parent)
{
}

EntityEntryCatalog::~EntityEntryCatalog()
{
}

/*! \fn getFolders() const
 *\brief TODO
 */
QHash<qlonglong, FolderEntry *> EntityEntryCatalog::getFolders() const
{
    Q_D(const EntityEntryCatalog);
    return d->folderHash;
}

/*! \fn getEntities() const
 *\brief TODO
 */
QHash<qlonglong, EntityEntry *> EntityEntryCatalog::getEntities() const
{
    Q_D(const EntityEntryCatalog);
    return d->entityHash;
}

/*! \fn *folderById(qlonglong folderId)
 *\brief TODO
 *\param folderId
 */
FolderEntry *EntityEntryCatalog::folderById(qlonglong folderId) const
{
    Q_D(const EntityEntryCatalog);
    if (d->folderHash.contains(folderId))
        return d->folderHash.value(folderId);
    return 0;
}

/*! \fn getFolderChildren(qlonglong parentFolderId = 0)
 *\brief TODO
 *\param parentFolderId
 */
QList<FolderEntry *> EntityEntryCatalog::getFolderChildren(qlonglong parentFolderId) const
{
    Q_D(const EntityEntryCatalog);
    QList<FolderEntry *> resultList;
    foreach(FolderEntry *folder, d->folderHash.values())
        if (folder->getParentID() == parentFolderId)
            resultList.append(folder);
    return resultList;
}

/*! \fn *entityEntryByUUID(QString uuid)
 *\brief TODO
 *\param uuid
 */
EntityEntry *EntityEntryCatalog::entityEntryByUUID(QString uuid) const
{
    Q_D(const EntityEntryCatalog);
    foreach(EntityEntry *entry, d->entityHash.values()) {
        if (entry->getUUID() == uuid)
            return entry;
    }
    return nullptr;
}

/*! \fn *entityEntryById(qlonglong entityEntryId)
 *\brief TODO
 *\param entityEntryId
 */
EntityEntry *EntityEntryCatalog::entityEntryById(qlonglong entityEntryId) const
{
    Q_D(const EntityEntryCatalog);
    if (d->entityHash.contains(entityEntryId))
        return d->entityHash.value(entityEntryId);
    return 0;
}

/*! \fn getEntityChildren(qlonglong parentFolderId = 0)
 *\brief TODO
 *\param parentFolderId
 */
QList<EntityEntry *> EntityEntryCatalog::getEntityChildren(qlonglong parentFolderId) const
{
    Q_D(const EntityEntryCatalog);
    QList<EntityEntry *> resultList;
    if (d->folderHash.contains(parentFolderId)) {
        foreach (EntityEntry *entry, getEntities()) {
            if (entry->getParentID() == parentFolderId)
                resultList.append(entry);
        }
    }
    return resultList;
}

/*! \fn addFolder(FolderEntry *folder)
 *\brief TODO
 *\param folder
 */
void EntityEntryCatalog::addFolder(FolderEntry *folder)
{
    Q_D(EntityEntryCatalog);
    AddFolderCookie *cookie = new AddFolderCookie(folder->getShortName());
    d->catalogQtProxy->begin_addFolder(folder->folderEntry(), this, SLOT(_addFolderSlot(::Ice::Long, QObject *)), 0, 0, cookie);
}

/*! \fn updateFolder(FolderEntry *folder)
 *\brief TODO
 *\param folder
 */
void EntityEntryCatalog::updateFolder(FolderEntry *folder)
{
    Q_D(EntityEntryCatalog);
    qlonglong folderID = folder->getID();
    if (!d->folderHash.contains(folderID)) {
        qDebug() << tr("Folder not in cache. Could not request update.");
        return;
    }
    _setFolderSyncing(folderID, Syncing);
    UpdateFolderCookie *cookie = new UpdateFolderCookie(folder->folderEntry());

    d->catalogQtProxy->begin_setACLsForCatalogItem(folder->getID(), folder->aclList(), this, SLOT(_setACLsForFolderSlot(bool,QObject*)), this, SLOT(_updateACLMethodFailedSlot(QSharedPointer<Ice::Exception>,QObject*)), cookie);
    delete folder;
}

/*! \fn deleteFolder(qlonglong folderId)
 *\brief TODO
 *\param folderId
 */
void EntityEntryCatalog::deleteFolder(qlonglong folderId)
{
    Q_D(EntityEntryCatalog);
    if (!d->folderHash.contains(folderId)) {
        qDebug() << tr("Folder not in cache. Could not request delete.");
        return;
    }
    _setFolderSyncing(folderId, Syncing);
    d->waitForDelete.insert(folderId, d->folderHash[folderId]->getParentID());

    DeleteFolderCookie *cookie = new DeleteFolderCookie(d->folderHash[folderId]->getShortName(), folderId);
    d->catalogQtProxy->begin_deleteFolder(folderId, this, SLOT(_deleteFolderSlot(bool, QObject *)), 0, 0, cookie);
}

/*! \fn moveFolder(qlonglong folderId, qlonglong newParentId)
 *\brief TODO
 *\param fodlerId
 *\param newParentId
 */
bool EntityEntryCatalog::moveFolder(qlonglong folderId, qlonglong newParentId)
{
    Q_D(EntityEntryCatalog);
    if (!d->folderHash.contains(folderId)) {
        qDebug() << tr("Folder not in cache. Could not move.");
        return false;
    }
    if (newParentId != 0 && !d->folderHash.contains(newParentId)) {
        qDebug() << tr("Destination folder not in cache. Could not move");
        return false;
    }

    FolderEntry * folderEntry = d->folderHash[folderId];

    qlonglong parentID = folderEntry->getParentID();
    if (parentID == newParentId) {
        qDebug() << tr("Operation forbidden: Can not move category in its own parent category.");
        return false;
    }
    Hns::FolderPtr folderData = folderEntry->folderEntry();
    folderData->parentId = newParentId;
    FolderEntry * updatedEntry = folderEntry->duplicateFolder(this);
    _setWaitForFolder(folderId);
    updateFolder(updatedEntry);
    return true;
}

/*! \fn updateEntityEntry(EntityEntry *updatedEntity, qlonglong focusParentId = 0)
 *\brief TODO
 *\param updateEntity
 *\param focusParentId
 */
void EntityEntryCatalog::updateEntityEntry(EntityEntry *updatedEntity, qlonglong focusParentId)
{
    Q_D(EntityEntryCatalog);
    qlonglong entityID = updatedEntity->getID();
    if (!d->entityHash.contains(entityID)) {
        qDebug() << tr("Entity not in cache. Could not request update");
        return;
    }
    if (focusParentId)
        _setWaitForEntity(entityID, focusParentId);
    _setEntitySyncing(entityID, Syncing);
    UpdateEntryCookie *cookie = new UpdateEntryCookie(updatedEntity->entityEntry());
    d->catalogQtProxy->begin_setACLsForCatalogItem(updatedEntity->getID(), updatedEntity->aclList(), this, SLOT(_setACLsForEntrySlot(bool, QObject*)), this, SLOT(_updateACLMethodFailedSlot(QSharedPointer<Ice::Exception>,QObject*)), cookie);
    delete updatedEntity;
}

/*! \fn cloneEntityEntry(qlonglong entityEntryId)
 *\brief TODO
 *\param entityEntryId
 */
void EntityEntryCatalog::cloneEntityEntry(qlonglong entityEntryId, int flags)
{
    Q_D(EntityEntryCatalog);
    if (d->entityHash.contains(entityEntryId)) {
        _setEntitySyncing(entityEntryId, Syncing);
        CloneEntryCookie *cookie = new CloneEntryCookie(d->entityHash[entityEntryId]);
        d->catalogQtProxy->begin_cloneEntityEntry(entityEntryId, flags, this, SLOT(_cloneRequestCompletedSlot(::Ice::Int, QObject *)),
                                                  this, SLOT(_cloneRequestFailedSlot(QSharedPointer<Ice::Exception>,QObject*)),
                                                  cookie);
    } else
        qCritical() << tr("Entity %1 could not be found in cache. Clone operation aborted.").arg(entityEntryId);
}

/*! \fn buildModelFromEntityEntry(qlonglong entityEntryId)
 *\brief TODO
 *\param entityEntryId
 */
void EntityEntryCatalog::buildModelFromEntityEntry(qlonglong entityEntryId)
{
    Q_D(EntityEntryCatalog);
    if (d->entityHash.contains(entityEntryId)) {
        _setEntitySyncing(entityEntryId, Syncing);
        BuildModelFromEntityEntryCookie *cookie = new BuildModelFromEntityEntryCookie(d->entityHash[entityEntryId]);
        d->catalogQtProxy->begin_buildModelFromEntityEntry(entityEntryId, this, SLOT(_buildModelCompletedSlot(::Ice::Long, QObject *)), 0, 0, cookie);
    } else
        qCritical() << tr("Entity %1 could not be found in cache. No model could be built.").arg(entityEntryId);
}

/*! \fn discardSaveState(qlonglong entityEntryId)
 *\brief TODO
 *\param entityEntryId
 */
void EntityEntryCatalog::discardSaveState(qlonglong entityEntryId)
{
    Q_D(EntityEntryCatalog);
    if (d->entityHash.contains(entityEntryId)) {
        _setEntitySyncing(entityEntryId, Syncing);
        DiscardSaveStateCookie *cookie = new DiscardSaveStateCookie(d->entityHash[entityEntryId]);
        d->catalogQtProxy->begin_discardSaveState(entityEntryId, this, SLOT(_discardSaveStateCompletedSlot(bool, QObject *)), 0, 0, cookie);
    } else
        qCritical() << tr("Entity %1 could not be found in cache. The save state was not discarded.").arg(entityEntryId);
}

/*! \fn deleteItemList(QList<qlonglong> folderIDList, QHash<qlonglong, QList<qlonglong> > entityPositionHash)
 *\brief TODO
 *\param folderIDList
 *\param entityPositionHash
 */
void EntityEntryCatalog::deleteItemList(QList<qlonglong> folderIDList, QHash<qlonglong, QList<qlonglong> > entityPositionHash)
{
    deleteEntityList(entityPositionHash);
    deleteFolderList(folderIDList);
}

/*! \fn deleteFolderList(QList<qlonglong> folderIDList)
 *\brief TODO
 *\param folderIDList
 */
void EntityEntryCatalog::deleteFolderList(QList<qlonglong> folderIDList)
{
    foreach(qlonglong itemID, folderIDList)
        deleteFolder(itemID);
}

/*! \fn deleteEntityList(QHash<qlonglong, QList<qlonglong> > entityPositionHash)
 *\brief TODO
 *\param entityPositionHash
 */
void EntityEntryCatalog::deleteEntityList(QHash<qlonglong, QList<qlonglong> > entityPositionHash)
{
    QMutableHashIterator<qlonglong, QList<qlonglong> >iterator(entityPositionHash);
    while (iterator.hasNext()) {
        iterator.next();
        deleteEntity(iterator.key(), iterator.value().first());
    }
}

/*! \fn deleteEntity(qlonglong entityId, qlonglong parentId = 0)
 *\brief TODO
 *\param entityId
 *\param parentId
 */
void EntityEntryCatalog::deleteEntity(qlonglong entityId, qlonglong parentId)
{
    Q_D(EntityEntryCatalog);
    if (!d->entityHash.contains(entityId)) {
        qDebug() << tr("Entity not in cache. Could not request delete.");
        return;
    }
    _setEntitySyncing(entityId, Syncing);
    entityEntryById(entityId)->setLocked(true);
    d->waitForDelete.insert(entityId, parentId);

    DeleteEntryCookie *cookie = new DeleteEntryCookie(d->entityHash[entityId]->getShortName(), entityId);
    d->catalogQtProxy->begin_deleteEntry(d->entityHash[entityId]->entityEntry(), this, SLOT(_deleteEntitySlot(bool,QObject*)), 0, 0, cookie);
}

/*! \fn EntityEntryCatalog::setCustomGraphic(qlonglong entityEntryId, const QString &fileName)
 *\brief TODO
 *\param entityEntryId
 *\param &filename
 */
void EntityEntryCatalog::setCustomGraphic(qlonglong entityEntryId, const QString &fileName)
{
    EntityEntry *source = entityEntryById(entityEntryId);
    if (!source)
        return;

    EntityEntry *updatedEntry = dynamic_cast<EntityEntry*>(source->duplicateEntry());
    QString fileKey = Session::instance()->resourceManager()->uploadResource(fileName);
    updatedEntry->entityEntry()->customGraphic = fileKey.toStdString();
    updateEntityEntry(updatedEntry);
}

void EntityEntryCatalog::getUnusedUUID(qlonglong entityId, QObject *receiver, const char *slot)
{
    Q_D(EntityEntryCatalog);
    GetUnusedUUIDCookie *cookie = new GetUnusedUUIDCookie(entityId, receiver, slot);
    d->catalogQtProxy->begin_getUnusedUUID(this, SLOT(_getUnusedUUIDSlot(::std::string,QObject*)), 0, 0, cookie);
}

void EntityEntryCatalog::addEntry(Hns::EntryPtr entry)
{
    Q_D(EntityEntryCatalog);
    d->catalogQtProxy->begin_addEntry(entry, this, SLOT(_addEntrySlot(::Ice::Long)));
}

void EntityEntryCatalog::_load(Hns::EntityCatalogQPrx catalogQtPrx, Hns::TopologyManagerQPrx topologyManager)
{
    Q_D(EntityEntryCatalog);
    d->catalogQtProxy = catalogQtPrx;
    d->topologyManagerQtProxy = topologyManager;
    d->remoteCallCount += 2;
    d->catalogQtProxy->begin_getCloneJobs(this, SLOT(_loadCloneJobsSlot(::Hns::LongIntMap)));
    d->catalogQtProxy->begin_getFolders(0, this, SLOT(_loadGetFoldersSlot(::Diamesh::LongList)));
    _loadEntries();
}

void EntityEntryCatalog::_callEnded()
{
    Q_D(EntityEntryCatalog);
    d->remoteCallCount--;
    _checkLoading();
}

void EntityEntryCatalog::_checkLoading()
{
    Q_D(EntityEntryCatalog);
    if (d->remoteCallCount == 0)
        _init();
}

void EntityEntryCatalog::_init()
{
    Q_D(EntityEntryCatalog);
    connect(d->catalogQtProxy.data(), SIGNAL(entryAdded(Ice::Long)), this, SLOT(_entryAddedSlot(Ice::Long)));
    connect(d->catalogQtProxy.data(), SIGNAL(entryChanged(Ice::Long)), this, SLOT(_entryChangedSlot(Ice::Long)));
    connect(d->catalogQtProxy.data(), SIGNAL(entryRemoved(Ice::Long)), this, SLOT(_entryRemovedSlot(Ice::Long)));
    connect(d->catalogQtProxy.data(), SIGNAL(entryLocked(Ice::Long, Ice::Int)), this, SLOT(_entryLockedSlot(Ice::Long, Ice::Int)));
    connect(d->catalogQtProxy.data(), SIGNAL(entryUnlocked(Ice::Long)), this, SLOT(_entryUnlockedSlot(Ice::Long)));
    connect(d->catalogQtProxy.data(), SIGNAL(entryACLsChanged(Ice::Long, Diamesh::ACLList)), this, SLOT(_entryACLsChangedSlot(Ice::Long, Diamesh::ACLList)));
    connect(d->catalogQtProxy.data(), SIGNAL(folderAdded(Ice::Long)), this, SLOT(_folderAddedSlot(Ice::Long)));
    connect(d->catalogQtProxy.data(), SIGNAL(folderChanged(Ice::Long)), this, SLOT(_folderChangedSlot(Ice::Long)));
    connect(d->catalogQtProxy.data(), SIGNAL(folderRemoved(Ice::Long)), this, SLOT(_folderRemovedSlot(Ice::Long)));
    connect(d->catalogQtProxy.data(), SIGNAL(folderACLsChanged(Ice::Long, Diamesh::ACLList)), this, SLOT(_folderACLsChangedSlot(Ice::Long,Diamesh::ACLList)));
    connect(d->catalogQtProxy.data(), SIGNAL(cloneJobStarted(Ice::Long,Ice::Int)), this, SLOT(_cloneJobStartedSlot(Ice::Long,Ice::Int)));
    emit loaded();
}

Hns::CatalogQPrx EntityEntryCatalog::_catalogProxy() const
{
    Q_D(const EntityEntryCatalog);
    return d->catalogQtProxy;
}

void EntityEntryCatalog::_parseEntryList(Diamesh::LongList entryList)
{
    Q_D(EntityEntryCatalog);
    d->remoteCallCount += entryList.size();
    foreach(Ice::Long entityID, entryList)
        d->catalogQtProxy->begin_getEntityEntry(entityID, this, SLOT(_loadGetEntityEntrySlot(::Hns::EntityEntryPtr)));
    _checkLoading();
}

void EntityEntryCatalog::_updateEntryLockedStatus(Ice::Long entryId, bool isLocked)
{
    Q_D(EntityEntryCatalog);
    if (d->entityHash.contains(entryId)) {
        d->entityHash[entryId]->setLocked(isLocked);
        if (isLocked)
            d->entityHash[entryId]->setLockType(lockType(entryId));
        if (isLocked && lockType(entryId) == Hns::LOCK_TYPE_NORMAL) {
            EntityEntryInfoUpdater * updater = new EntityEntryInfoUpdater(d->entityHash[entryId], d->topologyManagerQtProxy, d->entityHash[entryId]);
            connect(updater, SIGNAL(statusChanged(qlonglong)), this, SLOT(_entryModifiedByUpdater(qlonglong)));
            d->entityHash[entryId]->setInfoUpdater(updater);
        } else
            d->entityHash[entryId]->setInfoUpdater(0);

        emit entityUpdated(d->entityHash.value(entryId));
    }
}

void EntityEntryCatalog::_setFolderSyncing(qlonglong folderId, SynchronizationStatus status)
{
    Q_D(EntityEntryCatalog);
    if (d->folderHash.contains(folderId)) {
        FolderEntry * entry = d->folderHash[folderId];
        entry->setSyncStatus(status);
        emit folderSyncingStateChanged(folderId, status);
    }
}

void EntityEntryCatalog::_setEntitySyncing(qlonglong entityId, SynchronizationStatus status)
{
    Q_D(EntityEntryCatalog);
    if (d->entityHash.contains(entityId)) {
        EntityEntry * entry = d->entityHash[entityId];
        entry->setSyncStatus(status);
        emit entitySyncingStateChanged(entityId, status);
    }
}

void EntityEntryCatalog::_setWaitForFolder(qlonglong folderId)
{
    Q_D(EntityEntryCatalog);
    d->waitForFolder.append(folderId);
}

void EntityEntryCatalog::_checkWaitForFolder(qlonglong folderId)
{
    Q_D(EntityEntryCatalog);
    if (d->waitForFolder.contains(folderId)) {
        d->waitForFolder.removeAll(folderId);
        emit focusOnFolder(folderId);
    }
}

void EntityEntryCatalog::_setWaitForEntity(qlonglong entityId, qlonglong parentId)
{
    Q_D(EntityEntryCatalog);
    d->waitForEntity.insert(entityId, parentId);
}

void EntityEntryCatalog::_checkWaitForEntity(qlonglong entityId)
{
    Q_D(EntityEntryCatalog);
    if (d->waitForEntity.contains(entityId)) {
        qlonglong parentID = d->waitForEntity.take(entityId);
        emit focusOnEntry(entityId, parentID);
    }
}

bool EntityEntryCatalog::_handleFolderData(Hns::FolderPtr folder, QObject *receiver, const char *slot)
{
    Q_D(EntityEntryCatalog);

    if (!folder)
        return false;

    qlonglong folderId = folder->id;
    if (folderId == 0)
        return false;

    if (d->folderHash.contains(folderId))
        return false;

    FolderEntry * entry = new FolderEntry(folder, this);
    d->folderHash.insert(folderId, entry);
    GetACLsForEntryCookie *cookie = new GetACLsForEntryCookie(folderId);
    d->catalogQtProxy->begin_getACLsForCatalogItem(folderId, this, SLOT(_getACLsForFolderSlot(::Diamesh::ACLList,QObject*)), 0, 0, cookie);
    emit folderAdded(entry);
    _checkWaitForFolder(folderId);

    // Populate branch with child folders
    d->catalogQtProxy->begin_getFolders(folderId, receiver, slot);
    return true;
}

bool EntityEntryCatalog::_handleEntityEntryData(Hns::EntityEntryPtr entityEntry, qlonglong entryId)
{
    Q_D(EntityEntryCatalog);
    if (!entityEntry) {
        d->loadingEntryList.removeAll(entryId);
        return false;
    }

    if (d->entityHash.contains(entryId)) {
        qWarning() << tr("Entry with ID %1 already exists, could not create").arg(QString::number(entryId));
        d->loadingEntryList.removeAll(entryId);
        return false;
    }
    EntityEntry *entityEntryObj = EntryFactory::createEntityEntry(entityEntry, this);
    GetACLsForEntryCookie *cookie = new GetACLsForEntryCookie(entryId);
    d->catalogQtProxy->begin_getACLsForCatalogItem(entryId, this, SLOT(_getACLsForEntrySlot(::Diamesh::ACLList, QObject *)), 0, 0, cookie);
    d->entityHash.insert(entryId, entityEntryObj);
    if (entityEntryObj->isLocked() && lockType(entryId) == Hns::LOCK_TYPE_NORMAL) {
        EntityEntryInfoUpdater * updater = new EntityEntryInfoUpdater(entityEntryObj, d->topologyManagerQtProxy, entityEntryObj);
        connect(updater, SIGNAL(statusChanged(qlonglong)), this, SLOT(_entryModifiedByUpdater(qlonglong)));
        entityEntryObj->setInfoUpdater(updater);
    }
    d->loadingEntryList.removeAll(entityEntryObj->getID());
    emit entityAdded(entityEntryObj);

    int jobId = d->cloneJobToEntryHash.key(entryId, -1);
    if ( jobId != -1)
        PrivateSessionAccessor::instance()->session()->jobManager()->getJob(jobId, this, SLOT(_getJobSlot(HnsClient::Job*)));

    return true;
}

void EntityEntryCatalog::_parseCloneJobData(qlonglong entryId, qlonglong jobId)
{
    Q_D(EntityEntryCatalog);
    d->cloneJobToEntryHash.insert(jobId, entryId);
    if (d->entityHash.contains(entryId)) {
        PrivateSessionAccessor::instance()->session()->jobManager()->getJob(jobId, this, SLOT(_getJobSlot(HnsClient::Job*)));
    }
}

void EntityEntryCatalog::_removeFolderAndContent(qlonglong folderId)
{
    Q_D(EntityEntryCatalog);
    foreach (FolderEntry *folder, d->folderHash) {
        if (folder->getParentID() == folderId) {
            _removeFolderAndContent(folder->getID());
        }
    }

    FolderEntry *folderToDelete = d->folderHash.take(folderId);
    if (!folderToDelete)
        return;
    foreach (Entry *entry, d->entityHash) {
        if (entry->getParentID() == folderId) {
            emit entityRemoved(entry->getID());
        }
    }
    emit folderRemoved(folderToDelete->getID());
    delete folderToDelete;
}

void EntityEntryCatalog::_loadCloneJobsSlot(Hns::LongIntMap jobsMap)
{
    QMap<Ice::Long, Ice::Int> jobs(jobsMap);
    QMapIterator<Ice::Long, Ice::Int> iterator(jobs);
    while (iterator.hasNext()) {
        iterator.next();
        _parseCloneJobData(iterator.key(), iterator.value());
    }
    _callEnded();
}

void EntityEntryCatalog::_loadGetFoldersSlot(::Diamesh::LongList folderList)
{
    Q_D(EntityEntryCatalog);
    d->remoteCallCount += folderList.size();
    foreach(Ice::Long folderID, folderList)
        d->catalogQtProxy->begin_getFolder(folderID, this, SLOT(_loadGetFolderSlot(::Hns::FolderPtr)));
    _callEnded();
}

void EntityEntryCatalog::_loadGetFolderSlot(::Hns::FolderPtr folder)
{
    _handleFolderData(folder, this, SLOT(_loadGetFoldersSlot(::Diamesh::LongList)));
}

void EntityEntryCatalog::_loadGetEntityEntrySlot(Hns::EntityEntryPtr entityEntry)
{
    _handleEntityEntryData(entityEntry, entityEntry->id);
    _callEnded();
}

void EntityEntryCatalog::_addFolderSlot(::Ice::Long Id, QObject *obj)
{
    Q_D(EntityEntryCatalog);
    AddFolderCookie *cookie = qobject_cast<AddFolderCookie *>(obj);
    if (cookie) {
        if (Id == 0)
            qCritical() << tr("Could not add new folder with name %1").arg(cookie->getFolderName());
        else {
            if (d->folderHash.contains(Id))
                emit focusOnFolder(Id);
            else
                _setWaitForFolder(Id);
        }
    }
}

void EntityEntryCatalog::_updateFolderSlot(bool ok, QObject *obj)
{
    UpdateFolderCookie *cookie = qobject_cast<UpdateFolderCookie *>(obj);
    if (cookie) {
        if (!ok) {
            _setFolderSyncing(cookie->folder()->id, Error);
            qCritical() << tr("Folder '%1' update failed.").arg(QString::fromStdString(cookie->folder()->shortName));
        }
        else {
            _setFolderSyncing(cookie->folder()->id, Synced);
        }
        delete cookie;
    }
}

void EntityEntryCatalog::_deleteFolderSlot(bool ok, QObject *obj)
{
    Q_D(EntityEntryCatalog);
    DeleteFolderCookie *cookie = qobject_cast<DeleteFolderCookie *>(obj);
    if (cookie) {
        if (!ok) {
            qCritical() << tr("Could not delete folder '%1'").arg(cookie->getFolderName());
            _setFolderSyncing(cookie->getId(), Error);
            if (d->waitForDelete.contains(cookie->getId())) {
                d->waitForDelete.remove(cookie->getId());
                emit focusOnFolder(cookie->getId());
            }
        }
        else {
            if (d->waitForDelete.contains(cookie->getId())){
                focusOnFolder(d->waitForDelete.take(cookie->getId()));
            }
        }
    }
}

void EntityEntryCatalog::_updateEntitySlot(bool result, QObject *obj)
{
    Q_D(EntityEntryCatalog);
    UpdateEntryCookie *cookie = qobject_cast<UpdateEntryCookie *>(obj);
    if (cookie) {
        qlonglong entityEntryId = cookie->getEntryId();
        if (result) {
            _setEntitySyncing(entityEntryId, Synced);
            if (d->waitForDelete.contains(entityEntryId))
                emit focusOnFolder(d->waitForDelete.take(entityEntryId));
        } else {
            _setEntitySyncing(entityEntryId, Error);
            qCritical() << tr("Entity '%1' update failed.").arg(cookie->getEntryName());
        }
        delete cookie;
    }
}

void EntityEntryCatalog::_deleteEntitySlot(bool result, QObject *obj)
{
    Q_D(EntityEntryCatalog);
    DeleteEntryCookie *cookie = qobject_cast<DeleteEntryCookie *>(obj);
    if (cookie) {
        qlonglong entityEntryId = cookie->getEntryId();
        if (result) {
            d->waitForDelete.remove(entityEntryId);
        } else {
            _setEntitySyncing(entityEntryId, Synced);
            qCritical() << tr("Entity '%1' could not be deleted.").arg(cookie->getEntryName());
            entityEntryById(entityEntryId)->setLocked(false);
        }
    }
}

void EntityEntryCatalog::_setACLsForEntrySlot(bool result, QObject *obj)
{
    Q_D(EntityEntryCatalog);
    UpdateEntryCookie *cookie = qobject_cast<UpdateEntryCookie *>(obj);
    if (cookie) {
        qlonglong entityEntryId = cookie->getEntryId();
        if (result) {
            d->catalogQtProxy->begin_updateEntry(cookie->entityEntry(), this, SLOT(_updateEntitySlot(bool,QObject*)), 0, 0, cookie);
        } else {
            _setEntitySyncing(entityEntryId, Error);
            qCritical() << tr("Entity '%1' update failed.").arg(cookie->getEntryName());
            delete cookie;
        }
    }
}

void EntityEntryCatalog::_setACLsForFolderSlot(bool result, QObject *obj)
{
    Q_D(EntityEntryCatalog);
    UpdateFolderCookie *cookie = qobject_cast<UpdateFolderCookie *>(obj);
    if (cookie) {
        if (result)
            d->catalogQtProxy->begin_updateFolder(cookie->folder(), this, SLOT(_updateFolderSlot(bool, QObject *)), 0, 0, cookie);
        else {
            _setFolderSyncing(cookie->folder()->id, Error);
            qCritical() << tr("Folder '%1' update failed").arg(cookie->folder()->id);
            delete cookie;
        }
    }
}

void EntityEntryCatalog::_cloneRequestCompletedSlot(::Ice::Int jobId, QObject *obj)
{
    CloneEntryCookie *cookie = qobject_cast<CloneEntryCookie *>(obj);
    if (cookie){
        if (jobId == 0) {
            _setEntitySyncing(cookie->getEntryId(), Error);
            qCritical() << tr("Could not start clone operation for entity '%1'").arg(cookie->getEntryName());
        }
        delete cookie;
    }
}

void EntityEntryCatalog::_cloneRequestFailedSlot(QSharedPointer<Ice::Exception> exception, QObject *obj)
{
    QSharedPointer<Hns::HnsError> hnsError = qSharedPointerDynamicCast<Hns::HnsError>(exception);
    CloneEntryCookie *cookie = qobject_cast<CloneEntryCookie *>(obj);
    if (hnsError && cookie) {
        _setEntitySyncing(cookie->getEntryId(), Synced);
        qCritical() << tr("Unable to clone entry %1: %2").arg(cookie->getEntryName()).arg(QString::fromStdString(hnsError->message));
        delete cookie;
    }
}

void EntityEntryCatalog::_buildModelCompletedSlot(::Ice::Long newId, QObject *obj)
{
    BuildModelFromEntityEntryCookie *cookie = qobject_cast<BuildModelFromEntityEntryCookie *>(obj);
    if (cookie){
        if (newId < 1) {
            _setEntitySyncing(cookie->getEntryID(), Error);
            qCritical() << tr("Could not build model from entity '%1'").arg(cookie->getEntryName());
        } else {
            _setEntitySyncing(cookie->getEntryID(), Synced);
            emit focusOnEntry(newId, -1);
        }
    }
}

void EntityEntryCatalog::_discardSaveStateCompletedSlot(bool result, QObject *obj)
{
    DiscardSaveStateCookie *cookie = qobject_cast<DiscardSaveStateCookie *>(obj);
    if (cookie){
        if (!result) {
            _setEntitySyncing(cookie->getEntryID(), Error);
            qCritical() << tr("Could not discard the save state from entity '%1'").arg(cookie->getEntryName());
        } else {
            _setEntitySyncing(cookie->getEntryID(), Synced);
        }
    }
}

void EntityEntryCatalog::_getEntityEntrySlot(Hns::EntityEntryPtr entityEntry, QObject *obj)
{
    GetEntityEntryCookie *cookie = qobject_cast<GetEntityEntryCookie *>(obj);
    if (cookie) {
        _handleEntityEntryData(entityEntry, cookie->entryId());
        cookie->deleteLater();
    }
}

void EntityEntryCatalog::_getEntityEntryUpdateSlot(Hns::EntityEntryPtr entityEntry, QObject *obj)
{
    Q_D(EntityEntryCatalog);
    GetEntityEntryUpdateCookie *cookie = qobject_cast<GetEntityEntryUpdateCookie *>(obj);
    if (cookie) {
        qlonglong entryId = cookie->getEntryId();
        if (entityEntry) {
            EntityEntry *entryObj = entityEntryById(entryId);
            if (entryObj) {
                entryObj->setSourceCatalogItem(entityEntry);
                entryObj->refreshProperties();
                emit entityUpdated(entryObj);
                _checkWaitForEntity(entryId);
            } else if (!d->loadingEntryList.contains(entryId)) {
                d->loadingEntryList.append(entryId);
                _handleEntityEntryData(entityEntry, entryId);
            }
        }
        else
            _entryRemovedSlot(entryId);
    }
}

void EntityEntryCatalog::_getFoldersSlot(::Diamesh::LongList folderList)
{
    Q_D(EntityEntryCatalog);
    foreach(const qlonglong &folderId, folderList)
        d->catalogQtProxy->begin_getFolder(folderId, this, SLOT(_getFolderSlot(::Hns::FolderPtr)));
    _callEnded();
}

void EntityEntryCatalog::_getFolderSlot(Hns::FolderPtr folder)
{
    _handleFolderData(folder, this, SLOT(_getFoldersSlot(::Diamesh::LongList)));
}

void EntityEntryCatalog::_getFolderUpdateSlot(Hns::FolderPtr folder, QObject *obj)
{
    GetFolderUpdateCookie *cookie = qobject_cast<GetFolderUpdateCookie *>(obj);
    if (cookie) {
        qlonglong folderID = cookie->getFolderId();
        if (!folder) {
            _removeFolderAndContent(folderID);
        } else if (folder->id != 0) {
            FolderEntry *folderObj = folderById(folderID);
            if (folderObj) {
                folderObj->setSourceCatalogItem(folder);
                emit folderUpdated(folderObj);
                _checkWaitForFolder(folderID);
            } else {
                _getFolderSlot(folder);
            }
        } else {
            _folderRemovedSlot(folderID);
        }
        delete cookie;
    }
}

/******************************************************************************
  Slots connected to servers signals
  ****************************************************************************/
void EntityEntryCatalog::_entryAddedSlot(Ice::Long entryId)
{
    Q_D(EntityEntryCatalog);
    if (!d->loadingEntryList.contains(entryId)) {
        d->loadingEntryList.append(entryId);
        GetEntityEntryCookie *cookie = new GetEntityEntryCookie(entryId);
        d->catalogQtProxy->begin_getEntityEntry(entryId, this, SLOT(_getEntityEntrySlot(::Hns::EntityEntryPtr, QObject*)), 0, 0, cookie);
    }
}

void EntityEntryCatalog::_entryChangedSlot(Ice::Long entryId)
{
    Q_D(EntityEntryCatalog);
    GetEntityEntryUpdateCookie *cookie = new GetEntityEntryUpdateCookie(entryId);
    d->catalogQtProxy->begin_getEntityEntry(entryId, this, SLOT(_getEntityEntryUpdateSlot(::Hns::EntityEntryPtr, QObject *)), 0, 0, cookie);
}

void EntityEntryCatalog::_entryRemovedSlot(Ice::Long entryId)
{
    Q_D(EntityEntryCatalog);
    EntityEntry * entry = d->entityHash.take(entryId);
    if (entry) {
        emit entityRemoved(entryId);
        delete entry;
    }
}

void EntityEntryCatalog::_entryACLsChangedSlot(Ice::Long entryId, Diamesh::ACLList aclList)
{
    EntityEntry *entry = entityEntryById(entryId);
    if (entry)
        entry->_updateACLList(aclList);
    else
        _entryAddedSlot(entryId);
}

void EntityEntryCatalog::_folderAddedSlot(Ice::Long folderId)
{
    Q_D(EntityEntryCatalog);
    d->catalogQtProxy->begin_getFolder(folderId, this, SLOT(_getFolderSlot(::Hns::FolderPtr)));
}

void EntityEntryCatalog::_folderChangedSlot(Ice::Long folderId)
{
    Q_D(EntityEntryCatalog);
    GetFolderUpdateCookie *cookie = new GetFolderUpdateCookie(folderId);
    d->catalogQtProxy->begin_getFolder(folderId, this, SLOT(_getFolderUpdateSlot(::Hns::FolderPtr, QObject *)), 0, 0, cookie);
}

void EntityEntryCatalog::_folderRemovedSlot(Ice::Long folderId)
{
    Q_D(EntityEntryCatalog);
    FolderEntry *entry = d->folderHash.take(folderId);
    if (entry) {
        emit folderRemoved(folderId);
        delete entry;
    }
}

void EntityEntryCatalog::_folderACLsChangedSlot(Ice::Long folderId, Diamesh::ACLList aclList)
{
    FolderEntry *folder = folderById(folderId);
    if (folder)
        folder->_updateACLList(aclList);
    else
        _folderAddedSlot(folderId);
}

void EntityEntryCatalog::_cloneJobStartedSlot(Ice::Long entryId, Ice::Int jobId)
{
    _parseCloneJobData(entryId, jobId);
}

void EntityEntryCatalog::_getJobSlot(HnsClient::Job *job)
{
    Q_D(EntityEntryCatalog);

    bool cleanJob = false;

    int jobId = job->getId();
    qlonglong entityEntryId = d->cloneJobToEntryHash.value(jobId, -1);
    // TODO : if (entityEntryId == -1) ==> should never occurs
    EntityEntry *entityEntry = entityEntryById(entityEntryId);
    QString errorMsg = job->getErrorMessage();

        double jobProgress = job->getProgress();
        if (jobProgress != 1 && job->isRunning()) {
            if (entityEntry != 0) {
                entityEntry->setIsCloning(true);
                entityEntry->setCloneProgress(jobProgress);
                _setEntitySyncing(entityEntryId, Syncing);
                emit entityUpdated(entityEntry);
            }
            d->cloneJobHash.insert(jobId, job);
            connect(job, SIGNAL(progressUpdated(int,double)), this, SLOT(_cloneJobProgressChangedSlot(int,double)));
            connect(job, SIGNAL(finished(int)), this, SLOT(_cloneJobFinishedSlot(int)));
        }
        else {
            if (entityEntry != 0) {
                entityEntry->setCloneProgress(0);
                entityEntry->setIsCloning(false);
                _setEntitySyncing(entityEntryId, Synced);
            }
            if (!errorMsg.isEmpty())
                qCritical() << errorMsg;
            cleanJob = true;
        }
    if (cleanJob) {
        d->cloneJobToEntryHash.remove(jobId);
        delete job;
    }
}

void EntityEntryCatalog::_cloneJobProgressChangedSlot(int jobId, double jobProgress)
{
    Q_D(EntityEntryCatalog);

    Job *cloneJob = d->cloneJobHash.value(jobId, 0);
    if (cloneJob) {
        qlonglong entityEntryId = d->cloneJobToEntryHash.value(jobId, -1);
        // TODO : if (entityEntryId == -1) ==> should never occurs
        EntityEntry *entityEntry = entityEntryById(entityEntryId);

        if (entityEntry) {
            if (jobProgress == 1.0) {
                _setEntitySyncing(entityEntryId, Synced);
                entityEntry->setCloneProgress(0);
                entityEntry->setIsCloning(false);
            } else
                entityEntry->setCloneProgress(jobProgress);
            emit entityUpdated(entityEntry);
        }
        if (jobProgress == 1.0) {
            d->cloneJobToEntryHash.remove(jobId);
            d->cloneJobHash.remove(jobId);
            cloneJob->deleteLater();
        }
    }
}

void EntityEntryCatalog::_cloneJobFinishedSlot(int jobId)
{
    Q_D(EntityEntryCatalog);

    qlonglong entityEntryId = d->cloneJobToEntryHash.value(jobId, -1);
    if (entityEntryId != -1) {
        d->cloneJobToEntryHash.remove(jobId);
        EntityEntry *entityEntry = entityEntryById(entityEntryId);
        if (entityEntry) {
            _setEntitySyncing(entityEntryId, Synced);
            entityEntry->setCloneProgress(0);
            entityEntry->setIsCloning(false);
            emit entityUpdated(entityEntry);
        }
    }

    Job *cloneJob = d->cloneJobHash.value(jobId, 0);
    if (cloneJob) {
        cloneJob->deleteLater();
        d->cloneJobHash.remove(jobId);
    }
}

void EntityEntryCatalog::_entryModifiedByUpdater(qlonglong entryId)
{
    Q_D(EntityEntryCatalog);
    if (d->entityHash.contains(entryId))
        emit entityUpdated(d->entityHash[entryId]);
}

void EntityEntryCatalog::_getACLsForEntrySlot(Diamesh::ACLList aclList, QObject *obj)
{
    GetACLsForEntryCookie *cookie = qobject_cast<GetACLsForEntryCookie *>(obj);
    if (cookie) {
        qlonglong entryId = cookie->getEntryId();
        EntityEntry *entityEntry = entityEntryById(entryId);
        if (entityEntry) {
            entityEntry->_updateACLList(aclList);
        }
        delete cookie;
    }
}

void EntityEntryCatalog::_getACLsForFolderSlot(Diamesh::ACLList aclList, QObject *obj)
{
    GetACLsForEntryCookie *cookie = qobject_cast<GetACLsForEntryCookie *>(obj);
    if (cookie) {
        qlonglong folderId = cookie->getEntryId();
        FolderEntry *folder = folderById(folderId);
        if (folder) {
            folder->_updateACLList(aclList);
        }
        delete cookie;
    }
}

void EntityEntryCatalog::_updateACLMethodFailedSlot(QSharedPointer<Ice::Exception> exception, QObject *obj)
{
    QSharedPointer<Hns::RightError> rightError = qSharedPointerDynamicCast<Hns::RightError>(exception);
    UpdateEntryCookie *entryCookie = qobject_cast<UpdateEntryCookie *>(obj);
    if (rightError && entryCookie) {
        QString errorMessage = QString::fromStdString(rightError.data()->message);
        _setEntitySyncing(entryCookie->getEntryId(), Synced);
        emit updateACLFailed(errorMessage);
        return;
    }
    UpdateFolderCookie *folderCookie = qobject_cast<UpdateFolderCookie *>(obj);
    if (rightError && folderCookie) {
        QString errorMessage = QString::fromStdString(rightError.data()->message);
        _setFolderSyncing(folderCookie->folder()->id, Synced);
        emit updateACLFailed(errorMessage);
    }
}

void EntityEntryCatalog::_getUnusedUUIDSlot(std::string uuid, QObject *obj)
{
    GetUnusedUUIDCookie *cookie = qobject_cast<GetUnusedUUIDCookie *>(obj);
    if (cookie) {
        cookie->emitCompleted(QString::fromStdString(uuid));
    }
}

void EntityEntryCatalog::_addEntrySlot(Ice::Long entryId)
{
    if (entryId == 0)
        qDebug() << tr("Unable to add entry %1").arg(entryId);
}

} // namespace HnsClient
