/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef HNSCLIENT_ENTITYENTRYCATALO_H
#define HNSCLIENT_ENTITYENTRYCATALO_H

// hynesim hnsclient headers
#include "libhnsclient/catalog.h"
#include "libhnsclient/private/catalog_p.h"
#include "libhnsclient/common.h"

// Generated hynesim headers
#include "common/proxies_catalog.h"
#include "common/proxies_entitycatalog.h"
#include "common/proxies_topologymanager.h"

// Qt headers
#include <QHash>
#include <QList>

namespace HnsClient {

class EntityEntry;
class FolderEntry;
class EntityEntryCatalogPrivate;
class Session;
class Job;
/*******************************************************************************
* EntityEntryCatalog
*******************************************************************************/
class EntityEntryCatalog : public Catalog
{
    Q_OBJECT
    Q_DECLARE_PRIVATE(EntityEntryCatalog)
public:
    ~EntityEntryCatalog();

    // Local methods, access data stored in cache
    QHash<qlonglong, FolderEntry *> getFolders() const;
    QHash<qlonglong, EntityEntry *> getEntities() const;
    FolderEntry *folderById(qlonglong folderId) const;
    QList<FolderEntry *> getFolderChildren(qlonglong parentFolderId = 0) const;
    EntityEntry *entityEntryByUUID(QString uuid) const;
    EntityEntry *entityEntryById(qlonglong entityEntryId) const;
    QList<EntityEntry *> getEntityChildren(qlonglong parentFolderId = 0) const;


    // Methods based on remote calls
    void addFolder(FolderEntry *folder);
    void updateFolder(FolderEntry *folder);
    void deleteFolder(qlonglong folderId);
    bool moveFolder(qlonglong folderId, qlonglong newParentId);
    void updateEntityEntry(EntityEntry *updatedEntity, qlonglong focusParentId = 0);
    void cloneEntityEntry(qlonglong entityEntryId, int flags);
    void buildModelFromEntityEntry(qlonglong entityEntryId);
    void discardSaveState(qlonglong entityEntryId);
    void deleteItemList(QList<qlonglong> folderIDList, QHash<qlonglong, QList<qlonglong> > entityPositionHash);
    void deleteFolderList(QList<qlonglong> folderIDList);
    void deleteEntityList(QHash<qlonglong, QList<qlonglong> > entityPositionHash);
    void deleteEntity(qlonglong entityId, qlonglong parentId = 0);
    void setCustomGraphic(qlonglong entityEntryId, const QString &fileName);
    void getUnusedUUID(qlonglong entityId, QObject *receiver, const char *slot);
    void addEntry(Hns::EntryPtr entry);

signals:
    void loaded();
    void folderAdded(HnsClient::FolderEntry *folder);
    void folderUpdated(HnsClient::FolderEntry *folder);
    void folderRemoved(qlonglong folderId);
    void entityAdded(HnsClient::EntityEntry *entity);
    void entityUpdated(HnsClient::EntityEntry *entity);
    void entityRemoved(qlonglong entityId);
    void focusOnFolder(qlonglong folderId);
    void focusOnEntry(qlonglong entryId, qlonglong parentId);
    void folderSyncingStateChanged(qlonglong folderId, HnsClient::SynchronizationStatus status);
    void entitySyncingStateChanged(qlonglong entityId, HnsClient::SynchronizationStatus status);

protected:
    explicit EntityEntryCatalog(QObject *parent = 0);
    void _load(Hns::EntityCatalogQPrx catalogQtPrx, Hns::TopologyManagerQPrx topologyManager);
    void _callEnded();
    void _checkLoading();
    void _init();

    virtual Hns::CatalogQPrx _catalogProxy() const;
    virtual void _parseEntryList(Diamesh::LongList entryList);
    virtual void _updateEntryLockedStatus(Ice::Long entryId, bool isLocked);

    void _setFolderSyncing(qlonglong folderId, SynchronizationStatus status);
    void _setEntitySyncing(qlonglong entityId, SynchronizationStatus status);

    void _setWaitForFolder(qlonglong folderId);
    void _checkWaitForFolder(qlonglong folderId);
    void _setWaitForEntity(qlonglong entityId, qlonglong parentId);
    void _checkWaitForEntity(qlonglong entityId);

    bool _handleFolderData(Hns::FolderPtr folder, QObject *receiver, const char *slot);
    bool _handleEntityEntryData(Hns::EntityEntryPtr entityEntry, qlonglong entryId);

    void _parseCloneJobData(qlonglong entryId, qlonglong jobId);

    void _removeFolderAndContent(qlonglong folderId);

protected slots:
    void _loadCloneJobsSlot(::Hns::LongIntMap jobsMap);
    void _loadGetFoldersSlot(::Diamesh::LongList folderList);
    void _loadGetFolderSlot(::Hns::FolderPtr folder);
    void _loadGetEntityEntrySlot(::Hns::EntityEntryPtr entityEntry);

    void _addFolderSlot(::Ice::Long Id, QObject *obj);
    void _updateFolderSlot(bool ok, QObject *obj);
    void _deleteFolderSlot(bool ok, QObject *obj);
    void _updateEntitySlot(bool result, QObject *obj);
    void _deleteEntitySlot(bool result, QObject *obj);
    void _setACLsForEntrySlot(bool result, QObject*obj);
    void _setACLsForFolderSlot(bool result, QObject *obj);
    void _cloneRequestCompletedSlot(::Ice::Int jobId, QObject *obj);
    void _cloneRequestFailedSlot(QSharedPointer<Ice::Exception> exception, QObject *obj);
    void _buildModelCompletedSlot(::Ice::Long newId, QObject *obj );
    void _discardSaveStateCompletedSlot(bool result, QObject *obj);

    void _getEntityEntrySlot(::Hns::EntityEntryPtr entityEntry, QObject *obj);
    void _getEntityEntryUpdateSlot(::Hns::EntityEntryPtr entityEntry, QObject *obj);
    void _getFoldersSlot(::Diamesh::LongList folderList);
    void _getFolderSlot(::Hns::FolderPtr folder);
    void _getFolderUpdateSlot(::Hns::FolderPtr folder, QObject *obj);

    void _entryAddedSlot(Ice::Long entryId);
    void _entryChangedSlot(Ice::Long entryId);
    void _entryRemovedSlot(Ice::Long entryId);
    void _entryACLsChangedSlot(Ice::Long entryId, Diamesh::ACLList aclList);
    void _folderAddedSlot(Ice::Long folderId);
    void _folderChangedSlot(Ice::Long folderId);
    void _folderRemovedSlot(Ice::Long folderId);
    void _folderACLsChangedSlot(Ice::Long folderId, Diamesh::ACLList aclList);
    void _cloneJobStartedSlot(Ice::Long entryId, Ice::Int jobId);

    void _getJobSlot(HnsClient::Job *job);
    void _cloneJobProgressChangedSlot(int jobId, double jobProgress);
    void _cloneJobFinishedSlot(int jobId);
    void _entryModifiedByUpdater(qlonglong entryId);

    void _getACLsForEntrySlot(::Diamesh::ACLList aclList, QObject *obj);
    void _getACLsForFolderSlot(::Diamesh::ACLList aclList, QObject *obj);
    void _updateACLMethodFailedSlot(QSharedPointer<Ice::Exception> exception, QObject* obj);
    void _getUnusedUUIDSlot(::std::string uuid, QObject* obj);
    void _addEntrySlot(::Ice::Long entryId);

private:
    friend class Session;
};

} // namespace HnsClient

#endif //HNSCLIENT_ENTITYENTRYCATALO_H
