/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef HNSCLIENT_ENTITYINFO_H
#define HNSCLIENT_ENTITYINFO_H

// Generated hynesim headers
#include "common/proxies_entityinfo.h"
#include "common/entity.h"

// Qt headers
#include <QObject>
#include <QString>

namespace HnsClient {

class EntityInfoPrivate;
class EntityInfoListPrivate;
/*******************************************************************************
* EntityInfo
*******************************************************************************/
class EntityInfo : public QObject
{
    Q_OBJECT
    Q_DECLARE_PRIVATE(EntityInfo)
public:
    ~EntityInfo();
    Hns::Status getStatus() const;
    qlonglong entityId() const;
    qlonglong entryId() const;
    Hns::EntityInfoData infoData() const;
    QString getShortName() const;
    QString getLongName() const;
    QString getType() const;
    QString getSubType() const;

signals:
    void loaded();
    void statusChanged();
    void entryChanged();

protected:
    explicit EntityInfo(QObject *parent = 0);
    void _load(qlonglong entityId, Hns::EntityInfoQPrx qprx);
    void _callEnded();
    void _checkLoading();
    void _init();

private slots:
    void _loadGetStatusSlot(::Hns::Status status);
    void _loadEntryIdSlot(::Ice::Long entryId);
    void _loadInfoDataSlot(::Hns::EntityInfoData data);

    void _statusChangedSlot(Ice::Int status, Ice::Long statusIndex);
    void _entryChangedSlot(::Hns::EntityInfoData data);

private:
    EntityInfoPrivate * const d_ptr;
    friend class TopologyManager;
};


/*******************************************************************************
* EntityInfoList
*******************************************************************************/
class EntityInfoList : public QObject
{
    Q_OBJECT
    Q_DECLARE_PRIVATE(EntityInfoList)
public:
    ~EntityInfoList();
    EntityInfo *getEntityInfo(qlonglong entityId);
    bool hasEntityInfo(qlonglong entityId);

signals:
    void loaded();
    void entityInfoAdded(qlonglong entityId);
    void entityInfoRemoved(qlonglong entityId);

protected:
    explicit EntityInfoList(QObject *parent = 0);

    void _addEntityInfo(EntityInfo *info);
    void _removeEntityInfo(qlonglong entityId);

private:
    EntityInfoListPrivate * const d_ptr;
    friend class TopologyManager;
};

} // namespace HnsClient

#endif //HNSCLIENT_ENTITYINFO_H
