/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// hynesim hnsclient headers
#include "libhnsclient/entrypropertiesmanager.h"
#include "libhnsclient/private/entrypropertiesmanager_p.h"

namespace HnsClient {

/*******************************************************************************
* EntryPropertiesManagerPrivate
*******************************************************************************/
EntryPropertiesManagerPrivate::EntryPropertiesManagerPrivate()
{
    entryPropertiesManagerQPrx = 0;
}

EntryPropertiesManagerPrivate::~EntryPropertiesManagerPrivate()
{
    entryPropertiesManagerQPrx = 0;
}


/*******************************************************************************
* GetEntryPropertiesCookie
*******************************************************************************/
GetEntryPropertiesCookie::GetEntryPropertiesCookie(QObject *receiver, const char *slot)
    : QObject()
{
    connect(this, SIGNAL(completed(::Hns::EntryPropertiesPtr)), receiver, slot);
}

GetEntryPropertiesCookie::~GetEntryPropertiesCookie()
{
}

void GetEntryPropertiesCookie::emitCompleted(::Hns::EntryPropertiesPtr entryProperties)
{
    emit completed(entryProperties);
    deleteLater();
}


/*******************************************************************************
* EntryPropertiesManager
*******************************************************************************/
EntryPropertiesManager::EntryPropertiesManager(QObject *parent)
    : QObject(parent)
    , d_ptr(new EntryPropertiesManagerPrivate)
{
}

EntryPropertiesManager::~EntryPropertiesManager()
{
    delete d_ptr;
}

/*! \fn getEntryProperties(::Hns::EntryPtr e, QObject *receiver, const char *slot)
 *\brief Create a cookie object with receiver and slot parameters and give it to begin_getProperties
 *\param e Return entity pointer
 *\param receiver object connected to internal signal
 *\param slot Return slot connected to signal "completed(Hns::EntryPropertiesPtr entryProperties)"
 */
void EntryPropertiesManager::getEntryProperties(Hns::EntryPtr e, QObject *receiver, const char *slot)
{
    Q_D(EntryPropertiesManager);
    GetEntryPropertiesCookie *cookie = new GetEntryPropertiesCookie(receiver, slot);
    d->entryPropertiesManagerQPrx->begin_getProperties(e, this, SLOT(_getPropertiesSlot(::Hns::EntryPropertiesPtr, QObject *)), 0, 0, cookie); //OK
}

void EntryPropertiesManager::_load(Hns::EntryPropertiesManagerQPrx qprx)
{
    Q_D(EntryPropertiesManager);
    d->entryPropertiesManagerQPrx = qprx;
    emit loaded();
}

void EntryPropertiesManager::_getPropertiesSlot(::Hns::EntryPropertiesPtr entryProperties, QObject *obj) //signature est mauvaise slot de retour devrait être Hns::EntryPtr
{
    GetEntryPropertiesCookie *cookie = qobject_cast<GetEntryPropertiesCookie *>(obj);
    if (cookie)
        cookie->emitCompleted(entryProperties);
}

} // namespace HnsClient
