/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// hynesim hnsclient headers
#include "libhnsclient/filetransferclient.h"
#include "libhnsclient/private/filetransferclient_p.h"

// diamesh headers
#include <diamesh/filetransferclient.h>

// Qt headers
#include <QDir>

namespace HnsClient {

/*******************************************************************************
* FileTransferClientPrivate
*******************************************************************************/
FileTransferClientPrivate::FileTransferClientPrivate()
{
    transfertClient = 0;
}

FileTransferClientPrivate::~FileTransferClientPrivate()
{
    if (transfertClient)
        delete transfertClient;
    transfertClient = 0;
}


/*******************************************************************************
* FileTransferClient
*******************************************************************************/
FileTransferClient::FileTransferClient(Diamesh::FileTransferClient *transferClient, QString remoteFileName, QObject *parent)
    : QObject(parent)
    , d_ptr(new FileTransferClientPrivate)
{
    Q_D(FileTransferClient);
    d->transfertClient = transferClient;
    d->remoteFileName = remoteFileName;
    d->temporaryDirPath = QDir::tempPath() + QDir::separator() + ("hyneview") + QDir::separator();
    QDir dummyDir;
    dummyDir.setPath(d->temporaryDirPath);
    d->localFileAbsolutePath = dummyDir.absolutePath() +QDir::separator() + d->remoteFileName;
    connect(d->transfertClient, SIGNAL(fileSize(Ice::Long)), this, SLOT(_fileSizeSlot(Ice::Long)));
}

FileTransferClient::~FileTransferClient()
{
    delete d_ptr;
}

/*! \fn canDownload()
 *\brief TODO
 */
bool FileTransferClient::canDownload() const
{
    Q_D(const FileTransferClient);
    QDir dummyDir;
    bool result = dummyDir.mkpath(d->temporaryDirPath);
    if (!result)
        qCritical()<<tr("A problem occured while making the directory %1").arg(d->temporaryDirPath);
    return result;
}

/*! \fn localFileExists()
 *\brief TODO
 */
bool FileTransferClient::localFileExists() const
{
    Q_D(const FileTransferClient);
    return QFile::exists(d->localFileAbsolutePath);
}

/*! \fn startDownload(bool overwrite = false)
 *\brief TODO
 *\param overwrite
 */
bool FileTransferClient::startDownload(bool overwrite)
{
    Q_D(FileTransferClient);
    bool started = false;
    if (localFileExists() && !overwrite) {
        qWarning() << tr("File download aborted. File %1 already exists.").arg(d->localFileAbsolutePath);
    } else {
        connect(d->transfertClient, SIGNAL(percentTransfer(int)), this, SLOT(_percentTransferSlot(int)));
        connect(d->transfertClient, SIGNAL(transferIsFinished()), this, SLOT(_transferIsFinishedSlot()));
        d->transfertClient->start(d->localFileAbsolutePath);
        started = true;
    }
    return started;
}

/*! \fn diameshFileTransfertClient()
 *\brief TODO
 */
Diamesh::FileTransferClient* FileTransferClient::diameshFileTransfertClient() const
{
    Q_D(const FileTransferClient);
    return d->transfertClient;
}

/*! \fn remoteFileName()
 *\brief TODO
 */
QString FileTransferClient::remoteFileName() const
{
    Q_D(const FileTransferClient);
    return d->remoteFileName;
}

/*! \fn localFileAbsolutePath()
 *\brief TODO
 */
QString FileTransferClient::localFileAbsolutePath() const
{
    Q_D(const FileTransferClient);
    return d->localFileAbsolutePath;
}

void FileTransferClient::setLocalAbsoluteFileName(const QString &filePath)
{
    Q_D(FileTransferClient);
    d->localFileAbsolutePath = filePath;
}

void FileTransferClient::_percentTransferSlot(int percent)
{
    emit transferProgression(percent);
}

void FileTransferClient::_transferIsFinishedSlot()
{
    Q_D(FileTransferClient);
    emit transferFinished(d->remoteFileName, d->localFileAbsolutePath);
}

void FileTransferClient::_fileSizeSlot(Ice::Long size)
{
    Q_D(FileTransferClient);
    emit fileSize(d->remoteFileName, size);
}

} // namespace HnsClient
