/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef HNSCLIENT_FILETRANSFERCLIENT_H
#define HNSCLIENT_FILETRANSFERCLIENT_H

// ZeroC Ice headers
#include <Ice/Config.h>

// Qt headers
#include <QObject>

namespace Diamesh {
class FileTransferClient;
} // namespace Diamesh

namespace HnsClient {

class FileTransferClientPrivate;

/*******************************************************************************
* FileTransferClient
*******************************************************************************/
class FileTransferClient : public QObject
{
    Q_OBJECT
    Q_DECLARE_PRIVATE(FileTransferClient)
public:
    explicit FileTransferClient(Diamesh::FileTransferClient *transferClient, QString remoteFileName, QObject *parent);
    ~FileTransferClient();
    bool canDownload() const;
    bool localFileExists() const;
    bool startDownload(bool overwrite = false);
    Diamesh::FileTransferClient* diameshFileTransfertClient() const;
    QString remoteFileName() const;
    QString localFileAbsolutePath() const;
    void setLocalAbsoluteFileName(const QString &filePath);

signals :
    void fileSize(QString remoteFileName, qlonglong size);
    void transferProgression(int progress);
    void transferFinished(QString remoteFileName, QString localFileName);

protected slots:
    void _percentTransferSlot(int percent);
    void _transferIsFinishedSlot();
    void _fileSizeSlot(Ice::Long);
private:
    FileTransferClientPrivate * const d_ptr;
};

} // namespace HnsClient

#endif //HNSCLIENT_FILETRANSFERCLIENT_H
