/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// hynesim hnsclient headers
#include "libhnsclient/hsetmanager.h"
#include "libhnsclient/private/hsetmanager_p.h"

namespace HnsClient {

/*******************************************************************************
* HSetManagerPrivate
*******************************************************************************/
HSetManagerPrivate::HSetManagerPrivate()
{
    hSetManagerQPrx = 0;
    remoteCallCount = 0;
}

HSetManagerPrivate::~HSetManagerPrivate()
{
    hSetManagerQPrx = 0;
}

/*******************************************************************************
* HSetManager
*******************************************************************************/
HSetManager::HSetManager(QObject *parent)
    : QObject(parent)
    , d_ptr(new HSetManagerPrivate)
{
}

HSetManager::~HSetManager()
{
    delete d_ptr;
}

/*! \fn Diamesh::HierarchicalSet getHSetById(qlonglong hSetId) const
 *\brief TODO
 */
Diamesh::HierarchicalSet HSetManager::getHSetById(qlonglong hSetId) const
{
    Q_D(const HSetManager);
    Diamesh::HierarchicalSet result = HSetManager::invalidHSet();
    if (d->hSetHash.contains(hSetId))
        result = d->hSetHash.value(hSetId);
    return result;
}

/*! \fn Diamesh::HierarchicalSet invalidHSet()
 *\brief TODO
 */
Diamesh::HierarchicalSet HSetManager::invalidHSet()
{
    Diamesh::HierarchicalSet result;
    result.id = -1;
    return result;
}

void HSetManager::_load(Diamesh::HSetManagerQPrx qprx)
{
    Q_D(HSetManager);
    d->hSetManagerQPrx = qprx;
    d->remoteCallCount = 1;
    d->hSetManagerQPrx->begin_getHSets(this, SLOT(_loadGetHSetsSlot(::Diamesh::HierarchicalSetList)));
}

void HSetManager::_callEnded()
{
    Q_D(HSetManager);
    d->remoteCallCount--;
    _checkLoading();
}

void HSetManager::_checkLoading()
{
    Q_D(HSetManager);
    if (d->remoteCallCount == 0)
        _init();
}

void HSetManager::_init()
{
    Q_D(HSetManager);
    connect(d->hSetManagerQPrx.data(), SIGNAL(hSetAdded(Ice::Long)), this, SLOT(_hSetAddedSlot(Ice::Long)));
    connect(d->hSetManagerQPrx.data(), SIGNAL(hSetChanged(Ice::Long)), this, SLOT(_hSetChangedSlot(Ice::Long)));
    connect(d->hSetManagerQPrx.data(), SIGNAL(hSetRemoved(Ice::Long)), this, SLOT(_hSetRemovedSlot(Ice::Long)));
    emit loaded();
}

void HSetManager::_loadGetHSetsSlot(Diamesh::HierarchicalSetList list)
{
    Q_D(HSetManager);
    QList<Diamesh::HierarchicalSet> hSetList = QList<Diamesh::HierarchicalSet>::fromStdList(list);
    foreach(Diamesh::HierarchicalSet set, hSetList)
        d->hSetHash.insert(set.id, set);
    _callEnded();
}

void HSetManager::_hSetAddedSlot(Ice::Long hSetId)
{
    Q_UNUSED(hSetId)
}

void HSetManager::_hSetChangedSlot(Ice::Long hSetId)
{
    Q_UNUSED(hSetId)
}

void HSetManager::_hSetRemovedSlot(Ice::Long hSetId)
{
    Q_UNUSED(hSetId)
}

} // namespace HnsClient
