/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef HNSCLIENT_JOBMANAGER_H
#define HNSCLIENT_JOBMANAGER_H

// Generated hynesim headers
#include "common/proxies_job.h"
#include "common/proxies_jobmanager.h"

// Qt headers
#include <QObject>
#include <QString>

namespace HnsClient {

class JobPrivate;
class JobManagerPrivate;

/*******************************************************************************
* Job
*******************************************************************************/
class Job : public QObject
{
    Q_OBJECT
    Q_DECLARE_PRIVATE(Job)
public:
    ~Job();

    // Local methods, access to data stored in cache
    int getId() const;
    bool isRunning() const;
    QString getErrorMessage() const;
    double getProgress() const;

signals:
    void loaded();
    void progressUpdated(int jobId, double progress);
    void finished(int jobId);

protected:
    explicit Job(QObject *parent = 0);
    void _load(Hns::JobQPrx jobQPrx, int jobId);
    void _callEnded();
    void _checkLoading();
    void _init();

    JobPrivate * const d_ptr;

protected slots:
    void _progressChangedSlot(Ice::Int jobId, Ice::Double progress);
    void _finished(Ice::Int jobId);
    void _cancelled(Ice::Int jobId);

private slots:
    void _loadIsRunningSlot(bool isRunning);
    void _loadProgressSlot(::Ice::Double progress);

private:
    friend class GetJobCookie;
};

/*******************************************************************************
* JobManager
*******************************************************************************/
class JobManager : public QObject
{
    Q_OBJECT
    Q_DECLARE_PRIVATE(JobManager)
public:
    ~JobManager();
    // Methods based on remote calls

    void getJob(int jobId, QObject *receiver, const char *slot);

signals:
    void loaded();

protected:
    explicit JobManager(QObject *parent = 0);
    void _load(Hns::JobManagerQPrx jobManagerQPrx);

    JobManagerPrivate * const d_ptr;

protected slots:
    void _getJobSlot(::Hns::JobPrx jobPrx, QObject *obj);

private:
    friend class Session;
};

} // namespace HnsClient

#endif //HNSCLIENT_JOBMANAGER_H
