/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef HNSCLIENT_MACADDRESSMANAGER_H
#define HNSCLIENT_MACADDRESSMANAGER_H

// Generated hynesim headers
#include "common/proxies_macaddressmanager.h"

// Qt headers
#include <QObject>
#include <QStringList>

namespace HnsClient {

class MacAddressManagerPrivate;

/*******************************************************************************
* MacAddressManager
*******************************************************************************/
class MacAddressManager : public QObject {
    Q_OBJECT
    Q_DECLARE_PRIVATE(MacAddressManager)
public:
    ~MacAddressManager();
    // Methods based on remote calls
    void generateRandomAddress(const QString &vendor, QObject *receiver, const char *slot);
    void getVendorFromPrefix(const QString &macPrefix, QObject *receiver, const char *slot);
    void isUsed(const QString &macAddress, QObject *receiver, const char *slot);
    // Local method, access to data stored in cache
    QStringList *getVendorList();

signals:
    void loaded();

protected:
    explicit MacAddressManager(QObject *parent = 0);
    void _load(Hns::MacAddressManagerQPrx qprx);

    MacAddressManagerPrivate * const d_ptr;

protected slots:
    void _loadVendorListSlot(::Diamesh::StringList vendorList);

private slots:
    void _generateRandomAddressSlot(::std::string address, QObject *obj);
    void _getVendorFromPrefixSlot(::std::string vendor, QObject *obj);
    void _isUsedSlot(bool isUsed, QObject *obj);

private:
    friend class Session;
};

} // namespace HnsClient

#endif //HNSCLIENT_MACADDRESSMANAGER_H
