/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// hynesim hnsclient headers
#include "libhnsclient/platformmonitor.h"
#include "libhnsclient/private/platformmonitor_p.h"
#include "libhnsclient/private/privatesessionaccessor.h"
#include "libhnsclient/utility.h"

// ZeroC Ice headers
#include <Ice/ObjectAdapter.h>
#include <Ice/Communicator.h>

// Qt headers
#include <QStringList>
#include <QMapIterator>
#include <QDebug>

namespace HnsClient {

/*******************************************************************************
* NodeMonitorPrivate
*******************************************************************************/
NodeMonitorPrivate::NodeMonitorPrivate()
{
    nodeMonitorQtPrx = 0;
    remoteCallCount = 0;
}

NodeMonitorPrivate::~NodeMonitorPrivate()
{
    nodeMonitorQtPrx = 0;
    entityMonitoringDataMap.clear();
}

/*******************************************************************************
* ServiceDetailsPrivate
*******************************************************************************/
ServiceDetailsPrivate::ServiceDetailsPrivate()
{
}

ServiceDetailsPrivate::~ServiceDetailsPrivate()
{
}


/*******************************************************************************
* NodeInformationPrivate
*******************************************************************************/
NodeInformationPrivate::NodeInformationPrivate()
{
}

NodeInformationPrivate::~NodeInformationPrivate()
{
    nodeInfoQtPrx = 0;
}


/*******************************************************************************
* MasterInformationPrivate
*******************************************************************************/
MasterInformationPrivate::MasterInformationPrivate()
    : ServiceDetailsPrivate()
{
}

MasterInformationPrivate::~MasterInformationPrivate()
{
    masterQtPrx = 0;
    tracerQtPrx = 0;
}


/*******************************************************************************
* PlatformMonitorPrivate
*******************************************************************************/
PlatformMonitorPrivate::PlatformMonitorPrivate()
{
}

PlatformMonitorPrivate::~PlatformMonitorPrivate()
{
    qDeleteAll(runningLoaders);
    runningLoaders.clear();
    qDeleteAll(nodeInformationHash);
    nodeInformationHash.clear();
    delete masterInformation;
    masterInformation = 0;
    masterQtPrx = 0;
}


/*******************************************************************************
* NodeMonitor
*******************************************************************************/
NodeMonitor::NodeMonitor(QObject *parent)
    : QObject(parent)
    , d_ptr(new NodeMonitorPrivate)
{
}

NodeMonitor::~NodeMonitor()
{
    delete d_ptr;
}

void NodeMonitor::_load(Hns::NodeMonitorQPrx nodeMonitorQPrx)
{
    Q_D(NodeMonitor);
    d->nodeMonitorQtPrx = nodeMonitorQPrx;
    d->remoteCallCount = 1;
    d->nodeMonitorQtPrx->begin_getEntityMonitorDataMap(this, SLOT(_loadEntityMonitorDataMapSlot(::Hns::EntityMonitorDataMap)));
}

QMap<Ice::Long, Hns::EntityMonitoringDataPtr> NodeMonitor::getEntityMonitoringDataMap()
{
    Q_D(NodeMonitor);
    return d->entityMonitoringDataMap;
}

Hns::EntityMonitoringDataPtr NodeMonitor::getEntityMonitoringData(qlonglong entityId)
{
    Q_D(NodeMonitor);
    return d->entityMonitoringDataMap.value(entityId, 0);
}

void NodeMonitor::_loadEntityMonitorDataMapSlot(Hns::EntityMonitorDataMap dataMap)
{
    Q_D(NodeMonitor);
    d->entityMonitoringDataMap = QMap<Ice::Long, Hns::EntityMonitoringDataPtr>(dataMap);
    _callEnded();
}

void NodeMonitor::_entityDataChangedSlot(Hns::EntityMonitorDataMap dataMap)
{
    Q_D(NodeMonitor);
    QMap<Ice::Long, Hns::EntityMonitoringDataPtr> targetMap(dataMap);
    d->entityMonitoringDataMap = targetMap;
    emit entityMonitoringDataUpdated();
}

void NodeMonitor::_callEnded()
{
    Q_D(NodeMonitor);
    d->remoteCallCount--;
    _checkLoading();
}

void NodeMonitor::_checkLoading()
{
    Q_D(NodeMonitor);
    if (d->remoteCallCount == 0)
        _init();
}

void NodeMonitor::_init()
{
    Q_D(NodeMonitor);
    connect(d->nodeMonitorQtPrx.data(), SIGNAL(entityDataChanged(Hns::EntityMonitorDataMap)), this, SLOT(_entityDataChangedSlot(Hns::EntityMonitorDataMap)));
    emit loaded();
}

/*******************************************************************************
* ServiceDetails
*******************************************************************************/
ServiceDetails::ServiceDetails(ServiceDetailsPrivate *d, QObject *parent)
    : QObject(parent)
    , d_ptr(d)
{
}

ServiceDetails::~ServiceDetails()
{
    delete d_ptr;
}

/*! \fn refresh()
 *\brief TODO
 */
void ServiceDetails::refresh()
{
    refreshSystemInfo();
    refreshServiceInfo();
}

/*! \fn getVersionInfo()
 *\brief TODO
 */
Diamesh::VersionInfo ServiceDetails::getVersionInfo()
{
    Q_D(ServiceDetails);
    return d->versionInformation;
}

/*! \fn getSystemInfo()
 *\brief TODO
 */
Diamesh::SystemInfo ServiceDetails::getSystemInfo()
{
    Q_D(ServiceDetails);
    return d->systemInformation;
}

/*! \fn getServiceInfo()
 *\brief TODO
 */
Diamesh::ServiceInfo ServiceDetails::getServiceInfo()
{
    Q_D(ServiceDetails);
    return d->serviceInformation;
}

/*! \fn getVersionInfoAsString()
 *\brief TODO
 */
QString ServiceDetails::getVersionInfoAsString()
{
    //Required for use with service query
    return getVersionInfoAsString(false);
}

/*! \fn getVersionInfoAsString(bool includeDate = false)
 *\brief TODO
 *\param includeDate
 */
QString ServiceDetails::getVersionInfoAsString(bool includeDate)
{
    QString version = getVersion();
    QString versionDescription = getVersionDescription();
    QString revision = getRevision();

    if (!versionDescription.isEmpty())
        version.append(QString(" %1").arg(versionDescription));
    if (!revision.isEmpty())
        version.append(QString(" (rev %1)").arg(revision));
    if (includeDate) {
        version+= "<br/>" + getBuildDate();
        version+= " " + getBuildTime();
    }
    return version;
}

/*! \fn getVersion()
 *\brief TODO
 */
QString ServiceDetails::getVersion()
{
    Q_D(ServiceDetails);
    return QString::fromStdString(d->versionInformation.version);
}

/*! \fn getVersionDescription()
 *\brief TODO
 */
QString ServiceDetails::getVersionDescription()
{
    Q_D(ServiceDetails);
    return QString::fromStdString(d->versionInformation.versionDescription);
}

/*! \fn getRevision()
 *\brief TODO
 */
QString ServiceDetails::getRevision()
{
    Q_D(ServiceDetails);
    return QString::fromStdString(d->versionInformation.revision);
}

/*! \fn getBuildDate()
 *\brief TODO
 */
QString ServiceDetails::getBuildDate()
{
    Q_D(ServiceDetails);
    return QString::fromStdString(d->versionInformation.buildDate);
}

/*! \fn getBuildTime()
 *\brief TODO
 */
QString ServiceDetails::getBuildTime()
{
    Q_D(ServiceDetails);
    return QString::fromStdString(d->versionInformation.buildTime);
}

/*! \fn getKernelInformation()
 *\brief TODO
 */
QString ServiceDetails::getKernelInformation()
{
    Q_D(ServiceDetails);
    return QString::fromStdString(d->systemInformation.kernelInfo);
}

/*! \fn getSystemProcessors()
 *\brief TODO
 */
QStringList ServiceDetails::getSystemProcessors()
{
    Q_D(ServiceDetails);
    QStringList result;
    Diamesh::StringList procList = d->systemInformation.processors;
    foreach(std::string processor, procList)
        result.append(QString::fromStdString(processor));
    return result;
}

/*! \fn getSystemProcessorsAsString()
 *\brief TODO
 */
QString ServiceDetails::getSystemProcessorsAsString()
{
    return getSystemProcessors().join("<br/>");
}

/*! \fn getSystemUptime() const
 *\brief TODO
 */
int ServiceDetails::getSystemUptime() const
{
    Q_D(const ServiceDetails);
    return d->systemInformation.uptime;
}

/*! \fn getSystemUptimeAsString()
 *\brief TODO
 */
QString ServiceDetails::getSystemUptimeAsString()
{
    return Utility::makeTimeReadable(getSystemUptime());
}

/*! \fn getCpuUsage() const
 *\brief TODO
 */
double ServiceDetails::getCpuUsage() const
{
    Q_D(const ServiceDetails);
    return d->systemInformation.cpuUsage;
}

/*! \fn getCpuUsageAsString()
 *\brief TODO
 */
QString ServiceDetails::getCpuUsageAsString()
{
    return Utility::makePercentageReadable(getCpuUsage());
}

/*! \fn getFreeMemory() const
 *\brief TODO
 */
qlonglong ServiceDetails::getFreeMemory() const
{
    Q_D(const ServiceDetails);
    return d->systemInformation.freeMemory;
}

/*! \fn getFreeMemoryAsString()
 *\brief TODO
 */
QString ServiceDetails::getFreeMemoryAsString()
{
    return Utility::makeSizeReadable(getFreeMemory(), HnsClient::KibiBytes);
}

/*! \fn getTotalMemory() const
 *\brief TODO
 */
qlonglong ServiceDetails::getTotalMemory() const
{
    Q_D(const ServiceDetails);
    return d->systemInformation.totalMemory;
}

/*! \fn getTotalMemoryAsString()
 *\brief TODO
 */
QString ServiceDetails::getTotalMemoryAsString()
{
    return Utility::makeSizeReadable(getTotalMemory(), HnsClient::KibiBytes);
}

/*! \fn getMemoryLoad()
 *\brief TODO
 */
QString ServiceDetails::getMemoryLoad()
{
    long total = getTotalMemory();
    long free = getFreeMemory();
    return Utility::makeMemoryLoadReadable(total - free, total);
}

/*! \fn getFreeSwap() const
 *\brief TODO
 */
qlonglong ServiceDetails::getFreeSwap() const
{
    Q_D(const ServiceDetails);
    return d->systemInformation.freeSwap;
}

/*! \fn getFreeSwapAsString()
 *\brief TODO
 */
QString ServiceDetails::getFreeSwapAsString()
{
    return Utility::makeSizeReadable(getFreeSwap(), HnsClient::KibiBytes);
}

/*! \fn getTotalSwap() const
 *\brief TODO
 */
qlonglong ServiceDetails::getTotalSwap() const
{
    Q_D(const ServiceDetails);
    return d->systemInformation.totalSwap;
}

/*! \fn getTotalSwapAsString()
 *\brief TODO
 */
QString ServiceDetails::getTotalSwapAsString()
{
    return Utility::makeSizeReadable(getTotalSwap(), HnsClient::KibiBytes);
}

/*! \fn getSwapLoad()
 *\brief TODO
 */
QString ServiceDetails::getSwapLoad()
{
    long total = getTotalSwap();
    long free = getFreeSwap();
    return Utility::makeMemoryLoadReadable(total - free, total);
}

/*! \fn getDisksStatusAsString()
 *\brief TODO
 */
QString ServiceDetails::getDisksStatusAsString()
{
    Q_D(ServiceDetails);
    QString result;
    QList<Diamesh::DiskInfo> diskList = QList<Diamesh::DiskInfo>::fromStdList(d->systemInformation.disks);
    if (diskList.size() > 0) {
        result = "<table class='diskTable' cellspacing='0' width='100%'><tr><td><b>Disk</b> </td><td><b>Total</b> </td><td><b>Used</b> </td><td><b>%</b> </td><td><b>Free</b></td>";
        foreach(const Diamesh::DiskInfo & disk, diskList) {
            result.append(
                        QString("<tr><td><b>%1</b> </td><td>%2</td><td>%3</td><td>%4</td><td>%5</td></tr>")
                        .arg(QString::fromStdString(disk.mountPoint))
                        .arg(Utility::makeSizeReadable(disk.totalSpace, HnsClient::KibiBytes))
                        .arg(Utility::makeSizeReadable(disk.usedSpace, HnsClient::KibiBytes))
                        .arg(QString("%1%").arg(QString::number(disk.percentageUsed)))
                        .arg(Utility::makeSizeReadable(disk.freeSpace, HnsClient::KibiBytes))
                        );
        }
        result.append("</table>");
    }
    return result;
}

/*! \fn getName() const
 *\brief TODO
 */
QString ServiceDetails::getName() const
{
    Q_D(const ServiceDetails);
    return Utility::readableStringValue(d->serviceInformation.name);
}

/*! \fn getShortName()
 *\brief TODO
 */
QString ServiceDetails::getShortName()
{
    Q_D(ServiceDetails);
    return Utility::readableStringValue(d->serviceInformation.shortName, getName());
}

/*! \fn getDescription()
 *\brief TODO
 */
QString ServiceDetails::getDescription()
{
    Q_D(ServiceDetails);
    return Utility::readableStringValue(d->serviceInformation.description, "");
}

/*! \fn getServiceHost()
 *\brief TODO
 */
QString ServiceDetails::getServiceHost()
{
    Q_D(ServiceDetails);
    return Utility::readableStringValue(d->serviceInformation.host);
}

/*! \fn getServiceMaxThread()
 *\brief TODO
 */
QString ServiceDetails::getServiceMaxThread()
{
    Q_D(ServiceDetails);
    return Utility::readableStringValue(d->serviceInformation.maxThread);
}

/*! \fn getServiceUptime() const
 *\brief TODO
 */
int ServiceDetails::getServiceUptime() const
{
    Q_D(const ServiceDetails);
    return d->serviceInformation.uptime;
}

/*! \fn getServiceUptimeAsString()
 *\brief TODO
 */
QString ServiceDetails::getServiceUptimeAsString()
{
    return Utility::makeTimeReadable(getServiceUptime());
}

void ServiceDetails::_remoteMethodFailedSlot(QSharedPointer<Ice::Exception> exception)
{
    qCritical() << tr("Catched Ice exception: %1").arg(exception->ice_name().c_str());
}

void ServiceDetails::_getSystemInfoSlot(::Diamesh::SystemInfo systemInformation)
{
    Q_D(ServiceDetails);
    if (systemInformation != d->systemInformation) {
        d->systemInformation = systemInformation;
        emit infoChanged();
    }
}

void ServiceDetails::_getServiceInfoSlot(::Diamesh::ServiceInfo serviceInformation)
{
    Q_D(ServiceDetails);
    if (serviceInformation != d->serviceInformation) {
        d->serviceInformation = serviceInformation;
        emit infoChanged();
    }
}


/*******************************************************************************
* NodeInformation
*******************************************************************************/
NodeInformation::NodeInformation(QObject *parent)
    : ServiceDetails(new NodeInformationPrivate, parent)
{
}

NodeInformation::~NodeInformation()
{
}

/*! \fn refresh()
 *\brief TODO
 */
void NodeInformation::refresh()
{
    ServiceDetails::refresh();
    refreshResourceInfo();
}

/*! \fn getPlatforms()
 *\brief TODO
 */
QMap<QString, Hns::PlatformInfo> NodeInformation::getPlatforms()
{
    Q_D(NodeInformation);
    return d->platforms;
}

/*! \fn getPlatformsAsString()
 *\brief TODO
 */
QString NodeInformation::getPlatformsAsString()
{
    Q_D(NodeInformation);
    QStringList resultList;
    foreach(const Hns::PlatformInfo &platform, d->platforms) {
        if (platform.enabled)
            resultList.append(_translatePlatform(QString::fromStdString(platform.name)));
        else
            resultList.append("<font color=\"red\">"+_translatePlatform(QString::fromStdString(platform.name))
            + " (" + tr("reason:") + QString::fromStdString(platform.error) + ")</font>");
    }
    return resultList.join("<br/>");
}

/*! \fn getHnsResourceInformation()
 *\brief TODO
 */
HnsNode::ResourceInfo NodeInformation::getHnsResourceInformation()
{
    Q_D(NodeInformation);
    return d->resourceInformation;
}

/*! \fn getHnsMaxResources()
 *\brief TODO
 */
Hns::Resource NodeInformation::getHnsMaxResources()
{
    return getHnsResourceInformation().maxResources;
}

/*! \fn getHnsCurrentResources()
 *\brief TODO
 */
Hns::Resource NodeInformation::getHnsCurrentResources()
{
    return getHnsResourceInformation().currentResources;
}

/*! \fn getHnsMaxCpu()
 *\brief TODO
 */
qlonglong NodeInformation::getHnsMaxCpu()
{
    return getHnsMaxResources().cpu;
}

/*! \fn getHnsMaxCpuAsString()
 *\brief TODO
 */
QString NodeInformation::getHnsMaxCpuAsString()
{
    return QString::number(getHnsMaxCpu());
}

/*! \fn getHnsCurrentCpu()
 *\brief TODO
 */
qlonglong NodeInformation::getHnsCurrentCpu()
{
    return getHnsCurrentResources().cpu;
}

/*! \fn getHnsCpuUsage()
 *\brief TODO
 */
QString NodeInformation::getHnsCpuUsage()
{
    qlonglong used = getHnsCurrentCpu();
    qlonglong total = getHnsMaxCpu();
    return QString("%1 (%2) of %3")
            .arg(QString::number(used))
            .arg(Utility::makePercentageReadable(used, total))
            .arg(QString::number(total));
}

/*! \fn getHnsMaxMemory()
 *\brief TODO
 */
qlonglong NodeInformation::getHnsMaxMemory()
{
    return getHnsMaxResources().memory * 1024;
}

/*! \fn getHnsMaxMemoryAsString()
 *\brief TODO
 */
QString NodeInformation::getHnsMaxMemoryAsString()
{
    return Utility::makeSizeReadable(getHnsMaxMemory(), HnsClient::KibiBytes);
}

/*! \fn getHnsCurrentMemory()
 *\brief TODO
 */
qlonglong NodeInformation::getHnsCurrentMemory()
{
    return getHnsCurrentResources().memory * 1024;
}

/*! \fn getHnsCurrentMemoryAsString()
 *\brief TODO
 */
QString NodeInformation::getHnsCurrentMemoryAsString()
{
    return Utility::makeSizeReadable(getHnsCurrentMemory(), HnsClient::KibiBytes);
}

/*! \fn getHnsMemoryUsage()
 *\brief TODO
 */
QString NodeInformation::getHnsMemoryUsage()
{
    return Utility::makeMemoryLoadReadable(getHnsCurrentMemory(), getHnsMaxMemory());
}

/*! \fn getHnsDefinedEntities()
 *\brief TODO
 */
int NodeInformation::getHnsDefinedEntities()
{
    return getHnsResourceInformation().definedEntities;
}

/*! \fn getHnsDefinedEntitiesAsString()
 *\brief TODO
 */
QString NodeInformation::getHnsDefinedEntitiesAsString()
{
    return QString::number(getHnsDefinedEntities());
}

/*! \fn getHnsDefinedEntitiesAsString()
 *\brief TODO
 */
QString NodeInformation::getHnsNetcards()
{
    QString nodenetCard;
    QMapIterator<std::string, std::string> iter(QMap<std::string, std::string>(getHnsResourceInformation().hybridNetcardsMap));
    while (iter.hasNext()) {
        iter.next();
        nodenetCard = tr("%1Plug : %2 / Device : %3\n").arg(nodenetCard).arg(iter.key().c_str()).arg(iter.value().c_str());
    }
    return nodenetCard;
}

/*! \fn getNodeMonitor()
 *\brief TODO
 */
NodeMonitor *NodeInformation::getNodeMonitor()
{
    Q_D(NodeInformation);
    return d->nodeMonitor;
}

/*! \fn refreshSystemInfo()
 *\brief TODO
 */
void NodeInformation::refreshSystemInfo()
{
    Q_D(NodeInformation);
    d->nodeInfoQtPrx->begin_getSystemInfo(this, SLOT(_getSystemInfoSlot(::Diamesh::SystemInfo)), this, SLOT(_remoteMethodFailedSlot(QSharedPointer<Ice::Exception>)));
}

/*! \fn refreshServiceInfo()
 *\brief TODO
 */
void NodeInformation::refreshServiceInfo()
{
    Q_D(NodeInformation);
    d->nodeInfoQtPrx->begin_getServiceInfo(this, SLOT(_getServiceInfoSlot(::Diamesh::ServiceInfo)), this, SLOT(_remoteMethodFailedSlot(QSharedPointer<Ice::Exception>)));
}

/*! \fn refreshResourceInfo()
 *\brief TODO
 */
void NodeInformation::refreshResourceInfo()
{
    Q_D(NodeInformation);
    d->nodeInfoQtPrx->begin_getResourceInfo(this, SLOT(_getResourceInfoSlot(::HnsNode::ResourceInfo)), this, SLOT(_remoteMethodFailedSlot(QSharedPointer<Ice::Exception>)));
}

void NodeInformation::_getResourceInfoSlot(::HnsNode::ResourceInfo resourceInformation)
{
    _updateResourceInfo(resourceInformation);
}

void NodeInformation::_resourceInfoChangedSlot(HnsNode::ResourceInfo resourceInformation)
{
    _updateResourceInfo(resourceInformation);
}

void NodeInformation::_addEntity(long id)
{
    Q_D(NodeInformation);
    d->setEntityId.insert(id);
    emit addEntity(id);
}

void NodeInformation::_removeEntity(long id)
{
    Q_D(NodeInformation);
    d->setEntityId.remove(id);
    emit removeEntity(id);
}

/*! \fn getEntityIdList() const
 *\brief TODO
 */
const QSet<qlonglong> &NodeInformation::getEntityIdList() const
{
    Q_D(const NodeInformation);
    return d->setEntityId;
}

QString NodeInformation::_translatePlatform(QString platform)
{
    QString result = platform;
    if (platform == "VirtualBox")
        result = tr("VirtualBox");
    else if (platform == "OpenVZ")
        result = tr("OpenVZ");
    else if (platform == "KVM")
        result = tr("KVM");
    else if (platform == "HybridNetcard")
        result = tr("Hybrid netcard");
    else if (platform == "WifiAP")
        result = tr("Wi-Fi access point");
    else if (platform == "DynamipsRouter")
        result = tr("Dynamips routers");
    return result;
}

void NodeInformation::_updateResourceInfo(HnsNode::ResourceInfo resourceInformation)
{
    Q_D(NodeInformation);
    if (d->resourceInformation != resourceInformation) {
        d->resourceInformation = resourceInformation;
        emit infoChanged();
    }
}

void NodeInformation::_load(Hns::NodeInfoQPrx nodeInfoQtPrx)
{
    Q_D(NodeInformation);
    d->nodeInfoQtPrx = nodeInfoQtPrx;
    d->remoteCallCount = 8;
    d->nodeInfoQtPrx->begin_getName(this, SLOT(_loadNameSlot(::std::string)), this, SLOT(_remoteMethodFailedSlot(QSharedPointer<Ice::Exception>)));
    d->nodeInfoQtPrx->begin_getVersionInfo(this, SLOT(_loadVersionInfoSlot(::Diamesh::VersionInfo)), this, SLOT(_remoteMethodFailedSlot(QSharedPointer<Ice::Exception>)));
    d->nodeInfoQtPrx->begin_getSystemInfo(this, SLOT(_loadSystemInfoSlot(::Diamesh::SystemInfo)), this, SLOT(_remoteMethodFailedSlot(QSharedPointer<Ice::Exception>)));
    d->nodeInfoQtPrx->begin_getServiceInfo(this, SLOT(_loadServiceInfoSlot(::Diamesh::ServiceInfo)), this, SLOT(_remoteMethodFailedSlot(QSharedPointer<Ice::Exception>)));
    d->nodeInfoQtPrx->begin_getResourceInfo(this, SLOT(_loadResourceInfoSlot(::HnsNode::ResourceInfo)), this, SLOT(_remoteMethodFailedSlot(QSharedPointer<Ice::Exception>)));
    d->nodeInfoQtPrx->begin_getPlatforms(this, SLOT(_loadPlatformsSlot(::Hns::PlatformInfoMap)), this, SLOT(_remoteMethodFailedSlot(QSharedPointer<Ice::Exception>)));
    d->nodeInfoQtPrx->begin_getEntityDefineIds(this, SLOT(_loadEntityDefineIdsSlot(::Diamesh::LongList)), this, SLOT(_remoteMethodFailedSlot(QSharedPointer<Ice::Exception>)));
    d->nodeInfoQtPrx->begin_getNodeMonitor(this, SLOT(_loadNodeMonitorPrxSlot(::Hns::NodeMonitorPrx)), this, SLOT(_remoteMethodFailedSlot(QSharedPointer<Ice::Exception>)));
}

void NodeInformation::_callEnded()
{
    Q_D(NodeInformation);
    d->remoteCallCount--;
    _checkLoading();
}

void NodeInformation::_checkLoading()
{
    Q_D(NodeInformation);
    if (d->remoteCallCount == 0)
        _init();
}

void NodeInformation::_init()
{
    Q_D(NodeInformation);
    connect(d->nodeInfoQtPrx.data(), SIGNAL(resourcesChanged(HnsNode::ResourceInfo)), this, SLOT(_resourceInfoChangedSlot(HnsNode::ResourceInfo)));
    connect(d->nodeInfoQtPrx.data(), SIGNAL(addEntity(long)), this, SLOT(_addEntity(long)));
    connect(d->nodeInfoQtPrx.data(), SIGNAL(removeEntity(long)), this, SLOT(_removeEntity(long)));
    emit loaded();
}

void NodeInformation::_loadNameSlot(std::string name)
{
    Q_D(NodeInformation);
    d->name = QString::fromStdString(name);
    _callEnded();
}

void NodeInformation::_loadVersionInfoSlot(Diamesh::VersionInfo versionInformation)
{
    Q_D(NodeInformation);
    d->versionInformation = versionInformation;
    _callEnded();
}

void NodeInformation::_loadSystemInfoSlot(Diamesh::SystemInfo systemInformation)
{
    Q_D(NodeInformation);
    d->systemInformation = systemInformation;
    _callEnded();
}

void NodeInformation::_loadServiceInfoSlot(Diamesh::ServiceInfo serviceInformation)
{
    Q_D(NodeInformation);
    d->serviceInformation = serviceInformation;
    _callEnded();
}

void NodeInformation::_loadResourceInfoSlot(HnsNode::ResourceInfo resourceInformation)
{
    Q_D(NodeInformation);
    d->resourceInformation = resourceInformation;
    _callEnded();
}

void NodeInformation::_loadPlatformsSlot(Hns::PlatformInfoMap platformInformationMap)
{
    Q_D(NodeInformation);
    QMap<QString, Hns::PlatformInfo> resultMap;
    QMap<std::string, Hns::PlatformInfo> sourceMap(platformInformationMap);
    QMapIterator<std::string, Hns::PlatformInfo> iterator(sourceMap);
    while (iterator.hasNext()) {
        iterator.next();
        QString key = QString::fromStdString(iterator.key());
        resultMap.insert(key, iterator.value());
    }
    d->platforms = resultMap;
    _callEnded();
}

void NodeInformation::_loadEntityDefineIdsSlot(Diamesh::LongList entitiesIdList)
{
    Q_D(NodeInformation);
    d->setEntityId.clear();
    d->setEntityId = Utility::qLongSetFromLongList(entitiesIdList);
    _callEnded();
}

void NodeInformation::_loadNodeMonitorPrxSlot(Hns::NodeMonitorPrx nodeMonitorPrx)
{
    Q_D(NodeInformation);
    d->nodeMonitor = new NodeMonitor(this);
    connect(d->nodeMonitor, SIGNAL(loaded()), this, SLOT(_loadNodeMonitorSlot()));
    d->nodeMonitor->_load(Hns::NodeMonitorQPrx(nodeMonitorPrx, PrivateSessionAccessor::instance()->adapter()));
}

void NodeInformation::_loadNodeMonitorSlot()
{
    _callEnded();
}


/*******************************************************************************
* MasterInformation
*******************************************************************************/
MasterInformation::MasterInformation(QObject *parent)
    : ServiceDetails(new MasterInformationPrivate, parent)
{
}

MasterInformation::~MasterInformation()
{
}

/*! \fn refreshSystemInfo()
 *\brief TODO
 */
void MasterInformation::refreshSystemInfo()
{
    Q_D(MasterInformation);
    d->masterQtPrx->begin_getSystemInfo(this, SLOT(_getSystemInfoSlot(::Diamesh::SystemInfo)), this, SLOT(_remoteMethodFailedSlot(QSharedPointer<Ice::Exception>)));
}

/*! \fn refreshServiceInfo()
 *\brief TODO
 */
void MasterInformation::refreshServiceInfo()
{
    Q_D(MasterInformation);
    d->masterQtPrx->begin_getServiceInfo(this, SLOT(_getServiceInfoSlot(::Diamesh::ServiceInfo)), this, SLOT(_remoteMethodFailedSlot(QSharedPointer<Ice::Exception>)));
}

/*! \fn getQtTracerProxy()
 *\brief TODO
 */
Hns::TracerQPrx MasterInformation::getQtTracerProxy()
{
    Q_D(MasterInformation);
    return d->tracerQtPrx;
}

void MasterInformation::_load(Hns::MasterQPrx masterQtPrx)
{
    Q_D(MasterInformation);
    d->masterQtPrx = masterQtPrx;
    d->remoteCallCount = 5;
    d->masterQtPrx->begin_getName(this, SLOT(_loadNameSlot(::std::string)), this, SLOT(_remoteMethodFailedSlot(QSharedPointer<Ice::Exception>)));
    d->masterQtPrx->begin_getVersionInfo(this, SLOT(_loadVersionInfoSlot(::Diamesh::VersionInfo)), this, SLOT(_remoteMethodFailedSlot(QSharedPointer<Ice::Exception>)));
    d->masterQtPrx->begin_getSystemInfo(this, SLOT(_loadSystemInfoSlot(::Diamesh::SystemInfo)), this, SLOT(_remoteMethodFailedSlot(QSharedPointer<Ice::Exception>)));
    d->masterQtPrx->begin_getServiceInfo(this, SLOT(_loadServiceInfoSlot(::Diamesh::ServiceInfo)), this, SLOT(_remoteMethodFailedSlot(QSharedPointer<Ice::Exception>)));
    d->masterQtPrx->begin_getTracer(this, SLOT(_loadTracerSlot(::Hns::TracerPrx)), this, SLOT(_remoteMethodFailedSlot(QSharedPointer<Ice::Exception>)));
}

void MasterInformation::_callEnded()
{
    Q_D(MasterInformation);
    d->remoteCallCount--;
    _checkLoading();
}

void MasterInformation::_checkLoading()
{
    Q_D(MasterInformation);
    if (d->remoteCallCount == 0)
        _init();
}

void MasterInformation::_init()
{
    emit loaded();
}

void MasterInformation::_loadNameSlot(std::string name)
{
    Q_D(MasterInformation);
    d->name = QString::fromStdString(name);
    _callEnded();
}

void MasterInformation::_loadVersionInfoSlot(Diamesh::VersionInfo versionInformation)
{
    Q_D(MasterInformation);
    d->versionInformation = versionInformation;
    _callEnded();
}

void MasterInformation::_loadSystemInfoSlot(Diamesh::SystemInfo systemInformation)
{
    Q_D(MasterInformation);
    d->systemInformation = systemInformation;
    _callEnded();
}

void MasterInformation::_loadServiceInfoSlot(Diamesh::ServiceInfo serviceInformation)
{
    Q_D(MasterInformation);
    d->serviceInformation = serviceInformation;
    _callEnded();
}

void MasterInformation::_loadTracerSlot(Hns::TracerPrx tracerPrx)
{
    Q_D(MasterInformation);
    d->tracerQtPrx = Hns::TracerQPrx(tracerPrx, /*d->sessionHelper->adapter()*/PrivateSessionAccessor::instance()->adapter());
    _callEnded();
}


/*******************************************************************************
* PlatformMonitor
*******************************************************************************/
PlatformMonitor::PlatformMonitor(QObject* parent)
    : QObject(parent)
    , d_ptr(new PlatformMonitorPrivate)
{

}

PlatformMonitor::~PlatformMonitor()
{
    delete d_ptr;
}

/*! \fn getMasterName() const
 *\brief TODO
 */
QString PlatformMonitor::getMasterName() const
{
    Q_D(const PlatformMonitor);
    return d->masterInformation->getName();
}

/*! \fn getMasterInformation()
 *\brief TODO
 */
MasterInformation *PlatformMonitor::getMasterInformation()
{
    Q_D(PlatformMonitor);
    return d->masterInformation;
}

/*! \fn getNodeInformationHash() const
 *\brief TODO
 */
QHash<QString, NodeInformation *> PlatformMonitor::getNodeInformationHash() const
{
    Q_D(const PlatformMonitor);
    return d->nodeInformationHash;
}

/*! \fn getNodeNames() const
 *\brief TODO
 */
QStringList PlatformMonitor::getNodeNames() const
{
    Q_D(const PlatformMonitor);
    return d->nodeInformationHash.keys();
}

/*! \fn getNodeInformation(QString nodeName) const
 *\brief TODO
 */
NodeInformation * PlatformMonitor::getNodeInformation(QString nodeName) const
{
    Q_D(const PlatformMonitor);
    NodeInformation * result = 0;
    if (d->nodeInformationHash.contains(nodeName))
        result = d->nodeInformationHash.value(nodeName);
    return result;
}

void PlatformMonitor::_load(Hns::MasterQPrx masterQtPrx)
{
    Q_D(PlatformMonitor);
    d->masterQtPrx = masterQtPrx;
    MasterInformationLoader *masterLoader = new MasterInformationLoader(this);
    connect(masterLoader, SIGNAL(loaded(MasterInformation*)), this, SLOT(_loadMasterInformationSlot(MasterInformation*)));
    d->remoteCallCount = 2;
    masterLoader->load(d->masterQtPrx);
    masterQtPrx->begin_getNodeInfoMap(this, SLOT(_loadNodeInfoMapSlot(::Diamesh::NodeInfoMap)));
}

void PlatformMonitor::_callEnded()
{
    Q_D(PlatformMonitor);
    d->remoteCallCount--;
    _checkLoading();
}

void PlatformMonitor::_checkLoading()
{
    Q_D(PlatformMonitor);
    if (d->remoteCallCount == 0)
        _init();
}

void PlatformMonitor::_init()
{
    Q_D(PlatformMonitor);
    connect(d->masterQtPrx.data(), SIGNAL(nodeAdded(QString)), this, SLOT(_nodeAddedSlot(QString)));
    connect(d->masterQtPrx.data(), SIGNAL(nodeRemoved(QString)), this, SLOT(_nodeRemovedSlot(QString)));
    connect(d->masterQtPrx.data(), SIGNAL(refreshClassName(QString)), this, SIGNAL(refreshClassName(QString)));
    emit loaded();
}

NodeInformationLoader *PlatformMonitor::_createNodeInfoLoader()
{
    Q_D(PlatformMonitor);
    NodeInformationLoader *nodeLoader = new NodeInformationLoader(this);
    d->runningLoaders.append(nodeLoader);
    return nodeLoader;
}

void PlatformMonitor::_clearNodeInfoLoader(NodeInformationLoader *nodeLoader)
{
    Q_D(PlatformMonitor);
    if (nodeLoader && d->runningLoaders.contains(nodeLoader))
        d->runningLoaders.removeAll(nodeLoader);
}

void PlatformMonitor::_loadMasterInformationSlot(MasterInformation *masterInformation)
{
    Q_D(PlatformMonitor);
    d->masterInformation = masterInformation;
    _callEnded();
}

void PlatformMonitor::_loadNodeInfoMapSlot(Diamesh::NodeInfoMap nodeInformationMap)
{
    Q_D(PlatformMonitor);
    QMap<std::string, Diamesh::NodeInfoPrx> nodeInfoQMap(nodeInformationMap);
    int nodeInfoCount = nodeInfoQMap.count();
    if (nodeInfoCount > 0) {
        d->remoteCallCount += nodeInfoCount;
        QMapIterator<std::string, Diamesh::NodeInfoPrx> nodeInfoIterator(nodeInfoQMap);
        while(nodeInfoIterator.hasNext()) {
            nodeInfoIterator.next();
            Hns::NodeInfoPrx nodeInformationPrx = Hns::NodeInfoPrx::checkedCast(nodeInfoIterator.value());
            Hns::NodeInfoQPrx nodeInformationQtProxy = Hns::NodeInfoQPrx(nodeInformationPrx/*, d->sessionHelper.data()->adapter()*/, PrivateSessionAccessor::instance()->adapter());
            NodeInformationLoader *nodeLoader = _createNodeInfoLoader();
            connect(nodeLoader, SIGNAL(loaded(NodeInformation*)), this, SLOT(_loadNodeInformationSlot(NodeInformation*)));
            nodeLoader->load(nodeInformationQtProxy);
        }
    }
    _callEnded();
}

void PlatformMonitor::_loadNodeInformationSlot(NodeInformation *nodeInformation)
{
    Q_D(PlatformMonitor);
    _clearNodeInfoLoader(qobject_cast<NodeInformationLoader *>(sender()));
    d->nodeInformationHash.insert(nodeInformation->getName(), nodeInformation);
    _callEnded();
}

void PlatformMonitor::_nodeAddedSlot(QString nodeInfoProxyString)
{
    const Ice::CommunicatorPtr &communicator = PrivateSessionAccessor::instance()->communicator();
    Hns::NodeInfoPrx nodeInfoPrx = Hns::NodeInfoPrx::uncheckedCast(communicator->stringToProxy(nodeInfoProxyString.toStdString()));
    Hns::NodeInfoQPrx nodeInfoQPrx(nodeInfoPrx,PrivateSessionAccessor::instance()->adapter());
    NodeInformationLoader *nodeLoader = _createNodeInfoLoader();
    connect(nodeLoader, SIGNAL(loaded(NodeInformation*)), this, SLOT(_nodeInformationLoaderSlot(NodeInformation*)));
    nodeLoader->load(nodeInfoQPrx);
}

void PlatformMonitor::_nodeRemovedSlot(const QString &nodeName)
{
    Q_D(PlatformMonitor);
    if (d->nodeInformationHash.contains(nodeName)) {
        NodeInformation *nodeInformation = d->nodeInformationHash.take(nodeName);
        nodeInformation->deleteLater();
    }
    emit nodeRemoved(nodeName);
}

void PlatformMonitor::_nodeInformationLoaderSlot(NodeInformation *nodeInformation)
{
    Q_D(PlatformMonitor);
    _clearNodeInfoLoader(qobject_cast<NodeInformationLoader *>(sender()));
    QString nodeName = nodeInformation->getName();
    d->nodeInformationHash.insert(nodeName, nodeInformation);
    emit nodeAdded(nodeName);
}


/*******************************************************************************
* NodeInformationLoader
*******************************************************************************/
NodeInformationLoader::NodeInformationLoader(QObject *parent)
    : QObject(parent)
    , _nodeInformation(0)
{
}

NodeInformationLoader::~NodeInformationLoader()
{
}

void NodeInformationLoader::load(Hns::NodeInfoQPrx nodeInfoQtPrx)
{
    _nodeInformation = new NodeInformation(parent());
    connect(_nodeInformation, SIGNAL(loaded()), this, SLOT(_loadedSlot()));
    _nodeInformation->_load(nodeInfoQtPrx);
}

void NodeInformationLoader::_loadedSlot()
{
    emit loaded(_nodeInformation);
    deleteLater();
}


/*******************************************************************************
* MasterInformationLoader
*******************************************************************************/
MasterInformationLoader::MasterInformationLoader(QObject *parent)
    : QObject(parent)
    , _masterInformation(0)
{
}

MasterInformationLoader::~MasterInformationLoader()
{
}

void MasterInformationLoader::load(Hns::MasterQPrx masterQtPrx)
{
    _masterInformation = new MasterInformation(parent());
    connect(_masterInformation, SIGNAL(loaded()), this, SLOT(_loadedSlot()));
    _masterInformation->_load(masterQtPrx);
}

void MasterInformationLoader::_loadedSlot()
{
    emit loaded(_masterInformation);
    deleteLater();
}

} // namespace HnsClient
