/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef HNSCLIENT_CATALOGIMPORTER_P_H
#define HNSCLIENT_CATALOGIMPORTER_P_H

// hynesim hnsclient headers
#include "libhnsclient/entry.h"

// Generated hynesim headers
#include "common/proxies_catalogimporter.h"

// Qt headers
#include <QObject>
#include <QString>
#include <QMap>

namespace HnsClient {

/*******************************************************************************
* CatalogImporterPrivate
*******************************************************************************/
class CatalogImporterPrivate
{
public:
    explicit CatalogImporterPrivate();
    virtual ~CatalogImporterPrivate();

    Hns::CatalogImporterQPrx catalogImporterQPrx;
};

/*******************************************************************************
* ShowEntityEntryImportCookie
*******************************************************************************/
class ShowEntityEntryImportCookie : public QObject
{
    Q_OBJECT
public:
    explicit ShowEntityEntryImportCookie(QObject *receiver, const char *slot);
    ~ShowEntityEntryImportCookie();
    void emitCompleted(QMap<QString, HnsClient::EntityEntry *> importableMap);

signals:
    void completed(QMap<QString, HnsClient::EntityEntry *>);
};


/*******************************************************************************
* ImportArchiveCookie
*******************************************************************************/
class ImportArchiveCookie : public QObject
{
    Q_OBJECT
public:
    explicit ImportArchiveCookie(QObject *receiver, const char *slot);
    ~ImportArchiveCookie();
    void emitCompleted(bool success, const QString &message = QString());
signals:
    void completed(bool,QString);
};

/*******************************************************************************
* ImportFolderCookie
*******************************************************************************/
class ImportFolderCookie : public QObject
{
    Q_OBJECT
public:
    explicit ImportFolderCookie(QObject *receiver, const char *slot);
    ~ImportFolderCookie();
    void emitCompleted(bool success);

signals:
    void completed(bool success);
};

/*******************************************************************************
* ExportTopologyCookie
*******************************************************************************/
class ExportTopologyCookie : public QObject
{
    Q_OBJECT
public:
    explicit ExportTopologyCookie(QObject *receiver, const char *slot);
    ~ ExportTopologyCookie();
    void emitCompleted(bool result);
signals:
    void completed(bool result);
};

/*******************************************************************************
* ExportEntityEntryCookie
*******************************************************************************/
class ExportEntityEntryCookie : public QObject
{
    Q_OBJECT
public:
    explicit ExportEntityEntryCookie(qlonglong entryId, QObject *receiver, const char *slot);
    ~ ExportEntityEntryCookie();
    void emitCompleted(bool result);
signals:
    void completed(bool result);

private:
    qlonglong _entryId;
};


/*******************************************************************************
* GetArchiveListCookie
*******************************************************************************/
class GetArchiveListCookie : public QObject
{
    Q_OBJECT
public:
    explicit GetArchiveListCookie(QObject *receiver, const char *slot);
    ~GetArchiveListCookie();
    void emitCompleted(Hns::ImportableArchiveList list);
signals:
    void completed(Hns::ImportableArchiveList list);
};

} // namespace HnsClient

#endif //HNSCLIENT_CATALOGIMPORTER_P_H
