/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef HNSCLIENT_ENTITY_P_H
#define HNSCLIENT_ENTITY_P_H

// Generated hynesim headers
#include "common/proxies_entity.h"
#include "common/proxies_tracer.h"
#include "common/proxies_topologygate.h"
#include "common/proxies_hybridnetcard.h"
#include "common/proxies_wifiap.h"
#include "common/proxies_domain.h"
#include "common/layout.h"
#include "common/types.h"

// Qt headers
#include <QObject>
#include <QList>
#include <QStringList>
#include <QHash>
#include <QMap>

namespace HnsClient {

class GenericEntity;
class FileTransferClient;
class Topology;
class EntityEntry;

/*******************************************************************************
* DownloadCaptureFileCookie
*******************************************************************************/
class DownloadCaptureFileCookie : public QObject
{
    Q_OBJECT
public:
    explicit DownloadCaptureFileCookie(QString remoteFileName, QObject *receiver, const char *slot);
    ~DownloadCaptureFileCookie();
    void emitCompleted(HnsClient::FileTransferClient *client);
    QString remoteFileName();
signals:
    void completed(HnsClient::FileTransferClient *client);
private:
    QString _remoteFileName;
};

/*******************************************************************************
* RemoteDisplayInfoCookie
*******************************************************************************/
class RemoteDisplayInfoCookie : public QObject
{
    Q_OBJECT
public:
    explicit RemoteDisplayInfoCookie(int id, QObject *receiver, const char *slot);
    ~RemoteDisplayInfoCookie();
    void emitCompleted(Hns::RemoteDisplayInfo info);
signals:
    void completed(int id, Hns::RemoteDisplayInfo info);
private:
    int _id;
};

/*******************************************************************************
* SnapshotCookie
*******************************************************************************/
class SnapshotCookie : public QObject
{
    Q_OBJECT
public:
    explicit SnapshotCookie(const QString &snapshotName, QObject *receiver, const char *slot);
    ~SnapshotCookie();
    void emitCompleted(bool result);
signals:
    void completed(bool result, QString snapshotName);
private:
    QString _snapshotName;
};

/*******************************************************************************
* MemoryDumpCookie
*******************************************************************************/
class MemoryDumpCookie : public QObject
{
    Q_OBJECT
public:
    explicit MemoryDumpCookie(QObject *receiver, const char *slot);
    ~MemoryDumpCookie();
    void emitCompleted(int jobId);
    void setRemoteFileName(const QString &remoteFileName);
    void setLocalFileName(const QString &localFileName);
    QString remoteFileName() const;
    QString localFileName() const;

signals:
    void completed(int jobId);
};

/*******************************************************************************
* DownloadMemoryDumpCookie
*******************************************************************************/
class DownloadMemoryDumpCookie : public QObject
{
    Q_OBJECT
public:
    explicit DownloadMemoryDumpCookie(QObject *receiver, const char *slot);
    ~DownloadMemoryDumpCookie();
    void emitCompleted(HnsClient::FileTransferClient *client);

    QString remoteFileName() const;
    void setRemoteFileName(const QString &remoteFileName);
    QString localFileName() const;
    void setLocalFileName(const QString &localFileName);

signals:
    void completed(HnsClient::FileTransferClient *client);
private:
    QString _remoteFileName;
    QString _localFileName;
};


/*******************************************************************************
* GenericEntityPrivate
*******************************************************************************/
class GenericEntityPrivate
{
public:
    explicit GenericEntityPrivate();
    virtual ~GenericEntityPrivate();

    qlonglong entityId;
    Topology *topology;
    Hns::EntityQPrx entityQtProxy;
    Hns::TracerQPrx tracerQtProxy;
    EntityEntry *entry;
    EntityEntry *localEntry;
    Hns::Status status;
    QList<Hns::Status> allowedStatus;
    QMap<int, Hns::Plug> plugMap;                           // PlugId, Plug
    QStringList nodeList;                                   // Nodes which can define entity
    QHash<int, int> captureJobHash;                         // PlugId, JobId
    QList<int> availableInterfacesForCapture;
    QString nodeName;                                      // NodeName when define
    int capabilitiesFlags;
    bool remoteDisplayInfoLoaded;
    ::Hns::RemoteDisplayInfo remoteDisplayInfo;
    bool isImplementedFromModel;
    qlonglong cptStatus;
    int remoteCallCount;
    QMap<QString, QVariant> propertyInformationMap;
};


/*******************************************************************************
* WireEntityPrivate
*******************************************************************************/
class WireEntityPrivate : public GenericEntityPrivate
{
public:
    explicit WireEntityPrivate();
    virtual ~WireEntityPrivate();

    Hns::WireLayoutPtr layout;
    qlonglong fromId;
    qlonglong toId;
    int fromPlug;
    int toPlug;
    GenericEntity *toEntity;
    GenericEntity *fromEntity;
};


/*******************************************************************************
* EntityPrivate
*******************************************************************************/
class EntityPrivate : public GenericEntityPrivate
{
public:
    explicit EntityPrivate();
    virtual ~EntityPrivate();

    Hns::EntityLayoutPtr layout;
};


/*******************************************************************************
* DomainEntityPrivate
*******************************************************************************/
class DomainEntityPrivate : public EntityPrivate
{
public:
    explicit DomainEntityPrivate();
    virtual ~DomainEntityPrivate();

    QList<Hns::MemoryDumpInfo> memoryDumpInfoList;
    Hns::DomainQPrx domainQtProxy;
};


/*******************************************************************************
* SwitchEntityPrivate
*******************************************************************************/
class SwitchEntityPrivate : public EntityPrivate
{
public:
    explicit SwitchEntityPrivate();
    virtual ~SwitchEntityPrivate();
};


/*******************************************************************************
* HybridNetcardEntityPrivate
*******************************************************************************/
class HybridNetcardEntityPrivate : public EntityPrivate
{
public:
    explicit HybridNetcardEntityPrivate();
    virtual ~HybridNetcardEntityPrivate();

    QMap<QString, bool> macAddresses;           // MacAddress, isActive
    Hns::HybridNetcardQPrx hybridQtProxy;
};


/*******************************************************************************
* WifiAPEntityPrivate
*******************************************************************************/
class WifiAPEntityPrivate : public EntityPrivate
{
public:
    explicit WifiAPEntityPrivate();
    virtual ~WifiAPEntityPrivate();

    QStringList connectedPeers;
    Hns::WifiAPQPrx wifiQtProxy;
};


/*******************************************************************************
* DynamipsRouterEntityPrivate
*******************************************************************************/
class DynamipsRouterEntityPrivate : public EntityPrivate
{
public:
    explicit DynamipsRouterEntityPrivate();
    virtual ~DynamipsRouterEntityPrivate();
};

/*******************************************************************************
* TopologyGateEntityPrivate
*******************************************************************************/
class TopologyGateEntityPrivate : public EntityPrivate
{
public:
    explicit TopologyGateEntityPrivate();
    virtual ~TopologyGateEntityPrivate();

    Hns::TopologyGateQPrx topologyGateQtProxy;
};

} // namespace HnsClient

#endif //HNSCLIENT_ENTITY_P_H
