/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef HNSCLIENT_ENTITYENTRYCATALOG_P_H
#define HNSCLIENT_ENTITYENTRYCATALOG_P_H

// hynesim hnsclient headers
#include "libhnsclient/private/catalog_p.h"

// Generated hynesim headers
#include "common/proxies_entitycatalog.h"
#include "common/proxies_topologymanager.h"

// Qt headers
#include <QObject>
#include <QList>
#include <QHash>

namespace HnsClient {

class Job;
class FolderEntry;
class EntityEntry;

/*******************************************************************************
* GetEntityEntryCookie
*******************************************************************************/
class GetEntityEntryCookie : public QObject
{
    Q_OBJECT
public:
    GetEntityEntryCookie(qlonglong entryId);
    ~GetEntityEntryCookie();
    qlonglong entryId() const;

private:
    qlonglong _entryId;
};

/*******************************************************************************
* AddFolderCookie
*******************************************************************************/
class AddFolderCookie : public QObject
{
    Q_OBJECT
public:
    AddFolderCookie(QString folderName);
    ~AddFolderCookie();
    QString getFolderName();

private:
    QString _folderName;
};


/*******************************************************************************
* UpdateFolderCookie
*******************************************************************************/
class UpdateFolderCookie : public QObject
{
    Q_OBJECT
public:
    UpdateFolderCookie(Hns::FolderPtr folder);
    ~UpdateFolderCookie();
    Hns::FolderPtr folder();

private:
    Hns::FolderPtr _folder;
};


/*******************************************************************************
* DeleteFolderCookie
*******************************************************************************/
class DeleteFolderCookie : public QObject
{
    Q_OBJECT
public:
    DeleteFolderCookie(QString folderName, qlonglong Id);
    ~DeleteFolderCookie();
    QString getFolderName();
    qlonglong getId();

private:
    QString _folderName;
    qlonglong _Id;
};


/*******************************************************************************
* UpdateEntryCookie
*******************************************************************************/
class UpdateEntryCookie : public QObject
{
    Q_OBJECT
public:
    UpdateEntryCookie(Hns::EntityEntryPtr entry);
    ~UpdateEntryCookie();
    Hns::EntityEntryPtr entityEntry() const;
    QString getEntryName() const;
    qlonglong getEntryId() const;
private:
    Hns::EntityEntryPtr _entry;
};


/*******************************************************************************
* DeleteEntityCookie
*******************************************************************************/
class DeleteEntryCookie : public QObject
{
    Q_OBJECT
public:
    DeleteEntryCookie(QString entryName, qlonglong entryId);
    ~DeleteEntryCookie();
    QString getEntryName() const;
    qlonglong getEntryId() const;
private:
    QString _entryName;
    qlonglong _entryId;
};


/*******************************************************************************
* CloneEntityEntryCookie
*******************************************************************************/
class CloneEntryCookie : public QObject
{
    Q_OBJECT
public:
    CloneEntryCookie(EntityEntry *sourceEntry);
    ~CloneEntryCookie();
    QString getEntryName();
    qlonglong getEntryId();
private:
    QString _entryName;
    qlonglong _entryId;
};


/*******************************************************************************
* BuildModelFromEntityEntryCookie
*******************************************************************************/
class BuildModelFromEntityEntryCookie : public QObject
{
    Q_OBJECT
public:
    BuildModelFromEntityEntryCookie(EntityEntry *sourceEntry);
    ~BuildModelFromEntityEntryCookie();
    QString getEntryName();
    Ice::Long getEntryID();
    
private:
    QString _entryName;
    Ice::Long _entryId;
};


/*******************************************************************************
* DiscardSaveStateCookie
*******************************************************************************/
class DiscardSaveStateCookie : public QObject
{
    Q_OBJECT
public:
    DiscardSaveStateCookie(EntityEntry * sourceEntry);
    ~DiscardSaveStateCookie();
    Ice::Long getEntryID();
    QString getEntryName();
    
private:
    QString _entryName;
    Ice::Long _entryID;
};


/*******************************************************************************
* GetEntityEntryUpdateCookie
*******************************************************************************/
class GetEntityEntryUpdateCookie : public QObject
{
    Q_OBJECT
public:
    GetEntityEntryUpdateCookie(qlonglong entryId);
    ~GetEntityEntryUpdateCookie();
    qlonglong getEntryId();

private:
    qlonglong _entryId;
};


/*******************************************************************************
* GetFolderUpdateCookie
*******************************************************************************/
class GetFolderUpdateCookie : public QObject
{
    Q_OBJECT
public:
    GetFolderUpdateCookie(qlonglong entryId);
    ~GetFolderUpdateCookie();
    qlonglong getFolderId();

private:
    qlonglong _entryId;
};

/*******************************************************************************
* GetUnusedUUIDCookie
*******************************************************************************/
class GetUnusedUUIDCookie : public QObject
{
    Q_OBJECT
public:
    GetUnusedUUIDCookie(qlonglong entityId, QObject *receiver, const char *slot);
    ~GetUnusedUUIDCookie();
    void emitCompleted(QString uuid);
signals:
    void completed(QString uuid, qlonglong entityId);
private:
    qlonglong _entityId;
};
/*******************************************************************************
* EntityEntryCatalogPrivate
*******************************************************************************/
class EntityEntryCatalogPrivate : public CatalogPrivate
{
public:
    EntityEntryCatalogPrivate();
    virtual ~EntityEntryCatalogPrivate();

    Hns::EntityCatalogQPrx catalogQtProxy;
    Hns::TopologyManagerQPrx topologyManagerQtProxy;
    QHash<qlonglong, FolderEntry *> folderHash;                        // FolderEntryID, FolderEntry
    QHash<qlonglong, EntityEntry *> entityHash;                        // EntityEntryID, EntityEntry
    QList<qlonglong> waitForFolder;
    QHash<qlonglong, qlonglong> waitForDelete;
    QHash<qlonglong, qlonglong> waitForEntity;
    QHash<int, qlonglong> cloneJobToEntryHash;
    QHash<int, Job *> cloneJobHash;
    QList<qlonglong> loadingEntryList;
};

}//namespace HnsClient
#endif
