/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef HNSCLIENT_ENTRY_P_H
#define HNSCLIENT_ENTRY_P_H

// hynesim hnsclient headers
#include "libhnsclient/common.h"

// Generated hynesim headers
#include "common/catalogitem.h"
#include "common/entry.h"
#include "common/folder.h"
#include "common/entityentry.h"
#include "common/domainentry.h"
#include "common/switchentry.h"
#include "common/hybridnetcardentry.h"
#include "common/wifiapentry.h"
#include "common/dynamipsrouterentry.h"
#include "common/entryproperties.h"
#include "common/entityentryproperties.h"
#include "common/wireentryproperties.h"
#include "common/domainentryproperties.h"
#include "common/switchentryproperties.h"
#include "common/topologygateentry.h"
#include "common/proxies_entityinfo.h"
#include "common/proxies_topologymanager.h"
#include "common/catalog.h"

// Qt headers
#include <QString>
#include <QMap>
#include <QPixmap>

namespace HnsClient {

class CatalogItem;
class EntityEntry;
class FolderEntry;
class EntityEntryInfoUpdater;

/*******************************************************************************
* CatalogItemPrivate
*******************************************************************************/
class CatalogItemPrivate
{
public:
    explicit CatalogItemPrivate();
    CatalogItemPrivate(const CatalogItemPrivate &d);
    virtual ~CatalogItemPrivate();

    Hns::CatalogItemPtr catalogItem;
    SynchronizationStatus syncStatus;
    Diamesh::ACLList aclList;
};


/*******************************************************************************
* FolderEntryPrivate
*******************************************************************************/
class FolderEntryPrivate : public CatalogItemPrivate
{
public:
    explicit FolderEntryPrivate();
    FolderEntryPrivate(const FolderEntryPrivate &d, const Hns::FolderPtr &folder);
    virtual ~FolderEntryPrivate();
};


/*******************************************************************************
* EntryPrivate
*******************************************************************************/
class EntryPrivate : public CatalogItemPrivate
{
public:
    explicit EntryPrivate();
    EntryPrivate(const EntryPrivate &d, const Hns::EntryPtr &clonedEntry, const Hns::EntryPropertiesPtr &clonedProperties);
    virtual ~EntryPrivate();

    bool isLocked;
    ::Hns::LockType lockType;
    Hns::EntryPropertiesPtr entryProperties;
};


/*******************************************************************************
* EntityEntryInfoUpdaterPrivate
*******************************************************************************/
class EntityEntryInfoUpdaterPrivate
{
public:
    explicit EntityEntryInfoUpdaterPrivate();
    virtual ~EntityEntryInfoUpdaterPrivate();

    EntityEntry *entry;
    Hns::EntityInfoQPrx entityInfo;
    Hns::TopologyManagerQPrx topologyManager;
    Ice::Long cptStatus;
};


/*******************************************************************************
* EntityEntryPrivate
*******************************************************************************/
class EntityEntryPrivate : public EntryPrivate
{
public:
    explicit EntityEntryPrivate();
    EntityEntryPrivate(const EntityEntryPrivate &d, const Hns::EntityEntryPtr &clonedEntry, const Hns::EntityEntryPropertiesPtr &clonedProperties);
    virtual ~EntityEntryPrivate();

    EntityEntryInfoUpdater *infoUpdater;
    int entityStatus;
    double cloneProgress;
    bool isCloning;
};


/*******************************************************************************
* WireEntryPrivate
*******************************************************************************/
class WireEntryPrivate : public EntityEntryPrivate
{
public:
    explicit WireEntryPrivate();
    WireEntryPrivate(const WireEntryPrivate &d, const Hns::WireEntryPtr &clonedEntry, const Hns::WireEntryPropertiesPtr &clonedProperties);
    virtual ~WireEntryPrivate();

    QString strFrom;
    QString strTo;
    QMap<QString, QString> supportedImplementation;
};


/*******************************************************************************
* DomainEntryPrivate
*******************************************************************************/
class DomainEntryPrivate : public EntityEntryPrivate
{
public:
    explicit DomainEntryPrivate();
    DomainEntryPrivate(const DomainEntryPrivate &d, const Hns::DomainEntryPtr &clonedEntry, const Hns::DomainEntryPropertiesPtr &clonedProperties);
    virtual ~DomainEntryPrivate();

    bool macAdressIsUsed;
};


/*******************************************************************************
* SwitchEntryPrivate
*******************************************************************************/
class SwitchEntryPrivate : public EntityEntryPrivate
{
public:
    explicit SwitchEntryPrivate();
    SwitchEntryPrivate(const SwitchEntryPrivate &d, const Hns::SwitchEntryPtr &clonedEntry, const Hns::SwitchEntryPropertiesPtr &clonedProperties);
    virtual ~SwitchEntryPrivate();

    QMap<QString, QString> supportedImplementation;
};


/*******************************************************************************
* HybridNetcardEntryPrivate
*******************************************************************************/
class HybridNetcardEntryPrivate : public EntityEntryPrivate
{
public:
    explicit HybridNetcardEntryPrivate();
    HybridNetcardEntryPrivate(const HybridNetcardEntryPrivate &d, const Hns::HybridNetcardEntryPtr &clonedEntry, const Hns::EntityEntryPropertiesPtr &clonedProperties);
    virtual ~HybridNetcardEntryPrivate();
};


/*******************************************************************************
* WifiAPEntryPrivate
*******************************************************************************/
class WifiAPEntryPrivate : public EntityEntryPrivate
{
public:
    explicit WifiAPEntryPrivate();
    WifiAPEntryPrivate(const WifiAPEntryPrivate &d, const Hns::WifiAPEntryPtr &clonedEntry, const Hns::EntityEntryPropertiesPtr &clonedProperties);
    virtual ~WifiAPEntryPrivate();
};


/*******************************************************************************
* DynamipsRouterEntryPrivate
*******************************************************************************/
class DynamipsRouterEntryPrivate : public EntityEntryPrivate
{
public:
    explicit DynamipsRouterEntryPrivate();
    DynamipsRouterEntryPrivate(const DynamipsRouterEntryPrivate &d, const Hns::DynamipsRouterEntryPtr &clonedEntry, const Hns::EntityEntryPropertiesPtr &clonedProperties);
    virtual ~DynamipsRouterEntryPrivate();
    bool macAddressIsUsed;
};


/*******************************************************************************
* TopologyGateEntryPrivate
*******************************************************************************/
class TopologyGateEntryPrivate : public EntityEntryPrivate
{
public :
    explicit TopologyGateEntryPrivate();
    TopologyGateEntryPrivate(const TopologyGateEntryPrivate &d, const Hns::TopologyGateEntryPtr &clonedEntry, const Hns::EntityEntryPropertiesPtr &clonedProperties);
    ~TopologyGateEntryPrivate();
};


/*******************************************************************************
* TopologyEntryPrivate
*******************************************************************************/
class TopologyEntryPrivate : public EntryPrivate
{
public:
    explicit TopologyEntryPrivate();
    TopologyEntryPrivate(const TopologyEntryPrivate &d, const Hns::TopologyEntryPtr &clonedEntry, const Hns::EntryPropertiesPtr &clonedProperties);
    virtual ~TopologyEntryPrivate();

    QPixmap screenshot;
};

} // namespace HnsClient
#endif//HNSCLIENT_ENTRY_P_H
