/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef HNSCLIENT_PLATFORMMONITOR_P_H
#define HNSCLIENT_PLATFORMMONITOR_P_H

// Generated hynesim headers
#include "common/proxies_nodemonitor.h"
#include "common/proxies_nodeinfo.h"
#include "common/proxies_master.h"
#include "common/proxies_tracer.h"

// diamesh headers
#include <diamesh/info.h>

// Qt headers
#include <QMap>
#include <QHash>
#include <QSet>

namespace HnsClient {

class MasterInformation;
class NodeInformation;
class NodeInformationLoader;
class NodeMonitor;

/*******************************************************************************
* NodeMonitorPrivate
*******************************************************************************/
class NodeMonitorPrivate
{
public:
    explicit NodeMonitorPrivate();
    virtual ~NodeMonitorPrivate();

    Hns::NodeMonitorQPrx nodeMonitorQtPrx;
    QMap<Ice::Long, Hns::EntityMonitoringDataPtr> entityMonitoringDataMap;
    int remoteCallCount;
};

/*******************************************************************************
* ServiceDetailsPrivate
*******************************************************************************/
class ServiceDetailsPrivate
{
public:
    explicit ServiceDetailsPrivate();
    virtual ~ServiceDetailsPrivate();

    QString name;
    Diamesh::VersionInfo versionInformation;
    Diamesh::SystemInfo systemInformation;
    Diamesh::ServiceInfo serviceInformation;
};


/*******************************************************************************
* NodeInformationPrivate
*******************************************************************************/
class NodeInformationPrivate : public ServiceDetailsPrivate
{
public:
    explicit NodeInformationPrivate();
    virtual ~NodeInformationPrivate();

    Hns::NodeInfoQPrx nodeInfoQtPrx;
    HnsNode::ResourceInfo resourceInformation;
    QMap<QString, Hns::PlatformInfo> platforms;
    QSet<qlonglong> setEntityId;
    int remoteCallCount;
    NodeMonitor *nodeMonitor;
};


/*******************************************************************************
* MasterInformationPrivate
*******************************************************************************/
class MasterInformationPrivate : public ServiceDetailsPrivate
{
public:
    explicit MasterInformationPrivate();
    virtual ~MasterInformationPrivate();

    Hns::MasterQPrx masterQtPrx;
    Hns::TracerQPrx tracerQtPrx;
    int remoteCallCount;
};


/*******************************************************************************
* PlatformMonitorPrivate
*******************************************************************************/
class PlatformMonitorPrivate
{
public:
    explicit PlatformMonitorPrivate();
    virtual ~PlatformMonitorPrivate();

    MasterInformation *masterInformation;
    QHash<QString, NodeInformation *> nodeInformationHash;
    Hns::MasterQPrx masterQtPrx;
    int remoteCallCount;
    QList<NodeInformationLoader *> runningLoaders;
};


/*******************************************************************************
* NodeInformationLoader
*******************************************************************************/
class NodeInformationLoader : public QObject
{
    Q_OBJECT
public:
    explicit NodeInformationLoader(QObject *parent = 0);
    ~NodeInformationLoader();
    void load(Hns::NodeInfoQPrx nodeInfoQtPrx);

signals:
    void loaded(NodeInformation *nodeInformation);

protected slots:
    void _loadedSlot();

private:
    NodeInformation *_nodeInformation;
};


/*******************************************************************************
* MasterInformationLoader
*******************************************************************************/
class MasterInformationLoader : public QObject
{
    Q_OBJECT
public:
    explicit MasterInformationLoader(QObject *parent = 0);
    ~MasterInformationLoader();
    void load(Hns::MasterQPrx masterQtPrx);

signals:
    void loaded(MasterInformation *masterInformation);

protected slots:
    void _loadedSlot();

private:
    MasterInformation *_masterInformation;
};

} // namespace HnsClient

#endif //HNSCLIENT_PLATFORMMONITOR_P_H
