/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef HNSCLIENT_SESSION_P_H
#define HNSCLIENT_SESSION_P_H

// Generated hynesim headers
#include "common/proxies_master.h"
#include "common/proxies_catalogimporter.h"
#include "common/proxies_dynamipsrouterdb.h"
#include "common/proxies_entitycatalog.h"
#include "common/proxies_entrypropertiesmanager.h"
#include "common/proxies_jobmanager.h"
#include "common/proxies_macaddressmanager.h"
#include "common/proxies_topologycatalog.h"
#include "common/proxies_topologymanager.h"
#include "common/proxies_tracer.h"
#include "common/proxies_customresourcemanager.h"

// diamesh headers
#include <diamesh/sessionhelper.h>
#include <diamesh/proxies_sessionManager.h>
#include <diamesh/proxies_hsetmanager.h>
#include <diamesh/proxies_accessrights.h>

// Qt headers
#include <QSharedPointer>

namespace HnsClient {

class CatalogImporter;
class DynamipsRouterDB;
class EntityEntryCatalog;
class EntryPropertiesManager;
class JobManager;
class MacAddressManager;
class PlatformMonitor;
class TopologyEntryCatalog;
class TopologyManager;
class TracerManager;
class UserManager;
class EntityInfoList;
class HSetManager;
class ACLManager;
class ResourceManager;
class Session;

/*******************************************************************************
* SessionPrivate
*******************************************************************************/
class SessionPrivate {
public:
    explicit SessionPrivate();
    virtual ~SessionPrivate();

    void clear();

    QSharedPointer<Diamesh::SessionHelper> sessionHelper;
    Diamesh::SessionManagerQPrx sessionManagerQtProxy;
    Diamesh::SessionInfo sessionInformation;
    Diamesh::UserManagerQPrx userManagerQtProxy;
    Diamesh::AuthManagerPrx authManagerPrx;
    Diamesh::HSetManagerQPrx hSetManagerQtProxy;
    Diamesh::ACLManagerQPrx aclManagerQtProxy;

    Hns::MasterQPrx masterQtProxy;

    Hns::CatalogImporterQPrx catalogImporterQtProxy;
    Hns::DynamipsRouterDBQPrx dynamipsRouterDBQtProxy;
    Hns::EntityCatalogQPrx entityCatalogQtProxy;
    Hns::EntryPropertiesManagerQPrx entryPropertiesManagerQtProxy;
    Hns::JobManagerQPrx jobManagerQtProxy;
    Hns::MacAddressManagerQPrx macAddressManagerQtProxy;
    Hns::TopologyCatalogQPrx topologyCatalogQtProxy;
    Hns::TopologyManagerQPrx topologyManagerQtProxy;
    Hns::CustomResourceManagerQPrx resourceManagerQtProxy;

    bool isAdvancedMode;

    CatalogImporter *catalogImporter;
    DynamipsRouterDB *dynamipsRouterDB;
    EntityEntryCatalog *entityEntryCatalog;
    EntryPropertiesManager *entryPropertiesManager;
    JobManager *jobManager;
    MacAddressManager *macAddressManager;
    PlatformMonitor *platformMonitor;
    TopologyEntryCatalog *topologyEntryCatalog;
    TopologyManager *topologyManager;
    TracerManager *tracerManager;
    UserManager *userManager;
    HSetManager *hSetManager;
    ACLManager *aclManager;
    ResourceManager *resourceManager;
    EntityInfoList *entityInfoList;

    QMap<QString, QString> entityCatalogStorageMap;
    QMap<QString, QString> topologyCatalogStorageMap;

    int remoteCallCount;
    bool jobManagerLoaded;

private:
    void _deletePtr();
    void _clearPtr();
    void _clearProxies();
};


/*******************************************************************************
* SessionLoader
*******************************************************************************/
class SessionLoader : public QObject
{
    Q_OBJECT
public:
    static SessionLoader *instance();
    ~SessionLoader();
    void load(QSharedPointer<Diamesh::SessionHelper> sessionHelper, QObject *sessionParent = 0);
    bool isLoading();
    bool isCompleted();
    Session *getSession();

signals:
    void completed();

protected:
    explicit SessionLoader(QObject *parent = 0);

private slots:
    void _sessionLoaded();

private:
    bool _isLoading;
    bool _completed;
    Session *_session;
    static SessionLoader *_singleton;
};

} // namespace HnsClient

#endif //HNSCLIENT_SESSION_P_H
