/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef HNSCLIENT_TOPOLOGY_P_H
#define HNSCLIENT_TOPOLOGY_P_H

// Generated hynesim headers
#include "common/proxies_topology.h"
#include "common/proxies_tracer.h"

// Qt headers
#include <QObject>
#include <QSignalMapper>
#include <QList>
#include <QHash>
#include <QMap>

namespace HnsClient {

class TopologyEntry;
class GenericEntity;
class Entity;
class WireEntity;

/*******************************************************************************
* GetScreenshotCookie
*******************************************************************************/
class GetScreenshotCookie : public QObject
{
    Q_OBJECT
public:
    explicit GetScreenshotCookie(Hns::TopologyEntryPtr entry);
    ~GetScreenshotCookie();
    Hns::TopologyEntryPtr getPtr();
private:
    Hns::TopologyEntryPtr _entry;
};


/*******************************************************************************
* GetEntityLayoutCookie
*******************************************************************************/
class GetEntityLayoutCookie : public QObject
{
    Q_OBJECT
public:
    explicit GetEntityLayoutCookie(Entity *entity);
    ~GetEntityLayoutCookie();
    Entity *entity();

private:
    Entity *_entity;
};


/*******************************************************************************
* GetWireLayoutCookie
*******************************************************************************/
class GetWireLayoutCookie : public QObject
{
    Q_OBJECT
public:
    explicit GetWireLayoutCookie(WireEntity *entity);
    ~GetWireLayoutCookie();
    WireEntity *entity();

private:
    WireEntity *_entity;
};


/*******************************************************************************
* RefreshEntryCookie
*******************************************************************************/
class RefreshEntryCookie : public QObject
{
    Q_OBJECT
public:
    explicit RefreshEntryCookie(QObject *receiver, const char *slot);
    ~RefreshEntryCookie();
    void emitRefreshed(QString uuid);

signals:
    void refreshed(QString newUUID);
};


/*******************************************************************************
* CanBuildWithNodeCookie
*******************************************************************************/
class CanBuildWithNodeCookie : public QObject
{
    Q_OBJECT
public:
    explicit CanBuildWithNodeCookie(QString nodeName);
    ~CanBuildWithNodeCookie();
    QString nodeName();

private:
    QString _nodeName;
};


/*******************************************************************************
* EntityCookie
*******************************************************************************/
class EntityCookie : public QObject
{
    Q_OBJECT
public:
    explicit EntityCookie(qlonglong entityId);
    ~EntityCookie();
    qlonglong entityId();

private:
    qlonglong _entityId;
};


/*******************************************************************************
* CloneEntityFromEntryIdCookie
*******************************************************************************/
class CloneEntityFromEntryIdCookie : public QObject
{
    Q_OBJECT
public:
    explicit CloneEntityFromEntryIdCookie(qlonglong entryId, Hns::EntityLayoutPtr layout, QObject *receiver, const char *slot);
    ~CloneEntityFromEntryIdCookie();
    void emitCompleted(int jobId);

signals:
    void completed(int jobId, qlonglong entryId, int posX, int posY);

private:
    qlonglong _entryId;
    Hns::EntityLayoutPtr _layout;
};


/*******************************************************************************
* UpdateTopologyACLCookie
*******************************************************************************/
class UpdateTopologyACLCookie : public QObject
{
    Q_OBJECT
public:
    explicit UpdateTopologyACLCookie(Hns::TopologyEntryPtr entry);
    ~UpdateTopologyACLCookie();
    QString getEntryName() const;
    qlonglong getID() const;
    Hns::TopologyEntryPtr topologyEntry();

private:
    Hns::TopologyEntryPtr _entry;
};

/*******************************************************************************
* PlugEntityCookie
*******************************************************************************/
class PlugEntityCookie : public QObject
{
    Q_OBJECT
public:
    explicit PlugEntityCookie(Hns::TopologyLink link, QObject *receiver, const char *slot);
    ~PlugEntityCookie();
    void emitCompleted(bool result);

signals:
    void completed(Hns::TopologyLink link);

private:
    Hns::TopologyLink _link;
};

/*******************************************************************************
* TopologyPrivate
*******************************************************************************/
class TopologyPrivate
{
public:
    explicit TopologyPrivate();
    virtual ~TopologyPrivate();

    Hns::TopologyQPrx topologyQtProxy;
    Hns::TracerQPrx tracerQtProxy;
    qlonglong topologyId;
    TopologyEntry *entry;
    bool isModified;
    Hns::TopologyLayout layout;
    QList<qlonglong> connectedUsers;

    bool contentLoaded;
    int entitiesLoadingCount;
    int wiresLoadingCount;
    QHash<qlonglong, GenericEntity*> entityHash;            // EntityID, Entity
    QHash<qlonglong, GenericEntity*> wireHash;              // WireEntityID, WireEntity
    QList<qlonglong> pendingWires;

    QMultiHash<qlonglong, Hns::TopologyLink> linksHash;          //WireId, TopologyLink

    QMap<int, Hns::BalancingBehaviorInfo> balancingBehaviorMap;
    QSignalMapper *entityMapper;
    QSignalMapper *wireMapper;

    int remoteCallCount;
    bool loaded;
    bool wireLoaded;

    int minWidth;
    int minHeight;
};

} // namespace HnsClient

#endif //HNSCLIENT_TOPOLOGY_P_H
