/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef HNSCLIENT_TOPOLOGYENTRYCATALOG_P_H
#define HNSCLIENT_TOPOLOGYENTRYCATALOG_P_H

// hynesim hnsclient headers
#include "libhnsclient/private/catalog_p.h"

// Generated hynesim headers
#include "common/proxies_topologycatalog.h"

// Qt headers
#include <QObject>
#include <QString>
#include <QList>
#include <QHash>

namespace HnsClient {

class TopologyEntry;

/*******************************************************************************
* GetEntityEntryCookie
*******************************************************************************/
class GetTopologyEntryCookie : public QObject
{
    Q_OBJECT
public:
    GetTopologyEntryCookie(qlonglong entryId);
    ~GetTopologyEntryCookie();
    qlonglong entryId() const;

private:
    qlonglong _entryId;
};


/*******************************************************************************
* TopologyEntryCatalogCookie
*******************************************************************************/
class TopologyEntryCatalogCookie : public QObject
{
    Q_OBJECT
public:
    explicit TopologyEntryCatalogCookie(Hns::TopologyEntryPtr entry);
    ~TopologyEntryCatalogCookie();
    Hns::TopologyEntryPtr getPtr() const;

private:
    Hns::TopologyEntryPtr _entry;
};


/*******************************************************************************
* UpdateTopologyEntryCookie
*******************************************************************************/
class UpdateTopologyEntryCookie : public QObject
{
    Q_OBJECT
public:
    explicit UpdateTopologyEntryCookie(Hns::TopologyEntryPtr entry);
    ~UpdateTopologyEntryCookie();
    QString getEntryName() const;
    qlonglong getID() const;
    Hns::TopologyEntryPtr topologyEntry();

private:
    Hns::TopologyEntryPtr _entry;
};


/*******************************************************************************
* GetTopologyEntryUpdateCookie
*******************************************************************************/
class GetTopologyEntryUpdateCookie : public QObject
{
    Q_OBJECT
public:
    explicit GetTopologyEntryUpdateCookie(qlonglong entryId);
    ~GetTopologyEntryUpdateCookie();
    qlonglong topologyEntryId() const;

private:
    qlonglong _entryId;
};


/*******************************************************************************
* TopologyEntryCatalogPrivate
*******************************************************************************/
class TopologyEntryCatalogPrivate : public CatalogPrivate
{

public:
    explicit TopologyEntryCatalogPrivate();
    virtual ~TopologyEntryCatalogPrivate();
    
    Hns::TopologyCatalogQPrx catalogQtProxy;
    QHash<qlonglong, TopologyEntry*> topologyHash;
    QList<qlonglong> loadingEntryList;
};

} // namespace HnsClient

#endif //HNSCLIENT_TOPOLOGYENTRYCATALOG_P_H
