/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef HNSCLIENT_TOPOLOGYMANAGER_P_H
#define HNSCLIENT_TOPOLOGYMANAGER_P_H

// Generated hynesim headers
#include "common/proxies_topologymanager.h"

// Qt headers
#include <QObject>
#include <QString>
#include <QHash>
#include <QSignalMapper>

namespace HnsClient {

class EntityInfoList;
class Topology;

/*******************************************************************************
* TopologyCookie
*******************************************************************************/
class TopologyCookie : public QObject
{
    Q_OBJECT
public:
    explicit TopologyCookie(QString topologyUUID, QString topologyShortName, qlonglong entryId, QObject *receiver, const char *slot);
    ~TopologyCookie();
    void emitCompleted(bool result);
    qlonglong entryId();
    QString shortName();

signals:
    void completed(QString topologyUUID, QString topologyShortName, bool result);

private:
    QString _topologyUUID;
    QString _topologyShortName;
    qlonglong _entryId;
};


/*******************************************************************************
* LoadTopologyCookie
*******************************************************************************/
class LoadTopologyCookie : public QObject
{
    Q_OBJECT
public:
    explicit LoadTopologyCookie(QObject *receiver, const char *slot);
    ~LoadTopologyCookie();
    void setTopology(HnsClient::Topology *topology);
    Topology *topology();
    void postLoading(HnsClient::Topology *topology);
    void emitCompleted(HnsClient::Topology *topology);

signals:
    void completed(HnsClient::Topology *topology);

private:
    HnsClient::Topology *_topology;
};


/*******************************************************************************
* SaveTopologyCookie
*******************************************************************************/
class SaveTopologyCookie : public QObject
{
    Q_OBJECT
public:
    explicit SaveTopologyCookie(qlonglong topologyId, QString topologyUUID, QObject *receiver, const char *slot);
    ~SaveTopologyCookie();
    qlonglong topologyId();
    QString topologyUUID();
    void emitCompleted(bool result, QString exceptionMessage = QString());

signals:
    void completed(QString uuid, bool result, QString exceptionMessage);

private:
    qlonglong _topologyId;
    QString _topologyUUID;
};


/*******************************************************************************
* SaveAsTopologyCookie
*******************************************************************************/
class SaveAsTopologyCookie : public QObject
{
    Q_OBJECT
public:
    explicit SaveAsTopologyCookie(/*QString topologyUUID*/HnsClient::Topology *topology, QObject *receiver, const char *slot);
    ~SaveAsTopologyCookie();
    QString topologyUUID();
    void setResult(bool result, QString exceptionMessage = QString());
    HnsClient::Topology *topology();
    void finish();

public slots:
    void entryRefreshedSlot(QString uuid);

signals:
    void completed(QString oldUuid, QString newUuid, bool result, QString exceptionMessage);
    void updateHash(HnsClient::Topology*, QString newUuid, SaveAsTopologyCookie *cookie);

private:
    HnsClient::Topology *_topology;
    QString _topologyUUID;
    QString _newTopologyUUID;
};


/*******************************************************************************
* SwitchTopologyTypeCookie
*******************************************************************************/
class SwitchTopologyTypeCookie : public QObject
{
    Q_OBJECT
public:
    explicit SwitchTopologyTypeCookie(QObject *receiver, const char *slot);
    void emitCompleted(qlonglong newEntryId);

signals:
    void completed(qlonglong newEntryId);
};


/*******************************************************************************
* TopologyManagerPrivate
*******************************************************************************/
class TopologyManagerPrivate
{
public:
    explicit TopologyManagerPrivate();
    virtual ~TopologyManagerPrivate();

    Hns::TopologyManagerQPrx managerQtProxy;
    QHash<QString, Topology *> topologyHash;
    QHash<Hns::TopologyPrx, Topology *> loadingProxiesHash;
    QHash<Topology *, LoadTopologyCookie *> loadingCookieHash;
    QHash<QObject *, Hns::ByteArray> screenshotCookieHash;
    QHash<qlonglong, Hns::ByteArray> screenshotEntryHash;
    EntityInfoList *entityInfoList;
    QSignalMapper *entifyInfoMapper;
    QSignalMapper *topologyMapper;

    int remoteCallCount;
};

} // namespace HnsClient

#endif //HNSCLIENT_TOPOLOGYMANAGER_P_H
