/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef HNSCLIENT_USERMANAGER_P_H
#define HNSCLIENT_USERMANAGER_P_H

// Generated hynesim headers
#include "common/usermanager.h"

// diamesh headers
#include <diamesh/proxies_authmanager.h>
#include <diamesh/proxies_usermanager.h>
#include <diamesh/proxies_sessionManager.h>

// ZeroC Ice headers
#include <IceUtil/Shared.h>

// Qt headers
#include <QObject>
#include <QList>
#include <QString>
#include <QHash>

namespace HnsClient {

class UserManager;

/*******************************************************************************
* Callback_ProxyAuthManager_setPassword
*******************************************************************************/
class Callback_ProxyAuthManager_setPassword : public QObject, public ::IceUtil::Shared
{
    Q_OBJECT
public:
    explicit Callback_ProxyAuthManager_setPassword();
    ~Callback_ProxyAuthManager_setPassword();
    void _success(bool);
    void _failure(const ::Ice::Exception&);
signals:
    void succeeded(bool);
    void failed(QSharedPointer<Ice::Exception>);
};

typedef IceUtil::Handle<Callback_ProxyAuthManager_setPassword> Callback_ProxyAuthManager_setPasswordPtr;


/*******************************************************************************
* UserManagerPrivate
*******************************************************************************/
class UserManagerPrivate
{
public:
    explicit UserManagerPrivate();
    virtual ~UserManagerPrivate();

    Diamesh::UserManagerQPrx userManagerQtPrx;
    Diamesh::AuthManagerPrx authManagerPrx;
    Diamesh::SessionManagerQPrx sessionManagerQPrx;
    QHash <qlonglong, Hns::UserPtr> usersHash;
    QHash <qlonglong, Diamesh::GroupPtr> groupsHash;
    QHash <QString, Diamesh::SessionInfo> sessionInfoHash;
    int remoteCallCount;

private:
    void _resetData();
};


/*******************************************************************************
* SessionGetInfoCookie
*******************************************************************************/
class SessionGetInfoCookie : public QObject
{
    Q_OBJECT
public:
    explicit SessionGetInfoCookie();
    ~SessionGetInfoCookie();
    bool isCompleted();
    Diamesh::SessionInfo information();

    void callGetInfo(Diamesh::SessionPrx proxy);

signals:
    void completed();

protected slots:
    void _getInfoSlot(::Diamesh::SessionInfo information);
    void _remoteMethodFailedSlot(QSharedPointer<Ice::Exception> exception);

private:
    bool _completed;
    Diamesh::SessionInfo _information;
};


/*******************************************************************************
* UserManagerCookie
*******************************************************************************/
class UserManagerCookie : public QObject
{
    Q_OBJECT
public:
    explicit UserManagerCookie();
    ~UserManagerCookie();

protected:
    virtual void _handleSetPasswordResult(bool result) = 0;
    void _callSetPassword(Diamesh::AuthManagerPrx authPrx, std::string login, std::string pwd);

protected slots:
    void _remoteMethodFailedSlot(QSharedPointer<Ice::Exception>);
    void _setPasswordSlot(bool result);
};


/*******************************************************************************
* AddUserCookie
*******************************************************************************/
class AddUserCookie : public UserManagerCookie
{
    Q_OBJECT
public:
    explicit AddUserCookie(Hns::UserPtr user, QString password, QObject *receiver, const char *slot);
    ~AddUserCookie();
    void emitCompleted();
    void setUserId(qlonglong userId);
    QList<Ice::Long> groupIdList();
    void setPassword(Diamesh::AuthManagerPrx authPrx);

public slots:
    void addUserToGroupSlot(bool result);

signals:
    void completed(qlonglong userId, QString userName, bool result);

protected:
    void _handleSetPasswordResult(bool result);

private:
    void _remoteCallEnded(bool result);
    Hns::UserPtr _sourceUser;
    QString _pwd;
    bool _result;
    QList<Ice::Long> _groupIdList;
    int _remoteCalls;
};


/*******************************************************************************
* CloneUserCookie
*******************************************************************************/
class CloneUserCookie : public UserManagerCookie
{
    Q_OBJECT
public:
    explicit CloneUserCookie(Hns::UserPtr user, QString password, QObject *receiver, const char *slot);
    ~CloneUserCookie();
    void emitCompleted(bool result);
    void setCloneCount(int count);
    void setClonePassword(Diamesh::AuthManagerPrx authPrx, std::string cloneName);
    void fetchCloneFailed();
    QString userName();

signals:
    void completed(qlonglong userId, QString userName, bool result);

protected:
    void _handleSetPasswordResult(bool result);

private:
    void _remoteCallEnded(bool result);
    Hns::UserPtr _sourceUser;
    QString _pwd;
    bool _result;
    int _remoteCalls;

};


/*******************************************************************************
* UpdateUserCookie
*******************************************************************************/
class UpdateUserCookie : public UserManagerCookie
{
    Q_OBJECT
public:
    explicit UpdateUserCookie(Hns::UserPtr user, QString password, Diamesh::LongList originalGroupList, QObject *receiver, const char *slot);
    ~UpdateUserCookie();
    void emitCompleted(bool result);
    QList<Ice::Long> groupToAddIdList();
    QList<Ice::Long> groupToRemoveIdList();
    void setPassword(Diamesh::AuthManagerPrx authPrx);
    qlonglong getUserId();

signals:
    void completed(qlonglong userId, QString userName, bool result);

public slots:
    void addUserToGroup(bool result);
    void removeUserFromGroup(bool result);

protected:
    void _handleSetPasswordResult(bool result);

private:
    void _remoteCallEnded(bool result);
    Hns::UserPtr _sourceUser;
    QString _pwd;
    bool _result;
    int _remoteCalls;
    QList<Ice::Long> _groupsToAdd;
    QList<Ice::Long> _groupsToRemove;

};


/*******************************************************************************
* DeleteUserCookie
*******************************************************************************/
class DeleteUserCookie : public QObject
{
    Q_OBJECT
public:
    explicit DeleteUserCookie(qlonglong userId, QString userName, QObject *receiver, const char *slot);
    ~DeleteUserCookie();
    void emitCompleted(bool result);

signals:
    void completed(qlonglong userId, QString userName, bool result);

private:
    qlonglong _userId;
    QString _userName;
};


/*******************************************************************************
* AddGroupCookie
*******************************************************************************/
class AddGroupCookie : public QObject
{
    Q_OBJECT
public:
    explicit AddGroupCookie(Diamesh::GroupPtr group, QObject *receiver, const char *slot);
    ~AddGroupCookie();
    void emitCompleted();
    void setGroupId(qlonglong groupId);
    QList<Ice::Long> userIdList();

public slots:
    void addUserToGroupSlot(bool result);

signals:
    void completed(qlonglong groupId, QString groupName, bool result);

private:
    void _remoteCallEnded(bool result);
    Diamesh::GroupPtr _sourceGroup;
    bool _result;
    QList<Ice::Long> _userIdList;
    int _remoteCalls;
};


/*******************************************************************************
* UpdateGroupCookie
*******************************************************************************/
class UpdateGroupCookie : public QObject
{
    Q_OBJECT
public:
    explicit UpdateGroupCookie(Diamesh::GroupPtr group, Diamesh::LongList originalUserList, QObject *receiver, const char *slot);
    ~UpdateGroupCookie();
    void emitCompleted(bool result);
    QList<Ice::Long> userToAddIdList();
    QList<Ice::Long> userToRemoveIdList();
    qlonglong getGroupId();

signals:
    void completed(qlonglong groupId, QString groupName, bool result);

public slots:
    void addUserToGroup(bool result);
    void removeUserFromGroup(bool result);

protected:
    void _handleSetPasswordResult(bool result);

private:
    void _remoteCallEnded(bool result);
    Diamesh::GroupPtr _sourceGroup;
    bool _result;
    QList<Ice::Long> _usersToAdd;
    QList<Ice::Long> _usersToRemove;
    int _remoteCalls;
};


/*******************************************************************************
* DeleteGroupCookie
*******************************************************************************/
class DeleteGroupCookie : public QObject
{
    Q_OBJECT
public:
    explicit DeleteGroupCookie(qlonglong groupId, QString groupName, QObject *receiver, const char *slot);
    ~DeleteGroupCookie();
    void emitCompleted(bool result);

signals:
    void completed(qlonglong groupId, QString groupName, bool result);

private:
    qlonglong _groupId;
    QString _groupName;
};

} // namespace HnsClient
#endif //HNSCLIENT_USERMANAGER_P_H
