/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef HNSCLIENT_RESOURCEMANAGER_H
#define HNSCLIENT_RESOURCEMANAGER_H

// Generated hynesim headers
#include "common/proxies_customresourcemanager.h"
#include "common/types.h"

// diamesh headers
#include <diamesh/types.h>

// Qt headers
#include <QObject>
#include <QHash>
#include <QByteArray>

namespace HnsClient {

class ResourceManagerPrivate;

class ResourceManager : public QObject
{
    Q_OBJECT
    Q_DECLARE_PRIVATE(ResourceManager)
public:
    ResourceManager(QObject *parent = 0);
    ~ResourceManager();
    QHash<QString, QByteArray> getResources() const;
    QString uploadResource(const QString &filePath, bool force = false);
    QByteArray getResource(QString key) const;
    void deleteResource(const QString &key);

signals:
    void loaded();

protected:
    void _load(Hns::CustomResourceManagerQPrx managerQtPrx);
    void _callEnded();
    void _checkLoading();
    void _init();
    QString _uploadResource(const QString &filePath, bool force = false);
    void _handleGetResource(::Hns::ByteArray data, QObject *obj);

protected slots:
    void _loadGetResourceKeysSlot(::Diamesh::StringList resourceKeys);
    void _loadGetResourceSlot(::Hns::ByteArray data, QObject *obj);
    void _getResourceSlot(::Hns::ByteArray data, QObject *obj);
    void _loadGetResourceFailedSlot(QSharedPointer<Ice::Exception> exception, QObject *obj);
    void _getResourceFailedSlot(QSharedPointer<Ice::Exception> exception, QObject *obj);
    void _deleteResourceSlot(bool result, QObject *obj);
    void _resourceAddedSlot(QString key);
    void _resourceDeletedSlot(QString key);

private:
    ResourceManagerPrivate * const d_ptr;
    friend class Session;
};

} // namespace HnsClient

#endif //HNSCLIENT_RESOURCEMANAGER_H
