/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// hynesim hnsclient headers
#include "libhnsclient/session.h"
#include "libhnsclient/private/session_p.h"
#include "libhnsclient/private/common_p.h"
#include "libhnsclient/private/privatesessionaccessor.h"
#include "libhnsclient/catalogimporter.h"
#include "libhnsclient/dynamipsrouterdb.h"
#include "libhnsclient/entityentrycatalog.h"
#include "libhnsclient/entrypropertiesmanager.h"
#include "libhnsclient/jobmanager.h"
#include "libhnsclient/macaddressmanager.h"
#include "libhnsclient/topologyentrycatalog.h"
#include "libhnsclient/topologymanager.h"
#include "libhnsclient/platformmonitor.h"
#include "libhnsclient/usermanager.h"
#include "libhnsclient/tracermanager.h"
#include "libhnsclient/hsetmanager.h"
#include "libhnsclient/aclmanager.h"
#include "libhnsclient/resourcemanager.h"

// hynesim common headers
#include "common/hns.h"

// Qt headers
#include <QEventLoop>
#include <QMetaType>

namespace HnsClient {

/*******************************************************************************
* SessionPrivate
*******************************************************************************/
SessionPrivate::SessionPrivate()
{
    _clearProxies();
    _clearPtr();
    jobManagerLoaded = false;
}

SessionPrivate::~SessionPrivate()
{
    clear();
}

void SessionPrivate::clear()
{
    _deletePtr();
    _clearPtr();
    _clearProxies();
}

void SessionPrivate::_deletePtr()
{
    if (catalogImporter)
        delete catalogImporter;
    if (dynamipsRouterDB)
        delete dynamipsRouterDB;
    if (entityEntryCatalog)
        delete entityEntryCatalog;
    if (entryPropertiesManager)
        delete entryPropertiesManager;
    if (jobManager)
        delete jobManager;
    if (macAddressManager)
        delete macAddressManager;
    if (platformMonitor)
        delete platformMonitor;
    if (topologyEntryCatalog)
        delete topologyEntryCatalog;
    if (topologyManager)
        delete topologyManager;
    if (tracerManager)
        delete tracerManager;
    if (userManager)
        delete userManager;
    if (hSetManager)
        delete hSetManager;
    if (aclManager)
        delete aclManager;
    if (resourceManager)
        delete resourceManager;
}

void SessionPrivate::_clearPtr()
{
    catalogImporter = 0;
    dynamipsRouterDB = 0;
    entityEntryCatalog = 0;
    entryPropertiesManager = 0;
    jobManager = 0;
    macAddressManager = 0;
    platformMonitor = 0;
    topologyManager = 0;
    topologyEntryCatalog = 0;
    tracerManager = 0;
    userManager = 0;
    hSetManager = 0;
    aclManager = 0;
    resourceManager = 0;
}

void SessionPrivate::_clearProxies()
{
    resourceManagerQtProxy = 0;
    sessionManagerQtProxy = 0;
    userManagerQtProxy = 0;
    authManagerPrx = 0;
    hSetManagerQtProxy = 0;
    aclManagerQtProxy = 0;

    masterQtProxy = 0;

    catalogImporterQtProxy = 0;
    dynamipsRouterDBQtProxy = 0;
    entityCatalogQtProxy = 0;
    entryPropertiesManagerQtProxy = 0;
    jobManagerQtProxy = 0;
    macAddressManagerQtProxy = 0;
    topologyCatalogQtProxy = 0;
    topologyManagerQtProxy = 0;
    sessionHelper.clear();
}


/*******************************************************************************
* Session
*******************************************************************************/
Session * Session::_singleton = 0;

Session::Session(QObject *parent)
    : QObject(parent)
    , d_ptr(new SessionPrivate)
{
    new PrivateSessionAccessor(this, this);
}

Session::~Session()
{
    delete d_ptr;
    _singleton = 0;
}

/*! \fn *instance()
 *\brief TODO
 */
Session *Session::instance()
{
    return _singleton;
}

/*! \fn *create(QSharedPointer<Diamesh::SessionHelper> sessionHelper, QObject *sessionParent = 0)
 *\brief TODO
 *\param sessionHelper
 *\param sessionParent
 */
Session *Session::create(QSharedPointer<Diamesh::SessionHelper> sessionHelper, QObject *sessionParent)
{
    if (_singleton == 0) {
        Hns::loadTranslation(Constants::TRANSLATION_NAME, false);
        SessionLoader *loader = SessionLoader::instance();
        QEventLoop eventLoop;
        if (!loader->isCompleted()) {
            QObject::connect(loader, SIGNAL(completed()), &eventLoop, SLOT(quit()));
            if (!loader->isLoading()) {
                loader->load(sessionHelper, sessionParent);
                sessionHelper.clear();
            }
            eventLoop.exec();
        }
        if (_singleton == 0) {
            _singleton = loader->getSession();
            delete loader;
        }
    }
    return _singleton;
}

/*! \fn isAdvancedMode()
 *\brief TODO
 */
bool Session::isAdvancedMode()
{
    Q_D(Session);
    return d->isAdvancedMode;
}

/*! \fn sessionInformation()
 *\brief TODO
 */
Diamesh::SessionInfo Session::sessionInformation()
{
    Q_D(Session);
    return d->sessionInformation;
}

/*! \fn *catalogImporter()
 *\brief TODO
 */
CatalogImporter *Session::catalogImporter()
{
    Q_D(Session);
    return d->catalogImporter;
}

/*! \fn *dynamipsRouterDB()
 *\brief TODO
 */
DynamipsRouterDB *Session::dynamipsRouterDB()
{
    Q_D(Session);
    return d->dynamipsRouterDB;
}

/*! \fn *entityEntryCatalog()
 *\brief TODO
 */
EntityEntryCatalog *Session::entityEntryCatalog()
{
    Q_D(Session);
    return d->entityEntryCatalog;
}

/*! \fn *entryPropertiesManager()
 *\brief TODO
 */
EntryPropertiesManager *Session::entryPropertiesManager()
{
    Q_D(Session);
    return d->entryPropertiesManager;
}

/*! \fn *jobManager()
 *\brief TODO
 */
JobManager *Session::jobManager()
{
    Q_D(Session);
    return d->jobManager;
}

/*! \fn *macAddressManager()
 *\brief TODO
 */
MacAddressManager *Session::macAddressManager()
{
    Q_D(Session);
    return d->macAddressManager;
}

/*! \fn *topologyManager()
 *\brief TODO
 */
TopologyManager *Session::topologyManager()
{
    Q_D(Session);
    return d->topologyManager;
}

/*! \fn *topologyEntryCatalog()
 *\brief TODO
 */
TopologyEntryCatalog *Session::topologyEntryCatalog()
{
    Q_D(Session);
    return d->topologyEntryCatalog;
}

/*! \fn *platformMonitor()
 *\brief TODO
 */
PlatformMonitor *Session::platformMonitor()
{
    Q_D(Session);
    return d->platformMonitor;
}

/*! \fn *userManager()
 *\brief TODO
 */
UserManager *Session::userManager()
{
    Q_D(Session);
    return d->userManager;
}

/*! \fn *hSetManager()
 *\brief TODO
 */
HSetManager *Session::hSetManager()
{
    Q_D(Session);
    return d->hSetManager;
}

/*! \fn *aclManager()
 *\brief TODO
 */
ACLManager *Session::aclManager()
{
    Q_D(Session);
    return d->aclManager;
}

/*! \fn *entityInfoList()
 *\brief TODO
 */
EntityInfoList *Session::entityInfoList()
{
    Q_D(Session);
    return d->entityInfoList;
}

/*! \fn *tracerManager()
 *\brief TODO
 */
TracerManager *Session::tracerManager()
{
    Q_D(Session);
    return d->tracerManager;
}

/*! \fn resourceManager()
 *\brief TODO
 */
ResourceManager *Session::resourceManager()
{
    Q_D(Session);
    return d->resourceManager;
}

/*! \fn entityCatalogStorageMap()
 *\brief TODO
 */
QMap<QString, QString> Session::entityCatalogStorageMap()
{
    Q_D(Session);
    return d->entityCatalogStorageMap;
}

/*! \fn topologyCatalogStorageMap()
 *\brief TODO
 */
QMap<QString, QString> Session::topologyCatalogStorageMap()
{
    Q_D(Session);
    return d->topologyCatalogStorageMap;
}

/*! \fn user()
 *\brief TODO
 */
Hns::UserPtr Session::user()
{
    Q_D(Session);
    Hns::UserPtr user = 0;
    if (!d->sessionHelper.isNull())
        user = Hns::UserPtr::dynamicCast(d->sessionHelper.data()->user());
    return user;
}

/*! \fn getEntryNodeInfoMap(Hns::EntityEntryPtr const &entry, QObject *receiver, const char *slot)
 *\brief TODO
 *\param entry
 *\param receiver
 *\param slot
 */
void Session::getEntryNodeInfoMap(const Hns::EntityEntryPtr &entry, QObject *receiver, const char *slot)
{
    Q_D(Session);
    d->masterQtProxy->begin_getEntryNodeInfoMap(entry, receiver, slot);
}

/*! \fn disconnectSession()
 *\brief TODO
 */
void Session::disconnectSession()
{
    Q_D(Session);
    d->clear();
}

void Session::_load(QSharedPointer<Diamesh::SessionHelper> sessionHelper)
{
    Q_D(Session);
    d->sessionHelper = sessionHelper;

    connect(d->sessionHelper.data(), SIGNAL(disconnected()), this, SLOT(_disconnectedSlot()));

    Hns::MasterPrx masterPrx = Hns::MasterPrx::uncheckedCast(d->sessionHelper->master());
    d->masterQtProxy = Hns::MasterQPrx(masterPrx, d->sessionHelper->adapter());

    Callback_ProxySession_getInfoPtr callback = new Callback_ProxySession_getInfo;
    callback->moveToThread(this->thread());
    qRegisterMetaType< ::Diamesh::SessionInfo>("::Diamesh::SessionInfo");
    connect(callback.get(), SIGNAL(succeeded(::Diamesh::SessionInfo)), this, SLOT(_loadSessionInfoSlot(::Diamesh::SessionInfo)));
    connect(callback.get(), SIGNAL(failed(QSharedPointer<Ice::Exception>)), this, SLOT(_remoteMethodFailedSlot(QSharedPointer<Ice::Exception>)));
    Diamesh::Callback_Session_getInfoPtr cb = Diamesh::newCallback_Session_getInfo(callback, &Callback_ProxySession_getInfo::_success, &Callback_ProxySession_getInfo::_failure);

    d->platformMonitor = new PlatformMonitor(this);
    d->tracerManager = new TracerManager(this);

    d->remoteCallCount = 18;

    d->sessionHelper->session()->begin_getInfo(cb);
    d->masterQtProxy->begin_getAdvancedMode(this, SLOT(_loadAdvancedModeSlot(bool)), this, SLOT(_remoteMethodFailedSlot(QSharedPointer<Ice::Exception>)));

    d->masterQtProxy->begin_getUserManager(this, SLOT(_loadUserManagerPrxSlot(::Diamesh::UserManagerPrx)), this, SLOT(_remoteMethodFailedSlot(QSharedPointer<Ice::Exception>)));
    d->masterQtProxy->begin_getAuthManager(this, SLOT(_loadAuthManagerPrxSlot(::Diamesh::AuthManagerPrx)), this, SLOT(_remoteMethodFailedSlot(QSharedPointer<Ice::Exception>)));
    d->masterQtProxy->begin_getSessionManager(this, SLOT(_loadSessionManagerPrxSlot(::Diamesh::SessionManagerPrx)), this, SLOT(_remoteMethodFailedSlot(QSharedPointer<Ice::Exception>)));
    d->masterQtProxy->begin_getHSetManager(this, SLOT(_loadHSetManagerPrxSlot(::Diamesh::HSetManagerPrx)), this, SLOT(_remoteMethodFailedSlot(QSharedPointer<Ice::Exception>)));
    d->masterQtProxy->begin_getACLManager(this, SLOT(_loadACLManagerPrxSlot(::Diamesh::ACLManagerPrx)), this, SLOT(_remoteMethodFailedSlot(QSharedPointer<Ice::Exception>)));

    d->masterQtProxy->begin_getCatalogImporter(this, SLOT(_loadCatalogImporterPrxSlot(::Hns::CatalogImporterPrx)), this, SLOT(_remoteMethodFailedSlot(QSharedPointer<Ice::Exception>)));
    d->masterQtProxy->begin_getDynamipsRouterDB(this, SLOT(_loadDynamipsRouterDBPrxSlot(::Hns::DynamipsRouterDBPrx)), this, SLOT(_remoteMethodFailedSlot(QSharedPointer<Ice::Exception>)));
    d->masterQtProxy->begin_getEntryPropertiesManager(this, SLOT(_loadEntryPropertiesManagerPrxSlot(::Hns::EntryPropertiesManagerPrx)), this, SLOT(_remoteMethodFailedSlot(QSharedPointer<Ice::Exception>)));
    d->masterQtProxy->begin_getJobManager(this, SLOT(_loadJobManagerPrxSlot(::Hns::JobManagerPrx)), this, SLOT(_remoteMethodFailedSlot(QSharedPointer<Ice::Exception>)));
    d->masterQtProxy->begin_getMacAddressManager(this, SLOT(_loadMacAddressManagerPrxSlot(::Hns::MacAddressManagerPrx)), this, SLOT(_remoteMethodFailedSlot(QSharedPointer<Ice::Exception>)));
    d->masterQtProxy->begin_getEntityCatalog(this, SLOT(_loadEntityCatalogPrxSlot(::Hns::EntityCatalogPrx)), this, SLOT(_remoteMethodFailedSlot(QSharedPointer<Ice::Exception>)));
    d->masterQtProxy->begin_getTopologyCatalog(this, SLOT(_loadTopologyCatalogPrxSlot(::Hns::TopologyCatalogPrx)), this, SLOT(_remoteMethodFailedSlot(QSharedPointer<Ice::Exception>)));
    d->masterQtProxy->begin_getTopologyManager(this, SLOT(_loadTopologyManagerPrxSlot(::Hns::TopologyManagerPrx)), this, SLOT(_remoteMethodFailedSlot(QSharedPointer<Ice::Exception>)));
    d->masterQtProxy->begin_getTracer(this, SLOT(_loadMasterTracerPrxSlot(::Hns::TracerPrx)), this, SLOT(_remoteMethodFailedSlot(QSharedPointer<Ice::Exception>)));
    d->masterQtProxy->begin_getCustomResourceManager(this, SLOT(_loadResourceManagerPrxSlot(::Hns::CustomResourceManagerPrx)), this, SLOT(_remoteMethodFailedSlot(QSharedPointer<Ice::Exception>)));

    connect(d->platformMonitor, SIGNAL(loaded()), this, SLOT(_loadPlatforMonitorSlot()));
    d->platformMonitor->_load(d->masterQtProxy);
    sessionHelper.clear();
}

void Session::_callEnded()
{
    Q_D(Session);
    d->remoteCallCount--;
    _checkLoading();
}

void Session::_checkLoading()
{
    Q_D(Session);
    if (d->remoteCallCount == 0)
        _init();
}

void Session::_init()
{
    emit loaded();
}

void Session::_loadUserManager()
{
    Q_D(Session);
    if (d->userManager)
        return;

    if (d->userManagerQtProxy && d->authManagerPrx && d->sessionManagerQtProxy) {
        d->userManager = new UserManager(this);
        connect(d->userManager, SIGNAL(loaded()), this, SLOT(_loadUserManagerSlot()));
        d->remoteCallCount++;
        d->userManager->_load(d->userManagerQtProxy, d->authManagerPrx, d->sessionManagerQtProxy);
    }
}

void Session::_loadEntityEntryCatalog()
{
    Q_D(Session);
    if (d->entityEntryCatalog)
        return;

    if (d->entityCatalogQtProxy && d->topologyManagerQtProxy && d->jobManager) {
        d->entityEntryCatalog = new EntityEntryCatalog(this);
        connect(d->entityEntryCatalog, SIGNAL(loaded()), this, SLOT(_loadEntityEntryCatalogSlot()));
        d->remoteCallCount++;
        d->entityEntryCatalog->_load(d->entityCatalogQtProxy, d->topologyManagerQtProxy);
    }
}

void Session::_remoteMethodFailedSlot(QSharedPointer<Ice::Exception> exception)
{
    qCritical() << tr("Catched Ice exception: %1").arg(exception->ice_name().c_str());
    _callEnded();
}

void Session::_loadSessionInfoSlot(::Diamesh::SessionInfo sessionInformation)
{
    Q_D(Session);
    d->sessionInformation = sessionInformation;
    _callEnded();
}

void Session::_loadAdvancedModeSlot(bool isAdvancedMode)
{
    Q_D(Session);
    d->isAdvancedMode = isAdvancedMode;
    _callEnded();
}

void Session::_loadHSetManagerPrxSlot(Diamesh::HSetManagerPrx hSetManagerPrx)
{
    Q_D(Session);
    d->hSetManagerQtProxy = Diamesh::HSetManagerQPrx(hSetManagerPrx, d->sessionHelper->adapter());
    d->hSetManager = new HSetManager(this);
    connect(d->hSetManager, SIGNAL(loaded()), this, SLOT(_loadHSetManagerSlot()));
    d->hSetManager->_load(d->hSetManagerQtProxy);
}

void Session::_loadACLManagerPrxSlot(::Diamesh::ACLManagerPrx aclManagerPrx)
{
    Q_D(Session);
    d->aclManagerQtProxy = Diamesh::ACLManagerQPrx(aclManagerPrx, d->sessionHelper->adapter());
    d->aclManager = new ACLManager(this);
    connect(d->aclManager, SIGNAL(loaded()), this, SLOT(_loadACLManagerSlot()));
    d->aclManager->_load(d->aclManagerQtProxy);
}

void Session::_loadUserManagerPrxSlot(Diamesh::UserManagerPrx userManagerPrx)
{
    Q_D(Session);
    d->userManagerQtProxy = Diamesh::UserManagerQPrx(userManagerPrx, d->sessionHelper->adapter());
    _loadUserManager();
    _callEnded();
}

void Session::_loadAuthManagerPrxSlot(Diamesh::AuthManagerPrx authManagerPrx)
{
    Q_D(Session);
    d->authManagerPrx = authManagerPrx;
    _loadUserManager();
    _callEnded();
}

void Session::_loadSessionManagerPrxSlot(Diamesh::SessionManagerPrx sessionManagerPrx)
{
    Q_D(Session);
    d->sessionManagerQtProxy = Diamesh::SessionManagerQPrx(sessionManagerPrx, d->sessionHelper->adapter());
    _loadUserManager();
    _callEnded();
}

void Session::_loadCatalogImporterPrxSlot(::Hns::CatalogImporterPrx catalogImporterPrx)
{
    Q_D(Session);
    d->catalogImporterQtProxy = Hns::CatalogImporterQPrx(catalogImporterPrx, d->sessionHelper->adapter());
    d->catalogImporter = new CatalogImporter(this);
    connect(d->catalogImporter, SIGNAL(loaded()), this, SLOT(_loadCatalogImporterSlot()));
    d->catalogImporter->_load(d->catalogImporterQtProxy);
}

void Session::_loadDynamipsRouterDBPrxSlot(::Hns::DynamipsRouterDBPrx dynamipsRouterDBPrx)
{
    Q_D(Session);
    d->dynamipsRouterDBQtProxy = Hns::DynamipsRouterDBQPrx(dynamipsRouterDBPrx, d->sessionHelper->adapter());
    d->dynamipsRouterDB = new DynamipsRouterDB(this);
    connect(d->dynamipsRouterDB, SIGNAL(loaded()), this, SLOT(_loadDynamipsRouterDBSlot()));
    d->dynamipsRouterDB->_load(d->dynamipsRouterDBQtProxy);
}

void Session::_loadEntityCatalogPrxSlot(::Hns::EntityCatalogPrx entityCatalogPrx)
{
    Q_D(Session);
    d->entityCatalogQtProxy = Hns::EntityCatalogQPrx(entityCatalogPrx, d->sessionHelper->adapter());
    d->entityCatalogQtProxy->begin_getFileStorages(this, SLOT(_loadEntityCatalogGetFileStoragesSlot(::Diamesh::StringStringMap)), this, SLOT(_remoteMethodFailedSlot(QSharedPointer<Ice::Exception>)));
    _loadEntityEntryCatalog();
}

void Session::_loadEntryPropertiesManagerPrxSlot(::Hns::EntryPropertiesManagerPrx entryPropertiesManagerPrx)
{
    Q_D(Session);
    d->entryPropertiesManagerQtProxy = Hns::EntryPropertiesManagerQPrx(entryPropertiesManagerPrx, d->sessionHelper->adapter());
    d->entryPropertiesManager = new EntryPropertiesManager(this);
    connect(d->entryPropertiesManager, SIGNAL(loaded()), this, SLOT(_loadEntryPropertiesManagerSlot()));
    d->entryPropertiesManager->_load(d->entryPropertiesManagerQtProxy);
}

void Session::_loadJobManagerPrxSlot(::Hns::JobManagerPrx jobManagerPrx)
{
    Q_D(Session);
    d->jobManagerQtProxy = Hns::JobManagerQPrx(jobManagerPrx, d->sessionHelper->adapter());
    d->jobManager = new JobManager(this);
    connect(d->jobManager, SIGNAL(loaded()), this, SLOT(_loadJobManagerSlot()));
    d->jobManager->_load(d->jobManagerQtProxy);
}

void Session::_loadMacAddressManagerPrxSlot(::Hns::MacAddressManagerPrx macAddressManagerPrx)
{
    Q_D(Session);
    d->macAddressManagerQtProxy = Hns::MacAddressManagerQPrx(macAddressManagerPrx, d->sessionHelper->adapter());
    d->macAddressManager = new MacAddressManager(this);
    connect(d->macAddressManager, SIGNAL(loaded()), this, SLOT(_loadMacAddressManagerSlot()));
    d->macAddressManager->_load(d->macAddressManagerQtProxy);
}

void Session::_loadTopologyCatalogPrxSlot(::Hns::TopologyCatalogPrx topologyCatalogPrx)
{
    Q_D(Session);
    d->topologyCatalogQtProxy = Hns::TopologyCatalogQPrx(topologyCatalogPrx, d->sessionHelper->adapter());
    d->topologyCatalogQtProxy->begin_getFileStorages(this, SLOT(_loadTopologyCatalogGetFileStoragesSlot(::Diamesh::StringStringMap)), this, SLOT(_remoteMethodFailedSlot(QSharedPointer<Ice::Exception>)));
    d->topologyEntryCatalog = new TopologyEntryCatalog(this);
    connect(d->topologyEntryCatalog, SIGNAL(loaded()), this, SLOT(_loadTopologyEntryCatalogSlot()));
    d->remoteCallCount++;
    d->topologyEntryCatalog->_load(d->topologyCatalogQtProxy);
}

void Session::_loadTopologyManagerPrxSlot(::Hns::TopologyManagerPrx topologyManagerPrx)
{
    Q_D(Session);
    d->topologyManagerQtProxy = Hns::TopologyManagerQPrx(topologyManagerPrx, d->sessionHelper->adapter());
    _loadEntityEntryCatalog();
    _callEnded();
}

void Session::_loadMasterTracerPrxSlot(::Hns::TracerPrx masterTracerPrx)
{
    Q_D(Session);
    d->tracerManager->registerMasterTracer(Hns::TracerQPrx(masterTracerPrx, d->sessionHelper->adapter()));
    _callEnded();
}

void Session::_loadResourceManagerPrxSlot(Hns::CustomResourceManagerPrx managerPrx)
{
    Q_D(Session);
    d->resourceManagerQtProxy = Hns::CustomResourceManagerQPrx(managerPrx, d->sessionHelper->adapter());
    d->resourceManager = new ResourceManager(this);
    connect(d->resourceManager, SIGNAL(loaded()), this, SLOT(_loadResourceManagerSlot()));
    d->resourceManager->_load(d->resourceManagerQtProxy);
}

void Session::_loadEntityCatalogGetFileStoragesSlot(Diamesh::StringStringMap entityFileStorageMap)
{
    Q_D(Session);
    QMap<std::string, std::string> storageMap(entityFileStorageMap);
    QMapIterator<std::string, std::string> iterator(storageMap);
    while(iterator.hasNext()) {
        iterator.next();
        d->entityCatalogStorageMap.insert(QString::fromStdString(iterator.key()), QString::fromStdString(iterator.value()));
    }
    _callEnded();
}

void Session::_loadTopologyCatalogGetFileStoragesSlot(Diamesh::StringStringMap topologyFileStorageMap)
{
    Q_D(Session);
    QMap<std::string, std::string> storageMap(topologyFileStorageMap);
    QMapIterator<std::string, std::string> iterator(storageMap);
    while(iterator.hasNext()) {
        iterator.next();
        d->topologyCatalogStorageMap.insert(QString::fromStdString(iterator.key()), QString::fromStdString(iterator.value()));
    }
    _callEnded();
}

void Session::_loadUserManagerSlot()
{
    _callEnded();
}

void Session::_loadCatalogImporterSlot()
{
    _callEnded();
}

void Session::_loadDynamipsRouterDBSlot()
{
    _callEnded();
}

void Session::_loadEntityEntryCatalogSlot()
{
    Q_D(Session);
    d->topologyManager = new TopologyManager(this);
    d->remoteCallCount++;
    connect(d->topologyManager, SIGNAL(loaded()), this, SLOT(_loadTopologyManagerSlot()));
    d->topologyManager->_load(d->topologyManagerQtProxy);
    _callEnded();
}

void Session::_loadEntryPropertiesManagerSlot()
{
    _callEnded();
}

void Session::_loadJobManagerSlot()
{
    _loadEntityEntryCatalog();
    _callEnded();
}

void Session::_loadMacAddressManagerSlot()
{
    _callEnded();
}

void Session::_loadTopologyEntryCatalogSlot()
{
    _callEnded();
}

void Session::_loadTopologyManagerSlot()
{
    _callEnded();
}

void Session::_loadPlatforMonitorSlot()
{
    _callEnded();
}

void Session::_loadHSetManagerSlot()
{
    _callEnded();
}

void Session::_loadACLManagerSlot()
{
    _callEnded();
}

void Session::_loadResourceManagerSlot()
{
    _callEnded();
}

void Session::_disconnectedSlot()
{
    Q_D(Session);
    emit disconnected();
    d->clear();
    deleteLater();
}

QSharedPointer<Diamesh::SessionHelper> Session::sessionHelper()
{
    Q_D(Session);
    return d->sessionHelper;
}

/*******************************************************************************
* SessionLoader
*******************************************************************************/
SessionLoader *SessionLoader::_singleton = 0;

SessionLoader::SessionLoader(QObject *parent)
    : QObject(parent)
    , _isLoading(false)
    , _completed(false)
    , _session(0)
{
}

SessionLoader::~SessionLoader()
{
    _singleton = 0;
}

SessionLoader *SessionLoader::instance()
{
    if (_singleton == 0)
        _singleton = new SessionLoader();
    return _singleton;
}

void SessionLoader::load(QSharedPointer<Diamesh::SessionHelper> sessionHelper, QObject *sessionParent)
{
    if (_isLoading)
        return;
    _isLoading = true;
    _session = new Session(sessionParent);
    _session->_load(sessionHelper);
    sessionHelper.clear();
    connect(_session, SIGNAL(loaded()), this, SLOT(_sessionLoaded()));
}

bool SessionLoader::isLoading()
{
    return _isLoading;
}

bool SessionLoader::isCompleted()
{
    return _completed;
}

Session *SessionLoader::getSession()
{
    return _session;
}

void SessionLoader::_sessionLoaded()
{
    _isLoading = false;
    _completed = true;
    emit completed();
}

} // namespace HnsClient
