/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef HNSCLIENT_SESSION_H
#define HNSCLIENT_SESSION_H

// Generated hynesim headers
#include "common/proxies_usermanager.h"
#include "common/proxies_catalogimporter.h"
#include "common/proxies_dynamipsrouterdb.h"
#include "common/proxies_entitycatalog.h"
#include "common/proxies_entrypropertiesmanager.h"
#include "common/proxies_jobmanager.h"
#include "common/proxies_macaddressmanager.h"
#include "common/proxies_topologycatalog.h"
#include "common/proxies_topologymanager.h"
#include "common/proxies_customresourcemanager.h"

// diamesh headers
#include <diamesh/sessionhelper.h>

// Qt headers
#include <QObject>
#include <QSharedPointer>

namespace Diamesh {
struct VersionInfo;
struct SessionInfo;
} // namespace Diamesh

namespace HnsClient {

class SessionPrivate;
class CatalogImporter;
class DynamipsRouterDB;
class EntityEntryCatalog;
class EntryPropertiesManager;
class JobManager;
class MacAddressManager;
class PlatformMonitor;
class TopologyEntryCatalog;
class TopologyManager;
class TracerManager;
class UserManager;
class EntityInfoList;
class PrivateSessionAccessor;
class HSetManager;
class ACLManager;
class ResourceManager;

/*******************************************************************************
* Session
*******************************************************************************/
class Session : public QObject {
    Q_OBJECT
    Q_DECLARE_PRIVATE(Session)
public:
    ~Session();

    static Session *instance();
    static Session *create(QSharedPointer<Diamesh::SessionHelper> sessionHelper, QObject *sessionParent = 0);
    bool isAdvancedMode();
    Diamesh::SessionInfo sessionInformation();
    CatalogImporter *catalogImporter();
    DynamipsRouterDB *dynamipsRouterDB();
    EntityEntryCatalog *entityEntryCatalog();
    EntryPropertiesManager *entryPropertiesManager();
    JobManager *jobManager();
    MacAddressManager *macAddressManager();
    PlatformMonitor *platformMonitor();
    TopologyManager *topologyManager();
    TopologyEntryCatalog *topologyEntryCatalog();
    UserManager *userManager();
    HSetManager *hSetManager();
    ACLManager *aclManager();
    EntityInfoList *entityInfoList();
    TracerManager *tracerManager();
    ResourceManager *resourceManager();
    QMap<QString, QString> entityCatalogStorageMap();
    QMap<QString, QString> topologyCatalogStorageMap();
    Hns::UserPtr user();
    void getEntryNodeInfoMap(Hns::EntityEntryPtr const &entry, QObject *receiver, const char *slot);
    void disconnectSession();

signals:
    void loaded();
    void disconnected();

protected:
    void _load(QSharedPointer<Diamesh::SessionHelper> sessionHelper);
    void _callEnded();
    void _checkLoading();
    void _init();
    void _loadUserManager();
    void _loadEntityEntryCatalog();

protected slots:
    void _remoteMethodFailedSlot(QSharedPointer<Ice::Exception> exception);
    void _loadSessionInfoSlot(::Diamesh::SessionInfo sessionInformation);
    void _loadAdvancedModeSlot(bool isAdvancedMode);
    void _loadHSetManagerPrxSlot(::Diamesh::HSetManagerPrx hSetManagerPrx);
    void _loadACLManagerPrxSlot(::Diamesh::ACLManagerPrx aclManagerPrx);
    void _loadUserManagerPrxSlot(::Diamesh::UserManagerPrx userManagerPrx);
    void _loadAuthManagerPrxSlot(::Diamesh::AuthManagerPrx authManagerPrx);
    void _loadSessionManagerPrxSlot(::Diamesh::SessionManagerPrx sessionManagerPrx);
    void _loadCatalogImporterPrxSlot(::Hns::CatalogImporterPrx catalogImporterPrx);
    void _loadDynamipsRouterDBPrxSlot(::Hns::DynamipsRouterDBPrx dynamipsRouterDBPrx);
    void _loadEntityCatalogPrxSlot(::Hns::EntityCatalogPrx entityCatalogPrx);
    void _loadEntryPropertiesManagerPrxSlot(::Hns::EntryPropertiesManagerPrx entryPropertiesManagerPrx);
    void _loadJobManagerPrxSlot(::Hns::JobManagerPrx jobManagerPrx);
    void _loadMacAddressManagerPrxSlot(::Hns::MacAddressManagerPrx macAddressManagerPrx);
    void _loadTopologyCatalogPrxSlot(::Hns::TopologyCatalogPrx topologyCatalogPrx);
    void _loadTopologyManagerPrxSlot(::Hns::TopologyManagerPrx topologyManagerPrx);
    void _loadMasterTracerPrxSlot(::Hns::TracerPrx masterTracerPrx);
    void _loadResourceManagerPrxSlot(::Hns::CustomResourceManagerPrx managerPrx);

    void _loadEntityCatalogGetFileStoragesSlot(::Diamesh::StringStringMap entityFileStorageMap);
    void _loadTopologyCatalogGetFileStoragesSlot(::Diamesh::StringStringMap topologyFileStorageMap);

    void _loadUserManagerSlot();
    void _loadCatalogImporterSlot();
    void _loadDynamipsRouterDBSlot();
    void _loadEntityEntryCatalogSlot();
    void _loadEntryPropertiesManagerSlot();
    void _loadJobManagerSlot();
    void _loadMacAddressManagerSlot();
    void _loadTopologyEntryCatalogSlot();
    void _loadTopologyManagerSlot();
    void _loadPlatforMonitorSlot();
    void _loadHSetManagerSlot();
    void _loadACLManagerSlot();
    void _loadResourceManagerSlot();

    void _disconnectedSlot();
    QSharedPointer<Diamesh::SessionHelper> sessionHelper();

protected:
    explicit Session(QObject *parent = 0);

private:
    SessionPrivate *d_ptr;
    static Session *_singleton;
    friend class SessionLoader;
    friend class PrivateSessionAccessor;
};

} // namespace HnsClient

#endif //HNSCLIENT_SESSION_H
