/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef HNSCLIENT_TOPOLOGY_H
#define HNSCLIENT_TOPOLOGY_H

// Generated hynesim headers
#include "common/proxies_topology.h"

// Qt headers
#include <QObject>
#include <QString>
#include <QMap>
#include <QHash>
#include <QMultiHash>

namespace HnsClient {

class Session;
class TopologyPrivate;
class GenericEntity;
class TopologyEntry;
class Entity;
class WireEntity;

/*******************************************************************************
* Topology
*******************************************************************************/
class Topology : public QObject
{
    Q_OBJECT
    Q_DECLARE_PRIVATE(Topology)
public:
    explicit Topology(QObject *parent = 0);
    ~Topology();
    qlonglong topologyId();
    TopologyEntry *topologyEntry();
    bool isModified();
    bool isContentLoaded();
    Hns::TopologyLayout layout();
    QString getUuid();
    QList<qlonglong> connectedUsers() const;
    const QMap<int, Hns::BalancingBehaviorInfo> &getMapBehavior() const;
    void loadContent();
    void unloadContent();
    GenericEntity *getEntityById(qlonglong entityId);
    QHash<qlonglong, GenericEntity*> getEntities(bool includeWires = false);
    QHash<qlonglong, GenericEntity*> getWires();
    QMultiHash<qlonglong, Hns::TopologyLink> getLinks();
    QList<Hns::TopologyLink> getLinksByWireId(qlonglong wireId);
    void setConnectedUsers(Diamesh::LongList);
    void addConnectedUser(qlonglong id);
    void removeConnectedUser(qlonglong id);
    void updateEntry(TopologyEntry *entry);
    void setSize(int width, int height);
    void setBackgroundColor(QString colorCode);
    void setLayoutInformation(int width, int height, QString colorCode);
    void setMinimalSize(int width, int height);
    void updateEntityLayout(const Hns::EntityLayoutPtr &layout);
    void updateWireLayout(const Hns::WireLayoutPtr &layout);
    void removeEntity(qlonglong entityId);
    void unImplementEntity(qlonglong entityId);
    void removeWire(qlonglong wireId);
    void addWire(Hns::WireLayoutPtr layout, QString fromInfo, QString toInfo);
    void createEntity(qlonglong entryId, Hns::LayoutPtr layout, Hns::TopologyLinkList list);
    void cloneEntity(qlonglong entryId, Hns::EntityLayoutPtr layout, qlonglong modelEntityId, QObject *receiver, const char *slot);
    void implementEntity(HnsClient::Entity *modelEntity, qlonglong entryId);
    void importTopology(qlonglong topologyId, Hns::EntityLayoutPtr layout, QObject *receiver, const char *slot);
    void computeDefineInfos(QMap<Ice::Long, Hns::DefineInfo> map, Hns::BalancingBehavior behavior, QObject *receiver, const char *slot);
    void plugEntity(qlonglong entityId, int entityPortId, qlonglong wireId, int wirePortId, QObject *receiver, const char *slot);
    void unplugEntity(qlonglong entityId, int entityPortId, qlonglong wireId, int wirePortId, QObject *receiver, const char *slot);

signals:
    void loaded();
    void contentLoaded();
    void topologyChanged();
    void topologyLayoutChanged();
    void topologyLayoutOptionsChanged();
    void connectedUserAdded(qlonglong);
    void connectedUserRemoved(qlonglong);

    void entityCreated(HnsClient::Entity *entity);
    void entityEntryChanged(HnsClient::Entity *entity);
    void entityStatusChanged(HnsClient::Entity *entity);
    void entityRefreshed(HnsClient::Entity *entity);
    void entityLayoutChanged(Entity *entity);
    void entityRemoved(qlonglong entityID);
    void entityPlugged(qlonglong entityId, qlonglong wireId);
    void entityUnplugged(qlonglong entityId, qlonglong wireId, int wirePort);

    void wireCreated(HnsClient::WireEntity *wire);
    void wireRemoved(qlonglong wireId);

protected:
    bool _isLoaded() const;
    void _load(Hns::TopologyQPrx qprx);
    void _callEnded(bool loadingContent = false);
    void _checkLoading(bool loadingContent = false);
    void _init(bool loadingContent = false);
    void _handleEntityProxy(Hns::EntityPrx prx, bool countCalls = false);
    void _handleTopologyLink(Hns::TopologyLink link);
    Entity *_handleEntityData(Entity *entity, Hns::EntityLayoutPtr layout);
    void _updateEntry(TopologyEntry *entry, QObject *receiver, const char *slot, QObject *cookie);
    void _updateEntry(Hns::TopologyEntryPtr entry, QObject *receiver, const char *slot, QObject *cookie);
    void _refreshEntry(QObject *receiver, const char *slot);
    QString _getMetaClassHnv(const QString &metaClassHns);

    TopologyPrivate * const d_ptr;

private slots:
    void _loadGetIdSlot(::Ice::Long topologyId);
    void _loadGetLayoutSlot(::Hns::TopologyLayout layout);
    void _loadGetEntrySlot(::Hns::TopologyEntryPtr entry);
    void _loadGetScreenshotSlot(::Hns::ByteArray screenshotData, QObject *obj);
    void _loadGetTracerSlot(::Hns::TracerPrx tracerPrx);
    void _loadGetTopologyUsersSlot(::Diamesh::LongList list);
    void _loadGetIsModifiedSlot(bool isModified);
    void _loadGetBehaviorMapSlot(::Hns::BalancingBehaviorMap balancingBehaviorMap);
    void _loadGetACLsSlot(::Diamesh::ACLList aclList);

    void _loadGetEntitiesSlot(::Hns::LongEntityMap entities);
    void _loadEntityLoadedSlot(QObject *object);
    void _loadGetEntityLayoutSlot(::Hns::EntityLayoutPtr layout, QObject *obj);
    void _loadWireLoadedSlot(QObject* object);
    void _loadGetWireLayoutSlot(::Hns::WireLayoutPtr layout, QObject *obj);

    void _getEntrySlot(::Hns::TopologyEntryPtr entry);
    void _refreshEntrySlot(::Hns::TopologyEntryPtr entry, QObject *obj);
    void _getLayoutSlot(::Hns::TopologyLayout layout);
    void _getLayoutOptionsSlot(::Hns::TopologyLayoutOptions options);
    void _getEntitySlot(::Hns::EntityPrx prx);
    void _getEntityLoadedSlot(QObject *object);
    void _getEntityLayoutSlot(::Hns::EntityLayoutPtr layout, QObject *obj);
    void _getLinkSlot(::Hns::TopologyLink link);
    void _getWireLoadedSlot(QObject *object);
    void _getWireLayoutSlot(::Hns::WireLayoutPtr layout,QObject *obj);
    void _canBuildWithNodeSlot(::Diamesh::LongList entityDefinableList, QObject *obj);
    void _entityEntryChangedSlot(HnsClient::GenericEntity *generic);
    void _entityStatusChangedSlot(HnsClient::GenericEntity *generic);
    void _entityRefreshedSlot(HnsClient::GenericEntity *generic);
    void _removeEntitySlot(bool result, QObject *obj);
    void _unImplementModelEntitySlot(bool result, QObject *obj);
    void _removeWireSlot(bool result, QObject *obj);
    void _addWireSlot(::Hns::EntityTopologyLinkStruct);
    void _setACLsSlot(bool result, QObject *obj);
    void _plugEntitySlot(bool result, QObject *obj);
    void _loadGetTopologyLinksSlot(::Hns::LongLinkListMap map);
    void _handlePendingWires(qlonglong wireId);
    void _getTopologyLinks(::Hns::TopologyLinkList list);

    // Slots connected to topology proxy signals
    void _modifiedSlot(QString uuid, bool isModified);
    void _entryModifiedSlot(QString uuid);
    void _layoutChangedSlot();
    void _layoutOptionsChangedSlot();
    void _aclsChangedSlot(Diamesh::ACLList aclList);

    void _entityAddedSlot(Ice::Long topologyId, Ice::Long entityId);
    void _entityRemovedSlot(Ice::Long entityId);
    void _entityLayoutChangedSlot(Hns::EntityLayoutPtr layout);
    void _entityPluggedSlot(QString wireSide, QString entitySide);
    void _entityUnpluggedSlot(QString wireSide, QString entitySide);

    void _wireAddedSlot(Ice::Long topologyId, Ice::Long wireId);
    void _wireRemovedSlot(Ice::Long wireId);
    void _wireLayoutChangedSlot(Hns::WireLayoutPtr layout);

    // Slots connected to PlatformMonitor
    void _nodeAddedSlot(QString nodeName);
    void _nodeRemovedSlot(QString nodeName);
    void _refreshClassNameSlot(QString className);

    void _cloneStartedSlot(::Ice::Int cloneJobId, QObject *obj);
    void _entityPrxCreatedSlot(::Hns::EntityTopologyLinkStruct entityPrx);
    void _entityImplementedSlot(::Hns::EntityPrx entityPrx);

private:
    friend class TopologyManager;
};

} // namespace HnsClient

#endif //HNSCLIENT_TOPOLOGY_H
