/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// hynesim hnsclient headers
#include "libhnsclient/private/topologyentrycatalog_p.h"
#include "libhnsclient/topologyentrycatalog.h"
#include "libhnsclient/entry.h"
#include "libhnsclient/utility.h"

// Qt headers
#include <QSharedPointer>
#include <QDebug>

namespace HnsClient {

/*******************************************************************************
* GetTopologyEntryCookie
*******************************************************************************/
GetTopologyEntryCookie::GetTopologyEntryCookie(qlonglong entryId)
    : QObject()
    , _entryId(entryId)
{
}

GetTopologyEntryCookie::~GetTopologyEntryCookie()
{
}

qlonglong GetTopologyEntryCookie::entryId() const
{
    return _entryId;
}


/*******************************************************************************
* TopologyEntryCatalogCookie
*******************************************************************************/
TopologyEntryCatalogCookie::TopologyEntryCatalogCookie(Hns::TopologyEntryPtr entry)
    : QObject()
    , _entry(entry)
{
}

TopologyEntryCatalogCookie::~TopologyEntryCatalogCookie()
{
}

Hns::TopologyEntryPtr TopologyEntryCatalogCookie::getPtr() const
{
    return _entry;
}


/*******************************************************************************
* GetTopologyEntryUpdateCookie
*******************************************************************************/
GetTopologyEntryUpdateCookie::GetTopologyEntryUpdateCookie(qlonglong entryId)
    : QObject()
    , _entryId(entryId)
{
}

GetTopologyEntryUpdateCookie::~GetTopologyEntryUpdateCookie()
{
}

qlonglong GetTopologyEntryUpdateCookie::topologyEntryId() const
{
    return _entryId;
}


/*******************************************************************************
* TopologyEntryCatalogPrivate
*******************************************************************************/
TopologyEntryCatalogPrivate::TopologyEntryCatalogPrivate()
    : CatalogPrivate()
{
}

TopologyEntryCatalogPrivate::~TopologyEntryCatalogPrivate()
{
    catalogQtProxy = 0;
    qDeleteAll(topologyHash);
    topologyHash.clear();
}


/*******************************************************************************
* UpdateTopologyEntryCookie
*******************************************************************************/
UpdateTopologyEntryCookie::UpdateTopologyEntryCookie(Hns::TopologyEntryPtr entry)
    : QObject()
    , _entry(entry)
{
}

UpdateTopologyEntryCookie::~UpdateTopologyEntryCookie()
{
}

QString UpdateTopologyEntryCookie::getEntryName() const
{
    return QString::fromStdString(_entry->shortName);
}

qlonglong UpdateTopologyEntryCookie::getID() const
{
    return _entry->id;
}

Hns::TopologyEntryPtr UpdateTopologyEntryCookie::topologyEntry()
{
    return _entry;
}


/*******************************************************************************
* TopologyEntryCatalog
*******************************************************************************/
TopologyEntryCatalog::TopologyEntryCatalog(QObject *parent)
    : Catalog(new TopologyEntryCatalogPrivate, parent)
{
}

TopologyEntryCatalog::~TopologyEntryCatalog()
{
}

/*! \fn getTopologyEntryHash() const
 *\brief TODO
 */
QHash<qlonglong, TopologyEntry *> TopologyEntryCatalog::getTopologyEntryHash() const
{
    Q_D(const TopologyEntryCatalog);
    return d->topologyHash;
}

/*! \fn *topologyEntryByUUID(QString uuid) const
 *\brief TODO
 *\param uuid
 */
TopologyEntry *TopologyEntryCatalog::topologyEntryByUUID(QString uuid) const
{
    Q_D(const TopologyEntryCatalog);
    TopologyEntry *result = 0;
    foreach(TopologyEntry *entry, d->topologyHash.values()) {
        if (entry->getUUID() == uuid) {
            result = entry;
            break;
        }
    }
    return result;
}

/*! \fn *topologyEntryByID(qlonglong topologyEntryId)
 *\brief TODO
 *\param topologyEntryId
 */
TopologyEntry *TopologyEntryCatalog::topologyEntryByID(qlonglong topologyEntryId)
{
    Q_D(const TopologyEntryCatalog);
    return d->topologyHash.value(topologyEntryId, 0);
}

/*! \fn fixTopologyEntry(TopologyEntry *topologyEntry)
 *\brief TODO
 *\param topologyEntryId
 */
void TopologyEntryCatalog::fixTopologyEntry(TopologyEntry *topologyEntry)
{
    Q_D(TopologyEntryCatalog);
    qlonglong entryId = topologyEntry->getID();
    if (d->topologyHash.contains(entryId)) {
        _setTopologySyncing(entryId, Syncing);
        TopologyEntryCatalogCookie *cookie = new TopologyEntryCatalogCookie(topologyEntry->topologyEntry());
        d->catalogQtProxy->begin_fixTopologyEntry(topologyEntry->topologyEntry(), this, SLOT(_fixTopologyEntrySlot(bool, QObject*)), 0, 0, cookie);

    } else {
        qDebug() << tr("Topology not in cache. Could not request fix");
    }
}

/*! \fn setScreenshot(qlonglong entryId, Hns::ByteArray data)
 *\brief TODO
 *\param entryId
 *\param data
 */
bool TopologyEntryCatalog::setScreenshot(qlonglong entryId, Hns::ByteArray data)
{
    Q_D(TopologyEntryCatalog);
    if (d->topologyHash.contains(entryId)) {
        _setTopologySyncing(entryId, Syncing);
        TopologyEntryCatalogCookie *cookie = new TopologyEntryCatalogCookie(d->topologyHash[entryId]->topologyEntry());
        d->catalogQtProxy->begin_setScreenshot(entryId, data, this, SLOT(_setScreenshotSlot(bool, QObject*)), 0, 0, cookie);
        return true;
    } else {
        qDebug() << tr("Topology not in cache. Could not set screenshot fix");
        return false;
    }
}

/*! \fn setScreenshotByUUID(QString uuid, Hns::ByteArray data)
 *\brief TODO
 *\param uuid
 *\param data
 */
void TopologyEntryCatalog::setScreenshotByUUID(QString uuid, Hns::ByteArray data)
{
    TopologyEntry *topologyEntry = topologyEntryByUUID(uuid);
    if (topologyEntry)
        setScreenshot(topologyEntry->getID(), data);
}

/*! \fn getEntryScreenshot(qlonglong entryId, QObject *receiver, const char *slot, QObject *cookie = 0)
 *\brief TODO
 *\param entryId
 *\param receiver
 *\param slot
 *\param cookie
 */
void TopologyEntryCatalog::getEntryScreenshot(qlonglong entryId, QObject *receiver, const char *slot, QObject *cookie)
{
    Q_D(TopologyEntryCatalog);
    d->catalogQtProxy->begin_getScreenshot(entryId, receiver, slot, 0, 0, cookie);
}

/*! \fn updateTopologyEntry(TopologyEntry *updatedTopologyEntry)
 *\brief TODO
 *\param updatedTopologyEntry
 */
void TopologyEntryCatalog::updateTopologyEntry(TopologyEntry *updatedTopologyEntry)
{
    Q_D(TopologyEntryCatalog);
    qlonglong topologyEntryID = updatedTopologyEntry->getID();
    if (!d->topologyHash.contains(topologyEntryID)) {
        qDebug() << tr("Topology not in cache. Could not request update");
        return;
    }
    _setTopologySyncing(topologyEntryID, Syncing);
    UpdateTopologyEntryCookie *cookie = new UpdateTopologyEntryCookie(updatedTopologyEntry->topologyEntry());
    d->catalogQtProxy->begin_setACLsForCatalogItem(updatedTopologyEntry->getID(), updatedTopologyEntry->aclList(), this, SLOT(_setACLsForEntrySlot(bool,QObject*)), this, SLOT(_updateACLMethodFailedSlot(QSharedPointer<Ice::Exception>,QObject*)), cookie);
}

void TopologyEntryCatalog::_load(Hns::TopologyCatalogQPrx catalogQPrx)
{
    Q_D(TopologyEntryCatalog);
    d->catalogQtProxy = catalogQPrx;
    _loadEntries();
}

void TopologyEntryCatalog::_callEnded()
{
    Q_D(TopologyEntryCatalog);
    d->remoteCallCount--;
    _checkLoading();
}

void TopologyEntryCatalog::_checkLoading()
{
    Q_D(TopologyEntryCatalog);
    if (d->remoteCallCount == 0)
        _init();
}

void TopologyEntryCatalog::_init()
{
    Q_D(TopologyEntryCatalog);
    connect(d->catalogQtProxy.data(), SIGNAL(entryAdded(Ice::Long)), this, SLOT(_entryAddedSlot(Ice::Long)));
    connect(d->catalogQtProxy.data(), SIGNAL(entryChanged(Ice::Long)), this, SLOT(_entryChangedSlot(Ice::Long)));
    connect(d->catalogQtProxy.data(), SIGNAL(entryRemoved(Ice::Long)), this, SLOT(_entryRemovedSlot(Ice::Long)));
    connect(d->catalogQtProxy.data(), SIGNAL(entryLocked(Ice::Long, Ice::Int)), this, SLOT(_entryLockedSlot(Ice::Long, Ice::Int)));
    connect(d->catalogQtProxy.data(), SIGNAL(entryUnlocked(Ice::Long)), this, SLOT(_entryUnlockedSlot(Ice::Long)));
    connect(d->catalogQtProxy.data(), SIGNAL(entryACLsChanged(Ice::Long, Diamesh::ACLList)), this, SLOT(_entryACLsChangedSlot(Ice::Long, Diamesh::ACLList)));
    connect(d->catalogQtProxy.data(), SIGNAL(screenshotUpdated(Ice::Long)), this, SLOT(_screenshotUpdatedSlot(Ice::Long)));
    emit loaded();
}

Hns::CatalogQPrx TopologyEntryCatalog::_catalogProxy() const
{
    Q_D(const TopologyEntryCatalog);
    return d->catalogQtProxy;
}

void TopologyEntryCatalog::_parseEntryList(Diamesh::LongList entryList)
{
    Q_D(TopologyEntryCatalog);
    d->remoteCallCount += entryList.size();
    foreach(Ice::Long entryID, entryList)
        d->catalogQtProxy->begin_getTopologyEntry(entryID, this, SLOT(_loadGetTopologyEntrySlot(::Hns::TopologyEntryPtr)));
    _checkLoading();
}

void TopologyEntryCatalog::_updateEntryLockedStatus(Ice::Long entryId, bool isLocked)
{
    Q_D(TopologyEntryCatalog);
    TopologyEntry * entry = d->topologyHash.value(entryId, 0);
    if (entry) {
        entry->setLocked(isLocked);
        emit topologyUpdated(entry);
    }
}

void TopologyEntryCatalog::_setTopologySyncing(qlonglong topologyEntryId, SynchronizationStatus status)
{
    Q_D(TopologyEntryCatalog);
    TopologyEntry *entry = d->topologyHash.value(topologyEntryId, 0);
    if (entry) {
        entry->setSyncStatus(status);
        emit topologyUpdated(entry);
    }
}

bool TopologyEntryCatalog::_handleTopologyEntryData(Hns::TopologyEntryPtr topologyEntry, qlonglong topologyEntryId, QObject *receiver, const char *slot)
{
    Q_D(TopologyEntryCatalog);
    if (!topologyEntry) {
        d->loadingEntryList.removeAll(topologyEntryId);
        return false;
    }
    if (d->topologyHash.contains(topologyEntryId)) {
        d->loadingEntryList.removeAll(topologyEntryId);
        return false;
    }
    if (receiver) {
        TopologyEntryCatalogCookie *cookie = new TopologyEntryCatalogCookie(topologyEntry);
        d->catalogQtProxy->begin_getScreenshot(topologyEntryId, receiver, slot, 0, 0, cookie);
    }
    return true;
}

TopologyEntry *TopologyEntryCatalog::_handleScreenshotData(Hns::ByteArray screenshotData, Hns::TopologyEntryPtr topologyEntry)
{
    Q_D(TopologyEntryCatalog);
    QPixmap screenshot = Utility::pixmapFromByteArray(screenshotData);
    TopologyEntry *topologyEntryObj = new TopologyEntry(topologyEntry, screenshot);
    qlonglong entryId = topologyEntryObj->getID();
    GetACLsForEntryCookie *cookie = new GetACLsForEntryCookie(entryId);
    d->catalogQtProxy->begin_getACLsForCatalogItem(entryId, this, SLOT(_getACLsForEntrySlot(::Diamesh::ACLList,QObject*)), 0, 0, cookie);
    d->topologyHash.insert(topologyEntryObj->getID(), topologyEntryObj);
    return topologyEntryObj;
}

void TopologyEntryCatalog::_loadGetTopologyEntrySlot(Hns::TopologyEntryPtr entry)
{
    if (!_handleTopologyEntryData(entry, entry->id, this, SLOT(_loadGetScreenshotSlot(::Hns::ByteArray,QObject*))))
        _callEnded();
}

void TopologyEntryCatalog::_loadGetScreenshotSlot(Hns::ByteArray screenshotData, QObject *obj)
{
    TopologyEntryCatalogCookie *cookie = qobject_cast<TopologyEntryCatalogCookie *>(obj);
    if (cookie) {
        _handleScreenshotData(screenshotData, cookie->getPtr());
        cookie->deleteLater();
    }
    _callEnded();
}

void TopologyEntryCatalog::_loadGetScreenshotForEntryAddedSlot(Hns::ByteArray screenshotData, QObject *obj)
{
    Q_D(TopologyEntryCatalog);
    TopologyEntryCatalogCookie *cookie = qobject_cast<TopologyEntryCatalogCookie *>(obj);
    if (cookie) {
        TopologyEntry *topologyEntryObj = _handleScreenshotData(screenshotData, cookie->getPtr());
        d->loadingEntryList.removeAll(topologyEntryObj->getID());
        emit topologyAdded(topologyEntryObj);
        cookie->deleteLater();
    }
    _callEnded();
}

void TopologyEntryCatalog::_getTopologyEntrySlot(Hns::TopologyEntryPtr entry, QObject *obj)
{
    GetTopologyEntryCookie *cookie = qobject_cast<GetTopologyEntryCookie *>(obj);
    if (cookie) {
        _handleTopologyEntryData(entry, cookie->entryId(), this, SLOT(_loadGetScreenshotForEntryAddedSlot(::Hns::ByteArray,QObject*)));
        cookie->deleteLater();
    }
}

void TopologyEntryCatalog::_getScreenshotSlot(Hns::ByteArray screenshotData, QObject *obj)
{
    Q_D(TopologyEntryCatalog);
    TopologyEntryCatalogCookie *cookie = qobject_cast<TopologyEntryCatalogCookie *>(obj);
    if (cookie) {
        TopologyEntry *topologyEntryObj = _handleScreenshotData(screenshotData, cookie->getPtr());
        d->loadingEntryList.removeAll(topologyEntryObj->getID());
        emit topologyAdded(topologyEntryObj);
        cookie->deleteLater();
    }
}

void TopologyEntryCatalog::_getTopologyEntryUpdateSlot(Hns::TopologyEntryPtr entry, QObject *obj)
{
    Q_D(TopologyEntryCatalog);
    GetTopologyEntryUpdateCookie *cookie = qobject_cast<GetTopologyEntryUpdateCookie *>(obj);
    if (cookie) {
        qlonglong entryId = cookie->topologyEntryId();
        if (entry == 0)
            _entryRemovedSlot(entryId);
        else {
            TopologyEntry *topologyEntry = topologyEntryByID(entryId);
            if (topologyEntry) {
                topologyEntry->setSourceCatalogItem(entry);
                topologyEntry->refreshProperties();
                emit topologyUpdated(topologyEntry);
            } else if (!d->loadingEntryList.contains(entryId)) {
                d->loadingEntryList.append(entryId);
                _handleTopologyEntryData(entry, entryId, this, SLOT(_getScreenshotSlot(::Hns::ByteArray,QObject*)));
            }
        }
        cookie->deleteLater();
    }
}

void TopologyEntryCatalog::_getScreenshotUpdateSlot(::Hns::ByteArray screenshotData, QObject *obj)
{
    TopologyEntryCatalogCookie *cookie = qobject_cast<TopologyEntryCatalogCookie *>(obj);
    if (cookie) {
        TopologyEntry *topologyEntry = topologyEntryByID(cookie->getPtr()->id);
        if (topologyEntry) {
            QPixmap screenshot = Utility::pixmapFromByteArray(screenshotData);
            topologyEntry->setScreenshot(screenshot);
            emit topologyUpdated(topologyEntry);
        }
        cookie->deleteLater();
    }
}

void TopologyEntryCatalog::_fixTopologyEntrySlot(bool result, QObject *obj)
{
    TopologyEntryCatalogCookie *cookie = qobject_cast<TopologyEntryCatalogCookie *>(obj);
    if (cookie) {
        qlonglong entryId = cookie->getPtr()->id;
        _setTopologySyncing(entryId, Synced);
        if (!result)
            qCritical() << tr("Could not fix topology '%1'").arg(QString::fromStdString(cookie->getPtr()->shortName));
        cookie->deleteLater();
    }
}

void TopologyEntryCatalog::_setScreenshotSlot(bool result, QObject *obj)
{
    TopologyEntryCatalogCookie *cookie = qobject_cast<TopologyEntryCatalogCookie *>(obj);
    if (cookie) {
        qlonglong entryId = cookie->getPtr()->id;
        if (result) {
            _setTopologySyncing(entryId, Synced);
        } else {
            _setTopologySyncing(entryId, Error);
            qCritical() << tr("Could not set screenshot for topology '%1'").arg(QString::fromStdString(cookie->getPtr()->shortName));
        }
        cookie->deleteLater();
    }
}

void TopologyEntryCatalog::_updateTopologyEntrySlot(bool result, QObject *obj)
{
    UpdateTopologyEntryCookie *cookie = qobject_cast<UpdateTopologyEntryCookie *>(obj);
    if (cookie){
        qlonglong entryId = cookie->getID();
        if (result)
            _setTopologySyncing(entryId, Synced);
        else {
            _setTopologySyncing(entryId, Error);
            qCritical() << tr("Topology '%1' update failed.").arg(cookie->getEntryName());
        }
        delete cookie;
    }
}

void TopologyEntryCatalog::_setACLsForEntrySlot(bool result, QObject *obj)
{
    Q_D(TopologyEntryCatalog);
    UpdateTopologyEntryCookie *cookie = qobject_cast<UpdateTopologyEntryCookie *>(obj);
    if (cookie) {
        qlonglong entryId = cookie->getID();
        if (result) {
            d->catalogQtProxy->begin_updateEntry(cookie->topologyEntry(), this, SLOT(_updateTopologyEntrySlot(bool,QObject*)), 0, 0, cookie);
        } else {
            _setTopologySyncing(entryId, Synced);
            qCritical() << tr("Topology '%1' update failed.").arg(cookie->getEntryName());
            delete cookie;
        }
    }
}

void TopologyEntryCatalog::_entryAddedSlot(Ice::Long entryId)
{
    Q_D(TopologyEntryCatalog);
    if (!d->loadingEntryList.contains(entryId)) {
        d->loadingEntryList.append(entryId);
        GetTopologyEntryCookie *cookie = new GetTopologyEntryCookie(entryId);
        d->catalogQtProxy->begin_getTopologyEntry(entryId, this, SLOT(_getTopologyEntrySlot(::Hns::TopologyEntryPtr, QObject*)), 0, 0, cookie);
    }
}

void TopologyEntryCatalog::_entryChangedSlot(Ice::Long entryId)
{
    Q_D(TopologyEntryCatalog);
    GetTopologyEntryUpdateCookie *cookie = new GetTopologyEntryUpdateCookie(entryId);
    d->catalogQtProxy->begin_getTopologyEntry(entryId, this, SLOT(_getTopologyEntryUpdateSlot(::Hns::TopologyEntryPtr,QObject*)), 0, 0, cookie);
}

void TopologyEntryCatalog::_entryRemovedSlot(Ice::Long entryId)
{
    Q_D(TopologyEntryCatalog);
    TopologyEntry *entry = d->topologyHash.take(entryId);
    if (entry) {
        emit topologyRemoved(entryId);
        delete entry;
    }
}

void TopologyEntryCatalog::_entryACLsChangedSlot(Ice::Long entryId, Diamesh::ACLList aclList)
{
    TopologyEntry *topologyEntry = topologyEntryByID(entryId);
    if (topologyEntry)
        topologyEntry->_updateACLList(aclList);
    else
        _entryAddedSlot(entryId);
}

void TopologyEntryCatalog::_screenshotUpdatedSlot(Ice::Long entryId)
{
    Q_D(TopologyEntryCatalog);
    TopologyEntry *topologyEntry = topologyEntryByID(entryId);
    if (topologyEntry) {
        TopologyEntryCatalogCookie *cookie = new TopologyEntryCatalogCookie(topologyEntry->topologyEntry());
        d->catalogQtProxy->begin_getScreenshot(entryId, this, SLOT(_getScreenshotUpdateSlot(::Hns::ByteArray,QObject*)), 0, 0, cookie);
    }
}

void TopologyEntryCatalog::_getACLsForEntrySlot(Diamesh::ACLList aclList, QObject *obj)
{
    GetACLsForEntryCookie *cookie = qobject_cast<GetACLsForEntryCookie *>(obj);
    if (cookie) {
        qlonglong entryId = cookie->getEntryId();
        TopologyEntry *topologyEntry = topologyEntryByID(entryId);
        if (topologyEntry) {
            topologyEntry->_updateACLList(aclList);
        }
        delete cookie;
    }
}

void TopologyEntryCatalog::_updateACLMethodFailedSlot(QSharedPointer<Ice::Exception> exception, QObject* obj)
{
    QSharedPointer<Hns::RightError> rightError = qSharedPointerDynamicCast<Hns::RightError>(exception);
    UpdateTopologyEntryCookie *cookie = qobject_cast<UpdateTopologyEntryCookie *>(obj);
    if (rightError && cookie) {
        QString errorMessage = QString::fromStdString(rightError.data()->message);
        _setTopologySyncing(cookie->getID(), Synced);
        emit updateACLFailed(errorMessage);
    }
}

} // namespace HnsClient
