/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef HNSCLIENT_TOPOLOGYENTRYCATALOG_H
#define HNSCLIENT_TOPOLOGYENTRYCATALOG_H

// hynesim hnsclient headers
#include "libhnsclient/catalog.h"
#include "libhnsclient/common.h"

// Generated hynesim headers
#include "common/proxies_topologycatalog.h"

// Qt headers
#include <QHash>

namespace HnsClient {

class TopologyEntry;
class TopologyEntryCatalogPrivate;

/*******************************************************************************
* TopologyEntryCatalog
*******************************************************************************/
class TopologyEntryCatalog : public Catalog
{
    Q_OBJECT
    Q_DECLARE_PRIVATE(TopologyEntryCatalog)
    
public:
    TopologyEntryCatalog(QObject *parent = 0);
    ~TopologyEntryCatalog();
    QHash<qlonglong, TopologyEntry *> getTopologyEntryHash() const;
    TopologyEntry *topologyEntryByUUID(QString uuid) const;
    TopologyEntry *topologyEntryByID(qlonglong topologyEntryId);
    void fixTopologyEntry(TopologyEntry *topologyEntry);
    bool setScreenshot(qlonglong entryId, Hns::ByteArray data);
    void setScreenshotByUUID(QString uuid, Hns::ByteArray data);
    void getEntryScreenshot(qlonglong entryId, QObject *receiver, const char *slot, QObject *cookie = 0);
    void updateTopologyEntry(TopologyEntry *updatedTopologyEntry);
    
signals:
    void loaded();
    void topologyAdded(HnsClient::TopologyEntry *entry);
    void topologyUpdated(HnsClient::TopologyEntry *entry);
    void topologyRemoved(qlonglong topologyEntryID);

protected:
    void _load(Hns::TopologyCatalogQPrx catalogQPrx);
    void _callEnded();
    void _checkLoading();
    void _init();

    virtual Hns::CatalogQPrx _catalogProxy() const;
    virtual void _parseEntryList(Diamesh::LongList entryList);
    virtual void _updateEntryLockedStatus(Ice::Long entryId, bool isLocked);

    void _setTopologySyncing(qlonglong topologyEntryId, SynchronizationStatus status);

    bool _handleTopologyEntryData(Hns::TopologyEntryPtr topologyEntry, qlonglong topologyEntryId, QObject *receiver = 0, const char *slot = 0);
    TopologyEntry *_handleScreenshotData(Hns::ByteArray screenshotData, Hns::TopologyEntryPtr topologyEntry);

protected slots:
    void _loadGetTopologyEntrySlot(::Hns::TopologyEntryPtr entry);
    void _loadGetScreenshotSlot(::Hns::ByteArray screenshotData, QObject *obj);
    void _loadGetScreenshotForEntryAddedSlot(::Hns::ByteArray screenshotData, QObject *obj);

    void _getTopologyEntrySlot(::Hns::TopologyEntryPtr entry, QObject *obj);
    void _getScreenshotSlot(::Hns::ByteArray screenshotData, QObject *obj);

    void _getTopologyEntryUpdateSlot(::Hns::TopologyEntryPtr entry, QObject *obj);
    void _getScreenshotUpdateSlot(::Hns::ByteArray screenshotData, QObject *obj);

    void _fixTopologyEntrySlot(bool result, QObject *obj);
    void _setScreenshotSlot(bool result, QObject *obj);
    
    void _updateTopologyEntrySlot(bool result, QObject *obj);
    void _setACLsForEntrySlot(bool result, QObject*obj);

    virtual void _entryAddedSlot(Ice::Long entryId);
    virtual void _entryChangedSlot(Ice::Long entryId);
    virtual void _entryRemovedSlot(Ice::Long entryId);
    void _entryACLsChangedSlot(Ice::Long entryId, Diamesh::ACLList aclList);
    void _screenshotUpdatedSlot(Ice::Long entryId);
    void _getACLsForEntrySlot(::Diamesh::ACLList aclList, QObject *obj);
    void _updateACLMethodFailedSlot(QSharedPointer<Ice::Exception> exception, QObject* obj);

private:
    friend class Session;
    friend class TopologyManager;
};

} // namespace HnsClient

#endif //HNSCLIENT_TOPOLOGYENTRYCATALOG_H
