/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef HNSCLIENT_TOPOLOGYMANAGER_H
#define HNSCLIENT_TOPOLOGYMANAGER_H

// Generated hynesim headers
#include "common/proxies_topologymanager.h"

// Qt headers
#include <QObject>
#include <QString>

namespace HnsClient {

class TopologyManagerPrivate;
class Session;
class Topology;
class EntityInfoList;
class TopologyEntry;
class SaveTopologyCookie;
class SaveAsTopologyCookie;
class LoadTopologyCookie;

/*******************************************************************************
* TopologyManager
*******************************************************************************/
class TopologyManager : public QObject
{
    Q_OBJECT
    Q_DECLARE_PRIVATE(TopologyManager)
public:
    explicit TopologyManager(QObject *parent = 0);
    ~TopologyManager();
    EntityInfoList *entityInfoList();
    Topology *topologyByUUID(QString uuid);
    Topology *topologyById(qlonglong id);
    Topology *topologyByEntryId(qlonglong entryId);
    void createTopology(int width, int height, QString colorName, bool isModel, QObject *receiver, const char *slot);
    void loadTopology(qlonglong topologyEntryId, QObject *receiver, const char *slot);
    void loadTopologyByUUID(QString topologyUUID, QObject *receiver, const char *slot);
    void unloadTopology(Topology *topology, QObject *receiver, const char *slot);
    void unloadTopologyByUUID(QString uuid, QObject *receiver, const char *slot);
    void deleteTopology(Topology *topology, QObject *receiver, const char *slot);
    void deleteTopologyByUUID(QString uuid, QObject *receiver, const char *slot);
    void deleteTopologyByEntry(TopologyEntry *entry, QObject *receiver, const char *slot);
    void deleteTopologyByEntryId(qlonglong entryId, QObject *receiver, const char *slot);
    void switchTopologyType(QString topologyUUID, QString backgroundColorName, QObject *receiver, const char *slot);
    void closeTopology(Topology *topology);
    void closeTopologyByUUID(QString uuid);
    void getTopologyUsers(qlonglong topologyId, QObject *receiver, const char *slot);
    void saveTopology(Topology *topology, TopologyEntry *updatedEntry, QObject *receiver, const char *slot, Hns::ByteArray screenshotData = Hns::ByteArray());
    void saveTopologyAs(Topology *topology, TopologyEntry *updatedEntry, QObject *receiver, const char *slot, Hns::ByteArray screenshotData = Hns::ByteArray());

signals:
    void loaded();
    void topologyAdded(HnsClient::Topology *topology);
    void topologyRemoved(qlonglong topologyId);

    void topologyLoaded(QString topologyUUID);
    void topologyUnloaded(QString topologyUUID, bool isDeleted);
    void topologyModified(QString topologyUUID, bool isModified);
    void topologySavedAs(QString oldUUID, QString newUUID);

protected:
    void _load(Hns::TopologyManagerQPrx qprx);
    void _callEnded();
    void _checkLoading();
    void _init();
    void _handleEntityInfo(QObject *object);
    void _handleTopologyPrx(Hns::TopologyPrx topologyPrx, LoadTopologyCookie *cookie = 0);
    Topology *_handleTopology(QObject *object);
    Topology *_handleTopology(Topology *topology);
    void _saveTopology(SaveTopologyCookie *cookie, TopologyEntry *updatedEntry = 0);
    void _loadTopology(qlonglong entryId, LoadTopologyCookie *cookie);
    void _handleScreenshotDataCookie(QObject *obj, qlonglong entryId, bool result);
    Hns::ByteArray _getScreenshotDataCookie(QObject *obj);

protected slots:
    void _loadGetEntityListInfoSlot(::Hns::EntityInfoMap entityInfoMap);
    void _loadEntityInfoLoadedSlot(QObject *object);
    void _loadGetTopologiesSlot(::Diamesh::LongList topologyIdList);
    void _loadGetTopologyByIdSlot(::Hns::TopologyPrx topologyPrx);
    void _loadTopologyObjectLoadedSlot(QObject *object);

    void _addEntityInfoSlot(Ice::Long entityId, QString entityInfoPrxStr);
    void _removeEntityInfoSlot(Ice::Long entityId);
    void _topologyLoadedSlot(QString topologyUUID, Ice::Long topologyId);
    void _topologyUnloadedSlot(QString topologyUUID, Ice::Long topologyId, bool isDeleted);
    void _topologyModifiedSlot(QString topologyUUID, bool isModified);
    void _topologySavedAsSlot(QString oldUUID, QString newUUID);
    void _userOpenedTopologySlot(Ice::Long userId, Ice::Long topologyId);
    void _userClosedTopologySlot(Ice::Long userId, Ice::Long topologyId);

    void _getTopologyByIdSlot(::Hns::TopologyPrx topologyPrx);
    void _createTopologySlot(::Hns::TopologyPrx topologyPrx, QObject *obj);
    void _loadTopologySlot(::Hns::TopologyPrx topologyPrx, QObject *obj);
    void _unloadTopologySlot(bool result, QObject *obj);
    void _deleteTopologySlot(bool result, QObject *obj);
    void _saveTopologySlot(bool result, QObject *obj);
    void _savetopologyFailedSlot(QSharedPointer<Ice::Exception> exception, QObject *obj);
    void _saveTopologyAsSlot(bool result, QObject *obj);
    void _saveTopologyAsFailedSlot(QSharedPointer<Ice::Exception> exception, QObject *obj);
    void _switchTopologyTypeSlot(::Ice::Long newEntryId, QObject *obj);

    void _entityInfoLoadedSlot(QObject *object);
    void _topologyObjectLoadedSlot(QObject *object);
    void _loadCookieCompletedSlot(HnsClient::Topology *topology);

    void _updateHashSlot(HnsClient::Topology*, QString, SaveAsTopologyCookie *);
    void _topologyEntryAddedSlot(HnsClient::TopologyEntry *);

private:
    TopologyManagerPrivate * const d_ptr;
    friend class Session;
};

} // namespace HnsClient

#endif //HNSCLIENT_TOPOLOGYMANAGER_H
