/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// hynesim hnsclient headers
#include "libhnsclient/tracermanager.h"
#include "libhnsclient/private/tracermanager_p.h"
#include "libhnsclient/entity.h"
#include "libhnsclient/topology.h"

namespace HnsClient {

/*******************************************************************************
* TracerManagerPrivate
*******************************************************************************/
TracerManagerPrivate::TracerManagerPrivate()
{
    masterTracer = 0;
}

TracerManagerPrivate::~TracerManagerPrivate()
{
    entityTracerHash.clear();
    topologyTracerHash.clear();
    masterTracer = 0;
}

/*******************************************************************************
* TracerManager
*******************************************************************************/
TracerManager::TracerManager(QObject *parent)
    : QObject(parent)
    , d_ptr(new TracerManagerPrivate)
{
}

TracerManager::~TracerManager()
{
    delete d_ptr;
}

/*! \fn registerMasterTracer(Hns::TracerQPrx tracerProxy)
 *\brief TODO
 *\param tracerProxy
 */
void TracerManager::registerMasterTracer(Hns::TracerQPrx tracerProxy)
{
    Q_D(TracerManager);
    if (!tracerProxy)
        return;
    _initTracer(tracerProxy);
    d->masterTracer = tracerProxy;
}

/*! \fn registerEntityTracer(GenericEntity * entity, Hns::TracerQPrx tracerProxy)
 *\brief TODO
 *\param entity
 *\param tracerProxy
 */
void TracerManager::registerEntityTracer(GenericEntity *entity, Hns::TracerQPrx tracerProxy)
{
    Q_D(TracerManager);
    if (!entity || !tracerProxy)
        return;
    connect(entity, SIGNAL(destroyed(QObject*)), this, SLOT(_entityDestroyed(QObject *)));
    _initTracer(tracerProxy);
    d->entityTracerHash.insert(entity, tracerProxy);
}

/*! \fn registerTopologyTracer(Topology * topology, Hns::TracerQPrx tracerProxy)
 *\brief TODO
 *\param topology
 *\param tracerProxy
 */
void TracerManager::registerTopologyTracer(Topology *topology, Hns::TracerQPrx tracerProxy)
{
    Q_D(TracerManager);
    if (!topology || !tracerProxy)
        return;
    connect(topology, SIGNAL(destroyed(QObject*)), this, SLOT(_topologyDestroyed(QObject*)));
    _initTracer(tracerProxy);
    d->topologyTracerHash.insert(topology, tracerProxy);
}

/*! \fn unregisterTopologyTracer(Topology *topology, Hns::TracerQPrx tracerProxy)
 *\brief TODO
 *\param topology
 *\param tracerProxy
 */
void TracerManager::unregisterTopologyTracer(Topology *topology, Hns::TracerQPrx tracerProxy)
{
    Q_D(TracerManager);
    if (!topology || !tracerProxy)
        return;
    if (d->topologyTracerHash.contains(topology)) {
        tracerProxy.data()->disconnect(this);
        d->topologyTracerHash.remove(topology);
    }
}

/*! \fn unregisterEntityTracer(GenericEntity *entity, Hns::TracerQPrx tracerProxy)
 *\brief TODO
 *\param entity
 *\param tracerProxy
 */
void TracerManager::unregisterEntityTracer(GenericEntity *entity, Hns::TracerQPrx tracerProxy)
{
    Q_D(TracerManager);
    if (!entity || !tracerProxy) {
        return;
    }
    if (d->entityTracerHash.contains(entity)) {
        tracerProxy.data()->disconnect(this);
        d->entityTracerHash.remove(entity);
    }
}

void TracerManager::_initTracer(Hns::TracerQPrx tracerProxy)
{
    connect(tracerProxy.data(), SIGNAL(newTrace(Hns::Trace)), this, SLOT(_newTraceSlot(Hns::Trace)));
}

void TracerManager::_entityDestroyed(QObject *entityObject)
{
    Q_D(TracerManager);
    Entity * entity = qobject_cast<Entity*>(entityObject);
    if (d->entityTracerHash.contains(entity))
        d->entityTracerHash.remove(entity);
}

void TracerManager::_topologyDestroyed(QObject *topologyObject)
{
    Q_D(TracerManager);
    Topology * topology = qobject_cast<Topology *>(topologyObject);
    if (d->topologyTracerHash.contains(topology))
        d->topologyTracerHash.remove(topology);
}

void TracerManager::_newTraceSlot(Hns::Trace trace)
{
    emit newTrace(QString("%1:%2").arg(QString::fromStdString(trace.userName)).arg(QString::fromStdString(trace.content)), QDateTime::fromMSecsSinceEpoch(trace.date));
}

} // namespace HnsClient
