/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef HNSCLIENT_TRACERMANAGER_H
#define HNSCLIENT_TRACERMANAGER_H

// Generated hynesim headers
#include "common/proxies_tracer.h"

// Qt headers
#include <QObject>
#include <QDateTime>

namespace HnsClient {

class GenericEntity;
class Topology;
class TracerManagerPrivate;

/*******************************************************************************
* TracerManager
*******************************************************************************/
class TracerManager : public QObject
{
    Q_OBJECT
    Q_DECLARE_PRIVATE(TracerManager)
public:
    explicit TracerManager(QObject *parent = 0);
    ~TracerManager();
    void registerMasterTracer(Hns::TracerQPrx tracerProxy);
    void registerEntityTracer(GenericEntity * entity, Hns::TracerQPrx tracerProxy);
    void registerTopologyTracer(Topology * topology, Hns::TracerQPrx tracerProxy);
    void unregisterTopologyTracer(Topology *topology, Hns::TracerQPrx tracerProxy);
    void unregisterEntityTracer(GenericEntity *entity, Hns::TracerQPrx tracerProxy);

protected slots:
    void _newTraceSlot(Hns::Trace trace);
    void _entityDestroyed(QObject *entityObject);
    void _topologyDestroyed(QObject *topologyObject);

signals:
    void newTrace(QString msg, QDateTime dateTime);

protected:
    void _initTracer(Hns::TracerQPrx tracerProxy);

private:
    TracerManagerPrivate *d_ptr;
};

} // namespace HnsClient

#endif //HNSCLIENT_TRACERMANAGER_H
