/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef HNSCLIENT_UTILITY_H
#define HNSCLIENT_UTILITY_H

// hynesim hnsclient headers
#include "libhnsclient/common.h"

// Generated hynesim headers
#include "common/types.h"
#include "common/topology.h"

// diamesh headers
#include <diamesh/types.h>

// Qt headers
#include <QString>
#include <QSet>
#include <QPixmap>
#include <QPair>
#include <QStringList>
#include <QList>

namespace HnsClient {

/*******************************************************************************
* Utility
*******************************************************************************/
class Utility
{
public:
    static QString unitFromIndex(BytesUnitEnum index, bool asRate = false, QString rate = "s");
    static QPair<QString, BytesUnitEnum> buildReadableSize(const qlonglong &size, BytesUnitEnum from, int baseUnit = 1024);
    static QString makeSizeReadable(const qlonglong &size, BytesUnitEnum from, int baseUnit = 1024);
    static QString makeTimeReadable(int seconds);
    static QString makePercentageReadable(double used, double total);
    static QString makePercentageReadable(double value);
    static QString makeMemoryLoadReadable(qlonglong used, qlonglong total);
    static QString readableStringValue(std::string value, QString defaultValue = QString());
    static QStringList qStringListFromStringList(Diamesh::StringList list);
    static QList<qlonglong> qLongListFromLongList(Diamesh::LongList list);
    static QList<qlonglong> qLongListFromVectorList(::Hns::IdImportedList list);
    static QSet<qlonglong> qLongSetFromLongList(Diamesh::LongList list);
    static QPixmap pixmapFromByteArray(Hns::ByteArray data);
};

} // namespace HnsClient

#endif // HNSCLIENT_UTILITY_H
