/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef PROCESSRUNNER_H
#define PROCESSRUNNER_H

// Qt headers
#include <QObject>
#include <QProcess>

class QThread;
class QMetaObject;

namespace HnsServer {

class ProcessRunner;
/**************************************************
 *                 ProcessWrapper                 *
 **************************************************/
class ProcessWrapper : public QObject
{
    Q_OBJECT

public:
    ProcessWrapper(ProcessRunner *processRunner, const QMetaObject *processMetaObj = 0);
    bool errorFound();
    QString getOutputBuffer();
    QString getErrorBuffer();

protected slots:
    void _parseError();
    void _parseOutput();
    void _error();
    void _finished(int exitCode, QProcess::ExitStatus exitStatus);
    void _cancel();

public slots:
    void start();

signals:
    void finished(int exitCode, QProcess::ExitStatus exitStatus);

private:
    ProcessRunner *_processRunner;
    QProcess *_process;
    bool _errorFound;
    QString _outputBuffer;
    QString _errorBuffer;
    const QMetaObject *_processMetaObj;
};

/**************************************************
 *                 ProcessRunner                  *
 **************************************************/
class ProcessRunner : public QObject
{
    Q_OBJECT
    friend class ProcessWrapper;

public:
    explicit ProcessRunner(QObject *parent = 0, const QMetaObject *processMetaObj = 0);
    virtual ~ProcessRunner();

    void runAsynchronous(const QString &program, const QStringList &arguments);
    bool runSynchronous(const QString &program, const QStringList &arguments, int timeout = 15000);
    qint64 runDetached(const QString &program, const QStringList &arguments);
    void cancel();
    int exitCode();
    QString getOutput();
    QString getError();

 protected slots:
    void _finished(int exitCode, QProcess::ExitStatus exitStatus);

signals:
    void finished();
    void errorRead(QString error);
    void outputRead(QString output);
    void cancelProcess();

protected:
    virtual void _beforeStart();
    void _start();
    qint64 _startDetached();

protected:
    QString _program;
    QStringList _arguments;
    bool _isRunning;
    int _exitCode;
    QThread *_executionThread;
    ProcessWrapper *_processWrapper;
};

} // namespace HnsServer

#endif // PROCESSRUNNER_H
