/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// hynesim master headers
#include "capturejobI.h"
#include "masterI.h"

// ZeroC Ice headers
#include <Ice/Current.h>

// Qt headers
#include <QDebug>

namespace Hns {

CaptureJobI::CaptureJobI()
{
}

CaptureJobI::~CaptureJobI()
{
    _nodeJobProxyQtice = 0;
}

bool CaptureJobI::cancel(const ::Ice::Current &current)
{
    Q_UNUSED(current)
    bool result = false;
    try {
        result = _nodeJobProxy->cancel();
    } catch(...) {
        qCritical() << tr("Exception caught while trying to cancel a job on a node");
    }

    return result;
}

void CaptureJobI::_run()
{
    if (_nodeJobProxy) {
        _nodeJobProxyQtice = Hns::JobQPrx(_nodeJobProxy, MasterI::instance()->adapter());

        QObject::connect(
                    _nodeJobProxyQtice.data(), SIGNAL(progressChanged(Ice::Int,Ice::Double)),
                    this, SLOT(_updateNodeProgress(Ice::Int,Ice::Double)));
        _nodeJobProxy->execute();
    }
}

bool CaptureJobI::setOptions(const ::Hns::CaptureJobOptions &jobOptions, const ::Ice::Current &current)
{
    Q_UNUSED(current)
    if (!_nodeJobProxy)
        return false;
    return _nodeJobProxy->setOptions(jobOptions);
}

PcapInfo CaptureJobI::getPcapInfo(const ::Ice::Current &current)
{
    Q_UNUSED(current)
    if (getProgress() >= 1.0) {
        QReadLocker locker(&_pcapInfoLock);
        return _pcapInfo;
    } else {
        return _nodeJobProxy->getPcapInfo();
    }
}

HnsNode::CaptureJobPrx CaptureJobI::getCaptureJobPrx()
{
    return _nodeJobProxy;
}

void CaptureJobI::setCaptureJobPrx(HnsNode::CaptureJobPrx captureJobPrx)
{
    _nodeJobProxy = captureJobPrx;
}

void CaptureJobI::_updateNodeProgress(Ice::Int jobId, Ice::Double progress)
{
    Q_UNUSED(jobId); // Not the same id on the node and master
    _updateProgress(progress);
    if (progress >= 1) {
        QWriteLocker locker(&_pcapInfoLock);
        _pcapInfo = _nodeJobProxy->getPcapInfo();
        _nodeJobProxy->destroy();
    }
}

} // namespace Hns
