/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef _HNS_CATALOG_H_
#define _HNS_CATALOG_H_

// Generated hynesim headers
#include "qiceobj_catalog.h"
#include "common/entityentry.h"

#include <QStringList>

namespace Hns {

class CatalogDBStorage;
class CatalogFileStorage;
class CatalogItemWrapper;

class CatalogI : public _Hns_Catalog
{
    Q_OBJECT
public:
    enum ElementType {
        UnknownType = 0,
        EntityType = 1,
        TopologyType = 2,
        FolderType = 3
    };

    enum CatalogAddEntryFlags {
        LockEntry = 0x1
    };

    ~CatalogI();
    CatalogPrx proxy();

    Diamesh::LongList getEntries(const ::Ice::Current &current = ::Ice::Current());
    EntryPtr getEntry(::Ice::Long entryId, const ::Ice::Current &current = ::Ice::Current());
    Ice::Long addEntry(const EntryPtr &entry, const ::Ice::Current &current = ::Ice::Current());
    bool deleteEntry(const EntryPtr &entry, const ::Ice::Current &current = ::Ice::Current());
    bool updateEntry(const EntryPtr &entry, const ::Ice::Current &current);

    bool lockEntry(Ice::Long entryId, LockType lockType = LOCK_TYPE_NORMAL);
    bool unlockEntry(Ice::Long entryId);
    bool entryIsLocked(Ice::Long entryId, const ::Ice::Current &current = ::Ice::Current());
    LockTypeLongMap getLockedEntries(const ::Ice::Current &current = ::Ice::Current());
    LockType getLockType(Ice::Long entryId);

    Diamesh::StringStringMap getFileStorages(const ::Ice::Current &current = ::Ice::Current());
    std::string getFullEntryPathFromMixed(const std::string &mixedPath, const std::string &uuid = std::string(), const ::Ice::Current &current = ::Ice::Current());

    Diamesh::ACLList getACLsForCatalogItem(Ice::Long catalogItemId, const ::Ice::Current &current = ::Ice::Current());
    bool setACLsForCatalogItem(Ice::Long catalogItemId, const Diamesh::ACLList &aclList, const ::Ice::Current &current = ::Ice::Current());

    bool refresh(const ::Ice::Current &current = ::Ice::Current());
    std::string getUnusedUUID(const Ice::Current &current= ::Ice::Current());

    virtual Ice::Long addEntry(const EntryPtr &entry, const Diamesh::UserPtr &currentUser = Diamesh::UserPtr(), int flags = 0);
    bool updateEntry(const EntryPtr &entry);
    bool updateEntryNoLock(const EntryPtr &entry);

    virtual bool copyEntryResources(const EntryPtr &fromEntry, const EntryPtr &toEntry);
    virtual EntryPtr getEntryByUuid(const QString &entryUuid);
    CatalogFileStorage *getDefaultFileStorage();
    QString generateUnusedUUID();
    const QString &getSettingName();
    const std::string &idName();
    CatalogItemPtr getCatalogItem(qlonglong id);
    CatalogItemPtr catalogItemByUuid(const QString &uuid);
    CatalogItemWrapper *catalogItem(qlonglong id);

protected slots:
    virtual void _aclAddedSlot(QString target, QString subject);
    virtual void _aclChangedSlot(QString target, QString subject);
    virtual void _aclRemovedSlot(QString target, QString subject);

protected:
    CatalogI(const Ice::CommunicatorPtr &communicator, Ice::ObjectAdapterPtr adapter, const QString &settingsFileName);

    bool _init();

    Ice::Long _addEntry(const EntryPtr &entry, const Diamesh::UserPtr &currentUser = Diamesh::UserPtr(), int flags = 0);

    bool _loadFileStorages();
    virtual bool _addFileStorage(const QString &storagePath, const QString &storageName);
    CatalogFileStorage *_getFileStorage(const QString &entryMixedPath);
    bool _removeFileStorage(const QString &storagePath);
    CatalogFileStorage *_getEntryFileStorage(const EntryPtr &entry);
    std::list<std::string> _getSubDirList(const QString &dirPath);

    qint64 _currentTime();
    void _updateModificationTime(const EntryPtr &entry);
    virtual void _handleUuidConflicts(const QMultiHash<QString, EntryPtr> &multiHashUuidEntryPtr) = 0;
    virtual void _markEntryAsInvalid(const EntryPtr &entry) = 0;
    void _handleEntryLoading(EntryPtr storageEntry, EntryPtr dbEntry);

    qlonglong _entryIdFromTarget(const QString &target);
    virtual void _checkACL(const QString &target);

    bool _hasEntry(qlonglong entryId);

    Ice::CommunicatorPtr _communicator;
    Ice::ObjectAdapterPtr _adapter;

    Ice::Identity _id;
    QString _storageType;

    QHash<qlonglong, CatalogItemWrapper *> _catalogItemHash;
    QHash< QString, CatalogFileStorage *> _fileStoragesHash;

    CatalogDBStorage *_dbStorage;

    QString _settingsFileName;

    QHash<Ice::Long, LockType> _lockedEntriesHash;

    QList<QString> _usedUuidList;

    QReadWriteLock _entriesLock;
    QReadWriteLock _fileStoragesLock;
    QReadWriteLock _lockedEntriesLock;
    QReadWriteLock _usedUuidLock;

    QMultiHash<QString, EntryPtr> _hashUuidEntry;

    QStringList _aclTargetTypes;
    QString _storageDocType;

    QList<int> _dbtypes;

};

} // namespace Hns

#endif // _HNS_CATALOG_H_
