/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef HNS_CATALOG_DB_STORAGE_H
#define HNS_CATALOG_DB_STORAGE_H

// Generated hynesim headers
#include <common/entry.h>
#include <common/folder.h>

// diamesh headers
#include <diamesh/types.h>

#include <QObject>
#include <QSqlRecord>
#include <QHash>

namespace Diamesh {
class SqlDatabase;
} // namespace Diamesh

namespace Hns {

class TagManagerI;
class CatalogItemWrapper;

class CatalogDBStorage : public QObject
{
    Q_OBJECT
    typedef CatalogItemWrapper* (CatalogDBStorage::*createCatalogItemMethod) (const QSqlRecord&);
    typedef qlonglong (CatalogDBStorage::*addCatalogItemMethod) (const CatalogItemPtr &);
public:
    CatalogDBStorage(const QString &settingsFileName, QObject *parent = 0);
    ~CatalogDBStorage();

    qlonglong addCatalogItem(const CatalogItemPtr &catalogItem);
    bool deleteEntry(const EntryPtr &entry);
    bool deleteFolder(const FolderPtr folder);
    bool updateEntry(const EntryPtr &entry, const EntryPtr &oldEntry = NULL);
    bool updateFolder(const FolderPtr &newFolder, const FolderPtr &oldFolder);

    QList<CatalogItemWrapper *> getCatalogItems(QList<int> types);

private:
    CatalogItemWrapper *_createEntityEntry(const QSqlRecord &record);
    CatalogItemWrapper *_createTopologyEntry(const QSqlRecord &record);
    CatalogItemWrapper *_createFolder(const QSqlRecord &record);

    qlonglong _addEntityEntry(const CatalogItemPtr &catalogItem);
    qlonglong _addTopologyEntry(const CatalogItemPtr &catalogItem);
    qlonglong _addFolder(const CatalogItemPtr &catalogItem);

    QHash<int, createCatalogItemMethod> _createCatalogItemHash;
    QHash<QString, addCatalogItemMethod> _addCatalogItemHash;

    Diamesh::SqlDatabase *_database;
    TagManagerI *_tagManager;

    QHash<qlonglong, QList<qlonglong> > _parentIdHash; //folderId, entryId
};

} // namespace Hns

#endif // HNS_CATALOG_DB_STORAGE_H
