/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef HNS_CATALOG_FILE_STORAGE_H
#define HNS_CATALOG_FILE_STORAGE_H

// Generated hynesim headers
#include "common/entry.h"
#include "common/layout.h"

// Qt headers
#include <QObject>
#include <QString>

class QDomDocument;

namespace Hns {

class CatalogFileStorage : public QObject
{
    Q_OBJECT

public:

    CatalogFileStorage(const QString &storagePath, const QString &storageName, const QString &docType, QObject *parent = 0);

    virtual qulonglong addEntry(const ::Hns::EntryPtr &entry);
    virtual bool deleteEntry(const ::Hns::EntryPtr &entry);
    virtual bool updateEntry(const ::Hns::EntryPtr &newEntry, const ::Hns::EntryPtr &oldEntry);

    virtual QList<EntryPtr> getEntries();
    QString getFullPathFromMixed(const QString &mixedPath, const QString &uuid);

    bool entryFilesAvailable(const EntryPtr &entry);
    bool createEntryDir(const QString &entryPath);
    bool deleteEntryDir(const QString &entryPath);
    bool createEntryFiles(const EntryPtr &entry);

    QString storageName();
    QString storagePath();

    TopologyLayout getTopologyLayout(const ::Hns::EntryPtr &entry);
    bool updateTopologyLayout(const ::Hns::EntryPtr &entry, TopologyLayout layout);

protected:
    QList<EntryPtr> _getEntries(const QString &path = "");

    bool _saveEntryConfig(const QDomDocument &doc, const QString &path);
    QString _getEntryConfigFileName(const ::Hns::EntryPtr &entry);
    QDomDocument _getEntryDomDocument(const ::Hns::EntryPtr &entry);

    QString _storagePath;
    QString _storageName;
    QString _docType;
};

} // namespace Hns

#endif // HNS_CATALOG_FILE_STORAGE_H
