#include "catalogitemwrapper.h"
#include "masterI.h"

#include <diamesh/aclmanagerI.h>
#include <diamesh/sessionManagerI.h>

#include <QHash>
#include <QDebug>

namespace Hns {

typedef QHash<QString, QMetaObject> StringMetaObjectHash;
Q_GLOBAL_STATIC(StringMetaObjectHash, _typeHash)

CatalogItemWrapper::CatalogItemWrapper(QObject *parent)
    : QObject(parent)
    , _catalogItem(0)
{
}

void CatalogItemWrapper::setCatalogItem(CatalogItemPtr catalogItem)
{
    _catalogItem = catalogItem;
}

CatalogItemPtr CatalogItemWrapper::catalogItem()
{
    return _catalogItem;
}

void CatalogItemWrapper::init()
{
}

bool CatalogItemWrapper::testACL(const QString &suffixRights, const Ice::Current &current, bool verbose)
{
    Diamesh::UserPtr currentUser = MasterI::instance()->sessionManager()->currentUser(current);
    if (currentUser && !MasterI::instance()->aclManager()->testRightForObject(catalogItem(), currentUser, _prefixForRights + suffixRights)) {
        if (verbose)
            qCritical() << tr("You (id %1) don't have the permission %2 on entry %3").arg(currentUser->id).arg(_prefixForRights + suffixRights).arg(catalogItem()->id);
        return false;
    }
    return true;
}

Ice::ObjectPtr CatalogItemWrapper::iceClone()
{
    return _catalogItem->ice_clone();
}

CatalogItemWrapperFactory::CatalogItemWrapperFactory(QObject *parent)
    : QObject(parent)
{

}

CatalogItemWrapper *CatalogItemWrapperFactory::createWrapper(const QString &type)
{
    if (!_typeHash()->contains(type)) {
        qCritical() << tr("Unable to handle this type of catalog item: %1").arg(type);
        return 0;
    }
    QMetaObject metaObj = _typeHash()->value(type);
    QObject *obj = metaObj.newInstance(Q_ARG(QObject *, 0));
    CatalogItemWrapper *catalogItemWrapper = qobject_cast<CatalogItemWrapper *>(obj);
    if (!catalogItemWrapper)
        qCritical() << tr("Unable to build a CatalogItemWrapper for this type: %1").arg(type);
    return catalogItemWrapper;
}

void CatalogItemWrapperFactory::addType(const QString &type, QMetaObject metaObj)
{
    _typeHash()->insert(type, metaObj);
}

} // namespace Hns
