#ifndef CATALOGITEMWRAPPER_H
#define CATALOGITEMWRAPPER_H

#include <common/catalogitem.h>

#include <QObject>

namespace Hns {

class CatalogDBStorage;
class CatalogFileStorage;

class CatalogItemWrapper : public QObject
{
    Q_OBJECT
public:
    explicit CatalogItemWrapper(QObject *parent = 0);
    void setCatalogItem(CatalogItemPtr catalogItem);
    CatalogItemPtr catalogItem();
    virtual void init();
    virtual bool update(CatalogDBStorage *db, CatalogFileStorage *fileStorage, CatalogItemPtr newItem) = 0;
    virtual bool create(CatalogDBStorage *db, CatalogFileStorage *fileStorage, CatalogItemPtr newItem, const Diamesh::UserPtr &currentUser) = 0;
    virtual bool remove(CatalogDBStorage *db, CatalogFileStorage *fileStorage) = 0;
    bool testACL(const QString &suffixRights, const Ice::Current &current, bool verbose = true);
    virtual ::Ice::ObjectPtr iceClone();
protected:
    QString _prefixForRights;
private:
    CatalogItemPtr _catalogItem;
};

class CatalogItemWrapperFactory : public QObject
{
    Q_OBJECT
public:
    CatalogItemWrapperFactory(QObject *parent = 0);
    static CatalogItemWrapper *createWrapper(const QString &type);
    static void addType(const QString &type, QMetaObject metaObj);
    template<typename T> static void registerCatalogItemWrapper(const QString &type) {
        addType(type, T::staticMetaObject);
    }
};

template<typename T>
struct RegisterCatalogItemWrapper {
    RegisterCatalogItemWrapper(const QString &type)
    {
        CatalogItemWrapperFactory::registerCatalogItemWrapper<T>(type);
    }
};


} // namespace Hns
#endif // CATALOGITEMWRAPPER_H
