/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef _HNS_ENTITY_CATALOG_H_
#define _HNS_ENTITY_CATALOG_H_

// Generated hynesim headers
#include "qiceobj_entitycatalog.h"
#include "common/entitycatalog.h"
#include "common/entityentry.h"
#include "common/folder.h"

// diamesh headers
#include <diamesh/types.h>

// Qt headers
#include <QHash>
#include <QMultiHash>
#include <QReadWriteLock>
#include <QString>

namespace Hns {

class EntityCatalogDBStorage;

class EntityCatalogI : public _Hns_EntityCatalog
{
    Q_OBJECT

public:
    EntityCatalogI(const Ice::CommunicatorPtr &communicator, Ice::ObjectAdapterPtr adapter, const QString &settingsFileName);
    ~EntityCatalogI();
    EntityCatalogPrx proxy();

    virtual EntityEntryPtr getEntityEntry(::Ice::Long entryId, const ::Ice::Current &current = ::Ice::Current());

    virtual Ice::Int cloneEntityEntry(::Ice::Long entryId, Ice::Int flags, const ::Ice::Current &current = ::Ice::Current());
    virtual Ice::Long buildModelFromEntityEntry(::Ice::Long entryId, const ::Ice::Current &current = ::Ice::Current());
    virtual bool discardSaveState(::Ice::Long entryId, const ::Ice::Current &current = ::Ice::Current());

    virtual LongIntMap getCloneJobs(const ::Ice::Current &current = ::Ice::Current());

    Diamesh::LongList getFolders(::Ice::Long parentId, const ::Ice::Current &current = ::Ice::Current());
    virtual FolderPtr getFolder(::Ice::Long folderId, const ::Ice::Current &current = ::Ice::Current());
    virtual ::Ice::Long addFolder(const FolderPtr &folder, const ::Ice::Current &current = ::Ice::Current());
    virtual bool deleteFolder(::Ice::Long folderId, const ::Ice::Current &current = ::Ice::Current());
    virtual bool updateFolder(const FolderPtr &folder, const ::Ice::Current &current = ::Ice::Current());

protected slots:
    void _cloneEntityProgressChangedSlot(Ice::Int jobId, Ice::Double progress);
    void _cloneEntityCancelled(Ice::Int);

protected:
    virtual void _handleUuidConflicts(const QMultiHash<QString, EntryPtr> &multiHashUuidEntryPtr);
    virtual void _markEntryAsInvalid(const EntryPtr &entry);

    virtual void _checkACL(const QString &target);

private:
    bool _testFolderRightRecursiverly(qlonglong folderId, const Ice::Current &current);

    int _makeCloneJob(qlonglong entryId, int flags, const ::Ice::Current &current = ::Ice::Current());
    void _startNextCloneJob();
    bool _isEntryInCloneBatch(qlonglong entryId);

protected:
    QHash<Ice::Long, FolderPtr> _foldersHash;
    QReadWriteLock _foldersLock;
    QHash< ::Ice::Long, int> _cloneJobsHash; // entryId, jobId
    QList<QPair<qlonglong, int> > _cloneBatch;
    QReadWriteLock _cloneJobsLock;
    int _maxCloneJobs;
};

} // namespace Hns

#endif // _HNS_ENTITY_CATALOG_H_
