/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// hynesim master headers
#include "entitycatalogdbstorage.h"
#include "tagmanagerI.h"
#include "catalog/catalogI.h"

// Generated hynesim headers
#include "common/entityentry.h"

// diamesh headers
#include <diamesh/sqldatabase.h>

// Qt headers
#include <QString>
#include <QVariant>
#include <QDebug>
#include <QSqlRecord>
#include <QSqlField>

namespace Hns {

EntityCatalogDBStorage::EntityCatalogDBStorage(const QString &settingsFileName)
    : CatalogDBStorage(settingsFileName)
{
}

EntityCatalogDBStorage::~EntityCatalogDBStorage()
{
}

bool EntityCatalogDBStorage::addFolderEntry(::Ice::Long folderId, ::Ice::Long entryId)
{
    QString query("SELECT id FROM join_entries_folders WHERE entryId=? AND folderId=?");
    QList<QVariant> values;
    values.append(qulonglong(entryId));
    values.append(qulonglong(folderId));

    QList<QSqlRecord> results;
    if (_database->executeQuery(query, values, &results) == -1) {
        qCritical() << tr("Error while getting the existing matching entry folder for entry %1 and folder %2.").arg(entryId).arg(folderId);
        return false;
    }

    if (results.size()) {
        qWarning() << tr("The entry %1 is already in the folder %2, not adding it again.").arg(entryId).arg(folderId);
        return true;
    } else {
        query = "INSERT INTO join_entries_folders (entryId, folderId) VALUES(?, ?)";
        if (_database->executeQuery(query, values) == 0) {
            qCritical() << tr("Error while inserting the entry %1 in the folder %2").arg(entryId).arg(folderId);
            return false;
        }
    }
    return true;
}

bool EntityCatalogDBStorage::removeFolderEntry(::Ice::Long folderId, ::Ice::Long entryId)
{
    QString query("DELETE FROM join_entries_folders WHERE entryId=? AND folderId=?");
    QList<QVariant> values;
    values.append(qulonglong(entryId));
    values.append(qulonglong(folderId));
    if (_database->executeQuery(query, values) == -1) {
        qCritical() << tr("The entry %1 could not be removed from the folder %2").arg(entryId).arg(folderId);
        return false;
    }
    return true;
}

QList<FolderPtr> EntityCatalogDBStorage::getFolders()
{
    QList<FolderPtr> folderList;
    QList<QSqlRecord> results;
    if (_database->executeQuery("SELECT * FROM elements WHERE type=?", CatalogI::FolderType, &results) == -1) {
        qCritical() << tr("Error while getting the existing database folders.");
        return folderList;
    }

    foreach (QSqlRecord rec, results) {
        FolderPtr folder = new Folder();
        folder->id = rec.value("id").toULongLong();
        folder->shortName = rec.value("shortName").toString().toStdString();
        folder->longName = rec.value("longName").toString().toStdString();
        folder->description = rec.value("description").toString().toStdString();
        folder->parentId = rec.value("parentId").toULongLong();
        folder->tagList = _elementTags(folder->id);
        folder->entryList = _folderEntryList(folder->id);
        folderList.append(folder);
    }
    return folderList;
}

Diamesh::LongList EntityCatalogDBStorage::_entryFolderList(qulonglong entryId)
{
    Diamesh::LongList folderList;
    QList<QSqlRecord> results;
    if (_database->executeQuery("SELECT folderId FROM join_entries_folders WHERE entryId=?", qulonglong(entryId), &results) == -1) {
        qCritical() << tr("Error while getting the database element tags.");
        return folderList;
    }

    foreach (QSqlRecord rec, results)
        folderList.push_back(rec.value("folderId").toInt());

    return folderList;
}

} // namespace Hns
