#ifndef ENTITYENTRYWRAPPER_H
#define ENTITYENTRYWRAPPER_H

#include "entrywrapper.h"

#include <QObject>
#include <QReadWriteLock>

namespace Hns {

class EntityEntryWrapper : public EntryWrapper
{
    Q_OBJECT
public:
    Q_INVOKABLE explicit EntityEntryWrapper(QObject *parent = 0);
    virtual bool create(CatalogDBStorage *db, CatalogFileStorage *fileStorage, CatalogItemPtr newItem, const Diamesh::UserPtr &currentUser);
};

class DomainEntryWrapper : public EntityEntryWrapper
{
    Q_OBJECT
public:
    Q_INVOKABLE explicit DomainEntryWrapper(QObject *parent = 0);
    virtual void init();
    virtual bool update(CatalogDBStorage *db, CatalogFileStorage *fileStorage, CatalogItemPtr newItem);
    virtual bool remove(CatalogDBStorage *db, CatalogFileStorage *fileStorage);
    virtual void addDomainDiffUuids(QString uuid);
    virtual ::Ice::ObjectPtr iceClone();

private:
    QReadWriteLock _diffUuidsLock;
};

class DynamipsEntryWrapper : public EntityEntryWrapper
{
    Q_OBJECT
public:
    Q_INVOKABLE explicit DynamipsEntryWrapper(QObject *parent = 0);
};

class HybridNetcardEntryWrapper : public EntityEntryWrapper
{
    Q_OBJECT
public:
    Q_INVOKABLE explicit HybridNetcardEntryWrapper(QObject *parent = 0);
};

class SwitchEntryWrapper : public EntityEntryWrapper
{
    Q_OBJECT
public:
    Q_INVOKABLE explicit SwitchEntryWrapper(QObject *parent = 0);
};

class TopologyGateEntryWrapper : public EntityEntryWrapper
{
    Q_OBJECT
public:
    Q_INVOKABLE explicit TopologyGateEntryWrapper(QObject *parent = 0);
};

class WifiAPEntryWrapper : public EntityEntryWrapper
{
    Q_OBJECT
public:
    Q_INVOKABLE explicit WifiAPEntryWrapper(QObject *parent = 0);
};

class WireEntryWrapper : public EntityEntryWrapper
{
    Q_OBJECT
public:
    Q_INVOKABLE explicit WireEntryWrapper(QObject *parent = 0);
};

} // namespace Hns

#endif // ENTITYENTRYWRAPPER_H
