#include "entrywrapper.h"
#include "catalogdbstorage.h"
#include "catalogfilestorage.h"
#include "masterI.h"

#include <common/entry.h>

#include <diamesh/accessrights.h>
#include <diamesh/accessrightshelper.h>
#include <diamesh/aclmanagerI.h>
#include <diamesh/hsetmanagerI.h>
#include <diamesh/usermanager.h>

#include <QDebug>
#include <QStringList>

namespace Hns {

static RegisterCatalogItemWrapper<EntryWrapper> registerMe(QString::fromStdString(Entry::ice_staticId()));

EntryWrapper::EntryWrapper(QObject *parent) :
    CatalogItemWrapper(parent)
{
}

bool EntryWrapper::update(CatalogDBStorage *db, CatalogFileStorage *fileStorage, CatalogItemPtr newItem)
{
    if (!db->updateEntry(EntryPtr::dynamicCast(newItem), EntryPtr::dynamicCast(catalogItem()))) {
        qCritical() << tr("Unable to update entry %1 in database").arg(catalogItem()->id);
        return false;
    }
    if (fileStorage && !fileStorage->updateEntry(EntryPtr::dynamicCast(newItem), EntryPtr::dynamicCast(catalogItem()))) {
        qCritical() << tr("Unable to update entry %1 in catalog file storage");
        return false;
    }
    setCatalogItem(newItem);

    return true;
}

bool EntryWrapper::create(CatalogDBStorage *db, CatalogFileStorage *fileStorage, CatalogItemPtr newItem, const Diamesh::UserPtr &currentUser)
{
    Q_UNUSED(fileStorage)
    EntryPtr entry = EntryPtr::dynamicCast(newItem);
    if (entry->uuid == "") {
        qCritical() << tr("Unable to add entry, no UUID is set");
        return 0;
    }
    if (currentUser) {
        if (entry->hSetId == -1 && currentUser->hSetId != -1) {
            entry->hSetId = currentUser->hSetId;
        }
        if (entry->hSetId == -1 && currentUser->hSetId == -1) {
            entry->hSetId = MasterI::instance()->hSetManager()->highestHSetId();
        }
    } else {
        if (entry->hSetId == -1) {
            entry->hSetId = MasterI::instance()->hSetManager()->highestHSetId();
        }
    }
    qlonglong entryId = db->addCatalogItem(entry);
    if (!entryId) {
        qCritical() << tr("Unable to add entry in database");
        return false;
    }
    entry->id = entryId;
    if (currentUser) {
        std::string userSubject = QString("u.%1").arg(currentUser->id).toStdString();
        QStringList rightsList(_prefixForRights);
        MasterI::instance()->aclManager()->initACL(
                    MasterI::instance()->aclManager()->accessRightsHelper()->targetFromObject(entry).toStdString(),
                    userSubject,
                    MasterI::instance()->aclManager()->accessRightsHelper()->rightsFromNames(rightsList),
                    currentUser->hSetId, currentUser);
    }
    setCatalogItem(entry);
    return true;
}

bool EntryWrapper::remove(CatalogDBStorage *db, CatalogFileStorage *fileStorage)
{
    EntryPtr entry = EntryPtr::dynamicCast(catalogItem());
    if (!db->deleteEntry(entry)) {
        qCritical() << tr("Unable to delete entry %1 from database").arg(entry->id);
        return false;
    }
    if (entry->path != "" && !fileStorage->deleteEntry(entry)) {
        qCritical() << tr("Unable to delete entry %1 from filesystem").arg(entry->id);
        return false;
    }
    return true;
}

} // namespace Hns
