#include "folderwrapper.h"
#include "catalogdbstorage.h"
#include "masterI.h"

#include <common/folder.h>
#include <common/rightslist.h>

#include <diamesh/accessrights.h>
#include <diamesh/accessrightshelper.h>
#include <diamesh/aclmanagerI.h>
#include <diamesh/hsetmanagerI.h>
#include <diamesh/usermanager.h>

#include <QDebug>

namespace Hns {

static RegisterCatalogItemWrapper<FolderWrapper> registerMe(QString::fromStdString(Folder::ice_staticId()));

FolderWrapper::FolderWrapper(QObject *parent) :
    CatalogItemWrapper(parent)
{
    _prefixForRights = Rights::FOLDER_PREFIX;
}

bool FolderWrapper::update(CatalogDBStorage *db, CatalogFileStorage *fileStorage, CatalogItemPtr newItem)
{
    Q_UNUSED(fileStorage)
    if (!db->updateFolder(FolderPtr::dynamicCast(newItem), FolderPtr::dynamicCast(catalogItem()))) {
        qCritical() << tr("Unable to update folder %1 in database").arg(catalogItem()->id);
        return false;
    }
    setCatalogItem(newItem);
    return true;
}

bool FolderWrapper::create(CatalogDBStorage *db, CatalogFileStorage *fileStorage, CatalogItemPtr newItem, const Diamesh::UserPtr &currentUser)
{
    Q_UNUSED(fileStorage)
    FolderPtr folder = FolderPtr::dynamicCast(newItem);
    if (currentUser) {
        if (folder->hSetId == -1 && currentUser->hSetId != -1)
            folder->hSetId = currentUser->hSetId;
        if (folder->hSetId == -1 && currentUser->hSetId == -1)
            folder->hSetId = MasterI::instance()->hSetManager()->highestHSetId();
    } else {
        if (folder->hSetId == -1)
            folder->hSetId = MasterI::instance()->hSetManager()->highestHSetId();
    }
    qlonglong folderId = db->addCatalogItem(folder);
    if (!folderId) {
        qCritical() << tr("Unable to add folder in database");
        return false;
    }
    folder->id = folderId;
    if (currentUser) {
        std::string userSubject = QString("u.%1").arg(currentUser->id).toStdString();
        QStringList rightsList(_prefixForRights);
        MasterI::instance()->aclManager()->initACL(
                    MasterI::instance()->aclManager()->accessRightsHelper()->targetFromObject(folder).toStdString(),
                    userSubject,
                    MasterI::instance()->aclManager()->accessRightsHelper()->rightsFromNames(rightsList),
                    currentUser->hSetId, currentUser);
    }
    setCatalogItem(folder);
    return true;
}

bool FolderWrapper::remove(CatalogDBStorage *db, CatalogFileStorage *fileStorage)
{
    Q_UNUSED(fileStorage)
    FolderPtr folder = FolderPtr::dynamicCast(catalogItem());
    if (!db->deleteFolder(folder)) {
        qCritical() << tr("Unable to delete folder %1 from database").arg(folder->id);
        return false;
    }
    return true;
}

} // namespace Hns
