/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef _HNS_TOPOLOGY_CATALOG_H_
#define _HNS_TOPOLOGY_CATALOG_H_

// Generated hynesim headers
#include "qiceobj_topologycatalog.h"
#include "common/topologyentry.h"
#include "common/layout.h"
#include "common/types.h"

// Qt headers
#include <QReadWriteLock>

namespace Hns {

class TopologyCatalogI : public _Hns_TopologyCatalog
{
Q_OBJECT

public:
    TopologyCatalogI(const Ice::CommunicatorPtr &communicator, Ice::ObjectAdapterPtr adapter, const QString &settingsFileName);
    ~TopologyCatalogI();
    TopologyCatalogPrx proxy();

    TopologyEntryPtr getTopologyEntry(Ice::Long entryId, const ::Ice::Current &current = ::Ice::Current());
    bool fixTopologyEntry(const TopologyEntryPtr &topologyEntry, const ::Ice::Current &current = ::Ice::Current());
    Hns::ByteArray getScreenshot(Ice::Long entryId, const ::Ice::Current &current = ::Ice::Current());
    bool setScreenshot(Ice::Long entryId, const Hns::ByteArray &data, const ::Ice::Current &current = ::Ice::Current());

    bool copyEntryResources(const EntryPtr &fromEntry, const EntryPtr &toEntry, bool createStorage = false);
    TopologyLayout getTopologyLayout(const EntryPtr &entry);
    bool updateTopologyLayout(const EntryPtr entry, TopologyLayout layout);

    Ice::Int cloneTopology(Ice::Long topologyId, Ice::Int flags, const std::string &newName, const Ice::Current &current = ::Ice::Current());

private slots:
    void _entityEntryChangedSlot(Ice::Long entityEntryId);
    void _cloneJobCancelledSlot(Ice::Int jobId);
    void _cloneJobFinishedSlot(Ice::Int jobId);

protected:
    void _handleUuidConflicts(const QMultiHash<QString, EntryPtr> &multiHashUuidEntryPtr);
    virtual void _markEntryAsInvalid(const EntryPtr &entry);

private:
    QReadWriteLock _screenshotLock;
};

} // namespace Hns

#endif // _HNS_TOPOLOGY_CATALOG_H_
