#include "topologyentrywrapper.h"
#include "catalogfilestorage.h"

#include <common/topologyentry.h>
#include <common/rightslist.h>

#include <QDebug>

namespace Hns {

static RegisterCatalogItemWrapper<TopologyEntryWrapper> registerMe(QString::fromStdString(TopologyEntry::ice_staticId()));

TopologyEntryWrapper::TopologyEntryWrapper(QObject *parent)
    : EntryWrapper(parent)
{
    _prefixForRights = Rights::TOPOLOGY_ENTRY_PREFIX;
}

bool TopologyEntryWrapper::update(CatalogDBStorage *db, CatalogFileStorage *fileStorage, CatalogItemPtr newItem)
{
    EntryPtr currentEntry = EntryPtr::dynamicCast(catalogItem());

    TopologyLayout topologyLayout;
    if (fileStorage)
        topologyLayout = fileStorage->getTopologyLayout(currentEntry);

    if (!EntryWrapper::update(db, fileStorage, newItem))
        return false;
    if (fileStorage) {
        return fileStorage->updateTopologyLayout(EntryPtr::dynamicCast(newItem), topologyLayout);
    } else {
        qWarning() << tr("No filestorage, unable to add entry");
        return false;
    }
}

bool TopologyEntryWrapper::create(CatalogDBStorage *db, CatalogFileStorage *fileStorage, CatalogItemPtr newItem, const Diamesh::UserPtr &currentUser)
{
    if (!EntryWrapper::create(db, fileStorage, newItem, currentUser)) {
        return false;
    }
    if (!fileStorage) {
        qCritical() << tr("No filestorage, unable to add entry");
        return false;
    }
    TopologyEntryPtr entry = TopologyEntryPtr::dynamicCast(newItem);
    QString entryMixedPath;
    if (entry->path == "") {
        entryMixedPath = QString("%1:%2").arg(fileStorage->storageName()).arg("/.tmp/");
        entry->path = entryMixedPath.toStdString();
    } else {
        entryMixedPath = QString::fromStdString(entry->path);
    }
    QString entryPath = fileStorage->getFullPathFromMixed(entryMixedPath, QString::fromStdString(entry->uuid));
    if (!fileStorage->createEntryDir(entryPath)) {
        qCritical() << tr("Error while creating the topology directory");
        return false;
    }
    return true;
}

bool TopologyEntryWrapper::remove(CatalogDBStorage *db, CatalogFileStorage *fileStorage)
{
    TopologyEntryPtr entry =TopologyEntryPtr::dynamicCast(catalogItem());
    if (entry->status == TOPOLOGY_BUSY_EXPORTING) {
        qCritical() << tr("Topology %1 with UUID %2 is busy (export). Unable to delete it.")
                       .arg(QString::fromStdString(entry->shortName))
                       .arg(QString::fromStdString(entry->uuid));
        return false;
    }
    if (!EntryWrapper::remove(db, fileStorage))
        return false;

    if (entry->path == "") {
        QString entryMixedPath = QString("%1:%2").arg(fileStorage->storageName()).arg("/.tmp/");
        QString entryPath = fileStorage->getFullPathFromMixed(entryMixedPath, QString::fromStdString(entry->uuid));
        if (!fileStorage->deleteEntryDir(entryPath)) {
            qCritical() << tr("Error while deleting the topology directory");
            return false;
        }
    }

    return true;
}

} // namespace Hns
