/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef HNS_CLONE_JOB_DOMAIN_H
#define HNS_CLONE_JOB_DOMAIN_H

// hynesim master headers
#include "clonejobimplementation.h"

// Generated hynesim headers
#include "common/entityentry.h"
#include "common/domainentry.h"

// Qt headers
#include <QStringList>

class QTimer;

namespace Hns {

struct CloneJobDomainSize {
    double percent;
    qlonglong totalSize;
};

class CloneJobDomain : public CloneJobImplementation
{
    Q_OBJECT

public:
    CloneJobDomain(Hns::EntityEntryPtr originalEntry, Hns::EntityEntryPtr clonedEntry, int flags);
    virtual ~CloneJobDomain();

protected slots:
    virtual void _run();

protected:
    virtual bool _copyDisks(const Hns::DomainEntryPtr &domainEntry);
    bool _copyDisk(const QString &disk);
    virtual bool _createDiffDisk(const QString &disk);
    bool _copyFileSystems();
    bool _copyFileSystem(const QString &fileSystem);
    bool _copyAdditionalFiles();
    virtual void _updateDisks();

    QStringList fileSystems() const;
    void addFileSystem(const QString &fileSystem);
    QStringList additionalFiles() const;
    void addAdditionalFile(const QString &additionalFile);

    virtual double getGeneralProgress();

private slots:
    void _timeoutSlot();

private:
    void _initStoragePercent();
    QStringList _fileSystems;
    QStringList _additionalFiles;
    QTimer *_timer;
    int _currentStorage;
    QList<CloneJobDomainSize> _storageInformation;
};

} // namespace Hns

#endif // HNS_CLONE_JOB_H
