/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// hynesim master headers
#include "clonejobdynamipsrouter.h"
#include "masterI.h"

// hynesim server headers
#include "libhnsserver/processrunner.h"

// Generated hynesim headers
#include "common/dynamipsrouterentry.h"
#include "common/macaddressmanager.h"

// Qt headers
#include <QDebug>
#include <QFileInfo>
#include <QDir>
#include <QTimer>
#include <QDirIterator>

namespace Hns {

static RegisterCloneImplementation<CloneJobDynamipsRouter> registerMe(&CloneJobDynamipsRouter::canClone);

CloneJobDynamipsRouter::CloneJobDynamipsRouter(Hns::EntityEntryPtr originalEntry, Hns::EntityEntryPtr clonedEntry, int flags)
    : CloneJobImplementation(originalEntry,clonedEntry, flags)
    , _timer(NULL)
    , _originalFileSize(0)
{
}

CloneJobDynamipsRouter::~CloneJobDynamipsRouter()
{
    if(!_timer)
        delete _timer;
}

bool CloneJobDynamipsRouter::canClone(const EntityEntryPtr &entityEntry)
{
    if (Hns::DynamipsRouterEntryPtr::dynamicCast(entityEntry))
        return true;
    return false;
}

void CloneJobDynamipsRouter::_run()
{
    qDebug() << QString(tr("CloneJob::run - clone %1 to %2")).arg(_originalEntry->id).arg(_clonedEntry->id);
    Hns::DynamipsRouterEntryPtr dynamipsRouterEntry = Hns::DynamipsRouterEntryPtr::dynamicCast(_clonedEntry);
    dynamipsRouterEntry->macAddress = Hns::MasterI::instance()->getMacAddressManager()->generateRandomAddress("Cisco Systems, Inc."); //TODO: JUST FOR COMPILATION
    _copyDisksAndDatas(dynamipsRouterEntry);
}

void CloneJobDynamipsRouter::_copyDisksAndDatas(const Hns::DynamipsRouterEntryPtr &dynamipsRouterEntry)
{
    QString disk = QString::fromStdString(dynamipsRouterEntry->imageDisk);
    _originalSource = _fullSourcePath("") ;
    _clonedSource = _fullDestinationPath("");

    QFileInfo clonedFileInfo(_clonedSource);

    QDir cloneDir = clonedFileInfo.absoluteDir();
    if(!cloneDir.exists()) {
        QString dirName = cloneDir.dirName();
        cloneDir.cdUp();
        cloneDir.mkdir(dirName);
    }

    QStringList  listFilter;
    listFilter << QString("*%1*").arg(QString::fromStdString(_originalEntry->uuid));
    listFilter << disk;

    _listFilterCopie << disk;
    _listFilterCopie << QString("*%1*").arg(QString::fromStdString(_clonedEntry->uuid));

    QDirIterator dirIterator(_originalSource, listFilter ,QDir::Files | QDir::NoSymLinks);

    QStringList fileList;
    while(dirIterator.hasNext()){
        dirIterator.next();
        fileList << dirIterator.fileName();
        _originalFileSize+= dirIterator.fileInfo().size();
    }
    if(!_timer) _timer = new QTimer(this);
    QObject::connect(_timer, SIGNAL(timeout()), this, SLOT(_updateCloneProgress()));
    _timer->start(500);

    foreach (const QString &value, fileList) {
        QString clonedFile(value);
        clonedFile.replace(QString::fromStdString(_originalEntry->uuid) , QString::fromStdString(_clonedEntry->uuid) ) ;
        HnsServer::ProcessRunner process;
        QString program = "cp";
        QStringList arguments;
        arguments << _originalSource+value << _clonedSource+clonedFile;
        process.runSynchronous(program, arguments, -1);
    }
}

void CloneJobDynamipsRouter::_updateCloneProgress()
{
    long clonedFileSize = 0;
    QDirIterator dirIterator(_clonedSource, _listFilterCopie ,QDir::Files | QDir::NoSymLinks);

    while(dirIterator.hasNext()){
        dirIterator.next();
        clonedFileSize += dirIterator.fileInfo().size();
    }

    double progress = clonedFileSize / (double) _originalFileSize;
    _updateProgress(progress);
    if (progress == 1)
        _timer->stop();
}

} // namespace Hns
