/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// hynesim master headers
#include "clonejobimplementation.h"
#include "masterI.h"
#include "scriptrunner.h"

// hynesim server headers
#include "libhnsserver/storagemanager.h"

// hynesim common headers
#include "common/clonejobflags.h"

// Qt project headers
#include <QDebug>

typedef QList<Hns::CloneImplementationHelper *> HelperList;
Q_GLOBAL_STATIC(HelperList, _helperList)

namespace Hns{

CloneJobImplementation::CloneJobImplementation(Hns::EntityEntryPtr originalEntry, Hns::EntityEntryPtr clonedEntry, int flags)
    : JobI()
    , _originalEntry(originalEntry)
    , _clonedEntry(clonedEntry)
    , _flags(flags)
{
}

CloneJobImplementation::~CloneJobImplementation()
{
}

Hns::EntityEntryPtr CloneJobImplementation::getEntityEntry()
{
    return _clonedEntry;
}

CloneJobImplementation *CloneJobImplementation::createImplementation(EntityEntryPtr originalEntry, EntityEntryPtr clonedEntry, int flags)
{
    for (int i = 0, size = _helperList()->size() ; i < size ; ++i) {
        CloneImplementationHelper *helper = _helperList()->at(i);
        if (helper->canCloneChecker(originalEntry)) {
            QObject *metaObj = helper->metaObject.newInstance(Q_ARG(Hns::EntityEntryPtr,originalEntry), Q_ARG(Hns::EntityEntryPtr, clonedEntry), Q_ARG(int, flags));
            CloneJobImplementation *impl = qobject_cast<CloneJobImplementation *>(metaObj);
            if (!impl)
                qCritical() << tr("Failed to create %1").arg(helper->metaObject.className());
            return impl;
        }
    }
    qCritical() << tr("Failed to find a clone job implementation for entry %1").arg(originalEntry->id);
    return 0;
}

void CloneJobImplementation::_run()
{
}

QString CloneJobImplementation::_fullPath(const Hns::EntityEntryPtr &entry, const QString &fileName, const QString &type)
{
    HnsServer::StorageManager *storageManager = MasterI::instance()->storageManager();
    QString fullPath = storageManager->getFullPathFromMixed(type, QString::fromStdString(entry->path), QString::fromStdString(entry->uuid));
    if (fullPath.isEmpty())
        return "";
    return fullPath.append("/" + fileName);
}

int CloneJobImplementation::_getDirCount(const QString &dir)
{
    ScriptRunner runner;
    QString script = "dirCount.sh";
    QStringList arguments = QStringList() << dir;
    bool success = runner.runSynchronous(script, arguments, -1);
    if (success)
        return runner.getOutput().toInt();
    else
        return -1;
}

QString CloneJobImplementation::_fullSourcePath(const QString &fileName)
{
    QString type = "entity";
    return _fullPath(_originalEntry, fileName, type);
}

QString CloneJobImplementation::_fullDestinationPath(const QString &fileName)
{
    return _fullPath(_clonedEntry, fileName, "entity");
}

void CloneJobImplementation::addImplementation(CloneImplementationHelper *helper)
{
    _helperList()->append(helper);
}

} // namespace Hns
