/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef CLONEJOBIMPLEMENTATION_H
#define CLONEJOBIMPLEMENTATION_H

// Generated hynesim headers
#include "common/entityentry.h"
#include "common/jobI.h"

namespace Hns {

typedef bool (*CanCloneChecker) (const Hns::EntityEntryPtr &);

struct CloneImplementationHelper
{
    QMetaObject metaObject;
    CanCloneChecker canCloneChecker;
};

class CloneJobImplementation : public JobI
{
    Q_OBJECT

public:
    CloneJobImplementation(Hns::EntityEntryPtr originalEntry, Hns::EntityEntryPtr clonedEntry, int flags);
    virtual ~CloneJobImplementation();
    virtual Hns::EntityEntryPtr getEntityEntry();
    static CloneJobImplementation *createImplementation(EntityEntryPtr originalEntry, EntityEntryPtr clonedEntry, int flags);

    static void addImplementation(CloneImplementationHelper *helper);
    template<typename T> static void registerImplementation(CanCloneChecker checker) {
        CloneImplementationHelper *helper = new CloneImplementationHelper();
        helper->canCloneChecker = checker;
        helper->metaObject = T::staticMetaObject;
        addImplementation(helper);
    }

protected slots:
    virtual void _run();

protected:
    QString _fullPath(const Hns::EntityEntryPtr &entry, const QString &fileName, const QString &type);
    int _getDirCount(const QString &dir);
    QString _fullSourcePath(const QString &fileName);
    QString _fullDestinationPath(const QString &fileName);

    Hns::EntityEntryPtr _originalEntry;
    Hns::EntityEntryPtr _clonedEntry;

    int _flags;
};

template<typename T>
struct RegisterCloneImplementation {
    RegisterCloneImplementation(CanCloneChecker checker)
    {
        CloneJobImplementation::registerImplementation<T>(checker);
    }
};

} // namespace Hns

#endif // CLONEJOBIMPLEMENTATION_H
