/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// hynesim master headers
#include "clonejobkvm.h"

// hynesim server headers
#include "libhnsserver/processrunner.h"

// hynesim common headers
#include "common/clonejobflags.h"

#include <QFileInfo>
#include <QDir>
#include <QDebug>

namespace Hns {

static RegisterCloneImplementation<CloneJobKVM> registerMe(&CloneJobKVM::canClone);

CloneJobKVM::CloneJobKVM(Hns::EntityEntryPtr originalEntry, Hns::EntityEntryPtr clonedEntry, int flags)
    : CloneJobDomain(originalEntry, clonedEntry, flags)
{
    DomainEntryPtr domain = DomainEntryPtr::dynamicCast(originalEntry);
    addAdditionalFile(QString::fromStdString(domain->os.kernel));
    addAdditionalFile(QString::fromStdString(domain->os.initrd));
}

CloneJobKVM::~CloneJobKVM()
{
}

bool CloneJobKVM::canClone(const EntityEntryPtr &entityEntry)
{
    Hns::DomainEntryPtr domain = Hns::DomainEntryPtr::dynamicCast(entityEntry);
    if (domain && (domain->type == "kvm" || domain->type == "qemu"))
        return true;
    return false;
}

bool CloneJobKVM::_createDiffDisk(const QString &disk)
{
    QString source = _fullSourcePath(disk);
    QString destination = _fullDestinationPath(disk);

    QFileInfo destinationFileInfo(destination);
    QDir destinationDirUp(destinationFileInfo.absolutePath());

    if(!destinationDirUp.exists())
        destinationDirUp.mkpath(".");

    HnsServer::ProcessRunner process;
    QStringList arguments = QStringList() << "create" << "-b" << source << "-f" << "qcow2" << destination;
    if (!process.runSynchronous("qemu-img", arguments, -1)){
        qDebug() << tr("Unable to create a diff disk, qemu-img has failed");
        qDebug() << tr("qemu-img stdout is %1").arg(process.getOutput());
        qDebug() << tr("qemu-img stderr is %1").arg(process.getError());
        return false;
    }
    return true;
}

void CloneJobKVM::_updateDisks()
{
    if (_flags & LINKED_CLONE) {
        QList<Hns::Disk> disks;
        DomainEntryPtr domain = DomainEntryPtr::dynamicCast(_clonedEntry);
        foreach (Hns::Disk disk, domain->disks) {
            disk.driverType = "qcow2";
            disks.push_back(disk);
        }
        domain->disks = disks.toStdList();
    }
}

} // namespace Hns
