/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef _HNS_CUSTOM_RESOURCE_MANAGER_H_
#define _HNS_CUSTOM_RESOURCE_MANAGER_H_

// Generated hynesim headers
#include "qiceobj_customresourcemanager.h"
#include "common/customresourcemanager.h"
#include "common/types.h"

// diamesh headers
#include <diamesh/types.h>

// Qt headers
#include <QString>

namespace Hns {

class CustomResourceManagerI : public _Hns_CustomResourceManager
{
    Q_OBJECT
public:
    CustomResourceManagerI(const QString &settingsFileName);
    ~CustomResourceManagerI();

    CustomResourceManagerPrx proxy() const;

    virtual ::Diamesh::StringList getResourceKeys(const ::Ice::Current &current = ::Ice::Current());
    virtual ::Hns::ByteArray getResource(const std::string &key, const ::Ice::Current &current = ::Ice::Current());
    virtual bool uploadResource(const std::string &key, const ::Hns::ByteArray &data, const ::Ice::Current &current = ::Ice::Current());
    virtual bool deleteResource(const std::string &key, const Ice::Current & = ::Ice::Current());
    QString getResourcePath(const QString &key);
    QString generateNewKey(const QString &key);
    const QString& getResourceDirectory();
    void addResource(const QString &key);

protected:
    void _init();

private:
    Ice::Identity _id;
    QList<std::string> _resources;
    QString _resourceDirectoryPath;
    QReadWriteLock _resourcesLock;

    friend class MasterI;
};

} // namespace Hns

#endif // _HNS_CUSTOM_RESOURCE_MANAGER_H_
