/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef _HNS_DOMAIN_H
#define _HNS_DOMAIN_H

// Generated hynesim headers
#include "qiceobj_domain.h"

// hynesim common headers
#include "common/snapshot.h"

// Qt headers
#include <QMap>

namespace Hns {

class DomainI : public _Hns_Domain
{
    Q_OBJECT

public:
    Q_INVOKABLE DomainI();
    ~DomainI();

    MemoryDumpInfoList getMemoryDumpInfoList(const Ice::Current &current = ::Ice::Current());
    int createMemoryDumpJob(const std::string &name, const Ice::Current &current = ::Ice::Current());
    void deleteMemoryDump(const std::string &fileName, const Ice::Current &current = ::Ice::Current());
    void setMemoryDumpInfoList(const MemoryDumpInfoList &memoryDumpInfoList, const Ice::Current &current = ::Ice::Current());
    Diamesh::FileTransferPrx downloadMemoryDump(const std::string &filename, const Ice::Current &current = ::Ice::Current());

signals:
    void memoryDumpJobStarted();

protected:
    virtual bool _createSnapshot(const Hns::Snapshot &snapshot);
    virtual bool _revertSnapshot(const std::string &snapshotName);
    virtual bool _deleteSnapshot(const std::string &snapshotName);
    virtual Hns::Snapshot _getSnapshot(const std::string &snapshotName);
    virtual std::string _getCurrentSnapshotName();
    QMap<Ice::Long, Hns::Plug> _getPlugs();
    virtual void _initSnapshotFlags();
    virtual void _initStatusesList();

private slots:
    void _memoryDumpJobProgressChangedSlot(Ice::Int jobId, Ice::Double progress);
    void _memoryDumpJobCancelledSlot(Ice::Int jobId);

private:
    QReadWriteLock _memoryDumpLock;
    QHash<Ice::Int, MemoryDumpInfo> _memoryDumpInfoHash;
    bool _availableForMemoryDump;
};

} // namespace Hns

#endif // _HNS_DOMAIN_H
