/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// hynesim master headers
#include "dynamipsrouterI.h"

// Generated hynesim headers
#include "common/dynamipsrouterentry.h"
#include "common/proxies_nodedynamipsrouter.h"

// diamesh headers
#include <diamesh/utils.h>

// ZeroC Ice headers
#include <Ice/Current.h>

namespace Hns {

DynamipsRouterI::DynamipsRouterI()
{
    Diamesh::setFlags(Hns::NET_CAPTURE | Hns::REMOTE_DISPLAY, _flags);
}

DynamipsRouterI::~DynamipsRouterI()
{
}

::std::string DynamipsRouterI::fixIdle(const ::Ice::Current &current)
{
    DynamipsRouterEntryPtr dynamipsRouterEntry = DynamipsRouterEntryPtr::dynamicCast(_entityEntry);

    HnsNode::DynamipsRouterPrx dynamipsRouterPrx = HnsNode::DynamipsRouterPrx::uncheckedCast(_entityProxy);
    if (dynamipsRouterPrx) {
        dynamipsRouterEntry->idlePc = dynamipsRouterPrx->fixIdle();
        updateEntry(_entityEntry,current);
    }
    return dynamipsRouterEntry->idlePc;
}

QMap<Ice::Long, Hns::Plug> DynamipsRouterI::_getPlugs()
{
    DynamipsRouterEntryPtr dynamipsRouterEntry = DynamipsRouterEntryPtr::dynamicCast(_entityEntry);
    QMap<Ice::Long, Hns::Plug> plugMap;
    ModulePositionMap::iterator itorPositionMap = dynamipsRouterEntry->modulesMap.begin();
    int idxPlug = 0;
    while (itorPositionMap != dynamipsRouterEntry->modulesMap.end()) {
        for (int i = 0; i < itorPositionMap->second.interfaces; i++) {
            Hns::Plug plug;
            plug.id = idxPlug++;
            plug.name = QString("port_%1/%2").arg(itorPositionMap->first).arg(i).toStdString();
            plugMap.insert(plug.id, plug);
        }
        itorPositionMap++;
    }
    return plugMap;
}

bool DynamipsRouterI::_beforeUndefine()
{
    DynamipsRouterEntryPtr dynamipsRouterEntry = DynamipsRouterEntryPtr::dynamicCast(_entityEntry);
    if (dynamipsRouterEntry->autoExtract)
        (void) extractConfiguration();
    return true;
}

void DynamipsRouterI::_initStatusesList()
{
    QWriteLocker locker(&_statusesLock);
    _Hns_DynamipsRouter::_initStatusesList();
    _statusesList.append(Hns::PAUSED);
    emit allowedStatusListChanged();
}

::std::string DynamipsRouterI::extractConfiguration(const ::Ice::Current &current)
{
    DynamipsRouterEntryPtr dynamipsRouterEntry = DynamipsRouterEntryPtr::dynamicCast(_entityEntry);
    HnsNode::DynamipsRouterPrx dynamipsRouterPrx = HnsNode::DynamipsRouterPrx::uncheckedCast(_entityProxy);
    if (dynamipsRouterPrx) {
        dynamipsRouterEntry->configuration = dynamipsRouterPrx->extractConfiguration();
        updateEntry(_entityEntry, current);
    }
    return dynamipsRouterEntry->configuration;
}

bool DynamipsRouterI::pushConfiguration(const ::std::string &configuration, const ::Ice::Current &current)
{
    DynamipsRouterEntryPtr dynamipsRouterEntry = DynamipsRouterEntryPtr::dynamicCast(_entityEntry);
    HnsNode::DynamipsRouterPrx dynamipsRouterPrx = HnsNode::DynamipsRouterPrx::uncheckedCast(_entityProxy);
    if (dynamipsRouterPrx) {
        if (dynamipsRouterPrx->pushConfiguration(configuration)) {
            dynamipsRouterEntry->configuration = dynamipsRouterPrx->extractConfiguration();
            updateEntry(_entityEntry, current);
            return true;
        }
    }
    return false;
}

} // namespace Hns
