/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef _HNS_ENTITY_H_
#define _HNS_ENTITY_H_

// Generated hynesim headers
#include "qiceobj_entity.h"
#include "common/entityentry.h"
#include "common/tracer.h"
#include "common/snapshot.h"
#include "common/types.h"
#include "common/proxies_nodeentity.h"
#include "common/nodeentity.h"

// diamesh headers
#include <diamesh/filetransfer.h>

// Qt headers
#include <QString>
#include <QList>
#include <QHash>
#include <QMap>
#include <QReadWriteLock>
#include <QSharedPointer>

namespace Hns {

class NodeInfoI;
class TracerI;

class EntityI : public _Hns_Entity
{
    Q_OBJECT

public:
    EntityI();
    ~EntityI();

    ::Hns::EntityEntryPtr getEntry(const ::Ice::Current &current = ::Ice::Current());
    void setEntry(const ::Hns::EntityEntryPtr &entityEntry);
    bool unImplement();
    bool updateEntry(const ::Hns::EntityEntryPtr &entityEntry, const ::Ice::Current &current = ::Ice::Current());
    Hns::TracerPrx getTracer(const ::Ice::Current &current = Ice::Current());
    bool define(const std::string &nodeId, const ::Ice::Current &current = ::Ice::Current());
    void define_effective(const Hns::AMD_Entity_definePtr &amdPtr, const ::std::string &nodeId, const Ice::Current &current);
    bool start(const ::Ice::Current &current = ::Ice::Current());
    bool stop(const ::Ice::Current &current = ::Ice::Current());
    bool save(const ::Ice::Current &current = ::Ice::Current());
    bool restore(const ::Ice::Current &current = ::Ice::Current());
    bool suspend(const ::Ice::Current &current = ::Ice::Current());
    bool resume(const ::Ice::Current &current = ::Ice::Current());
    bool createSnapshot(const Hns::Snapshot &snapshot, const Ice::Current &current = ::Ice::Current());
    bool revertSnapshot(const std::string &snapshotName, const Ice::Current &current = ::Ice::Current());
    bool deleteSnapshot(const std::string &snapshotName, const Ice::Current &current = ::Ice::Current());
    Hns::Snapshot getSnapshot(const std::string &snapshotName, const Ice::Current &current = ::Ice::Current());
    std::string getCurrentSnapshotName(const Ice::Current &current = ::Ice::Current());

    bool isDefinable(const ::Ice::Current &current = ::Ice::Current());
    bool isDefined(const ::Ice::Current &current = ::Ice::Current());
    bool undefine(const ::Ice::Current &current = ::Ice::Current());
    bool destroy(const ::Ice::Current &current = ::Ice::Current());
    bool setProxy(HnsNode::EntityPrx entityProxy);
    Ice::Long getId(const ::Ice::Current &current = ::Ice::Current());
    Hns::Status getStatus(const ::Ice::Current &current = ::Ice::Current());
    std::string getNodeName(const ::Ice::Current & = ::Ice::Current());

    ::Hns::PlugMap getPlugs(const ::Ice::Current &current = ::Ice::Current());
    ::Hns::PlugMap getAvailablePlugs(const ::Ice::Current &current = ::Ice::Current());
    std::string getPlugName(::Ice::Long, const ::Ice::Current &current = ::Ice::Current());
    ::Hns::StatusList getAllowedStatuses(const ::Ice::Current &current = ::Ice::Current());
    ::Hns::PcapInfoList getPcapInfoList(const ::Ice::Current &current = ::Ice::Current());
    void setPcapInfoList(const ::Hns::PcapInfoList &pcapInfoList, const ::Ice::Current &current = ::Ice::Current());
    bool plug(int port);
    void unplug(int port);
    void initPlugs();
    void updatePlugs();
    QHash<Ice::Int, Ice::Int> changePlugsForImplementation();
    virtual bool isCaptureValid(CaptureJobOptions jobOptions);
    void eraseCaptureFile(const std::string &file, const::Ice::Current &current);
    Ice::Int createCaptureJob(const::Hns::CaptureJobOptions &jobOptions,const Ice::Current &);
    bool startCaptureJob(Ice::Int jobId);
    bool cancelCaptureJob(Ice::Int jobId,const Ice::Current &current);
    bool isCapturing();
    ::Diamesh::StringList getCaptureFiles(const ::Ice::Current &current);
    ::Diamesh::FileTransferPrx downloadCaptureFile (const std::string &file,const ::Ice::Current &current);
    ::Diamesh::IntList getAvailablePlugsForCapture(const ::Ice::Current &current = ::Ice::Current());
    ::Ice::Int getFlags(const ::Ice::Current &current = ::Ice::Current());
    ::Hns::mapLongString getCaptureJobIdPath(const ::Ice::Current &current = ::Ice::Current());
    ::Hns::PcapInfo getPcapInfoByJobId(::Ice::Int jobId, const ::Ice::Current &current = ::Ice::Current());

    void setIdentity(Ice::Identity identity);
    Ice::Identity identity();

    virtual void setTopologyPath(QString topologyPath);
    RemoteDisplayInfo getRemoteDisplayInfo(const ::Ice::Current &current = ::Ice::Current());
    bool isImplementFromModel(const ::Ice::Current &current = ::Ice::Current());
    void checkCaptureFile(const ::Ice::Current &current = ::Ice::Current());

    IntIntMap getRunningCaptureJobs(const Ice::Current &current = ::Ice::Current());

    static QString statusToString(Status status);

signals:
    void plugsRemoved();
    void captureJobStarted();
    void checkPcapFiles();

protected slots :
    void _captureProgressChangedSlot(Ice::Int jobId, Ice::Double progress);
    void _updateCapturableInterfacesList(::Diamesh::IntList interfaceList);
    void _statusChangedSlot(Ice::Int status, Ice::Long idx);
    void _nodeRemovedSlot();
    void _afterUndefine();

protected:
    virtual bool _define(const std::string &nodeId, const ::Ice::Current &current = ::Ice::Current());
    virtual bool _start(const ::Ice::Current &current = ::Ice::Current());
    virtual bool _stop(const ::Ice::Current &current = ::Ice::Current());
    virtual bool _undefine(const ::Ice::Current &current = ::Ice::Current());

    virtual bool _beforeDefine(const HnsNode::EntityPrx &entityProxy);
    virtual bool _afterDefine();
    virtual bool _beforeUndefine();
    HnsNode::EntityPrx _getProxy(EntityI *entity);
    bool _isExecutable(const ::Ice::Current &current);
    virtual QMap<Ice::Long, Hns::Plug> _getPlugs();
    void _checkDestroyed();
    void _propagateStatus(Hns::Status status);

    virtual bool _createSnapshot(const Hns::Snapshot &snapshot);
    virtual bool _revertSnapshot(const std::string &snapshotName);
    virtual bool _deleteSnapshot(const std::string &snapshotName);
    virtual Hns::Snapshot _getSnapshot(const std::string &snapshotName);
    virtual std::string _getCurrentSnapshotName();
    virtual void _initSnapshotFlags();
    virtual void _initStatusesList();
    void _captureJobTerminated(Ice::Long id);

    virtual QString defineDoneMessage();

protected:
    long _id;
    EntityEntryPtr _entityEntry;
    HnsNode::EntityPrx _entityProxy;
    HnsNode::EntityQPrx _entityProxyQtice;
    QSharedPointer<NodeInfoI> _nodeInfo;
    TracerI *_tracer;
    QHash<Ice::Int, Ice::Int> _captureJobHash;
    QHash<Ice::Int, std::string>_jobIdPathHash;

    QMap<Ice::Long, Hns::Plug> _plugsMap;
    QList<Hns::Status> _statusesList;
    QReadWriteLock _entityLock;
    QReadWriteLock _plugLock;
    QReadWriteLock _captureLock;
    QReadWriteLock _statusesLock;
    QList<Ice::Int>_availableInterfacesForCapture;
    QHash<Ice::Int, PcapInfo> _pcapInfosHash;
    Ice::Identity _identity;

    QString _topologyPath;

    int _flags;
    bool _isDestroyed;
    Hns::Status _status;
    EntityEntryPtr _entityEntryModel;
    qlonglong _idxCptStatus;
    qlonglong _idxCptNodeStatus;
};

class EntityUndefineCallback : public QObject
{
    Q_OBJECT
public:
    explicit EntityUndefineCallback(QObject *receiver, const char *slot, QSharedPointer<NodeInfoI> nodeInfo, HnsNode::EntityPrx prx, QObject *parent = 0);
    ~EntityUndefineCallback();
    void start();
signals:
    void completed();
private slots:
    void _entityUndefineSlot(bool result);
    void _entityUndefinedFailedSlot(QSharedPointer<Ice::Exception> exception);
private:
    QSharedPointer<NodeInfoI> _nodeInfo;
    HnsNode::EntityPrx _prx;
    HnsNode::EntityQPrx _qprx;
};

} // namespace Hns

#endif // _HNS_ENTITY_CATALOG_H_


