/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// hynesim master headers
#include "entityinfoI.h"
#include "entityI.h"

// Generated hynesim headers
#include "common/entityentry.h"
#include "common/domainentry.h"

// diamesh headers
#include <diamesh/masterI.h>

// ZeroC Ice headers
#include <Ice/ObjectAdapter.h>

namespace Hns {

EntityInfoI::EntityInfoI(EntityI * entity)
    : _entity(entity)
    , _idxStatus(0)
{
    Ice::ObjectAdapterPtr adapter = Diamesh::MasterI::instance()->adapter();
    Ice::ObjectPrx proxy = adapter->addWithUUID(this);
    _proxyIdentity = proxy->ice_getIdentity();
    _entityId = _entity->getId();
    _refreshDataFromEntry();
    QObject::connect(_entity, SIGNAL(statusChanged(Ice::Int, Ice::Long)), this, SLOT(_statusChanged(Ice::Int, Ice::Long)));
    QObject::connect(_entity, SIGNAL(entryChanged()), this, SLOT(_entryChanged()));
}

EntityInfoI::~EntityInfoI()
{
}

void EntityInfoI::_entryChanged()
{
    _refreshDataFromEntry();
    emit entryChanged(_infoData);
}

void EntityInfoI::_refreshDataFromEntry()
{
    _entryId = _entity->getEntry()->id;
    _infoData.longName = _entity->getEntry()->longName;
    _infoData.shortName = _entity->getEntry()->shortName;
    _infoData.type = _entity->getEntry()->ice_id();
    if (_infoData.type == DomainEntry::ice_staticId() ) {
        DomainEntryPtr domainEntryModel = DomainEntryPtr::dynamicCast(_entity->getEntry());
        _infoData.subtype = domainEntryModel->type;
    } else {
        _infoData.subtype = "";
    }
    _status = _entity->getStatus();
}

void EntityInfoI::_statusChanged(Ice::Int status, Ice::Long cpt)
{
    if (cpt > _idxStatus) {
        _idxStatus = cpt;
        _status = Hns::Status(status);
        emit statusChanged(status, _idxStatus);
    }
}

Status EntityInfoI::getStatus(const ::Ice::Current &current)
{
    Q_UNUSED(current)
    return _status;
}

Ice::Long EntityInfoI::entryId(const ::Ice::Current &current)
{
    Q_UNUSED(current)
    return _entryId;
}

Ice::Long EntityInfoI::entityId()
{
    return _entityId;
}

void EntityInfoI::removeFromAdapter()
{
    emit requestDeletion(Diamesh::MasterI::instance()->adapter(), _proxyIdentity);
}

EntityInfoPrx EntityInfoI::proxy()
{
    Ice::ObjectAdapterPtr adapter = Diamesh::MasterI::instance()->adapter();
    return EntityInfoPrx::uncheckedCast(adapter->createProxy(_proxyIdentity));
}

::Hns::EntityInfoData EntityInfoI::infoData(const ::Ice::Current &current)
{
    Q_UNUSED(current)
    return _infoData;
}

} // namespace Hns
