/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// hynesim master headers
#include "entrypropertiesmanagerI.h"

// hynesim common headers
#include "common/initializer.h"

// Generated hynesim headers
#include "common/entityentry.h"
#include "common/domainentry.h"
#include "common/topologyentry.h"
#include "common/wireentry.h"
#include "common/switchentry.h"
#include "common/topologygateentry.h"
#include "common/hybridnetcardentry.h"
#include "common/wifiapentry.h"

// Qt headers
#include <QHash>
#include <QDebug>
#include <QScopedPointer>
#include <QMetaObject>

namespace Hns {

HNS_STATIC_HASH_INITIALIZER(_entryPropertiesSetterTypes, EntryPropertiesSetterTypes, QString, QMetaObject)
EntryPropertiesSetterTypes::EntryPropertiesSetterTypes()
{
    insert(QString::fromStdString(EntityEntry::ice_staticId()), EntityEntryPropertiesSetter::staticMetaObject);
    insert(QString::fromStdString(DomainEntry::ice_staticId()), DomainEntryPropertiesSetter::staticMetaObject);
    insert(QString::fromStdString(TopologyEntry::ice_staticId()), TopologyEntryPropertiesSetter::staticMetaObject);
    insert(QString::fromStdString(WireEntry::ice_staticId()), WireEntryPropertiesSetter::staticMetaObject);
    insert(QString::fromStdString(SwitchEntry::ice_staticId()), SwitchEntryPropertiesSetter::staticMetaObject);
    insert(QString::fromStdString(TopologyGateEntry::ice_staticId()), TopologyGatePropertiesSetter::staticMetaObject);
    insert(QString::fromStdString(HybridNetcardEntry::ice_staticId()), HybridNetcardPropertiesSetter::staticMetaObject);
    insert(QString::fromStdString(WifiAPEntry::ice_staticId()), WifiApPropertiesSetter::staticMetaObject);
}

EntryPropertiesManagerI::EntryPropertiesManagerI()
{
}

EntryPropertiesPtr EntryPropertiesManagerI::getProperties(const EntryPtr &entry, const ::Ice::Current &current)
{
    Q_UNUSED(current)
    QScopedPointer<EntryPropertiesSetter> setter(_getPropertiesSetter(entry));
    if (!setter)
        return NULL;

    return setter->getProperties(entry);
}

bool EntryPropertiesManagerI::canValidateEntry(const Hns::EntryPtr &entry, const Ice::Current &current)
{
    Q_UNUSED(current)
    return _entryPropertiesSetterTypes()->contains(QString::fromStdString(entry->ice_id()));
}

EntryPtr EntryPropertiesManagerI::validateEntry(const EntryPtr &entry, const ::Ice::Current &current)
{
    Q_UNUSED(current)
    QScopedPointer<EntryPropertiesSetter> setter(_getPropertiesSetter(entry));
    if (!setter)
        return NULL;

    return setter->validateEntry(entry);
}

QSharedPointer<EntryPropertiesSetter> EntryPropertiesManagerI::propertiesSetter(const EntryPtr &entry)
{
    return QSharedPointer<EntryPropertiesSetter>(_getPropertiesSetter(entry));
}

EntryPropertiesSetter* EntryPropertiesManagerI::_getPropertiesSetter(const EntryPtr &entry)
{
    QString className = QString::fromStdString(entry->ice_id());
    if (!_entryPropertiesSetterTypes()->contains(className)) {
        qDebug() << tr("Classname %1 is not recognized in the properties setters hash").arg(className);
        return NULL;
    }
    QMetaObject metaObject = _entryPropertiesSetterTypes()->value(className);
    EntryPropertiesSetter* setter(qobject_cast<EntryPropertiesSetter *>(metaObject.newInstance()));
    if (!setter) {
        qCritical() << tr("There is no setter available for the type %1")
                       .arg(className);
        return NULL;
    }
    return setter;
}

} // namespace Hns
