/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef ENTRYPROPERTIESSETTER_H
#define ENTRYPROPERTIESSETTER_H

// hynesim common headers
#include "common/entry.h"
#include "common/entityentry.h"
#include "common/entryproperties.h"

// Qt headers
#include <QObject>

namespace Hns {

class EntryPropertiesSetter : public QObject
{
    Q_OBJECT
public:
    Q_INVOKABLE explicit EntryPropertiesSetter(QObject *parent = 0);
    ~EntryPropertiesSetter();
    virtual EntryPropertiesPtr getProperties(const EntryPtr &entry);
    virtual EntryPtr validateEntry(const EntryPtr &entry);

protected:
    virtual bool _fillProperties(const EntryPtr &entry);
    EntryPropertiesPtr _properties;
};

class EntityEntryPropertiesSetter : public EntryPropertiesSetter
{
    Q_OBJECT
public:
    Q_INVOKABLE explicit EntityEntryPropertiesSetter(QObject *parent = 0);
    virtual EntryPropertiesPtr getProperties(const EntryPtr &entry);
    virtual EntryPtr validateEntry(const EntryPtr &entry);
    virtual bool validateEntryWhenDefined(const EntityEntryPtr &oldEntry, const EntityEntryPtr &newEntry);

protected:
    virtual bool _fillProperties(const EntryPtr &entry);
};

class DomainEntryPropertiesSetter : public EntityEntryPropertiesSetter
{
    Q_OBJECT
public:
    Q_INVOKABLE explicit DomainEntryPropertiesSetter(QObject *parent = 0);
    virtual EntryPropertiesPtr getProperties(const EntryPtr &entry);
    virtual EntryPtr validateEntry(const EntryPtr &entry);
    bool validateEntryWhenDefined(const EntityEntryPtr &oldEntry, const EntityEntryPtr &newEntry);

protected:
    virtual bool _fillProperties(const EntryPtr &entry);
};

class TopologyEntryPropertiesSetter : public EntryPropertiesSetter
{
    Q_OBJECT
public:
    Q_INVOKABLE explicit TopologyEntryPropertiesSetter(QObject *parent = 0);
    virtual EntryPropertiesPtr getProperties(const EntryPtr &entry);
    virtual EntryPtr validateEntry(const EntryPtr &entry);

protected:
    virtual bool _fillProperties(const EntryPtr &entry);
};

class WireEntryPropertiesSetter : public EntityEntryPropertiesSetter
{
    Q_OBJECT
public:
    Q_INVOKABLE explicit WireEntryPropertiesSetter(QObject *parent = 0);
    virtual EntryPropertiesPtr getProperties(const EntryPtr &entry);
    virtual EntryPtr validateEntry(const EntryPtr &entry);

protected:
    virtual bool _fillProperties(const EntryPtr &entry);
};

class SwitchEntryPropertiesSetter : public EntityEntryPropertiesSetter
{
    Q_OBJECT
public:
    Q_INVOKABLE explicit SwitchEntryPropertiesSetter(QObject *parent = 0);
    virtual EntryPtr validateEntry(const EntryPtr &entry);
    virtual EntryPropertiesPtr getProperties(const EntryPtr &entry);
    virtual bool validateEntryWhenDefined(const EntityEntryPtr &oldEntry, const EntityEntryPtr &newEntry);
    
protected:
    virtual bool _fillProperties(const EntryPtr &entry);
};

class WifiApPropertiesSetter : public EntityEntryPropertiesSetter
{
    Q_OBJECT
public:
    Q_INVOKABLE explicit WifiApPropertiesSetter(QObject *parent = 0);
    virtual EntryPtr validateEntry(const EntryPtr &entry);
    virtual EntryPropertiesPtr getProperties(const EntryPtr &entry);
    virtual bool validateEntryWhenDefined(const EntityEntryPtr &oldEntry, const EntityEntryPtr &newEntry);

protected:
    virtual bool _fillProperties(const EntryPtr &entry);
};

class TopologyGatePropertiesSetter : public EntityEntryPropertiesSetter
{
    Q_OBJECT
public:
    Q_INVOKABLE explicit TopologyGatePropertiesSetter(QObject *parent = 0);
    virtual EntryPtr validateEntry(const EntryPtr &entry);
    virtual EntryPropertiesPtr getProperties(const EntryPtr &entry);
    virtual bool validateEntryWhenDefined(const EntityEntryPtr &oldEntry, const EntityEntryPtr &newEntry);

protected:
    virtual bool _fillProperties(const EntryPtr &entry);
};

class HybridNetcardPropertiesSetter : public EntityEntryPropertiesSetter
{
    Q_OBJECT
public:
    Q_INVOKABLE explicit HybridNetcardPropertiesSetter(QObject *parent = 0);
    virtual EntryPtr validateEntry(const EntryPtr &entry);
    virtual EntryPropertiesPtr getProperties(const EntryPtr &entry);
    virtual bool validateEntryWhenDefined(const EntityEntryPtr &oldEntry, const EntityEntryPtr &newEntry);

protected:
    virtual bool _fillProperties(const EntryPtr &entry);
};

} // namespace Hns

#endif // ENTRYPROPERTIESSETTER_H
