/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef HNS_ENTRY_SERIALIZER_H
#define HNS_ENTRY_SERIALIZER_H

// Generated hynesim headers
#include "common/entry.h"

// Qt headers
#include <QObject>
#include <QDomDocument>

namespace Hns {

class EntrySerializer : public QObject
{
    Q_OBJECT
public:
    EntrySerializer(const QDomDocument &doc);
    EntrySerializer();
    static EntryPtr readEntry(const QString &path, const QString &docType);
    static EntryPtr readEntry(const QDomDocument &doc, const QString &uuid = QString());
    static bool writeEntry(const EntryPtr &entry, const QString &docType, const QString& fullPath);
    static QString writeEntryToString(const EntryPtr &entry, const QString &docType);
    virtual void init();
    EntryPtr getEntry();
    QString getEntryUUID();
    virtual bool read();
    virtual bool write(const EntryPtr &entry, const QString &docType, const QString &fullPath);
    QString writeToString(const EntryPtr &entry, const QString &docType);

protected:
    virtual bool _fillDoc();
    QDomElement _addNode(QDomElement &parent, const QString &nodeName, const QString &nodeValue = "");
    QDomDocument _doc;
    EntryPtr _entry;
    QString _entryUUID;
    QString _docType;
    double _version;
};

class EntityEntrySerializer : public EntrySerializer
{
    Q_OBJECT
public:
    Q_INVOKABLE EntityEntrySerializer(const QDomDocument &doc);
    Q_INVOKABLE EntityEntrySerializer();
    virtual bool read();

protected:
    virtual bool _fillDoc();
};

class DomainEntrySerializer : public EntityEntrySerializer
{
    Q_OBJECT
public:
    Q_INVOKABLE DomainEntrySerializer(const QDomDocument &doc);
    Q_INVOKABLE DomainEntrySerializer();
    virtual void init();
    virtual bool read();

protected:
    virtual bool _fillDoc();
};

class SwitchEntrySerializer : public EntityEntrySerializer
{
    Q_OBJECT
public:
    Q_INVOKABLE SwitchEntrySerializer(const QDomDocument &doc);
    Q_INVOKABLE SwitchEntrySerializer();
    virtual void init();
    virtual bool read();

protected:
    virtual bool _fillDoc();
};

class HybridNetcardEntrySerializer : public EntityEntrySerializer
{
    Q_OBJECT
public:
    Q_INVOKABLE HybridNetcardEntrySerializer(const QDomDocument &doc);
    Q_INVOKABLE HybridNetcardEntrySerializer();
    virtual void init();
    virtual bool read();

protected:
    virtual bool _fillDoc();
};

class TopologyEntrySerializer : public EntrySerializer
{
    Q_OBJECT
public:
    Q_INVOKABLE TopologyEntrySerializer(const QDomDocument &doc);
    Q_INVOKABLE TopologyEntrySerializer();
    virtual void init();
    virtual bool read();

protected:
    virtual bool _fillDoc();
};

class WifiAPEntrySerializer : public EntityEntrySerializer
{
    Q_OBJECT
public :
    Q_INVOKABLE WifiAPEntrySerializer(const QDomDocument &doc);
    Q_INVOKABLE WifiAPEntrySerializer();
    virtual void init();
    virtual bool read();

protected:
    virtual bool _fillDoc();
};

class DynamipsRouterEntrySerializer : public EntityEntrySerializer
{
    Q_OBJECT
public :
    Q_INVOKABLE DynamipsRouterEntrySerializer(const QDomDocument &doc);
    Q_INVOKABLE DynamipsRouterEntrySerializer();
    virtual void init();
    virtual bool read();

protected:
    virtual bool _fillDoc();
};

class WireEntrySerializer : public EntityEntrySerializer
{
    Q_OBJECT

public :
    Q_INVOKABLE WireEntrySerializer(const QDomDocument &doc);
    Q_INVOKABLE WireEntrySerializer();
    virtual void init();
    virtual bool read();

protected:
    virtual bool _fillDoc();
};

class TopologyGateEntrySerializer : public EntityEntrySerializer
{
    Q_OBJECT
public:
    Q_INVOKABLE TopologyGateEntrySerializer(const QDomDocument &doc);
    Q_INVOKABLE TopologyGateEntrySerializer();
    virtual void init();
    virtual bool read();

protected:
    virtual bool _fillDoc();
};

} // namespace Hns

#endif // HNS_ENTRY_SERIALIZER_H
