/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// hynesim master headers
#include "hybridnetcardI.h"

// Generated hynesim headers
#include "common/entity.h"

// diamesh headers
#include <diamesh/utils.h>

namespace Hns {

HybridNetcardI::HybridNetcardI()
{
    Diamesh::setFlags(Hns::NET_CAPTURE, _flags);
}

HybridNetcardI::~HybridNetcardI()
{
}

QMap<Ice::Long, Hns::Plug> HybridNetcardI::_getPlugs()
{
    QMap<Ice::Long, Hns::Plug> plugMap;
    Hns::Plug plug;
    plug.id = 0;
    plug.name = "hybrid";
    plugMap.insert(0, plug);
    return plugMap;
}

bool HybridNetcardI::_afterDefine()
{
    _hybridNetcardQPrx = HnsNode::HybridNetcardQPrx::uncheckedCast(_entityProxyQtice);
    if (_hybridNetcardQPrx) {
        QObject::connect(_hybridNetcardQPrx.data(), SIGNAL(newMacAddr(QString)), this, SIGNAL(newMacAddr(QString)));
        QObject::connect(_hybridNetcardQPrx.data(), SIGNAL(macAddrGone(QString)), this, SIGNAL(macAddrGone(QString)));
        QObject::connect(_hybridNetcardQPrx.data(), SIGNAL(macBecameActive(QString)), this, SIGNAL(macBecameActive(QString)));
        QObject::connect(_hybridNetcardQPrx.data(), SIGNAL(macBecameInactive(QString)), this, SIGNAL(macBecameInactive(QString)));
        return true;
    }
    return false;
}

Hns::MacAddressMap HybridNetcardI::getMacAddresses(const Ice::Current &current)
{
    Q_UNUSED(current)
    HnsNode::HybridNetcardPrx hybridNetcardPrx = HnsNode::HybridNetcardPrx::uncheckedCast(_entityProxy);
    if (hybridNetcardPrx)
        return hybridNetcardPrx->getMacAddresses();
    else
        return Hns::MacAddressMap();
}

} // namespace Hns
