/******************************************************************************
**
** Copyright (C) 2011-2012 DIATEAM. All rights reserved.
**
** This file is part of HYNESIM.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef ARCHIVEIMPORTER_H
#define ARCHIVEIMPORTER_H

// Generated hynesim headers
#include "common/archivedescriptor.h"
#include "common/importable.h"

// Qt headers
#include <QString>
#include <QStringList>
#include <QHash>

namespace Hns {

class ArchiveImporter : public QObject
{
    Q_OBJECT
public:
    enum CopyPolicy {
        COPY,
        MOVE
    };
    ArchiveImporter(QString tStorage, QString eStorage="", QString exportFolder = "", QObject *parent = 0);
    ~ArchiveImporter();
    bool importArchive(ImportableArchivePtr archive, const ImportOptions &options, const Ice::Current &current = Ice::Current());
    qlonglong importEntityEntry(const ::std::string& folder, const ::std::string& storage, const Ice::Current &iceCurrent);
    bool exportArchive(const ArchiveDescriptorPtr &archiveDescriptor, bool isArchive);
    bool exportEntityEntry(EntityDescriptor &descriptor);
    void setCopyFlag(ArchiveImporter::CopyPolicy copyFlag);

private:
    QList<qlonglong> _importFolderEntryInArchive(QList<FolderDescriptor> fdList, const Ice::Current &current = Ice::Current());
    qlonglong _importEntityEntryInArchive(EntityDescriptor entityDescriptor, const ImportOptions &options, const Ice::Current &current = Ice::Current());
    qlonglong _importTopologyInArchive(TopologyDescriptor topologyDescriptor, QString newName, const Ice::Current &current = Ice::Current());
    QList<qlonglong> _processEntriesInCompressedArchive(QString archivePath, QList<EntityDescriptor> edList, const ImportOptions &options, const Ice::Current &current = Ice::Current());
    QList<qlonglong> _processEntriesInFolderArchive(QString archivePath, QList<EntityDescriptor> edList, const ImportOptions &options, const Ice::Current &current = Ice::Current());
    QList<qlonglong> _processTopologiesInCompressedArchive(QString archivePath, QList<TopologyDescriptor> tdList, const ImportOptions &options, const Ice::Current &current = Ice::Current());
    QList<qlonglong> _processTopologiesInFolderArchive(QString archivePath, QList<TopologyDescriptor> tdList, const ImportOptions &options, const Ice::Current &current = Ice::Current());
    void _processCustomResourcesInCompressedArchive(const QString &archivePath, Diamesh::StringList customResources);
    void _processCustomResourcesInFolderArchive(const QString &archivePath, Diamesh::StringList customResources);

    bool _exportTopology(const TopologyDescriptor &topologyDescriptor);
    bool _exportEntityEntry(const EntityDescriptor &entityDescriptor, QString dstPath, bool insideArchive, bool appendDataToName = false);
    void _exportCustomResources(const QString &customGraphic, bool insideArchive);
    void _copyDirectory(const QString &srcPath, const QString &dstPath, bool letCpCreateDir = false);
    qlonglong _importEntityEntry(const std::string &folderName, const std::string &storage, CopyPolicy policy, const ImportOptions &options, const Ice::Current &iceCurrent);
    void _applyACL(QList<qlonglong> entries, QList<qlonglong> topologies, QList<qlonglong> folders, const ImportOptions &options);
    Diamesh::AccessControlList _generateACL(const Diamesh::ACLTargetPtr &target, const Diamesh::AccessControlList &oldAcl, const QString &rightPrefix);
    Diamesh::AccessControlList _generateFolderACL(const Diamesh::ACLTargetPtr &target, const Diamesh::AccessControlList &oldAcl);
    void _rebaseEntities(BaseEntityMap relationships);

    QString _topologyStorage;
    QString _entityStorage;
    QString _exportFolder;
    long _folderImportId;
    QHash<long, long> _folderIDHash; //<oldId, newID>
    QHash<QString, QString> _entityUUIDHash; //<oldUUID, new UUID>
    QHash<QString, QString> _oldNewResourcesHash;
    QStringList _compressedSuffixes;
    CopyPolicy _copyFlag;
    bool _isArchive;
};

} // namespace Hns
#endif // ARCHIVEIMPORTER_H
