/******************************************************************************
**
** Copyright (C) 2011-2012 DIATEAM. All rights reserved.
**
** This file is part of HYNESIM.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// hynesim master headers
#include "handler.h"
#include "importableserializer.h"
#include "libarchive/libarchivehandler.h"

// hynesim common headers
#include "common/initializer.h"

// Qt headers
#include <QDebug>
#include <QFileInfo>
#include <QDir>
#include <QHash>

namespace Hns {

HNS_STATIC_HASH_INITIALIZER(_deserializerHash, DeserializerHash, QString, QMetaObject)
DeserializerHash::DeserializerHash()
{
    insert(QString("hnsEntryConfig.xml"), ImportableEntrySerializer::staticMetaObject);
    insert(QString("hnsArchive.xml"), ImportableArchiveSerializer::staticMetaObject);
}

/************************************************************
 **********************HANDLER*******************************
 ***********************************************************/
Handler::Handler()
{

}

Handler::~Handler()
{

}

/************************************************************
 **********************FOLDERHANDLER*************************
 ***********************************************************/

FolderHandler::FolderHandler()
{
}

FolderHandler::~FolderHandler()
{

}

bool FolderHandler::canHandle(QString path)
{
    if (QFileInfo(path).isDir()) {
        return true;
    }
    return false;
}

ImportablePtr FolderHandler::handle(QString path)
{
    QDir dir(path);
    QStringList listFiles = dir.entryList(QDir::Files);

    foreach (const QString &fileName, listFiles) {
        if (_deserializerHash()->contains(fileName)) {
            QMetaObject obj = _deserializerHash()->value(fileName);
            ImportableSerializer *serializer = qobject_cast<ImportableSerializer *>(obj.newInstance());
            QFile file(dir.absoluteFilePath(fileName));
            if (!file.open(QIODevice::ReadOnly)) {
                qCritical() << QObject::tr("Can not open the file %1").arg(dir.absoluteFilePath(fileName));
            }
            QString fileContent(file.readAll());
            file.close();
            ImportablePtr ptr = serializer->deserialize(fileContent);
            if (ptr == 0) {
                qCritical() << QObject::tr("Error while deserializing archive");
                return 0;
            }
            ptr->size = -1;
            ptr->path = path.toStdString();
            return ptr;
        }
    }
    return NULL;
}

/************************************************************
 **********************COMPRESSEDHANDLER*********************
 ***********************************************************/

CompressedHandler::CompressedHandler()
{
    _handledExtensions.append("tar");
    //Uncomment below if you want to accept archives in xz or gz format, this is not recommended because it is much more longer for parsing and extracting single files.
//    _handledExtensions.append("tar.gz");
//    _handledExtensions.append("tar.xz");
}

CompressedHandler::~CompressedHandler()
{

}

bool CompressedHandler::canHandle(QString path)
{
    QFileInfo fileInfo(path);
    if (fileInfo.isDir()){
        return false;
    }
    if (_handledExtensions.contains(fileInfo.completeSuffix())){
        return true;
    }
    return false;

}

ImportablePtr CompressedHandler::handle(QString path)
{
    QStringList files;
    files << "hnsArchive.xml";
    QList<QByteArray> list;
    LibArchiveHandler handler(path);
    list = handler.extractOnMemory(files);
    if (!list.isEmpty()) {
        QMetaObject obj = _deserializerHash()->value(files.first());
        ImportableSerializer *serializer = qobject_cast<ImportableSerializer *>(obj.newInstance());
        ImportablePtr ptr = 0;
        QString fileContent = QString(list.first());
        if (!fileContent.isEmpty()) {
            ptr = serializer->deserialize(fileContent);
            if (ptr != 0) {
                ptr->size = handler.getSize();
                ptr->path = path.toStdString();
            }
            return ptr;
        }
    }
    return 0;
}

} // namespace Hns
