/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// hynesim master headers
#include "jobmanagerI.h"
#include "masterI.h"

// hynesim common headers
#include "common/jobI.h"

// ZeroC Ice headers
#include <Ice/ObjectAdapter.h>
#include <Ice/Identity.h>

// Qt headers
#include <QDebug>

namespace Hns {

JobManagerI *JobManagerI::_instance = NULL;

JobManagerI::JobManagerI()
    : _jobId(0)
{
    _id.name = "jobManagerI";
    MasterI::instance()->adapter()->add(this, _id);
}

JobManagerI::~JobManagerI()
{
}

JobManagerI *JobManagerI::create()
{
    if (!_instance)
        _instance = new JobManagerI();
    return instance();
}

JobManagerI *JobManagerI::instance()
{
    return _instance;
}

JobManagerPrx JobManagerI::proxy()
{
    return JobManagerPrx::uncheckedCast(MasterI::instance()->adapter()->createProxy(_id));
}

::Ice::Int JobManagerI::addJob(JobI *job)
{
    if(!job) {
        qCritical() << tr("Job is null. Unable to add it to job manager");
        return false;
    }

    if(job->id()) {
        qCritical() << tr("Job %1 has already been added to job manager").arg(job->id());
        return false;
    }

    Ice::ObjectPrx proxy = MasterI::instance()->adapter()->addWithUUID(job);
    JobPrx jobPrx = JobPrx::uncheckedCast(proxy);

    _jobId++;
    job->setId(_jobId);
    _jobHash[_jobId] = jobPrx;

    return _jobId;
}

::Hns::JobPrx JobManagerI::getJob(::Ice::Int id, const ::Ice::Current &current)
{
    Q_UNUSED(current)
    if (!_jobHash.contains(id)) {
        qCritical() << tr("Job %1 was not found").arg(id);
        return NULL;
    }
    return _jobHash[id];
}

bool JobManagerI::removeJob(::Ice::Int id, const::Ice::Current &current)
{
    Q_UNUSED(current)
    if (!_jobHash.contains(id)) {
        qCritical() << tr("Job %1 was not found").arg(id);
        return false;
    }
    _jobHash.remove(id);
    return true;
}

JobI *JobManagerI::job(::Ice::Int id)
{
    JobPrx jobPrx = getJob(id);
    JobI *job = dynamic_cast<JobI *>(MasterI::instance()->adapter()->findByProxy(jobPrx).get());
    return job;
}

} // namespace Hns
