/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// hynesim master headers
#include "layoutserializer.h"

// hynesim common headers
#include "common/initializer.h"

// Qt headers
#include <QHash>
#include <QDomDocument>
#include <QDomElement>

namespace Hns {

HNS_STATIC_HASH_INITIALIZER(_hashLabelPolicyString, HashLabelPolicyString, ::Hns::LabelPolicy, QString)
HashLabelPolicyString::HashLabelPolicyString()
{
    insert(::Hns::LabelPolicyDefault, "default");
    insert(::Hns::LabelPolicyShow, "show");
    insert(::Hns::LabelPolicyHide, "hide");
}
/*****************************************************************************/
/* EntityLayoutSerializer class                                               */
/*****************************************************************************/
EntityLayoutSerializer::EntityLayoutSerializer()
{
}

QDomElement EntityLayoutSerializer::serialize(QDomDocument *doc, EntityLayoutPtr layout)
{
    QDomElement element = doc->createElement("EntityLayout");
    element.setAttribute("id", layout->id);
    element.setAttribute("posX", layout->x);
    element.setAttribute("posY", layout->y);
    element.setAttribute("labelPolicy", _hashLabelPolicyString()->value(layout->labelPolicyState));
    return element;
}

EntityLayoutPtr EntityLayoutSerializer::unserialize(const QDomElement &element)
{
    EntityLayoutPtr layout= new EntityLayout();
    layout->id = element.attribute("id").toInt();
    layout->x = element.attribute("posX").toInt();
    layout->y = element.attribute("posY").toInt();
    layout->labelPolicyState = _hashLabelPolicyString()->key(element.attribute("labelPolicy", "default"));
    return layout;
}

/*****************************************************************************/
/* WireLayoutSerializer class                                             */
/*****************************************************************************/

WireLayoutSerializer::WireLayoutSerializer()
{
}

QDomElement WireLayoutSerializer::serialize(QDomDocument *doc, Hns::WireLayoutPtr layout)
{
    QDomElement element = doc->createElement("WireLayout");
    element.setAttribute("id", layout->id);
    element.setAttribute("numberAngles", layout->numberAngles);
    element.setAttribute("color", QString::fromStdString(layout->colorCode));
    element.setAttribute("fromX", layout->x1);
    element.setAttribute("fromY", layout->y1);
    element.setAttribute("toX", layout->x2);
    element.setAttribute("toY", layout->y2);

    return element;
}

Hns::WireLayoutPtr WireLayoutSerializer::unserialize(const QDomElement &element)
{
    WireLayoutPtr layout = new WireLayout();
    layout->id = element.attribute("id").toInt();
    layout->numberAngles = element.attribute("numberAngles").toInt();
    layout->colorCode = element.attribute("color").toStdString();
    layout->x1 = element.attribute("fromX", "0").toInt();
    layout->y1 = element.attribute("fromY", "0").toInt();
    layout->x2 = element.attribute("toX", "0").toInt();
    layout->y2 = element.attribute("toY", "0").toInt();
    return layout;
}

} // namespace Hns
