#ifndef LIBARCHIVEHANDLER_H
#define LIBARCHIVEHANDLER_H


// Qt headers
#include <QObject>
#include <QString>
#include <QHash>
#include <QVariant>
#include <QStringList>

// C++ std headers
#include <archive.h>
#include <archive_entry.h>

typedef QHash<QString, QVariant> CompressionOptions;
typedef QHash<QString, QVariant> ExtractionOptions;

enum COMPRESS_TYPE {
    COMPRESS_TAR_GZ,
    COMPRESS_TAR
};

class LibArchiveHandler : public QObject
{
    Q_OBJECT
public:
    explicit LibArchiveHandler(QString fileName, QObject *parent = 0);
    ~LibArchiveHandler();
    QString getFilename();

    bool list();
    bool extractOnDisk(QStringList files, const QString& destinationDirectory, bool removeArchive = false);
    bool compress(QString srcPath, QString dstPath, int flag);
    QList<QByteArray> extractOnMemory(QStringList files, bool removeArchive = false);
    qint64 getSize();

private:
    struct ArchiveReadCustomDeleter;
    struct ArchiveWriteCustomDeleter;
    typedef QScopedPointer<struct archive, ArchiveReadCustomDeleter> ArchiveRead;
    typedef QScopedPointer<struct archive, ArchiveWriteCustomDeleter> ArchiveWrite;

    void _copyDataOnDisk(struct archive *source, struct archive *dest);
    QByteArray _copyDataOnMemory(struct archive *source);
    bool _checkArchiveRead(ArchiveRead& archive);
    bool _checkArchiveWrite(ArchiveWrite& archive);
    bool _writeFile(const QString &fileName, struct archive* arch_writer, QString srcPath);
    void _copyData(const QString &fileName, struct archive *dest);
    void _writeEntry(struct archive *arch, struct archive_entry *entry, struct archive *archReader);
    QStringList _getFileNames(QString path);
    void _constructRootDirectories();

    QString _fileName;
    int _cachedArchiveEntryCount;
    qlonglong _extractedFilesSize;
    qint64 _archiveSize;
    QStringList _directories;
    ArchiveRead _archiveReadDisk;
    QStringList _rootDirectories;

};

#endif // LIBARCHIVEHANDLER_H
