/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef HNS_MACADDRESS_MANAGER_H
#define HNS_MACADDRESS_MANAGER_H

// Generated hynesim headers
#include <qiceobj_macaddressmanager.h>

// diamesh headers
#include <diamesh/types.h>

// Qt headers
#include <QString>
#include <QStringList>
#include <QSet>
#include <QReadWriteLock>

namespace Hns {

class MacAddressManagerI : public _Hns_MacAddressManager
{
    Q_OBJECT
public:
    virtual ~MacAddressManagerI();
    static MacAddressManagerI *create();
    static MacAddressManagerI *instance();
    static bool isValid(QString macAddress);
    MacAddressManagerPrx proxy();
    ::std::string generateRandomAddress(const ::std::string &vendorStr, const ::Ice::Current &current = ::Ice::Current());
    bool reserveMacAddress(Ice::Long entryId, const QString &macAddress);
    bool registerMacAddress(Ice::Long entryId, const QString &macAddress);
    bool unregisterMacAddress(Ice::Long entryId, const QString &macAddress);
    bool isUsed(const ::std::string &macAddress, const ::Ice::Current &current = ::Ice::Current());
    bool isReserved(const QString &macAddress);
    ::Diamesh::StringList getVendorList(const ::Ice::Current &current);
    ::std::string getVendorFromPrefix(const ::std::string &macPrefix, const ::Ice::Current &current = ::Ice::Current());

private:
    explicit MacAddressManagerI();
    QString _getRandomVendor();
    QString _getRandomVendorPrefix(const QString &vendor);
    QString _getRandomMacSuffix();
    void _fillVendorPrefixData();
    QSet<QString> _getUsedMacAddresses();
    QSet<QString> _getReservedMacAddresses();

private:
    static MacAddressManagerI *_instance;
    Ice::Identity _id;
    QMap<QString, QStringList > _vendorPrefixMap;
    QMap<Ice::Long, QSet<QString> > _usedMacAddresses;
    QMap<Ice::Long, QSet<QString> > _reservedMacAddresses;
    QReadWriteLock _macAddressesLock;
};

} // namespace Hns

#endif // HNS_MACADDRESS_MANAGER_H
