/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef HNS_MASTER_H
#define HNS_MASTER_H

// Generated hynesim headers
#include "qiceobj_master.h"
#include "common/entityentry.h"

// Qt headers
#include <QSharedPointer>


namespace HnsServer {
class StorageManager;
} // namespace HnsServer

namespace Hns {

class TopologyCatalogI;
class EntityCatalogI;
class NodeInfoI;
class EntityFactoryI;
class TopologyManagerI;
class EntryPropertiesManagerI;
class CustomResourceManagerI;
class CatalogFileStorage;
class TagManagerI;

class MasterI : public _Hns_Master
{
    Q_OBJECT
public:
    using Diamesh::MasterI::nodeInfoList;
    MasterI();
    ~MasterI();
    static MasterI *instance();
    EntityCatalogPrx getEntityCatalog(const ::Ice::Current &current = ::Ice::Current());
    EntityCatalogI *entityCatalog();
    TopologyCatalogPrx getTopologyCatalog(const ::Ice::Current &current = ::Ice::Current());
    TopologyCatalogI *topologyCatalog();
    TagManagerPrx  getTagManager(const ::Ice::Current &current = ::Ice::Current());
    TagManagerI *tagManager();
    EntityFactoryPrx getEntityFactory(const ::Ice::Current &current = ::Ice::Current());
    TopologyManagerPrx getTopologyManager(const ::Ice::Current &current = ::Ice::Current());
    EntryPropertiesManagerPrx getEntryPropertiesManager(const ::Ice::Current &current = ::Ice::Current());
    EntryPropertiesManagerI *entryPropertiesManager();
    JobManagerPrx getJobManager(const ::Ice::Current &current = Ice::Current());
    CatalogImporterPrx getCatalogImporter(const ::Ice::Current &current = Ice::Current());
    MacAddressManagerPrx getMacAddressManager(const ::Ice::Current &current = Ice::Current());
    DynamipsRouterDBPrx  getDynamipsRouterDB(const ::Ice::Current &current = Ice::Current());
    TracerPrx getTracer(const ::Ice::Current &current = Ice::Current());
    NodeInfoMap getEntryNodeInfoMap(const EntityEntryPtr &entry, const ::Ice::Current &current = ::Ice::Current());
    bool getAdvancedMode(const ::Ice::Current &current = ::Ice::Current());
    Diamesh::VersionInfo getVersionInfo(const ::Ice::Current &current);
    QList<QSharedPointer<NodeInfoI> > nodeInfoList(const ::Hns::EntityEntryPtr &entry, QHash<QString, QList<EntityEntryPtr> > alreadyComputedEntries = QHash<QString, QList<Hns::EntityEntryPtr> >());
    QSharedPointer<NodeInfoI> getSuitableNode(const ::Hns::EntityEntryPtr &entry, QHash<QString, QList<EntityEntryPtr> > alreadyComputedEntries = QHash<QString, QList<Hns::EntityEntryPtr> >());
    CustomResourceManagerPrx getCustomResourceManager(const ::Ice::Current &current = ::Ice::Current());
    CustomResourceManagerI *customResourceManager();
    CatalogFileStorage *getFirstEntityFileStorage();
    HnsServer::StorageManager *storageManager();

public slots:
    void refreshEntityClassSlot(QString);

protected:
    virtual bool _init();
    bool _addNode(const Diamesh::NodePrx &node, const QString &key);
    virtual void _registerACLTargetTypes();
    virtual void _registerRights();
    TopologyCatalogI *_topologyCatalog;
    EntityCatalogI *_entityCatalog;
    EntityFactoryI *_entityFactory;
    TopologyManagerI *_topologyManager;
    EntryPropertiesManagerI *_entryPropertiesManager;
    CustomResourceManagerI *_customResourceManager;
    TagManagerI *_tagManager;
    HnsServer::StorageManager *_storageManager;
    bool _advancedMode;
};

} // namespace Hns
#endif // HNS_MASTER_H
