#include "memorydumpjob.h"

#include "masterI.h"

#include "libhnsserver/storagemanager.h"

#include <QDateTime>
#include <QDebug>
#include <QDir>

namespace Hns {

MemoryDumpJob::MemoryDumpJob(HnsNode::DomainPrx domainProxy)
    : JobI()
    , _domainProxy(domainProxy)
{
    _memoryDumpInfo.dateTime = QDateTime::currentMSecsSinceEpoch();
}

MemoryDumpJob::~MemoryDumpJob()
{
    _domainProxy = 0;
}

MemoryDumpInfo MemoryDumpJob::memoryDumpInfo()
{
    return _memoryDumpInfo;
}

void MemoryDumpJob::setName(const QString &name)
{
    _memoryDumpInfo.name = name.toStdString();
}

void MemoryDumpJob::setTopologyPath(const QString &path)
{
    _topologyPath = path;
    QDateTime dateTime;
    dateTime.setMSecsSinceEpoch(_memoryDumpInfo.dateTime);
    QString fileName = QString("%1_%2.dump")
            .arg(dateTime.toString("yyyyMMdd_hhmm"))
            .arg(_memoryDumpInfo.name.c_str());
    _memoryDumpInfo.fileName = fileName.toStdString();

    HnsServer::StorageManager *storageManager = MasterI::instance()->storageManager();
    QString topologyFullPath = storageManager->getFullPathFromMixed("topology", path);

    QDir memoryDumpsDir(topologyFullPath + "/memorydumps");
    QDir absoluteMemoryDumpsDir(memoryDumpsDir.absolutePath());
    if (!memoryDumpsDir.exists()) {
        absoluteMemoryDumpsDir.mkdir(memoryDumpsDir.absolutePath());
    }
}

void MemoryDumpJob::_run()
{
    QString path = QString("%1/memorydumps/%2")
            .arg(_topologyPath)
            .arg(QString::fromStdString(_memoryDumpInfo.fileName));
    int ret = _domainProxy->createMemoryDump(path.toStdString());
    if (ret < 0) {
        qCritical() << tr("Unable to create memory dump");
        cancel();
        return;
    }
    _memoryDumpInfo.fileName += ".tar.gz";
    path += ".tar.gz";
    HnsServer::StorageManager *storageManager = Hns::MasterI::instance()->storageManager();
    QString fullPath = storageManager->getFullPathFromMixed("topology", path);
    QFile file(fullPath);
    _memoryDumpInfo.size = file.size();
    _updateProgress(1.0);
}

} // namespace Hns
