/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// hynesim master headers
#include "nodeinfoI.h"
#include "masterI.h"
#include "nodemonitorI.h"

namespace Hns {

NodeInfoI::NodeInfoI(const QString &nodeKey, const NodePrx &node)
    : _Hns_NodeInfo(nodeKey, node)
{
    _nodeProxyQtice = NodeQPrx(node, MasterI::instance()->adapter());
    QObject::connect(_nodeProxyQtice.data(), SIGNAL(resourcesChanged(HnsNode::ResourceInfo)), this, SIGNAL(resourcesChanged(HnsNode::ResourceInfo)));
    QObject::connect(_nodeProxyQtice.data(), SIGNAL(refreshClassName(QString)), MasterI::instance(), SLOT(refreshEntityClassSlot(QString)));
    QObject::connect(_nodeProxyQtice.data(), SIGNAL(addEntity(long)), this, SLOT(_addEntity(long)));
    QObject::connect(_nodeProxyQtice.data(), SIGNAL(removeEntity(long)), this, SLOT(_removeEntity(long)));
    _nodeMonitor = new NodeMonitorI(_nodeProxyQtice->getNodeMonitor(), nodeKey);
    QObject::connect(this, SIGNAL(nodeRemoved()), _nodeMonitor, SLOT(destroy()));
}

bool NodeInfoI::canBuild(const::Hns::EntityEntryPtr &entry, const::Ice::Current &current)
{
    Q_UNUSED(current)
    NodePrx nodePrx = NodePrx::uncheckedCast(node());
    return nodePrx->getEntityFactory()->canBuild(entry);
}

bool NodeInfoI::canBuildAll(const EntityEntryPtr &entityEntry, const EntityEntryList &alreadyComputedEntries, const Ice::Current &current)
{
    Q_UNUSED(current)
    NodePrx nodePrx = NodePrx::uncheckedCast(node());
    return nodePrx->getEntityFactory()->canBuildAll(entityEntry, alreadyComputedEntries);
}

Diamesh::LongList NodeInfoI::canBuildFromEntryMap(QMap<Ice::Long, EntityEntryPtr> entryMap)
{
    Diamesh::LongList resultList;
    NodePrx nodePrx = NodePrx::uncheckedCast(node());
    for (QMap<Ice::Long, EntityEntryPtr>::const_iterator it = entryMap.begin(); it != entryMap.end(); ++it) {
        Ice::Long entityId = it.key();
        EntityEntryPtr entry = it.value();
        if (nodePrx->getEntityFactory()->canBuild(entry))
            resultList.push_back(entityId);
    }
    return resultList;
}

Diamesh::VersionInfo NodeInfoI::getVersionInfo(const ::Ice::Current &current)
{
    Q_UNUSED(current)
    NodePrx nodePrx = NodePrx::uncheckedCast(node());
    return nodePrx->getVersionInfo();
}

HnsNode::ResourceInfo NodeInfoI::getResourceInfo(const ::Ice::Current &current)
{
    Q_UNUSED(current)
    NodePrx nodePrx = NodePrx::uncheckedCast(node());
    return nodePrx->getResourceInfo();
}

Hns::PlatformInfoMap NodeInfoI::getPlatforms(const ::Ice::Current &current)
{
    Q_UNUSED(current)
    NodePrx nodePrx = NodePrx::uncheckedCast(node());
    return nodePrx->getPlatforms();
}

void NodeInfoI::_addEntity(long entityId)
{
    _setEntityId.insert(entityId);
    emit(addEntity(entityId));
}

void NodeInfoI::_removeEntity(long entityId)
{
    _setEntityId.remove(entityId);
    emit(removeEntity(entityId));
}

::Diamesh::LongList NodeInfoI::getEntityDefineIds(const ::Ice::Current &current)
{
    Q_UNUSED(current)
    return _setEntityId.toList().toStdList();
}

NodeMonitorPrx NodeInfoI::getNodeMonitor(const Ice::Current &current)
{
    Q_UNUSED(current)
    return _nodeMonitor->proxy();
}

} // namespace Hns
