/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef _HNS_TOPOLOGY_H_
#define _HNS_TOPOLOGY_H_

// Generated hynesim headers
#include "qiceobj_topology.h"
#include "common/topologyentry.h"
#include "common/topologylink.h"

// Qt headers
#include <QSignalMapper>
#include <QStringList>
#include <QMap>
#include <QHash>
#include <QMultiHash>
#include <QSet>
#include <QMutex>
#include <QReadWriteLock>

namespace Diamesh {
class SessionI;
} // namespace Diamesh

namespace Hns {

class EntityCloneJob;
class EntityI;
class TracerI;

class TopologyI : public _Hns_Topology
{    
    Q_OBJECT

    typedef void (TopologyI::*ComputeBalancing) (QMap<Ice::Long, Hns::DefineInfo> &);

protected:
    struct JobInfo
    {
        EntityCloneJob *cloneJob;
        Hns::EntityLayoutPtr layout;
        ::Ice::Long entryModelId;
        ::Ice::Current current;
    };

public:
    TopologyI(const::Hns::TopologyEntryPtr &topologyEntry);
    ~TopologyI();

    EntityTopologyLinkStruct createEntityFromEntry(const ::Hns::EntityEntryPtr &entityEntry, const LayoutPtr &layout, const ::Ice::Current &current = ::Ice::Current(), bool initialLoading = false, const TopologyLinkList &links = TopologyLinkList());
    EntityTopologyLinkStruct createEntityFromEntry(const ::Hns::EntityEntryPtr &entityEntry, const Hns::LayoutPtr &layout, const TopologyLinkList &links = TopologyLinkList(), const ::Ice::Current &current = ::Ice::Current());
    EntityTopologyLinkStruct createEntityFromEntryId(::Ice::Long entityId, const Hns::LayoutPtr &layout, const TopologyLinkList &links = TopologyLinkList(), const ::Ice::Current &current = ::Ice::Current());
    Ice::Int createEntityFromClonedEntryId(::Ice::Long entryId, const Hns::EntityLayoutPtr &layout, ::Ice::Long entryModelId , const ::Ice::Current &current = ::Ice::Current());
    bool removeEntity(::Ice::Long entityId, const ::Ice::Current &current = ::Ice::Current());
    bool plugEntity(::Ice::Long entityId, ::Ice::Int entityPortId, ::Ice::Long wireId, ::Ice::Int wirePortId, const ::Ice::Current &current = ::Ice::Current());
    bool unplugEntity(::Ice::Long entityId, ::Ice::Int entityPortId, ::Ice::Long wireId, ::Ice::Int wirePortId, const ::Ice::Current &current = ::Ice::Current());
    LongEntityMap getEntities(const ::Ice::Current &current = ::Ice::Current());
    EntityPrx getEntity(::Ice::Long entityId, const ::Ice::Current &current = ::Ice::Current());
    LongLinkListMap getTopologyLinks(const Ice::Current &current);
    TopologyLinkList getTopologyLink(Ice::Long wireId, const Ice::Current &current);
    Ice::Long getId(const ::Ice::Current &current = ::Ice::Current());
    TopologyEntryPtr getEntry(const ::Ice::Current &current = ::Ice::Current());
    bool updateEntry(const TopologyEntryPtr &entry, const ::Ice::Current &current = ::Ice::Current());
    void setEntry(const TopologyEntryPtr &entry);
    bool canBeSaved(QString &exceptionMessage);
    void disconnectSignalsFromEntity(EntityPrx entity);
    EntityPrx implementModelEntity(Ice::Long entityId, Ice::Long entryId, const Ice::Current &current = ::Ice::Current());
    bool unImplementModelEntity(Ice::Long entityId, const Ice::Current &current= ::Ice::Current());
    Hns::IdImportedList importTopology(Ice::Long topologyEntryId, const EntityLayoutPtr &positionLayout, const Ice::Current &current = ::Ice::Current());

    bool isModified(const ::Ice::Current &current = ::Ice::Current());
    TracerPrx getTracer(const ::Ice::Current &current = ::Ice::Current());
    TopologyLayout getLayout(const ::Ice::Current &current = ::Ice::Current());
    void setLayout(const ::Hns::TopologyLayout &layout, bool initialLoading = false, const ::Ice::Current &current = ::Ice::Current());
    void setLayout(const ::Hns::TopologyLayout &layout, const ::Ice::Current &current = ::Ice::Current());
    TopologyLayoutOptions getLayoutOptions(const ::Ice::Current &current = ::Ice::Current());
    void setLayoutOptions(const ::Hns::TopologyLayoutOptions &options, const ::Ice::Current &current = ::Ice::Current());
    EntityLayoutPtr getEntityLayout(::Ice::Long entityId, const ::Ice::Current &current = ::Ice::Current());
    WireLayoutPtr getWireLayout(::Ice::Long wireId, const ::Ice::Current &current = ::Ice::Current());
    void updateEntityLayout(const EntityLayoutPtr &layout, const ::Ice::Current &current = ::Ice::Current());
    void updateWireLayout(const WireLayoutPtr &layout, const ::Ice::Current &current = ::Ice::Current());
    void updateWireLayout(const WireLayoutPtr &layout, bool initialLoading = false, const ::Ice::Current &current = ::Ice::Current());
    ::Diamesh::LongList canBuildWithNode(const ::std::string &nodeName, const ::Ice::Current &current = ::Ice::Current());
    ::Hns::LongDefineInfoMap computeDefineInfos(const::Hns::LongDefineInfoMap &defineInfoMap, ::Hns::BalancingBehavior behavior, const ::Ice::Current &current);
    ::Hns::BalancingBehaviorMap getBehaviorMap(const ::Ice::Current &current = ::Ice::Current());
    static TopologyI::ComputeBalancing getComputeBalancing(::Hns::BalancingBehavior behavior);

    EntityI *getEntityI(::Ice::Long entityID);
    EntityI *wireEntity(::Ice::Long wireID);
    void clear(QList< Ice::Long > &entityRemoved);

    QList<Diamesh::SessionI*> getSessions();
    QHash< ::Ice::Long, EntityPrx> entities();
    TopologyPrx proxy();
    void setModified(bool modified);
    QMultiHash<Ice::Long, TopologyLink> wires();
    TopologyEntryPtr topologyEntry();
    TracerI *tracer();
    void addSession(const QString &sessionId);
    void removeSession(const QString &sessionId);
    bool hasSession(const QString &sessionId);

    void defineWireIfNeeded(EntityI *entity);
    void entityUndefined(EntityI *entity);
    void removeFromAdapter();

    Diamesh::ACLList getACLs(const ::Ice::Current &current = ::Ice::Current());
    bool setACLs(const Diamesh::ACLList &aclList, const ::Ice::Current &current = ::Ice::Current());

public slots:
    void entityChanged(QObject *object);
    void checkWires(QObject *object);
    void checkCpapFiles(QObject *object);

private slots:
    void _init();
    void _entityEntryChanged();
    void _wireEntryChanged();
    void _captureJobStartedSlot();
    void _memoryDumpJobStartedSlot();
    void _createOrImplementFromCloneJob(Ice::Int jobId);
    void _entryACLsChangedSlot(Ice::Long entryId, Diamesh::ACLList aclList);

protected:
    bool _addEntity(EntityPrx entity, const ::Ice::Current &current = ::Ice::Current(), bool initialLoading = false);
    EntityPrx _getEntity(::Ice::Long entityId);
    QString _getPath();
    void _defineWireIfNeeded(EntityI *entity);
    bool _removeWire(Ice::Long wireId, const ::Ice::Current &current = ::Ice::Current());
    EntityI *_getLinkedEntity(const TopologyLink &link);
    int _wireEntityId(const ::std::string &wireEntity);
    int _wireEntityPort(const ::std::string &wireEntity);
    QString _wireNewPort(const ::std::string &wireEntity, int newPort);
    EntityPrx _handleCloneJob(EntityCloneJob *cloneJob, Hns::EntityLayoutPtr &layout, Ice::Long entryModelId, const ::Ice::Current &current);
    void _computeRandomDefineInfos(QMap<Ice::Long, Hns::DefineInfo> &defineInfos);
    void _computeFirstDefineInfos(QMap<Ice::Long, Hns::DefineInfo> &defineInfos);
    void _computeAutoDefineInfos(QMap<Ice::Long, Hns::DefineInfo> &defineInfos);
    EntityI *_entity(::Ice::Long entityID);
    void _plugChanged(::Ice::Long entityID, const QHash<Ice::Int, Ice::Int> &hashOld2New);
    TopologyLink _getOtherSideLink (const TopologyLink &link);
    std::string _getRandomNodeNameForWire(const std::string &node1, const std::string &node2);
    WirePrx _getWirePrx(Ice::Long wireId);
    void _validateWireLayout(WireLayoutPtr wireLayout, QRectF itemRect, QRectF sceneRect);

protected:
    static QHash<Hns::BalancingBehavior, TopologyI::ComputeBalancing> *_computationalBehaviors;
    long _id;
    QReadWriteLock _entitiesLock;
    QReadWriteLock _wiresLock;
    QReadWriteLock _entitiesSupLock;
    QHash< ::Ice::Long, EntityPrx> _entities;
    QMultiHash< ::Ice::Long, TopologyLink> _wiresLink; //wireId, TopologyLink
    TopologyEntryPtr _topologyEntry;
    QSignalMapper *_statusSignalMapper;
    QSignalMapper *_plugCountSignalMapper;
    QSignalMapper *_checkcpapFileSignalMapper;
    bool _modified;
    TracerI *_tracer;
    QStringList _sessions;
    TopologyLayout _layout;
    Ice::Identity _proxyIdentity;
    QSet<qlonglong> _setEntityBusy;
    QMap<int , JobInfo> _jobInfoMap;
    QMutex _jobInfoMutex;
};

} // namespace Hns

#endif // _HNS_TOPOLOGY_H_


