/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef _HNS_TOPOLOGY_MANAGER_H_
#define _HNS_TOPOLOGY_MANAGER_H_

// Generated hynesim headers
#include "qiceobj_topologymanager.h"
#include "common/topologyentry.h"
#include "common/entityinfo.h"

// diamesh headers
#include <diamesh/types.h>

// Qt headers
#include <QString>
#include <QSet>
#include <QReadWriteLock>

namespace Hns {

class TopologyI;
class EntityInfoI;

class TopologyManagerI : public _Hns_TopologyManager
{
    Q_OBJECT

public:
    TopologyManagerI();
    ~TopologyManagerI();
    TopologyPrx createTopology(bool isModel, const ::Hns::TopologyLayoutOptions& layoutOptions, const ::Ice::Current &current = ::Ice::Current());
    TopologyPrx loadTopology(Ice::Long topologyEntryId, const ::Ice::Current &current = ::Ice::Current());
    bool saveTopology(::Ice::Long topologyId, const ::Ice::Current &current = ::Ice::Current());
    bool saveNewTopology(::Ice::Long topologyId, const TopologyEntryPtr &entry, const ::Ice::Current &current = ::Ice::Current());
    bool saveTopologyAs(::Ice::Long topologyId, const TopologyEntryPtr &entry, const ::Ice::Current &current = ::Ice::Current());
    bool closeTopology(::Ice::Long topologyId, const ::Ice::Current &current = ::Ice::Current());
    bool unloadTopology(::Ice::Long topologyId, const ::Ice::Current &current = ::Ice::Current());
    bool deleteTopology(::Ice::Long entryId, const ::Ice::Current &current = ::Ice::Current());
    Ice::Long switchTopologyType(::Ice::Long topologyId, const ::std::string &newBackgroundColor, const ::Ice::Current &current = ::Ice::Current());
    Diamesh::LongList getTopologies(const ::Ice::Current &current = ::Ice::Current());
    TopologyPrx getTopologyById(::Ice::Long id, const ::Ice::Current &current = ::Ice::Current());
    TopologyPrx getTopologyByUuid(const std::string &topologyUuid, const ::Ice::Current &current = ::Ice::Current());
    ::Diamesh::LongList getTopologyUsers(::Ice::Long topologyId, const ::Ice::Current &current = ::Ice::Current());

    EntityInfoPrx getEntityInfo(::Ice::Long entryId, const ::Ice::Current &current = ::Ice::Current());
    void requestGetEntityInfo(::Ice::Long, const ::Ice::Current &current = ::Ice::Current());
    ::Hns::EntityInfoMap getListEntityInfo(const ::Ice::Current &current = ::Ice::Current());

private slots:
    void _topologyModifiedSlot(const QString &uuid, bool modified);
    void _entityRemovedSlot(Ice::Long entityId);
    void _entityAddSlot(Ice::Long topologyId, Ice::Long entityId);
    void _wireAddSlot(Ice::Long topologyId, Ice::Long entityId);
    void _entityUnImplemented(Ice::Long entityEntryID);
    void _entityImplemented(Ice::Long entityId, Ice::Long entityEntryID);
    void _entryUsedSlot(Ice::Long entryId);

private:
    void _addTopologyToHashes(TopologyI *topology, const QString &uuid);
    void _connectToTopology(TopologyI *topology);
    bool _clearTopology(TopologyI *topology);
    bool _closeTopology(::Ice::Long topologyId, const ::Ice::Current &current, bool isDelete = false);
    bool _saveTopology(TopologyI *topology, const ::Ice::Current &current);
    bool _exportTopology(TopologyI *topology, TopologyEntryPtr &entry, const ::Ice::Current &current, const ::std::string &newBackgroundColor = "");
    bool _deleteUnknownPcapFiles(TopologyI *topology);
    bool _deleteUnknownMemoryDumpFiles(TopologyI *topology);
    TopologyPrx _getTopologyById(::Ice::Long id, const ::Ice::Current &current);
    TopologyPrx _getTopologyByUuid(const std::string &topologyUuid, const ::Ice::Current &current);
    bool _setTopologyUsed(long topologyId, const ::Ice::Current &current);
    QHash<QString, ::Ice::Long> _uuidHash; // topology UUID, topology id
    QHash< ::Ice::Long, TopologyI *> _topologyHash; // topology id, topology proxy
    QHash< ::Ice::Long, EntityInfoI*> _entityInfoHash; // entity, entity info
    QHash< ::Ice::Long, ::Ice::Long > _entryEntityHash; // entryId, entityId
    QSet< ::Ice::Long> _setEntityWait;
    QReadWriteLock _topologiesLock;
    QReadWriteLock _entityInfoLock;
    QReadWriteLock _entityWaitInfoLock;
    QReadWriteLock _topologiesBusyLock;
    int _topologyCounter;
    QSet< ::Ice::Long> _setBusyTopology;
};

} // namespace Hns

#endif // _HNS_TOPOLOGY_MANAGER_H_
