/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef HNS_TRACER_H
#define HNS_TRACER_H

// Generated hynesim headers
#include "qiceobj_tracer.h"

// Qt headers
#include <QString>
#include <QList>
#include <QReadWriteLock>

namespace Diamesh {
class SqlDatabase;
} // namespace Diamesh

namespace Hns {

class TracerI : public _Hns_Tracer
{
    Q_OBJECT
public:
    explicit TracerI();
    virtual ~TracerI();
    static TracerI *get(const QString &directory = QString());
    static Diamesh::SqlDatabase *getTracerDb(const QString &directory);
    void setDirectory(const QString &directory);
    const QString &directory();
    void close();
    void trace(const QString &content, const Ice::Current &current = Ice::Current(), const QString &category = QString());
    TracerPrx proxy();
    void destroy();

    TraceList getLatestTraces(const ::Ice::Current &current);
    TraceList getHistory(::Ice::Int pageNumber, ::Ice::Int count, const::std::string &match, ::Ice::Long beginDate, ::Ice::Long endDate, const ::Ice::Current &current);

private:
    void _sqlTrace(const Trace &t);
    QString _directory;
    Diamesh::SqlDatabase *_db;
    QList<Trace> _traceList;
    QReadWriteLock _traceListLock;
    TracerPrx _prx;

};

} // namespace Hns

#endif // HNS_TRACER_H
