/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// hynesim master headers
#include "wireI.h"
#include "masterI.h"

// Generated hynesim headers
#include "common/proxies_nodewire.h"

// ZeroC Ice headers
#include <Ice/ObjectAdapter.h>

namespace Hns {

WireI::WireI()
{
    setObjectName(tr("Wire %1").arg(_id));
}

WireI::~WireI()
{
}

void WireI::setLinkedEntities(EntityPrx fromEntity, int fromPort, EntityPrx toEntity, int toPort)
{
    _fromEntity = fromEntity;
    _fromPort = fromPort;
    _toEntity = toEntity;
    _toPort = toPort;
}

bool WireI::_beforeDefine(const HnsNode::EntityPrx &entityProxy)
{
    HnsNode::WirePrx wireProxy = HnsNode::WirePrx::uncheckedCast(entityProxy);
    EntityI *fromEntityPointer = dynamic_cast<EntityI *>(MasterI::instance()->adapter()->findByProxy(_fromEntity).get());
    EntityI *toEntityPointer = dynamic_cast<EntityI *>(MasterI::instance()->adapter()->findByProxy(_toEntity).get());
    bool result = wireProxy->setLinkedEntities(_getProxy(fromEntityPointer), _fromPort, _getProxy(toEntityPointer), _toPort);
    return result;
}

QString WireI::defineDoneMessage()
{
    return tr("Define done for wire between %1 and %2").arg(_fromEntity->getEntry()->uuid.c_str())
            .arg(_toEntity->getEntry()->uuid.c_str());
}

void WireI::changePortFrom(int port)
{
    _fromPort = port;
    emit portChangedFrom(port);
}

void WireI::changePortTo(int port)
{
    _toPort = port;
    emit portChangedTo(port);
}

} // namespace Hns
