/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef _HNS_NODE_BRIDGEMONITOR_H
#define _HNS_NODE_BRIDGEMONITOR_H

// Qt headers
#include <QObject>
#include <QHash>
#include <QReadWriteLock>

#include <libmnl/libmnl.h>

class QSocketNotifier;

namespace HnsNode {

class QNFlog;

struct MacAddressInformation
{
    QString macAddress;
    bool isActive;
    qint64 timestamp;
    bool operator ==(const MacAddressInformation &a) const
    {
        if (a.macAddress == this->macAddress)
            return true;
        return false;
    }
};

/*******************************************************************************
* BridgeMonitor
*******************************************************************************/
class BridgeMonitor : public QObject
{
    Q_OBJECT
public:
    ~BridgeMonitor();
    static BridgeMonitor *instance();
    static void create(QObject *parent = 0);
    QHash<QString, bool> getMacAddresses(QString bridgeName, int groupId);
    Q_INVOKABLE void registerEntity(QString bridgeName, int groupId, QObject *entity);
    Q_INVOKABLE void unregisterEntity(QObject *entity);

signals:
    void newMacAddr(QString);
    void macAddrGone(QString);
    void macBecameActive(QString);
    void macBecameInactive(QString);

protected:
    void timerEvent(QTimerEvent *event);

private:
    BridgeMonitor(QObject *parent = 0);
    void _stop();
    void _start(int interval = 10000); // interval in milliseconds

    static BridgeMonitor *_instance;
    QString _bridgeName;
    QReadWriteLock _lock;
    int _timerId;
    int _activeTimeLimit; // time limit in seconds
    QHash<QString, bool> _macAddrSeenHash;
    QHash<QPair<QString, int>, QList<MacAddressInformation> > _data;
    QHash<QPair<QString, int>, QObject*> _objectHash;
    QHash<int, QNFlog*> _nflogHash;

};

/*******************************************************************************
* QNFlog
*******************************************************************************/
class QNFlog : public QObject
{
    Q_OBJECT
public:
    QNFlog(int groupId, QObject * parent = 0);
    ~QNFlog();
    bool start();
    void stop();
    QList<MacAddressInformation> getMacAddressInformation(QString bridgeName);
    void addBridge(QString bridgeName);
    void removeBridge(QString bridgeName);
    bool canHandleBridge(QString bridgeName);

private slots:
    void _notifierSlot(int fd);

private:
    static int _QNFlogCallback(const struct nlmsghdr *nlh, void *data);
    nlmsghdr *_nflogBuildConfigRequest(char *buf, uint8_t command, bool useGroupId = false);
    nlmsghdr *_nflogBuildConfigParams(char *buf, uint8_t mode, int range);

    int _groupId;
    QSocketNotifier *_socketNotifier;
    struct mnl_socket *_mnlSocket;
    int _portId;
    QHash<QString, QList<MacAddressInformation> > _data;
};

} // namespace HnsNode

#endif // BRIDGEMONITOR_H
