/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef CAPTUREHANDLER_H
#define CAPTUREHANDLER_H

// Generated hynesim headers
#include "common/capturejoboptions.h"
#include "common/types.h"

// Generated diamesh headers
#include <diamesh/types.h>

// Qt headers
#include <QObject>
#include <QTime>
#include <QReadWriteLock>

class QTimer;

namespace HnsNode {

class EntityI;

class CaptureHandler : public QObject
{
    Q_OBJECT

public:
    CaptureHandler();
    ~CaptureHandler();

    Q_INVOKABLE void init(::HnsNode::EntityI *entity, QString captureJobPath);
    virtual bool setOptions(const Hns::CaptureJobOptions &jobOptions);
    QString getCaptureFileMixedPath();
    virtual bool initCapture() = 0;
    virtual void startCapture() = 0;
    Hns::PcapInfo getPcapInfo();

public slots:
    virtual void endCapture() = 0;

protected:
    virtual QString _getFileNamePrefix() = 0;
    virtual int _getPacketsCount() = 0;
    void _generateFileName();
    bool started();
    void setStarted(bool started);

protected slots:
    virtual void _updateAvailableInterfacesList(::Diamesh::IntList listInterfaces);
    virtual void _updateCaptureProgress();

signals:
    void progressChanged(double progress);

protected:
    void _setEndDateTime();
    long _getEndDateTime();

    EntityI *_entity;
    QString _captureFullPath;
    QString _captureMixedPath;
    Hns::CaptureJobOptions _jobOptions;
    QList<Ice::Int> _availableInterfaces;
    QString _captureFileName;

    QReadWriteLock _finishTimeLock;
    long _startDateTime;
    long _endDateTime;

    QTimer *_captureTimer;
    QTimer *_progressTimer;
    QTime _elapsedTime;
private:
    QReadWriteLock _startedLock;
    bool _started;

protected slots:
    void _progressChangedSlot(double progress);
};

} // namespace HnsNode

#endif // CAPTUREHANDLER_H
