/******************************************************************************
**
** Copyright (C) 2013 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// hynesim node headers
#include "capturehandlerfactory.h"
#include "domaincapturehandler.h"
#include "dynamipsroutercapturehandler.h"
#include "hybridnetcardcapturehandler.h"
#include "kernelbridgecapturehandler.h"
#include "openvswitchcapturehandler.h"
#include "topologygatecapturehandler.h"
#include "wifiapcapturehandler.h"

// Generated hynesim headers
#include "common/domainentry.h"
#include "common/dynamipsrouterentry.h"
#include "common/hybridnetcardentry.h"
#include "common/topologygateentry.h"
#include "common/switchentry.h"
#include "common/wifiapentry.h"
#include "common/initializer.h"

// Qt headers
#include <QHash>

namespace HnsNode {

HNS_STATIC_HASH_INITIALIZER(_captureHandlerHelperHash, CaptureHandlerHelperHash, QString, CaptureHandlerHelper)
CaptureHandlerHelperHash::CaptureHandlerHelperHash()
{
    insert(QString::fromStdString(::Hns::DomainEntry::ice_staticId()), &CaptureHandlerFactory::registerDomainCaptureHandler);
    insert(QString::fromStdString(::Hns::DynamipsRouterEntry::ice_staticId()), &CaptureHandlerFactory::registerDynamipsCaptureHandler);
    insert(QString::fromStdString(::Hns::WifiAPEntry::ice_staticId()), &CaptureHandlerFactory::registerWifiAPCaptureHandler);
    insert(QString::fromStdString(::Hns::HybridNetcardEntry::ice_staticId()), &CaptureHandlerFactory::registerHybridCaptureHandler);
    insert(QString::fromStdString(::Hns::SwitchEntry::ice_staticId()), &CaptureHandlerFactory::registerSwitchCaptureHandler);
    insert(QString::fromStdString(::Hns::TopologyGateEntry::ice_staticId()), &CaptureHandlerFactory::registerTopologyGateCaptureHandler);
}

HNS_STATIC_HASH_INITIALIZER(_switchCaptureHandlerHash, SwitchCaptureHandlerHash, QString, QMetaObject)
SwitchCaptureHandlerHash::SwitchCaptureHandlerHash()
{
    insert("kernelBridge", KernelBridgeCaptureHandler::staticMetaObject);
    insert("openVSwitch", OpenVSwitchCaptureHandler::staticMetaObject);
}

QMetaObject CaptureHandlerFactory::getCaptureHandler(const Hns::EntityEntryPtr &entry)
{
    Q_UNUSED(entry)
    return _captureHandlerHelperHash()->value(QString::fromStdString(entry->ice_id()))(entry);
}

QMetaObject CaptureHandlerFactory::registerDomainCaptureHandler(const Hns::EntityEntryPtr &entry)
{
    Q_UNUSED(entry)
    return DomainCaptureHandler::staticMetaObject;
}

QMetaObject CaptureHandlerFactory::registerDynamipsCaptureHandler(const Hns::EntityEntryPtr &entry)
{
    Q_UNUSED(entry)
    return DynamipsRouterCaptureHandler::staticMetaObject;
}

QMetaObject CaptureHandlerFactory::registerHybridCaptureHandler(const Hns::EntityEntryPtr &entry)
{
    Q_UNUSED(entry)
    return HybridNetcardCaptureHandler::staticMetaObject;
}

QMetaObject CaptureHandlerFactory::registerSwitchCaptureHandler(const Hns::EntityEntryPtr &entry)
{
    Hns::SwitchEntryPtr switchEntry = Hns::SwitchEntryPtr::dynamicCast(entry);
    if (switchEntry) {
        return _switchCaptureHandlerHash()->value(QString::fromStdString(switchEntry->type));
    }
    return QMetaObject();
}

QMetaObject CaptureHandlerFactory::registerWifiAPCaptureHandler(const Hns::EntityEntryPtr &entry)
{
    Q_UNUSED(entry)
    return WifiApCaptureHandler::staticMetaObject;
}

QMetaObject CaptureHandlerFactory::registerTopologyGateCaptureHandler(const Hns::EntityEntryPtr &entry)
{
    Q_UNUSED(entry)
    return TopologyGateCaptureHandler::staticMetaObject;
}

} // namespace HnsNode
