/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// hynesim node headers
#include "capturejobI.h"
#include "nodeentityI.h"
#include "nodeI.h"

#include "pcapcapturehandler.h"
#include "dynamipsroutercapturehandler.h"
#include "domaincapturehandler.h"
#include "wifiapcapturehandler.h"
#include "hybridnetcardcapturehandler.h"
#include "capturehandlerfactory.h"

// Qt headers
#include <QDebug>

namespace HnsNode {

CaptureJobI::CaptureJobI(EntityI *entity, QString captureJobPath)
{
    _setAdapter(Hns::NodeI::instance()->adapter());
    QMetaObject metaObject = CaptureHandlerFactory::getCaptureHandler(entity->getEntry());
    _captureHandler = qobject_cast<CaptureHandler *>(metaObject.newInstance());
    _captureHandler->moveToThread(executionThread());

    _captureHandler->init(entity, captureJobPath);
    QObject::connect(
                _captureHandler, SIGNAL(progressChanged(double)),
                this, SLOT(_updateProgress(double)));
}

CaptureJobI::~CaptureJobI()
{
    delete _captureHandler;
}

bool CaptureJobI::cancel(const::Ice::Current &current)
{
    Q_UNUSED(current)
    _captureHandler->endCapture();
    return true;
}

bool CaptureJobI::setOptions(const Hns::CaptureJobOptions &jobOptions, const Ice::Current &current)
{
    Q_UNUSED(current)
    return _captureHandler->setOptions(jobOptions);
}

std::string CaptureJobI::getCaptureJobFileMixedPath(const Ice::Current &current)
{
    Q_UNUSED(current)
    return _captureHandler->getCaptureFileMixedPath().toStdString();
}

Hns::PcapInfo CaptureJobI::getPcapInfo(const Ice::Current &current)
{
    Q_UNUSED(current)
    return _captureHandler->getPcapInfo();
}

void CaptureJobI::_run()
{
    qDebug() << "CaptureJob::run"; // TODO: Translation
    if (_captureHandler->initCapture())
        _captureHandler->startCapture();
}

} // namespace HnsNode
