/******************************************************************************
**
** Copyright (C) 2013 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// hynesim node headers
#include "kernelbridgecapturehandler.h"
#include "nodeswitchI.h"

// Qt headers
#include <QDebug>

namespace HnsNode {

KernelBridgeCaptureHandler::KernelBridgeCaptureHandler()
{
}

KernelBridgeCaptureHandler::~KernelBridgeCaptureHandler()
{
}

QString KernelBridgeCaptureHandler::_getFileNamePrefix()
{
    SwitchI *switchEntity = qobject_cast<SwitchI *>(_entity);
    QString interfaceName;
    QHash<int, QString> tapHash = switchEntity->getTapHash();
    if (_jobOptions.interfaceList.front() == -1 && _availableInterfaces.contains(-1)) {
        interfaceName = QString("portBridge_all");
    } else {
        for (int portId = 0; portId < tapHash.keys().size(); ++portId) {
            if (portId == _jobOptions.interfaceList.front() && _availableInterfaces.contains(portId))
                interfaceName = QString("portBridge_%1").arg(portId);
        }
    }
    return interfaceName;
}

QString KernelBridgeCaptureHandler::_getInterfaceToCapture()
{
    SwitchI *switchEntity = qobject_cast<SwitchI *>(_entity);
    if (!(_entity->getStatus() == Hns::STARTED)) {
        qCritical() << tr("Libpcap can capture only on interfaces wich are UP so please start the switch");
        return QString();
    }
    QString interfaceName;
    QHash<int, QString> tapHash = switchEntity->getTapHash();
    if (_jobOptions.interfaceList.front() == -1 && _availableInterfaces.contains(-1)) {
        interfaceName = switchEntity->getBridgeName();
    } else {
        for (int portId = 0; portId < tapHash.keys().size(); ++portId) {
            if (portId == _jobOptions.interfaceList.front() &&  _availableInterfaces.contains(portId))
                interfaceName = tapHash.value(portId);
        }
    }
    return interfaceName;
}

} // namespace HnsNode
