/******************************************************************************
**
** Copyright (C) 2013 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// hynesim node headers
#include "kernelbridgeswitchimplementation.h"
#include "nodeentityI.h"
#include "netutils.h"
#include "entityplugI.h"

// Generated hynesim headers
#include "common/switchentry.h"

// Qt headers
#include <QAtomicInt>

namespace HnsNode {

Q_GLOBAL_STATIC_WITH_ARGS(QAtomicInt, _idCounter, (1))

KernelBridgeSwitchImplementation::KernelBridgeSwitchImplementation(EntityI *entity)
    : SwitchImplementation(entity)
    , _mutex(QMutex::Recursive)
{
    int id= _idCounter()->fetchAndAddOrdered(1);
    setBridgeName(QString("hns%1s").arg(QString::number(id)));
}

bool KernelBridgeSwitchImplementation::define()
{
    Hns::SwitchEntryPtr switchEntry = Hns::SwitchEntryPtr::dynamicCast(entityEntry());
    return createBridge(getBridgeName(), HnsNode::Down, switchEntry->isHub);
}

bool KernelBridgeSwitchImplementation::destroy()
{
    QMutexLocker locker(&_mutex);
    return removeBridge(getBridgeName());
}

bool KernelBridgeSwitchImplementation::start()
{
    return setInterfaceState(getBridgeName(), HnsNode::Up);
}

bool KernelBridgeSwitchImplementation::stop()
{
    return setInterfaceState(getBridgeName(), HnsNode::Down);
}

bool KernelBridgeSwitchImplementation::updateEntry(const Hns::EntityEntryPtr &entry)
{
    Q_UNUSED(entry)
    return true;
}

HnsNode::EntityPlugI *KernelBridgeSwitchImplementation::createPlug(Ice::Int port)
{
    if (port > Hns::SwitchEntryPtr::dynamicCast(entityEntry())->ports) {
        return NULL;
    } else {
        SwitchPlug *entityPlug = new SwitchPlug(getBridgeName());
        entityPlug->setPort(port);
        return entityPlug;
    }
}

QList<Ice::Int> KernelBridgeSwitchImplementation::initAvailablePlugsForCapture()
{
    QList<int> availablePlugs = getTapHash().keys();
    availablePlugs.append(Hns::captureAll);
    qSort(availablePlugs.begin(), availablePlugs.end());
    return availablePlugs;
}

void KernelBridgeSwitchImplementation::plug(int port, const QString &tapName)
{
    addPort(port, tapName);
    addToBridge(getBridgeName(), tapName);
    entity()->addInterfaceForCapture(port);
}

void KernelBridgeSwitchImplementation::unplug(int port)
{
    QMutexLocker locker(&_mutex);
    removeFromBridge(getBridgeName(), getTapHash().value(port));
    removePort(port);
    entity()->removeInterfaceForCapture(port);
}

} // namespace HnsNode
