/******************************************************************************
**
** Copyright (C) 2013 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef LIBVIRTNETWORKHELPER_H
#define LIBVIRTNETWORKHELPER_H

// Qt headers
#include <QHostAddress>
#include <QDomDocument>

// Libvirt headers
#include "libvirt/libvirt.h"

class QString;

namespace HnsNode {

class LibvirtNetworkHelper
{
public:
    LibvirtNetworkHelper();
    ~LibvirtNetworkHelper();
    bool init(const QString &address, const QString &bridgeName, const QString &networkName);
    bool defineDHCP();
    bool startDHCP();
    bool destroyDHCP();
    bool undefineDHCP();
    QString addMacAddress(const QString &macAddress);
    void removeMacAddress(const QString &macAddress);
    bool applyRules(const QString &fileName, bool adding);
    QString networkIp() const;
    void setCustomXmlPath(const QString &path);
    void setStaticHostXmlPath(const QString &path);

private:
    QString _generateIP();
    void _generatePlage();
    void _createXML();
    void _importCustomXml();
    void _initStaticHosts();
    bool _isInStaticHosts(const QString &ip);

    QPair<QHostAddress, int> _address;
    QHash<QString, QString> _macAddressHash; // MAC address, IP address
    QHash<QString, QString> _staticHosts;
    int _ipCount;
    QString _bridgeName;
    QString _networkName;
    QString _customXmlPath;
    QString _staticHostsXmlPath;
    virConnectPtr _connection;
    virNetworkPtr _network;
    QDomDocument _xml;
};

} // namespace HnsNode

#endif // LIBVIRTNETWORKHELPER_H
