/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// hynesim node headers
#include "nodeI.h"
#include "nodeconstants.h"

// hynesim common headers
#include "common/hns.h"

// Generated hynesim headers
#include "common/version.h"

// diamesh headers
#include <diamesh/coreapplication.h>
#include <diamesh/constants.h>

// ZeroC Ice headers
#include <Ice/ObjectAdapter.h>

// Qt headers
#include <QDebug>

// C++ std headers
#include "getopt.h"

int main(int argc, char **argv)
{
    option app_options[] = {
        { HnsNode::Constants::GETOPT_PLATFORM_BLACKLIST, required_argument, NULL, HnsNode::Constants::GETOPT_PLATFORM_BLACKLIST_KEY },
        { NULL, 0, NULL, 0 }
    };

    QHash<QString, QString> appConfigHash;
    appConfigHash.insert(Diamesh::Constants::APP_CONFIG_NAME, "hynesim-node");
    appConfigHash.insert(Diamesh::Constants::APP_CONFIG_ORGANIZATION, "diateam");
    Diamesh::CoreApplication * app = Diamesh::CoreApplication::create(argc, argv, app_options, appConfigHash);
    Diamesh::VersionInfo info = Hns::versionInfo();
    QString version=QString("Node: %1 %2 %3  %4 %5  %6 %7 %8").arg(QString("Version:"),QString::fromStdString(info.version),QString::fromStdString(info.versionDescription),QString("Revision:"),QString::fromStdString(info.revision),QString("Build date:"),QString::fromStdString(info.buildDate),QString::fromStdString(info.buildTime));
    qDebug()<<version;
    Hns::loadTranslation(QString("hynesim_"), false);
    Hns::NodeI node;
    node.adapter()->activate();
    if (!node.connectToMaster()){
          qCritical() << QObject::tr("Node: unable to connect to the master");
          return 1;
    }

    return app->exec();
}
