/******************************************************************************
**
** Copyright (C) 2013 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// hynesim node headers
#include "nativewireimplementation.h"
#include "nodeentityI.h"
#include "netutils.h"
#include "nodeI.h"
#include "wireinterfacemanager.h"

// hynesim server headers
#include "libhnsserver/processrunner.h"

// ZeroC Ice headers
#include <Ice/LocalException.h>

// Qt headers
#include <QDebug>

// C++ std headers
#include <signal.h>

namespace HnsNode {

NativeWireImplementation::NativeWireImplementation(Hns::EntityEntryPtr entityEntryPtr)
    : WireImplementation()
{
    _entityEntry = entityEntryPtr;
}

NativeWireImplementation::~NativeWireImplementation()
{
    _fromPlug = 0;
    _toPlug = 0;
}

bool NativeWireImplementation::define(EntityI *entityI)
{
    Hns::WireEntryPtr wireEntry = Hns::WireEntryPtr::dynamicCast(_entityEntry);

    QString fromHost = QString::fromStdString(_fromPlug->getHost());
    QString toHost = QString::fromStdString(_toPlug->getHost());
    if (fromHost != toHost) {
        _fromPlug->createVxlan(entityI->getId(), _toPlug->getHost());
        _toPlug->createVxlan(entityI->getId(), _fromPlug->getHost());
    } else {
        QString fromInterface = QString("hnsw%1a").arg(entityI->getId(), 6, 10, QChar('0'));
        QString toInterface = QString("hnsw%1b").arg(entityI->getId(), 6, 10, QChar('0'));
        _interfaceName = fromInterface;
        Hns::NodeI::instance()->wireInterfaceManager()->createVeth(fromInterface, toInterface);
        _fromPlug->setInterface(QString(fromInterface).toStdString());
        _toPlug->setInterface(QString(toInterface).toStdString());
    }
    _fromPlug->plug();
    _toPlug->plug();

    _applySettings(wireEntry);

    return true;
}

bool NativeWireImplementation::destroy()
{
    try {
        _fromPlug->ice_oneway()->unplug();
    } catch (const Ice::ObjectNotExistException &) {
        qDebug() << tr("Plug 'from' already destroyed");
    }

    try {
        _toPlug->ice_oneway()->unplug();
    } catch (const Ice::ObjectNotExistException &) {
        qDebug() << tr("Plug 'to' already destroyed.");
    }

    Hns::NodeI::instance()->wireInterfaceManager()->deleteInterface(_interfaceName);

    return true;
}

bool NativeWireImplementation::updateEntry(const Hns::EntityEntryPtr &entry)
{
    Hns::WireEntryPtr wireEntry = Hns::WireEntryPtr::dynamicCast(entry);
    _applySettings(wireEntry);
//    return _HnsNode_Wire::_updateEntry(entry); //TODO: old code be careful
    return true;
}

bool NativeWireImplementation::setLinkedEntities(const EntityPrx &fromEntity, Ice::Int fromPort, const EntityPrx &toEntity, Ice::Int toPort)
{
    _fromPlug = fromEntity->getPlug(fromPort);
    _toPlug = toEntity->getPlug(toPort);
    return (_fromPlug && _toPlug);
}

void NativeWireImplementation::_applySettings(const Hns::WireEntryPtr &entry)
{
    _fromPlug->applyProperties(entry->propertyLR);
    _toPlug->applyProperties(entry->propertyRL);
}

} // namespace HnsNode
