/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef NETUTILS_H
#define NETUTILS_H

#include <QtGlobal>

class QString;

namespace HnsNode {

enum InterfaceState
{
    Down = 0,
    Up = 1
};

const char *const VMNET_NAME = "vmnet";

bool createTap(const QString &tapName, InterfaceState state = Up);
bool createAMTap(const QString &tapName, const QString &actionManagerBridge, InterfaceState state = Up);
bool removeTap(const QString &tapName);
bool unplugTap(const QString &tapName);
bool setupTap(const QString &tapName);
bool unsetupTap(const QString &tapName);
bool createBridge(const QString &devName, InterfaceState state = Up, bool isHub = false);
bool removeBridge(const QString &devName);
bool addToBridge(const QString &bridgeName, const QString &devName);
bool setInterfaceState(const QString &ifName, InterfaceState state);
void startSpiceTerm(QString vmName, int port, QString keymap);
void startVncTerm(QString veId, int port, QString type);
void stopVncTerm(QString veId, int port, QString type);
QString generateHnsDevice(void);
bool createVmnet(const QString &vmnet, bool setupIf = true);
bool removeVmnet(const QString &vmnet);
bool createVeth(const QString &firstInterface, const QString &secondInterface);
bool createVxlan(const QString &interfaceName, qlonglong id, const QString &sourceInterface, const QString &dstIp);
bool deleteInterface(const QString &interfaceName);
bool removeFromBridge(const QString &bridgeName, const QString &interfaceName);

} // namespace HnsNode

#endif // NETUTILS_H
