/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef HNS_NODE_H
#define HNS_NODE_H

// hynesim node headers
#include "resourcemanager.h"

// Generated hynesim headers
#include "qiceobj_node.h"
#include "common/master.h"
#include "common/platforminfo.h"

namespace HnsNode {
class NodeMonitorI;
class EntityFactoryI;
class WireInterfaceManager;
} // namespace HnsNode


namespace HnsServer {
class StorageManager;
} //namespace HnsServer

namespace Hns {

class PluginManager;

class NodeI : public _Hns_Node
{
    Q_OBJECT
public:
    NodeI();
    ~NodeI();

    MasterPrx hnsMaster();

    static NodeI *instance();

    HnsNode::EntityFactoryPrx getEntityFactory(const ::Ice::Current &current = Ice::Current());
    Diamesh::VersionInfo getVersionInfo(const ::Ice::Current &current);
    HnsNode::ResourceInfo getResourceInfo(const ::Ice::Current &current);

    HnsNode::EntityFactoryI *entityFactory();
    HnsServer::StorageManager *storageManager();
    HnsNode::ResourceManager *resourceManager();
    QString sharedFolder();

    void emitSignalRefresh(const QString &className);

    Hns::PlatformInfoMap getPlatforms(const ::Ice::Current &current = Ice::Current());
    Hns::PluginManager * pluginManager();
    HnsNode::NodeMonitorI *nodeMonitor();
    HnsNode::NodeMonitorPrx getNodeMonitor(const ::Ice::Current &current = Ice::Current());

    QString vxlanInterface() const;
    QString vxlanHost() const;
    QString actionManagerBridge() const;
    HnsNode::WireInterfaceManager *wireInterfaceManager();
    bool actionManagerBridgeAvailable() const;

protected:
    HnsNode::EntityFactoryI *_entityFactory;
    HnsServer::StorageManager *_storageManager;
    HnsNode::ResourceManager *_resourceManager;
    QString _sharedFolder;
    Hns::PluginManager * _pluginManager;
    HnsNode::NodeMonitorI *_nodeMonitor;
    QString _vxlanInterface;
    QString _vxlanHost;
    QString _actionManagerBridge;
    bool _actionManagerBridgeAvailable;

private slots:
    virtual void _shutdownSlot();

private:
    HnsNode::WireInterfaceManager *_wireInterfaceDeleter;

};

} // namespace Hns

#endif // HNS_NODE_H
