/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef _HNS_NODE_DOMAIN_H
#define _HNS_NODE_DOMAIN_H

// Generated hynesim headers
#include "qiceobj_nodedomain.h"

// Libvirt headers
#include <libvirt/libvirt.h>

// Qt headers
#include <QMap>
#include <QTimer>

namespace HnsNode {

class Netcard;

class DomainI : public _HnsNode_Domain
{
    Q_OBJECT

    friend class PlatformDomain;

public:
    Q_INVOKABLE DomainI();
    ~DomainI();
    virtual void init();
    ::Diamesh::IntList getPlugs(const::Ice::Current &current = ::Ice::Current());
    virDomainPtr virDomain();
    void setVirDomainPtr(virDomainPtr newVirDomainPtr);
    QList<Netcard *> netcards();

protected:
    struct DomainMonitoringInfo {
        qint64 monitoringTimeStamp;
        qulonglong previousCPUTime;
    };

    bool _define(const ::Ice::Current &current = ::Ice::Current());
    bool _destroy(const::Ice::Current &current = ::Ice::Current());
    bool _start(const::Ice::Current &current = ::Ice::Current());
    bool _stop(const::Ice::Current &current = ::Ice::Current());
    bool _save(const::Ice::Current &current = ::Ice::Current());
    bool _restore(const::Ice::Current &current = ::Ice::Current());
    bool _suspend(const::Ice::Current &current = ::Ice::Current());
    bool _resume(const::Ice::Current &current = ::Ice::Current());
    void createMemoryDump_effective(const ::HnsNode::AMD_Domain_createMemoryDumpPtr& amdPtr, const ::std::string &path, const Ice::Current &current = ::Ice::Current());
    bool _createSnapshot(QString xmlDesc);
    bool _revertSnapshot(QString snapshotName);
    bool _deleteSnapshot(QString snapshotName);
    Hns::Snapshot _getSnapshot(QString snapshotName);
    QString _getCurrentSnapshotName();
    EntityPlugI *_createPlug(::Ice::Int port);
    void _updateMonitoringData();
    void _interfaceStateChangedSlot(QString interfaceName, bool stateUp);

protected slots:
    void _checkAliveSlot();

private:
    virDomainPtr _virDomain;
    DomainMonitoringInfo _monitoringInfo;

    QMap<int, Netcard *> _netcardMap;
    QTimer _aliveTimer;
};

} // namespace HnsNode

#endif // _HNS_NODE_DOMAIN_H
