/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// hynesim node headers
#include "nodedynamipsrouterI.h"
#include "platformdynamipsrouter.h"
#include "entityplugI.h"

namespace HnsNode {

DynamipsRouterI::DynamipsRouterI()
    : _plugin(NULL)
    , _routerId(0)
    , _telnetPort(0)
{
    _remoteDisplayInfo.host = getHost();
}

DynamipsRouterI::~DynamipsRouterI()
{
}

bool DynamipsRouterI::_define(const ::Ice::Current &current)
{    
    Q_UNUSED(current)
    if  (!PlatformDynamipsRouter::instance()->defineEntity(this)){
        return false;
    }
    setRemoteDisplayPort(_telnetPort);
    setRemoteDisplayType(Hns::TelnetViewer);
    return true;
}

bool DynamipsRouterI::_destroy(const ::Ice::Current &current)
{
    Q_UNUSED(current)
    return PlatformDynamipsRouter::instance()->undefineEntity(this);
}

bool DynamipsRouterI::_start(const ::Ice::Current &current)
{
    Q_UNUSED(current)
    return PlatformDynamipsRouter::instance()->createEntity(this);
}

bool DynamipsRouterI::_stop(const ::Ice::Current &current)
{
    Q_UNUSED(current)
    return PlatformDynamipsRouter::instance()->destroyEntity(this);
}

bool DynamipsRouterI::_suspend(const ::Ice::Current &current)
{
    Q_UNUSED(current)
    return PlatformDynamipsRouter::instance()->suspendEntity(this);
}

bool DynamipsRouterI::_resume(const ::Ice::Current &current)
{
    Q_UNUSED(current)
    return PlatformDynamipsRouter::instance()->resumeEntity(this);
}

EntityPlugI *DynamipsRouterI::_createPlug(::Ice::Int port)
{
    if(_vectVdeTapPlug.value(port) == NULL)
        return NULL;
    EntityPlugI *entityPlug = new EntityPlugI(_vectVdeTapPlug[port]);
    return entityPlug;
}

std::string DynamipsRouterI::fixIdle(const Ice::Current &current)
{
    Q_UNUSED(current)
    return PlatformDynamipsRouter::instance()->generateIdleEntity(this);
}

std::string DynamipsRouterI::extractConfiguration(const Ice::Current &current)
{
    Q_UNUSED(current)
    return PlatformDynamipsRouter::instance()->extractConfigurationEntity(this);
}

bool DynamipsRouterI::pushConfiguration(const ::std::string &configuration, const Ice::Current &current)
{
    Q_UNUSED(current)
    return PlatformDynamipsRouter::instance()->pushConfigurationEntity(this, QString::fromStdString(configuration));
}

QVector<QString>  DynamipsRouterI::getNetCardTap()
{
    return _vectVdeTapPlug;
}

::Diamesh::IntList DynamipsRouterI::getPlugs(const::Ice::Current &current)
{
    Q_UNUSED(current)
    std::list<int> listPlug;
    for (int i = 0 ; i < _vectVdeTapPlug.size() ; i++) {
        listPlug.push_back(i);
    }
    return listPlug;
}

int DynamipsRouterI::routerId()
{
    return _routerId;
}

int DynamipsRouterI::telnetPort()
{
    return _telnetPort;
}

DynamipsPlugin * DynamipsRouterI::plugin()
{
    return _plugin;
}

void DynamipsRouterI::setPlugin(DynamipsPlugin *plugin)
{
    _plugin = plugin;
}

} // namespace HnsNode
