/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef _HNS_NODE_ENTITY_H_
#define _HNS_NODE_ENTITY_H_

// hynesim node headers
#include "capturejobI.h"

// Generated hynesim headers
#include "qiceobj_nodeentity.h"
#include "common/entityentry.h"
#include "common/snapshot.h"
#include "common/entityplug.h"

namespace HnsNode {

class EntityPlugI;

class EntityI : public _HnsNode_Entity
{
    Q_OBJECT

public:
    enum EntityFlags {
        Monitoring = 0x1,
        CPUMonitoring = 0x2,
        MemoryMonitoring = 0x4,
        NetworkMonitoring = 0x8,
        IOMonitoring = 0x10
    };

    EntityI();
    ~EntityI();
    virtual void init();
    ::Hns::EntityEntryPtr getEntry(const ::Ice::Current &current = ::Ice::Current());
    void setEntry(const ::Hns::EntityEntryPtr &entityEntry);
    bool updateEntry(const ::Hns::EntityEntryPtr &entityEntry, const ::Ice::Current &current);
    bool define(const ::Ice::Current &current = ::Ice::Current());
    bool start(const ::Ice::Current &current = ::Ice::Current());
    bool stop(const ::Ice::Current &current = ::Ice::Current());
    bool save(const ::Ice::Current &current = ::Ice::Current());
    bool restore(const ::Ice::Current &current = ::Ice::Current());
    bool suspend(const ::Ice::Current &current = ::Ice::Current());
    bool resume(const ::Ice::Current &current = ::Ice::Current());
    bool undefine(const ::Ice::Current &current = ::Ice::Current());
    bool createSnapshot(const std::string &xmlDesc, const Ice::Current &current = ::Ice::Current());
    bool revertSnapshot(const std::string &snapshotName, const Ice::Current &current = ::Ice::Current());
    bool deleteSnapshot(const std::string &snapshotName, const Ice::Current &current = ::Ice::Current());
    Hns::Snapshot getSnapshot(const std::string &snapshotName, const Ice::Current &current = ::Ice::Current());
    std::string getCurrentSnapshotName(const Ice::Current &current = ::Ice::Current());

    bool destroy();

    QString path();
    std::string getHost(const ::Ice::Current &current = ::Ice::Current());
    EntityPlugPrx getPlug(::Ice::Int port, const ::Ice::Current &current = ::Ice::Current());
    ::Diamesh::IntList getPlugs(const ::Ice::Current &current = ::Ice::Current());
    ::Diamesh::IntList getAvailablePlugsForCapture(const ::Ice::Current &current = ::Ice::Current());
    void setAvailablePlugsForCapture(const ::Diamesh::IntList &interfacesList, const ::Ice::Current &current = ::Ice::Current());
    void removeInterfaceForCapture(::Ice::Int interface, const ::Ice::Current &current = ::Ice::Current());
    void addInterfaceForCapture(::Ice::Int interface, const ::Ice::Current &current = ::Ice::Current());

    HnsNode::CaptureJobPrx createCaptureJob(const std::string &captureJobPath,const ::Ice::Current &current);
    void setStatus(Hns::Status status);
    Hns::Status getStatus(const ::Ice::Current &current = ::Ice::Current());

    Hns::RemoteDisplayInfo getRemoteDisplayInfo(const ::Ice::Current &current);
    void setRemoteDisplayPort(int remoteDisplayPort);
    int getRemoteDisplayPort();
    void setRemoteDisplayType(int remoteDisplayType);
    int getRemoteDisplayType();
    void setRemoteDisplayPassword(QString password);
    void setRemoteDisplayTls(bool tls);
    void setRemoteDisplayCa(QString ca);
    void setRemoteDisplayCertSubject(QString certSubject);
    QString getRemoteDisplayPassword();
    void setIdentity(Ice::Identity identity);
    virtual void setId(::Ice::Long id, const ::Ice::Current &current = ::Ice::Current());
    virtual ::Ice::Long getId(const ::Ice::Current &current = ::Ice::Current());
    bool hasFlag(EntityFlags flag);
    virtual Hns::EntityMonitoringDataPtr getMonitoringData();

public slots:
    void interfaceStateChangedSlot(QString interfaceName, bool stateUp);

protected:
    virtual EntityPlugI *_createPlug(::Ice::Int port);
    bool _removePlugs();
    virtual bool _checkEntry(const ::Hns::EntityEntryPtr &current);
    virtual bool _updateEntry(const ::Hns::EntityEntryPtr &current);
    virtual bool _define(const ::Ice::Current &current = ::Ice::Current());
    virtual bool _start(const ::Ice::Current &current = ::Ice::Current());
    virtual bool _stop(const ::Ice::Current &current = ::Ice::Current());
    virtual bool _save(const ::Ice::Current &current = ::Ice::Current());
    virtual bool _restore(const ::Ice::Current &current = ::Ice::Current());
    virtual bool _suspend(const ::Ice::Current &current = ::Ice::Current());
    virtual bool _resume(const ::Ice::Current &current = ::Ice::Current());
    virtual bool _destroy(const ::Ice::Current &current = ::Ice::Current());
    virtual bool _createSnapshot(QString xmlDesc);
    virtual bool _revertSnapshot(QString snapshotName);
    virtual bool _deleteSnapshot(QString snaptshotName);
    virtual Hns::Snapshot _getSnapshot(QString snapshotName);
    virtual QString _getCurrentSnapshotName();
    virtual void _initAvailablePlugsForCapture(const ::Ice::Current &current = ::Ice::Current());
    virtual void _interfaceStateChangedSlot(QString interfaceName, bool stateUp);

private:
    QString _getStatusName();

protected:
    Hns::EntityEntryPtr _entityEntry;
    QMap< ::Ice::Int, EntityPlugPrx> _plugs;
    Hns::Status _status;
    QReadWriteLock _entityLocker;
    QList<Ice::Int> _availablePlugsForCapture;
    Hns::RemoteDisplayInfo _remoteDisplayInfo;
    qlonglong _cptStatus;
    Ice::Identity _identity;
    long _id;
    int _flags;
    Hns::EntityMonitoringDataPtr _monitoringData;
};

} // namespace HnsNode

#endif // _HNS_NODE_ENTITY_H_
