/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef _HNS_NODE_ENTITY_FACTORY_H
#define _HNS_NODE_ENTITY_FACTORY_H

// Generated hynesim headers
#include "qiceobj_nodeentityfactory.h"
#include "common/entityentry.h"

namespace HnsNode {

class EntityI;

typedef bool (*BuildChecker) (const::Hns::EntityEntryPtr &, const QList<Hns::EntityEntryPtr> &alreadyDefinedEntities);

struct FactoryHelper
{
    QMetaObject metaObject;
    BuildChecker buildChecker;
};

class EntityFactoryI : public _HnsNode_EntityFactory
{
    Q_OBJECT

public:
    EntityFactoryI();
    ~EntityFactoryI();
    EntityPrx createEntity(const ::Hns::EntityEntryPtr &entry, ::Ice::Long id, const ::Ice::Current &current = ::Ice::Current());
    EntityPrx createEntityFromId(::Ice::Long entryId, ::Ice::Long id, const ::Ice::Current &current = ::Ice::Current());
    bool destroyEntity(const EntityPrx &entityPrx, const Ice::Current &current);
    bool canBuild(const ::Hns::EntityEntryPtr &entityEntry, const ::Ice::Current &current = ::Ice::Current());
    bool canBuildAll(const Hns::EntityEntryPtr &entityEntry, const Hns::EntityEntryList &alreadyDefinedEntities, const Ice::Current &current);

signals:
    void addEntity(long id);
    void entityAdded(EntityI *entity);
    void removeEntity(long entityId);

public:    
    template <typename T1, typename T2> FactoryHelper *registerChecker(BuildChecker buildChecker)
    {
        FactoryHelper *factoryHelper = new FactoryHelper();
        factoryHelper->buildChecker = buildChecker;
        //factoryHelper->cloneJobBuilder = NULL;
        factoryHelper->metaObject = T2::staticMetaObject;
        _helpers.insert(QString::fromStdString(T1::ice_staticId()), factoryHelper);
        _refresh[factoryHelper->metaObject.className()] = QString::fromStdString(T1::ice_staticId());
        return factoryHelper;
    }
    template <typename T> bool unregisterCheckers()
    {
        _helpers.remove(QString::fromStdString(T::ice_staticId()));
        return true;
    }
    template <typename T1, typename T2> FactoryHelper *registerDefaultChecker()
    {
        return registerChecker<T1, T2>(&_defaultChecker<T1>);
    }

    const QString  & getNameKeyChecker(const QString & metaClass) const;

protected:    
    QMultiHash<QString, FactoryHelper*> _helpers;
    QHash<QString, QString> _refresh;
    template <typename T> static bool _defaultChecker(const::Hns::EntityEntryPtr &entityEntry, const QList<Hns::EntityEntryPtr> & = QList<Hns::EntityEntryPtr>())
    {
        return (entityEntry->ice_id() == T::ice_staticId());
    }
    FactoryHelper *_getFactoryHelper(const::Hns::EntityEntryPtr &, const QList<Hns::EntityEntryPtr> &alreadyDefinedEntities = QList<Hns::EntityEntryPtr>());
};

} // namespace HnsNode

#endif // _HNS_NODE_ENTITY_FACTORY_H
