/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// hynesim node headers
#include "nodehybridnetcardI.h"
#include "nodeconstants.h"
#include "platformhybridnetcard.h"
#include "bridgemonitor.h"
#include "entityplugI.h"

// Qt headers
#include <QDebug>

namespace HnsNode {

HybridNetcardI::HybridNetcardI()
{
}

HybridNetcardI::~HybridNetcardI()
{
}

QString HybridNetcardI::interfaceName()
{
    return _interfaceName;
}

bool HybridNetcardI::_define(const ::Ice::Current &current)
{
    Q_UNUSED(current)
    if  (!PlatformHybridNetcard::instance()->defineEntity(this)){
        return false;
    }
    return true;
}

bool HybridNetcardI::_destroy(const ::Ice::Current &current)
{
    Q_UNUSED(current)
    return PlatformHybridNetcard::instance()->undefineEntity(this);
}

bool HybridNetcardI::_start(const ::Ice::Current &current)
{
    Q_UNUSED(current)
    if (!PlatformHybridNetcard::instance()->createEntity(this))
        return false;
    QGenericArgument arg1 = Q_ARG(QString, _bridgeName);
    QGenericArgument arg2 = Q_ARG(int, Constants::SETTINGS_LOG_FILTER_GROUP);
    QGenericArgument arg3 = Q_ARG(QObject *, this);
    QMetaObject::invokeMethod(BridgeMonitor::instance(), "registerEntity", arg1, arg2, arg3);
    return true;
}

bool HybridNetcardI::_stop(const ::Ice::Current &current)
{
    Q_UNUSED(current)
    if (!PlatformHybridNetcard::instance()->destroyEntity(this))
        return false;
    QGenericArgument arg = Q_ARG(QObject *, this);
    QMetaObject::invokeMethod(BridgeMonitor::instance(), "unregisterEntity", arg);
    return true;
}

EntityPlugI *HybridNetcardI::_createPlug(::Ice::Int port)
{
    if (port != 0) {
        qWarning() << tr("The hybrid netcard can only be plugged on port 0. Use 0 instead of %1").arg(port);
        return NULL;
    }
    EntityPlugI *entityPlug = new EntityPlugI(_bridgeName);
    return entityPlug;
}

::Diamesh::IntList HybridNetcardI::getPlugs(const ::Ice::Current &current)
{
    Q_UNUSED(current)
    ::Diamesh::IntList plugList;
    plugList.push_back(0);
    return plugList;
}

Hns::MacAddressMap HybridNetcardI::getMacAddresses(const Ice::Current &current)
{
    Q_UNUSED(current)
    QMap<std::string, bool> macAddrMap;
    QHash<QString, bool> macAddrHash;
    macAddrHash = HnsNode::BridgeMonitor::instance()->getMacAddresses(_bridgeName, Constants::SETTINGS_LOG_FILTER_GROUP);

    for (QHash<QString, bool>::const_iterator it = macAddrHash.begin(); it != macAddrHash.end(); ++it) {
        macAddrMap.insert(it.key().toStdString(), it.value());
    }
    return macAddrMap.toStdMap();
}

} // namespace HnsNode
