/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// hynesim node headers
#include "nodeswitchI.h"
#include "platformswitch.h"
#include "switchimplementation.h"

// Generated hynesim headers
#include "common/switchentry.h"

namespace HnsNode {

SwitchI::SwitchI()
    : _implementation(0)
    , _type("")
{
}

SwitchI::~SwitchI()
{
    _implementation->deleteLater();
}

::Diamesh::IntList SwitchI::getPlugs(const ::Ice::Current &current)
{
    Q_UNUSED(current)
    Hns::SwitchEntryPtr switchEntry = Hns::SwitchEntryPtr::dynamicCast(_entityEntry);
    ::Diamesh::IntList plugList;
    for (int i = 1; i <= switchEntry->ports; i++)
        plugList.push_back(i);
    return plugList;
}

void SwitchI::setImplementation(SwitchImplementation *implementation)
{
    _implementation = implementation;
}

SwitchImplementation *SwitchI::implementation()
{
    return _implementation;
}

void SwitchI::enableCapture(QString captureFilePath, QStringList portsToCapture, int maxPackets, QString pcapFilter)
{
    PlatformSwitch::get(_type)->enableCapture(captureFilePath, portsToCapture, maxPackets, pcapFilter, _implementation);
}

void SwitchI::disableCapture()
{
    return PlatformSwitch::get(_type)->disableCapture(_implementation);
}

int SwitchI::getPacketsCount()
{
    return PlatformSwitch::get(_type)->getPacketsCount(_implementation);
}

bool SwitchI::_define(const ::Ice::Current &current)
{
    Q_UNUSED(current)
    Hns::SwitchEntryPtr switchEntry = Hns::SwitchEntryPtr::dynamicCast(_entityEntry);
    _type = QString::fromStdString(switchEntry->type);
    return PlatformSwitch::get(_type)->defineEntity(this);
}

bool SwitchI::_destroy(const ::Ice::Current &current)
{
    Q_UNUSED(current)
    return PlatformSwitch::get(_type)->undefineEntity(this);
}

bool SwitchI::_start(const ::Ice::Current &current)
{
    Q_UNUSED(current)
    return PlatformSwitch::get(_type)->createEntity(this);
}

bool SwitchI::_stop(const ::Ice::Current &current)
{
    Q_UNUSED(current)
    return PlatformSwitch::get(_type)->destroyEntity(this);
}

EntityPlugI *SwitchI::_createPlug(::Ice::Int port)
{
    return PlatformSwitch::get(_type)->createPlug(this, port);
}

bool SwitchI::_updateEntry(const Hns::EntityEntryPtr &entry)
{
    return PlatformSwitch::get(_type)->updateEntry(entry, _implementation);
}

void SwitchI::_initAvailablePlugsForCapture(const Ice::Current &current)
{
    Q_UNUSED(current)
    _availablePlugsForCapture = PlatformSwitch::get(_type)->initAvailablePlugsForCapture(_implementation);
}

QHash<int, QString> SwitchI::getTapHash()
{
    return _implementation->getTapHash();
}

QString SwitchI::getBridgeName()
{
    return _implementation->getBridgeName();
}

} // namespace HnsNode
