/******************************************************************************
**
** Copyright (C) 2003-2013 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// hynesim node headers
#include "nodetopologygateI.h"
#include "platformtopologygate.h"
#include "entityplugI.h"

// Generated hynesim headers
#include "common/topologygateentry.h"

// Qt headers
#include <QDebug>

namespace HnsNode {

TopologyGateI::TopologyGateI()
{
}

TopologyGateI::~TopologyGateI()
{
}

QString TopologyGateI::getNetworkName()
{
    return _networkName;
}

::Diamesh::IntList TopologyGateI::getPlugs(const Ice::Current &current)
{
    Q_UNUSED(current)
    ::Diamesh::IntList plugList;
    plugList.push_back(0);
    return plugList;
}

EntityPlugI *TopologyGateI::_createPlug(Ice::Int port)
{
    if (port != 0) {
        qWarning() << tr("The Topology bridge can only be plugged on port 0. Use 0 instead of %1").arg(port);
        return NULL;
    }
    return new EntityPlugI(_bridgeName);
}

bool TopologyGateI::_define(const Ice::Current &current)
{
    Q_UNUSED(current)
    Hns::TopologyGateEntryPtr topologyGateEntry = Hns::TopologyGateEntryPtr::dynamicCast(this->getEntry());
    _networkName = QString::fromStdString(topologyGateEntry->networkName);
    return PlatformTopologyGate::instance()->defineEntity(this);
}

bool TopologyGateI::_destroy(const Ice::Current &current)
{
    Q_UNUSED(current)
    return PlatformTopologyGate::instance()->undefineEntity(this);
}

bool TopologyGateI::_start(const Ice::Current &current)
{
    Q_UNUSED(current)
    return PlatformTopologyGate::instance()->createEntity(this);
}

bool TopologyGateI::_stop(const Ice::Current &current)
{
    Q_UNUSED(current)
    return PlatformTopologyGate::instance()->destroyEntity(this);
}

QString TopologyGateI::getBridgeName() const
{
    return _bridgeName;
}

void TopologyGateI::setBridgeName(const QString &bridgeName)
{
    _bridgeName = bridgeName;
}

} // namespace HnsNode
