/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// hynesim node headers
#include "nodewifiapI.h"
#include "platformwifiap.h"
#include "entityplugI.h"

// Qt headers
#include <QDebug>
#include <QCoreApplication>

namespace HnsNode {

WifiAPI::WifiAPI()
    : _HnsNode_WifiAP()
{
    QObject::connect(&_checkPeersTimer,SIGNAL(timeout()),this,SLOT(_checkPeersSlot()));
    _checkPeersTimer.moveToThread(qApp->thread());
}

::Diamesh::StringList WifiAPI::connectedPeers(const ::Ice::Current &current)
{
    Q_UNUSED(current)
    std::list<std::string> output;
    foreach (QString s,_connectedPeers){
        output.push_back(s.toStdString());
    }
    return output;
}

WifiAPI::~WifiAPI()
{
}

::Diamesh::IntList WifiAPI::getPlugs(const ::Ice::Current &current)
{
    Q_UNUSED(current)
    ::Diamesh::IntList plugList;
    plugList.push_back(0);
    return plugList;
}

bool WifiAPI::_define(const::Ice::Current &current)
{
    Q_UNUSED(current)
    return PlatformWifiAp::instance()->defineEntity(this);
}

bool WifiAPI::_destroy(const::Ice::Current &current)
{
    Q_UNUSED(current)
    return PlatformWifiAp::instance()->undefineEntity(this);
}

bool WifiAPI::_start(const::Ice::Current &current)
{
    Q_UNUSED(current)
    _checkPeersTimer.start(10000);
    return PlatformWifiAp::instance()->createEntity(this);
}

bool WifiAPI::_stop(const::Ice::Current &current)
{
    Q_UNUSED(current)
    _checkPeersTimer.stop();
    foreach(QString s,_connectedPeers)
        emit peerLeft(s);
    _connectedPeers.clear();
    return PlatformWifiAp::instance()->destroyEntity(this);
}

void WifiAPI::_checkPeersSlot()
{
    QStringList connectedPeers = PlatformWifiAp::instance()->checkPeers(_ssid);
    foreach (QString s,connectedPeers){
        if (!_connectedPeers.contains(s)){
            qDebug()<<tr("The peer %1 has joined %2").arg(s,_ssid);
            emit peerJoined(s);
        }
    }
    foreach(QString s,_connectedPeers){
        if (!connectedPeers.contains(s)){
            qDebug()<<tr("The peer %1 has left %2").arg(s,_ssid);
            emit peerLeft(s);
        }
    }
    _connectedPeers = connectedPeers;
}

QString WifiAPI::getInterfaceName() const
{
    return _interfaceName;
}

void WifiAPI::setInterfaceName(const QString &interfaceName)
{
    _interfaceName = interfaceName;
}

void WifiAPI::setBridgeName(const QString &bridgeName)
{
    _bridgeName = bridgeName;
}

void WifiAPI::setSsid(const QString &ssid)
{
    _ssid = ssid;
}

QString WifiAPI::getBridgeName() const
{
    return _bridgeName;
}

QString WifiAPI::getSsid() const
{
    return _ssid;
}

EntityPlugI *WifiAPI::_createPlug(::Ice::Int port)
{
    if (port != 0) {
        qWarning() << tr("The wifiAP can only be plugged on port 0. Use 0 instead of %1").arg(port);
        return NULL;
    }
    return new EntityPlugI(getBridgeName());
}

} // namespace HnsNode
