/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// hynesim node headers
#include "nodewireI.h"
#include "platformwire.h"
#include "wireimplementation.h"

// Generated hynesim headers
#include "common/wireentry.h"

namespace HnsNode {

WireI::WireI()
    : _HnsNode_Wire()
    , _wireImplementation(0)
{
}

WireI::~WireI()
{
    _wireImplementation->deleteLater();
}

bool WireI::_define(const ::Ice::Current &current)
{
    Q_UNUSED(current)
    Hns::WireEntryPtr wireEntry = Hns::WireEntryPtr::dynamicCast(getEntry());
    PlatformWire *platform = PlatformWire::get(wireEntry->type);
    return platform->defineEntity(this);
}

bool WireI::_destroy(const ::Ice::Current &current)
{
    Q_UNUSED(current)
    Hns::WireEntryPtr wireEntry = Hns::WireEntryPtr::dynamicCast(getEntry());
    PlatformWire *platform = PlatformWire::get(wireEntry->type);
    return platform->destroyEntity(this);
}

bool WireI::setLinkedEntities(const ::HnsNode::EntityPrx &fromEntity, ::Ice::Int fromPort, const ::HnsNode::EntityPrx &toEntity, ::Ice::Int toPort, const ::Ice::Current &current)
{
    Q_UNUSED(current)
    Hns::WireEntryPtr wireEntry = Hns::WireEntryPtr::dynamicCast(getEntry());
    _fromEntity = fromEntity;
    _fromPort = fromPort;
    _toEntity = toEntity;
    _toPort = toPort;
    PlatformWire *platform = PlatformWire::get(wireEntry->type);
    return platform->setLinkedEntities(this, fromEntity, fromPort, toEntity, toPort);
}

bool WireI::_updateEntry(const ::Hns::EntityEntryPtr &entry)
{
    Hns::WireEntryPtr oldEntry = Hns::WireEntryPtr::dynamicCast(getEntry());
    PlatformWire *oldPlatform = PlatformWire::get(oldEntry->type);
    Hns::WireEntryPtr newEntry = Hns::WireEntryPtr::dynamicCast(entry);
    if (newEntry->type == oldEntry->type) {
        /*Case we don't change the implementation*/
        return oldPlatform->updateEntry(this, entry);
    } else {
        /*Else we want to change the implementation*/
        _destroy();
        _entityEntry = entry;
        PlatformWire *platform = PlatformWire::get(newEntry->type);
        platform->beforeDefineEntity(this);
        platform->setLinkedEntities(this, _fromEntity, _fromPort, _toEntity, _toPort);
        bool ret = platform->defineEntity(this);
        return ret;
    }
}

CaptureJobPrx WireI::createCaptureJob(const std::string &captureJobPath, const Ice::Current &current)
{
    Q_UNUSED(captureJobPath)
    Q_UNUSED(current)
    return HnsNode::CaptureJobPrx();
}

void WireI::setImplementation(WireImplementation *wireImplementation)
{
    _wireImplementation = wireImplementation;
}

WireImplementation *WireI::implementation()
{
    return _wireImplementation;
}

} //HnsNode
