// hynesim node headers
#include "openvswitchcapturehandler.h"
#include "nodeswitchI.h"

// Qt headers
#include <QDebug>

namespace HnsNode {

OpenVSwitchCaptureHandler::OpenVSwitchCaptureHandler()
{
}

OpenVSwitchCaptureHandler::~OpenVSwitchCaptureHandler()
{
}

QString OpenVSwitchCaptureHandler::_getFileNamePrefix()
{
    SwitchI *switchEntity = qobject_cast<SwitchI *>(_entity);
    QString interfaceName;
    QHash<int, QString> tapHash = switchEntity->getTapHash();

    if ((_jobOptions.interfaceList.front() == -1) && _availableInterfaces.contains(-1)) { /*not implemented yet*/
        interfaceName = QString("portBridge_all");
    } else {
        interfaceName = "portBridge";
        foreach (Ice::Int portId, _jobOptions.interfaceList) {
            if (tapHash.contains(portId))
                interfaceName.append(QString("_%1").arg(portId));
        }
    }
    return interfaceName;
}

QString OpenVSwitchCaptureHandler::_getInterfaceToCapture()
{
    SwitchI *switchEntity = qobject_cast<SwitchI *>(_entity);
    QString interfaceName;

    if (!(_entity->getStatus() == Hns::STARTED)) {
        qCritical() << tr("Libpcap can capture only on interfaces wich are UP so please start the switch");
        return QString();
    }

    if ((_jobOptions.interfaceList.front() == -1) && _availableInterfaces.contains(-1)) { /*not implemented yet*/
        interfaceName = "";
    } else {
        /*only handle 1 tap, we take the first*/
        Ice::Int portId = _jobOptions.interfaceList.front();
        QHash<Ice::Int, QString> tapHash = switchEntity->getTapHash();

        if (!tapHash.contains(portId))
            interfaceName = "";
        else
            interfaceName = tapHash.value(portId);
    }
    return interfaceName;
}

} // namespace HnsNode
