#ifndef OPENVSWITCHIMPLEMENTATION_H
#define OPENVSWITCHIMPLEMENTATION_H

// hynesim node headers
#include "switchimplementation.h"

// Qt headers
#include <QHash>
#include <QPair>
#include <QList>

namespace HnsNode {

class OpenVSwitchImplementation : public SwitchImplementation
{
    Q_OBJECT

public:
    explicit OpenVSwitchImplementation(EntityI *entity);

    bool define();
    bool destroy();
    bool start();
    bool stop();
    bool updateEntry(const::Hns::EntityEntryPtr &entityEntry);
    EntityPlugI* createPlug(Ice::Int port);
    QList<Ice::Int> initAvailablePlugsForCapture();

private:
    bool _define();
    bool _destroy();
    bool _start();
    bool _stop();
    bool _updateEntry(const::Hns::EntityEntryPtr &entityEntry);
    EntityPlugI *_createPlug(Ice::Int port);

    void plug(int port, const QString &interfaceName);
    void unplug(int port);

private:
    bool _setPortEnabled(QString portName, bool enabled);
    QHash<int, QPair<int, QList<int> > > _generateVlanHash(const::Hns::EntityEntryPtr &entityEntry);
    void _applyVlan(int port, QPair<int, QList<int> > pair);
    void _applyPortMirroring();
    void _disapplyPortMirroring();

    bool _isStarted;
    QHash<int, QPair<int, QList<int> > > _vlanHash; // QHash<portNumber, QPair<vlanId, QList<vlanTagged> > >
    int _portMirroring;
};

} // namespace HnsNode

#endif // OPENVSWITCHIMPLEMENTATION_H
