/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// hynesim node headers
#include "platform.h"
#include "nodeconstants.h"

// Qt headers
#include <QHash>
#include <QSettings>
#include <QHashIterator>
#include <QDebug>

// diamesh headers
#include <diamesh/diamesh.h>

namespace HnsNode {

typedef QHash<QString, Platform::RegisterPlatformInformation> PlatformCreatorHash;
Q_GLOBAL_STATIC(PlatformCreatorHash, platformCreatorHash)

QList<Hns::PlatformInfo> Platform::_registeredPlatforms = QList<Hns::PlatformInfo>();

Platform::Platform()
    : QObject()
    , _features(0)
{
}

Platform::~Platform()
{
}

void Platform::createPlatforms()
{
    QSettings settings(Diamesh::instance()->configFileName(), QSettings::IniFormat);
    settings.beginGroup(Constants::SETTINGS_PLATFORMS_GROUP);

    QStringList blacklistedPlatform = Diamesh::instance()->parsedOpts()[HnsNode::Constants::GETOPT_PLATFORM_BLACKLIST].split(",", QString::SkipEmptyParts);
    bool blockAll = blacklistedPlatform.contains("all");
    QStringList autorizedPlatforms = settings.childGroups();

    QHashIterator<QString, Platform::RegisterPlatformInformation> iterator(*platformCreatorHash());
    while (iterator.hasNext()) {
        iterator.next();
        QString platformName = iterator.key();
        Hns::PlatformInfo platformInfo;
        platformInfo.name = platformName.toStdString();
        platformInfo.enabled = false;
        platformInfo.error = "";
        if (blockAll || blacklistedPlatform.contains(iterator.key())) {
            platformInfo.error = tr("Platform blacklisted").toStdString();
            _registeredPlatforms.append(platformInfo);
            continue;
        }
        Platform::RegisterPlatformInformation registerPlatformInformation = iterator.value();
        if (registerPlatformInformation.platformAutoLoad == Platform::Autoload
                || (autorizedPlatforms.contains(platformName) && registerPlatformInformation.platformAutoLoad  == Platform::LoadPolicyAccordingToSettings)) {
            Platform *platform = registerPlatformInformation.platformCreator();
            if (platform) {
                // add extended capabilities if any
                QString extendedCapabilities = settings.value(platformName + "/extendedCapabilities").toString();
                platform->_extendedCapabilities.append(extendedCapabilities.split(",", QString::SkipEmptyParts));
                platformInfo.enabled = true;
            }
            else {
                platformInfo.error = tr("Platform blacklisted").toStdString();
            }
            _registeredPlatforms.append(platformInfo);
        }
    }
}

QList<Hns::PlatformInfo> Platform::getPlatforms()
{
    return _registeredPlatforms;
}

bool Platform::registerPlatform(const QString &name, RegisterPlatformInformation info)
{
    platformCreatorHash()->insert(name, info);
    return true;
}

bool Platform::getFeatures(int features)
{
    //return true if _features has the bits of features
    return ((_features & features) == features);
}

int Platform::getEntityFlags()
{
    return 0;
}

} // namespace HnsNode
