/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef _HNS_PLATFORM_H
#define _HNS_PLATFORM_H

// Generated hynesim headers
#include "common/platforminfo.h"

// Qt headers
#include <QObject>
#include <QStringList>

// C++ std headers
#include <stdexcept>

namespace HnsNode {

class EntityI;
class Platform;

typedef Platform * (*PlatformCreator)();

enum Features {
    //Used as flags
    Snapshot = 0x1,
    StateLink = 0x2
};

class Platform : public QObject
{
    Q_OBJECT

public:
    enum PlatformAutoLoad {
        LoadPolicyAccordingToSettings,
        Autoload
    };

    struct RegisterPlatformInformation
    {
        Platform::PlatformAutoLoad platformAutoLoad;
        PlatformCreator platformCreator;
    };

    virtual bool defineEntity(EntityI *entity) = 0;
    virtual bool undefineEntity(EntityI *entity) = 0;
    virtual bool createEntity(EntityI *entity) = 0;
    virtual bool destroyEntity(EntityI *entity) = 0;

    static void createPlatforms();
    static QList<Hns::PlatformInfo> getPlatforms();
    static bool registerPlatform(const QString &name, RegisterPlatformInformation info);
    bool getFeatures(int features);
    virtual int getEntityFlags();

protected:
    Platform();
    virtual ~Platform();
    static QList<Hns::PlatformInfo> _registeredPlatforms;
    QStringList _extendedCapabilities;
    int _features;

};

struct QObjectCastException : public std::runtime_error
{
    QObjectCastException() : std::runtime_error("qobject_cast error") { }
};

template<class T>
struct RegisterPlatform
{
    RegisterPlatform(const QString &name, Platform::PlatformAutoLoad platformAutoLoad = Platform::LoadPolicyAccordingToSettings)
    {
        Platform::RegisterPlatformInformation info;
        info.platformAutoLoad = platformAutoLoad;
        info.platformCreator = &T::create;
        Platform::registerPlatform(name, info);
    }
};

template<class T>
T *tryQObjectCast(QObject *toCast)
{
    T *result = qobject_cast<T*>(toCast);
    if (result == 0) {
        throw HnsNode::QObjectCastException();
    }
    return result;
}

} // namespace HnsNode

#endif // _HNS_PLATFORM_H
