/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef _HNS_PLATFORM_DOMAIN_H
#define _HNS_PLATFORM_DOMAIN_H

// hynesim node headers
#include "platform.h"
#include "nodeentityfactoryI.h"

// Generated hynesim headers
#include "common/domainentry.h"
#include "common/monitordata.h"
#include "common/snapshot.h"

// Qt headers
#include <QMutex>

// Libvirt headers
#include <libvirt/libvirt.h>

// C++ std headers
#include <stdexcept>

class QDomDocument;
class QDomElement;

namespace HnsNode {

class DomainI;
class Netcard;

class PlatformDomain : public Platform
{
    Q_OBJECT

public:
    static PlatformDomain *get(const std::string &domainType);
    bool defineEntity(EntityI *entity);
    bool undefineEntity(EntityI *entity);
    bool createEntity(EntityI *entity);
    bool destroyEntity(EntityI *entity);
    bool saveEntity(EntityI *entity);
    bool restoreEntity(EntityI *entity);
    bool suspendEntity(EntityI *entity);
    bool resumeEntity(EntityI *entity);
    virtual bool createSnapshot(EntityI *entity, QString xmlDesc);
    virtual bool revertSnapshot(EntityI *entity, QString snapshotName);
    virtual bool deleteSnapshot(EntityI *entity, QString snapshotName);
    virtual Hns::Snapshot getSnapshot(EntityI *entity, QString snapshotName);
    virtual QString getCurrentSnapshotName(EntityI *entity);
    int domainStatus(DomainI *domain);
    void createNetcards(DomainI *domain);
    static QString suspendPath(DomainI *domain);
    void initMonitoringInformation(DomainI *domain);
    bool updateMonitoringData(DomainI *domain);
    void setInterfaceUp(DomainI *domain, const QString &devName, bool up);

protected:
    PlatformDomain(const QStringList &domainType, const QString &uri, bool needPooledConnection = false);
    virtual ~PlatformDomain();
    virConnectPtr _openNewConnectionIfNeeded();
    void _cleanupIfNeeded(virConnectPtr connection, DomainI *hnsDomain, virDomainPtr domain);
    virDomainPtr _getVirDomainForConnection(virDomainPtr domain, virConnectPtr connection);
    virtual bool _beforeLibvirtDefine(QDomDocument &xml, DomainI *domain);
    virtual bool _afterLibvirtDefine(DomainI *domain);
    virtual bool _beforeLibvirtCreate(DomainI *domain);
    virtual bool _afterLibvirtCreate(DomainI *domain);
    virtual bool _beforeLibvirtDestroy(DomainI *domain);
    virtual bool _afterLibvirtDestroy(DomainI *domain);
    virtual bool _beforeLibvirtUndefine(DomainI *domain);
    virtual bool _afterLibvirtUndefine(DomainI *domain);
    virtual bool _beforeLibvirtRestore(DomainI *domain);
    virtual bool _afterLibvirtResume(DomainI *domain);
    virtual bool _saveDomain(DomainI *domain);
    virtual bool _restoreDomain(DomainI *domain);
    virtual void _createEntityCleanup();
    virtual Netcard * _createNetcard(const Hns::Netcard &netcardEntry);
    virtual virConnectPtr _getConnection();
    virtual void _closeConnection(virConnectPtr connection);
    bool _addDisks(DomainI *domain, QDomElement &xmlDevices);
    QDomElement _addTextNode(QDomElement &elem, const QString &name, const QString &value);
    int _nextRemoteDisplayPort();
    void _releaseRemoteDisplayPort(int port);
    QString _nextRemoteDisplayPassword();
    void _buildFactoryHelper(BuildChecker buildChecker);
    virtual void _monitorCpu(DomainI *domain, Hns::DomainMonitoringDataPtr data);
    virtual void _monitorMemory(DomainI *domain, Hns::DomainMonitoringDataPtr data);
    virtual void _monitorNetwork(DomainI *domain, Hns::DomainMonitoringDataPtr data);
    virtual void _monitorIO(DomainI *domain, Hns::DomainMonitoringDataPtr data, Hns::DomainEntryPtr domainEntry);
    static bool canBuild(const ::Hns::EntityEntryPtr &entityEntry, const QList<Hns::EntityEntryPtr> &alreadyDefinedEntities = QList<Hns::EntityEntryPtr>());
    bool _needPooledConnection;
    bool _closeTemporaryConnections;
    static const char *saveFileName;
    QMutex _connectionMutex;

private:
    void _registerPlatformDomain(const QString &domainType);
    Hns::SnapshotList _sortSnapshotListForChildrenFirst(Hns::SnapshotList);
    Hns::SnapshotList _sortSnapshotListForParentFirst(Hns::SnapshotList);
    bool _contains(Hns::SnapshotList snapList, QString name);
    void _refreshSnapshotList(DomainI *domain);
    virConnectPtr _connection;
    QString _uri;
    int _hostNbCpu;
};

struct LibvirtConnectException : public std::runtime_error
{
    LibvirtConnectException() : std::runtime_error("Libvirt connection error") { }
};

} // namespace HnsNode

#endif // _HNS_PLATFORM_DOMAIN_H
