/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef HNSNODE_PLATFORMDYNAMIPSROUTER_H
#define HNSNODE_PLATFORMDYNAMIPSROUTER_H

// hynesim node headers
#include "hnsnodeplugins.h"
#include "platform.h"
#include "nodeentityfactoryI.h"

// Generated hynesim headers
#include "common/entityentry.h"

// Qt headers
#include <QList>
#include <QReadWriteLock>
#include <QMutex>

namespace HnsNode {

class EntityI;
class TelnetWrapper;
class DynamipsRouterI;

/*******************************************************************************
* DynamipsPlugin
*/
class DynamipsPlugin : public Plugin
{
    Q_OBJECT
    Q_INTERFACES(HnsNode::Plugin)

public:
    DynamipsPlugin(QObject *parent = 0);
    virtual bool canBuild(const ::Hns::EntityEntryPtr &entityEntry = 0) = 0;
    bool define(DynamipsRouterI *dynamipsRouter);
    bool undefine(DynamipsRouterI *dynamipsRouter);
    bool create(DynamipsRouterI *dynamipsRouter);
    bool destroy(DynamipsRouterI *dynamipsRouter);
    bool suspend(DynamipsRouterI *dynamipsRouter);
    bool resume(DynamipsRouterI *dynamipsRouter);

protected:
    virtual bool _preDefine(DynamipsRouterI *dynamipsRouter);
    virtual bool _define(DynamipsRouterI *dynamipsRouter);
    virtual bool _postDefine(DynamipsRouterI *dynamipsRouter);
    virtual bool _preUndefine(DynamipsRouterI *dynamipsRouter);
    virtual bool _undefine(DynamipsRouterI *dynamipsRouter);
    virtual bool _postUndefine(DynamipsRouterI *dynamipsRouter);
    virtual bool _preCreate(DynamipsRouterI *dynamipsRouter);
    virtual bool _create(DynamipsRouterI *dynamipsRouter);
    virtual bool _postCreate(DynamipsRouterI *dynamipsRouter);
    virtual bool _preDestroy(DynamipsRouterI *dynamipsRouter);
    virtual bool _destroy(DynamipsRouterI *dynamipsRouter);
    virtual bool _postDestroy(DynamipsRouterI *dynamipsRouter);
    virtual bool _preSuspend(DynamipsRouterI *dynamipsRouter);
    virtual bool _suspend(DynamipsRouterI *dynamipsRouter);
    virtual bool _postSuspend(DynamipsRouterI *dynamipsRouter);
    virtual bool _preResume(DynamipsRouterI *dynamipsRouter);
    virtual bool _resume(DynamipsRouterI *dynamipsRouter);
    virtual bool _postResume(DynamipsRouterI *dynamipsRouter);
    void _addVdeTapPlug(DynamipsRouterI *dynamipsRouterI);
    QString _getTapPlugName(DynamipsRouterI *dynamipsRouterI, int plugId);
    void _clearVdeTapPlug(DynamipsRouterI *dynamipsRouterI);
    void _deleteRomFile(DynamipsRouterI *dynamipsRouterI);
};

class PlatformDynamipsRouter : public Platform
{
    Q_OBJECT

public:
    static Platform *create();
    static PlatformDynamipsRouter *instance();
    static bool canBuild(const::Hns::EntityEntryPtr &entityEntry, const QList<Hns::EntityEntryPtr> &alreadyDefinedEntities = QList<Hns::EntityEntryPtr>());
    bool defineEntity(EntityI *entity);
    bool undefineEntity(EntityI *entity);
    bool createEntity(EntityI *entity);
    bool destroyEntity(EntityI *entity);
    bool suspendEntity(EntityI *entity);
    bool resumeEntity(EntityI *entity);
    std::string generateIdleEntity(EntityI *entity);
    std::string extractConfigurationEntity(EntityI *entity);
    bool pushConfigurationEntity(EntityI *entity, const QString &configuration);
    TelnetWrapper *getTelnet(DynamipsRouterI *dynamipsRouterI);

protected:
    explicit PlatformDynamipsRouter();
    ~PlatformDynamipsRouter();
    void _buildFactoryHelper(BuildChecker buildChecker);

private:
    static PlatformDynamipsRouter *_instance;
    QReadWriteLock _lock;
    int _portDynamips;
    int _nextRouterId;
    QList<DynamipsPlugin *> _dynamipsPluginList;
    QMutex _telnetLocker;
};

} // namespace HnsNode

Q_DECLARE_INTERFACE(HnsNode::DynamipsPlugin, "org.hynesim.DynamipsPlugin")

#endif // HNSNODE_PLATFORMDYNAMIPSROUTER_H
