/******************************************************************************
**
** Copyright (C) 2013 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// hynesim node headers
#include "platformkernelbridgeswitch.h"
#include "nodeentityfactoryI.h"
#include "nodeconstants.h"
#include "nodeI.h"
#include "nodeswitchI.h"
#include "kernelbridgeswitchimplementation.h"

// Generated hynesim headers
#include "common/switchentry.h"

// Qt headers
#include <QDebug>

// C++ std headers
#include <stdexcept>

namespace HnsNode {

static RegisterPlatform<PlatformKernelBridgeSwitch> registerMe(Constants::SETTINGS_KERNEL_BRIDGE_SWITCH_PLATFORM_GROUP, Platform::Autoload);
PlatformKernelBridgeSwitch *PlatformKernelBridgeSwitch::_instance = 0;

Platform *PlatformKernelBridgeSwitch::create()
{
    if (!_instance) {
        try {
        _instance = new PlatformKernelBridgeSwitch();
        return _instance;
        } catch (const std::runtime_error &) {
            qCritical() << tr("Failed to create KernelBridgeSwitch platform");
        }
    }
    return NULL;
}

PlatformKernelBridgeSwitch *PlatformKernelBridgeSwitch::instance()
{
    return _instance;
}

bool PlatformKernelBridgeSwitch::canBuild(const::Hns::EntityEntryPtr &entry, const QList<Hns::EntityEntryPtr> &alreadyDefinedEntities)
{
    Q_UNUSED(alreadyDefinedEntities)
    return PlatformSwitch::canBuildSwitchEntry(entry, "kernelBridge");
}

bool PlatformKernelBridgeSwitch::updateEntry(const Hns::EntityEntryPtr &entry, SwitchImplementation *implementation)
{
    return tryQObjectCast<KernelBridgeSwitchImplementation>(implementation)->updateEntry(entry);
}

PlatformKernelBridgeSwitch::PlatformKernelBridgeSwitch()
    : PlatformSwitch(QStringList("kernelBridge"))
{
    Hns::NodeI::instance()->entityFactory()->registerChecker<Hns::SwitchEntry, SwitchI>(&PlatformKernelBridgeSwitch::canBuild);
}

PlatformKernelBridgeSwitch::~PlatformKernelBridgeSwitch()
{
}

void PlatformKernelBridgeSwitch::_beforeDefineSwitch(EntityI *entity)
{
    SwitchI *switchEntity = tryQObjectCast<SwitchI>(entity);
    KernelBridgeSwitchImplementation *implementation = new KernelBridgeSwitchImplementation(entity);
//    connect(switchEntity, SIGNAL(destroyed()), implementation, SLOT(deleteLater()));
    implementation->moveToThread(entity->thread());
    switchEntity->setImplementation(implementation);
}

QList<Ice::Int> PlatformKernelBridgeSwitch::initAvailablePlugsForCapture(SwitchImplementation *implementation)
{
    return tryQObjectCast<KernelBridgeSwitchImplementation>(implementation)->initAvailablePlugsForCapture();
}

} // namespace HnsNode
