/******************************************************************************
**
** Copyright (C) 2013 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// hynesim node headers
#include "platformnativewire.h"
#include "nativewireimplementation.h"
#include "nodeconstants.h"
#include "nodedomainI.h"
#include "nodewireI.h"

// Qt headers
#include <QDebug>

// C++ std headers
#include <stdexcept>

namespace HnsNode {

static RegisterPlatform<PlatformNativeWire> registerMe(Constants::SETTINGS_NATIVE_WIRE_PLATFORM_GROUP, Platform::Autoload);

PlatformNativeWire *PlatformNativeWire::_instance = 0;

Platform *PlatformNativeWire::create()
{
    if (!_instance) {
        try {
            _instance = new PlatformNativeWire();
            return _instance;
        } catch (const std::runtime_error &) {
            qCritical() << tr("Failed to create NativeWire platform");
        }
    }
    return NULL;
}

PlatformNativeWire *PlatformNativeWire::instance()
{
    return _instance;
}

PlatformNativeWire::PlatformNativeWire()
    : PlatformWire(QStringList("nativeWire"))
{
}

PlatformNativeWire::~PlatformNativeWire()
{
}

void PlatformNativeWire::beforeDefineEntity(EntityI *entity)
{
    WireI *wire = tryQObjectCast<WireI>(entity);
    NativeWireImplementation *nativeWireImplementation = new NativeWireImplementation(entity->getEntry());
    nativeWireImplementation->moveToThread(entity->thread());
    wire->setImplementation(nativeWireImplementation);
}

} // namespace HnsNode
