// hynesim node headers
#include "platformopenvswitch.h"
#include "nodeentityfactoryI.h"
#include "nodeconstants.h"
#include "nodeI.h"
#include "nodeswitchI.h"
#include "openvswitchimplementation.h"

// Generated hynesim headers
#include "common/switchentry.h"

// Qt headers
#include <QDebug>

// C++ std headers
#include <stdexcept>

namespace HnsNode {

static RegisterPlatform<PlatformOpenVSwitch> registerMe(Constants::SETTINGS_OPEN_V_SWITCH_PLATFORM_GROUP, Platform::Autoload);
PlatformOpenVSwitch *PlatformOpenVSwitch::_instance = 0;

Platform *PlatformOpenVSwitch::create()
{
    if (!_instance) {
        try {
            _instance = new PlatformOpenVSwitch();
            return _instance;
        } catch (const std::runtime_error &) {
            qCritical() << tr("Failed to create OpenVSwitch platform");
        }
    }
    return NULL;
}

PlatformOpenVSwitch *PlatformOpenVSwitch::instance()
{
    return _instance;
}

bool PlatformOpenVSwitch::canBuild(const::Hns::EntityEntryPtr &entry, const QList<Hns::EntityEntryPtr> &alreadyDefinedEntities)
{
    Q_UNUSED(alreadyDefinedEntities)
    return PlatformSwitch::canBuildSwitchEntry(entry, "openVSwitch");
}

bool PlatformOpenVSwitch::updateEntry(const Hns::EntityEntryPtr &entry, SwitchImplementation *implementation)
{
    return tryQObjectCast<OpenVSwitchImplementation>(implementation)->updateEntry(entry);
}

QList<Ice::Int> PlatformOpenVSwitch::initAvailablePlugsForCapture(SwitchImplementation *implementation)
{
    return tryQObjectCast<OpenVSwitchImplementation>(implementation)->initAvailablePlugsForCapture();
}

PlatformOpenVSwitch::PlatformOpenVSwitch()
    : PlatformSwitch(QStringList("openVSwitch"))
{
    Hns::NodeI::instance()->entityFactory()->registerChecker<Hns::SwitchEntry, SwitchI>(&PlatformOpenVSwitch::canBuild);
}

PlatformOpenVSwitch::~PlatformOpenVSwitch()
{
}

void PlatformOpenVSwitch::_beforeDefineSwitch(EntityI *entity)
{
    SwitchI *switchEntity = tryQObjectCast<SwitchI>(entity);
    OpenVSwitchImplementation *implementation = new OpenVSwitchImplementation(switchEntity);
    connect(switchEntity, SIGNAL(destroyed()), implementation, SLOT(deleteLater()));
    implementation->moveToThread(entity->thread());
    switchEntity->setImplementation(implementation);
}

} // namespace HnsNode
