/******************************************************************************
**
** Copyright (C) 2013 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// hynesim node headers
#include "platformswitch.h"
#include "switchimplementation.h"
#include "nodeentityI.h"
#include "nodeswitchI.h"

#include "common/switchentry.h"

// Qt headers
#include <QHash>

namespace HnsNode {

typedef QHash<QString, PlatformSwitch*> PlatformSwitchHash;
Q_GLOBAL_STATIC(PlatformSwitchHash, platformSwitchHash)

PlatformSwitch::PlatformSwitch(const QStringList &switchType)
{
    foreach (const QString &_switchType, switchType)
        _registerPlatformSwitch(_switchType);
}

PlatformSwitch::~PlatformSwitch()
{
}

PlatformSwitch *PlatformSwitch::get(const QString &switchType)
{
    return platformSwitchHash()->value(switchType);
}

bool PlatformSwitch::defineEntity(EntityI *entity)
{
    _beforeDefineSwitch(entity);
    return tryQObjectCast<SwitchI>(entity)->implementation()->define();
}

bool PlatformSwitch::undefineEntity(EntityI *entity)
{
    return tryQObjectCast<SwitchI>(entity)->implementation()->destroy();
}

bool PlatformSwitch::createEntity(EntityI *entity)
{
    return tryQObjectCast<SwitchI>(entity)->implementation()->start();
}

bool PlatformSwitch::destroyEntity(EntityI *entity)
{
    return tryQObjectCast<SwitchI>(entity)->implementation()->stop();
}

bool PlatformSwitch::updateEntry(const::Hns::EntityEntryPtr &entry, SwitchImplementation *implementation)
{
    Q_UNUSED(entry)
    Q_UNUSED(implementation)
    return false;
}

QList<Ice::Int> PlatformSwitch::initAvailablePlugsForCapture(SwitchImplementation *implementation)
{
    Q_UNUSED(implementation)
    return QList<Ice::Int>();
}

void PlatformSwitch::enableCapture(QString captureFilePath, QStringList portsToCapture, int maxPackets, QString pcapFilter, SwitchImplementation *implementation)
{
    Q_UNUSED(captureFilePath)
    Q_UNUSED(portsToCapture)
    Q_UNUSED(maxPackets)
    Q_UNUSED(pcapFilter)
    Q_UNUSED(implementation)
}

void PlatformSwitch::disableCapture(SwitchImplementation *implementation)
{
    Q_UNUSED(implementation)
}

int PlatformSwitch::getPacketsCount(SwitchImplementation *implementation)
{
    Q_UNUSED(implementation)
    return 0;
}

void PlatformSwitch::_beforeDefineSwitch(EntityI *entity)
{
    Q_UNUSED(entity)
}

bool PlatformSwitch::canBuildSwitchEntry(const Hns::EntityEntryPtr &entityEntry, const QString &switchType)
{
    Hns::SwitchEntryPtr switchEntry = Hns::SwitchEntryPtr::dynamicCast(entityEntry);

    if (!switchEntry)
        return false;

    if (QString::fromStdString(switchEntry->type) != switchType)
        return false;

    return true;
}

void PlatformSwitch::_registerPlatformSwitch(const QString &switchType)
{
    platformSwitchHash()->insert(switchType, this);
}

EntityPlugI *PlatformSwitch::createPlug(EntityI *entity, Ice::Int port)
{
    return tryQObjectCast<SwitchI>(entity)->implementation()->createPlug(port);
}

} // namespace HnsNode
