/******************************************************************************
**
** Copyright (C) 2013 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef PLATFORMSWITCH_H
#define PLATFORMSWITCH_H

// hynesim node headers
#include "platform.h"

// Generated hynesim headers
#include "common/entityentry.h"

// Qt headers
#include <QStringList>

namespace HnsNode {

class EntityI;
class SwitchImplementation;
class EntityPlugI;

class PlatformSwitch : public Platform
{
    Q_OBJECT

public:
    static PlatformSwitch *get(const QString &switchType);
    virtual bool defineEntity(EntityI *entity);
    virtual bool undefineEntity(EntityI *entity);
    virtual bool createEntity(EntityI *entity);
    virtual bool destroyEntity(EntityI *entity);
    virtual bool updateEntry(const::Hns::EntityEntryPtr &entry, SwitchImplementation *implementation);
    virtual QList<Ice::Int> initAvailablePlugsForCapture(SwitchImplementation *implementation);
    virtual void enableCapture(QString captureFilePath, QStringList portsToCapture, int maxPackets, QString pcapFilter, SwitchImplementation *implementation);
    virtual void disableCapture(SwitchImplementation *implementation);
    virtual int getPacketsCount(SwitchImplementation *implementation);
    HnsNode::EntityPlugI *createPlug(EntityI *entity, Ice::Int port);

protected:
    PlatformSwitch(const QStringList &switchType);
    virtual ~PlatformSwitch();
    virtual void _beforeDefineSwitch(EntityI *entity);

    static bool canBuildSwitchEntry(const::Hns::EntityEntryPtr &entityEntry, const QString &switchType);

private:
    void _registerPlatformSwitch(const QString &switchType);
};

} // namespace HnsNode
#endif // PLATFORMSWITCH_H
