/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef _HNS_PLATFORM_VBOX_H
#define _HNS_PLATFORM_VBOX_H

// hynesim node headers
#include "platformdomain.h"

// hynesim common headers
#include "common/poolhandler.h"

// Generated hynesim headers
#include "common/entityentry.h"
#include "common/snapshot.h"
#include "common/monitordata.h"

// Libvirt headers
#include "libvirt/libvirt.h"

// Qt headers
#include <QMetaObject>

class QDomDocument;

namespace HnsNode {

class EntityI;
class DomainI;

template <class T>
class LibvirtVboxConnectionPoolHandler : public Hns::PoolHandler<T>
{
public:
    LibvirtVboxConnectionPoolHandler(double nmax);
    virtual ~LibvirtVboxConnectionPoolHandler();
    virtual T create();
    virtual void closeMethod(T item);
};

class PlatformVBox : public PlatformDomain
{
    Q_OBJECT

public:
    static Platform *create();
    static PlatformVBox *instance();
    virtual Hns::Snapshot getSnapshot(EntityI *entity, QString snapshotName);
    virtual int getEntityFlags();

protected:
    PlatformVBox();
    ~PlatformVBox();
    bool _beforeLibvirtDefine(QDomDocument &xml, DomainI *domain);
    bool _afterLibvirtDefine(DomainI *domain);
    bool _afterLibvirtUndefine(DomainI *domain);
    bool _saveDomain(DomainI *domain);
    bool _restoreDomain(DomainI *domain);
    virtual virConnectPtr _getConnection();
    virtual void _closeConnection(virConnectPtr connection);
    virtual void _monitorMemory(DomainI *domain, Hns::DomainMonitoringDataPtr data);
    static bool canBuild(const ::Hns::EntityEntryPtr &entityEntry, const QList<Hns::EntityEntryPtr> &alreadyDefinedEntities = QList<Hns::EntityEntryPtr>());
    static PlatformVBox *_instance;
    LibvirtVboxConnectionPoolHandler<virConnectPtr>* _poolHandler;

private:
    void _cleanNetworkInterfaces(const QStringList &interfaces);
};

} // namespace HnsNode

#endif // _HNS_PLATFORM_VBOX_H
