/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef PLATFORMWIFIAP_H
#define PLATFORMWIFIAP_H

// hynesim node headers
#include "platform.h"

// Generated hynesim headers
#include "common/entityentry.h"

// Qt headers
#include <QStringList>
#include <QHash>
#include <QAtomicInt>
#include <QReadWriteLock>

namespace HnsNode {

class EntityI;
class WifiAPI;

class PlatformWifiAp : public Platform
{
    Q_OBJECT
public:
    static Platform *create();
    static PlatformWifiAp *instance();
    static bool canBuild(const::Hns::EntityEntryPtr &entityEntry, const QList<Hns::EntityEntryPtr> &alreadyDefinedEntities = QList<Hns::EntityEntryPtr>());

    bool defineEntity(EntityI *entity);
    bool undefineEntity(EntityI *entity);
    bool createEntity(EntityI *entity);
    bool destroyEntity(EntityI *entity);
    QStringList checkPeers(QString ssid);

protected :
    PlatformWifiAp();
    ~PlatformWifiAp();
    bool _loadWifiDevices();

private :
    static PlatformWifiAp *_instance;
    QStringList _ssidList;
    QStringList _macAdressList;
    QHash<QString,QString> _ssidInterface;
    QReadWriteLock _locker;
    QStringList _wifiDevices;
    QStringList _availableWifiDevices;
    QHash<QString, int> _maxSsidPerDevicesHash;
    QHash<QString, int> _busyDevicesHash; /*<device, numberOfSsidDefinedOnThisDevice>*/
    QHash<QString, QString> _ssidDevicesHash;/*<ssid, deviceOnWhichTheSsidIsDefined>*/
    int _maxSsidPerNode;
    QAtomicInt _singleCounter;
};

} // namespace HnsNode
#endif // PLATFORMWIFIAP_H
