/******************************************************************************
**
** Copyright (C) 2013 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// hynesim node headers
#include "platformwire.h"
#include "nodeentityI.h"
#include "nodewireI.h"
#include "wireimplementation.h"

// Qt headers
#include <QHash>

namespace HnsNode {

typedef QHash<QString, PlatformWire*> PlatformWireHash;
Q_GLOBAL_STATIC(PlatformWireHash, platformWireHash)

PlatformWire *PlatformWire::get(const std::string &wireType)
{
    return platformWireHash()->value(QString::fromStdString(wireType));
}

PlatformWire::PlatformWire(const QStringList &wireType)
{
    foreach (QString type, wireType) {
        _registerPlatformWire(type);
    }
}

PlatformWire::~PlatformWire()
{
}

bool PlatformWire::defineEntity(EntityI *entity)
{
    return tryQObjectCast<WireI>(entity)->implementation()->define(entity);
}

bool PlatformWire::undefineEntity(EntityI *entity)
{
    Q_UNUSED(entity)
    return true;
}

bool PlatformWire::createEntity(EntityI *entity)
{
    Q_UNUSED(entity)
    return true;
}

bool PlatformWire::destroyEntity(EntityI *entity)
{
return tryQObjectCast<WireI>(entity)->implementation()->destroy();
}

bool PlatformWire::updateEntry(EntityI *entity, const::Hns::EntityEntryPtr &entityEntry)
{
    return tryQObjectCast<WireI>(entity)->implementation()->updateEntry(entityEntry);
}

bool PlatformWire::setLinkedEntities(EntityI *entity, const ::HnsNode::EntityPrx &fromEntity, Ice::Int fromPort, const::HnsNode::EntityPrx &toEntity, Ice::Int toPort)
{
    beforeDefineEntity(entity);
    return tryQObjectCast<WireI>(entity)->implementation()->setLinkedEntities(fromEntity, fromPort, toEntity, toPort);
}

void PlatformWire::beforeDefineEntity(EntityI *entity)
{
    Q_UNUSED(entity)
}

void PlatformWire::_registerPlatformWire(const QString &wireType)
{
    platformWireHash()->insert(wireType, this);
}

} // namespace HnsNode
