/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef QPCAP_H
#define QPCAP_H

// Qt headers
#include <QObject>
#include <QString>
#include <QMutex>

// external headers
#include <pcap.h>

// C++ std headers
#include <sys/time.h>

class QPcapHeader
{
public:
    QPcapHeader( const struct pcap_pkthdr *header );
    QPcapHeader();
    ~QPcapHeader();

    bool isValid() const;

    timeval timeStamp() const;
    uint capturedLength() const;
    uint packetLength() const;
    const struct pcap_pkthdr* pcap_header() const;

private:
    const struct pcap_pkthdr *header;
    friend class QPcap;
};

class QPcap : public QObject
{
    Q_OBJECT
public:
    QPcap( QObject *parent=0 );
    ~QPcap();

    bool isValid() const;

    QString errorString() const;

    QString lookupDevice();

    bool open( const QString &dev, int snaplen=65536, bool promisc=true );

    bool readPacket();

    void start();
    void stop();

    bool setFilter( const QString &filter );

    bool isBlocking() const;
    void setBlocking( bool enable );

    QPcapHeader header() const;
    const uchar *packet() const;
    pcap_t *handle() const;

signals:
    void packetReady();
    void packetReady( const uchar *packet );
    void packetReady( QPcapHeader header, const uchar *packet );
    void pcapClosed();

private slots:
    void dataAvailable();
    bool close();


private:
    static void packet_callback( uchar *self, const struct pcap_pkthdr *header, const uchar *packet );

private:
    QMutex _mutex;
    struct QPcapPrivate *d;
};

#endif // QPCAP_H
