/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// hynesim node headers
#include "resourcemanager.h"
#include "nodeconstants.h"
#include "platformhybridnetcard.h"

// diamesh headers
#include <diamesh/diamesh.h>

// Qt headers
#include <QSettings>
#include <QHashIterator>

namespace HnsNode {

ResourceManager::ResourceManager(QObject *parent)
    : QObject(parent)
{
    QSettings settings(Diamesh::instance()->configFileName(), QSettings::IniFormat, this);
    settings.beginGroup(Constants::SETTINGS_NODE_GROUP);
    _resourceInfo.maxResources.cpu = settings.value("resources/maxCpu", "2").toInt();
    _resourceInfo.maxResources.memory = settings.value("resources/maxMemory", "256").toInt();

    PlatformHybridNetcard *hybridNetcardsPlatform = PlatformHybridNetcard::instance();
    if (hybridNetcardsPlatform) {
        QHashIterator<QString, QString> iter(hybridNetcardsPlatform->getHybridNetCards());
        while (iter.hasNext()) {
            iter.next();
            _resourceInfo.hybridNetcardsMap.insert(std::pair<std::string, std::string>(iter.key().toStdString(), iter.value().toStdString()));
        }
    }
}

void ResourceManager::addResource(Hns::Resource resource)
{
    _resourceInfo.currentResources.cpu += resource.cpu;
    _resourceInfo.currentResources.memory += resource.memory;
    _resourceInfo.definedEntities++;
    emit resourcesChanged(_resourceInfo);
}

void ResourceManager::removeResource(Hns::Resource resource)
{
    _resourceInfo.currentResources.cpu -= resource.cpu;
    _resourceInfo.currentResources.memory -= resource.memory;
    _resourceInfo.definedEntities--;
    emit resourcesChanged(_resourceInfo);
}

HnsNode::ResourceInfo ResourceManager::getResourceInfo()
{
    return _resourceInfo;
}

} // namespace HnsNode
