/******************************************************************************
**
** Copyright (C) 2013 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// hynesim node headers
#include "switchimplementation.h"
#include "nodeentityI.h"

// Generated hynesim headers
#include "common/switchentry.h"

namespace HnsNode {

SwitchImplementation::SwitchImplementation(EntityI *entity)
    : _entityEntry(entity->getEntry())
    , _entity(entity)
{
}

SwitchImplementation::~SwitchImplementation()
{
}

bool SwitchImplementation::define()
{
    return false;
}

bool SwitchImplementation::destroy()
{
    return false;
}

bool SwitchImplementation::start()
{
    return false;
}

bool SwitchImplementation::stop()
{
    return false;
}

::Diamesh::IntList SwitchImplementation::getPlugs()
{
    Hns::SwitchEntryPtr switchEntry = Hns::SwitchEntryPtr::dynamicCast(_entityEntry);

    ::Diamesh::IntList plugList;
    for (int i = 1; i <= switchEntry->ports; ++i)
        plugList.push_back(i);
    return plugList;
}

QHash<int, QString> SwitchImplementation::getTapHash()
{
    QReadLocker locker(&_tapHashLock);
    return _tapHash;
}

QString SwitchImplementation::getBridgeName()
{
    return _bridgeName;
}

Hns::EntityEntryPtr SwitchImplementation::entityEntry()
{
    return _entityEntry;
}

void SwitchImplementation::addPort(int port, const QString &tapName)
{
    QWriteLocker locker(&_tapHashLock);
    _tapHash.insert(port, tapName);
}

void SwitchImplementation::removePort(int port)
{
    QWriteLocker locker(&_tapHashLock);
    _tapHash.remove(port);
}

void SwitchImplementation::setBridgeName(const QString &bridgeName)
{
    _bridgeName = bridgeName;
}

EntityI *SwitchImplementation::entity()
{
    return _entity;
}

} // namespace HnsNode
