/******************************************************************************
**
** Copyright (C) 2013 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef SWITCHIMPLEMENTATION_H
#define SWITCHIMPLEMENTATION_H

// Generated hynesim headers
#include "common/entityentry.h"

// diamesh headers
#include <diamesh/types.h>

// Qt headers
#include <QHash>
#include <QObject>
#include <QReadWriteLock>

namespace HnsNode {

class EntityI;
class EntityPlugI;

class SwitchImplementation : public QObject
{
    Q_OBJECT

public:
    SwitchImplementation(EntityI *entity);
    virtual ~SwitchImplementation();

    virtual bool define();
    virtual bool destroy();
    virtual bool start();
    virtual bool stop();
    virtual EntityPlugI *createPlug(::Ice::Int port) = 0;

    virtual ::Diamesh::IntList getPlugs();
    virtual QHash<int, QString> getTapHash();
    virtual QString getBridgeName() /*const*/;

    virtual void plug(int port, const QString &tapName) = 0;

public slots:
    virtual void unplug(int port) = 0;

protected:
    Hns::EntityEntryPtr entityEntry();
    void addPort(int port, const QString &tapName);
    void removePort(int port);
    void setBridgeName(const QString &bridgeName);
    EntityI *entity();

private:
    Hns::EntityEntryPtr _entityEntry;
    QReadWriteLock _tapHashLock;
    QHash<int, QString> _tapHash; //port, TapName
    QString _bridgeName;
    EntityI *_entity;

};

} // namespace HnsNode
#endif // SWITCHIMPLEMENTATION_H
