/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

// hynesim node headers
#include "telnetwrapper.h"

// Qt headers
#include <QRegExp>
#include <QTcpSocket>
#include <QtDebug>

namespace HnsNode {

TelnetWrapper::TelnetWrapper(QObject *parent)
    : QObject(parent)
    , _socket(new QTcpSocket())
{
}

TelnetWrapper::~TelnetWrapper()
{
    if (_socket->isOpen())
        _socket->close();
    delete _socket;
}

bool TelnetWrapper::connectToHost(const QString &host, int port)
{
    _socket->connectToHost(host, port);
    return _socket->isOpen();
}

QByteArray TelnetWrapper::send(const QString &command, bool longWait)
{
    return send(command.toLocal8Bit(), longWait);
}

QByteArray TelnetWrapper::send(const QByteArray &command, bool longWait)
{
    Q_UNUSED(longWait)
    qDebug() << "QByteArray TelnetWrapper::send " <<command; // TODO: Translation
    static QRegExp errorRx("^2[0-9][0-9]-");

    _socket->write(command);
    _socket->write("\n");
    emit dataSent(command);

    QByteArray data;
    if (!_socket->waitForReadyRead(50000)) {
        throw DynamipsError("No response from server");
    }

    QByteArray buf;
    while(1) {
        QByteArray chunk = _socket->readAll();
        if (chunk.isEmpty()) {
            qDebug() << QObject::tr("waiting for more data...");
            if (_socket->waitForReadyRead(10000))
                continue;
            throw DynamipsError("Incomplete or empty message received");
        }
        buf += chunk;

        // if the buffer doesn't end in '\n' then we can't be done
        if (!buf.endsWith("\n"))
            continue;

        QList<QByteArray> dataLines = buf.split('\n');
        if (dataLines.last().isEmpty())
            dataLines.removeLast();
        if (dataLines.last().startsWith("100-")) {
            data = buf;
            break;
        }

        QString lastLine(dataLines.last());
        if (errorRx.indexIn(lastLine) != -1) {
            emit dataReceived(buf);
            throw DynamipsError("ERROR:" + lastLine);
        }
    }
    emit dataReceived(data);
    return data;
}

bool TelnetWrapper::isConnect()
{
    return _socket->isOpen();
}

} // namespace HnsNode
