/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#include <diamesh/coreapplication.h>
#include <diamesh/diamesh.h>
#include <diamesh/sessionhelper.h>
#include <diamesh/sessionManager.h>
#include <diamesh/utils.h>
#include <QtCore/QDebug>
#include <QtCore/QUuid>
#include <common/entityentry.h>
#include <common/usermanager.h>

#include "client.h"

int main(int argc, char **argv)
{
    Diamesh::CoreApplication * app = Diamesh::CoreApplication::create(argc, argv, NULL);
    Q_UNUSED(app);
    Diamesh::SessionHelperFactory sessionFactory;

    QSharedPointer<Diamesh::SessionHelper> sessionHelper = sessionFactory.connect();

    if(!sessionHelper)
    {
        qCritical() << QObject::tr("The session should have been created, as the password is right.");
        return 1;
    }

    Hns::Client client(sessionHelper.data());
    Hns::EntityCatalogPrx catalog = client.entityCatalog();
    if (!catalog) {
        qCritical() << QObject::tr("The catalog has not been created");
        return 1;
    }

    Hns::MasterPrx master = client.master();
    Diamesh::UserManagerPrx userManager = master->getUserManager();

    Ice::Long myUserId = userManager->getUserIdFromName(sessionHelper->userName().toStdString());
    //Diamesh::UserPtr user = userManager->getUser(folder1.ownerId);
    //bool addUserToGroup(::Ice::Long userId, ::Ice::Long groupId, const::Ice::Current &);
    ::Diamesh::LongList myGroupIds;

    Diamesh::UserPtr user = userManager->getUser(myUserId);
    if (!user) {
        qCritical() << QObject::tr("Unable to get user %1").arg(myUserId);
        return 1;
    }

    Hns::UserPtr hnsUser = Hns::UserPtr::dynamicCast(user);
    if (!hnsUser) {
        qCritical() << QObject::tr("User %1 is not an Hns user").arg(myUserId);
        return 1;
    }

    hnsUser->name="john";
    hnsUser->firstname="john";
    hnsUser->lastname="doe";
    Ice::Long newUserId = userManager->addUser(hnsUser);
    if (!newUserId) {
        qCritical() << QObject::tr("unable to add new user");
        return 1;
    }

    Hns::UserPtr johnUser = Hns::UserPtr::dynamicCast(userManager->getUser(newUserId));
    if (!johnUser) {
        qCritical() << QObject::tr("unable to retrieve newly added Hns user");
        return 1;
    }

    if (johnUser->firstname != hnsUser->firstname || johnUser->lastname != hnsUser->lastname) {
        qCritical() << QObject::tr("new user first name or last name was not correctly returned");
        return 1;
    }

    if (!userManager->updateUser(johnUser)) {
        qCritical() << QObject::tr("Unable to update user");
        return 1;
    }

    // Folders

    Hns::Folder folder1;
    folder1.shortName="mysql";
    folder1.id = catalog->addFolder(folder1);

    if (!folder1.id) {
        qCritical() << QObject::tr("Failed to add a folder");
        return 1;
    }

    // Entries

    Hns::EntryPtr entry1 = new Hns::EntityEntry();
    entry1->uuid = QUuid::createUuid().toString().toStdString();
    entry1->shortName="mysql";

    if (!catalog->addEntry(entry1)) {
        qCritical() << QObject::tr("Failed to add an entry");
        return 1;
    }

    return 0;
}
