/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#include <diamesh/coreapplication.h>
#include <diamesh/sessionhelper.h>
#include <diamesh/diamesh.h>
#include <diamesh/sessionManager.h>
#include <QtCore/QDebug>
#include <QtCore/QUuid>

#include "client.h"
#include "common/domainentry.h"
#include "common/entityentry.h"
#include "common/tagmanager.h"
#include "testcallback.h"
#include "common/proxies_entitycatalog.h"

int main(int argc, char **argv)
{
    Diamesh::CoreApplication * app = Diamesh::CoreApplication::create(argc, argv, NULL);
    Q_UNUSED(app);

    Diamesh::SessionHelperFactory factory;
    QSharedPointer<Diamesh::SessionHelper> sessionHelper = factory.connect();
    if(!sessionHelper)
    {
        qCritical() << QObject::tr("The session should have been created, as the password is right.");
        return 1;
    }

    Hns::Client client(sessionHelper.data());
    Hns::EntityCatalogPrx catalog = client.entityCatalog();
    if (!catalog) {
        qCritical() << QObject::tr("The catalog has not been created");
        return 1;
    }

    Hns::TagManagerPrx tagManager = client.master()->getTagManager();

    // Folders

    int baseFoldersNumber = catalog->getFolders(0).size();

    if (!baseFoldersNumber) {
        qCritical() << QString(QObject::tr("There should already be 1 folder in catalog. %1 found")).arg(baseFoldersNumber);
        return 1;
    }


    Hns::Folder folder1;
    folder1.shortName="mysql";
    folder1.longName="mysql servers";
    folder1.tagList.push_back("linux");
    folder1.tagList.push_back("server");
    folder1.tagList.push_back("john");

    folder1.id = catalog->addFolder(folder1);

    if (!folder1.id) {
        qCritical() << QObject::tr("Failed to add a folder");
        return 1;
    }

    Hns::Folder folder2;
    folder2.shortName="desktops";
    folder2.longName="desktop machines";
    folder2.tagList.push_back("windows");
    folder2.tagList.push_back("linux");
    folder2.tagList.push_back("desktop");
    folder2.tagList.push_back("john");

    folder2.id = catalog->addFolder(folder2);

    if (!folder2.id) {
        qCritical() << QObject::tr("Failed to add a folder");
        return 1;
    }

    int newFoldersNumber = catalog->getFolders(0).size();
    if (newFoldersNumber != (baseFoldersNumber+2)) {
        qCritical() << QObject::tr("The catalog did not returned an increased folder number (old: %1, new: %2).").arg(baseFoldersNumber).arg(newFoldersNumber);
        return 1;
    }

    ::Diamesh::StringList foldersTags = tagManager->getTags();
    if (foldersTags.size() != 5) {
        qCritical() << QObject::tr("There should be 5 folder tags in the catalog. %1 found.").arg(foldersTags.size());
        return 1;
    }

    /*::Diamesh::StringList folderTags = tagManager->getTags("folders", folder2.id);
    if (folderTags.size() != 4) {
        qCritical() << QObject::tr("There should be 4 folder tags for folder %1 in the catalog. %2 found.").arg(folder2.id).arg(folderTags.size());
        return 1;
    }*/

    if (!tagManager->removeTag(folder1.id, "linux")) {
        qCritical() << QObject::tr("Error while deleting the folder %1 tag %2").arg(folder2.id).arg("linux");
        return 1;
    }

    /*folderTags = tagManager->getFolderTags(folder1.id);
    if (folderTags.size() != 2) {
        qCritical() << QObject::tr("There should be 2 folder tags remaining for folder %1 in the catalog. %2 found.").arg(folder1.id).arg(folderTags.size());
        return 1;
    }*/

    foldersTags = tagManager->getTags();
    if (foldersTags.size() != 5) {
        qCritical() << QObject::tr("There should be still 5 folder tags in the catalog. %1 found.").arg(foldersTags.size());
        return 1;
    }

    foldersTags = tagManager->getTags();
    if (foldersTags.size() != 5) {
        qCritical() << QObject::tr("There should be still 5 folder tags in the catalog. %1 found.").arg(foldersTags.size());
        return 1;
    }

    folder1.shortName="desktops2";
    folder1.longName="desktop machines2";
    folder1.tagList.remove("john");
    folder1.tagList.push_back("paul");

    if (!catalog->updateFolder(folder1)) {
        qCritical() << QObject::tr("Error while updating the folder %1").arg(folder1.id);
        return 1;
    }

    Hns::Folder folder3;
    folder3.shortName="desktops 32b";
    folder3.parentId = folder2.id;

    folder3.id = catalog->addFolder(folder3);

    if (!folder3.id) {
        qCritical() << QObject::tr("Failed to add folder 3");
        return 1;
    }

    Hns::Folder folder4;
    folder4.shortName="desktops 32b windows";
    folder4.parentId = folder3.id;

    folder4.id = catalog->addFolder(folder4);

    if (!folder4.id) {
        qCritical() << QObject::tr("Failed to add folder 4");
        return 1;
    }

    if (!catalog->deleteFolder(folder3.id)) {
        qCritical() << QObject::tr("Failed to delete folder 3");
        return 1;
    }

    QList<Ice::Long> folderList = QList<Ice::Long>::fromStdList(catalog->getFolders(0));

    if (folderList.contains(folder3.id)) {
        qCritical() << QObject::tr("Error: the folder %1 has been removed but is still present").arg(folder3.id);
        return 1;
    }

    folder4 = catalog->getFolder(folder4.id);
    if (folder4.parentId != folder2.id) {
        qCritical() << QObject::tr("As folder 4's parent was removed, its current parrent should now be folder 2 (grand parent)");
        return 1;
    }

    // Entries

    int baseEntriesNumber = catalog->getEntries().size();

    Hns::EntityEntryPtr entry1 = new Hns::EntityEntry();
    entry1->uuid = QUuid::createUuid().toString().toStdString();
    entry1->shortName="mysql";
    entry1->longName="mysql front server";
    entry1->tagList.push_back("linux");
    entry1->tagList.push_back("server");
    entry1->tagList.push_back("john");

    qulonglong entry1id = catalog->addEntry(entry1);
    if (!entry1id) {
        qCritical() << QObject::tr("Failed to add an entry");
        return 1;
    }

    entry1->id = entry1id;

    Hns::EntryPtr entry2 = new Hns::EntityEntry();
    entry2->shortName="admin";
    entry2->longName="admin desktop";
    entry2->tagList.push_back("windows");
    entry2->tagList.push_back("linux");
    entry2->tagList.push_back("desktop");
    entry2->tagList.push_back("john");

    qulonglong entry2id = catalog->addEntry(entry2);
    if (entry2id) {
        qCritical() << QObject::tr("The entry should not have been added, as it has no UUID");
        return 1;
    }

    entry2->uuid = QUuid::createUuid().toString().toStdString();

    entry2id = catalog->addEntry(entry2);
    if (!entry2id) {
        qCritical() << QObject::tr("Failed to add an entry");
        return 1;
    }

    entry2->id = entry2id;

    int newEntriesNumber = catalog->getEntries().size();
    if (newEntriesNumber != (baseEntriesNumber+2)) {
        qCritical() << QObject::tr("The catalog did not returned an increased entry number (old: %1, new: %2).").arg(baseEntriesNumber).arg(newEntriesNumber);
        return 1;
    }

    ::Diamesh::StringList entriesTags = tagManager->getTags();
    if (entriesTags.size() != 6) {
        qCritical() << QObject::tr("There should be 6 entry tags in the catalog. %1 found.").arg(entriesTags.size());
        return 1;
    }

    /*::Diamesh::StringList entryTags = tagManager->getEntryTags(entry2->uuid);
    if (entryTags.size() != 4) {
        qCritical() << QObject::tr("There should be 4 entry tags for entry %1 in the catalog. %2 found.").arg(entry2id).arg(entryTags.size());
        return 1;
    }*/

    if (!tagManager->removeTag(entry1->id, "linux")) {
        qCritical() << QObject::tr("Error while deleting the entry %1 tag %2").arg(entry2id).arg("linux");
        return 1;
    }

    /*entryTags = tagManager->getEntryTags(entry1->uuid);
    if (entryTags.size() != 2) {
        qCritical() << QObject::tr("There should be 2 entry tags remaining for entry %1 in the catalog. %2 found.").arg(entry1id).arg(entryTags.size());
        return 1;
    }*/

    entriesTags = tagManager->getTags();
    if (entriesTags.size() != 6) {
        qCritical() << QObject::tr("There should be still 6 entry tags in the catalog. %1 found.").arg(entriesTags.size());
        return 1;
    }

    if (!catalog->deleteEntry(entry2)) {
        qCritical() << QObject::tr("Error while deleting the entry %1").arg(entry2id);
        return 1;
    }

    entriesTags = tagManager->getTags();
    if (entriesTags.size() != 6) {
        qCritical() << QObject::tr("There should be still 6 entry tags in the catalog. %1 found.").arg(entriesTags.size());
        return 1;
    }

    entry1->shortName="admin2";
    entry1->longName="admin machine 2";
    entry1->tagList.remove("john");
    entry1->tagList.push_back("paul");
    entry1->folderList.push_back(folder1.id);
    entry1->folderList.push_back(folder2.id);

    if (!catalog->updateEntry(entry1)) {
        qCritical() << QObject::tr("Error while updating the entry %1").arg(entry1id);
        return 1;
    }

    entry1->folderList.push_back(folder1.id);

    if (!catalog->updateEntry(entry1)) {
        qCritical() << QObject::tr("Error while updating the entry %1").arg(entry1id);
        return 1;
    }

    entry1 = catalog->getEntityEntry(entry1->id);
    if (entry1->folderList.size() > 2) {
        qCritical() << QObject::tr("Entry1 should have 2 folders ( %1 found)").arg(entry1->folderList.size());
        return 1;
    }

    Ice::Long dummyFolderId = 42;
    entry1->folderList.push_back(dummyFolderId);
    if (catalog->updateEntry(entry1)) {
        qCritical() << QObject::tr("The folder %1 does not exist. The catalog should not be able to update the entry").arg(dummyFolderId);
        return 1;
    }

    entry1->folderList.clear();

    entry1->folderList.push_back(folder4.id);

    if (!catalog->updateEntry(entry1)) {
        qCritical() << QObject::tr("Error while updating the entry %1").arg(entry1id);
        return 1;
    }

    if (!catalog->deleteFolder(folder4.id)) {
        qCritical() << QObject::tr("Failed to delete folder 4");
        return 1;
    }

    // check that the entry folder has been updated
    entry1 = catalog->getEntityEntry(entry1->id);

    if (!entry1->folderList.size()) {
        qCritical() << QObject::tr("The entity should still have a folder");
        return 1;
    }
    if (entry1->folderList.front() == folder4.id) {
        qCritical() << QObject::tr("The entity folder should have changed, as the previous (%1) does not exist any more").arg(folder4.id);
        return 1;
    }
    if (entry1->folderList.front() != folder4.parentId) {
        qCritical() << QObject::tr("The entity folder should be the previous parent's parent (%1) instead of %2").arg(folder4.parentId).arg(entry1->folderList.front());
        return 1;
    }

    // check that it is not possible to set a folder as its own parent or grand parent

    Hns::Folder folder5;
    folder5.shortName = "myFolder5";

    folder5.id = catalog->addFolder(folder5);
    if (!folder5.id) {
        qCritical() << QObject::tr("Failed to add Folder5");
        return 1;
    }

    Hns::Folder folder6;
    folder6.shortName = "myFolder6";
    folder6.parentId = folder5.id;

    folder6.id = catalog->addFolder(folder6);
    if (!folder6.id) {
        qCritical() << QObject::tr("Failed to add Folder6");
        return 1;
    }

    folder5.parentId = folder5.id;
    if (catalog->updateFolder(folder5)) {
        qCritical() << QObject::tr("Folder5 should not have been updated, as it can not be its own parent.");
        return 1;
    }

    folder5.parentId = folder6.id;
    if (catalog->updateFolder(folder5)) {
        qCritical() << QObject::tr("Folder5 should not have been updated, as it can not be its own grand parent.");
        return 1;
    }

    // test asynchronous multiple folder removal
    Hns::Folder folder7;
    folder7.parentId = folder6.id;
    folder7.id = catalog->addFolder(folder7);
    if (!folder7.id) {
        qCritical() << QObject::tr("Failed to add Folder7");
        return 1;
    }

    Hns::Folder folder8;
    folder8.parentId = folder7.id;
    folder8.id = catalog->addFolder(folder8);
    if (!folder8.id) {
        qCritical() << QObject::tr("Failed to add Folder8");
        return 1;
    }

    Hns::Folder folder9;
    folder9.parentId = folder8.id;
    folder9.id = catalog->addFolder(folder9);
    if (!folder9.id) {
        qCritical() << QObject::tr("Failed to add folder9");
        return 1;
    }

    Hns::Folder folder10;
    folder10.parentId = folder9.id;
    folder10.id = catalog->addFolder(folder10);
    if (!folder10.id) {
        qCritical() << QObject::tr("Failed to add folder10");
        return 1;
    }

    Hns::Folder folder11;
    folder11.parentId = folder10.id;
    folder11.id = catalog->addFolder(folder11);
    if (!folder11.id) {
        qCritical() << QObject::tr("Failed to add folder11");
        return 1;
    }

    Hns::Folder folder12;
    folder12.parentId = folder11.id;
    folder12.id = catalog->addFolder(folder12);
    if (!folder12.id) {
        qCritical() << QObject::tr("Failed to add folder12");
        return 1;
    }

    Hns::Folder folder13;
    folder13.parentId = folder12.id;
    folder13.id = catalog->addFolder(folder13);
    if (!folder13.id) {
        qCritical() << QObject::tr("Failed to add folder13");
        return 1;
    }

    TestCallback *tcb = new TestCallback();

    Hns::EntityCatalogQPrx catalogPrx(catalog, client.session()->adapter());
    catalogPrx->begin_deleteFolder(folder7.id, tcb, SLOT(folderRemoved(bool)));
    catalogPrx->begin_deleteFolder(folder12.id, tcb, SLOT(folderRemoved(bool)));
    catalogPrx->begin_deleteFolder(folder13.id, tcb, SLOT(folderRemoved(bool)));
    catalogPrx->begin_deleteFolder(folder6.id, tcb, SLOT(folderRemoved(bool)));
    catalogPrx->begin_deleteFolder(folder8.id, tcb, SLOT(folderRemoved(bool)));
    catalogPrx->begin_deleteFolder(folder9.id, tcb, SLOT(folderRemoved(bool)));
    catalogPrx->begin_deleteFolder(folder10.id, tcb, SLOT(folderRemoved(bool)));
    catalogPrx->begin_deleteFolder(folder11.id, tcb, SLOT(folderRemoved(bool)));
    tcb->startLoop();

    // test entry properties retrieval
    Hns::EntryPropertiesManagerPrx propertiesManager = client.master()->getEntryPropertiesManager();

    Hns::EntryPtr entry = new Hns::Entry();
    Hns::EntryPropertiesPtr properties = propertiesManager->getProperties(entry);
    if (properties) {
        qCritical() << QObject::tr("There should not be any properties for Hns:Entry");
        return 1;
    }

    Hns::EntityEntryPtr entityEntry = new Hns::EntityEntry();
    properties = propertiesManager->getProperties(entityEntry);
    if (!properties) {
        qCritical() << QObject::tr("Unable to get entity entry properties");
        return 1;
    }

    Hns::DomainEntryPtr domainEntry = new Hns::DomainEntry();
    properties = propertiesManager->getProperties(domainEntry);
    if (!properties) {
        qCritical() << QObject::tr("Unable to get domain entry properties");
        return 1;
    }

    return 0;
}
