/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#include <diamesh/coreapplication.h>
#include <diamesh/sessionhelper.h>
#include <diamesh/diamesh.h>
#include <diamesh/sessionManager.h>
#include <QtCore/QDebug>

#include "client.h"
#include "common/domainentry.h"
#include "entityI.h"

int main(int argc, char **argv)
{
    Diamesh::CoreApplication * app = Diamesh::CoreApplication::create(argc, argv, NULL);
    Q_UNUSED(app);

    Diamesh::SessionHelperFactory factory;
    QSharedPointer<Diamesh::SessionHelper> sessionHelper = factory.connect();
    if(!sessionHelper)
    {
        qCritical() << QObject::tr("The session should have been created, as the password is right.");
        return 1;
    }

    Hns::Client client(sessionHelper.data());
    Hns::EntityCatalogPrx catalog = client.entityCatalog();
    if (!catalog) {
        qCritical() << QObject::tr("The catalog has not been created");
        return 1;
    }

    Hns::EntityFactoryPrx entityFactory = client.master()->getEntityFactory();

    Hns::EntityEntryPtr entityEntry = new Hns::EntityEntry();
    Hns::EntityPrx entityProxy = entityFactory->createEntity(entityEntry);

    if (entityProxy) {
        qCritical() << QObject::tr("The entity should not have been created, since the EntityEntry class is not registered with the factory.");
        return 1;
    }

    qulonglong entityId = 0;
    entityProxy = entityFactory->createEntityFromId(entityId);

    if (entityProxy) {
        qCritical() << QObject::tr("The entity should not have been created, since the entity id %1 does not exist in the catalog.").arg(entityId);
        return 1;
    }

    entityId = 1;
    entityProxy = entityFactory->createEntityFromId(entityId);

    if (entityProxy) {
        qCritical() << QObject::tr("The entity should not have been created, since the entity found in the database does not match a class registered with the factory.");
        return 1;
    }

    entityId = 3;
    entityProxy = entityFactory->createEntityFromId(entityId);

    if (!entityProxy) {
        qCritical() << QObject::tr("The entity should have been created, since the entity id %1 does exist in the catalog.").arg(entityId);
        return 1;
    }

    Hns::EntityPrx entityProxy2 = entityFactory->createEntityFromId(entityId);

    if (entityProxy2) {
        qCritical() << QObject::tr("The entity should not have been created, since the entity is already created.");
        return 1;
    }

    if (!entityProxy) {
        qCritical() << QObject::tr("The entity should have been created, since the entity id %1 does exist in the catalog.").arg(entityId);
        return 1;
    }

    Hns::DomainEntryPtr domainEntry = new Hns::DomainEntry();
    domainEntry->longName = "long name test";
    Hns::EntityPrx domainProxy = entityFactory->createEntity(domainEntry);
    if (!domainProxy) {
        qCritical() << QObject::tr("The factory could not create the entity");
        return 1;
    }

    Hns::DomainEntryPtr returnDomainEntry = Hns::DomainEntryPtr::dynamicCast(domainProxy->getEntry());
    if (domainEntry->longName != returnDomainEntry->longName) {
        qCritical() << QObject::tr("The names between the input and output entries are different.");
        return 1;
    }


    return 0;
}
