/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#include "nodeI.h"
#include "common/domainentry.h"
#include "entityI.h"
#include "nodedomainI.h"
#include <diamesh/coreapplication.h>
#include <diamesh/diamesh.h>
#include <QUuid>
#include <QtCore/QDebug>
#include "common/wireentry.h"
#include <Ice/ObjectAdapter.h>

int main(int argc, char **argv)
{
    Diamesh::CoreApplication * app = Diamesh::CoreApplication::create(argc, argv, NULL);
    Q_UNUSED(app);
    Hns::NodeI node;
    node.adapter()->activate();

    Hns::DomainEntryPtr domainEntry = new Hns::DomainEntry();
    domainEntry->longName = "long name test";
    domainEntry->type = "foo";
    domainEntry->uuid = QUuid::createUuid().toString().mid(1, 36).toStdString();
    domainEntry->path = "Test Catalog:/" + domainEntry->uuid;
    domainEntry->vcpu = 1;
    Hns::Disk disk;
    disk.type = "file";
    disk.device = "disk";
    disk.source = "dummy";
    disk.target = "sda";
    disk.targetBus = "sata";
    domainEntry->disks.push_back(disk);

    HnsNode::EntityFactoryI *entityFactory = node.entityFactory();

    HnsNode::EntityPrx domainProxy = entityFactory->createEntity(domainEntry,0);
    if (domainProxy) {
        qDebug() << QObject::tr("The factory should not have been able to create the entity because the domain type is invalid");
        return 1;
    }

    domainEntry->type = "vbox";
    domainProxy = entityFactory->createEntity(domainEntry,0);
    if (!domainProxy) {
        qDebug() << QObject::tr("The factory should have created the entity because it knows how to build VBox domains");
        return 1;
    }

    if (!domainProxy->define()) {
        qDebug() << QObject::tr("Defining the domain failed");
        return 1;
    }

    if (!entityFactory->destroyEntity(domainProxy, Ice::Current())) {
        qDebug() << QObject::tr("Destroying domain failed");
        return 1;
    }

    return 0;
}
