/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#include <diamesh/coreapplication.h>
#include <diamesh/sessionhelper.h>
#include <diamesh/diamesh.h>
#include <diamesh/sessionManager.h>
#include <diamesh/utils.h>
#include <QtCore/QDebug>
#include <QtCore/QUuid>
#include <QtCore/QFile>
#include <QtCore/QDir>
#include <QtXml/QDomDocument>
#include <QtCore/QDateTime>

#include "client.h"
#include "common/domainentry.h"

int main(int argc, char **argv)
{
    Diamesh::CoreApplication * app = Diamesh::CoreApplication::create(argc, argv, NULL);
    Q_UNUSED(app);

    Diamesh::SessionHelperFactory factory;
    QSharedPointer<Diamesh::SessionHelper> sessionHelper = factory.connect();
    if(!sessionHelper)
    {
        qCritical() << QObject::tr("The session should have been created, as the password is right.");
        return 1;
    }

    Hns::Client client(sessionHelper.data());
    Hns::TopologyCatalogPrx catalog = client.topologyCatalog();
    if (!catalog) {
        qCritical() << QObject::tr("The catalog has not been created");
        return 1;
    }

    Diamesh::LongList topologyEntryIdList = catalog->getEntries();
    if (!topologyEntryIdList.size()) {
        qCritical() << QObject::tr("no topology entries found in the catalog");
        return 1;
    }
    Hns::TopologyEntryPtr topologyEntry = Hns::TopologyEntryPtr::dynamicCast(catalog->getEntry(topologyEntryIdList.front()));

    Hns::TopologyManagerPrx topologyManager = client.master()->getTopologyManager();
    Hns::TopologyPrx topology = topologyManager->loadTopology(topologyEntry->id);
    if(!topology) {
        qCritical() << QObject::tr("error while loading topologies");
        return 1;
    }

    int topologyId = topologyEntry->id;
    if (!topologyManager->saveTopology(topologyId)) {
        qCritical() << QObject::tr("error while saving topology %1").arg(topologyId);
        return 1;
    }

    if(!catalog->refresh()) {
        qCritical() << QObject::tr("the catalog failed to refresh. Topology %1 may not have been correctly saved").arg(topologyEntry->uuid.c_str());
        return 1;
    }

    if (!topologyManager->closeTopology(topologyId)) {
        qCritical() << QObject::tr("error while closing topology %1").arg(topologyId);
        return 1;
    }

    Hns::ByteArray screenshotData = catalog->getScreenshot(topologyEntry->id);

    if (screenshotData.size() == 0) {
        qCritical() << QObject::tr("Unable to get topology screenshot for entry %1").arg(topologyEntry->id);
        return 1;
    }

    if(!catalog->setScreenshot(topologyEntry->id, screenshotData)) {
        qCritical() << QObject::tr("Unable to set topology screenshot for entry %1").arg(topologyEntry->id);
        return 1;
    }

    return 0;
}
