/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#include <diamesh/coreapplication.h>
#include <diamesh/sessionhelper.h>
#include <diamesh/diamesh.h>
#include <diamesh/sessionManager.h>
#include <QtCore/QDebug>
#include <QDir>
#include <QUuid>
#include <QTimer>

#include "client.h"
#include "common/entity.h"
#include "common/wire.h"
#include "common/entry.h"
#include "common/domainentry.h"
#include "common/wireentry.h"
#include "common/topology.h"
#include "entryserializer.h"

int main(int argc, char **argv)
{
    Diamesh::CoreApplication * app = Diamesh::CoreApplication::create(argc, argv, NULL);
    Q_UNUSED(app);

    Diamesh::SessionHelperFactory factory;
    QSharedPointer<Diamesh::SessionHelper> sessionHelper = factory.connect();
    if(!sessionHelper)
    {
        qCritical() << QObject::tr("The session should have been created, as the password is right.");
        return 1;
    }

    Hns::Client client(sessionHelper.data());

    Hns::TopologyManagerPrx topologyManager = client.master()->getTopologyManager();
    Hns::TopologyPrx topology = topologyManager->createTopology(false);

    //Hns::EntityFactoryPrx entityFactoryProxy = client.master()->getEntityFactory();

    Hns::DomainEntryPtr domainEntry = new Hns::DomainEntry();
    domainEntry->uuid = QUuid::createUuid().toString().mid(1, 36).toStdString();
    domainEntry->path = "Test Catalog:/";
    domainEntry->shortName = "short name test 1";
    domainEntry->longName = "long name test ";
    domainEntry->type = "vbox";
    domainEntry->vcpu = 1;
    Hns::Disk disk;
    disk.type = "file";
    disk.device = "disk";
    disk.source = "dummy";
    disk.target = "sda";
    disk.targetBus = "sata";
    domainEntry->disks.push_back(disk);
    Hns::Netcard netcard;
    netcard.macAddress = "11:22:33:44:55:66";
    netcard.type = "bridge";
    netcard.model = "am79c973";
    domainEntry->netcards[0] = netcard;

    Hns::EntityLayoutPtr layout = new Hns::EntityLayout();
    Hns::EntityPrx domainProxy = topology->createEntityFromEntry(domainEntry, layout);

    Hns::DomainEntryPtr domainEntry2 = new Hns::DomainEntry();
    domainEntry2->uuid = QUuid::createUuid().toString().mid(1, 36).toStdString();
    domainEntry2->path = "Test Catalog:/";
    domainEntry2->shortName = "short name test 2";
    domainEntry2->longName = "long name test 2";
    domainEntry2->type = "vbox";
    domainEntry2->vcpu = 1;
    domainEntry2->disks.push_back(disk);
    domainEntry2->netcards[0] = netcard;

    Hns::EntityPrx domainProxy2 = topology->createEntityFromEntry(domainEntry2, layout);

    Hns::WireEntryPtr wireEntry = new Hns::WireEntry();
    wireEntry->uuid = QUuid::createUuid().toString().mid(1, 36).toStdString();
    wireEntry->longName = "wire 1";
    wireEntry->propertyLR.loss = 0;
    wireEntry->propertyLR.delay = 0;
    wireEntry->propertyRL.loss = 0;
    wireEntry->propertyRL.delay = 0;
    Hns::WirePrx wireProxy = topology->addWire(wireEntry,
                      QString("%1:%2").arg(domainProxy->getId()).arg("0").toStdString(),
                      QString("%1:%2").arg(domainProxy2->getId()).arg("0").toStdString(),
                      Hns::WireLayoutPtr());

    domainProxy->define("");
    if (!domainProxy->isDefined()) {
        qCritical() << QObject::tr("The first domain should have been defined.");
        return 1;
    }

    domainProxy2->define("");
    if (!domainProxy2->isDefined()) {
        qCritical() << QObject::tr("The second domain should have been defined.");
        return 1;
    }

    QEventLoop l;
    QTimer::singleShot(1000, &l, SLOT(quit()));
    l.exec();

    if (!wireProxy->isDefined()) {
        qCritical() << QObject::tr("The wire should have been defined since it links two defined entities");
        return 1;
    }

    if (!topology->removeEntity(domainProxy->getId()) ||
            !topology->removeEntity(domainProxy2->getId())) {
        qDebug() << QObject::tr("destroying entities failed");
        return 1;
    }

    return 0;
}
