/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#include <diamesh/coreapplication.h>
#include <diamesh/sessionhelper.h>
#include <diamesh/diamesh.h>
#include <diamesh/sessionManager.h>
#include <QtCore/QDebug>
#include <QDir>
#include <QUuid>
#include <QTimer>

#include "client.h"
#include "common/entity.h"
#include "common/wire.h"
#include "common/entry.h"
#include "common/domainentry.h"
#include "common/wireentry.h"
#include "common/topology.h"
#include "entryserializer.h"

int main(int argc, char **argv)
{
    Diamesh::CoreApplication * app = Diamesh::CoreApplication::create(argc, argv, NULL);
    Q_UNUSED(app);

    Diamesh::SessionHelperFactory factory;
    QSharedPointer<Diamesh::SessionHelper> sessionHelper = factory.connect();
    if(!sessionHelper)
    {
        qCritical() << QObject::tr("The session should have been created, as the password is right.");
        return 1;
    }

    Hns::Client client(sessionHelper.data());

   ::Hns::EntityEntryToImportMap mapEntity = client.master()->getCatalogImporter()->showEntityEntry();


   ::Hns::EntityEntryToImportMap::iterator it =  mapEntity.begin();
    QString nameFolder("");
    while(it != mapEntity.end()){
        nameFolder = it->first.c_str();
        qDebug()<< QObject::tr("\tfolder %1").arg(nameFolder);
        qDebug() << QObject::tr("\tname: %1").arg(it->second->longName.c_str());
        it++;
    }

    if(nameFolder.isEmpty()) {
        qDebug() << QObject::tr("\tno import test");
        return 0;
    }
    Hns::ImportableEntryPtr importable = new Hns::ImportableEntry();
    importable->path = nameFolder.toStdString();
    ::Ice::Long id = client.master()->getCatalogImporter()->importData(importable, "/tmp/hnsTestCatalog/entities", std::string(), std::string());
    qDebug() << QObject::tr("\timport folder id: %1").arg(id);
    ::Hns::EntityEntryPtr entryImported = client.master()->getEntityCatalog()->getEntityEntry(id);
    Q_ASSERT(entryImported);
    qDebug() << QObject::tr("\timport name entity: %1").arg(entryImported->longName.c_str());


    return 0;
}
