/******************************************************************************
**
** Copyright (C) 2003-2012 DIATEAM. All rights reserved.
**
** This file is part of hynesim.
**
** hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#include "masterI.h"
#include "catalogI.h"
#include "common/folder.h"
#include "sqluserstorage.h"
#include <diamesh/coreapplication.h>
#include <diamesh/diamesh.h>
#include <QUuid>
#include <QtCore/QDebug>
#include "importer/importableserializer.h"
#include "common/importable.h"
#include <QtCore/QFile>
#include "common/archivedescriptor.h"
#include "importer/handler.h"
#include "importer/catalogimporterI.h"
#include "libarchive/libarchivehandler.h"

using namespace Hns;
int testSerialize() {
    //TEST IMPORTABLEARCHIVESERIALIZER
    QFile file("../../../tests/015-importCatalog/xml/hnsArchive.xml");
    if (!file.open(QIODevice::ReadWrite | QFile::Text)) {
        qCritical() << QString(QObject::tr("Unable to open file hnsArchive.xml"));
        return false;
    }
    QTextStream out(&file);
    ImportableArchiveSerializer serializer;
    ImportablePtr ptr = serializer.deserialize(out.readAll());
    file.close();
    ImportableArchivePtr archivePtr = ImportableArchivePtr::dynamicCast(ptr);
    if (!archivePtr) {
        qCritical() << QObject::tr("Unable to retrieve archivePtr !");
        return EXIT_FAILURE;
    }
    qDebug() << "**************HnsArchive.xml***********";
    qDebug() << "name of importableArchive : " << QString::fromStdString(archivePtr->archive->name);
    qDebug() << "creationDateTime of importableArchive : "<< archivePtr->archive->creationDateTime;
    //folders
    QList<FolderDescriptor> listFD = QList<FolderDescriptor>::fromStdList(archivePtr->archive->folders);
    foreach (FolderDescriptor fd,listFD) {
        qDebug() << "********new folderDescriptor************";
        qDebug() << "shortName of folderDescriptor " << QString::fromStdString(fd.shortName);
        qDebug() << "longName of folderDescriptor " << QString::fromStdString(fd.longName);
        qDebug() << "description of folderDescriptor " << QString::fromStdString(fd.description);
//        qDebug() << "tags of folderDescriptor " << QString::fromStdString(fd.tags);
        qDebug() << "id of folderDescriptor " << fd.id;
        qDebug() << "parentId of folderDescriptor" << fd.parentId;
    }
    //EntityDescriptor
    QList<EntityDescriptor> listED = QList<EntityDescriptor>::fromStdList(archivePtr->archive->entities);
    foreach(EntityDescriptor ed,listED) {
        qDebug() << "**********new EntityDescriptor********";
        qDebug() << "type of entityDescriptor " << QString::fromStdString(ed.type);
        qDebug() << "shortName of entityDescriptor " << QString::fromStdString(ed.shortName);
        qDebug() << "longName of entityDescriptor " << QString::fromStdString(ed.longName);
        qDebug() << "description of entityDescriptor " << QString::fromStdString(ed.description);
        qDebug() << "path of entityDescriptor " << QString::fromStdString(ed.path);
        qDebug() << "uuid of entityDescriptor " << QString::fromStdString(ed.uuid);
        QList<long> listFolders = QList<long>::fromStdList(ed.folders);
        foreach (long folder,listFolders){
            qDebug()<<"id of entityFolders"<<folder;
        }
    }
    //TopologyDescriptor
    QList<TopologyDescriptor> listTD = QList<TopologyDescriptor>::fromStdList(archivePtr->archive->topologies);
    foreach (TopologyDescriptor td, listTD) {
        qDebug() << "**********new TopologyDescriptor********";
        qDebug() << "shortName of topologyDescriptor " << QString::fromStdString(td.shortName);
        qDebug() << "longName of topologyDescriptor " << QString::fromStdString(td.longName);
        qDebug() << "description of topologyDescriptor " << QString::fromStdString(td.description);
        qDebug() << "path of topologyDescriptor " << QString::fromStdString(td.path);
        qDebug() << "uuid of topologyDescriptor " << QString::fromStdString(td.uuid);
        QList<std::string> listDependencies = QList<std::string>::fromStdList(td.dependencies);
        foreach (std::string dependency,listDependencies) {
            qDebug()<< "Dependency uuid"<<QString::fromStdString(dependency);
        }
    }

    //TEST IMPORTABLEENTRYSERIALIZER
    QFile file2("/home/manuel/Devel/hynesim/trunk/tests/015-importCatalog/xml/hnsEntryConfig.xml");
    if (!file2.open(QIODevice::ReadWrite | QFile::Text)) {
        qCritical() << QString(QObject::tr("Unable to open file hnsEntryConfig.xml"));
        return false;
    }
    QTextStream out2(&file2);
    ImportableEntrySerializer serializer2;
    ImportablePtr ptr2 = serializer2.deserialize(out2.readAll());
    ImportableEntryPtr entryPtr = ImportableEntryPtr::dynamicCast(ptr2);
    if (!archivePtr) {
        qCritical() << QObject::tr("Unable to retrieve archivePtr !");
        return EXIT_FAILURE;
    }
    qDebug() << "**************HnsEntry.xml***********";
    qDebug() << "creationDateTime : " << entryPtr->entry->creationDateTime;
    qDebug() << "description : " << QString::fromStdString(entryPtr->entry->description);
    qDebug() << "longName : " << QString::fromStdString(entryPtr->entry->longName);
    qDebug() << "shortName : " << QString::fromStdString(entryPtr->entry->shortName);

    file2.close();
    return EXIT_SUCCESS;
}
int testHandler() {
    FolderHandler fh;
    qDebug()<<"/home/manuel"<<fh.canHandle("/home/manuel");
    qDebug()<<"/home/manuel/"<<fh.canHandle("/home/manuel/");
    qDebug()<<"/home/manuel/toto"<<fh.canHandle("/home/manuel/toto");

    ImportablePtr ptr = fh.handle("/home/manuel/Devel/fakeCatalog/test/");
    ImportableEntryPtr entry = ImportableEntryPtr::dynamicCast(ptr);
    ImportableArchivePtr archive = ImportableArchivePtr::dynamicCast(ptr);
    if (entry != 0) {
        qDebug() << "OK";
        qDebug() << entry->entry->creationDateTime;
        qDebug() << QString::fromStdString(entry->path);
        qDebug() << entry->size;
    }
    else
        qDebug() << "Pas OK Entry";

    if (archive != 0) {
        qDebug() << "OK";
        qDebug() << archive->archive->creationDateTime;
        qDebug() << archive->size;
        qDebug() << QString::fromStdString(archive->path);
    }
    else
        qDebug()<<"Pas OK archive";

    return 0;
}

int testParse() {
//    CatalogImporterI catalogImporter;
//    EntityEntryImporter importer;
//    QList<ImportablePtr> list = catalogImporter.getImportableList(Hns::CatalogImporterFlags::FLAG_IMPORT_ENTITIES);
//    foreach (ImportablePtr ptr, list) {
//        ImportableEntryPtr entry = ImportableEntryPtr::dynamicCast(ptr);
//        if (entry) {
//            qDebug() << entry->entry->creationDateTime;
//            qDebug() << QString::fromStdString(entry->entry->shortName);
//            qDebug() << QString::fromStdString(entry->entry->path);
//            qDebug() << QString::fromStdString(entry->path);
//            importer.importFolder(entry->path,"entity",Ice::Current());
//        }
//    }

    return 1;
}

void testCompressHandler() {
    LibArchiveHandler handler("");
//    qDebug() << "compress : " << handler.compress("/home/yohan/soutenance.pdf", "/home/yohan/toto.tar.gz");
}

int main(int argc, char **argv)
{
    Q_UNUSED(argc)
    Q_UNUSED(argv)

//   return testSerialize();
//    return testHandler();
//    return testParse();
    testCompressHandler();
    return 0;


}
