/******************************************************************************
**
** Copyright (C) 2011-2012 DIATEAM. All rights reserved.
**
** This file is part of QtIce.
**
** Hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** Hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef QTICE_OPERATORS_H
#define QTICE_OPERATORS_H

#include <list>
#include <map>
#include <QtCore/QDataStream>
#include <QtCore/QList>
#include <QtCore/QMap>
#include <QtCore/QVector>
#include <Ice/Config.h>

namespace
{

inline QDataStream &operator<<(QDataStream &out, const std::string &s)
{
    out << QString::fromStdString(s);
    return out;
}

inline QDataStream &operator>>(QDataStream &in, std::string &s)
{
    QString tmp;
    in >> tmp;
    s = tmp.toStdString();
    return in;
}

template <typename T> QDataStream &operator<<(QDataStream &out, const std::list<T> &list)
{
    out << QList<T>::fromStdList(list);
    return out;
}

template <typename T> QDataStream &operator>>(QDataStream &in, std::list<T> &list)
{
    QList<T> tmpList;
    in >> tmpList;
    list = tmpList.toStdList();
    return in;
}

template <typename K, typename T> QDataStream &operator<<(QDataStream &out, const std::map<K, T> &map)
{
    out << QMap<K, T>(map);
    return out;
}

template <typename K, typename T> QDataStream &operator>>(QDataStream &in, std::map<K, T> &map)
{
    QMap<K, T> tmpMap;
    in >> tmpMap;
    map = tmpMap.toStdMap();
    return in;
}

template <typename T> QDataStream &operator<<(QDataStream &out, const std::vector<T> &vector)
{
    out << QVector<T>::fromStdVector(vector);
    return out;
}

template <typename T> QDataStream &operator>>(QDataStream &in, std::vector<T> &vector)
{
    QVector<T> tmpVector;
    in >> tmpVector;
    vector = tmpVector.toStdVector();
    return in;
}

}

#endif // QTICE_OPERATORS_H
