/******************************************************************************
**
** Copyright (C) 2011-2012 DIATEAM. All rights reserved.
**
** This file is part of QtIce.
**
** Hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** Hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef QICEOBJECT_H
#define QICEOBJECT_H

#include <QtCore/QObject>
#include <QtIce/qtice.h>

#include <QtCore/QVariantList>
#include <QtCore/QReadWriteLock>
#include <QtCore/QMultiHash>
#include <Ice/BuiltinSequences.h>
#include <Ice/Stream.h>

class QIceObject : public QObject, virtual public QtIce::SignalEmitter
{
  Q_OBJECT

  public:
    QIceObject(QObject * parent = 0);
    virtual ~QIceObject();

    void connect(const QtIce::SignalForwarderPrx &, const std::string &, const Ice::Current &);
    void disconnect(const QtIce::SignalForwarderPrx &, const std::string &, const Ice::Current &);

  protected:
    void emitSignal(const char * signal, const char * internalSignal, const QVariantList & args);
    void timerEvent(QTimerEvent *event);
    Ice::CommunicatorPtr _getIceCommunicator();
    Ice::OutputStreamPtr _createOutputStream(const Ice::CommunicatorPtr &communicator);
    void _displayError(const QString &message);
    template<typename T> bool _serializeData(T data, Ice::ByteSeq &bytes) {
      bool result = false;
      Ice::CommunicatorPtr communicator = _getIceCommunicator();
      if (communicator != 0) {
        Ice::OutputStreamPtr out = _createOutputStream(communicator);
        out->startEncapsulation();
        out->write(data);
        out->writePendingObjects();
        out->endEncapsulation();
        out->finished(bytes);
        result = true;
      } else
        _displayError("error while serializing Ice::ObjectPtr. QtIce::iceCommunicator is not initialized.");
      return result;
    }

  private slots:
    void _removeFromAdapter(Ice::ObjectAdapterPtr adapter, Ice::Identity identity);

  signals:
    void requestDeletion(Ice::ObjectAdapterPtr adapter, Ice::Identity identity);
    void removedFromAdapter();

  protected:
    QReadWriteLock _signalsLock;
    QMultiHash<QString, QtIce::SignalForwarderPrx> _signalsHash;
};

#endif // QICEOBJECT_H
