/******************************************************************************
**
** Copyright (C) 2011-2012 DIATEAM. All rights reserved.
**
** This file is part of QtIce.
**
** Hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** Hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#include "qiceobjectproxy.h"

#include <IceUtil/UUID.h>
#include <Ice/Connection.h>
#include <Ice/LocalException.h>
#include <Ice/ObjectAdapter.h>
#include <Ice/Communicator.h>
#include <Ice/Initialize.h>
#include <Glacier2/Router.h>

#include <QtCore/QVariantList>
#include <QtCore/QMetaMethod>
#include <QtCore/QCoreApplication>
#include <QtCore/QRegExp>
#include <QtCore/QStringList>
#include "QtIce/utils.h"

using namespace std;
using namespace QtIce;

QIceObjectProxy::QIceObjectProxy(SignalEmitterPrx prx, Ice::ObjectAdapterPtr adapter) :
    _prx(prx), _adapter(adapter)
{
  moveToThread(qApp->thread());
  _ident.name = IceUtil::generateUUID();
  if(Ice::RouterPrx router = _adapter->getCommunicator()->getDefaultRouter())
  {
    _ident.category = Glacier2::RouterPrx::uncheckedCast(router)->getCategoryForClient();
  }
  _adapter->add(this, _ident);
  try
  {
    _prx->ice_getConnection()->setAdapter(adapter);
  }
  catch (const Ice::CollocationOptimizationException &){}
}

QIceObjectProxy::~QIceObjectProxy()
{
}

void QIceObjectProxy::qtice_destroy()
{
  disconnect();
  __setNoDelete(true);
  _adapter->remove(_ident);
  QObject::connect(this, SIGNAL(removedFromAdapter()), this, SLOT(deleteLater()));
  QMetaObject::invokeMethod(this, "_startDeletionTimer", Qt::QueuedConnection);
}

void QIceObjectProxy::connectNotify(const char * signal)
{
  if (QLatin1String(signal) == SIGNAL(removedFromAdapter()))
    return;
//  cout << "QIceObjectProxy::connectNotify() " << _ident.name << " " << ++signal << endl;
  ++signal;
  _oneWayPrx()->connect(SignalForwarderPrx::uncheckedCast(_adapter->createProxy(_ident)), signal);
}

void QIceObjectProxy::disconnectNotify(const char * signal)
{
//  cout << "QIceObjectProxy::disconnectNotify()" << _ident.name << " " << (signal ? ++signal : "") << endl;
  if(signal)
    ++signal;
  try {
    _oneWayPrx()->begin_disconnect(SignalForwarderPrx::uncheckedCast(_adapter->createProxy(_ident)), (signal ? signal : ""));
  } catch (const Ice::Exception & e) {
    cerr << "QIceObjectProxy::disconnectNotify() _prx->disconnect() failed" << endl;
  }
}

#define GENERIC_ARG(I,L) ( ( L ).size() > I ) ? \
          QGenericArgument(( L ).at( I ).typeName(), ( L ).at( I ).constData()) \
        : QGenericArgument();

void QIceObjectProxy::forward(const::string & signal, const string &internalSignal, const::SignalArgs & args, const::Ice::Current &)
{
//  cout << "QIceObjectProxy::forward " << signal.c_str() << endl;
  QByteArray block;
  for(SignalArgs::const_iterator it = args.begin() ; it != args.end() ; ++it)
    block.append(*it);

  QDataStream stream(block);
  stream.setVersion(QDataStream::Qt_4_6);
  QVariantList params;
  stream >> params;
  if (stream.status() != QDataStream::Ok) {
      cerr << "error deserializing signal \"" << signal << "\" arguments" << endl;
      return;
  }

  QGenericArgument arg0 = GENERIC_ARG(0, params);
  QGenericArgument arg1 = GENERIC_ARG(1, params);
  QGenericArgument arg2 = GENERIC_ARG(2, params);
  QGenericArgument arg3 = GENERIC_ARG(3, params);
  QGenericArgument arg4 = GENERIC_ARG(4, params);
  QGenericArgument arg5 = GENERIC_ARG(5, params);
  QGenericArgument arg6 = GENERIC_ARG(6, params);
  QGenericArgument arg7 = GENERIC_ARG(7, params);
  QGenericArgument arg8 = GENERIC_ARG(8, params);
  QGenericArgument arg9 = GENERIC_ARG(9, params);

  std::string signalForwarderName = "_forward_" + internalSignal;
  int i = metaObject()->indexOfMethod(signalForwarderName.c_str());
  QMetaMethod m = metaObject()->method(i);
  m.invoke(this, Qt::QueuedConnection, arg0, arg1, arg2, arg3, arg4, arg5, arg6, arg7, arg8, arg9);
}

void QIceObjectProxy::_startDeletionTimer()
{
  startTimer(1000);
}

void QIceObjectProxy::timerEvent(QTimerEvent */*event*/)
{
  if (!__getRef())
      emit removedFromAdapter();
}

Ice::CommunicatorPtr QIceObjectProxy::_getIceCommunicator()
{
  return adapter()->getCommunicator();
}

Ice::InputStreamPtr QIceObjectProxy::_createInputStream(const Ice::CommunicatorPtr &communicator, Ice::ByteSeq bytes)
{
  return Ice::createInputStream(communicator, bytes);
}
