/******************************************************************************
**
** Copyright (C) 2011-2012 DIATEAM. All rights reserved.
**
** This file is part of QtIce.
**
** Hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** Hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#ifndef QICEOBJECTPROXY_H
#define QICEOBJECTPROXY_H

#include <QtCore/QObject>
#include <QtCore/QSharedPointer>
#include <QtIce/qtice.h>
#include <Ice/BuiltinSequences.h>
#include <Ice/Stream.h>

template <typename T> class QIceObjectProxyPointer;

class QIceObjectProxy : public QObject, virtual public QtIce::SignalForwarder
{
    Q_OBJECT

    friend class QIceObjectProxyPointer<QIceObjectProxy>;
protected:
    QIceObjectProxy(QtIce::SignalEmitterPrx prx, Ice::ObjectAdapterPtr adapter = 0);
public:
    typedef QtIce::SignalEmitterPrx prx_type;

    virtual ~QIceObjectProxy();

    void qtice_destroy();

    inline Ice::ObjectAdapterPtr adapter() { return _adapter; }
    inline QtIce::SignalEmitterPrx prx() { return _prx; }

  private:
    void connectNotify(const char * signal);
    void disconnectNotify(const char *signal);
    void forward(const::std::string &signal, const::std::string &internalSignal, const::QtIce::SignalArgs &, const::Ice::Current &);

    inline QtIce::SignalEmitterPrx _oneWayPrx() { return _prx->ice_oneway(); }

  private slots:
    void _startDeletionTimer();

  protected:
    virtual void timerEvent(QTimerEvent *event);
    Ice::CommunicatorPtr _getIceCommunicator();
    Ice::InputStreamPtr _createInputStream(const Ice::CommunicatorPtr &communicator, Ice::ByteSeq bytes);
    template <typename T> bool _deserializeData(Ice::ByteSeq bytes, T &data) {
      bool result = false;
      Ice::CommunicatorPtr communicator = _getIceCommunicator();
      if (communicator != 0) {
        Ice::InputStreamPtr in = _createInputStream(communicator, bytes);
        in->startEncapsulation();
        in->read(data);
        in->readPendingObjects();
        in->endEncapsulation();
        result = true;
      }
      return result;
    }

  signals:
    void removedFromAdapter();

  private:
    Ice::Identity _ident;
    QtIce::SignalEmitterPrx _prx;
    Ice::ObjectAdapterPtr _adapter;

};

class QIceObjectProxy;
typedef QIceObjectProxyPointer< QIceObjectProxy> QPrx;

template<typename P> P uncheckedCastImpl(const QPrx& b);

// Upcast

template<typename T, typename Y> inline QIceObjectProxyPointer<T>
uncheckedCastHelper(const QIceObjectProxyPointer<Y> &b, T*)
{
    return b;
}

// Downcast

template<typename T, typename Y> inline QIceObjectProxyPointer<T>
uncheckedCastHelper(const QIceObjectProxyPointer<Y> &b, void *)
{
    return uncheckedCastImpl<QIceObjectProxyPointer<T> >(b);
}

template <typename T>
class QIceObjectProxyPointer
{
public:
    typedef T element_type;

    QIceObjectProxyPointer() {}
    QIceObjectProxyPointer(T *p) : _ptr(p) {}

    template <typename Y>
    QIceObjectProxyPointer(const QIceObjectProxyPointer<Y> &r) : _ptr(r._ptr) {}

    template <typename Y>
    QIceObjectProxyPointer(::IceInternal::ProxyHandle<Y> prx, Ice::ObjectAdapterPtr adapter) : _ptr(new T(prx, adapter), &T::qtice_destroy) {}

    inline T *data() const { return _ptr.data(); }
    inline bool isNull() const { return _ptr.isNull(); }
    inline operator bool() const { return !_ptr.isNull(); }
    inline bool operator!() const { return _ptr.operator !(); }
    inline T &operator*() const { return *_ptr; }
    inline T *operator->() const { return _ptr.operator ->(); }

    QIceObjectProxyPointer &operator=(T *p)
    {
        _ptr = QSharedPointer<T>(p, &T::qtice_destroy);
        return *this;
    }

    template <class Y>
    static QIceObjectProxyPointer uncheckedCast(const QIceObjectProxyPointer<Y> &r)
    {
        Y* tag = 0;
        return uncheckedCastHelper<T>(r, tag);
    }

    QSharedPointer<T> _ptr;
};

template<typename P> P
uncheckedCastImpl(const QPrx& b)
{
    P d;
    if (b) {
        typedef typename P::element_type T;

        d._ptr = qSharedPointerDynamicCast<T>(b._ptr);
        if (!d)
            d = P(T::proxy_type::uncheckedCast(b._ptr->prx()), b._ptr->adapter());
    }
    return d;
}

#endif // QICEOBJECTPROXY_H
