/******************************************************************************
**
** Copyright (C) 2011-2012 DIATEAM. All rights reserved.
**
** This file is part of QtIce.
**
** Hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** Hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#include "generator.h"

#include <QtCore/QFileInfo>
#include <QtCore/QStringList>
#include <QMetaType>

using namespace std;
using namespace Slice;

Generator::Generator(Slice::UnitPtr u, const QString & outputDirPath, const QStringList &includeDirs, const QString & baseNamePrefix, const QString &prefix) :
    _unit(u), _outputDirPath(outputDirPath), _baseNamePrefix(baseNamePrefix), _includeDirs(includeDirs), _prefix(prefix)
{
}

QString Generator::_headerGuardString(const QString & text)
{
  return QString("%1_%2_H").arg(_baseNamePrefix).arg(text).toUpper();
}

QString Generator::_baseName()
{
  QFileInfo inputFileInfo(QString::fromStdString(_unit->topLevelFile()));
  return QString("%1_%2").arg(_baseNamePrefix).arg(inputFileInfo.completeBaseName());
}

QMultiHash<QString, QString> Generator::_extractMetaData(const list<string> & metaData, const QString & prefix)
{
  QStringList list;
  foreach(const string & str, metaData)
    list << QString::fromStdString(str);
  QRegExp rx;
  if(!prefix.isEmpty())
    rx.setPattern(QString("^%1:").arg(prefix));
  list = list.filter(rx).replaceInStrings(rx, "");

  QMultiHash<QString, QString> hash;
  foreach(const QString & str, list)
  {
    int index = str.indexOf(':');
    if(index != -1 && index == str.indexOf("::"))
      index = -1;
    hash.insert(str.left(index), str.right(str.size()-index-1));
  }
  return hash;
}

ClassList Generator::_classList(const ModuleList &modules)
{
    ClassList list;
    for (ModuleList::const_iterator m = modules.begin() ; m != modules.end() ; ++m) {
        if ((*m)->includeLevel() != 0)
          continue;

        ClassList classes = (*m)->classes();
        for (ClassList::const_iterator ci = classes.begin() ; ci != classes.end() ; ++ci) {
          ClassDefPtr c = *ci;
          if (c->isInterface())
              list.push_back(c);
        }

        classes = _classList((*m)->modules());
        list.insert(list.end(), classes.begin(), classes.end());
    }
    return list;
}

GeneratorFactory::GeneratorFactory()
{
}
