/******************************************************************************
**
** Copyright (C) 2011-2012 DIATEAM. All rights reserved.
**
** This file is part of QtIce.
**
** Hynesim is free software: you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 
** as published by the Free Software Foundation.
**
** Hynesim is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with this program.  If not, see <http://www.gnu.org/licenses/>.
**
******************************************************************************/

#include "generatorset.h"

#include <QtCore/QCoreApplication>
#include <QtCore/QDir>
#include <QtCore/QDebug>
#include <Slice/Preprocessor.h>
#include "generator.h"

using namespace std;
using namespace Slice;

GeneratorSet::GeneratorSet()
{
}

void GeneratorSet::addGenerator(GeneratorFactory * g)
{
  _generators << g;
}

bool GeneratorSet::generate(const QString & inputFile, const QString & outputDirPath, const QStringList & includeDirs, QString &prefix)
{
  vector<string> cppArgs;
  foreach(const QString & dir, includeDirs)
    cppArgs.push_back("-I" + Preprocessor::normalizeIncludePath(dir.toStdString()));

  QDir outputDir(outputDirPath.isEmpty() ? QDir::current() : outputDirPath);
  if(!outputDir.exists() && !outputDir.mkpath(outputDir.path()))
  {
    qCritical() << "Unable to create output dir" << outputDir;
    return false;
  }

  UnitPtr unit = _parse(inputFile, cppArgs);
  if(!unit)
    return false;

  foreach(GeneratorFactory * factory, _generators)
  {
    Generator * g = factory->create(unit, outputDir.absolutePath(), includeDirs, prefix);
    if(!g->generate())
      return false;
  }

  return true;
}

UnitPtr GeneratorSet::_parse(const QString & inputFile, const std::vector<std::string> & cppArgs)
{
  string fileName = inputFile.toStdString();
  PreprocessorPtr icecpp = Preprocessor::create(qApp->applicationFilePath().toStdString(), fileName, cppArgs);
  FILE * cppHandle = icecpp->preprocess(false);

  if(!cppHandle)
  {
    qCritical() << "Error during preprocessing of" << inputFile;
    return 0;
  }
  // createUnit(bool ignRedefs, bool all, bool allowIcePrefix, bool allowUnderscore, const StringList& defaultGlobalMetadata)
  UnitPtr u = Unit::createUnit(false, false, false, true); 
  int parseStatus = u->parse(fileName, cppHandle, false);
  if(parseStatus != 0)
  {
    qCritical() << "Error when parsing file" << inputFile;
    return 0;
  }

  return u;
}
